(function() {
  var AccountCommands, AccountStore, Actions, MenuHelpers, _, _ref,
    __slice = [].slice;

  _ = require('underscore');

  _ref = require('nylas-exports'), Actions = _ref.Actions, AccountStore = _ref.AccountStore, MenuHelpers = _ref.MenuHelpers;

  AccountCommands = (function() {
    function AccountCommands() {}

    AccountCommands._focusAccounts = function(accounts) {
      Actions.focusDefaultMailboxPerspectiveForAccounts(accounts);
      if (!NylasEnv.isVisible()) {
        return NylasEnv.show();
      }
    };

    AccountCommands._isSelected = function(account, sidebarAccountIds) {
      if (sidebarAccountIds.length > 1) {
        return account instanceof Array;
      } else if (sidebarAccountIds.length === 1) {
        return (account != null ? account.id : void 0) === sidebarAccountIds[0];
      } else {
        return false;
      }
    };

    AccountCommands.registerCommands = function(accounts) {
      var allKey, commands, _ref1;
      if ((_ref1 = this._commandsDisposable) != null) {
        _ref1.dispose();
      }
      commands = {};
      allKey = "window:select-account-0";
      commands[allKey] = this._focusAccounts.bind(this, accounts);
      [1, 2, 3, 4, 5, 6, 7, 8].forEach((function(_this) {
        return function(index) {
          var account, key;
          account = accounts[index - 1];
          if (!account) {
            return;
          }
          key = "window:select-account-" + index;
          return commands[key] = _this._focusAccounts.bind(_this, [account]);
        };
      })(this));
      return this._commandsDisposable = NylasEnv.commands.add(document.body, commands);
    };

    AccountCommands.registerMenuItems = function(accounts, sidebarAccountIds) {
      var idx, submenu, template, windowMenu;
      windowMenu = _.find(NylasEnv.menu.template, function(_arg) {
        var label;
        label = _arg.label;
        return MenuHelpers.normalizeLabel(label) === 'Window';
      });
      if (!windowMenu) {
        return;
      }
      submenu = _.reject(windowMenu.submenu, function(item) {
        return item.account;
      });
      if (!submenu) {
        return;
      }
      idx = _.findIndex(submenu, function(_arg) {
        var type;
        type = _arg.type;
        return type === 'separator';
      });
      if (!(idx > 0)) {
        return;
      }
      template = this.menuTemplate(accounts, sidebarAccountIds);
      submenu.splice.apply(submenu, [idx + 1, 0].concat(__slice.call(template)));
      windowMenu.submenu = submenu;
      return NylasEnv.menu.update();
    };

    AccountCommands.menuItem = function(account, idx, _arg) {
      var accounts, clickHandlers, isSelected, item, _ref1, _ref2;
      _ref1 = _arg != null ? _arg : {}, isSelected = _ref1.isSelected, clickHandlers = _ref1.clickHandlers;
      item = {
        label: (_ref2 = account.label) != null ? _ref2 : "All Accounts",
        command: "window:select-account-" + idx,
        account: true
      };
      if (isSelected) {
        item.type = 'checkbox';
        item.checked = true;
      }
      if (clickHandlers) {
        accounts = account instanceof Array ? account : [account];
        item.click = AccountCommands._focusAccounts.bind(AccountCommands, accounts);
        item.accelerator = "CmdOrCtrl+" + (idx + 1);
      }
      return item;
    };

    AccountCommands.menuTemplate = function(accounts, sidebarAccountIds, _arg) {
      var clickHandlers, isSelected, multiAccount, template;
      clickHandlers = (_arg != null ? _arg : {}).clickHandlers;
      template = [];
      multiAccount = accounts.length > 1;
      if (multiAccount) {
        isSelected = AccountCommands._isSelected(accounts, sidebarAccountIds);
        template = [
          AccountCommands.menuItem(accounts, 0, {
            isSelected: isSelected,
            clickHandlers: clickHandlers
          })
        ];
      }
      template = template.concat(accounts.map(function(account, idx) {
        var accIdx;
        accIdx = multiAccount ? idx + 1 : idx;
        isSelected = AccountCommands._isSelected(account, sidebarAccountIds);
        return AccountCommands.menuItem(account, accIdx, {
          isSelected: isSelected,
          clickHandlers: clickHandlers
        });
      }));
      return template;
    };

    AccountCommands.register = function(accounts, sidebarAccountIds) {
      this.registerCommands(accounts);
      return this.registerMenuItems(accounts, sidebarAccountIds);
    };

    return AccountCommands;

  })();

  module.exports = AccountCommands;

}).call(this);
