(function() {
  var Actions, CategoryStore, DestroyCategoryTask, FocusedPerspectiveStore, MailboxPerspective, OutlineViewItem, SidebarActions, SidebarItem, SyncbackCategoryTask, Utils, WorkspaceStore, countForItem, idForCategories, isItemCollapsed, isItemDeleted, isItemSelected, onDeleteItem, onEditItem, toggleItemCollapsed, _, _ref, _str,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  _str = require('underscore.string');

  _ref = require('nylas-exports'), WorkspaceStore = _ref.WorkspaceStore, MailboxPerspective = _ref.MailboxPerspective, FocusedPerspectiveStore = _ref.FocusedPerspectiveStore, SyncbackCategoryTask = _ref.SyncbackCategoryTask, DestroyCategoryTask = _ref.DestroyCategoryTask, CategoryStore = _ref.CategoryStore, Actions = _ref.Actions, Utils = _ref.Utils;

  OutlineViewItem = require('nylas-component-kit').OutlineViewItem;

  SidebarActions = require('./sidebar-actions');

  idForCategories = function(categories) {
    return _.pluck(categories, 'id').join('-');
  };

  countForItem = function(perspective) {
    var unreadCountEnabled;
    unreadCountEnabled = NylasEnv.config.get('core.workspace.showUnreadForAllCategories');
    if (perspective.isInbox() || unreadCountEnabled) {
      return perspective.unreadCount();
    }
    return 0;
  };

  isItemSelected = function(perspective) {
    var _ref1;
    return ((_ref1 = WorkspaceStore.rootSheet()) === WorkspaceStore.Sheet.Threads || _ref1 === WorkspaceStore.Sheet.Drafts) && FocusedPerspectiveStore.current().isEqual(perspective);
  };

  isItemDeleted = function(perspective) {
    return _.any(perspective.categories(), function(c) {
      return c.isDeleted;
    });
  };

  isItemCollapsed = function(id) {
    if (NylasEnv.savedState.sidebarKeysCollapsed[id] !== void 0) {
      return NylasEnv.savedState.sidebarKeysCollapsed[id];
    } else {
      return true;
    }
  };

  toggleItemCollapsed = function(item) {
    if (!(item.children.length > 0)) {
      return;
    }
    return SidebarActions.setKeyCollapsed(item.id, !isItemCollapsed(item.id));
  };

  onDeleteItem = function(item) {
    var category;
    if (item.deleted === true) {
      return;
    }
    category = item.perspective.category();
    if (!category) {
      return;
    }
    return Actions.queueTask(new DestroyCategoryTask({
      category: category
    }));
  };

  onEditItem = function(item, value) {
    var category;
    if (!value) {
      return;
    }
    if (item.deleted === true) {
      return;
    }
    category = item.perspective.category();
    if (!category) {
      return;
    }
    return Actions.queueTask(new SyncbackCategoryTask({
      category: category,
      displayName: value
    }));
  };

  SidebarItem = (function() {
    function SidebarItem() {}

    SidebarItem.forPerspective = function(id, perspective, opts) {
      var counterStyle, _ref1;
      if (opts == null) {
        opts = {};
      }
      if (perspective.isInbox()) {
        counterStyle = OutlineViewItem.CounterStyles.Alt;
      }
      return _.extend({
        id: id,
        name: perspective.name,
        contextMenuLabel: perspective.name,
        count: countForItem(perspective),
        iconName: perspective.iconName,
        children: [],
        perspective: perspective,
        className: isItemDeleted(perspective) ? 'deleted' : '',
        selected: isItemSelected(perspective),
        collapsed: (_ref1 = isItemCollapsed(id)) != null ? _ref1 : true,
        counterStyle: counterStyle,
        dataTransferType: 'nylas-threads-data',
        onDelete: opts.deletable ? onDeleteItem : void 0,
        onEdited: opts.editable ? onEditItem : void 0,
        onCollapseToggled: toggleItemCollapsed,
        onDrop: function(item, event) {
          var data, jsonString;
          jsonString = event.dataTransfer.getData(item.dataTransferType);
          data = Utils.jsonParse(jsonString);
          if (!data) {
            return;
          }
          return item.perspective.receiveThreads(data.threadIds);
        },
        shouldAcceptDrop: function(item, event) {
          var current, data, jsonString, target, _ref2;
          target = item.perspective;
          current = FocusedPerspectiveStore.current();
          jsonString = event.dataTransfer.getData(item.dataTransferType);
          data = Utils.jsonParse(jsonString);
          if (!data) {
            return false;
          }
          if (!target) {
            return false;
          }
          if (target.isEqual(current)) {
            return false;
          }
          if (!target.canReceiveThreadsFromAccountIds(data.accountIds)) {
            return false;
          }
          return _ref2 = item.dataTransferType, __indexOf.call(event.dataTransfer.types, _ref2) >= 0;
        },
        onSelect: function(item) {
          return Actions.focusMailboxPerspective(item.perspective);
        }
      }, opts);
    };

    SidebarItem.forCategories = function(categories, opts) {
      var contextMenuLabel, id, perspective, _ref1;
      if (categories == null) {
        categories = [];
      }
      if (opts == null) {
        opts = {};
      }
      id = idForCategories(categories);
      contextMenuLabel = _str.capitalize((_ref1 = categories[0]) != null ? _ref1.displayType() : void 0);
      perspective = MailboxPerspective.forCategories(categories);
      if (opts.deletable == null) {
        opts.deletable = true;
      }
      if (opts.editable == null) {
        opts.editable = true;
      }
      opts.contextMenuLabel = contextMenuLabel;
      return this.forPerspective(id, perspective, opts);
    };

    SidebarItem.forSnoozed = function(accountIds, opts) {
      var categories, constants, displayName, id, perspective;
      if (opts == null) {
        opts = {};
      }
      constants = require('../../thread-snooze/lib/snooze-constants');
      displayName = constants.SNOOZE_CATEGORY_NAME;
      id = displayName;
      if (opts.name) {
        id += "-" + opts.name;
      }
      if (!opts.name) {
        opts.name = "Snoozed";
      }
      opts.iconName = 'snooze.png';
      categories = accountIds.map((function(_this) {
        return function(accId) {
          return _.findWhere(CategoryStore.categories(accId), {
            displayName: displayName
          });
        };
      })(this));
      categories = _.compact(categories);
      perspective = MailboxPerspective.forCategories(categories);
      if (!perspective.name) {
        perspective.name = id;
      }
      return this.forPerspective(id, perspective, opts);
    };

    SidebarItem.forStarred = function(accountIds, opts) {
      var id, perspective;
      if (opts == null) {
        opts = {};
      }
      perspective = MailboxPerspective.forStarred(accountIds);
      id = 'Starred';
      if (opts.name) {
        id += "-" + opts.name;
      }
      return this.forPerspective(id, perspective, opts);
    };

    SidebarItem.forUnread = function(accountIds, opts) {
      var categories, id, perspective;
      if (opts == null) {
        opts = {};
      }
      categories = accountIds.map((function(_this) {
        return function(accId) {
          return CategoryStore.getStandardCategory(accId, 'inbox');
        };
      })(this));
      categories = _.compact(categories);
      perspective = MailboxPerspective.forUnread(categories);
      id = 'Unread';
      if (opts.name) {
        id += "-" + opts.name;
      }
      return this.forPerspective(id, perspective, opts);
    };

    SidebarItem.forDrafts = function(accountIds, opts) {
      var id, perspective;
      if (opts == null) {
        opts = {};
      }
      perspective = MailboxPerspective.forDrafts(accountIds);
      id = "Drafts-" + opts.name;
      return this.forPerspective(id, perspective, opts);
    };

    return SidebarItem;

  })();

  module.exports = SidebarItem;

}).call(this);
