(function() {
  var Actions, Category, CategoryStore, DestroyCategoryTask, SidebarActions, SidebarItem, SidebarSection, SyncbackCategoryTask, isSectionCollapsed, toggleSectionCollapsed, _, _ref,
    __slice = [].slice;

  _ = require('underscore');

  _ref = require('nylas-exports'), Actions = _ref.Actions, SyncbackCategoryTask = _ref.SyncbackCategoryTask, DestroyCategoryTask = _ref.DestroyCategoryTask, CategoryStore = _ref.CategoryStore, Category = _ref.Category;

  SidebarItem = require('./sidebar-item');

  SidebarActions = require('./sidebar-actions');

  isSectionCollapsed = function(title) {
    if (NylasEnv.savedState.sidebarKeysCollapsed[title] !== void 0) {
      return NylasEnv.savedState.sidebarKeysCollapsed[title];
    } else {
      return false;
    }
  };

  toggleSectionCollapsed = function(section) {
    if (!section) {
      return;
    }
    return SidebarActions.setKeyCollapsed(section.title, !isSectionCollapsed(section.title));
  };

  SidebarSection = (function() {
    function SidebarSection() {}

    SidebarSection.empty = function(title) {
      return {
        title: title,
        items: []
      };
    };

    SidebarSection.standardSectionForAccount = function(account) {
      var cats, draftsItem, items, snoozedItem, starredItem, unreadItem;
      if (!account) {
        throw new Error("standardSectionForAccount: You must pass an account.");
      }
      cats = CategoryStore.standardCategories(account);
      if (cats.length === 0) {
        return this.empty(account.label);
      }
      items = _.reject(cats, function(cat) {
        return cat.name === 'drafts';
      }).map((function(_this) {
        return function(cat) {
          return SidebarItem.forCategories([cat], {
            editable: false,
            deletable: false
          });
        };
      })(this));
      unreadItem = SidebarItem.forUnread([account.id]);
      starredItem = SidebarItem.forStarred([account.id]);
      draftsItem = SidebarItem.forDrafts([account.id]);
      snoozedItem = SidebarItem.forSnoozed([account.id]);
      items.splice(1, 0, unreadItem, starredItem, snoozedItem);
      items.push(draftsItem);
      return {
        title: account.label,
        items: items
      };
    };

    SidebarSection.standardSectionForAccounts = function(accounts) {
      var accountIds, categories, children, draftsItem, items, names, snoozedItem, standardNames, starredItem, unreadItem, _i, _len;
      if (!accounts || accounts.length === 0) {
        return this.empty('All Accounts');
      }
      if (CategoryStore.categories().length === 0) {
        return this.empty('All Accounts');
      }
      if (accounts.length === 1) {
        return this.standardSectionForAccount(accounts[0]);
      }
      standardNames = ['inbox', 'important', 'sent', ['archive', 'all'], 'spam', 'trash'];
      items = [];
      for (_i = 0, _len = standardNames.length; _i < _len; _i++) {
        names = standardNames[_i];
        names = Array.isArray(names) ? names : [names];
        categories = CategoryStore.getStandardCategories.apply(CategoryStore, [accounts].concat(__slice.call(names)));
        if (categories.length === 0) {
          continue;
        }
        children = [];
        accounts.forEach(function(acc) {
          var cat;
          cat = _.first(_.compact(names.map(function(name) {
            return CategoryStore.getStandardCategory(acc, name);
          })));
          if (!cat) {
            return;
          }
          return children.push(SidebarItem.forCategories([cat], {
            name: acc.label,
            editable: false,
            deletable: false
          }));
        });
        items.push(SidebarItem.forCategories(categories, {
          children: children,
          editable: false,
          deletable: false
        }));
      }
      accountIds = _.pluck(accounts, 'id');
      starredItem = SidebarItem.forStarred(accountIds, {
        children: accounts.map(function(acc) {
          return SidebarItem.forStarred([acc.id], {
            name: acc.label
          });
        })
      });
      unreadItem = SidebarItem.forUnread(accountIds, {
        children: accounts.map(function(acc) {
          return SidebarItem.forUnread([acc.id], {
            name: acc.label
          });
        })
      });
      draftsItem = SidebarItem.forDrafts(accountIds, {
        children: accounts.map(function(acc) {
          return SidebarItem.forDrafts([acc.id], {
            name: acc.label
          });
        })
      });
      snoozedItem = SidebarItem.forSnoozed(accountIds, {
        children: accounts.map(function(acc) {
          return SidebarItem.forSnoozed([acc.id], {
            name: acc.label
          });
        })
      });
      items.splice(1, 0, unreadItem, starredItem, snoozedItem);
      items.push(draftsItem);
      return {
        title: 'All Accounts',
        items: items
      };
    };

    SidebarSection.forUserCategories = function(account, _arg) {
      var category, collapsed, collapsible, i, item, itemDisplayName, itemKey, items, onCollapseToggled, parent, parentComponents, parentKey, seenItems, title, _i, _j, _len, _ref1, _ref2, _ref3;
      _ref1 = _arg != null ? _arg : {}, title = _ref1.title, collapsible = _ref1.collapsible;
      if (!account) {
        return;
      }
      items = [];
      seenItems = {};
      _ref2 = CategoryStore.userCategories(account);
      for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
        category = _ref2[_i];
        itemKey = category.displayName.replace(/[./\\]/g, '/');
        parent = null;
        parentComponents = itemKey.split('/');
        for (i = _j = _ref3 = parentComponents.length; _j >= 1; i = _j += -1) {
          parentKey = parentComponents.slice(0, i).join('/');
          parent = seenItems[parentKey];
          if (parent) {
            break;
          }
        }
        if (parent) {
          itemDisplayName = category.displayName.substr(parentKey.length + 1);
          item = SidebarItem.forCategories([category], {
            name: itemDisplayName
          });
          parent.children.push(item);
        } else {
          item = SidebarItem.forCategories([category]);
          items.push(item);
        }
        seenItems[itemKey] = item;
      }
      if (title == null) {
        title = account.categoryLabel();
      }
      collapsed = isSectionCollapsed(title);
      if (collapsible) {
        onCollapseToggled = toggleSectionCollapsed;
      }
      return {
        title: title,
        iconName: account.categoryIcon(),
        items: items,
        collapsed: collapsed,
        onCollapseToggled: onCollapseToggled,
        onItemCreated: function(displayName) {
          if (!displayName) {
            return;
          }
          category = new Category({
            displayName: displayName,
            accountId: account.id
          });
          return Actions.queueTask(new SyncbackCategoryTask({
            category: category
          }));
        }
      };
    };

    return SidebarSection;

  })();

  module.exports = SidebarSection;

}).call(this);
