(function() {
  var AccountCommands, AccountStore, Actions, CategoryStore, FocusedPerspectiveStore, NylasStore, OutboxStore, Sections, SidebarActions, SidebarSection, SidebarStore, ThreadCountsStore, WorkspaceStore, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  NylasStore = require('nylas-store');

  _ref = require('nylas-exports'), Actions = _ref.Actions, AccountStore = _ref.AccountStore, ThreadCountsStore = _ref.ThreadCountsStore, WorkspaceStore = _ref.WorkspaceStore, OutboxStore = _ref.OutboxStore, FocusedPerspectiveStore = _ref.FocusedPerspectiveStore, CategoryStore = _ref.CategoryStore;

  SidebarSection = require('./sidebar-section');

  SidebarActions = require('./sidebar-actions');

  AccountCommands = require('./account-commands');

  Sections = {
    "Standard": "Standard",
    "User": "User"
  };

  SidebarStore = (function(_super) {
    __extends(SidebarStore, _super);

    function SidebarStore() {
      this._updateSections = __bind(this._updateSections, this);
      this._onFocusedPerspectiveChanged = __bind(this._onFocusedPerspectiveChanged, this);
      this._onAccountsChanged = __bind(this._onAccountsChanged, this);
      this._registerMenuItems = __bind(this._registerMenuItems, this);
      this._registerCommands = __bind(this._registerCommands, this);
      this._onSetCollapsedByName = __bind(this._onSetCollapsedByName, this);
      this._onSetCollapsedByKey = __bind(this._onSetCollapsedByKey, this);
      var _base;
      if ((_base = NylasEnv.savedState).sidebarKeysCollapsed == null) {
        _base.sidebarKeysCollapsed = {};
      }
      this._sections = {};
      this._sections[Sections.Standard] = {};
      this._sections[Sections.User] = [];
      this._registerCommands();
      this._registerMenuItems();
      this._registerListeners();
      this._updateSections();
    }

    SidebarStore.prototype.accounts = function() {
      return AccountStore.accounts();
    };

    SidebarStore.prototype.sidebarAccountIds = function() {
      return FocusedPerspectiveStore.sidebarAccountIds();
    };

    SidebarStore.prototype.standardSection = function() {
      return this._sections[Sections.Standard];
    };

    SidebarStore.prototype.userSections = function() {
      return this._sections[Sections.User];
    };

    SidebarStore.prototype._registerListeners = function() {
      this.listenTo(Actions.setCollapsedSidebarItem, this._onSetCollapsedByName);
      this.listenTo(SidebarActions.setKeyCollapsed, this._onSetCollapsedByKey);
      this.listenTo(AccountStore, this._onAccountsChanged);
      this.listenTo(FocusedPerspectiveStore, this._onFocusedPerspectiveChanged);
      this.listenTo(WorkspaceStore, this._updateSections);
      this.listenTo(OutboxStore, this._updateSections);
      this.listenTo(ThreadCountsStore, this._updateSections);
      this.listenTo(CategoryStore, this._updateSections);
      this.configSubscription = NylasEnv.config.onDidChange('core.workspace.showUnreadForAllCategories', this._updateSections);
    };

    SidebarStore.prototype._onSetCollapsedByKey = function(itemKey, collapsed) {
      var currentValue;
      currentValue = NylasEnv.savedState.sidebarKeysCollapsed[itemKey];
      if (currentValue !== collapsed) {
        NylasEnv.savedState.sidebarKeysCollapsed[itemKey] = collapsed;
        return this._updateSections();
      }
    };

    SidebarStore.prototype._onSetCollapsedByName = function(itemName, collapsed) {
      var item, section, _i, _len, _ref1;
      item = _.findWhere(this.standardSection().items, {
        name: itemName
      });
      if (!item) {
        _ref1 = this.userSections();
        for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
          section = _ref1[_i];
          item = _.findWhere(section.items, {
            name: itemName
          });
          if (item) {
            break;
          }
        }
      }
      if (!item) {
        return;
      }
      return this._onSetCollapsedByKey(item.id, collapsed);
    };

    SidebarStore.prototype._registerCommands = function(accounts) {
      if (accounts == null) {
        accounts = AccountStore.accounts();
      }
      return AccountCommands.registerCommands(accounts);
    };

    SidebarStore.prototype._registerMenuItems = function(accounts) {
      if (accounts == null) {
        accounts = AccountStore.accounts();
      }
      return AccountCommands.registerMenuItems(accounts, FocusedPerspectiveStore.sidebarAccountIds());
    };

    SidebarStore.prototype._onAccountsChanged = function() {
      return this._updateSections();
    };

    SidebarStore.prototype._onFocusedPerspectiveChanged = function() {
      return _.defer((function(_this) {
        return function() {
          _this._registerCommands();
          _this._registerMenuItems();
          return _this._updateSections();
        };
      })(this));
    };

    SidebarStore.prototype._updateSections = function() {
      var accounts, multiAccount;
      accounts = _.compact(FocusedPerspectiveStore.sidebarAccountIds().map((function(_this) {
        return function(id) {
          return AccountStore.accountForId(id);
        };
      })(this)));
      if (accounts.length === 0) {
        return;
      }
      multiAccount = accounts.length > 1;
      this._sections[Sections.Standard] = SidebarSection.standardSectionForAccounts(accounts);
      this._sections[Sections.User] = accounts.map(function(acc) {
        var opts;
        opts = {};
        if (multiAccount) {
          opts.title = acc.label;
          opts.collapsible = true;
        }
        return SidebarSection.forUserCategories(acc, opts);
      });
      return this.trigger();
    };

    return SidebarStore;

  })(NylasStore);

  module.exports = new SidebarStore();

}).call(this);
