(function() {
  var AccountStore, Actions, Categories, Category, CategoryPickerPopover, CategoryStore, DatabaseStore, FocusedPerspectiveStore, MailboxPerspective, NylasTestUtils, React, ReactDOM, ReactTestUtils, SyncbackCategoryTask, TaskFactory, TaskQueueStatusStore, Thread, Utils, _, _ref;

  _ = require('underscore');

  React = require("react");

  ReactDOM = require('react-dom');

  ReactTestUtils = require('react-addons-test-utils');

  CategoryPickerPopover = require('../lib/category-picker-popover')["default"];

  _ref = require('nylas-exports'), Utils = _ref.Utils, Category = _ref.Category, Thread = _ref.Thread, Actions = _ref.Actions, AccountStore = _ref.AccountStore, CategoryStore = _ref.CategoryStore, DatabaseStore = _ref.DatabaseStore, TaskFactory = _ref.TaskFactory, SyncbackCategoryTask = _ref.SyncbackCategoryTask, FocusedPerspectiveStore = _ref.FocusedPerspectiveStore, MailboxPerspective = _ref.MailboxPerspective, NylasTestUtils = _ref.NylasTestUtils, TaskQueueStatusStore = _ref.TaskQueueStatusStore;

  Categories = require('nylas-observables').Categories;

  describe('CategoryPickerPopover', function() {
    var setupFor, setupForCreateNew;
    beforeEach(function() {
      return CategoryStore._categoryCache = {};
    });
    afterEach(function() {
      return NylasEnv.testOrganizationUnit = null;
    });
    setupFor = function(organizationUnit) {
      var observable;
      NylasEnv.testOrganizationUnit = organizationUnit;
      this.account = {
        id: TEST_ACCOUNT_ID,
        usesLabels: function() {
          return organizationUnit === "label";
        },
        usesFolders: function() {
          return organizationUnit !== "label";
        }
      };
      this.inboxCategory = new Category({
        id: 'id-123',
        name: 'inbox',
        displayName: "INBOX",
        accountId: TEST_ACCOUNT_ID
      });
      this.archiveCategory = new Category({
        id: 'id-456',
        name: 'archive',
        displayName: "ArCHIVe",
        accountId: TEST_ACCOUNT_ID
      });
      this.userCategory = new Category({
        id: 'id-789',
        name: null,
        displayName: "MyCategory",
        accountId: TEST_ACCOUNT_ID
      });
      observable = NylasTestUtils.mockObservable([this.inboxCategory, this.archiveCategory, this.userCategory]);
      observable.sort = (function(_this) {
        return function() {
          return observable;
        };
      })(this);
      spyOn(Categories, "forAccount").andReturn(observable);
      spyOn(CategoryStore, "getStandardCategory").andReturn(this.inboxCategory);
      spyOn(AccountStore, "accountForItems").andReturn(this.account);
      spyOn(Actions, "closePopover");
      return spyOn(FocusedPerspectiveStore, 'current').andCallFake((function(_this) {
        return function() {
          return MailboxPerspective.forCategory(_this.inboxCategory);
        };
      })(this));
    };
    setupForCreateNew = function(orgUnit) {
      if (orgUnit == null) {
        orgUnit = "folder";
      }
      setupFor.call(this, orgUnit);
      this.testThread = new Thread({
        id: 't1',
        subject: "fake",
        accountId: TEST_ACCOUNT_ID,
        categories: []
      });
      return this.picker = ReactTestUtils.renderIntoDocument(React.createElement(CategoryPickerPopover, {
        "threads": [this.testThread],
        "account": this.account
      }));
    };
    describe('when using labels', function() {
      return beforeEach(function() {
        return setupFor.call(this, "label");
      });
    });
    describe('when using folders', function() {
      beforeEach(function() {
        setupFor.call(this, "folder");
        this.testThread = new Thread({
          id: 't1',
          subject: "fake",
          accountId: TEST_ACCOUNT_ID,
          categories: []
        });
        return this.picker = ReactTestUtils.renderIntoDocument(React.createElement(CategoryPickerPopover, {
          "threads": [this.testThread],
          "account": this.account
        }));
      });
      return it('lists the desired categories', function() {
        var data;
        data = this.picker.state.categoryData;
        expect(data.length).toBe(3);
        expect(data[0].id).toBe("id-456");
        expect(data[0].name).toBe("archive");
        expect(data[0].category).toBe(this.archiveCategory);
        expect(data[1].divider).toBe(true);
        expect(data[1].id).toBe("category-divider");
        expect(data[2].id).toBe("id-789");
        expect(data[2].name).toBeUndefined();
        return expect(data[2].category).toBe(this.userCategory);
      });
    });
    describe("'create new' item", function() {
      beforeEach(function() {
        return setupForCreateNew.call(this);
      });
      afterEach(function() {
        return NylasEnv.testOrganizationUnit = null;
      });
      it("is not visible when the search box is empty", function() {
        var count;
        count = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.picker, 'category-create-new').length;
        return expect(count).toBe(0);
      });
      it("is visible when the search box has text", function() {
        var count, inputNode;
        inputNode = ReactDOM.findDOMNode(ReactTestUtils.scryRenderedDOMComponentsWithTag(this.picker, "input")[0]);
        ReactTestUtils.Simulate.change(inputNode, {
          target: {
            value: "calendar"
          }
        });
        count = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.picker, 'category-create-new').length;
        return expect(count).toBe(1);
      });
      return it("shows folder icon if we're using exchange", function() {
        var count, inputNode;
        inputNode = ReactDOM.findDOMNode(ReactTestUtils.scryRenderedDOMComponentsWithTag(this.picker, "input")[0]);
        ReactTestUtils.Simulate.change(inputNode, {
          target: {
            value: "calendar"
          }
        });
        count = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.picker, 'category-create-new-folder').length;
        return expect(count).toBe(1);
      });
    });
    describe("'create new' item with labels", function() {
      beforeEach(function() {
        return setupForCreateNew.call(this, "label");
      });
      return it("shows label icon if we're using gmail", function() {
        var count, inputNode;
        inputNode = ReactDOM.findDOMNode(ReactTestUtils.scryRenderedDOMComponentsWithTag(this.picker, "input")[0]);
        ReactTestUtils.Simulate.change(inputNode, {
          target: {
            value: "calendar"
          }
        });
        count = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.picker, 'category-create-new-tag').length;
        return expect(count).toBe(1);
      });
    });
    return describe("_onSelectCategory", function() {
      beforeEach(function() {
        setupForCreateNew.call(this, "folder");
        spyOn(TaskFactory, 'taskForRemovingCategory').andCallThrough();
        spyOn(TaskFactory, 'taskForApplyingCategory').andCallThrough();
        return spyOn(Actions, "queueTask");
      });
      it("closes the popover", function() {
        this.picker._onSelectCategory({
          usage: 0,
          category: "asdf"
        });
        return expect(Actions.closePopover).toHaveBeenCalled();
      });
      describe("when selecting a category currently on all the selected items", function() {
        return it("fires a task to remove the category", function() {
          var input;
          input = {
            category: "asdf",
            usage: 1
          };
          this.picker._onSelectCategory(input);
          expect(TaskFactory.taskForRemovingCategory).toHaveBeenCalledWith({
            threads: [this.testThread],
            category: "asdf"
          });
          return expect(Actions.queueTask).toHaveBeenCalled();
        });
      });
      describe("when selecting a category not on all the selected items", function() {
        return it("fires a task to add the category", function() {
          var input;
          input = {
            category: "asdf",
            usage: 0
          };
          this.picker._onSelectCategory(input);
          expect(TaskFactory.taskForApplyingCategory).toHaveBeenCalledWith({
            threads: [this.testThread],
            category: "asdf"
          });
          return expect(Actions.queueTask).toHaveBeenCalled();
        });
      });
      return describe("when selecting a new category", function() {
        beforeEach(function() {
          this.input = {
            newCategoryItem: true
          };
          return this.picker.setState({
            searchValue: "teSTing!"
          });
        });
        it("queues a new syncback task for creating a category", function() {
          var newCategory, syncbackTask;
          this.picker._onSelectCategory(this.input);
          expect(Actions.queueTask).toHaveBeenCalled();
          syncbackTask = Actions.queueTask.calls[0].args[0];
          newCategory = syncbackTask.category;
          expect(newCategory instanceof Category).toBe(true);
          expect(newCategory.displayName).toBe("teSTing!");
          return expect(newCategory.accountId).toBe(TEST_ACCOUNT_ID);
        });
        return it("queues a task for applying the category after it has saved", function() {
          var category, resolveSave;
          category = false;
          resolveSave = false;
          spyOn(TaskQueueStatusStore, "waitForPerformRemote").andCallFake(function(task) {
            expect(task instanceof SyncbackCategoryTask).toBe(true);
            return new Promise(function(resolve, reject) {
              return resolveSave = resolve;
            });
          });
          spyOn(DatabaseStore, "findBy").andCallFake(function(klass, _arg) {
            var clientId;
            clientId = _arg.clientId;
            expect(klass).toBe(Category);
            expect(typeof clientId).toBe("string");
            return Promise.resolve(category);
          });
          this.picker._onSelectCategory(this.input);
          waitsFor(function() {
            return Actions.queueTask.callCount > 0;
          });
          runs(function() {
            category = Actions.queueTask.calls[0].args[0].category;
            return resolveSave();
          });
          waitsFor(function() {
            return TaskFactory.taskForApplyingCategory.calls.length === 1;
          });
          return runs(function() {
            return expect(TaskFactory.taskForApplyingCategory).toHaveBeenCalledWith({
              threads: [this.testThread],
              category: category
            });
          });
        });
      });
    });
  });

}).call(this);
