(function() {
  var Actions, ComposerHeaderActions, Fields, React, ReactDOM, ReactTestUtils;

  React = require('react');

  ReactDOM = require('react-dom');

  ComposerHeaderActions = require('../lib/composer-header-actions')["default"];

  Fields = require('../lib/fields')["default"];

  ReactTestUtils = require('react-addons-test-utils');

  Actions = require('nylas-exports').Actions;

  describe("ComposerHeaderActions", function() {
    var makeField;
    makeField = function(props) {
      if (props == null) {
        props = {};
      }
      this.onShowAndFocusField = jasmine.createSpy("onShowAndFocusField");
      props.onShowAndFocusField = this.onShowAndFocusField;
      if (props.enabledFields == null) {
        props.enabledFields = [];
      }
      props.draftClientId = 'a';
      return this.component = ReactTestUtils.renderIntoDocument(React.createElement(ComposerHeaderActions, React.__spread({}, props)));
    };
    it("renders the 'show' buttons for 'cc', 'bcc' when participantsFocused", function() {
      var showBcc, showCc, showSubject;
      makeField.call(this, {
        enabledFields: [Fields.To],
        participantsFocused: true
      });
      showCc = ReactTestUtils.findRenderedDOMComponentWithClass(this.component, "show-cc");
      showBcc = ReactTestUtils.findRenderedDOMComponentWithClass(this.component, "show-bcc");
      showSubject = ReactTestUtils.findRenderedDOMComponentWithClass(this.component, "show-subject");
      expect(showCc).toBeDefined();
      return expect(showBcc).toBeDefined();
    });
    it("does not render the 'show' buttons for 'cc', 'bcc' when participantsFocused is false", function() {
      var showBcc, showCc, showSubject;
      makeField.call(this, {
        enabledFields: [Fields.To],
        participantsFocused: false
      });
      showCc = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.component, "show-cc");
      showBcc = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.component, "show-bcc");
      showSubject = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.component, "show-subject");
      expect(showCc.length).toBe(0);
      return expect(showBcc.length).toBe(0);
    });
    it("hides show cc if it's enabled", function() {
      var els;
      makeField.call(this, {
        enabledFields: [Fields.To, Fields.Cc],
        participantsFocused: true
      });
      els = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.component, "show-cc");
      return expect(els.length).toBe(0);
    });
    it("hides show bcc if it's enabled", function() {
      var els;
      makeField.call(this, {
        enabledFields: [Fields.To, Fields.Bcc],
        participantsFocused: true
      });
      els = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.component, "show-bcc");
      return expect(els.length).toBe(0);
    });
    it("hides show subject if it's enabled", function() {
      var els;
      makeField.call(this, {
        enabledFields: [Fields.To, Fields.Subject],
        participantsFocused: true
      });
      els = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.component, "show-subject");
      return expect(els.length).toBe(0);
    });
    it("renders 'popout composer' in the inline mode", function() {
      var els;
      makeField.call(this, {
        enabledFields: [Fields.To],
        participantsFocused: true
      });
      els = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.component, "show-popout");
      return expect(els.length).toBe(1);
    });
    it("doesn't render 'popout composer' if in a composer window", function() {
      var els;
      spyOn(NylasEnv, 'isComposerWindow').andReturn(true);
      makeField.call(this, {
        enabledFields: [Fields.To],
        participantsFocused: true
      });
      els = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.component, "show-popout");
      return expect(els.length).toBe(0);
    });
    it("pops out the composer when clicked", function() {
      var el;
      spyOn(Actions, "composePopoutDraft");
      makeField.call(this, {
        enabledFields: [Fields.To],
        participantsFocused: true
      });
      el = ReactTestUtils.findRenderedDOMComponentWithClass(this.component, "show-popout");
      ReactTestUtils.Simulate.click(ReactDOM.findDOMNode(el));
      return expect(Actions.composePopoutDraft).toHaveBeenCalled();
    });
    it("shows and focuses cc when clicked", function() {
      var el;
      makeField.call(this, {
        enabledFields: [Fields.To],
        participantsFocused: true
      });
      el = ReactTestUtils.findRenderedDOMComponentWithClass(this.component, "show-cc");
      ReactTestUtils.Simulate.click(ReactDOM.findDOMNode(el));
      return expect(this.onShowAndFocusField).toHaveBeenCalledWith(Fields.Cc);
    });
    it("shows and focuses bcc when clicked", function() {
      var el;
      makeField.call(this, {
        enabledFields: [Fields.To],
        participantsFocused: true
      });
      el = ReactTestUtils.findRenderedDOMComponentWithClass(this.component, "show-bcc");
      ReactTestUtils.Simulate.click(ReactDOM.findDOMNode(el));
      return expect(this.onShowAndFocusField).toHaveBeenCalledWith(Fields.Bcc);
    });
    return it("shows subject when clicked", function() {
      var el;
      makeField.call(this, {
        enabledFields: [Fields.To],
        participantsFocused: false
      });
      el = ReactTestUtils.findRenderedDOMComponentWithClass(this.component, "show-subject");
      ReactTestUtils.Simulate.click(ReactDOM.findDOMNode(el));
      return expect(this.onShowAndFocusField).toHaveBeenCalledWith(Fields.Subject);
    });
  });

}).call(this);
