(function() {
  var Account, AccountStore, Actions, ComponentRegistry, ComposerEditor, ComposerView, Contact, ContactStore, DRAFT_CLIENT_ID, DatabaseStore, DraftEditingSession, DraftStore, Fields, File, FileUploadStore, FocusedContentStore, InjectedComponent, Message, NylasTestUtils, ParticipantsTextField, React, ReactDOM, ReactTestUtils, f1, f2, makeComposer, proxyquire, u1, u2, u3, u4, u5, useDraft, useFullDraft, users, _, _ref, _ref1;

  _ = require("underscore");

  proxyquire = require("proxyquire");

  React = require("react");

  ReactDOM = require('react-dom');

  ReactTestUtils = require('react-addons-test-utils');

  _ref = require("nylas-exports"), Actions = _ref.Actions, File = _ref.File, Contact = _ref.Contact, Message = _ref.Message, Account = _ref.Account, DraftStore = _ref.DraftStore, DatabaseStore = _ref.DatabaseStore, NylasTestUtils = _ref.NylasTestUtils, AccountStore = _ref.AccountStore, FileUploadStore = _ref.FileUploadStore, ContactStore = _ref.ContactStore, FocusedContentStore = _ref.FocusedContentStore, ComponentRegistry = _ref.ComponentRegistry;

  _ref1 = require('nylas-component-kit'), InjectedComponent = _ref1.InjectedComponent, ParticipantsTextField = _ref1.ParticipantsTextField;

  DraftEditingSession = require('../../../src/flux/stores/draft-editing-session');

  ComposerEditor = require('../lib/composer-editor')["default"];

  Fields = require('../lib/fields')["default"];

  u1 = new Contact({
    name: "Christine Spang",
    email: "spang@nylas.com"
  });

  u2 = new Contact({
    name: "Michael Grinich",
    email: "mg@nylas.com"
  });

  u3 = new Contact({
    name: "Evan Morikawa",
    email: "evan@nylas.com"
  });

  u4 = new Contact({
    name: "Zoë Leiper",
    email: "zip@nylas.com"
  });

  u5 = new Contact({
    name: "Ben Gotow",
    email: "ben@nylas.com"
  });

  f1 = new File({
    id: 'file_1_id',
    filename: 'a.png',
    contentType: 'image/png',
    size: 10,
    object: "file"
  });

  f2 = new File({
    id: 'file_2_id',
    filename: 'b.pdf',
    contentType: '',
    size: 999999,
    object: "file"
  });

  users = [u1, u2, u3, u4, u5];

  ComposerView = require("../lib/composer-view")["default"];

  DRAFT_CLIENT_ID = "local-123";

  useDraft = function(draftAttributes) {
    if (draftAttributes == null) {
      draftAttributes = {};
    }
    this.draft = new Message(_.extend({
      draft: true,
      body: ""
    }, draftAttributes));
    this.draft.clientId = DRAFT_CLIENT_ID;
    this.session = new DraftEditingSession(DRAFT_CLIENT_ID, this.draft);
    return DraftStore._draftSessions[DRAFT_CLIENT_ID] = this.session;
  };

  useFullDraft = function() {
    return useDraft.call(this, {
      from: [AccountStore.accounts()[0].me()],
      to: [u2],
      cc: [u3, u4],
      bcc: [u5],
      files: [f1, f2],
      subject: "Test Message 1",
      body: "Hello <b>World</b><br/> This is a test",
      replyToMessageId: null
    });
  };

  makeComposer = function(props) {
    if (props == null) {
      props = {};
    }
    this.composer = NylasTestUtils.renderIntoDocument(React.createElement(ComposerView, React.__spread({
      "draft": this.draft,
      "session": this.session
    }, props)));
    return advanceClock();
  };

  describe("ComposerView", function() {
    beforeEach(function() {
      ComposerEditor.containerRequired = false;
      ComponentRegistry.register(ComposerEditor, {
        role: "Composer:Editor"
      });
      this.isSending = false;
      spyOn(DraftStore, "isSendingDraft").andCallFake((function(_this) {
        return function() {
          return _this.isSending;
        };
      })(this));
      spyOn(DraftEditingSession.prototype, 'changeSetCommit').andCallFake((function(_this) {
        return function(draft) {
          return _this.draft = draft;
        };
      })(this));
      spyOn(ContactStore, "searchContacts").andCallFake((function(_this) {
        return function(email) {
          return _.filter(users, u(u.email.toLowerCase() === email.toLowerCase()));
        };
      })(this));
      return spyOn(Contact.prototype, "isValid").andCallFake(function(contact) {
        return this.email.indexOf('@') > 0;
      });
    });
    afterEach(function() {
      ComposerEditor.containerRequired = void 0;
      ComponentRegistry.unregister(ComposerEditor);
      DraftStore._cleanupAllSessions();
      return NylasTestUtils.removeFromDocument(this.composer);
    });
    describe("when sending a new message", function() {
      return it('makes a request with the message contents', function() {
        var body, editableNode;
        useDraft.call(this);
        makeComposer.call(this);
        editableNode = ReactDOM.findDOMNode(this.composer).querySelector('[contenteditable]');
        spyOn(this.session.changes, "add");
        editableNode.innerHTML = "Hello <strong>world</strong>";
        this.composer.refs[Fields.Body]._onDOMMutated(["mutated"]);
        expect(this.session.changes.add).toHaveBeenCalled();
        expect(this.session.changes.add.calls.length).toBe(1);
        body = this.session.changes.add.calls[0].args[0].body;
        return expect(body).toBe("<head></head><body>Hello <strong>world</strong></body>");
      });
    });
    describe("when sending a reply-to message", function() {
      beforeEach(function() {
        this.replyBody = "<blockquote class=\"gmail_quote\">On Sep 3 2015, at 12:14 pm, Evan Morikawa &lt;evan@evanmorikawa.com&gt; wrote:<br>This is a test!</blockquote>";
        useDraft.call(this, {
          from: [u1],
          to: [u2],
          subject: "Test Reply Message 1",
          body: this.replyBody
        });
        makeComposer.call(this);
        this.editableNode = ReactDOM.findDOMNode(this.composer).querySelector('[contenteditable]');
        return spyOn(this.session.changes, "add");
      });
      it('begins with the replying message collapsed', function() {
        return expect(this.editableNode.innerHTML).toBe("");
      });
      return it('saves the full new body, plus quoted text', function() {
        var body;
        this.editableNode.innerHTML = "Hello <strong>world</strong>";
        this.composer.refs[Fields.Body]._onDOMMutated(["mutated"]);
        expect(this.session.changes.add).toHaveBeenCalled();
        expect(this.session.changes.add.calls.length).toBe(1);
        body = this.session.changes.add.calls[0].args[0].body;
        return expect(body).toBe("<head></head><body>Hello <strong>world</strong>" + this.replyBody + "</body>");
      });
    });
    describe("when sending a forwarded message message", function() {
      beforeEach(function() {
        this.fwdBody = "<br><br><blockquote class=\"gmail_quote\" style=\"margin:0 0 0 .8ex;border-left:1px #ccc solid;padding-left:1ex;\">\n  ---------- Forwarded message ---------\n  <br><br>\n  From: Evan Morikawa &lt;evan@evanmorikawa.com&gt;<br>Subject: Test Forward Message 1<br>Date: Sep 3 2015, at 12:14 pm<br>To: Evan Morikawa &lt;evan@nylas.com&gt;\n  <br><br>\n\n<meta content=\"text/html; charset=us-ascii\">This is a test!\n</blockquote>";
        useDraft.call(this, {
          from: [u1],
          to: [u2],
          subject: "Fwd: Test Forward Message 1",
          body: this.fwdBody
        });
        makeComposer.call(this);
        this.editableNode = ReactDOM.findDOMNode(this.composer).querySelector('[contenteditable]');
        return spyOn(this.session.changes, "add");
      });
      it('begins with the forwarded message expanded', function() {
        return expect(this.editableNode.innerHTML).toBe(this.fwdBody);
      });
      return it('saves the full new body, plus forwarded text', function() {
        var body;
        this.editableNode.innerHTML = "Hello <strong>world</strong>" + this.fwdBody;
        this.composer.refs[Fields.Body]._onDOMMutated(["mutated"]);
        expect(this.session.changes.add).toHaveBeenCalled();
        expect(this.session.changes.add.calls.length).toBe(1);
        body = this.session.changes.add.calls[0].args[0].body;
        return expect(body).toBe("Hello <strong>world</strong>" + this.fwdBody);
      });
    });
    describe("When sending a message", function() {
      beforeEach(function() {
        var remote;
        spyOn(NylasEnv, "isMainWindow").andReturn(true);
        remote = require('electron').remote;
        this.dialog = remote.dialog;
        spyOn(remote, "getCurrentWindow");
        spyOn(this.dialog, "showMessageBox");
        return spyOn(Actions, "sendDraft");
      });
      it("shows an error if there are no recipients", function() {
        var dialogArgs, status;
        useDraft.call(this, {
          subject: "no recipients"
        });
        makeComposer.call(this);
        status = this.composer._isValidDraft();
        expect(status).toBe(false);
        expect(this.dialog.showMessageBox).toHaveBeenCalled();
        dialogArgs = this.dialog.showMessageBox.mostRecentCall.args[1];
        expect(dialogArgs.detail).toEqual("You need to provide one or more recipients before sending the message.");
        return expect(dialogArgs.buttons).toEqual(['Edit Message', 'Cancel']);
      });
      it("shows an error if a recipient is invalid", function() {
        var dialogArgs, status;
        useDraft.call(this, {
          subject: 'hello world!',
          to: [
            new Contact({
              email: 'lol',
              name: 'lol'
            })
          ]
        });
        makeComposer.call(this);
        status = this.composer._isValidDraft();
        expect(status).toBe(false);
        expect(this.dialog.showMessageBox).toHaveBeenCalled();
        dialogArgs = this.dialog.showMessageBox.mostRecentCall.args[1];
        expect(dialogArgs.detail).toEqual("lol is not a valid email address - please remove or edit it before sending.");
        return expect(dialogArgs.buttons).toEqual(['Edit Message', 'Cancel']);
      });
      describe("empty body warning", function() {
        it("warns if the body of the email is still the pristine body", function() {
          var dialogArgs, pristineBody, status;
          pristineBody = "<head></head><body><br><br></body>";
          useDraft.call(this, {
            to: [u1],
            subject: "Hello World",
            body: pristineBody
          });
          makeComposer.call(this);
          spyOn(this.session, 'draftPristineBody').andCallFake(function() {
            return pristineBody;
          });
          status = this.composer._isValidDraft();
          expect(status).toBe(false);
          expect(this.dialog.showMessageBox).toHaveBeenCalled();
          dialogArgs = this.dialog.showMessageBox.mostRecentCall.args[1];
          return expect(dialogArgs.buttons).toEqual(['Send Anyway', 'Cancel']);
        });
        it("does not warn if the body of the email is all quoted text, but the email is a forward", function() {
          var status;
          useDraft.call(this, {
            to: [u1],
            subject: "Fwd: Hello World",
            body: "<br><br><blockquote class='gmail_quote'>This is my quoted text!</blockquote>"
          });
          makeComposer.call(this);
          status = this.composer._isValidDraft();
          return expect(status).toBe(true);
        });
        return it("does not warn if the user has attached a file", function() {
          var status;
          useDraft.call(this, {
            to: [u1],
            subject: "Hello World",
            body: "",
            files: [f1]
          });
          makeComposer.call(this);
          status = this.composer._isValidDraft();
          expect(status).toBe(true);
          return expect(this.dialog.showMessageBox).not.toHaveBeenCalled();
        });
      });
      it("shows a warning if there's no subject", function() {
        var dialogArgs, status;
        useDraft.call(this, {
          to: [u1],
          subject: ""
        });
        makeComposer.call(this);
        status = this.composer._isValidDraft();
        expect(status).toBe(false);
        expect(this.dialog.showMessageBox).toHaveBeenCalled();
        dialogArgs = this.dialog.showMessageBox.mostRecentCall.args[1];
        return expect(dialogArgs.buttons).toEqual(['Send Anyway', 'Cancel']);
      });
      it("doesn't show a warning if requirements are satisfied", function() {
        var status;
        useFullDraft.apply(this);
        makeComposer.call(this);
        status = this.composer._isValidDraft();
        expect(status).toBe(true);
        return expect(this.dialog.showMessageBox).not.toHaveBeenCalled();
      });
      describe("Checking for attachments", function() {
        var noWarn, warn;
        warn = function(body) {
          var dialogArgs, status;
          useDraft.call(this, {
            subject: "Subject",
            to: [u1],
            body: body
          });
          makeComposer.call(this);
          status = this.composer._isValidDraft();
          expect(status).toBe(false);
          expect(this.dialog.showMessageBox).toHaveBeenCalled();
          dialogArgs = this.dialog.showMessageBox.mostRecentCall.args[1];
          return expect(dialogArgs.buttons).toEqual(['Send Anyway', 'Cancel']);
        };
        noWarn = function(body) {
          var status;
          useDraft.call(this, {
            subject: "Subject",
            to: [u1],
            body: body
          });
          makeComposer.call(this);
          status = this.composer._isValidDraft();
          expect(status).toBe(true);
          return expect(this.dialog.showMessageBox).not.toHaveBeenCalled();
        };
        it("warns", function() {
          return warn.call(this, "Check out the attached file");
        });
        it("warns", function() {
          return warn.call(this, "I've added an attachment");
        });
        it("warns", function() {
          return warn.call(this, "I'm going to attach the file");
        });
        it("warns", function() {
          return warn.call(this, "Hey attach me <blockquote class='gmail_quote'>sup</blockquote>");
        });
        it("doesn't warn", function() {
          return noWarn.call(this, "sup yo");
        });
        it("doesn't warn", function() {
          return noWarn.call(this, "Look at the file");
        });
        return it("doesn't warn", function() {
          return noWarn.call(this, "Hey there <blockquote class='gmail_quote'>attach</blockquote>");
        });
      });
      it("doesn't show a warning if you've attached a file", function() {
        var status;
        useDraft.call(this, {
          subject: "Subject",
          to: [u1],
          body: "Check out attached file",
          files: [f1]
        });
        makeComposer.call(this);
        status = this.composer._isValidDraft();
        expect(status).toBe(true);
        return expect(this.dialog.showMessageBox).not.toHaveBeenCalled();
      });
      it("bypasses the warning if force bit is set", function() {
        var status;
        useDraft.call(this, {
          to: [u1],
          subject: ""
        });
        makeComposer.call(this);
        status = this.composer._isValidDraft({
          force: true
        });
        expect(status).toBe(true);
        return expect(this.dialog.showMessageBox).not.toHaveBeenCalled();
      });
      it("sends when you click the send button", function() {
        var sendBtn;
        useFullDraft.apply(this);
        makeComposer.call(this);
        sendBtn = this.composer.refs.sendActionButton;
        sendBtn.primaryClick();
        expect(Actions.sendDraft).toHaveBeenCalledWith(DRAFT_CLIENT_ID);
        return expect(Actions.sendDraft.calls.length).toBe(1);
      });
      it("doesn't send twice if you double click", function() {
        var sendBtn;
        useFullDraft.apply(this);
        makeComposer.call(this);
        sendBtn = this.composer.refs.sendActionButton;
        sendBtn.primaryClick();
        this.isSending = true;
        DraftStore.trigger();
        sendBtn.primaryClick();
        expect(Actions.sendDraft).toHaveBeenCalledWith(DRAFT_CLIENT_ID);
        return expect(Actions.sendDraft.calls.length).toBe(1);
      });
      return describe("when sending a message with keyboard inputs", function() {
        beforeEach(function() {
          useFullDraft.apply(this);
          makeComposer.call(this);
          return this.$composer = this.composer.refs.composerWrap;
        });
        it("sends the draft on cmd-enter", function() {
          var mod;
          if (process.platform === "darwin") {
            mod = 'command';
          } else {
            mod = 'ctrl';
          }
          ReactDOM.findDOMNode(this.$composer).dispatchEvent(new CustomEvent('composer:send-message'));
          expect(Actions.sendDraft).toHaveBeenCalled();
          return expect(Actions.sendDraft.calls.length).toBe(1);
        });
        return it("doesn't let you send twice", function() {
          ReactDOM.findDOMNode(this.$composer).dispatchEvent(new CustomEvent('composer:send-message'));
          expect(Actions.sendDraft).toHaveBeenCalled();
          expect(Actions.sendDraft.calls.length).toBe(1);
          this.isSending = true;
          DraftStore.trigger();
          ReactDOM.findDOMNode(this.$composer).dispatchEvent(new CustomEvent('composer:send-message'));
          expect(Actions.sendDraft).toHaveBeenCalled();
          return expect(Actions.sendDraft.calls.length).toBe(1);
        });
      });
    });
    describe("drag and drop", function() {
      beforeEach(function() {
        useDraft.call(this, {
          to: [u1],
          subject: "Hello World",
          body: "",
          files: [f1]
        });
        return makeComposer.call(this);
      });
      describe("_shouldAcceptDrop", function() {
        it("should return true if the event is carrying native files", function() {
          var event;
          event = {
            dataTransfer: {
              files: [
                {
                  'pretend': 'imafile'
                }
              ],
              types: []
            }
          };
          return expect(this.composer._shouldAcceptDrop(event)).toBe(true);
        });
        it("should return true if the event is carrying a non-native file URL", function() {
          var event;
          event = {
            dataTransfer: {
              files: [],
              types: ['text/uri-list']
            }
          };
          spyOn(this.composer, '_nonNativeFilePathForDrop').andReturn("file://one-file");
          expect(this.composer._shouldAcceptDrop(event)).toBe(true);
          return expect(this.draft.files.length).toBe(1);
        });
        return it("should return false otherwise", function() {
          var event;
          event = {
            dataTransfer: {
              files: [],
              types: ['text/plain']
            }
          };
          return expect(this.composer._shouldAcceptDrop(event)).toBe(false);
        });
      });
      return describe("_nonNativeFilePathForDrop", function() {
        it("should return a path in the text/nylas-file-url data", function() {
          var event;
          event = {
            dataTransfer: {
              types: ['text/nylas-file-url'],
              getData: function() {
                return "image/png:test.png:file:///Users/bengotow/Desktop/test.png";
              }
            }
          };
          return expect(this.composer._nonNativeFilePathForDrop(event)).toBe("/Users/bengotow/Desktop/test.png");
        });
        it("should return a path in the text/uri-list data", function() {
          var event;
          event = {
            dataTransfer: {
              types: ['text/uri-list'],
              getData: function() {
                return "file:///Users/bengotow/Desktop/test.png";
              }
            }
          };
          return expect(this.composer._nonNativeFilePathForDrop(event)).toBe("/Users/bengotow/Desktop/test.png");
        });
        it("should return null otherwise", function() {
          var event;
          event = {
            dataTransfer: {
              types: ['text/plain'],
              getData: function() {
                return "Hello world";
              }
            }
          };
          return expect(this.composer._nonNativeFilePathForDrop(event)).toBe(null);
        });
        it("should urldecode the contents of the text/uri-list field", function() {
          var event;
          event = {
            dataTransfer: {
              types: ['text/uri-list'],
              getData: function() {
                return "file:///Users/bengotow/Desktop/Screen%20shot.png";
              }
            }
          };
          return expect(this.composer._nonNativeFilePathForDrop(event)).toBe("/Users/bengotow/Desktop/Screen shot.png");
        });
        it("should return null if text/uri-list contains a non-file path", function() {
          var event;
          event = {
            dataTransfer: {
              types: ['text/uri-list'],
              getData: function() {
                return "http://apple.com";
              }
            }
          };
          return expect(this.composer._nonNativeFilePathForDrop(event)).toBe(null);
        });
        return it("should return null if text/nylas-file-url contains a non-file path", function() {
          var event;
          event = {
            dataTransfer: {
              types: ['text/nylas-file-url'],
              getData: function() {
                return "application/json:filename.json:undefined";
              }
            }
          };
          return expect(this.composer._nonNativeFilePathForDrop(event)).toBe(null);
        });
      });
    });
    return describe("A draft with files (attachments) and uploads", function() {
      beforeEach(function() {
        this.file1 = new File({
          id: "f_1",
          filename: "f1.pdf",
          size: 1230
        });
        this.file2 = new File({
          id: "f_2",
          filename: "f2.jpg",
          size: 4560
        });
        this.file3 = new File({
          id: "f_3",
          filename: "f3.png",
          size: 7890
        });
        spyOn(Actions, "fetchFile");
        useDraft.call(this, {
          files: [this.file1, this.file2]
        });
        return makeComposer.call(this);
      });
      it('starts fetching attached files', function() {
        waitsFor(function() {
          return Actions.fetchFile.callCount === 1;
        });
        return runs(function() {
          expect(Actions.fetchFile).toHaveBeenCalled();
          expect(Actions.fetchFile.calls.length).toBe(1);
          return expect(Actions.fetchFile.calls[0].args[0]).toBe(this.file2);
        });
      });
      it('injects an Attachment component for non image files', function() {
        var els;
        els = ReactTestUtils.scryRenderedComponentsWithTypeAndProps(this.composer, InjectedComponent, {
          matching: {
            role: "Attachment"
          }
        });
        return expect(els.length).toBe(1);
      });
      return it('injects an Attachment:Image component for image files', function() {
        var els;
        els = ReactTestUtils.scryRenderedComponentsWithTypeAndProps(this.composer, InjectedComponent, {
          matching: {
            role: "Attachment:Image"
          }
        });
        return expect(els.length).toBe(1);
      });
    });
  });

  describe("when the DraftStore `isSending` isn't stubbed out", function() {
    beforeEach(function() {
      return DraftStore._draftsSending = {};
    });
    it("doesn't send twice in a popout", function() {
      var firstStatus, secondStatus;
      spyOn(Actions, "queueTask");
      spyOn(Actions, "sendDraft").andCallThrough();
      useFullDraft.call(this);
      makeComposer.call(this);
      firstStatus = this.composer._isValidDraft();
      expect(firstStatus).toBe(true);
      Actions.sendDraft(DRAFT_CLIENT_ID);
      secondStatus = this.composer._isValidDraft();
      return expect(secondStatus).toBe(false);
    });
    return it("doesn't send twice in the main window", function() {
      var firstStatus, secondStatus;
      spyOn(Actions, "queueTask");
      spyOn(Actions, "sendDraft").andCallThrough();
      spyOn(NylasEnv, "isMainWindow").andReturn(true);
      useFullDraft.call(this);
      makeComposer.call(this);
      firstStatus = this.composer._isValidDraft();
      expect(firstStatus).toBe(true);
      Actions.sendDraft(DRAFT_CLIENT_ID);
      secondStatus = this.composer._isValidDraft();
      return expect(secondStatus).toBe(false);
    });
  });

}).call(this);
