(function() {
  var ComponentRegistry, Composer, ComposerEditor, DraftStore, Fields, Message, React, ReactDOM, ReactTestUtils, _, _ref;

  _ = require("underscore");

  React = require("react");

  ReactDOM = require('react-dom');

  ReactTestUtils = require('react-addons-test-utils');

  Fields = require('../lib/fields')["default"];

  Composer = require("../lib/composer-view")["default"];

  ComposerEditor = require('../lib/composer-editor')["default"];

  _ref = require('nylas-exports'), Message = _ref.Message, DraftStore = _ref.DraftStore, ComponentRegistry = _ref.ComponentRegistry;

  describe("Composer Quoted Text", function() {
    var setHTML;
    beforeEach(function() {
      ComponentRegistry.register(ComposerEditor, {
        role: "Composer:Editor"
      });
      this.onChange = jasmine.createSpy('onChange');
      this.htmlNoQuote = 'Test <strong>HTML</strong><br>';
      this.htmlWithQuote = 'Test <strong>HTML</strong><br><blockquote class="gmail_quote">QUOTE</blockquote>';
      this.draft = new Message({
        draft: true,
        clientId: "client-123"
      });
      return this.session = {
        trigger: function() {},
        changes: {
          add: jasmine.createSpy('changes.add')
        },
        draft: (function(_this) {
          return function() {
            return _this.draft;
          };
        })(this)
      };
    });
    afterEach(function() {
      DraftStore._cleanupAllSessions();
      ComposerEditor.containerRequired = void 0;
      return ComponentRegistry.unregister(ComposerEditor);
    });
    setHTML = function(newHTML) {
      this.$contentEditable.innerHTML = newHTML;
      return this.contentEditable._onDOMMutated(["mutated"]);
    };
    describe("quoted-text-control toggle button", function() {});
    describe("when there's no quoted text", function() {
      beforeEach(function() {
        this.draft.body = this.htmlNoQuote;
        this.composer = ReactTestUtils.renderIntoDocument(React.createElement(Composer, {
          "draft": this.draft,
          "session": this.session
        }));
        this.composer.setState({
          showQuotedText: true
        });
        this.contentEditable = this.composer.refs[Fields.Body];
        this.$contentEditable = ReactDOM.findDOMNode(this.contentEditable).querySelector('[contenteditable]');
        return this.$composerBodyWrap = ReactDOM.findDOMNode(this.composer.refs.composerBodyWrap);
      });
      it('should not display any quoted text', function() {
        return expect(this.$contentEditable.innerHTML).toBe(this.htmlNoQuote);
      });
      it("allows the text to update", function() {
        var ev, textToAdd;
        textToAdd = "MORE <strong>TEXT</strong>!";
        expect(this.$contentEditable.innerHTML).toBe(this.htmlNoQuote);
        setHTML.call(this, textToAdd + this.htmlNoQuote);
        ev = this.session.changes.add.mostRecentCall.args[0].body;
        return expect(ev).toEqual(textToAdd + this.htmlNoQuote);
      });
      return it('should not render the quoted-text-control toggle', function() {
        var toggles;
        toggles = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.composer, 'quoted-text-control');
        return expect(toggles.length).toBe(0);
      });
    });
    describe('when there is quoted text, and showQuotedText is true', function() {
      beforeEach(function() {
        this.draft.body = this.htmlWithQuote;
        this.composer = ReactTestUtils.renderIntoDocument(React.createElement(Composer, {
          "draft": this.draft,
          "session": this.session
        }));
        this.composer.setState({
          showQuotedText: true
        });
        this.contentEditable = this.composer.refs[Fields.Body];
        this.$contentEditable = ReactDOM.findDOMNode(this.contentEditable).querySelector('[contenteditable]');
        return this.$composerBodyWrap = ReactDOM.findDOMNode(this.composer.refs.composerBodyWrap);
      });
      it('should display the quoted text', function() {
        return expect(this.$contentEditable.innerHTML).toBe(this.htmlWithQuote);
      });
      it("should call add changes with the entire HTML string", function() {
        var ev, textToAdd;
        textToAdd = "MORE <strong>TEXT</strong>!";
        expect(this.$contentEditable.innerHTML).toBe(this.htmlWithQuote);
        setHTML.call(this, textToAdd + this.htmlWithQuote);
        ev = this.session.changes.add.mostRecentCall.args[0].body;
        return expect(ev).toEqual(textToAdd + this.htmlWithQuote);
      });
      it("should allow the quoted text to be changed", function() {
        var ev, newText;
        newText = 'Test <strong>NEW 1 HTML</strong><blockquote class="gmail_quote">QUOTE CHANGED!!!</blockquote>';
        expect(this.$contentEditable.innerHTML).toBe(this.htmlWithQuote);
        setHTML.call(this, newText);
        ev = this.session.changes.add.mostRecentCall.args[0].body;
        return expect(ev).toEqual(newText);
      });
      return describe('quoted text control toggle button', function() {
        beforeEach(function() {
          return this.toggle = ReactTestUtils.findRenderedDOMComponentWithClass(this.composer, 'quoted-text-control');
        });
        return it('should be rendered', function() {
          return expect(this.toggle).toBeDefined();
        });
      });
    });
    return describe('when there is quoted text, an showQuotedText is false', function() {
      var wrapBody;
      beforeEach(function() {
        this.draft.body = this.htmlWithQuote;
        this.composer = ReactTestUtils.renderIntoDocument(React.createElement(Composer, {
          "draft": this.draft,
          "session": this.session
        }));
        this.composer.setState({
          showQuotedText: false
        });
        this.contentEditable = this.composer.refs[Fields.Body];
        this.$contentEditable = ReactDOM.findDOMNode(this.contentEditable).querySelector('[contenteditable]');
        return this.$composerBodyWrap = ReactDOM.findDOMNode(this.composer.refs.composerBodyWrap);
      });
      wrapBody = function(html) {
        return "<head></head><body>" + html + "</body>";
      };
      it('should not display any quoted text', function() {
        return expect(this.$contentEditable.innerHTML).toBe(this.htmlNoQuote);
      });
      it("should let you change the text, and then append the quoted text part to the end before firing adding changes", function() {
        var ev, textToAdd;
        textToAdd = "MORE <strong>TEXT</strong>!";
        expect(this.$contentEditable.innerHTML).toBe(this.htmlNoQuote);
        setHTML.call(this, textToAdd + this.htmlNoQuote);
        ev = this.session.changes.add.mostRecentCall.args[0].body;
        return expect(ev).toEqual(wrapBody(textToAdd + this.htmlWithQuote));
      });
      it("should let you add more html that looks like quoted text, and still properly appends the old quoted text", function() {
        var ev, textToAdd;
        textToAdd = "Yo <blockquote class=\"gmail_quote\">I'm a fake quote</blockquote>";
        expect(this.$contentEditable.innerHTML).toBe(this.htmlNoQuote);
        setHTML.call(this, textToAdd + this.htmlNoQuote);
        ev = this.session.changes.add.mostRecentCall.args[0].body;
        return expect(ev).toEqual(wrapBody(textToAdd + this.htmlWithQuote));
      });
      return describe('quoted text control toggle button', function() {
        beforeEach(function() {
          return this.toggle = ReactTestUtils.findRenderedDOMComponentWithClass(this.composer, 'quoted-text-control');
        });
        return it('should be rendered', function() {
          return expect(this.toggle).toBeDefined();
        });
      });
    });
  });

}).call(this);
