(function() {
  var Actions, ButtonDropdown, ComposerExtension, ExtensionRegistry, GoodExtension, Message, NAExtension, NullExtension, React, ReactDOM, ReactTestUtils, RetinaImg, SecondExtension, SendActionButton, isValidDraft, _ref, _ref1,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require("react");

  ReactDOM = require('react-dom');

  ReactTestUtils = require('react-addons-test-utils');

  SendActionButton = require('../lib/send-action-button')["default"];

  _ref = require('nylas-exports'), Actions = _ref.Actions, Message = _ref.Message, ComposerExtension = _ref.ComposerExtension, ExtensionRegistry = _ref.ExtensionRegistry;

  _ref1 = require('nylas-component-kit'), ButtonDropdown = _ref1.ButtonDropdown, RetinaImg = _ref1.RetinaImg;

  NAExtension = (function(_super) {
    __extends(NAExtension, _super);

    function NAExtension() {
      return NAExtension.__super__.constructor.apply(this, arguments);
    }

    return NAExtension;

  })(ComposerExtension);

  GoodExtension = (function(_super) {
    __extends(GoodExtension, _super);

    function GoodExtension() {
      return GoodExtension.__super__.constructor.apply(this, arguments);
    }

    GoodExtension.sendActionConfig = function(_arg) {
      var draft;
      draft = _arg.draft;
      return {
        title: "Good Extension",
        content: function() {
          return React.createElement("div", {
            "className": "btn-good"
          });
        },
        onSend: function() {}
      };
    };

    return GoodExtension;

  })(ComposerExtension);

  SecondExtension = (function(_super) {
    __extends(SecondExtension, _super);

    function SecondExtension() {
      return SecondExtension.__super__.constructor.apply(this, arguments);
    }

    SecondExtension.sendActionConfig = function(_arg) {
      var draft;
      draft = _arg.draft;
      return {
        title: "Second Extension",
        content: function() {
          return React.createElement("div", {
            "className": "btn-second"
          });
        },
        onSend: function() {}
      };
    };

    return SecondExtension;

  })(ComposerExtension);

  NullExtension = (function(_super) {
    __extends(NullExtension, _super);

    function NullExtension() {
      return NullExtension.__super__.constructor.apply(this, arguments);
    }

    NullExtension.sendActionConfig = function(_arg) {
      var draft;
      draft = _arg.draft;
      return null;
    };

    return NullExtension;

  })(ComposerExtension);

  isValidDraft = null;

  describe("SendActionButton", function() {
    var render;
    render = function(draft, valid) {
      if (valid == null) {
        valid = true;
      }
      isValidDraft = jasmine.createSpy("isValidDraft").andReturn(valid);
      return ReactTestUtils.renderIntoDocument(React.createElement(SendActionButton, {
        "draft": draft,
        "isValidDraft": isValidDraft
      }));
    };
    beforeEach(function() {
      spyOn(NylasEnv, "reportError");
      spyOn(Actions, "sendDraft");
      this.clientId = "client-23";
      return this.draft = new Message({
        clientId: this.clientId,
        draft: true
      });
    });
    it("renders without error", function() {
      this.sendActionButton = render(this.draft);
      return expect(ReactTestUtils.isCompositeComponentWithType(this.sendActionButton, SendActionButton)).toBe(true);
    });
    it("is a single button when there are no extensions", function() {
      var buttons, dropdowns;
      spyOn(ExtensionRegistry.Composer, "extensions").andReturn([]);
      this.sendActionButton = render(this.draft);
      dropdowns = ReactTestUtils.scryRenderedComponentsWithType(this.sendActionButton, ButtonDropdown);
      buttons = ReactTestUtils.scryRenderedDOMComponentsWithTag(this.sendActionButton, "button");
      expect(buttons.length).toBe(1);
      return expect(dropdowns.length).toBe(0);
    });
    it("is a dropdown when there's another valid extension", function() {
      var buttons, dropdowns;
      spyOn(ExtensionRegistry.Composer, "extensions").andReturn([GoodExtension]);
      this.sendActionButton = render(this.draft);
      dropdowns = ReactTestUtils.scryRenderedComponentsWithType(this.sendActionButton, ButtonDropdown);
      buttons = ReactTestUtils.scryRenderedDOMComponentsWithTag(this.sendActionButton, "button");
      expect(buttons.length).toBe(0);
      return expect(dropdowns.length).toBe(1);
    });
    it("has the correct primary item", function() {
      var dropdown;
      spyOn(ExtensionRegistry.Composer, "extensions").andReturn([GoodExtension, SecondExtension]);
      spyOn(NylasEnv.config, 'get').andReturn('second-extension');
      this.sendActionButton = render(this.draft);
      dropdown = ReactTestUtils.findRenderedComponentWithType(this.sendActionButton, ButtonDropdown);
      return expect(dropdown.props.primaryTitle).toBe("Second Extension");
    });
    it("falls back to a default if the primary item can't be found", function() {
      var dropdown;
      spyOn(ExtensionRegistry.Composer, "extensions").andReturn([GoodExtension, SecondExtension]);
      spyOn(NylasEnv.config, 'get').andReturn('does-not-exist');
      this.sendActionButton = render(this.draft);
      dropdown = ReactTestUtils.findRenderedComponentWithType(this.sendActionButton, ButtonDropdown);
      return expect(dropdown.props.primaryTitle).toBe("Send");
    });
    it("is a single button when a valid extension returns null", function() {
      var buttons, dropdowns;
      spyOn(ExtensionRegistry.Composer, "extensions").andReturn([NullExtension]);
      this.sendActionButton = render(this.draft);
      dropdowns = ReactTestUtils.scryRenderedComponentsWithType(this.sendActionButton, ButtonDropdown);
      buttons = ReactTestUtils.scryRenderedDOMComponentsWithTag(this.sendActionButton, "button");
      expect(buttons.length).toBe(1);
      return expect(dropdowns.length).toBe(0);
    });
    it("still renders but catches when an extension is missing a title", function() {
      var NoTitle, buttons, dropdowns;
      NoTitle = (function(_super) {
        __extends(NoTitle, _super);

        function NoTitle() {
          return NoTitle.__super__.constructor.apply(this, arguments);
        }

        NoTitle.sendActionConfig = function(_arg) {
          var draft;
          draft = _arg.draft;
          return {
            title: null,
            iconUrl: "nylas://foo/bar/baz",
            onSend: function() {}
          };
        };

        return NoTitle;

      })(ComposerExtension);
      spyOn(ExtensionRegistry.Composer, "extensions").andReturn([NoTitle]);
      this.sendActionButton = render(this.draft);
      dropdowns = ReactTestUtils.scryRenderedComponentsWithType(this.sendActionButton, ButtonDropdown);
      buttons = ReactTestUtils.scryRenderedDOMComponentsWithTag(this.sendActionButton, "button");
      expect(buttons.length).toBe(1);
      expect(dropdowns.length).toBe(0);
      expect(NylasEnv.reportError).toHaveBeenCalled();
      return expect(NylasEnv.reportError.calls[0].args[0].message).toMatch(/title/);
    });
    it("still renders with a null iconUrl and doesn't show the image", function() {
      var NoIconUrl, buttons, dropdowns, icons;
      NoIconUrl = (function(_super) {
        __extends(NoIconUrl, _super);

        function NoIconUrl() {
          return NoIconUrl.__super__.constructor.apply(this, arguments);
        }

        NoIconUrl.sendActionConfig = function(_arg) {
          var draft;
          draft = _arg.draft;
          return {
            title: "some title",
            iconUrl: null,
            onSend: function() {}
          };
        };

        return NoIconUrl;

      })(ComposerExtension);
      spyOn(ExtensionRegistry.Composer, "extensions").andReturn([NoIconUrl]);
      spyOn(NylasEnv.config, 'get').andReturn('some-title');
      this.sendActionButton = render(this.draft);
      dropdowns = ReactTestUtils.scryRenderedComponentsWithType(this.sendActionButton, ButtonDropdown);
      icons = ReactTestUtils.scryRenderedComponentsWithType(this.sendActionButton, RetinaImg);
      buttons = ReactTestUtils.scryRenderedDOMComponentsWithTag(this.sendActionButton, "button");
      expect(buttons.length).toBe(0);
      expect(dropdowns.length).toBe(1);
      return expect(icons.length).toBe(3);
    });
    it("still renders but catches when an extension is missing an onSend", function() {
      var NoClick, buttons, dropdowns;
      NoClick = (function(_super) {
        __extends(NoClick, _super);

        function NoClick() {
          return NoClick.__super__.constructor.apply(this, arguments);
        }

        NoClick.sendActionConfig = function(_arg) {
          var draft;
          draft = _arg.draft;
          return {
            title: "some title",
            iconUrl: "nylas://foo/bar/baz",
            onSend: null
          };
        };

        return NoClick;

      })(ComposerExtension);
      spyOn(ExtensionRegistry.Composer, "extensions").andReturn([NoClick]);
      this.sendActionButton = render(this.draft);
      dropdowns = ReactTestUtils.scryRenderedComponentsWithType(this.sendActionButton, ButtonDropdown);
      buttons = ReactTestUtils.scryRenderedDOMComponentsWithTag(this.sendActionButton, "button");
      expect(buttons.length).toBe(1);
      expect(dropdowns.length).toBe(0);
      expect(NylasEnv.reportError).toHaveBeenCalled();
      return expect(NylasEnv.reportError.calls[0].args[0].message).toMatch(/onSend/);
    });
    it("sends a draft by default", function() {
      var button;
      this.sendActionButton = render(this.draft);
      button = ReactTestUtils.findRenderedDOMComponentWithTag(this.sendActionButton, "button");
      ReactTestUtils.Simulate.click(button);
      expect(isValidDraft).toHaveBeenCalled();
      expect(Actions.sendDraft).toHaveBeenCalled();
      return expect(Actions.sendDraft.calls[0].args[0]).toBe(this.draft.clientId);
    });
    it("doesn't send a draft if the isValidDraft fails", function() {
      var button;
      this.sendActionButton = render(this.draft, false);
      button = ReactTestUtils.findRenderedDOMComponentWithTag(this.sendActionButton, "button");
      ReactTestUtils.Simulate.click(button);
      expect(isValidDraft).toHaveBeenCalled();
      return expect(Actions.sendDraft).not.toHaveBeenCalled();
    });
    it("does the primaryClick action of the extension", function() {
      var Click, button, clicked;
      clicked = false;
      Click = (function(_super) {
        __extends(Click, _super);

        function Click() {
          return Click.__super__.constructor.apply(this, arguments);
        }

        Click.sendActionConfig = function(_arg) {
          var draft;
          draft = _arg.draft;
          return {
            title: "click",
            iconUrl: "nylas://foo/bar/baz",
            onSend: function() {
              return clicked = "onSend fired";
            }
          };
        };

        return Click;

      })(ComposerExtension);
      spyOn(ExtensionRegistry.Composer, "extensions").andReturn([Click]);
      spyOn(NylasEnv.config, 'get').andReturn('click');
      this.sendActionButton = render(this.draft);
      button = ReactDOM.findDOMNode(ReactTestUtils.findRenderedDOMComponentWithClass(this.sendActionButton, "primary-item"));
      ReactTestUtils.Simulate.click(button);
      return expect(clicked).toBe("onSend fired");
    });
    it("catches any errors in an extension's primaryClick method", function() {
      var Click, button, clicked;
      clicked = false;
      Click = (function(_super) {
        __extends(Click, _super);

        function Click() {
          return Click.__super__.constructor.apply(this, arguments);
        }

        Click.sendActionConfig = function(_arg) {
          var draft;
          draft = _arg.draft;
          return {
            title: "click",
            iconUrl: "nylas://foo/bar/baz",
            onSend: function() {
              throw new Error("BOO");
            }
          };
        };

        return Click;

      })(ComposerExtension);
      spyOn(ExtensionRegistry.Composer, "extensions").andReturn([Click]);
      spyOn(NylasEnv.config, 'get').andReturn('click');
      this.sendActionButton = render(this.draft);
      button = ReactDOM.findDOMNode(ReactTestUtils.findRenderedDOMComponentWithClass(this.sendActionButton, "primary-item"));
      ReactTestUtils.Simulate.click(button);
      expect(clicked).toBe(false);
      expect(NylasEnv.reportError).toHaveBeenCalled();
      return expect(NylasEnv.reportError.calls[0].args[0].message).toMatch(/BOO/);
    });
    return it("initializes with the default and shows the standard Send option", function() {
      var button;
      spyOn(NylasEnv.config, 'get').andReturn(null);
      this.sendActionButton = render(this.draft);
      button = ReactDOM.findDOMNode(this.sendActionButton);
      return expect(button.innerText).toEqual('Send');
    });
  });

}).call(this);
