(function() {
  var AccountStore, Actions, ComponentRegistry, DatabaseStore, DateUtils, Event, EventHeader, EventRSVPTask, Message, React, RetinaImg, Utils, moment, path, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  path = require('path');

  React = require('react');

  RetinaImg = require('nylas-component-kit').RetinaImg;

  _ref = require('nylas-exports'), Actions = _ref.Actions, DateUtils = _ref.DateUtils, Message = _ref.Message, Event = _ref.Event, Utils = _ref.Utils, ComponentRegistry = _ref.ComponentRegistry, EventRSVPTask = _ref.EventRSVPTask, DatabaseStore = _ref.DatabaseStore, AccountStore = _ref.AccountStore;

  moment = require('moment-timezone');

  EventHeader = (function(_super) {
    __extends(EventHeader, _super);

    EventHeader.displayName = 'EventHeader';

    EventHeader.propTypes = {
      message: React.PropTypes.instanceOf(Message).isRequired
    };

    function EventHeader(props) {
      this.props = props;
      this._rsvp = __bind(this._rsvp, this);
      this._renderEventActions = __bind(this._renderEventActions, this);
      this.render = __bind(this.render, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentWillReceiveProps = __bind(this.componentWillReceiveProps, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this._onChange = __bind(this._onChange, this);
      this.state = {
        event: this.props.message.events[0]
      };
    }

    EventHeader.prototype._onChange = function() {
      if (!this.state.event) {
        return;
      }
      return DatabaseStore.find(Event, this.state.event.id).then((function(_this) {
        return function(event) {
          if (!event) {
            return;
          }
          return _this.setState({
            event: event
          });
        };
      })(this));
    };

    EventHeader.prototype.componentDidMount = function() {
      this._unlisten = DatabaseStore.listen((function(_this) {
        return function(change) {
          var updated;
          if (_this.state.event && change.objectClass === Event.name) {
            updated = _.find(change.objects, function(o) {
              return o.id === _this.state.event.id;
            });
            if (updated) {
              return _this.setState({
                event: updated
              });
            }
          }
        };
      })(this));
      return this._onChange();
    };

    EventHeader.prototype.componentWillReceiveProps = function(nextProps) {
      this.setState({
        event: nextProps.message.events[0]
      });
      return this._onChange();
    };

    EventHeader.prototype.componentWillUnmount = function() {
      return typeof this._unlisten === "function" ? this._unlisten() : void 0;
    };

    EventHeader.prototype.render = function() {
      var timeFormat;
      timeFormat = DateUtils.getTimeFormat({
        timeZone: true
      });
      if (this.state.event != null) {
        return React.createElement("div", {
          "className": "event-wrapper"
        }, React.createElement("div", {
          "className": "event-header"
        }, React.createElement(RetinaImg, {
          "name": "icon-RSVP-calendar-mini@2x.png",
          "mode": RetinaImg.Mode.ContentPreserve
        }), React.createElement("span", {
          "className": "event-title-text"
        }, "Event: "), React.createElement("span", {
          "className": "event-title"
        }, this.state.event.title)), React.createElement("div", {
          "className": "event-body"
        }, React.createElement("div", {
          "className": "event-date"
        }, React.createElement("div", {
          "className": "event-day"
        }, moment(this.state.event.start * 1000).tz(Utils.timeZone).format("dddd, MMMM Do")), React.createElement("div", null, React.createElement("div", {
          "className": "event-time"
        }, moment(this.state.event.start * 1000).tz(Utils.timeZone).format(timeFormat)), this._renderEventActions()))));
      } else {
        return React.createElement("div", null);
      }
    };

    EventHeader.prototype._renderEventActions = function() {
      var actions, me;
      me = this.state.event.participantForMe();
      if (!me) {
        return false;
      }
      actions = [["yes", "Accept"], ["maybe", "Maybe"], ["no", "Decline"]];
      return React.createElement("div", {
        "className": "event-actions"
      }, actions.map((function(_this) {
        return function(_arg) {
          var classes, label, status;
          status = _arg[0], label = _arg[1];
          classes = "btn-rsvp ";
          if (me.status === status) {
            classes += status;
          }
          return React.createElement("div", {
            "key": status,
            "className": classes,
            "onClick": (function() {
              return _this._rsvp(status);
            })
          }, label);
        };
      })(this)));
    };

    EventHeader.prototype._rsvp = function(status) {
      var me;
      me = this.state.event.participantForMe();
      return Actions.queueTask(new EventRSVPTask(this.state.event, me.email, status));
    };

    return EventHeader;

  })(React.Component);

  module.exports = EventHeader;

}).call(this);
