(function() {
  var Actions, DecryptMessageButton, FileDownloadStore, MessageBodyProcessor, MessageStore, PGPKeyStore, PassphrasePopover, React, ReactDOM, pgp, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ref = require('nylas-exports'), MessageStore = _ref.MessageStore, React = _ref.React, ReactDOM = _ref.ReactDOM, FileDownloadStore = _ref.FileDownloadStore, MessageBodyProcessor = _ref.MessageBodyProcessor, Actions = _ref.Actions;

  PGPKeyStore = require('./pgp-key-store');

  PassphrasePopover = require('./passphrase-popover');

  pgp = require('kbpgp');

  DecryptMessageButton = (function(_super) {
    __extends(DecryptMessageButton, _super);

    DecryptMessageButton.displayName = 'DecryptMessageButton';

    DecryptMessageButton.propTypes = {
      message: React.PropTypes.object.isRequired
    };

    function DecryptMessageButton(props) {
      this.render = __bind(this.render, this);
      this._decryptAttachmentsPopoverDone = __bind(this._decryptAttachmentsPopoverDone, this);
      this._decryptPopoverDone = __bind(this._decryptPopoverDone, this);
      this._onClickDecryptAttachments = __bind(this._onClickDecryptAttachments, this);
      this._onClickDecrypt = __bind(this._onClickDecrypt, this);
      DecryptMessageButton.__super__.constructor.call(this, props);
      this.state = this._getStateFromStores();
    }

    DecryptMessageButton.prototype._getStateFromStores = function() {
      return {
        isDecrypted: PGPKeyStore.isDecrypted(this.props.message),
        wasEncrypted: PGPKeyStore.hasEncryptedComponent(this.props.message),
        encryptedAttachments: PGPKeyStore.fetchEncryptedAttachments(this.props.message),
        status: PGPKeyStore.msgStatus(this.props.message)
      };
    };

    DecryptMessageButton.prototype.componentDidMount = function() {
      return this.unlistenKeystore = PGPKeyStore.listen(this._onKeystoreChange, this);
    };

    DecryptMessageButton.prototype.componentWillUnmount = function() {
      return this.unlistenKeystore();
    };

    DecryptMessageButton.prototype._onKeystoreChange = function() {
      this.setState(this._getStateFromStores());
      if (!this.state.isDecrypted) {
        return PGPKeyStore.decrypt(this.props.message);
      }
    };

    DecryptMessageButton.prototype._onClickDecrypt = function(event) {
      var popoverTarget;
      popoverTarget = event.target.getBoundingClientRect();
      return Actions.openPopover(React.createElement(PassphrasePopover, {
        "onPopoverDone": this._decryptPopoverDone
      }), {
        originRect: popoverTarget,
        direction: 'down'
      });
    };

    DecryptMessageButton.prototype._onClickDecryptAttachments = function(event) {
      var popoverTarget;
      popoverTarget = event.target.getBoundingClientRect();
      return Actions.openPopover(React.createElement(PassphrasePopover, {
        "onPopoverDone": this._decryptAttachmentsPopoverDone
      }), {
        originRect: popoverTarget,
        direction: 'down'
      });
    };

    DecryptMessageButton.prototype._decryptPopoverDone = function(passphrase) {
      var message, privateKey, privateKeys, recipient, _i, _len, _ref1, _results;
      message = this.props.message;
      _ref1 = message.to;
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        recipient = _ref1[_i];
        privateKeys = PGPKeyStore.privKeys({
          address: recipient.email,
          timed: false
        });
        _results.push((function() {
          var _j, _len1, _results1;
          _results1 = [];
          for (_j = 0, _len1 = privateKeys.length; _j < _len1; _j++) {
            privateKey = privateKeys[_j];
            _results1.push(PGPKeyStore.getKeyContents({
              key: privateKey,
              passphrase: passphrase
            }));
          }
          return _results1;
        })());
      }
      return _results;
    };

    DecryptMessageButton.prototype._decryptAttachmentsPopoverDone = function(passphrase) {
      var message, privateKey, privateKeys, recipient, _i, _len, _ref1, _results;
      message = this.props.message;
      _ref1 = message.to;
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        recipient = _ref1[_i];
        privateKeys = PGPKeyStore.privKeys({
          address: recipient.email,
          timed: false
        });
        _results.push((function() {
          var _j, _len1, _results1;
          _results1 = [];
          for (_j = 0, _len1 = privateKeys.length; _j < _len1; _j++) {
            privateKey = privateKeys[_j];
            _results1.push(PGPKeyStore.getKeyContents({
              key: privateKey,
              passphrase: passphrase,
              callback: (function(_this) {
                return function(identity) {
                  return PGPKeyStore.decryptAttachments(identity, _this.state.encryptedAttachments);
                };
              })(this)
            }));
          }
          return _results1;
        }).call(this));
      }
      return _results;
    };

    DecryptMessageButton.prototype.render = function() {
      var decryptAttachments, decryptBody, decryptionInterface, title, _ref1, _ref2;
      if (!(this.state.wasEncrypted || this.state.encryptedAttachments.length > 0)) {
        return false;
      }
      title = this.state.isDecrypted ? "Message Decrypted" : "Message Encrypted";
      decryptBody = false;
      if (!this.state.isDecrypted) {
        decryptBody = React.createElement("button", {
          "title": "Decrypt email body",
          "className": "btn btn-toolbar",
          "onClick": this._onClickDecrypt,
          "ref": "button"
        }, "Decrypt");
      }
      decryptAttachments = false;
      if (((_ref1 = this.state.encryptedAttachments) != null ? _ref1.length : void 0) === 1) {
        decryptAttachments = React.createElement("button", {
          "onClick": this._onClickDecryptAttachments,
          "className": "btn btn-toolbar"
        }, "Decrypt Attachment");
        title = "Attachment Encrypted";
      } else if (((_ref2 = this.state.encryptedAttachments) != null ? _ref2.length : void 0) > 1) {
        decryptAttachments = React.createElement("button", {
          "onClick": this._onClickDecryptAttachments,
          "className": "btn btn-toolbar"
        }, "Decrypt Attachments");
        title = "Attachments Encrypted";
      }
      if (decryptAttachments || decryptBody) {
        decryptionInterface = React.createElement("div", {
          "className": "decryption-interface"
        }, decryptBody, decryptAttachments);
      }
      return React.createElement("div", {
        "className": "keybase-decrypt"
      }, React.createElement("div", {
        "className": "line-w-label"
      }, React.createElement("div", {
        "className": "border"
      }), React.createElement("div", {
        "className": "decrypt-bar"
      }, React.createElement("div", {
        "className": "title-text"
      }, title), decryptionInterface), React.createElement("div", {
        "className": "border"
      })));
    };

    return DecryptMessageButton;

  })(React.Component);

  module.exports = DecryptMessageButton;

}).call(this);
