(function() {
  var Actions, Contact, DatabaseStore, DraftStore, EncryptMessageButton, Identity, ModalKeyRecommender, PGPKeyStore, React, ReactDOM, RetinaImg, Utils, pgp, remote, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ref = require('nylas-exports'), Utils = _ref.Utils, DraftStore = _ref.DraftStore, React = _ref.React, Actions = _ref.Actions, DatabaseStore = _ref.DatabaseStore, Contact = _ref.Contact, ReactDOM = _ref.ReactDOM;

  PGPKeyStore = require('./pgp-key-store');

  Identity = require('./identity');

  ModalKeyRecommender = require('./modal-key-recommender');

  RetinaImg = require('nylas-component-kit').RetinaImg;

  remote = require('electron').remote;

  pgp = require('kbpgp');

  _ = require('underscore');

  EncryptMessageButton = (function(_super) {
    __extends(EncryptMessageButton, _super);

    EncryptMessageButton.displayName = 'EncryptMessageButton';

    EncryptMessageButton.propTypes = {
      draft: React.PropTypes.object.isRequired,
      session: React.PropTypes.object.isRequired
    };

    function EncryptMessageButton(props) {
      this._checkKeysAndEncrypt = __bind(this._checkKeysAndEncrypt, this);
      this._encrypt = __bind(this._encrypt, this);
      this._toggleCrypt = __bind(this._toggleCrypt, this);
      this._onClick = __bind(this._onClick, this);
      this._onKeystoreChange = __bind(this._onKeystoreChange, this);
      EncryptMessageButton.__super__.constructor.call(this, props);
      this.state = {
        plaintext: "",
        cryptotext: "",
        currentlyEncrypted: false
      };
    }

    EncryptMessageButton.prototype.componentDidMount = function() {
      return this.unlistenKeystore = PGPKeyStore.listen(this._onKeystoreChange, this);
    };

    EncryptMessageButton.prototype.componentWillUnmount = function() {
      return this.unlistenKeystore();
    };

    EncryptMessageButton.prototype.componentWillReceiveProps = function(nextProps) {
      var body;
      if (this.state.currentlyEncrypted && nextProps.draft.body !== this.props.draft.body && nextProps.draft.body !== this.state.cryptotext) {
        body = this.state.cryptotext;
        return this.props.session.changes.add({
          body: body
        });
      }
    };

    EncryptMessageButton.prototype._getKeys = function() {
      var keys, publicKey, publicKeys, recipient, _i, _j, _len, _len1, _ref1;
      keys = [];
      _ref1 = this.props.draft.participants({
        includeFrom: false,
        includeBcc: true
      });
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        recipient = _ref1[_i];
        publicKeys = PGPKeyStore.pubKeys(recipient.email);
        if (publicKeys.length < 1) {
          keys.push(new Identity({
            addresses: [recipient.email]
          }));
        } else {
          for (_j = 0, _len1 = publicKeys.length; _j < _len1; _j++) {
            publicKey = publicKeys[_j];
            if (publicKey.key == null) {
              PGPKeyStore.getKeyContents({
                key: publicKey
              });
            } else {
              keys.push(publicKey);
            }
          }
        }
      }
      return keys;
    };

    EncryptMessageButton.prototype._onKeystoreChange = function() {
      var newKeys, oldKeys;
      if (this.state.currentlyEncrypted) {
        newKeys = _.map(this.props.draft.participants(), function(participant) {
          return PGPKeyStore.pubKeys(participant.email);
        });
        newKeys = _.flatten(newKeys);
        oldKeys = _.map(this.props.draft.participants(), function(participant) {
          return PGPKeyStore.pubKeys(participant.email);
        });
        oldKeys = _.flatten(oldKeys);
        if (newKeys.length !== oldKeys.length) {
          return this._toggleCrypt();
        }
      }
    };

    EncryptMessageButton.prototype.render = function() {
      var classnames;
      classnames = "btn btn-toolbar";
      if (this.state.currentlyEncrypted) {
        classnames += " btn-enabled";
      }
      return React.createElement("div", {
        "className": "n1-keybase"
      }, React.createElement("button", {
        "title": "Encrypt email body",
        "className": classnames,
        "onClick": ((function(_this) {
          return function() {
            return _this._onClick();
          };
        })(this)),
        "ref": "button"
      }, React.createElement(RetinaImg, {
        "url": "nylas://keybase/encrypt-composer-button@2x.png",
        "mode": RetinaImg.Mode.ContentIsMask
      })));
    };

    EncryptMessageButton.prototype._onClick = function() {
      return this._toggleCrypt();
    };

    EncryptMessageButton.prototype._toggleCrypt = function() {
      var identities, plaintext;
      if (this.state.currentlyEncrypted) {
        this.props.session.changes.add({
          body: this.state.plaintext
        });
        return this.setState({
          currentlyEncrypted: false
        });
      } else {
        plaintext = this.props.draft.body;
        identities = this._getKeys();
        return this._checkKeysAndEncrypt(plaintext, identities, (function(_this) {
          return function(err, cryptotext) {
            if (err) {
              console.warn(err);
              NylasEnv.showErrorDialog(err);
            }
            if ((cryptotext != null) && cryptotext !== "") {
              cryptotext = "<pre>" + cryptotext + "</pre>";
              _this.setState({
                currentlyEncrypted: true,
                plaintext: plaintext,
                cryptotext: cryptotext
              });
              return _this.props.session.changes.add({
                body: cryptotext
              });
            }
          };
        })(this));
      }
    };

    EncryptMessageButton.prototype._encrypt = function(text, identities, cb) {
      var keys, kms, params;
      keys = _.pluck(identities, "key");
      kms = _.compact(keys);
      if (kms.length === 0) {
        NylasEnv.showErrorDialog("There are no PGP public keys loaded, so the message cannot be encrypted. Compose a message, add recipients in the To: field, and try again.");
        return;
      }
      params = {
        encrypt_for: kms,
        msg: text
      };
      return pgp.box(params, cb);
    };

    EncryptMessageButton.prototype._checkKeysAndEncrypt = function(text, identities, cb) {
      var emails;
      emails = _.chain(identities).pluck("addresses").flatten().uniq().value();
      if (_.every(identities, function(identity) {
        return identity.key != null;
      })) {
        return this._encrypt(text, identities, cb);
      } else {
        return DatabaseStore.findAll(Contact, {
          email: emails
        }).then((function(_this) {
          return function(contacts) {
            var component;
            component = React.createElement(ModalKeyRecommender, {
              "contacts": contacts,
              "emails": emails,
              "callback": (function(newIdentities) {
                return _this._encrypt(text, newIdentities, cb);
              })
            });
            return Actions.openPopover(component, {
              originRect: ReactDOM.findDOMNode(_this).getBoundingClientRect(),
              direction: 'up',
              closeOnAppBlur: false
            });
          };
        })(this));
      }
    };

    return EncryptMessageButton;

  })(React.Component);

  module.exports = EncryptMessageButton;

}).call(this);
