(function() {
  var Identity, Utils, path;

  Utils = require('nylas-exports').Utils;

  path = require('path');

  module.exports = Identity = (function() {
    function Identity(_arg) {
      var addresses, isPriv, key, keybase_profile;
      key = _arg.key, addresses = _arg.addresses, isPriv = _arg.isPriv, keybase_profile = _arg.keybase_profile;
      this.clientId = Utils.generateTempId();
      this.key = key != null ? key : null;
      this.isPriv = isPriv != null ? isPriv : false;
      this.timeout = null;
      this.addresses = addresses != null ? addresses : [];
      this.keybase_profile = keybase_profile != null ? keybase_profile : null;
      Object.defineProperty(this, 'keyPath', {
        get: function() {
          var keyDir, keyPath, thisDir;
          if (this.addresses.length > 0) {
            keyDir = path.join(NylasEnv.getConfigDirPath(), 'keys');
            thisDir = this.isPriv ? path.join(keyDir, 'private') : path.join(keyDir, 'public');
            keyPath = path.join(thisDir, this.addresses.join(" "));
          } else {
            keyPath = null;
          }
          return keyPath;
        }
      });
      if (this.isPriv) {
        this.setTimeout();
      }
    }

    Identity.prototype.fingerprint = function() {
      if (this.key != null) {
        return this.key.get_pgp_fingerprint().toString('hex');
      }
      return null;
    };

    Identity.prototype.setTimeout = function() {
      var delay;
      delay = 1000 * 60 * 30;
      return this.timeout = Date.now() + delay;
    };

    Identity.prototype.isTimedOut = function() {
      return this.timeout < Date.now();
    };

    Identity.prototype.uid = function() {
      var uid;
      if (this.key != null) {
        uid = this.key.get_pgp_fingerprint().toString('hex');
      } else if (this.keybase_profile != null) {
        uid = this.keybase_profile.components.username.val;
      } else if (this.addresses.length > 0) {
        uid = this.addresses.join('');
      } else {
        uid = this.clientId;
      }
      return uid;
    };

    return Identity;

  })();

}).call(this);
