(function() {
  var Identity, KeyAdder, PGPKeyStore, React, RegExpUtils, RetinaImg, Utils, fs, kb, pgp, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ref = require('nylas-exports'), Utils = _ref.Utils, React = _ref.React, RegExpUtils = _ref.RegExpUtils;

  RetinaImg = require('nylas-component-kit').RetinaImg;

  PGPKeyStore = require('./pgp-key-store');

  Identity = require('./identity');

  kb = require('./keybase');

  pgp = require('kbpgp');

  _ = require('underscore');

  fs = require('fs');

  module.exports = KeyAdder = (function(_super) {
    __extends(KeyAdder, _super);

    KeyAdder.displayName = 'KeyAdder';

    function KeyAdder(props) {
      this._onKeyChange = __bind(this._onKeyChange, this);
      this._onPassphraseChange = __bind(this._onPassphraseChange, this);
      this._onAddressChange = __bind(this._onAddressChange, this);
      this._saveNewKey = __bind(this._saveNewKey, this);
      this._generateKeypair = __bind(this._generateKeypair, this);
      this._onInnerGenerateButtonClick = __bind(this._onInnerGenerateButtonClick, this);
      this._onImportButtonClick = __bind(this._onImportButtonClick, this);
      this._onGenerateButtonClick = __bind(this._onGenerateButtonClick, this);
      this._onPasteButtonClick = __bind(this._onPasteButtonClick, this);
      this.state = {
        address: "",
        keyContents: "",
        passphrase: "",
        generate: false,
        paste: false,
        "import": false,
        isPriv: false,
        loading: false,
        validAddress: false,
        validKeyBody: false
      };
    }

    KeyAdder.prototype._onPasteButtonClick = function(event) {
      return this.setState({
        generate: false,
        paste: !this.state.paste,
        "import": false,
        address: "",
        validAddress: false,
        keyContents: ""
      });
    };

    KeyAdder.prototype._onGenerateButtonClick = function(event) {
      return this.setState({
        generate: !this.state.generate,
        paste: false,
        "import": false,
        address: "",
        validAddress: false,
        keyContents: "",
        passphrase: ""
      });
    };

    KeyAdder.prototype._onImportButtonClick = function(event) {
      return NylasEnv.showOpenDialog({
        title: "Import PGP Key",
        buttonLabel: "Import",
        properties: ['openFile']
      }, (function(_this) {
        return function(filepath) {
          if (filepath != null) {
            _this.setState({
              generate: false,
              paste: false,
              "import": true,
              address: "",
              validAddress: false,
              passphrase: ""
            });
            return fs.readFile(filepath[0], function(err, data) {
              return pgp.KeyManager.import_from_armored_pgp({
                armored: data
              }, function(err, km) {
                var keyBody, privateStart;
                if (err) {
                  PGPKeyStore._displayError("The file you selected for import is not a valid PGP Key.");
                } else {
                  privateStart = "-----BEGIN PGP PRIVATE KEY BLOCK-----";
                  keyBody = km.armored_pgp_private != null ? km.armored_pgp_private : km.armored_pgp_public;
                  return _this.setState({
                    keyContents: keyBody,
                    isPriv: keyBody.indexOf(privateStart) >= 0,
                    validKeyBody: true
                  });
                }
              });
            });
          }
        };
      })(this));
    };

    KeyAdder.prototype._onInnerGenerateButtonClick = function(event) {
      this.setState({
        loading: true
      });
      return this._generateKeypair();
    };

    KeyAdder.prototype._generateKeypair = function() {
      return pgp.KeyManager.generate_rsa({
        userid: this.state.address
      }, (function(_this) {
        return function(err, km) {
          return km.sign({}, function(err) {
            if (err) {
              console.warn(err);
            }
            km.export_pgp_private({
              passphrase: _this.state.passphrase
            }, function(err, pgp_private) {
              var ident;
              ident = new Identity({
                addresses: [_this.state.address],
                isPriv: true
              });
              return PGPKeyStore.saveNewKey(ident, pgp_private);
            });
            return km.export_pgp_public({}, function(err, pgp_public) {
              var ident;
              ident = new Identity({
                addresses: [_this.state.address],
                isPriv: false
              });
              PGPKeyStore.saveNewKey(ident, pgp_public);
              return _this.setState({
                keyContents: pgp_public,
                loading: false
              });
            });
          });
        };
      })(this));
    };

    KeyAdder.prototype._saveNewKey = function() {
      var ident;
      ident = new Identity({
        addresses: [this.state.address],
        isPriv: this.state.isPriv
      });
      return PGPKeyStore.saveNewKey(ident, this.state.keyContents);
    };

    KeyAdder.prototype._onAddressChange = function(event) {
      var address, valid;
      address = event.target.value;
      valid = false;
      if (address && address.length > 0 && RegExpUtils.emailRegex().test(address)) {
        valid = true;
      }
      return this.setState({
        address: event.target.value,
        validAddress: valid
      });
    };

    KeyAdder.prototype._onPassphraseChange = function(event) {
      return this.setState({
        passphrase: event.target.value
      });
    };

    KeyAdder.prototype._onKeyChange = function(event) {
      var privateStart;
      privateStart = "-----BEGIN PGP PRIVATE KEY BLOCK-----";
      this.setState({
        keyContents: event.target.value,
        isPriv: event.target.value.indexOf(privateStart) >= 0
      });
      return pgp.KeyManager.import_from_armored_pgp({
        armored: event.target.value
      }, (function(_this) {
        return function(err, km) {
          var valid;
          if (err) {
            valid = false;
          } else {
            valid = true;
          }
          return _this.setState({
            validKeyBody: valid
          });
        };
      })(this));
    };

    KeyAdder.prototype._renderAddButtons = function() {
      return React.createElement("div", null, "Add a PGP Key:", React.createElement("button", {
        "className": "btn key-creation-button",
        "title": "Paste",
        "onClick": this._onPasteButtonClick
      }, "Paste in a New Key"), React.createElement("button", {
        "className": "btn key-creation-button",
        "title": "Import",
        "onClick": this._onImportButtonClick
      }, "Import a Key From File"), React.createElement("button", {
        "className": "btn key-creation-button",
        "title": "Generate",
        "onClick": this._onGenerateButtonClick
      }, "Generate a New Keypair"));
    };

    KeyAdder.prototype._renderManualKey = function() {
      var buttonClass, invalidInputs, invalidMsg, passphraseInput;
      if (!this.state.validAddress && this.state.address.length > 0) {
        invalidMsg = React.createElement("span", {
          "className": "invalid-msg"
        }, "Invalid email address");
      } else if (!this.state.validKeyBody && this.state.keyContents.length > 0) {
        invalidMsg = React.createElement("span", {
          "className": "invalid-msg"
        }, "Invalid key body");
      } else {
        invalidMsg = React.createElement("span", {
          "className": "invalid-msg"
        }, " ");
      }
      invalidInputs = !(this.state.validAddress && this.state.validKeyBody);
      buttonClass = invalidInputs ? "btn key-add-btn btn-disabled" : "btn key-add-btn";
      passphraseInput = React.createElement("input", {
        "type": "password",
        "value": this.state.passphrase,
        "placeholder": "Private Key Password",
        "className": "key-passphrase-input",
        "onChange": this._onPassphraseChange
      });
      return React.createElement("div", {
        "className": "key-adder"
      }, React.createElement("div", {
        "className": "key-text"
      }, React.createElement("textarea", {
        "ref": "key-input",
        "value": this.state.keyContents || "",
        "onChange": this._onKeyChange,
        "placeholder": "Paste in your PGP key here!"
      })), React.createElement("div", {
        "className": "credentials"
      }, React.createElement("input", {
        "type": "text",
        "value": this.state.address,
        "placeholder": "Email Address",
        "className": "key-email-input",
        "onChange": this._onAddressChange
      }), (this.state.isPriv ? passphraseInput : void 0), invalidMsg, React.createElement("button", {
        "className": buttonClass,
        "disabled": invalidInputs,
        "title": "Save",
        "onClick": this._saveNewKey
      }, "Save")));
    };

    KeyAdder.prototype._renderGenerateKey = function() {
      var buttonClass, invalidMsg, keyPlaceholder, loading;
      if (!this.state.validAddress && this.state.address.length > 0) {
        invalidMsg = React.createElement("span", {
          "className": "invalid-msg"
        }, "Invalid email address");
      } else {
        invalidMsg = React.createElement("span", {
          "className": "invalid-msg"
        }, " ");
      }
      loading = React.createElement(RetinaImg, {
        "style": {
          width: 20,
          height: 20
        },
        "name": "inline-loading-spinner.gif",
        "mode": RetinaImg.Mode.ContentPreserve
      });
      if (this.state.loading) {
        keyPlaceholder = "Generating your key now. This could take a while.";
      } else {
        keyPlaceholder = "Your generated public key will appear here. Share it with your friends!";
      }
      buttonClass = !this.state.validAddress ? "btn key-add-btn btn-disabled" : "btn key-add-btn";
      return React.createElement("div", {
        "className": "key-adder"
      }, React.createElement("div", {
        "className": "credentials"
      }, React.createElement("input", {
        "type": "text",
        "value": this.state.address,
        "placeholder": "Email Address",
        "className": "key-email-input",
        "onChange": this._onAddressChange
      }), React.createElement("input", {
        "type": "password",
        "value": this.state.passphrase,
        "placeholder": "Private Key Password",
        "className": "key-passphrase-input",
        "onChange": this._onPassphraseChange
      }), invalidMsg, React.createElement("button", {
        "className": buttonClass,
        "disabled": !this.state.validAddress,
        "title": "Generate",
        "onClick": this._onInnerGenerateButtonClick
      }, "Generate")), React.createElement("div", {
        "className": "key-text"
      }, React.createElement("div", {
        "className": "loading"
      }, (this.state.loading ? loading : void 0)), React.createElement("textarea", {
        "ref": "key-output",
        "value": this.state.keyContents || "",
        "disabled": true,
        "placeholder": keyPlaceholder
      })));
    };

    KeyAdder.prototype.render = function() {
      return React.createElement("div", null, this._renderAddButtons(), (this.state.generate ? this._renderGenerateKey() : void 0), (this.state.paste || this.state["import"] ? this._renderManualKey() : void 0));
    };

    return KeyAdder;

  })(React.Component);

}).call(this);
