(function() {
  var KeybaseAPI, request, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require('underscore');

  request = require('request');

  KeybaseAPI = (function() {
    function KeybaseAPI() {
      this._keybaseRequest = __bind(this._keybaseRequest, this);
      this.autocomplete = __bind(this.autocomplete, this);
      this.getKey = __bind(this.getKey, this);
      this.getUser = __bind(this.getUser, this);
      this.baseUrl = "https://keybase.io";
    }

    KeybaseAPI.prototype.getUser = function(key, keyType, callback) {
      var _ref;
      if ((_ref = !keyType) === 'usernames' || _ref === 'domain' || _ref === 'twitter' || _ref === 'github' || _ref === 'reddit' || _ref === 'hackernews' || _ref === 'coinbase' || _ref === 'key_fingerprint') {
        console.error('keyType must be a supported Keybase query type.');
      }
      return this._keybaseRequest("/_/api/1.0/user/lookup.json?" + keyType + "=" + key, (function(_this) {
        return function(err, resp, obj) {
          if (err) {
            return callback(err, null);
          }
          if ((obj == null) || (obj.them == null)) {
            return callback(new Error("Empty response!"), null);
          }
          if (obj.status != null) {
            if (obj.status.name !== "OK") {
              return callback(new Error(obj.status.desc), null);
            }
          }
          return callback(null, _.map(obj.them, _this._regularToAutocomplete));
        };
      })(this));
    };

    KeybaseAPI.prototype.getKey = function(username, callback) {
      return request({
        url: this.baseUrl + ("/" + username + "/key.asc"),
        headers: {
          'User-Agent': 'request'
        }
      }, (function(_this) {
        return function(err, resp, obj) {
          if (err) {
            return callback(err, null);
          }
          if (obj == null) {
            return callback(new Error("No key found for " + username), null);
          }
          if (!obj.startsWith("-----BEGIN PGP PUBLIC KEY BLOCK-----")) {
            return callback(new Error("No key returned from keybase for " + username), null);
          }
          return callback(null, obj);
        };
      })(this));
    };

    KeybaseAPI.prototype.autocomplete = function(query, callback) {
      var url;
      url = "/_/api/1.0/user/autocomplete.json";
      return request({
        url: this.baseUrl + url,
        form: {
          q: query
        },
        headers: {
          'User-Agent': 'request'
        },
        json: true
      }, (function(_this) {
        return function(err, resp, obj) {
          if (err) {
            return callback(err, null);
          }
          if (obj.status != null) {
            if (obj.status.name !== "OK") {
              return callback(new Error(obj.status.desc), null);
            }
          }
          return callback(null, obj.completions);
        };
      })(this));
    };

    KeybaseAPI.prototype._keybaseRequest = function(url, callback) {
      return request({
        url: this.baseUrl + url,
        headers: {
          'User-Agent': 'request'
        },
        json: true
      }, callback);
    };

    KeybaseAPI.prototype._regularToAutocomplete = function(profile) {
      var cleanedProfile, safe_name, _ref;
      cleanedProfile = {
        components: {}
      };
      cleanedProfile.thumbnail = null;
      if (((_ref = profile.pictures) != null ? _ref.primary : void 0) != null) {
        cleanedProfile.thumbnail = profile.pictures.primary.url;
      }
      safe_name = profile.profile != null ? profile.profile.full_name : "";
      cleanedProfile.components = {
        full_name: {
          val: safe_name
        },
        username: {
          val: profile.basics.username
        }
      };
      _.each(profile.proofs_summary.all, (function(_this) {
        return function(connectedAccount) {
          var component;
          component = {};
          component[connectedAccount.proof_type] = {
            val: connectedAccount.nametag
          };
          return cleanedProfile.components = _.extend(cleanedProfile.components, component);
        };
      })(this));
      return cleanedProfile;
    };

    return KeybaseAPI;

  })();

  module.exports = new KeybaseAPI();

}).call(this);
