(function() {
  var Actions, KeybaseSearch, KeybaseUser, ModalKeyRecommender, PGPKeyStore, React, Utils, kb, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ref = require('nylas-exports'), Utils = _ref.Utils, React = _ref.React, Actions = _ref.Actions;

  PGPKeyStore = require('./pgp-key-store');

  KeybaseSearch = require('./keybase-search');

  KeybaseUser = require('./keybase-user');

  kb = require('./keybase');

  _ = require('underscore');

  module.exports = ModalKeyRecommender = (function(_super) {
    __extends(ModalKeyRecommender, _super);

    ModalKeyRecommender.displayName = 'ModalKeyRecommender';

    ModalKeyRecommender.propTypes = {
      contacts: React.PropTypes.array.isRequired,
      emails: React.PropTypes.array,
      callback: React.PropTypes["function"]
    };

    ModalKeyRecommender.defaultProps = {
      callback: function() {}
    };

    function ModalKeyRecommender(props) {
      this._onManageKeys = __bind(this._onManageKeys, this);
      this._onDone = __bind(this._onDone, this);
      this._setPage = __bind(this._setPage, this);
      this._onPrev = __bind(this._onPrev, this);
      this._onNext = __bind(this._onNext, this);
      this._selectProfile = __bind(this._selectProfile, this);
      this._getStateFromStores = __bind(this._getStateFromStores, this);
      this._onKeystoreChange = __bind(this._onKeystoreChange, this);
      ModalKeyRecommender.__super__.constructor.call(this, props);
      this.state = Object.assign({
        currentContact: 0
      }, this._getStateFromStores());
    }

    ModalKeyRecommender.prototype.componentDidMount = function() {
      return this.unlistenKeystore = PGPKeyStore.listen(this._onKeystoreChange);
    };

    ModalKeyRecommender.prototype.componentWillUnmount = function() {
      return this.unlistenKeystore();
    };

    ModalKeyRecommender.prototype._onKeystoreChange = function() {
      return this.setState(this._getStateFromStores());
    };

    ModalKeyRecommender.prototype._getStateFromStores = function() {
      return {
        identities: PGPKeyStore.pubKeys(this.props.emails)
      };
    };

    ModalKeyRecommender.prototype._selectProfile = function(address, identity) {
      var keybaseUsername;
      keybaseUsername = identity.keybase_profile.components.username.val;
      identity.addresses.push(address);
      return kb.getKey(keybaseUsername, (function(_this) {
        return function(error, key) {
          if (error) {
            return console.error(error);
          } else {
            return PGPKeyStore.saveNewKey(identity, key);
          }
        };
      })(this));
    };

    ModalKeyRecommender.prototype._onNext = function() {
      return this.setState({
        currentContact: this.state.currentContact + 1
      });
    };

    ModalKeyRecommender.prototype._onPrev = function() {
      return this.setState({
        currentContact: this.state.currentContact - 1
      });
    };

    ModalKeyRecommender.prototype._setPage = function(page) {
      return this.setState({
        currentContact: page
      });
    };

    ModalKeyRecommender.prototype._onDone = function() {
      if (this.state.identities.length < this.props.emails.length) {
        if (!PGPKeyStore._displayDialog('Encrypt without keys for all recipients?', 'Some recipients are missing PGP public keys. They will not be able to decrypt this message.', ['Encrypt', 'Cancel'])) {
          return;
        }
      }
      Actions.closePopover();
      return this.props.callback(this.state.identities);
    };

    ModalKeyRecommender.prototype._onManageKeys = function() {
      Actions.switchPreferencesTab('Encryption');
      return Actions.openPreferences();
    };

    ModalKeyRecommender.prototype.render = function() {
      var backButton, body, contact, contactString, deleteButton, email, identity, importFunc, nextButton, prefsButton, prefsLink, query;
      email = this.props.emails[this.state.currentContact];
      contact = _.findWhere(this.props.contacts, {
        'email': email
      });
      contactString = contact != null ? contact.toString() : email;
      identity = _.find(this.state.identities, function(identity) {
        return __indexOf.call(identity.addresses, email) >= 0;
      });
      if (this.state.currentContact === (this.props.emails.length - 1)) {
        if (this.props.emails.length === 1) {
          backButton = false;
        } else {
          backButton = React.createElement("button", {
            "className": "btn modal-back-button",
            "onClick": this._onPrev
          }, "Back");
        }
        nextButton = React.createElement("button", {
          "className": "btn modal-next-button",
          "onClick": this._onDone
        }, "Done");
      } else if (this.state.currentContact === 0) {
        backButton = false;
        nextButton = React.createElement("button", {
          "className": "btn modal-next-button",
          "onClick": this._onNext
        }, "Next");
      } else {
        backButton = React.createElement("button", {
          "className": "btn modal-back-button",
          "onClick": this._onPrev
        }, "Back");
        nextButton = React.createElement("button", {
          "className": "btn modal-next-button",
          "onClick": this._onNext
        }, "Next");
      }
      if (identity != null) {
        deleteButton = React.createElement("button", {
          "title": "Delete Public",
          "className": "btn btn-toolbar btn-danger",
          "onClick": ((function(_this) {
            return function() {
              return PGPKeyStore.deleteKey(identity);
            };
          })(this)),
          "ref": "button"
        }, "Delete Key");
        body = [
          React.createElement("div", {
            "key": "title",
            "className": "picker-title"
          }, "This PGP public key has been saved for ", React.createElement("br", null), React.createElement("b", null, contactString, ".")), React.createElement("div", {
            "className": "keybase-profile-solo"
          }, React.createElement(KeybaseUser, {
            "key": "keybase-user",
            "profile": identity,
            "displayEmailList": false,
            "actionButton": deleteButton
          }))
        ];
      } else {
        if (contact != null) {
          query = contact.fullName();
          if (query.match(/\s/) == null) {
            query = "";
          }
        } else {
          query = "";
        }
        importFunc = ((function(_this) {
          return function(identity) {
            return _this._selectProfile(email, identity);
          };
        })(this));
        body = [
          React.createElement("div", {
            "key": "title",
            "className": "picker-title"
          }, "There is no PGP public key saved for ", React.createElement("br", null), React.createElement("b", null, contactString, ".")), React.createElement(KeybaseSearch, {
            "key": "keybase-search",
            "initialSearch": query,
            "importFunc": importFunc
          })
        ];
      }
      prefsButton = React.createElement("button", {
        "className": "btn modal-prefs-button",
        "onClick": this._onManageKeys
      }, "Advanced Key Management");
      prefsLink = React.createElement("a", {
        "className": "preferences-references",
        "onClick": this._onManageKeys
      }, "Advanced Key Management");
      return React.createElement("div", {
        "className": "key-picker-modal"
      }, body, React.createElement("div", {
        "style": {
          flex: 1
        }
      }), React.createElement("div", {
        "className": "picker-controls"
      }, React.createElement("div", {
        "style": {
          width: 60
        }
      }, " ", backButton, " "), prefsButton, React.createElement("div", {
        "style": {
          width: 60
        }
      }, " ", nextButton, " ")));
    };

    return ModalKeyRecommender;

  })(React.Component);

}).call(this);
