(function() {
  var DraftStore, EncryptMessageButton, Message, PGPKeyStore, React, ReactDOM, ReactTestUtils, pgp, _ref;

  _ref = require('nylas-exports'), React = _ref.React, ReactDOM = _ref.ReactDOM, ReactTestUtils = _ref.ReactTestUtils, DraftStore = _ref.DraftStore, Message = _ref.Message;

  pgp = require('kbpgp');

  EncryptMessageButton = require('../lib/encrypt-button');

  PGPKeyStore = require('../lib/pgp-key-store');

  describe("EncryptMessageButton", function() {
    beforeEach(function() {
      var add, key;
      key = "-----BEGIN PGP PRIVATE KEY BLOCK-----\nVersion: GnuPG v1\n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\n217s2OKjpJqtpHPf2vY=\n=UY7Y\n-----END PGP PRIVATE KEY BLOCK-----";
      pgp.KeyManager.import_from_armored_pgp({
        armored: key
      }, (function(_this) {
        return function(err, km) {
          return _this.km = km;
        };
      })(this));
      waitsFor(((function(_this) {
        return function() {
          return _this.km != null;
        };
      })(this)), "getting a key took too long", 1000);
      this.msg = new Message({
        subject: 'Subject',
        body: '<p>Body</p>'
      });
      this.output = null;
      add = jasmine.createSpy('add');
      spyOn(DraftStore, 'sessionForClientId').andCallFake((function(_this) {
        return function(draftClientId) {
          var d, session;
          d = _this.msg;
          session = {
            draft: function() {
              return d;
            },
            changes: {
              add: function(changes) {
                return _this.output = changes;
              }
            }
          };
          return Promise.resolve(session);
        };
      })(this));
      return this.component = ReactTestUtils.renderIntoDocument(React.createElement(EncryptMessageButton, {
        "draftClientId": "test"
      }));
    });
    it("should render into the page", function() {
      return expect(this.component).toBeDefined();
    });
    it("should have a displayName", function() {
      return expect(EncryptMessageButton.displayName).toBe('EncryptMessageButton');
    });
    it("should have an onClick behavior which encrypts the message", function() {
      var buttonNode;
      spyOn(this.component, '_onClick');
      buttonNode = ReactDOM.findDOMNode(this.component.refs.button);
      ReactTestUtils.Simulate.click(buttonNode);
      return expect(this.component._onClick).toHaveBeenCalled();
    });
    it("should store the message body's plaintext on encryption", function() {
      var buttonNode;
      spyOn(this.component, '_onClick');
      buttonNode = ReactDOM.findDOMNode(this.component.refs.button);
      ReactTestUtils.Simulate.click(buttonNode);
      return expect(this.component.plaintext === this.msg.body);
    });
    it("should mark itself as encrypted", function() {
      var buttonNode;
      spyOn(this.component, '_onClick');
      buttonNode = ReactDOM.findDOMNode(this.component.refs.button);
      ReactTestUtils.Simulate.click(buttonNode);
      return expect(this.component.currentlyEncrypted === true);
    });
    return xit("should be able to encrypt messages", function() {
      runs((function(_this) {
        return function() {
          console.log(_this.km);
          _this.component._encrypt("test text", [_this.km]);
          _this.flag = false;
          return pgp.box({
            encrypt_for: [_this.km],
            msg: "test text"
          }, function(err, result_string) {
            expect(err == null);
            _this.err = err;
            _this.result_string = result_string;
            return _this.flag = true;
          });
        };
      })(this));
      waitsFor(((function(_this) {
        return function() {
          console.log(_this.flag);
          return _this.flag;
        };
      })(this)), "encryption took too long", 5000);
      return runs((function(_this) {
        return function() {
          console.log(_this.err);
          console.log(_this.result_string);
          console.log(_this.output);
          return expect(_this.output === _this.result_string);
        };
      })(this));
    });
  });

}).call(this);
