(function() {
  var DraftStore, Identity, Message, PGPKeyStore, React, ReactTestUtils, fs, pgp, _, _ref;

  _ref = require('nylas-exports'), React = _ref.React, ReactTestUtils = _ref.ReactTestUtils, DraftStore = _ref.DraftStore, Message = _ref.Message;

  pgp = require('kbpgp');

  _ = require('underscore');

  fs = require('fs');

  Identity = require('../lib/identity');

  PGPKeyStore = require('../lib/pgp-key-store');

  describe("PGPKeyStore", function() {
    beforeEach(function() {
      var body, privIdent, pubIdent;
      this.TEST_KEY = "-----BEGIN PGP PRIVATE KEY BLOCK-----\nVersion: GnuPG v1\n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\n217s2OKjpJqtpHPf2vY=\n=UY7Y\n-----END PGP PRIVATE KEY BLOCK-----";
      spyOn(PGPKeyStore, "getKeyContents").andCallFake((function(_this) {
        return function(_arg) {
          var callback, data, key, passphrase;
          key = _arg.key, passphrase = _arg.passphrase, callback = _arg.callback;
          data = _this.TEST_KEY;
          return pgp.KeyManager.import_from_armored_pgp({
            armored: data
          }, function(err, km) {
            expect(err).toEqual(null);
            if (km.is_pgp_locked()) {
              expect(passphrase).toBeDefined();
              km.unlock_pgp({
                passphrase: passphrase
              }, function(err) {
                return expect(err).toEqual(null);
              });
            }
            key.key = km;
            key.setTimeout();
            if (callback != null) {
              return callback();
            }
          });
        };
      })(this));
      this.unencryptedMsg = new Message({
        clientId: 'test',
        subject: 'Subject',
        body: '<p>Body</p>'
      });
      body = "-----BEGIN PGP MESSAGE-----\nVersion: Keybase OpenPGP v2.0.52 Comment: keybase.io/crypto\n\nwcBMA5nwa6GWVDOUAQf+MjiVRIBWJyM6The6/h2MgSJTDyrN9teFFJTizOvgHNnD W4EpEmmhShNyERI67qXhC03lFczu2Zp2Qofgs8YePIEv7wwb27/cviODsE42YJvX 1zGir+jBp81s9ZiF4dex6Ir9XfiZJlypI2QV2dHjO+5pstW+XhKIc1R5vKvoFTGI 1XmZtL3EgtKfj/HkPUkq2N0G5kAoB2MTTQuurfXm+3TRkftqesyTKlek652sFjCv nSF+LQ1GYq5hI4YaUBiHnZd7wKUgDrIh2rzbuGq+AHjrHdVLMfRTbN0Xsy3OWRcC 9uWU8Nln00Ly6KbTqPXKcBDcMrOJuoxYcpmLlhRds9JoAY7MyIsj87M2mkTtAtMK hqK0PPvJKfepV+eljDhQ7y0TQ0IvNtO5/pcY2CozbFJncm/ToxxZPNJueKRcz+EH M9uBvrWNTwfHj26g405gpRDN1T8CsY5ZeiaDHduIKnBWd4za0ak0Xfw=\n=1aPN\n-----END PGP MESSAGE-----";
      this.encryptedMsg = new Message({
        clientId: 'test2',
        subject: 'Subject',
        body: body
      });
      PGPKeyStore._identities = {};
      pubIdent = new Identity({
        addresses: ["benbitdiddle@icloud.com"],
        isPriv: false
      });
      PGPKeyStore._identities[pubIdent.clientId] = pubIdent;
      privIdent = new Identity({
        addresses: ["benbitdiddle@icloud.com"],
        isPriv: true
      });
      return PGPKeyStore._identities[privIdent.clientId] = privIdent;
    });
    describe("when handling private keys", function() {
      it('should be able to retrieve and unlock a private key', function() {
        var key;
        expect(PGPKeyStore.privKeys().some((function(_this) {
          return function(cv, index, array) {
            return cv.hasOwnProperty("key");
          };
        })(this))).toBeFalsey;
        key = PGPKeyStore.privKeys({
          address: "benbitdiddle@icloud.com",
          timed: false
        })[0];
        return PGPKeyStore.getKeyContents({
          key: key,
          passphrase: "",
          callback: (function(_this) {
            return function() {
              return expect(PGPKeyStore.privKeys({
                timed: false
              }).some(function(cv, index, array) {
                return cv.hasOwnProperty("key");
              })).toBeTruthy;
            };
          })(this)
        });
      });
      it('should not return a private key after its timeout has passed', function() {
        expect(PGPKeyStore.privKeys({
          address: "benbitdiddle@icloud.com",
          timed: false
        }).length).toEqual(1);
        PGPKeyStore.privKeys({
          address: "benbitdiddle@icloud.com",
          timed: false
        })[0].timeout = Date.now() - 5;
        expect(PGPKeyStore.privKeys({
          address: "benbitdiddle@icloud.com",
          timed: true
        }).length).toEqual(0);
        return PGPKeyStore.privKeys({
          address: "benbitdiddle@icloud.com",
          timed: false
        })[0].setTimeout();
      });
      it('should only return the key(s) corresponding to a supplied email address', function() {
        return expect(PGPKeyStore.privKeys({
          address: "wrong@example.com",
          timed: true
        }).length).toEqual(0);
      });
      it('should return all private keys when an address is not supplied', function() {
        return expect(PGPKeyStore.privKeys({
          timed: false
        }).length).toEqual(1);
      });
      return it('should update an existing key when it is unlocked, not add a new one', function() {
        var timeout;
        timeout = PGPKeyStore.privKeys({
          address: "benbitdiddle@icloud.com",
          timed: false
        })[0].timeout;
        return PGPKeyStore.getKeyContents({
          key: PGPKeyStore.privKeys({
            timed: false
          })[0],
          passphrase: "",
          callback: (function(_this) {
            return function() {
              expect(PGPKeyStore.privKeys({
                timed: false
              }).length).toEqual(1);
              return expect(timeout < PGPKeyStore.privKeys({
                address: "benbitdiddle@icloud.com",
                timed: false
              }).timeout);
            };
          })(this)
        });
      });
    });
    describe("when decrypting messages", function() {
      xit('should be able to decrypt a message', function() {
        runs((function(_this) {
          return function() {
            spyOn(PGPKeyStore, 'trigger');
            return PGPKeyStore.getKeyContents({
              key: PGPKeyStore.privKeys({
                timed: false
              })[0],
              passphrase: "",
              callback: function() {
                return PGPKeyStore.decrypt(_this.encryptedMsg);
              }
            });
          };
        })(this));
        waitsFor(((function(_this) {
          return function() {
            return PGPKeyStore.trigger.callCount > 0;
          };
        })(this)), 'message to decrypt');
        return runs((function(_this) {
          return function() {
            return expect(_.findWhere(PGPKeyStore._msgCache, {
              clientId: _this.encryptedMsg.clientId
            })).toExist();
          };
        })(this));
      });
      it('should be able to handle an unencrypted message', function() {
        PGPKeyStore.decrypt(this.unencryptedMsg);
        return expect(_.findWhere(PGPKeyStore._msgCache, {
          clientId: this.unencryptedMsg.clientId
        })).not.toBeDefined();
      });
      it('should be able to tell when a message has no encrypted component', function() {
        expect(PGPKeyStore.hasEncryptedComponent(this.unencryptedMsg)).not;
        return expect(PGPKeyStore.hasEncryptedComponent(this.encryptedMsg));
      });
      it('should be able to handle a message with no BEGIN PGP MESSAGE block', function() {
        var badMsg, body;
        body = "Version: Keybase OpenPGP v2.0.52 Comment: keybase.io/crypto\n\nwcBMA5nwa6GWVDOUAQf+MjiVRIBWJyM6The6/h2MgSJTDyrN9teFFJTizOvgHNnD W4EpEmmhShNyERI67qXhC03lFczu2Zp2Qofgs8YePIEv7wwb27/cviODsE42YJvX 1zGir+jBp81s9ZiF4dex6Ir9XfiZJlypI2QV2dHjO+5pstW+XhKIc1R5vKvoFTGI 1XmZtL3EgtKfj/HkPUkq2N0G5kAoB2MTTQuurfXm+3TRkftqesyTKlek652sFjCv nSF+LQ1GYq5hI4YaUBiHnZd7wKUgDrIh2rzbuGq+AHjrHdVLMfRTbN0Xsy3OWRcC 9uWU8Nln00Ly6KbTqPXKcBDcMrOJuoxYcpmLlhRds9JoAY7MyIsj87M2mkTtAtMK hqK0PPvJKfepV+eljDhQ7y0TQ0IvNtO5/pcY2CozbFJncm/ToxxZPNJueKRcz+EH M9uBvrWNTwfHj26g405gpRDN1T8CsY5ZeiaDHduIKnBWd4za0ak0Xfw=\n=1aPN\n-----END PGP MESSAGE-----";
        badMsg = new Message({
          clientId: 'test2',
          subject: 'Subject',
          body: body
        });
        return PGPKeyStore.getKeyContents({
          key: PGPKeyStore.privKeys({
            timed: false
          })[0],
          passphrase: "",
          callback: (function(_this) {
            return function() {
              PGPKeyStore.decrypt(badMsg);
              return expect(_.findWhere(PGPKeyStore._msgCache, {
                clientId: badMsg.clientId
              })).not.toBeDefined();
            };
          })(this)
        });
      });
      it('should be able to handle a message with no END PGP MESSAGE block', function() {
        var badMsg, body;
        body = "-----BEGIN PGP MESSAGE-----\nVersion: Keybase OpenPGP v2.0.52 Comment: keybase.io/crypto\n\nwcBMA5nwa6GWVDOUAQf+MjiVRIBWJyM6The6/h2MgSJTDyrN9teFFJTizOvgHNnD W4EpEmmhShNyERI67qXhC03lFczu2Zp2Qofgs8YePIEv7wwb27/cviODsE42YJvX 1zGir+jBp81s9ZiF4dex6Ir9XfiZJlypI2QV2dHjO+5pstW+XhKIc1R5vKvoFTGI 1XmZtL3EgtKfj/HkPUkq2N0G5kAoB2MTTQuurfXm+3TRkftqesyTKlek652sFjCv nSF+LQ1GYq5hI4YaUBiHnZd7wKUgDrIh2rzbuGq+AHjrHdVLMfRTbN0Xsy3OWRcC 9uWU8Nln00Ly6KbTqPXKcBDcMrOJuoxYcpmLlhRds9JoAY7MyIsj87M2mkTtAtMK hqK0PPvJKfepV+eljDhQ7y0TQ0IvNtO5/pcY2CozbFJncm/ToxxZPNJueKRcz+EH M9uBvrWNTwfHj26g405gpRDN1T8CsY5ZeiaDHduIKnBWd4za0ak0Xfw=\n=1aPN";
        badMsg = new Message({
          clientId: 'test2',
          subject: 'Subject',
          body: body
        });
        return PGPKeyStore.getKeyContents({
          key: PGPKeyStore.privKeys({
            timed: false
          })[0],
          passphrase: "",
          callback: (function(_this) {
            return function() {
              PGPKeyStore.decrypt(badMsg);
              return expect(_.findWhere(PGPKeyStore._msgCache, {
                clientId: badMsg.clientId
              })).not.toBeDefined();
            };
          })(this)
        });
      });
      it('should not return a decrypted message which has timed out', function() {
        var msg;
        PGPKeyStore._msgCache.push({
          clientId: "testID",
          body: "example body",
          timeout: Date.now()
        });
        msg = new Message({
          clientId: "testID"
        });
        return expect(PGPKeyStore.getDecrypted(msg)).toEqual(null);
      });
      return it('should return a decrypted message', function() {
        var msg, timeout;
        timeout = Date.now() + (1000 * 60 * 60);
        PGPKeyStore._msgCache.push({
          clientId: "testID2",
          body: "example body",
          timeout: timeout
        });
        msg = new Message({
          clientId: "testID2",
          body: "example body"
        });
        return expect(PGPKeyStore.getDecrypted(msg)).toEqual(msg.body);
      });
    });
    return describe("when handling public keys", function() {
      return it("should immediately return a pre-cached key", function() {
        return expect(PGPKeyStore.pubKeys('benbitdiddle@icloud.com').length).toEqual(1);
      });
    });
  });

}).call(this);
