(function() {
  var AccountStore, Actions, ButtonDropdown, Menu, MessageControls, NylasAPI, React, RetinaImg, remote, _ref, _ref1,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require('react');

  remote = require('electron').remote;

  _ref = require('nylas-exports'), Actions = _ref.Actions, NylasAPI = _ref.NylasAPI, AccountStore = _ref.AccountStore;

  _ref1 = require('nylas-component-kit'), RetinaImg = _ref1.RetinaImg, ButtonDropdown = _ref1.ButtonDropdown, Menu = _ref1.Menu;

  MessageControls = (function(_super) {
    __extends(MessageControls, _super);

    MessageControls.displayName = "MessageControls";

    MessageControls.propTypes = {
      thread: React.PropTypes.object.isRequired,
      message: React.PropTypes.object.isRequired
    };

    function MessageControls(props) {
      this.props = props;
      this._onCopyToClipboard = __bind(this._onCopyToClipboard, this);
      this._onLogData = __bind(this._onLogData, this);
      this._onShowOriginal = __bind(this._onShowOriginal, this);
      this._onReport = __bind(this._onReport, this);
      this._onShowActionsMenu = __bind(this._onShowActionsMenu, this);
      this._onForward = __bind(this._onForward, this);
      this._onReplyAll = __bind(this._onReplyAll, this);
      this._onReply = __bind(this._onReply, this);
      this._account = __bind(this._account, this);
      this.render = __bind(this.render, this);
    }

    MessageControls.prototype.render = function() {
      var items;
      items = this._items();
      return React.createElement("div", {
        "className": "message-actions-wrap"
      }, React.createElement(ButtonDropdown, {
        "primaryItem": React.createElement(RetinaImg, {
          "name": items[0].image,
          "mode": RetinaImg.Mode.ContentIsMask
        }),
        "primaryTitle": items[0].name,
        "primaryClick": items[0].select,
        "closeOnMenuClick": true,
        "menu": this._dropdownMenu(items.slice(1))
      }), React.createElement("div", {
        "className": "message-actions-ellipsis",
        "onClick": this._onShowActionsMenu
      }, React.createElement(RetinaImg, {
        "name": "message-actions-ellipsis.png",
        "mode": RetinaImg.Mode.ContentIsMask
      })));
    };

    MessageControls.prototype._items = function() {
      var defaultReplyType, forward, reply, replyAll;
      reply = {
        name: 'Reply',
        image: 'ic-dropdown-reply.png',
        select: this._onReply
      };
      replyAll = {
        name: 'Reply All',
        image: 'ic-dropdown-replyall.png',
        select: this._onReplyAll
      };
      forward = {
        name: 'Forward',
        image: 'ic-dropdown-forward.png',
        select: this._onForward
      };
      if (this.props.message.canReplyAll()) {
        defaultReplyType = NylasEnv.config.get('core.sending.defaultReplyType');
        if (defaultReplyType === 'reply-all') {
          return [replyAll, reply, forward];
        } else {
          return [reply, replyAll, forward];
        }
      } else {
        return [reply, forward];
      }
    };

    MessageControls.prototype._account = function() {
      return AccountStore.accountForId(this.props.message.accountId);
    };

    MessageControls.prototype._dropdownMenu = function(items) {
      var itemContent;
      itemContent = function(item) {
        return React.createElement("span", null, React.createElement(RetinaImg, {
          "name": item.image,
          "mode": RetinaImg.Mode.ContentIsMask
        }), "\u00a0\u00a0", item.name);
      };
      return React.createElement(Menu, {
        "items": items,
        "itemKey": (function(item) {
          return item.name;
        }),
        "itemContent": itemContent,
        "onSelect": ((function(_this) {
          return function(item) {
            return item.select();
          };
        })(this))
      });
    };

    MessageControls.prototype._onReply = function() {
      var message, thread, _ref2;
      _ref2 = this.props, thread = _ref2.thread, message = _ref2.message;
      return Actions.composeReply({
        thread: thread,
        message: message,
        type: 'reply',
        behavior: 'prefer-existing-if-pristine'
      });
    };

    MessageControls.prototype._onReplyAll = function() {
      var message, thread, _ref2;
      _ref2 = this.props, thread = _ref2.thread, message = _ref2.message;
      return Actions.composeReply({
        thread: thread,
        message: message,
        type: 'reply-all',
        behavior: 'prefer-existing-if-pristine'
      });
    };

    MessageControls.prototype._onForward = function() {
      return Actions.composeForward({
        thread: this.props.thread,
        message: this.props.message
      });
    };

    MessageControls.prototype._onShowActionsMenu = function() {
      var SystemMenu, SystemMenuItem, menu;
      SystemMenu = remote.Menu;
      SystemMenuItem = remote.MenuItem;
      menu = new SystemMenu();
      menu.append(new SystemMenuItem({
        label: 'Log Data',
        click: (function(_this) {
          return function() {
            return _this._onLogData();
          };
        })(this)
      }));
      menu.append(new SystemMenuItem({
        label: 'Show Original',
        click: (function(_this) {
          return function() {
            return _this._onShowOriginal();
          };
        })(this)
      }));
      menu.append(new SystemMenuItem({
        label: 'Copy Debug Info to Clipboard',
        click: (function(_this) {
          return function() {
            return _this._onCopyToClipboard();
          };
        })(this)
      }));
      menu.append(new SystemMenuItem({
        type: 'separator'
      }));
      menu.append(new SystemMenuItem({
        label: 'Report Issue: Quoted Text',
        click: (function(_this) {
          return function() {
            return _this._onReport('Quoted Text');
          };
        })(this)
      }));
      menu.append(new SystemMenuItem({
        label: 'Report Issue: Rendering',
        click: (function(_this) {
          return function() {
            return _this._onReport('Rendering');
          };
        })(this)
      }));
      return menu.popup(remote.getCurrentWindow());
    };

    MessageControls.prototype._onReport = function(issueType) {
      var Contact, DatabaseStore, Message, draft, _ref2;
      _ref2 = require('nylas-exports'), Contact = _ref2.Contact, Message = _ref2.Message, DatabaseStore = _ref2.DatabaseStore, AccountStore = _ref2.AccountStore;
      draft = new Message({
        from: [this._account().me()],
        to: [
          new Contact({
            name: "Nylas Team",
            email: "n1-support@nylas.com"
          })
        ],
        date: new Date,
        draft: true,
        subject: "Feedback - Message Display Issue (" + issueType + ")",
        accountId: this._account().id,
        body: this.props.message.body
      });
      return DatabaseStore.inTransaction((function(_this) {
        return function(t) {
          return t.persistModel(draft);
        };
      })(this)).then((function(_this) {
        return function() {
          var dialog;
          Actions.sendDraft(draft.clientId);
          dialog = remote.dialog;
          return dialog.showMessageBox(remote.getCurrentWindow(), {
            type: 'warning',
            buttons: ['OK'],
            message: "Thank you.",
            detail: "The contents of this message have been sent to the N1 team and will be added to a test suite."
          });
        };
      })(this));
    };

    MessageControls.prototype._onShowOriginal = function() {
      var BrowserWindow, app, fs, path, tmpfile;
      fs = require('fs');
      path = require('path');
      BrowserWindow = remote.BrowserWindow;
      app = remote.app;
      tmpfile = path.join(app.getPath('temp'), this.props.message.id);
      return NylasAPI.makeRequest({
        headers: {
          Accept: 'message/rfc822'
        },
        path: "/messages/" + this.props.message.id,
        accountId: this.props.message.accountId,
        json: false,
        success: (function(_this) {
          return function(body) {
            return fs.writeFile(tmpfile, body, function() {
              var window;
              window = new BrowserWindow({
                width: 800,
                height: 600,
                title: "" + _this.props.message.subject + " - RFC822"
              });
              return window.loadURL('file://' + tmpfile);
            });
          };
        })(this)
      });
    };

    MessageControls.prototype._onLogData = function() {
      console.log(this.props.message);
      window.__message = this.props.message;
      window.__thread = this.props.thread;
      return console.log("Also now available in window.__message and window.__thread");
    };

    MessageControls.prototype._onCopyToClipboard = function() {
      var clipboard, data;
      clipboard = require('electron').clipboard;
      data = ("AccountID: " + this.props.message.accountId + "\n") + ("Message ID: " + this.props.message.serverId + "\n") + ("Message Metadata: " + (JSON.stringify(this.props.message.pluginMetadata, null, '  ')) + "\n") + ("Thread ID: " + this.props.thread.serverId + "\n") + ("Thread Metadata: " + (JSON.stringify(this.props.thread.pluginMetadata, null, '  ')) + "\n");
      return clipboard.writeText(data);
    };

    return MessageControls;

  })(React.Component);

  module.exports = MessageControls;

}).call(this);
