(function() {
  var CanvasUtils, DraftHelpers, EmailFrame, FileDownloadStore, InjectedComponentSet, MessageBodyProcessor, MessageItemBody, MessageUtils, NylasAPI, QuotedHTMLTransformer, React, RetinaImg, TransparentPixel, _, _ref, _ref1,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require('react');

  _ = require('underscore');

  EmailFrame = require('./email-frame')["default"];

  _ref = require('nylas-exports'), DraftHelpers = _ref.DraftHelpers, CanvasUtils = _ref.CanvasUtils, NylasAPI = _ref.NylasAPI, MessageUtils = _ref.MessageUtils, MessageBodyProcessor = _ref.MessageBodyProcessor, QuotedHTMLTransformer = _ref.QuotedHTMLTransformer, FileDownloadStore = _ref.FileDownloadStore;

  _ref1 = require('nylas-component-kit'), InjectedComponentSet = _ref1.InjectedComponentSet, RetinaImg = _ref1.RetinaImg;

  TransparentPixel = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAQAAAC1HAwCAAAAC0lEQVR4nGNikAQAACIAHF/uBd8AAAAASUVORK5CYII=";

  MessageItemBody = (function(_super) {
    __extends(MessageItemBody, _super);

    MessageItemBody.displayName = 'MessageItemBody';

    MessageItemBody.propTypes = {
      message: React.PropTypes.object.isRequired,
      downloads: React.PropTypes.object.isRequired
    };

    function MessageItemBody(props) {
      this.props = props;
      this._mergeBodyWithFiles = __bind(this._mergeBodyWithFiles, this);
      this._onFetchBody = __bind(this._onFetchBody, this);
      this._toggleQuotedText = __bind(this._toggleQuotedText, this);
      this._renderQuotedTextControl = __bind(this._renderQuotedTextControl, this);
      this._renderBody = __bind(this._renderBody, this);
      this.render = __bind(this.render, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.componentWillMount = __bind(this.componentWillMount, this);
      this._unmounted = false;
      this.state = {
        showQuotedText: DraftHelpers.isForwardedMessage(this.props.message),
        processedBody: null,
        error: null
      };
    }

    MessageItemBody.prototype.componentWillMount = function() {
      return this._unsub = MessageBodyProcessor.subscribe(this.props.message, (function(_this) {
        return function(processedBody) {
          return _this.setState({
            processedBody: processedBody
          });
        };
      })(this));
    };

    MessageItemBody.prototype.componentDidMount = function() {
      if (!_.isString(this.props.message.body)) {
        return this._onFetchBody();
      }
    };

    MessageItemBody.prototype.componentWillReceiveProps = function(nextProps) {
      if (nextProps.message.id !== this.props.message.id) {
        if (typeof this._unsub === "function") {
          this._unsub();
        }
        return this._unsub = MessageBodyProcessor.subscribe(nextProps.message, (function(_this) {
          return function(processedBody) {
            return _this.setState({
              processedBody: processedBody
            });
          };
        })(this));
      }
    };

    MessageItemBody.prototype.componentWillUnmount = function() {
      this._unmounted = true;
      return typeof this._unsub === "function" ? this._unsub() : void 0;
    };

    MessageItemBody.prototype.render = function() {
      return React.createElement("span", null, React.createElement(InjectedComponentSet, {
        "matching": {
          role: "message:BodyHeader"
        },
        "exposedProps": {
          message: this.props.message
        },
        "direction": "column",
        "style": {
          width: '100%'
        }
      }), this._renderBody(), this._renderQuotedTextControl());
    };

    MessageItemBody.prototype._renderBody = function() {
      if (_.isString(this.props.message.body) && _.isString(this.state.processedBody)) {
        return React.createElement(EmailFrame, {
          "showQuotedText": this.state.showQuotedText,
          "content": this._mergeBodyWithFiles(this.state.processedBody),
          "message": this.props.message
        });
      } else if (this.state.error) {
        return React.createElement("div", {
          "className": "message-body-error"
        }, "Sorry, this message could not be loaded. (Status code ", this.state.error.statusCode, ")", React.createElement("a", {
          "onClick": this._onFetchBody
        }, "Try Again"));
      } else {
        return React.createElement("div", {
          "className": "message-body-loading"
        }, React.createElement(RetinaImg, {
          "name": "inline-loading-spinner.gif",
          "mode": RetinaImg.Mode.ContentDark,
          "style": {
            width: 14,
            height: 14
          }
        }));
      }
    };

    MessageItemBody.prototype._renderQuotedTextControl = function() {
      if (!QuotedHTMLTransformer.hasQuotedHTML(this.props.message.body)) {
        return null;
      }
      return React.createElement("a", {
        "className": "quoted-text-control",
        "onClick": this._toggleQuotedText
      }, React.createElement("span", {
        "className": "dots"
      }, "\u2022\u2022\u2022"));
    };

    MessageItemBody.prototype._toggleQuotedText = function() {
      return this.setState({
        showQuotedText: !this.state.showQuotedText
      });
    };

    MessageItemBody.prototype._onFetchBody = function() {
      return NylasAPI.makeRequest({
        path: "/messages/" + this.props.message.id,
        accountId: this.props.message.accountId,
        returnsModel: true
      }).then((function(_this) {
        return function() {
          if (_this._unmounted) {
            return;
          }
          return _this.setState({
            error: null
          });
        };
      })(this))["catch"]((function(_this) {
        return function(error) {
          if (_this._unmounted) {
            return;
          }
          return _this.setState({
            error: error
          });
        };
      })(this));
    };

    MessageItemBody.prototype._mergeBodyWithFiles = function(body) {
      var cidRegexp, download, file, _i, _len, _ref2;
      _ref2 = this.props.message.files;
      for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
        file = _ref2[_i];
        download = this.props.downloads[file.id];
        cidRegexp = new RegExp("cid:" + file.contentId + "(['\"]+)", 'gi');
        if (download && download.state !== 'finished') {
          body = body.replace(cidRegexp, function(text, quoteCharacter) {
            var dataUri;
            dataUri = CanvasUtils.dataURIForLoadedPercent(download.percent);
            return "" + dataUri + quoteCharacter + " style=" + quoteCharacter + " object-position: 50% 50%; object-fit: none; ";
          });
        } else {
          body = body.replace(cidRegexp, function(text, quoteCharacter) {
            return "" + (FileDownloadStore.pathForFile(file)) + quoteCharacter;
          });
        }
      }
      body = body.replace(MessageUtils.cidRegex, "src=\"" + TransparentPixel + "\"");
      return body;
    };

    return MessageItemBody;

  })(React.Component);

  module.exports = MessageItemBody;

}).call(this);
