(function() {
  var ComponentRegistry, DraftStore, MessageItem, MessageItemContainer, MessageStore, React, Utils, classNames, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require('react');

  classNames = require('classnames');

  MessageItem = require('./message-item');

  _ref = require('nylas-exports'), Utils = _ref.Utils, DraftStore = _ref.DraftStore, ComponentRegistry = _ref.ComponentRegistry, MessageStore = _ref.MessageStore;

  MessageItemContainer = (function(_super) {
    __extends(MessageItemContainer, _super);

    MessageItemContainer.displayName = 'MessageItemContainer';

    MessageItemContainer.propTypes = {
      thread: React.PropTypes.object.isRequired,
      message: React.PropTypes.object.isRequired,
      collapsed: React.PropTypes.bool,
      isLastMsg: React.PropTypes.bool,
      isBeforeReplyArea: React.PropTypes.bool,
      scrollTo: React.PropTypes.func
    };

    function MessageItemContainer(props) {
      this.props = props;
      this._onSendingStateChanged = __bind(this._onSendingStateChanged, this);
      this._classNames = __bind(this._classNames, this);
      this._renderComposer = __bind(this._renderComposer, this);
      this._renderMessage = __bind(this._renderMessage, this);
      this.render = __bind(this.render, this);
      this.focus = __bind(this.focus, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.shouldComponentUpdate = __bind(this.shouldComponentUpdate, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.state = this._getStateFromStores();
    }

    MessageItemContainer.prototype.componentWillReceiveProps = function(newProps) {
      return this.setState(this._getStateFromStores(newProps));
    };

    MessageItemContainer.prototype.componentDidMount = function() {
      if (this.props.message.draft) {
        return this._unlisten = DraftStore.listen(this._onSendingStateChanged);
      }
    };

    MessageItemContainer.prototype.shouldComponentUpdate = function(nextProps, nextState) {
      return !Utils.isEqualReact(nextProps, this.props) || !Utils.isEqualReact(nextState, this.state);
    };

    MessageItemContainer.prototype.componentWillUnmount = function() {
      if (this._unlisten) {
        return this._unlisten();
      }
    };

    MessageItemContainer.prototype.focus = function() {
      return this.refs.message.focus();
    };

    MessageItemContainer.prototype.render = function() {
      if (this.state.isSending) {
        return this._renderMessage({
          pending: true
        });
      } else if (this.props.message.draft) {
        return this._renderComposer();
      } else {
        return this._renderMessage({
          pending: false
        });
      }
    };

    MessageItemContainer.prototype._renderMessage = function(_arg) {
      var pending;
      pending = _arg.pending;
      return React.createElement(MessageItem, {
        "ref": "message",
        "pending": pending,
        "thread": this.props.thread,
        "message": this.props.message,
        "className": this._classNames(),
        "collapsed": this.props.collapsed,
        "isLastMsg": this.props.isLastMsg
      });
    };

    MessageItemContainer.prototype._renderComposer = function() {
      var Composer;
      Composer = ComponentRegistry.findComponentsMatching({
        role: 'Composer'
      })[0];
      if (!Composer) {
        return React.createElement("span", null);
      }
      return React.createElement(Composer, {
        "ref": "message",
        "draftClientId": this.props.message.clientId,
        "className": this._classNames(),
        "mode": "inline",
        "threadId": this.props.thread.id,
        "scrollTo": this.props.scrollTo
      });
    };

    MessageItemContainer.prototype._classNames = function() {
      return classNames({
        "draft": this.props.message.draft,
        "unread": this.props.message.unread,
        "collapsed": this.props.collapsed,
        "message-item-wrap": true,
        "before-reply-area": this.props.isBeforeReplyArea
      });
    };

    MessageItemContainer.prototype._onSendingStateChanged = function(draftClientId) {
      if (draftClientId === this.props.message.clientId) {
        return this.setState(this._getStateFromStores());
      }
    };

    MessageItemContainer.prototype._getStateFromStores = function(props) {
      if (props == null) {
        props = this.props;
      }
      return {
        isSending: DraftStore.isSendingDraft(props.message.clientId)
      };
    };

    return MessageItemContainer;

  })(React.Component);

  module.exports = MessageItemContainer;

}).call(this);
