(function() {
  var AccountStore, Actions, ComponentRegistry, EmailFrame, FileDownloadStore, InjectedComponent, InjectedComponentSet, MessageBodyProcessor, MessageControls, MessageItem, MessageItemBody, MessageParticipants, MessageStore, MessageTimestamp, MessageUtils, QuotedHTMLTransformer, React, RetinaImg, TransparentPixel, Utils, classNames, _, _ref, _ref1,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  React = require('react');

  classNames = require('classnames');

  _ = require('underscore');

  EmailFrame = require('./email-frame')["default"];

  MessageParticipants = require("./message-participants");

  MessageItemBody = require("./message-item-body");

  MessageTimestamp = require("./message-timestamp");

  MessageControls = require('./message-controls');

  _ref = require('nylas-exports'), Utils = _ref.Utils, Actions = _ref.Actions, MessageUtils = _ref.MessageUtils, AccountStore = _ref.AccountStore, MessageStore = _ref.MessageStore, MessageBodyProcessor = _ref.MessageBodyProcessor, QuotedHTMLTransformer = _ref.QuotedHTMLTransformer, ComponentRegistry = _ref.ComponentRegistry, FileDownloadStore = _ref.FileDownloadStore;

  _ref1 = require('nylas-component-kit'), RetinaImg = _ref1.RetinaImg, InjectedComponentSet = _ref1.InjectedComponentSet, InjectedComponent = _ref1.InjectedComponent;

  TransparentPixel = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAQAAAC1HAwCAAAAC0lEQVR4nGNikAQAACIAHF/uBd8AAAAASUVORK5CYII=";

  MessageItem = (function(_super) {
    __extends(MessageItem, _super);

    MessageItem.displayName = 'MessageItem';

    MessageItem.propTypes = {
      thread: React.PropTypes.object.isRequired,
      message: React.PropTypes.object.isRequired,
      collapsed: React.PropTypes.bool
    };

    function MessageItem(props) {
      this.props = props;
      this._onDownloadStoreChange = __bind(this._onDownloadStoreChange, this);
      this._attachmentComponents = __bind(this._attachmentComponents, this);
      this._formatContacts = __bind(this._formatContacts, this);
      this._toggleCollapsed = __bind(this._toggleCollapsed, this);
      this._renderHeaderDetailToggle = __bind(this._renderHeaderDetailToggle, this);
      this._renderAttachments = __bind(this._renderAttachments, this);
      this._renderDownloadAllButton = __bind(this._renderDownloadAllButton, this);
      this._onDownloadAll = __bind(this._onDownloadAll, this);
      this._onClickHeader = __bind(this._onClickHeader, this);
      this._onClickParticipants = __bind(this._onClickParticipants, this);
      this._renderFolder = __bind(this._renderFolder, this);
      this._renderToParticipants = __bind(this._renderToParticipants, this);
      this._renderFromParticipants = __bind(this._renderFromParticipants, this);
      this._renderHeader = __bind(this._renderHeader, this);
      this._renderFull = __bind(this._renderFull, this);
      this._renderCollapsed = __bind(this._renderCollapsed, this);
      this.render = __bind(this.render, this);
      this.shouldComponentUpdate = __bind(this.shouldComponentUpdate, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.state = {
        downloads: FileDownloadStore.downloadDataForFiles(this.props.message.fileIds()),
        detailedHeaders: false
      };
    }

    MessageItem.prototype.componentDidMount = function() {
      return this._storeUnlisten = FileDownloadStore.listen(this._onDownloadStoreChange);
    };

    MessageItem.prototype.componentWillUnmount = function() {
      if (this._storeUnlisten) {
        return this._storeUnlisten();
      }
    };

    MessageItem.prototype.shouldComponentUpdate = function(nextProps, nextState) {
      return !Utils.isEqualReact(nextProps, this.props) || !Utils.isEqualReact(nextState, this.state);
    };

    MessageItem.prototype.render = function() {
      if (this.props.collapsed) {
        return this._renderCollapsed();
      } else {
        return this._renderFull();
      }
    };

    MessageItem.prototype._renderCollapsed = function() {
      var attachmentIcon, _ref2, _ref3;
      attachmentIcon = [];
      if (Utils.showIconForAttachments(this.props.message.files)) {
        attachmentIcon = React.createElement("div", {
          "className": "collapsed-attachment"
        });
      }
      return React.createElement("div", {
        "className": this.props.className,
        "onClick": this._toggleCollapsed
      }, React.createElement("div", {
        "className": "message-item-white-wrap"
      }, React.createElement("div", {
        "className": "message-item-area"
      }, React.createElement("div", {
        "className": "collapsed-from"
      }, (_ref2 = this.props.message.from) != null ? (_ref3 = _ref2[0]) != null ? _ref3.displayName({
        compact: true
      }) : void 0 : void 0), React.createElement("div", {
        "className": "collapsed-snippet"
      }, this.props.message.snippet), React.createElement("div", {
        "className": "collapsed-timestamp"
      }, React.createElement(MessageTimestamp, {
        "date": this.props.message.date
      })), attachmentIcon)));
    };

    MessageItem.prototype._renderFull = function() {
      return React.createElement("div", {
        "className": this.props.className
      }, React.createElement("div", {
        "className": "message-item-white-wrap"
      }, React.createElement("div", {
        "className": "message-item-area"
      }, this._renderHeader(), React.createElement(MessageItemBody, {
        "message": this.props.message,
        "downloads": this.state.downloads
      }), this._renderAttachments())));
    };

    MessageItem.prototype._renderHeader = function() {
      var classes;
      classes = classNames({
        "message-header": true,
        "pending": this.props.pending
      });
      return React.createElement("header", {
        "className": classes,
        "onClick": this._onClickHeader
      }, this._renderHeaderSideItems(), React.createElement("div", {
        "className": "message-header-right"
      }, React.createElement(MessageTimestamp, {
        "className": "message-time",
        "isDetailed": this.state.detailedHeaders,
        "date": this.props.message.date
      }), React.createElement(InjectedComponentSet, {
        "className": "message-header-status",
        "matching": {
          role: "MessageHeaderStatus"
        },
        "exposedProps": {
          message: this.props.message,
          thread: this.props.thread,
          detailedHeaders: this.state.detailedHeaders
        }
      }), React.createElement(MessageControls, {
        "thread": this.props.thread,
        "message": this.props.message
      })), this._renderFromParticipants(), this._renderToParticipants(), this._renderFolder(), this._renderHeaderDetailToggle());
    };

    MessageItem.prototype._renderFromParticipants = function() {
      return React.createElement(MessageParticipants, {
        "from": this.props.message.from,
        "onClick": this._onClickParticipants,
        "isDetailed": this.state.detailedHeaders
      });
    };

    MessageItem.prototype._renderToParticipants = function() {
      return React.createElement(MessageParticipants, {
        "to": this.props.message.to,
        "cc": this.props.message.cc,
        "bcc": this.props.message.bcc,
        "onClick": this._onClickParticipants,
        "isDetailed": this.state.detailedHeaders
      });
    };

    MessageItem.prototype._renderFolder = function() {
      var acct, acctUsesFolders, folder, _ref2;
      if (!this.state.detailedHeaders) {
        return [];
      }
      acct = AccountStore.accountForId(this.props.message.accountId);
      acctUsesFolders = acct && acct.usesFolders();
      folder = (_ref2 = this.props.message.categories) != null ? _ref2[0] : void 0;
      if (!(folder && acctUsesFolders)) {
        return;
      }
      return React.createElement("div", {
        "className": "header-row"
      }, React.createElement("div", {
        "className": "header-label"
      }, "Folder:\u00a0"), React.createElement("div", {
        "className": "header-name"
      }, folder.displayName));
    };

    MessageItem.prototype._onClickParticipants = function(e) {
      var el;
      el = e.target;
      while (el !== e.currentTarget) {
        if (__indexOf.call(el.classList, "collapsed-participants") >= 0) {
          this.setState({
            detailedHeaders: true
          });
          e.stopPropagation();
          return;
        }
        el = el.parentElement;
      }
    };

    MessageItem.prototype._onClickHeader = function(e) {
      var el, wl;
      if (this.state.detailedHeaders) {
        return;
      }
      el = e.target;
      while (el !== e.currentTarget) {
        wl = ["message-header-right", "collapsed-participants", "header-toggle-control"];
        if (__indexOf.call(el.classList, "message-header-right") >= 0) {
          return;
        }
        if (__indexOf.call(el.classList, "collapsed-participants") >= 0) {
          return;
        }
        el = el.parentElement;
      }
      return this._toggleCollapsed();
    };

    MessageItem.prototype._onDownloadAll = function() {
      return Actions.fetchAndSaveAllFiles(this.props.message.files);
    };

    MessageItem.prototype._renderDownloadAllButton = function() {
      return React.createElement("div", {
        "className": "download-all"
      }, React.createElement("div", {
        "className": "attachment-number"
      }, React.createElement(RetinaImg, {
        "name": "ic-attachments-all-clippy.png",
        "mode": RetinaImg.Mode.ContentIsMask
      }), React.createElement("span", null, this.props.message.files.length, " attachments")), React.createElement("div", {
        "className": "separator"
      }, "-"), React.createElement("div", {
        "className": "download-all-action",
        "onClick": this._onDownloadAll
      }, React.createElement(RetinaImg, {
        "name": "ic-attachments-download-all.png",
        "mode": RetinaImg.Mode.ContentIsMask
      }), React.createElement("span", null, "Download all")));
    };

    MessageItem.prototype._renderAttachments = function() {
      var attachments;
      attachments = this._attachmentComponents();
      if (attachments.length > 0) {
        return React.createElement("div", null, (attachments.length > 1 ? this._renderDownloadAllButton() : void 0), React.createElement("div", {
          "className": "attachments-area"
        }, attachments));
      } else {
        return React.createElement("div", null);
      }
    };

    MessageItem.prototype._renderHeaderSideItems = function() {
      var styles;
      styles = {
        position: "absolute",
        marginTop: -2
      };
      return React.createElement("div", {
        "className": "pending-spinner",
        "style": styles
      }, React.createElement(RetinaImg, {
        "ref": "spinner",
        "name": "sending-spinner.gif",
        "mode": RetinaImg.Mode.ContentPreserve
      }));
    };

    MessageItem.prototype._renderHeaderDetailToggle = function() {
      if (this.props.pending) {
        return null;
      }
      if (this.state.detailedHeaders) {
        return React.createElement("div", {
          "className": "header-toggle-control",
          "style": {
            top: "18px",
            left: "-14px"
          },
          "onClick": ((function(_this) {
            return function(e) {
              _this.setState({
                detailedHeaders: false
              });
              return e.stopPropagation();
            };
          })(this))
        }, React.createElement(RetinaImg, {
          "name": "message-disclosure-triangle-active.png",
          "mode": RetinaImg.Mode.ContentIsMask
        }));
      } else {
        return React.createElement("div", {
          "className": "header-toggle-control inactive",
          "style": {
            top: "18px"
          },
          "onClick": ((function(_this) {
            return function(e) {
              _this.setState({
                detailedHeaders: true
              });
              return e.stopPropagation();
            };
          })(this))
        }, React.createElement(RetinaImg, {
          "name": "message-disclosure-triangle.png",
          "mode": RetinaImg.Mode.ContentIsMask
        }));
      }
    };

    MessageItem.prototype._toggleCollapsed = function() {
      if (this.props.isLastMsg) {
        return;
      }
      return Actions.toggleMessageIdExpanded(this.props.message.id);
    };

    MessageItem.prototype._formatContacts = function(contacts) {
      if (contacts == null) {
        contacts = [];
      }
    };

    MessageItem.prototype._attachmentComponents = function() {
      var file, imageAttachments, otherAttachments, _i, _len, _ref2, _ref3;
      imageAttachments = [];
      otherAttachments = [];
      _ref3 = (_ref2 = this.props.message.files) != null ? _ref2 : [];
      for (_i = 0, _len = _ref3.length; _i < _len; _i++) {
        file = _ref3[_i];
        if (!this._isRealFile(file)) {
          continue;
        }
        if (Utils.shouldDisplayAsImage(file)) {
          imageAttachments.push(file);
        } else {
          otherAttachments.push(file);
        }
      }
      otherAttachments = otherAttachments.map((function(_this) {
        return function(file) {
          return React.createElement(InjectedComponent, {
            "className": "file-wrap",
            "matching": {
              role: "Attachment"
            },
            "exposedProps": {
              file: file,
              download: _this.state.downloads[file.id]
            },
            "key": file.id
          });
        };
      })(this));
      imageAttachments = imageAttachments.map((function(_this) {
        return function(file) {
          var props;
          props = {
            file: file,
            download: _this.state.downloads[file.id],
            targetPath: FileDownloadStore.pathForFile(file)
          };
          return React.createElement(InjectedComponent, {
            "className": "file-wrap file-image-wrap",
            "matching": {
              role: "Attachment:Image"
            },
            "exposedProps": props,
            "key": file.id
          });
        };
      })(this));
      return otherAttachments.concat(imageAttachments);
    };

    MessageItem.prototype._isRealFile = function(file) {
      var hasCIDInBody, _ref2;
      hasCIDInBody = (file.contentId != null) && ((_ref2 = this.props.message.body) != null ? _ref2.indexOf(file.contentId) : void 0) > 0;
      return !hasCIDInBody;
    };

    MessageItem.prototype._onDownloadStoreChange = function() {
      return this.setState({
        downloads: FileDownloadStore.downloadDataForFiles(this.props.message.fileIds())
      });
    };

    return MessageItem;

  })(React.Component);

  module.exports = MessageItem;

}).call(this);
