(function() {
  var Contact, MAX_COLLAPSED, MessageParticipants, React, classnames, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  React = require("react");

  classnames = require('classnames');

  Contact = require('nylas-exports').Contact;

  MAX_COLLAPSED = 5;

  MessageParticipants = (function(_super) {
    __extends(MessageParticipants, _super);

    function MessageParticipants() {
      this.render = __bind(this.render, this);
      this._renderCollapsed = __bind(this._renderCollapsed, this);
      this._renderExpanded = __bind(this._renderExpanded, this);
      this._renderExpandedField = __bind(this._renderExpandedField, this);
      this._renderFullContacts = __bind(this._renderFullContacts, this);
      this._shortNames = __bind(this._shortNames, this);
      this._selectText = __bind(this._selectText, this);
      this._allToParticipants = __bind(this._allToParticipants, this);
      return MessageParticipants.__super__.constructor.apply(this, arguments);
    }

    MessageParticipants.displayName = 'MessageParticipants';

    MessageParticipants.propTypes = {
      to: React.PropTypes.array,
      cc: React.PropTypes.array,
      bcc: React.PropTypes.array,
      from: React.PropTypes.array,
      onClick: React.PropTypes.func,
      isDetailed: React.PropTypes.bool
    };

    MessageParticipants.defaultProps = {
      to: [],
      cc: [],
      bcc: [],
      from: []
    };

    MessageParticipants.prototype._allToParticipants = function() {
      return _.union(this.props.to, this.props.cc, this.props.bcc);
    };

    MessageParticipants.prototype._selectText = function(e) {
      var range, selection, textNode;
      textNode = e.currentTarget.childNodes[0];
      range = document.createRange();
      range.setStart(textNode, 0);
      range.setEnd(textNode, textNode.length);
      selection = document.getSelection();
      selection.removeAllRanges();
      return selection.addRange(range);
    };

    MessageParticipants.prototype._shortNames = function(contacts, max) {
      var extra, names;
      if (contacts == null) {
        contacts = [];
      }
      if (max == null) {
        max = MAX_COLLAPSED;
      }
      names = _.map(contacts, function(c) {
        return c.displayName({
          includeAccountLabel: true,
          compact: true
        });
      });
      if (names.length > max) {
        extra = names.length - max;
        names = names.slice(0, max);
        names.push("and " + extra + " more");
      }
      return names.join(", ");
    };

    MessageParticipants.prototype._renderFullContacts = function(contacts) {
      if (contacts == null) {
        contacts = [];
      }
      return _.map(contacts, (function(_this) {
        return function(c, i) {
          var comma, _ref;
          if (contacts.length === 1) {
            comma = "";
          } else if (i === contacts.length - 1) {
            comma = "";
          } else {
            comma = ",";
          }
          if (((_ref = c.name) != null ? _ref.length : void 0) > 0 && c.name !== c.email) {
            return React.createElement("div", {
              "key": "" + c.email + "-" + i,
              "className": "participant selectable"
            }, React.createElement("div", {
              "className": "participant-primary",
              "onClick": _this._selectText
            }, c.fullName()), React.createElement("div", {
              "className": "participant-secondary"
            }, "<", React.createElement("span", {
              "onClick": _this._selectText
            }, c.email), ">" + comma));
          } else {
            return React.createElement("div", {
              "key": "" + c.email + "-" + i,
              "className": "participant selectable"
            }, React.createElement("div", {
              "className": "participant-primary"
            }, React.createElement("span", {
              "onClick": _this._selectText
            }, c.email), comma));
          }
        };
      })(this));
    };

    MessageParticipants.prototype._renderExpandedField = function(name, field, _arg) {
      var includeLabel;
      includeLabel = (_arg != null ? _arg : {}).includeLabel;
      if (includeLabel == null) {
        includeLabel = true;
      }
      return React.createElement("div", {
        "className": "participant-type",
        "key": "participant-type-" + name
      }, (includeLabel ? React.createElement("div", {
        "className": "participant-label " + name + "-label"
      }, name, ":\u00a0") : void 0), React.createElement("div", {
        "className": "participant-name " + name + "-contact"
      }, this._renderFullContacts(field)));
    };

    MessageParticipants.prototype._renderExpanded = function() {
      var expanded;
      expanded = [];
      if (this.props.from.length > 0) {
        expanded.push(this._renderExpandedField('from', this.props.from, {
          includeLabel: false
        }));
      }
      if (this.props.to.length > 0) {
        expanded.push(this._renderExpandedField('to', this.props.to));
      }
      if (this.props.cc.length > 0) {
        expanded.push(this._renderExpandedField('cc', this.props.cc));
      }
      if (this.props.bcc.length > 0) {
        expanded.push(this._renderExpandedField('bcc', this.props.bcc));
      }
      return React.createElement("div", {
        "className": "expanded-participants"
      }, expanded);
    };

    MessageParticipants.prototype._renderCollapsed = function() {
      var childSpans, toParticipants;
      childSpans = [];
      toParticipants = this._allToParticipants();
      if (this.props.from.length > 0) {
        childSpans.push(React.createElement("span", {
          "className": "participant-name from-contact",
          "key": "from"
        }, this._shortNames(this.props.from)));
      }
      if (toParticipants.length > 0) {
        childSpans.push(React.createElement("span", {
          "className": "participant-label to-label",
          "key": "to-label"
        }, "To:\u00a0"), React.createElement("span", {
          "className": "participant-name to-contact",
          "key": "to-value"
        }, this._shortNames(toParticipants)));
      }
      return React.createElement("span", {
        "className": "collapsed-participants"
      }, childSpans);
    };

    MessageParticipants.prototype.render = function() {
      var classSet;
      classSet = classnames({
        "participants": true,
        "message-participants": true,
        "collapsed": !this.props.isDetailed,
        "from-participants": this.props.from.length > 0,
        "to-participants": this._allToParticipants().length > 0
      });
      return React.createElement("div", {
        "className": classSet,
        "onClick": this.props.onClick
      }, (this.props.isDetailed ? this._renderExpanded() : this._renderCollapsed()));
    };

    return MessageParticipants;

  })(React.Component);

  module.exports = MessageParticipants;

}).call(this);
