(function() {
  var Contact, EmailFrameStub, File, FileDownloadStore, InjectedComponent, Message, MessageBodyProcessor, MessageItemBody, React, ReactDOM, ReactTestUtils, download, download_inline, file, file_cid_but_not_referenced, file_cid_but_not_referenced_or_image, file_inline, file_inline_downloading, file_inline_not_downloaded, file_not_downloaded, file_without_filename, proxyquire, user_1, user_2, user_3, user_4, _ref;

  proxyquire = require('proxyquire');

  React = require("react");

  ReactDOM = require("react-dom");

  ReactTestUtils = require('react-addons-test-utils');

  _ref = require("nylas-exports"), Contact = _ref.Contact, Message = _ref.Message, File = _ref.File, FileDownloadStore = _ref.FileDownloadStore, MessageBodyProcessor = _ref.MessageBodyProcessor;

  EmailFrameStub = React.createClass({
    render: function() {
      return React.createElement("div", null);
    }
  });

  InjectedComponent = require('nylas-component-kit').InjectedComponent;

  file = new File({
    id: 'file_1_id',
    filename: 'a.png',
    contentType: 'image/png',
    size: 10
  });

  file_not_downloaded = new File({
    id: 'file_2_id',
    filename: 'b.png',
    contentType: 'image/png',
    size: 10
  });

  file_inline = new File({
    id: 'file_inline_id',
    filename: 'c.png',
    contentId: 'file_inline_id',
    contentType: 'image/png',
    size: 10
  });

  file_inline_downloading = new File({
    id: 'file_inline_downloading_id',
    filename: 'd.png',
    contentId: 'file_inline_downloading_id',
    contentType: 'image/png',
    size: 10
  });

  file_inline_not_downloaded = new File({
    id: 'file_inline_not_downloaded_id',
    filename: 'e.png',
    contentId: 'file_inline_not_downloaded_id',
    contentType: 'image/png',
    size: 10
  });

  file_cid_but_not_referenced = new File({
    id: 'file_cid_but_not_referenced',
    filename: 'f.png',
    contentId: 'file_cid_but_not_referenced',
    contentType: 'image/png',
    size: 10
  });

  file_cid_but_not_referenced_or_image = new File({
    id: 'file_cid_but_not_referenced_or_image',
    filename: 'ansible notes.txt',
    contentId: 'file_cid_but_not_referenced_or_image',
    contentType: 'text/plain',
    size: 300
  });

  file_without_filename = new File({
    id: 'file_without_filename',
    contentType: 'image/png',
    size: 10
  });

  download = {
    fileId: 'file_1_id'
  };

  download_inline = {
    fileId: 'file_inline_downloading_id'
  };

  user_1 = new Contact({
    name: "User One",
    email: "user1@nylas.com"
  });

  user_2 = new Contact({
    name: "User Two",
    email: "user2@nylas.com"
  });

  user_3 = new Contact({
    name: "User Three",
    email: "user3@nylas.com"
  });

  user_4 = new Contact({
    name: "User Four",
    email: "user4@nylas.com"
  });

  MessageItemBody = proxyquire('../lib/message-item-body', {
    './email-frame': {
      "default": EmailFrameStub
    }
  });

  describe("MessageItem", function() {
    beforeEach(function() {
      spyOn(FileDownloadStore, 'pathForFile').andCallFake(function(f) {
        if (f.id === file.id) {
          return '/fake/path.png';
        }
        if (f.id === file_inline.id) {
          return '/fake/path-inline.png';
        }
        if (f.id === file_inline_downloading.id) {
          return '/fake/path-downloading.png';
        }
        return null;
      });
      spyOn(MessageBodyProcessor, '_addToCache').andCallFake(function() {});
      this.downloads = {
        'file_1_id': download,
        'file_inline_downloading_id': download_inline
      };
      this.message = new Message({
        id: "111",
        from: [user_1],
        to: [user_2],
        cc: [user_3, user_4],
        bcc: null,
        body: "Body One",
        date: new Date(1415814587),
        draft: false,
        files: [],
        unread: false,
        snippet: "snippet one...",
        subject: "Subject One",
        threadId: "thread_12345",
        accountId: window.TEST_ACCOUNT_ID
      });
      return this.createComponent = (function(_this) {
        return function(_arg) {
          var collapsed;
          collapsed = (_arg != null ? _arg : {}).collapsed;
          if (collapsed == null) {
            collapsed = false;
          }
          _this.component = ReactTestUtils.renderIntoDocument(React.createElement(MessageItemBody, {
            "message": _this.message,
            "downloads": _this.downloads
          }));
          return advanceClock();
        };
      })(this);
    });
    describe("when the message contains attachments", function() {
      beforeEach(function() {
        return this.message.files = [file, file_not_downloaded, file_cid_but_not_referenced, file_cid_but_not_referenced_or_image, file_inline, file_inline_downloading, file_inline_not_downloaded, file_without_filename];
      });
      return describe("inline", function() {
        beforeEach(function() {
          this.message.body = "<img alt=\"A\" src=\"cid:" + file_inline.contentId + "\"/>\n<img alt=\"B\" src=\"cid:" + file_inline_downloading.contentId + "\"/>\n<img alt=\"C\" src=\"cid:" + file_inline_not_downloaded.contentId + "\"/>\n<img src=\"cid:missing-attachment\"/>";
          return this.createComponent();
        });
        it("should never leave src=cid:// in the message body", function() {
          var body;
          body = ReactTestUtils.findRenderedComponentWithType(this.component, EmailFrameStub).props.content;
          return expect(body.indexOf('cid')).toEqual(-1);
        });
        it("should replace cid://<file.contentId> with the FileDownloadStore's path for the file", function() {
          var body;
          body = ReactTestUtils.findRenderedComponentWithType(this.component, EmailFrameStub).props.content;
          return expect(body.indexOf('alt="A" src="/fake/path-inline.png"')).toEqual(this.message.body.indexOf('alt="A"'));
        });
        return it("should not replace cid://<file.contentId> with the FileDownloadStore's path if the download is in progress", function() {
          var body;
          body = ReactTestUtils.findRenderedComponentWithType(this.component, EmailFrameStub).props.content;
          return expect(body.indexOf('/fake/path-downloading.png')).toEqual(-1);
        });
      });
    });
    return describe("showQuotedText", function() {
      it("should be initialized to false", function() {
        this.createComponent();
        return expect(this.component.state.showQuotedText).toBe(false);
      });
      it("shouldn't render the quoted text control if there's no quoted text", function() {
        var toggles;
        this.message.body = "no quotes here!";
        this.createComponent();
        toggles = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.component, 'quoted-text-control');
        return expect(toggles.length).toBe(0);
      });
      describe('quoted text control toggle button', function() {
        beforeEach(function() {
          this.message.body = "Message\n<blockquote class=\"gmail_quote\">\n  Quoted message\n</blockquote>";
          this.createComponent();
          return this.toggle = ReactTestUtils.findRenderedDOMComponentWithClass(this.component, 'quoted-text-control');
        });
        return it('should be rendered', function() {
          return expect(this.toggle).toBeDefined();
        });
      });
      it("should be initialized to true if the message contains `Forwarded`...", function() {
        this.message.body = "Hi guys, take a look at this. Very relevant. -mg\n<br>\n<br>\n<div class=\"gmail_quote\">\n  ---- Forwarded Message -----\n  blablalba\n</div>";
        this.createComponent();
        return expect(this.component.state.showQuotedText).toBe(true);
      });
      it("should be initialized to false if the message is a response to a Forwarded message", function() {
        this.message.body = "Thanks mg, that indeed looks very relevant. Will bring it up\nwith the rest of the team.\n\nOn Sunday, March 4th at 12:32AM, Michael Grinich Wrote:\n<div class=\"gmail_quote\">\n  Hi guys, take a look at this. Very relevant. -mg\n  <br>\n  <br>\n  <div class=\"gmail_quote\">\n    ---- Forwarded Message -----\n    blablalba\n  </div>\n</div>";
        this.createComponent();
        return expect(this.component.state.showQuotedText).toBe(false);
      });
      return describe("when showQuotedText is true", function() {
        beforeEach(function() {
          this.message.body = "Message\n<blockquote class=\"gmail_quote\">\n  Quoted message\n</blockquote>";
          this.createComponent();
          return this.component.setState({
            showQuotedText: true
          });
        });
        describe('quoted text control toggle button', function() {
          beforeEach(function() {
            return this.toggle = ReactTestUtils.findRenderedDOMComponentWithClass(this.component, 'quoted-text-control');
          });
          return it('should be rendered', function() {
            return expect(this.toggle).toBeDefined();
          });
        });
        return it("should pass the value into the EmailFrame", function() {
          var frame;
          frame = ReactTestUtils.findRenderedComponentWithType(this.component, EmailFrameStub);
          return expect(frame.props.showQuotedText).toBe(true);
        });
      });
    });
  });

}).call(this);
