(function() {
  var ComponentRegistry, DraftStore, Message, MessageItemContainer, React, ReactTestUtils, StubComposer, StubMessageItem, Thread, proxyquire, testClientId, testDraft, testMessage, testThread, _ref,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require("react");

  proxyquire = require("proxyquire").noPreserveCache();

  ReactTestUtils = require('react-addons-test-utils');

  _ref = require('nylas-exports'), Thread = _ref.Thread, Message = _ref.Message, ComponentRegistry = _ref.ComponentRegistry, DraftStore = _ref.DraftStore;

  StubMessageItem = (function(_super) {
    __extends(StubMessageItem, _super);

    function StubMessageItem() {
      return StubMessageItem.__super__.constructor.apply(this, arguments);
    }

    StubMessageItem.displayName = "StubMessageItem";

    StubMessageItem.prototype.render = function() {
      return React.createElement("span", null);
    };

    return StubMessageItem;

  })(React.Component);

  StubComposer = (function(_super) {
    __extends(StubComposer, _super);

    function StubComposer() {
      return StubComposer.__super__.constructor.apply(this, arguments);
    }

    StubComposer.displayName = "StubComposer";

    StubComposer.prototype.render = function() {
      return React.createElement("span", null);
    };

    return StubComposer;

  })(React.Component);

  MessageItemContainer = proxyquire('../lib/message-item-container', {
    "./message-item": StubMessageItem
  });

  testThread = new Thread({
    id: "t1",
    accountId: TEST_ACCOUNT_ID
  });

  testClientId = "local-id";

  testMessage = new Message({
    id: "m1",
    draft: false,
    unread: true,
    accountId: TEST_ACCOUNT_ID
  });

  testDraft = new Message({
    id: "d1",
    draft: true,
    unread: true,
    accountId: TEST_ACCOUNT_ID
  });

  describe('MessageItemContainer', function() {
    var renderContainer;
    beforeEach(function() {
      this.isSendingDraft = false;
      spyOn(DraftStore, "isSendingDraft").andCallFake((function(_this) {
        return function() {
          return _this.isSendingDraft;
        };
      })(this));
      return ComponentRegistry.register(StubComposer, {
        role: 'Composer'
      });
    });
    afterEach(function() {
      return ComponentRegistry.register(StubComposer, {
        role: 'Composer'
      });
    });
    renderContainer = function(message) {
      return ReactTestUtils.renderIntoDocument(React.createElement(MessageItemContainer, {
        "thread": testThread,
        "message": message,
        "draftClientId": testClientId
      }));
    };
    it("shows composer if it's a draft", function() {
      var doc, items;
      this.isSendingDraft = false;
      doc = renderContainer(testDraft);
      items = ReactTestUtils.scryRenderedComponentsWithType(doc, StubComposer);
      return expect(items.length).toBe(1);
    });
    it("renders a message if it's a draft that is sending", function() {
      var doc, items;
      this.isSendingDraft = true;
      doc = renderContainer(testDraft);
      items = ReactTestUtils.scryRenderedComponentsWithType(doc, StubMessageItem);
      expect(items.length).toBe(1);
      return expect(items[0].props.pending).toBe(true);
    });
    return it("renders a message if it's not a draft", function() {
      var doc, items;
      this.isSendingDraft = false;
      doc = renderContainer(testMessage);
      items = ReactTestUtils.scryRenderedComponentsWithType(doc, StubMessageItem);
      return expect(items.length).toBe(1);
    });
  });

}).call(this);
