(function() {
  var Contact, File, FileDownloadStore, InjectedComponent, Message, MessageBodyProcessor, MessageItem, MessageItemBody, MessageTimestamp, QuotedHTMLTransformer, React, ReactDOM, ReactTestUtils, Thread, Utils, download, download_inline, file, file_cid_but_not_referenced, file_cid_but_not_referenced_or_image, file_inline, file_inline_downloading, file_inline_not_downloaded, file_not_downloaded, file_without_filename, proxyquire, user_1, user_2, user_3, user_4, user_5, _ref;

  proxyquire = require('proxyquire');

  React = require("react");

  ReactDOM = require("react-dom");

  ReactTestUtils = require('react-addons-test-utils');

  _ref = require("nylas-exports"), Contact = _ref.Contact, Message = _ref.Message, File = _ref.File, Thread = _ref.Thread, Utils = _ref.Utils, QuotedHTMLTransformer = _ref.QuotedHTMLTransformer, FileDownloadStore = _ref.FileDownloadStore, MessageBodyProcessor = _ref.MessageBodyProcessor;

  MessageItemBody = React.createClass({
    render: function() {
      return React.createElement("div", null);
    }
  });

  InjectedComponent = require('nylas-component-kit').InjectedComponent;

  file = new File({
    id: 'file_1_id',
    filename: 'a.png',
    contentType: 'image/png',
    size: 10
  });

  file_not_downloaded = new File({
    id: 'file_2_id',
    filename: 'b.png',
    contentType: 'image/png',
    size: 10
  });

  file_inline = new File({
    id: 'file_inline_id',
    filename: 'c.png',
    contentId: 'file_inline_id',
    contentType: 'image/png',
    size: 10
  });

  file_inline_downloading = new File({
    id: 'file_inline_downloading_id',
    filename: 'd.png',
    contentId: 'file_inline_downloading_id',
    contentType: 'image/png',
    size: 10
  });

  file_inline_not_downloaded = new File({
    id: 'file_inline_not_downloaded_id',
    filename: 'e.png',
    contentId: 'file_inline_not_downloaded_id',
    contentType: 'image/png',
    size: 10
  });

  file_cid_but_not_referenced = new File({
    id: 'file_cid_but_not_referenced',
    filename: 'f.png',
    contentId: 'file_cid_but_not_referenced',
    contentType: 'image/png',
    size: 10
  });

  file_cid_but_not_referenced_or_image = new File({
    id: 'file_cid_but_not_referenced_or_image',
    filename: 'ansible notes.txt',
    contentId: 'file_cid_but_not_referenced_or_image',
    contentType: 'text/plain',
    size: 300
  });

  file_without_filename = new File({
    id: 'file_without_filename',
    contentType: 'image/png',
    size: 10
  });

  download = {
    fileId: 'file_1_id'
  };

  download_inline = {
    fileId: 'file_inline_downloading_id'
  };

  user_1 = new Contact({
    name: "User One",
    email: "user1@nylas.com"
  });

  user_2 = new Contact({
    name: "User Two",
    email: "user2@nylas.com"
  });

  user_3 = new Contact({
    name: "User Three",
    email: "user3@nylas.com"
  });

  user_4 = new Contact({
    name: "User Four",
    email: "user4@nylas.com"
  });

  user_5 = new Contact({
    name: "User Five",
    email: "user5@nylas.com"
  });

  MessageItem = proxyquire('../lib/message-item', {
    './message-item-body': MessageItemBody
  });

  MessageTimestamp = require('../lib/message-timestamp');

  describe("MessageItem", function() {
    beforeEach(function() {
      spyOn(FileDownloadStore, 'pathForFile').andCallFake(function(f) {
        if (f.id === file.id) {
          return '/fake/path.png';
        }
        if (f.id === file_inline.id) {
          return '/fake/path-inline.png';
        }
        if (f.id === file_inline_downloading.id) {
          return '/fake/path-downloading.png';
        }
        return null;
      });
      spyOn(FileDownloadStore, 'downloadDataForFiles').andCallFake(function(ids) {
        return {
          'file_1_id': download,
          'file_inline_downloading_id': download_inline
        };
      });
      spyOn(MessageBodyProcessor, '_addToCache').andCallFake(function() {});
      this.message = new Message({
        id: "111",
        from: [user_1],
        to: [user_2],
        cc: [user_3, user_4],
        bcc: null,
        body: "Body One",
        date: new Date(1415814587),
        draft: false,
        files: [],
        unread: false,
        snippet: "snippet one...",
        subject: "Subject One",
        threadId: "thread_12345",
        accountId: TEST_ACCOUNT_ID
      });
      this.thread = new Thread({
        id: 'thread-111',
        accountId: TEST_ACCOUNT_ID
      });
      this.threadParticipants = [user_1, user_2, user_3, user_4];
      return this.createComponent = (function(_this) {
        return function(_arg) {
          var collapsed;
          collapsed = (_arg != null ? _arg : {}).collapsed;
          if (collapsed == null) {
            collapsed = false;
          }
          return _this.component = ReactTestUtils.renderIntoDocument(React.createElement(MessageItem, {
            "key": _this.message.id,
            "message": _this.message,
            "thread": _this.thread,
            "collapsed": collapsed
          }));
        };
      })(this);
    });
    describe("when displaying detailed headers", function() {
      beforeEach(function() {
        this.createComponent({
          collapsed: false
        });
        return this.component.setState({
          detailedHeaders: true
        });
      });
      it("correctly sets the participant states", function() {
        var participants;
        participants = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.component, "expanded-participants");
        expect(participants.length).toBe(2);
        return expect(function() {
          return ReactTestUtils.findRenderedDOMComponentWithClass(this.component, "collapsed-participants");
        }).toThrow();
      });
      return it("correctly sets the timestamp", function() {
        var ts;
        ts = ReactTestUtils.findRenderedComponentWithType(this.component, MessageTimestamp);
        return expect(ts.props.isDetailed).toBe(true);
      });
    });
    describe("when not collapsed", function() {
      beforeEach(function() {
        return this.createComponent({
          collapsed: false
        });
      });
      it("should render the MessageItemBody", function() {
        var frame;
        frame = ReactTestUtils.findRenderedComponentWithType(this.component, MessageItemBody);
        return expect(frame).toBeDefined();
      });
      return it("should not have the `collapsed` class", function() {
        return expect(ReactDOM.findDOMNode(this.component).className.indexOf('collapsed') >= 0).toBe(false);
      });
    });
    return describe("when the message contains attachments", function() {
      beforeEach(function() {
        this.message.files = [file, file_not_downloaded, file_cid_but_not_referenced, file_cid_but_not_referenced_or_image, file_inline, file_inline_downloading, file_inline_not_downloaded, file_without_filename];
        this.message.body = "<img alt=\"A\" src=\"cid:" + file_inline.contentId + "\"/>\n<img alt=\"B\" src=\"cid:" + file_inline_downloading.contentId + "\"/>\n<img alt=\"C\" src=\"cid:" + file_inline_not_downloaded.contentId + "\"/>\n<img src=\"cid:missing-attachment\"/>";
        return this.createComponent();
      });
      it("should include the attachments area", function() {
        var attachments;
        attachments = ReactTestUtils.findRenderedDOMComponentWithClass(this.component, 'attachments-area');
        return expect(attachments).toBeDefined();
      });
      it("should render the registered an injected component for each attachment", function() {
        var attachments;
        attachments = ReactTestUtils.scryRenderedComponentsWithTypeAndProps(this.component, InjectedComponent, {
          matching: {
            role: 'Attachment'
          }
        });
        return expect(attachments[0].props.exposedProps.file).toBe(file);
      });
      it("should list attachments that are not mentioned in the body via cid", function() {
        var attachments;
        attachments = ReactTestUtils.scryRenderedComponentsWithTypeAndProps(this.component, InjectedComponent, {
          matching: {
            role: 'Attachment'
          }
        });
        expect(attachments.length).toEqual(5);
        expect(attachments[0].props.exposedProps.file).toBe(file);
        expect(attachments[1].props.exposedProps.file).toBe(file_not_downloaded);
        expect(attachments[2].props.exposedProps.file).toBe(file_cid_but_not_referenced);
        return expect(attachments[3].props.exposedProps.file).toBe(file_cid_but_not_referenced_or_image);
      });
      it("should provide file download state to each InjectedComponent", function() {
        var attachments;
        attachments = ReactTestUtils.scryRenderedComponentsWithTypeAndProps(this.component, InjectedComponent, {
          matching: {
            role: 'Attachment'
          }
        });
        expect(attachments[0].props.exposedProps.download).toBe(download);
        return expect(attachments[1].props.exposedProps.download).toBe(void 0);
      });
      return it("should still list attachments when the message has no body", function() {
        var attachments;
        this.message.body = "";
        this.createComponent();
        attachments = ReactTestUtils.scryRenderedComponentsWithTypeAndProps(this.component, InjectedComponent, {
          matching: {
            role: 'Attachment'
          }
        });
        return expect(attachments.length).toEqual(8);
      });
    });
  });

}).call(this);
