(function() {
  var Account, AccountStore, Actions, ComponentRegistry, Contact, DraftStore, Message, MessageItemContainer, MessageList, MessageParticipants, MessageStore, NylasTestUtils, React, ReactDOM, ReactTestUtils, Thread, draftMessages, m1, m2, m3, m4, m5, moment, proxyquire, testMessages, test_thread, user_1, user_2, user_3, user_4, user_5, _, _ref;

  _ = require("underscore");

  moment = require("moment");

  proxyquire = require("proxyquire").noPreserveCache();

  React = require("react");

  ReactDOM = require("react-dom");

  ReactTestUtils = require('react-addons-test-utils');

  _ref = require("nylas-exports"), Thread = _ref.Thread, Contact = _ref.Contact, Actions = _ref.Actions, Message = _ref.Message, Account = _ref.Account, DraftStore = _ref.DraftStore, MessageStore = _ref.MessageStore, AccountStore = _ref.AccountStore, NylasTestUtils = _ref.NylasTestUtils, ComponentRegistry = _ref.ComponentRegistry;

  MessageParticipants = require("../lib/message-participants");

  MessageItemContainer = require("../lib/message-item-container");

  MessageList = require('../lib/message-list');

  user_1 = new Contact({
    name: TEST_ACCOUNT_NAME,
    email: TEST_ACCOUNT_EMAIL
  });

  user_2 = new Contact({
    name: "User Two",
    email: "user2@nylas.com"
  });

  user_3 = new Contact({
    name: "User Three",
    email: "user3@nylas.com"
  });

  user_4 = new Contact({
    name: "User Four",
    email: "user4@nylas.com"
  });

  user_5 = new Contact({
    name: "User Five",
    email: "user5@nylas.com"
  });

  m1 = (new Message).fromJSON({
    "id": "111",
    "from": [user_1],
    "to": [user_2],
    "cc": [user_3, user_4],
    "bcc": null,
    "body": "Body One",
    "date": 1415814587,
    "draft": false,
    "files": [],
    "unread": false,
    "object": "message",
    "snippet": "snippet one...",
    "subject": "Subject One",
    "thread_id": "thread_12345",
    "account_id": TEST_ACCOUNT_ID
  });

  m2 = (new Message).fromJSON({
    "id": "222",
    "from": [user_2],
    "to": [user_1],
    "cc": [user_3, user_4],
    "bcc": null,
    "body": "Body Two",
    "date": 1415814587,
    "draft": false,
    "files": [],
    "unread": false,
    "object": "message",
    "snippet": "snippet Two...",
    "subject": "Subject Two",
    "thread_id": "thread_12345",
    "account_id": TEST_ACCOUNT_ID
  });

  m3 = (new Message).fromJSON({
    "id": "333",
    "from": [user_3],
    "to": [user_1],
    "cc": [user_2, user_4],
    "bcc": [],
    "body": "Body Three",
    "date": 1415814587,
    "draft": false,
    "files": [],
    "unread": false,
    "object": "message",
    "snippet": "snippet Three...",
    "subject": "Subject Three",
    "thread_id": "thread_12345",
    "account_id": TEST_ACCOUNT_ID
  });

  m4 = (new Message).fromJSON({
    "id": "444",
    "from": [user_4],
    "to": [user_1],
    "cc": [],
    "bcc": [user_5],
    "body": "Body Four",
    "date": 1415814587,
    "draft": false,
    "files": [],
    "unread": false,
    "object": "message",
    "snippet": "snippet Four...",
    "subject": "Subject Four",
    "thread_id": "thread_12345",
    "account_id": TEST_ACCOUNT_ID
  });

  m5 = (new Message).fromJSON({
    "id": "555",
    "from": [user_1],
    "to": [user_4],
    "cc": [],
    "bcc": [],
    "body": "Body Five",
    "date": 1415814587,
    "draft": false,
    "files": [],
    "unread": false,
    "object": "message",
    "snippet": "snippet Five...",
    "subject": "Subject Five",
    "thread_id": "thread_12345",
    "account_id": TEST_ACCOUNT_ID
  });

  testMessages = [m1, m2, m3, m4, m5];

  draftMessages = [
    (new Message).fromJSON({
      "id": "666",
      "from": [user_1],
      "to": [],
      "cc": [],
      "bcc": null,
      "body": "Body One",
      "date": 1415814587,
      "draft": true,
      "files": [],
      "unread": false,
      "object": "draft",
      "snippet": "draft snippet one...",
      "subject": "Draft One",
      "thread_id": "thread_12345",
      "account_id": TEST_ACCOUNT_ID
    })
  ];

  test_thread = (new Thread).fromJSON({
    "id": "12345",
    "id": "thread_12345",
    "subject": "Subject 12345",
    "account_id": TEST_ACCOUNT_ID
  });

  describe("MessageList", function() {
    beforeEach(function() {
      MessageStore._items = [];
      MessageStore._threadId = null;
      spyOn(MessageStore, "itemsLoading").andCallFake(function() {
        return false;
      });
      this.messageList = ReactTestUtils.renderIntoDocument(React.createElement(MessageList, null));
      return this.messageList_node = ReactDOM.findDOMNode(this.messageList);
    });
    it("renders into the document", function() {
      return expect(ReactTestUtils.isCompositeComponentWithType(this.messageList, MessageList)).toBe(true);
    });
    it("by default has zero children", function() {
      var items;
      items = ReactTestUtils.scryRenderedComponentsWithType(this.messageList, MessageItemContainer);
      return expect(items.length).toBe(0);
    });
    describe("Populated Message list", function() {
      beforeEach(function() {
        MessageStore._items = testMessages;
        MessageStore._expandItemsToDefault();
        MessageStore.trigger(MessageStore);
        this.messageList.setState({
          currentThread: test_thread
        });
        return NylasTestUtils.loadKeymap("keymaps/base");
      });
      it("renders all the correct number of messages", function() {
        var items;
        items = ReactTestUtils.scryRenderedComponentsWithType(this.messageList, MessageItemContainer);
        return expect(items.length).toBe(5);
      });
      it("renders the correct number of expanded messages", function() {
        var msgs;
        msgs = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.messageList, "collapsed message-item-wrap");
        return expect(msgs.length).toBe(4);
      });
      it("displays lists of participants on the page", function() {
        var items;
        items = ReactTestUtils.scryRenderedComponentsWithType(this.messageList, MessageParticipants);
        return expect(items.length).toBe(2);
      });
      return it("includes drafts as message item containers", function() {
        var items, msgs;
        msgs = this.messageList.state.messages;
        this.messageList.setState({
          messages: msgs.concat(draftMessages)
        });
        items = ReactTestUtils.scryRenderedComponentsWithType(this.messageList, MessageItemContainer);
        return expect(items.length).toBe(6);
      });
    });
    describe("reply type", function() {
      it("prompts for a reply when there's only one participant", function() {
        var cs;
        MessageStore._items = [m3, m5];
        MessageStore._thread = test_thread;
        MessageStore.trigger();
        expect(this.messageList._replyType()).toBe("reply");
        cs = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.messageList, "footer-reply-area");
        return expect(cs.length).toBe(1);
      });
      it("prompts for a reply-all when there's more than one participant and the default is reply-all", function() {
        var cs;
        spyOn(NylasEnv.config, "get").andReturn("reply-all");
        MessageStore._items = [m5, m3];
        MessageStore._thread = test_thread;
        MessageStore.trigger();
        expect(this.messageList._replyType()).toBe("reply-all");
        cs = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.messageList, "footer-reply-area");
        return expect(cs.length).toBe(1);
      });
      it("prompts for a reply-all when there's more than one participant and the default is reply", function() {
        var cs;
        spyOn(NylasEnv.config, "get").andReturn("reply");
        MessageStore._items = [m5, m3];
        MessageStore._thread = test_thread;
        MessageStore.trigger();
        expect(this.messageList._replyType()).toBe("reply");
        cs = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.messageList, "footer-reply-area");
        return expect(cs.length).toBe(1);
      });
      return it("hides the reply type if the last message is a draft", function() {
        var cs;
        MessageStore._items = [m5, m3, draftMessages[0]];
        MessageStore._thread = test_thread;
        MessageStore.trigger();
        cs = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.messageList, "footer-reply-area");
        return expect(cs.length).toBe(0);
      });
    });
    return describe("Message minification", function() {
      beforeEach(function() {
        this.messageList.MINIFY_THRESHOLD = 3;
        this.messageList.setState({
          minified: true
        });
        return this.messages = [
          {
            id: 'a'
          }, {
            id: 'b'
          }, {
            id: 'c'
          }, {
            id: 'd'
          }, {
            id: 'e'
          }, {
            id: 'f'
          }, {
            id: 'g'
          }
        ];
      });
      it("ignores the first message if it's collapsed", function() {
        var out;
        this.messageList.setState({
          messagesExpandedState: {
            a: false,
            b: false,
            c: false,
            d: false,
            e: false,
            f: false,
            g: "default"
          }
        });
        out = this.messageList._messagesWithMinification(this.messages);
        return expect(out).toEqual([
          {
            id: 'a'
          }, {
            type: "minifiedBundle",
            messages: [
              {
                id: 'b'
              }, {
                id: 'c'
              }, {
                id: 'd'
              }, {
                id: 'e'
              }
            ]
          }, {
            id: 'f'
          }, {
            id: 'g'
          }
        ]);
      });
      it("ignores the first message if it's expanded", function() {
        var out;
        this.messageList.setState({
          messagesExpandedState: {
            a: "default",
            b: false,
            c: false,
            d: false,
            e: false,
            f: false,
            g: "default"
          }
        });
        out = this.messageList._messagesWithMinification(this.messages);
        return expect(out).toEqual([
          {
            id: 'a'
          }, {
            type: "minifiedBundle",
            messages: [
              {
                id: 'b'
              }, {
                id: 'c'
              }, {
                id: 'd'
              }, {
                id: 'e'
              }
            ]
          }, {
            id: 'f'
          }, {
            id: 'g'
          }
        ]);
      });
      it("doesn't minify the last collapsed message", function() {
        var out;
        this.messageList.setState({
          messagesExpandedState: {
            a: false,
            b: false,
            c: false,
            d: false,
            e: false,
            f: "default",
            g: "default"
          }
        });
        out = this.messageList._messagesWithMinification(this.messages);
        return expect(out).toEqual([
          {
            id: 'a'
          }, {
            type: "minifiedBundle",
            messages: [
              {
                id: 'b'
              }, {
                id: 'c'
              }, {
                id: 'd'
              }
            ]
          }, {
            id: 'e'
          }, {
            id: 'f'
          }, {
            id: 'g'
          }
        ]);
      });
      it("allows explicitly expanded messages", function() {
        var out;
        this.messageList.setState({
          messagesExpandedState: {
            a: false,
            b: false,
            c: false,
            d: false,
            e: false,
            f: "explicit",
            g: "default"
          }
        });
        out = this.messageList._messagesWithMinification(this.messages);
        return expect(out).toEqual([
          {
            id: 'a'
          }, {
            type: "minifiedBundle",
            messages: [
              {
                id: 'b'
              }, {
                id: 'c'
              }, {
                id: 'd'
              }, {
                id: 'e'
              }
            ]
          }, {
            id: 'f'
          }, {
            id: 'g'
          }
        ]);
      });
      it("doesn't minify if the threshold isn't reached", function() {
        var out;
        this.messageList.setState({
          messagesExpandedState: {
            a: false,
            b: "default",
            c: false,
            d: "default",
            e: false,
            f: "default",
            g: "default"
          }
        });
        out = this.messageList._messagesWithMinification(this.messages);
        return expect(out).toEqual([
          {
            id: 'a'
          }, {
            id: 'b'
          }, {
            id: 'c'
          }, {
            id: 'd'
          }, {
            id: 'e'
          }, {
            id: 'f'
          }, {
            id: 'g'
          }
        ]);
      });
      it("doesn't minify if the threshold isn't reached due to the rule about not minifying the last collapsed messages", function() {
        var out;
        this.messageList.setState({
          messagesExpandedState: {
            a: false,
            b: false,
            c: false,
            d: false,
            e: "default",
            f: "default",
            g: "default"
          }
        });
        out = this.messageList._messagesWithMinification(this.messages);
        return expect(out).toEqual([
          {
            id: 'a'
          }, {
            id: 'b'
          }, {
            id: 'c'
          }, {
            id: 'd'
          }, {
            id: 'e'
          }, {
            id: 'f'
          }, {
            id: 'g'
          }
        ]);
      });
      it("minifies at the threshold if the message is explicitly expanded", function() {
        var out;
        this.messageList.setState({
          messagesExpandedState: {
            a: false,
            b: false,
            c: false,
            d: false,
            e: "explicit",
            f: "default",
            g: "default"
          }
        });
        out = this.messageList._messagesWithMinification(this.messages);
        return expect(out).toEqual([
          {
            id: 'a'
          }, {
            type: "minifiedBundle",
            messages: [
              {
                id: 'b'
              }, {
                id: 'c'
              }, {
                id: 'd'
              }
            ]
          }, {
            id: 'e'
          }, {
            id: 'f'
          }, {
            id: 'g'
          }
        ]);
      });
      it("can have multiple minification blocks", function() {
        var messages, out;
        messages = [
          {
            id: 'a'
          }, {
            id: 'b'
          }, {
            id: 'c'
          }, {
            id: 'd'
          }, {
            id: 'e'
          }, {
            id: 'f'
          }, {
            id: 'g'
          }, {
            id: 'h'
          }, {
            id: 'i'
          }, {
            id: 'j'
          }, {
            id: 'k'
          }, {
            id: 'l'
          }
        ];
        this.messageList.setState({
          messagesExpandedState: {
            a: false,
            b: false,
            c: false,
            d: false,
            e: false,
            f: "default",
            g: false,
            h: false,
            i: false,
            j: false,
            k: false,
            l: "default"
          }
        });
        out = this.messageList._messagesWithMinification(messages);
        return expect(out).toEqual([
          {
            id: 'a'
          }, {
            type: "minifiedBundle",
            messages: [
              {
                id: 'b'
              }, {
                id: 'c'
              }, {
                id: 'd'
              }
            ]
          }, {
            id: 'e'
          }, {
            id: 'f'
          }, {
            type: "minifiedBundle",
            messages: [
              {
                id: 'g'
              }, {
                id: 'h'
              }, {
                id: 'i'
              }, {
                id: 'j'
              }
            ]
          }, {
            id: 'k'
          }, {
            id: 'l'
          }
        ]);
      });
      return it("can have multiple minification blocks next to explicitly expanded messages", function() {
        var messages, out;
        messages = [
          {
            id: 'a'
          }, {
            id: 'b'
          }, {
            id: 'c'
          }, {
            id: 'd'
          }, {
            id: 'e'
          }, {
            id: 'f'
          }, {
            id: 'g'
          }, {
            id: 'h'
          }, {
            id: 'i'
          }, {
            id: 'j'
          }, {
            id: 'k'
          }, {
            id: 'l'
          }
        ];
        this.messageList.setState({
          messagesExpandedState: {
            a: false,
            b: false,
            c: false,
            d: false,
            e: "explicit",
            f: "default",
            g: false,
            h: false,
            i: false,
            j: false,
            k: "explicit",
            l: "default"
          }
        });
        out = this.messageList._messagesWithMinification(messages);
        return expect(out).toEqual([
          {
            id: 'a'
          }, {
            type: "minifiedBundle",
            messages: [
              {
                id: 'b'
              }, {
                id: 'c'
              }, {
                id: 'd'
              }
            ]
          }, {
            id: 'e'
          }, {
            id: 'f'
          }, {
            type: "minifiedBundle",
            messages: [
              {
                id: 'g'
              }, {
                id: 'h'
              }, {
                id: 'i'
              }, {
                id: 'j'
              }
            ]
          }, {
            id: 'k'
          }, {
            id: 'l'
          }
        ]);
      });
    });
  });

}).call(this);
