(function() {
  var Contact, DOMUtils, Message, MessageParticipants, React, ReactDOM, ReactTestUtils, big_test_message, i, many_thread_users, many_users, test_message, user_1, user_2, user_3, user_4, user_5, _, _ref;

  _ = require('underscore');

  React = require("react");

  ReactDOM = require("react-dom");

  ReactTestUtils = require('react-addons-test-utils');

  _ref = require("nylas-exports"), Contact = _ref.Contact, Message = _ref.Message, DOMUtils = _ref.DOMUtils;

  MessageParticipants = require("../lib/message-participants");

  user_1 = {
    name: "User One",
    email: "user1@nylas.com"
  };

  user_2 = {
    name: "User Two",
    email: "user2@nylas.com"
  };

  user_3 = {
    name: "User Three",
    email: "user3@nylas.com"
  };

  user_4 = {
    name: "User Four",
    email: "user4@nylas.com"
  };

  user_5 = {
    name: "User Five",
    email: "user5@nylas.com"
  };

  many_users = (function() {
    var _i, _results;
    _results = [];
    for (i = _i = 0; _i <= 100; i = ++_i) {
      _results.push(new Contact({
        name: "User " + i,
        email: "" + i + "@app.com"
      }));
    }
    return _results;
  })();

  test_message = (new Message).fromJSON({
    "id": "111",
    "from": [user_1],
    "to": [user_2],
    "cc": [user_3, user_4],
    "bcc": [user_5]
  });

  big_test_message = (new Message).fromJSON({
    "id": "222",
    "from": [user_1],
    "to": many_users
  });

  many_thread_users = [user_1].concat(many_users);

  describe("MessageParticipants", function() {
    describe("when collapsed", function() {
      var makeParticipants;
      makeParticipants = function(props) {
        return ReactTestUtils.renderIntoDocument(React.createElement(MessageParticipants, React.__spread({}, props)));
      };
      it("renders into the document", function() {
        var participants;
        participants = makeParticipants({
          to: test_message.to,
          cc: test_message.cc
        }, {
          from: test_message.from,
          message_participants: test_message.participants()
        });
        return expect(participants).toBeDefined();
      });
      it("uses short names", function() {
        var actualOut, to;
        actualOut = makeParticipants({
          to: test_message.to
        });
        to = ReactTestUtils.findRenderedDOMComponentWithClass(actualOut, "to-contact");
        return expect(ReactDOM.findDOMNode(to).innerHTML).toBe("User");
      });
      it("doesn't render any To nodes if To array is empty", function() {
        var actualOut, findToField;
        actualOut = makeParticipants({
          to: []
        });
        findToField = function() {
          return ReactTestUtils.findRenderedDOMComponentWithClass(actualOut, "to-contact");
        };
        return expect(findToField).toThrow();
      });
      it("doesn't render any Cc nodes if Cc array is empty", function() {
        var actualOut, findCcField;
        actualOut = makeParticipants({
          cc: []
        });
        findCcField = function() {
          return ReactTestUtils.findRenderedDOMComponentWithClass(actualOut, "cc-contact");
        };
        return expect(findCcField).toThrow();
      });
      return it("doesn't render any Bcc nodes if Bcc array is empty", function() {
        var actualOut, findBccField;
        actualOut = makeParticipants({
          bcc: []
        });
        findBccField = function() {
          return ReactTestUtils.findRenderedDOMComponentWithClass(actualOut, "bcc-contact");
        };
        return expect(findBccField).toThrow();
      });
    });
    return describe("when expanded", function() {
      beforeEach(function() {
        return this.participants = ReactTestUtils.renderIntoDocument(React.createElement(MessageParticipants, {
          "to": test_message.to,
          "cc": test_message.cc,
          "from": test_message.from,
          "isDetailed": true,
          "message_participants": test_message.participants()
        }));
      });
      it("renders into the document", function() {
        var participants;
        participants = ReactTestUtils.findRenderedDOMComponentWithClass(this.participants, "expanded-participants");
        return expect(participants).toBeDefined();
      });
      return it("uses full names", function() {
        var to;
        to = ReactTestUtils.findRenderedDOMComponentWithClass(this.participants, "to-contact");
        return expect(ReactDOM.findDOMNode(to).innerText.trim()).toEqual("User Two<user2@nylas.com>");
      });
    });
  });

}).call(this);
