(function() {
  var MessageTimestamp, React, ReactDOM, ReactTestUtils, moment, msgTime;

  moment = require('moment');

  React = require("react");

  ReactDOM = require("react-dom");

  ReactTestUtils = require('react-addons-test-utils');

  MessageTimestamp = require('../lib/message-timestamp');

  msgTime = function() {
    return moment([2010, 1, 14, 15, 25, 50, 125]);
  };

  describe("MessageTimestamp", function() {
    beforeEach(function() {
      return this.item = ReactTestUtils.renderIntoDocument(React.createElement(MessageTimestamp, {
        "date": msgTime()
      }));
    });
    it("still processes one day, even if it crosses a month divider", function() {
      var feb28, mar01;
      feb28 = moment([2015, 1, 28]);
      mar01 = moment([2015, 2, 1]);
      return expect(mar01.diff(feb28, 'days')).toBe(1);
    });
    it("displays the full time when in detailed timestamp mode", function() {
      return expect(this.item._formattedDate(msgTime(), null, true)).toBe("February 14, 2010 at 3:25 PM");
    });
    it("displays the time from messages shown today", function() {
      var now;
      now = msgTime().add(2, 'hours');
      return expect(this.item._formattedDate(msgTime(), now)).toBe("3:25 PM");
    });
    it("displays the time from messages yesterday with the relative time if it's less than 36 hours ago", function() {
      var now;
      now = msgTime().add(21, 'hours');
      expect(this.item._formattedDate(msgTime(), now)).toBe("3:25 PM (21 hours ago)");
      now = msgTime().add(30, 'hours');
      return expect(this.item._formattedDate(msgTime(), now)).toBe("3:25 PM (a day ago)");
    });
    it("displays month, day for messages less than a year ago, but more than 24 hours ago", function() {
      var now;
      now = msgTime().add(2, 'months');
      return expect(this.item._formattedDate(msgTime(), now)).toBe("Feb 14");
    });
    return it("displays month, day, and year for messages over a year ago", function() {
      var now;
      now = msgTime().add(2, 'years');
      return expect(this.item._formattedDate(msgTime(), now)).toBe("Feb 14, 2010");
    });
  });

}).call(this);
