(function() {
  var Actions, GithubStore, KeyCommandsRegion, React, RetinaImg, ViewOnGithubButton, shell, _ref, _ref1,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  shell = require('electron').shell;

  GithubStore = require('./github-store')["default"];

  _ref = require('nylas-exports'), React = _ref.React, Actions = _ref.Actions;

  _ref1 = require('nylas-component-kit'), RetinaImg = _ref1.RetinaImg, KeyCommandsRegion = _ref1.KeyCommandsRegion;


  /*
  The `ViewOnGithubButton` displays a button whenever there's a relevant
  Github asset to link to.
  
  When creating this React component the first consideration was when &
  where we'd be rendered. The next consideration was what data we need to
  display.
  
  Unlike a traditional React application, N1 components have very few
  guarantees on who will render them and where they will be rendered. In our
  `lib/main.cjsx` file we registered this component with our
  {ComponentRegistry} for the `"ThreadActionsToolbarButton"` role. That means that
  whenever the "ThreadActionsToolbarButton" region gets rendered, we'll render
  everything registered with that area. Other buttons, such as "Archive" and
  the "Change Label" button are reigstered with that role, so we should
  expect ourselves to showup alongside them.
  
  The only data we need is a single relevant to Github. If we have one,
  we'll open it up in a browser. If we don't have one, we'll hide the
  component.
  
  Getting that url takes a bit of message parsing. We need to retrieve a
  message body then implement some kind of regex to find and parse out that
  link.
  
  We could have put all of that logic in this React Component, but that's
  not what React components should be doing. In N1 a component's only job is
  to display known data and be the first responders to user interaction.
  
  We instead create a {GithubStore} to handle the fetching and preparation
  of the data. See that file's documentation for more on how that works.
  
  As far as this component is concerned, there will be an entity called
  `GitHubStore` that will expose the correct `link`. That store will then
  notify us when the `link` changes so we can update our state.
  
  Once we know our `link` our `render` method can simply be a description of
  how we want to display that link. In this case we're going to make a
  simple button with a GitHub logo in it.
  
  We'll also display nothing if there is no link.
   */

  ViewOnGithubButton = (function(_super) {
    __extends(ViewOnGithubButton, _super);

    ViewOnGithubButton.displayName = "ViewOnGithubButton";

    ViewOnGithubButton.containerRequired = false;

    ViewOnGithubButton.propTypes = {
      items: React.PropTypes.array
    };

    function ViewOnGithubButton(props) {
      this.props = props;
      this._openLink = __bind(this._openLink, this);
      this._onStoreChanged = __bind(this._onStoreChanged, this);
      this.state = this._getStateFromStores();
    }

    ViewOnGithubButton.prototype.componentDidMount = function() {
      return this._unlisten = GithubStore.listen(this._onStoreChanged);
    };

    ViewOnGithubButton.prototype.componentWillUnmount = function() {
      return typeof this._unlisten === "function" ? this._unlisten() : void 0;
    };

    ViewOnGithubButton.prototype._keymapHandlers = function() {
      return {
        'github:open': this._openLink
      };
    };

    ViewOnGithubButton.prototype.render = function() {
      if (this.props.items.length !== 1) {
        return null;
      }
      if (!this.state.link) {
        return null;
      }
      return React.createElement(KeyCommandsRegion, {
        "globalHandlers": this._keymapHandlers()
      }, React.createElement("button", {
        "className": "btn btn-toolbar btn-view-on-github",
        "onClick": this._openLink,
        "title": "Visit Thread on GitHub"
      }, React.createElement(RetinaImg, {
        "mode": RetinaImg.Mode.ContentIsMask,
        "url": "nylas://message-view-on-github/assets/github@2x.png"
      })));
    };

    ViewOnGithubButton.prototype._onStoreChanged = function() {
      return this.setState(this._getStateFromStores());
    };

    ViewOnGithubButton.prototype._getStateFromStores = function() {
      return {
        link: GithubStore.link()
      };
    };

    ViewOnGithubButton.prototype._openLink = function() {
      Actions.recordUserEvent("Github Thread Opened", {
        pageUrl: this.state.link
      });
      if (this.state.link) {
        return shell.openExternal(this.state.link);
      }
    };

    return ViewOnGithubButton;

  })(React.Component);

  module.exports = ViewOnGithubButton;

}).call(this);
