(function() {
  var Actions, ipcRenderer, remote, _ref;

  Actions = require('nylas-exports').Actions;

  _ref = require('electron'), ipcRenderer = _ref.ipcRenderer, remote = _ref.remote;

  module.exports = {
    activate: function(state) {
      var configVersion, currentVersion, updater;
      this.state = state;
      updater = remote.getGlobal('application').autoUpdateManager;
      this._unlisten = Actions.notificationActionTaken.listen(this._onNotificationActionTaken, this);
      configVersion = NylasEnv.config.get("lastVersion");
      currentVersion = NylasEnv.getVersion();
      if (configVersion && configVersion !== currentVersion) {
        NylasEnv.config.set("lastVersion", currentVersion);
        this.displayThanksNotification();
      }
      if (updater.getState() === 'update-available') {
        this.displayNotification(updater.releaseVersion);
      }
      return NylasEnv.onUpdateAvailable((function(_this) {
        return function(_arg) {
          var releaseNotes, releaseVersion, _ref1;
          _ref1 = _arg != null ? _arg : {}, releaseVersion = _ref1.releaseVersion, releaseNotes = _ref1.releaseNotes;
          return _this.displayNotification(releaseVersion);
        };
      })(this));
    },
    displayThanksNotification: function() {
      return Actions.postNotification({
        type: 'info',
        tag: 'app-update',
        sticky: true,
        message: "You're running the latest version of N1 - view the changelog to see what's new.",
        icon: 'fa-magic',
        actions: [
          {
            dismisses: true,
            label: 'Thanks',
            id: 'release-bar:no-op'
          }, {
            "default": true,
            dismisses: true,
            label: 'See What\'s New',
            id: 'release-bar:view-changelog'
          }
        ]
      });
    },
    displayNotification: function(version) {
      version = version ? "(" + version + ")" : '';
      return Actions.postNotification({
        type: 'info',
        tag: 'app-update',
        sticky: true,
        message: "An update to N1 is available " + version + " - click to update now!",
        icon: 'fa-flag',
        actions: [
          {
            label: 'See What\'s New',
            id: 'release-bar:view-changelog'
          }, {
            label: 'Install Now',
            dismisses: true,
            "default": true,
            id: 'release-bar:install-update'
          }
        ]
      });
    },
    deactivate: function() {
      return this._unlisten();
    },
    _onNotificationActionTaken: function(_arg) {
      var action, notification;
      notification = _arg.notification, action = _arg.action;
      if (action.id === 'release-bar:install-update') {
        ipcRenderer.send('command', 'application:install-update');
        true;
      }
      if (action.id === 'release-bar:view-changelog') {
        require('electron').shell.openExternal('https://github.com/nylas/N1/blob/master/CHANGELOG.md');
        return false;
      }
    }
  };

}).call(this);
