(function() {
  var Actions, PackageMain, Reflux, ipcSendArgs, proxyquire, stubUpdaterReleaseVersion, stubUpdaterState, _;

  _ = require('underscore');

  proxyquire = require('proxyquire');

  Reflux = require('reflux');

  Actions = require('nylas-exports').Actions;

  stubUpdaterState = null;

  stubUpdaterReleaseVersion = null;

  ipcSendArgs = null;

  PackageMain = proxyquire("../lib/main", {
    "electron": {
      "ipcRenderer": {
        send: function() {
          return ipcSendArgs = arguments;
        }
      },
      "remote": {
        getGlobal: function(global) {
          return {
            autoUpdateManager: {
              releaseVersion: stubUpdaterReleaseVersion,
              getState: function() {
                return stubUpdaterState;
              }
            }
          };
        }
      }
    }
  });

  describe("NotificationUpdateAvailable", function() {
    beforeEach(function() {
      stubUpdaterState = 'idle';
      stubUpdaterReleaseVersion = void 0;
      ipcSendArgs = null;
      return this["package"] = PackageMain;
    });
    afterEach(function() {
      return this["package"].deactivate();
    });
    describe("activate", function() {
      it("should display a notification immediately if one is available", function() {
        spyOn(this["package"], 'displayNotification');
        stubUpdaterState = 'update-available';
        this["package"].activate();
        return expect(this["package"].displayNotification).toHaveBeenCalled();
      });
      it("should not display a notification if no update is avialable", function() {
        spyOn(this["package"], 'displayNotification');
        stubUpdaterState = 'no-update-available';
        this["package"].activate();
        return expect(this["package"].displayNotification).not.toHaveBeenCalled();
      });
      return it("should listen for `window:update-available`", function() {
        spyOn(NylasEnv, 'onUpdateAvailable').andCallThrough();
        this["package"].activate();
        return expect(NylasEnv.onUpdateAvailable).toHaveBeenCalled();
      });
    });
    return describe("displayNotification", function() {
      beforeEach(function() {
        return this["package"].activate();
      });
      it("should fire a postNotification Action", function() {
        spyOn(Actions, 'postNotification');
        this["package"].displayNotification();
        return expect(Actions.postNotification).toHaveBeenCalled();
      });
      it("should include the version if one is provided", function() {
        var notifOptions, version;
        spyOn(Actions, 'postNotification');
        version = '0.515.0-123123';
        this["package"].displayNotification(version);
        expect(Actions.postNotification).toHaveBeenCalled();
        notifOptions = Actions.postNotification.mostRecentCall.args[0];
        return expect(notifOptions.message.indexOf(version) > 0).toBe(true);
      });
      return describe("when the action is taken", function() {
        return it("should fire the `application:install-update` IPC event", function() {
          Actions.notificationActionTaken({
            notification: {},
            action: {
              id: 'release-bar:install-update'
            }
          });
          return expect(Array.prototype.slice.call(ipcSendArgs)).toEqual(['command', 'application:install-update']);
        });
      });
    });
  });

}).call(this);
