(function() {
  var Actions, DISPLAY_TIME, Notification, NotificationStore, NylasStore, VERBOSE, uuid_count, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  Actions = require('nylas-exports').Actions;

  NylasStore = require('nylas-store');

  VERBOSE = false;

  DISPLAY_TIME = 3000;

  uuid_count = 0;

  Notification = (function() {
    function Notification(_arg) {
      var action, _i, _len, _ref, _ref1, _ref2;
      _ref = _arg != null ? _arg : {}, this.message = _ref.message, this.type = _ref.type, this.tag = _ref.tag, this.sticky = _ref.sticky, this.actions = _ref.actions, this.icon = _ref.icon;
      if (!(this instanceof Notification)) {
        throw new Error("No `new` keyword when constructing Notification");
      }
      if ((_ref1 = this.type) !== 'info' && _ref1 !== 'error' && _ref1 !== 'success' && _ref1 !== 'developer') {
        throw new Error("Type must be `info`, `developer`, `error`, or `success`");
      }
      if (!this.message) {
        throw new Error("Message must be provided for notification");
      }
      if (this.actions) {
        _ref2 = this.actions;
        for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
          action = _ref2[_i];
          if (!action['label']) {
            throw new Error("Actions must have an `label`");
          }
          if (!action['id']) {
            throw new Error("Actions must have an `id`");
          }
        }
      }
      if (this.tag == null) {
        this.tag = uuid_count++;
      }
      this.creation = Date.now();
      if (this.sticky == null) {
        this.sticky = false;
      }
      if (!this.sticky) {
        this.expiry = this.creation + DISPLAY_TIME;
      }
      if (VERBOSE) {
        console.log("Created new notif with " + this.tag + ": " + this.message);
      }
      this;
    }

    Notification.prototype.valid = function() {
      return this.sticky || this.expiry > Date.now();
    };

    Notification.prototype.toString = function() {
      return "Notification." + this.constructor.name + "(" + this.tag + ")";
    };

    return Notification;

  })();

  NotificationStore = (function(_super) {
    __extends(NotificationStore, _super);

    function NotificationStore() {
      this._inWindowContext = __bind(this._inWindowContext, this);
      this._removeNotification = __bind(this._removeNotification, this);
      this._postNotification = __bind(this._postNotification, this);
      this._flush = __bind(this._flush, this);
      this.stickyNotifications = __bind(this.stickyNotifications, this);
      this.notifications = __bind(this.notifications, this);
      this._flush();
      this.listenTo(Actions.notificationActionTaken, (function(_this) {
        return function(_arg) {
          var action, notification;
          notification = _arg.notification, action = _arg.action;
          if (action.dismisses) {
            return _this._removeNotification(notification);
          }
        };
      })(this));
      this.listenTo(Actions.postNotification, (function(_this) {
        return function(data) {
          return _this._postNotification(new Notification(data));
        };
      })(this));
      this.listenTo(Actions.dismissNotificationsMatching, (function(_this) {
        return function(criteria) {
          var notif, tag, _ref;
          _ref = _this._notifications;
          for (tag in _ref) {
            notif = _ref[tag];
            if (_.isMatch(notif, criteria)) {
              delete _this._notifications[tag];
            }
          }
          return _this.trigger();
        };
      })(this));
    }

    NotificationStore.prototype.notifications = function() {
      var sorted;
      if (VERBOSE) {
        console.log(JSON.stringify(this._notifications));
      }
      sorted = _.sortBy(_.values(this._notifications), function(n) {
        return -1 * (n.creation + n.tag);
      });
      return _.reject(sorted, function(n) {
        return n.sticky;
      });
    };

    NotificationStore.prototype.stickyNotifications = function() {
      var sorted;
      if (VERBOSE) {
        console.log(JSON.stringify(this._notifications));
      }
      sorted = _.sortBy(_.values(this._notifications), function(n) {
        return -1 * (n.creation + n.tag);
      });
      return _.filter(sorted, function(n) {
        return n.sticky;
      });
    };

    NotificationStore.prototype.Notification = Notification;

    NotificationStore.prototype._flush = function() {
      return this._notifications = {};
    };

    NotificationStore.prototype._postNotification = function(notification) {
      var timeoutId, timeoutVal;
      if (VERBOSE) {
        console.log("Queue Notification." + notification);
      }
      this._notifications[notification.tag] = notification;
      if (notification.expiry != null) {
        timeoutVal = Math.max(0, notification.expiry - Date.now());
        timeoutId = setTimeout((function(_this) {
          return function() {
            return _this._removeNotification(notification);
          };
        })(this), timeoutVal);
        notification.timeoutId = timeoutId;
      }
      return this.trigger();
    };

    NotificationStore.prototype._removeNotification = function(notification) {
      if (VERBOSE) {
        console.log("Removed " + notification);
      }
      if (notification.timeoutId) {
        clearTimeout(notification.timeoutId);
      }
      delete this._notifications[notification.tag];
      return this.trigger();
    };

    NotificationStore.prototype._inWindowContext = function(context) {
      if (context == null) {
        context = {};
      }
      return true;
    };

    return NotificationStore;

  })(NylasStore);

  module.exports = new NotificationStore();

}).call(this);
