(function() {
  var AccountStore, Actions, ActivitySidebar, InitialSyncActivity, NotificationStore, NylasSyncStatusStore, React, ReactCSSTransitionGroup, ReactDOM, StreamingSyncActivity, TaskQueue, TaskQueueStatusStore, classNames, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require('react');

  ReactDOM = require('react-dom');

  ReactCSSTransitionGroup = require('react-addons-css-transition-group');

  _ = require('underscore');

  classNames = require('classnames');

  NotificationStore = require('../notifications-store');

  StreamingSyncActivity = require('./streaming-sync-activity');

  InitialSyncActivity = require('./initial-sync-activity');

  _ref = require('nylas-exports'), Actions = _ref.Actions, TaskQueue = _ref.TaskQueue, AccountStore = _ref.AccountStore, NylasSyncStatusStore = _ref.NylasSyncStatusStore, TaskQueueStatusStore = _ref.TaskQueueStatusStore;

  ActivitySidebar = (function(_super) {
    __extends(ActivitySidebar, _super);

    ActivitySidebar.displayName = 'ActivitySidebar';

    ActivitySidebar.containerRequired = false;

    ActivitySidebar.containerStyles = {
      minWidth: 165,
      maxWidth: 400
    };

    function ActivitySidebar(props) {
      this.props = props;
      this._getStateFromStores = __bind(this._getStateFromStores, this);
      this._onDataChanged = __bind(this._onDataChanged, this);
      this._renderNotificationActivityItems = __bind(this._renderNotificationActivityItems, this);
      this._renderTaskActivityItems = __bind(this._renderTaskActivityItems, this);
      this.render = __bind(this.render, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.state = this._getStateFromStores();
    }

    ActivitySidebar.prototype.componentDidMount = function() {
      this._unlisteners = [];
      this._unlisteners.push(TaskQueueStatusStore.listen(this._onDataChanged));
      this._unlisteners.push(NotificationStore.listen(this._onDataChanged));
      return this._unlisteners.push(NylasSyncStatusStore.listen(this._onDataChanged));
    };

    ActivitySidebar.prototype.componentWillUnmount = function() {
      var unlisten, _i, _len, _ref1, _results;
      _ref1 = this._unlisteners;
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        unlisten = _ref1[_i];
        _results.push(unlisten());
      }
      return _results;
    };

    ActivitySidebar.prototype.render = function() {
      var inside, items, names, wrapperClass;
      items = [this._renderNotificationActivityItems(), this._renderTaskActivityItems()];
      if (this.state.isInitialSyncComplete) {
        items.push(React.createElement(StreamingSyncActivity, {
          "key": "streaming-sync"
        }));
      } else {
        items.push(React.createElement(InitialSyncActivity, {
          "key": "initial-sync"
        }));
      }
      names = classNames({
        "sidebar-activity": true,
        "sidebar-activity-error": typeof error !== "undefined" && error !== null
      });
      wrapperClass = "sidebar-activity-transition-wrapper ";
      if (items.length === 0) {
        wrapperClass += "sidebar-activity-empty";
      } else {
        inside = React.createElement(ReactCSSTransitionGroup, {
          "className": names,
          "transitionLeaveTimeout": 625.,
          "transitionEnterTimeout": 125.,
          "transitionName": "activity-opacity"
        }, items);
      }
      return React.createElement(ReactCSSTransitionGroup, {
        "className": wrapperClass,
        "transitionLeaveTimeout": 625.,
        "transitionEnterTimeout": 125.,
        "transitionName": "activity-opacity"
      }, inside);
    };

    ActivitySidebar.prototype._renderTaskActivityItems = function() {
      var summary;
      summary = {};
      this.state.tasks.map(function(task) {
        var label;
        label = typeof task.label === "function" ? task.label() : void 0;
        if (!label) {
          return;
        }
        if (summary[label] == null) {
          summary[label] = 0;
        }
        return summary[label] += task.numberOfImpactedItems();
      });
      return _.pairs(summary).map(function(_arg) {
        var count, label;
        label = _arg[0], count = _arg[1];
        return React.createElement("div", {
          "className": "item",
          "key": label
        }, React.createElement("div", {
          "className": "inner"
        }, React.createElement("span", {
          "className": "count"
        }, "(", new Number(count).toLocaleString(), ")"), label));
      });
    };

    ActivitySidebar.prototype._renderNotificationActivityItems = function() {
      return this.state.notifications.map(function(notification) {
        return React.createElement("div", {
          "className": "item",
          "key": notification.id
        }, React.createElement("div", {
          "className": "inner"
        }, notification.message));
      });
    };

    ActivitySidebar.prototype._onDataChanged = function() {
      return this.setState(this._getStateFromStores());
    };

    ActivitySidebar.prototype._getStateFromStores = function() {
      return {
        notifications: NotificationStore.notifications(),
        tasks: TaskQueueStatusStore.queue(),
        isInitialSyncComplete: NylasSyncStatusStore.isSyncComplete()
      };
    };

    return ActivitySidebar;

  })(React.Component);

  module.exports = ActivitySidebar;

}).call(this);
