(function() {
  var Actions, Notification, NotificationsStore;

  NotificationsStore = require('../lib/notifications-store');

  Notification = NotificationsStore.Notification;

  Actions = require('nylas-exports').Actions;

  describe('Notification', function() {
    it('should assert that a message has been provided', function() {
      return expect(function() {
        return new Notification({});
      }).toThrow();
    });
    it('should assert that a valid type has been provided', function() {
      var type, _i, _len, _ref;
      _ref = ['info', 'success', 'error'];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        type = _ref[_i];
        expect(function() {
          return new Notification({
            type: type,
            message: 'bla'
          });
        }).not.toThrow();
      }
      return expect(function() {
        return new Notification({
          type: 'extreme'
        });
      }).toThrow();
    });
    it('should assert that any actions have ids and labels', function() {
      expect(function() {
        return new Notification({
          type: 'info',
          message: '1',
          actions: [
            {
              id: 'a'
            }
          ]
        });
      }).toThrow();
      expect(function() {
        return new Notification({
          type: 'info',
          message: '2',
          actions: [
            {
              label: 'b'
            }
          ]
        });
      }).toThrow();
      return expect(function() {
        return new Notification({
          type: 'info',
          message: '3',
          actions: [
            {
              id: 'a',
              label: 'b'
            }
          ]
        });
      }).not.toThrow();
    });
    it('should assign a tag and creation time', function() {
      this.n = new Notification({
        type: 'info',
        message: 'A',
        actions: [
          {
            id: 'a',
            label: 'b'
          }
        ]
      });
      expect(this.n.tag).toBeDefined();
      return expect(this.n.creation).toBeDefined();
    });
    it('should use a provided tag if the notification is meant to replace an existing tag', function() {
      this.n = new Notification({
        tag: 'update',
        type: 'info',
        message: 'A',
        actions: [
          {
            id: 'a',
            label: 'b'
          }
        ]
      });
      return expect(this.n.tag).toBe('update');
    });
    return it('should be valid at creation', function() {
      this.n = new Notification({
        type: 'info',
        message: 'A',
        actions: [
          {
            id: 'a',
            label: 'b'
          }
        ]
      });
      return expect(this.n.valid()).toBe(true);
    });
  });

  describe('NotificationStore', function() {
    beforeEach(function() {
      return NotificationsStore._flush();
    });
    it('should have no notifications by default', function() {
      return expect(NotificationsStore.notifications().length).toEqual(0);
    });
    it('should register a notification', function() {
      var message, n;
      message = "Hello";
      Actions.postNotification({
        type: 'info',
        message: message
      });
      n = NotificationsStore.notifications()[0];
      return expect(n.message).toEqual(message);
    });
    it('should unregister on removeNotification', function() {
      var n;
      Actions.postNotification({
        type: 'info',
        message: 'hi'
      });
      n = NotificationsStore.notifications()[0];
      NotificationsStore._removeNotification(n);
      return expect(NotificationsStore.notifications().length).toEqual(0);
    });
    return describe("with a few notifications", function() {
      beforeEach(function() {
        Actions.postNotification({
          type: 'info',
          message: 'A',
          sticky: true
        });
        Actions.postNotification({
          type: 'info',
          message: 'B',
          sticky: false
        });
        Actions.postNotification({
          type: 'info',
          message: 'C'
        });
        return Actions.postNotification({
          type: 'info',
          message: 'D',
          sticky: true
        });
      });
      describe("stickyNotifications", function() {
        return it('should return all of the notifications with the sticky flag, ordered by date DESC', function() {
          var messages;
          messages = NotificationsStore.stickyNotifications().map(function(n) {
            return n.message;
          });
          return expect(messages).toEqual(['D', 'A']);
        });
      });
      return describe("notifications", function() {
        return it('should return all of the notifications without the sticky flag, ordered by date DESC', function() {
          var messages;
          messages = NotificationsStore.notifications().map(function(n) {
            return n.message;
          });
          return expect(messages).toEqual(['C', 'B']);
        });
      });
    });
  });

}).call(this);
