(function() {
  var AccountStore, ClearbitDataSource, EdgehillAPI, MAX_RETRY, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ref = require('nylas-exports'), AccountStore = _ref.AccountStore, EdgehillAPI = _ref.EdgehillAPI;

  MAX_RETRY = 10;

  module.exports = ClearbitDataSource = (function() {
    function ClearbitDataSource() {
      this.parseResponse = __bind(this.parseResponse, this);
    }

    ClearbitDataSource.prototype.clearbitAPI = function() {
      return "https://person.clearbit.com/v2/combined";
    };

    ClearbitDataSource.prototype.find = function(_arg) {
      var email, tok, tryCount;
      email = _arg.email, tryCount = _arg.tryCount;
      if ((tryCount != null ? tryCount : 0) >= MAX_RETRY) {
        return Promise.resolve(null);
      }
      tok = AccountStore.tokenForAccountId(AccountStore.accounts()[0].id);
      return new Promise((function(_this) {
        return function(resolve, reject) {
          return EdgehillAPI.makeRequest({
            auth: {
              user: tok,
              pass: ""
            },
            path: "/proxy/clearbit/" + (_this.clearbitAPI()) + "/find?email=" + email,
            success: function(body, response) {
              return _this.parseResponse(body, response, email, tryCount).then(resolve)["catch"](reject);
            },
            error: reject
          });
        };
      })(this));
    };

    ClearbitDataSource.prototype.parseResponse = function(body, response, requestedEmail, tryCount) {
      if (body == null) {
        body = {};
      }
      if (tryCount == null) {
        tryCount = 0;
      }
      return new Promise((function(_this) {
        return function(resolve, reject) {
          var person, _ref1, _ref2, _ref3, _ref4, _ref5, _ref6, _ref7, _ref8;
          if (response.statusCode === 202) {
            setTimeout(function() {
              return _this.find({
                email: requestedEmail,
                tryCount: tryCount + 1
              }).then(resolve)["catch"](reject);
            }, 1000);
            return;
          } else if (response.statusCode !== 200) {
            resolve(null);
            return;
          }
          person = body.person;
          if (!person) {
            person = {
              email: requestedEmail
            };
          }
          return resolve({
            cacheDate: Date.now(),
            email: requestedEmail,
            bio: (_ref1 = (_ref2 = person.bio) != null ? _ref2 : (_ref3 = person.twitter) != null ? _ref3.bio : void 0) != null ? _ref1 : (_ref4 = person.aboutme) != null ? _ref4.bio : void 0,
            location: (_ref5 = person.location) != null ? _ref5 : (_ref6 = person.geo) != null ? _ref6.city : void 0,
            currentTitle: (_ref7 = person.employment) != null ? _ref7.title : void 0,
            currentEmployer: (_ref8 = person.employment) != null ? _ref8.name : void 0,
            profilePhotoUrl: person.avatar,
            socialProfiles: _this._socialProfiles(person)
          });
        };
      })(this));
    };

    ClearbitDataSource.prototype._socialProfiles = function(person) {
      var profiles, _ref1, _ref2, _ref3, _ref4, _ref5, _ref6;
      if (person == null) {
        person = {};
      }
      profiles = {};
      if (((_ref1 = (_ref2 = person.twitter) != null ? _ref2.handle : void 0) != null ? _ref1 : "").length > 0) {
        profiles.twitter = {
          handle: person.twitter.handle,
          url: "https://twitter.com/" + person.twitter.handle
        };
      }
      if (((_ref3 = (_ref4 = person.facebook) != null ? _ref4.handle : void 0) != null ? _ref3 : "").length > 0) {
        profiles.facebook = {
          handle: person.facebook.handle,
          url: "https://facebook.com/" + person.facebook.handle
        };
      }
      if (((_ref5 = (_ref6 = person.linkedin) != null ? _ref6.handle : void 0) != null ? _ref5 : "").length > 0) {
        profiles.linkedin = {
          handle: person.linkedin.handle,
          url: "https://linkedin.com/" + person.linkedin.handle
        };
      }
      return profiles;
    };

    return ClearbitDataSource;

  })();

}).call(this);
