(function() {
  var FocusedPerspectiveStore, InjectedComponentSet, ListTabular, MailImportantIcon, MailLabelSet, React, RetinaImg, Thread, ThreadArchiveQuickAction, ThreadListIcon, ThreadListParticipants, ThreadListStore, ThreadTrashQuickAction, TimestampComponentForPerspective, Utils, c1, c2, c3, c4, c5, cNarrow, classNames, subject, _, _ref, _ref1, _ref2;

  _ = require('underscore');

  React = require('react');

  classNames = require('classnames');

  _ref = require('nylas-component-kit'), ListTabular = _ref.ListTabular, RetinaImg = _ref.RetinaImg, MailLabelSet = _ref.MailLabelSet, MailImportantIcon = _ref.MailImportantIcon, InjectedComponentSet = _ref.InjectedComponentSet;

  _ref1 = require('nylas-exports'), Thread = _ref1.Thread, FocusedPerspectiveStore = _ref1.FocusedPerspectiveStore, Utils = _ref1.Utils;

  _ref2 = require('./thread-list-quick-actions'), ThreadArchiveQuickAction = _ref2.ThreadArchiveQuickAction, ThreadTrashQuickAction = _ref2.ThreadTrashQuickAction;

  ThreadListParticipants = require('./thread-list-participants');

  ThreadListStore = require('./thread-list-store');

  ThreadListIcon = require('./thread-list-icon');

  TimestampComponentForPerspective = function(thread) {
    if (FocusedPerspectiveStore.current().isSent()) {
      return React.createElement("span", {
        "className": "timestamp"
      }, Utils.shortTimeString(thread.lastMessageSentTimestamp));
    } else {
      return React.createElement("span", {
        "className": "timestamp"
      }, Utils.shortTimeString(thread.lastMessageReceivedTimestamp));
    }
  };

  subject = function(subj) {
    var part, subjComponents, subjParts, _i, _len;
    if ((subj != null ? subj : "").trim().length === 0) {
      return React.createElement("span", {
        "className": "no-subject"
      }, "(No Subject)");
    } else if (subj.split(/([\uD800-\uDBFF][\uDC00-\uDFFF])/g).length > 1) {
      subjComponents = [];
      subjParts = subj.split(/([\uD800-\uDBFF][\uDC00-\uDFFF])/g);
      for (_i = 0, _len = subjParts.length; _i < _len; _i++) {
        part = subjParts[_i];
        if (part.match(/([\uD800-\uDBFF][\uDC00-\uDFFF])/g)) {
          subjComponents.push(React.createElement("span", {
            "className": "emoji"
          }, part));
        } else {
          subjComponents.push(React.createElement("span", null, part));
        }
      }
      return subjComponents;
    } else {
      return subj;
    }
  };

  c1 = new ListTabular.Column({
    name: "★",
    resolver: (function(_this) {
      return function(thread) {
        return [
          React.createElement(ThreadListIcon, {
            "key": "thread-list-icon",
            "thread": thread
          }), React.createElement(MailImportantIcon, {
            "key": "mail-important-icon",
            "thread": thread,
            "showIfAvailableForAnyAccount": true
          }), React.createElement(InjectedComponentSet, {
            "key": "injected-component-set",
            "inline": true,
            "containersRequired": false,
            "matching": {
              role: "ThreadListIcon"
            },
            "className": "thread-injected-icons",
            "exposedProps": {
              thread: thread
            }
          })
        ];
      };
    })(this)
  });

  c2 = new ListTabular.Column({
    name: "Participants",
    width: 200,
    resolver: (function(_this) {
      return function(thread) {
        var hasDraft, _ref3;
        hasDraft = _.find((_ref3 = thread.metadata) != null ? _ref3 : [], function(m) {
          return m.draft;
        });
        if (hasDraft) {
          return React.createElement("div", {
            "style": {
              display: 'flex'
            }
          }, React.createElement(ThreadListParticipants, {
            "thread": thread
          }), React.createElement(RetinaImg, {
            "name": "icon-draft-pencil.png",
            "className": "draft-icon",
            "mode": RetinaImg.Mode.ContentPreserve
          }));
        } else {
          return React.createElement(ThreadListParticipants, {
            "thread": thread
          });
        }
      };
    })(this)
  });

  c3 = new ListTabular.Column({
    name: "Message",
    flex: 4,
    resolver: (function(_this) {
      return function(thread) {
        var attachment, hasAttachments, metadata, _ref3;
        attachment = false;
        metadata = (_ref3 = thread.metadata) != null ? _ref3 : [];
        hasAttachments = thread.hasAttachments && metadata.find(function(m) {
          return Utils.showIconForAttachments(m.files);
        });
        if (hasAttachments) {
          attachment = React.createElement("div", {
            "className": "thread-icon thread-icon-attachment"
          });
        }
        return React.createElement("span", {
          "className": "details"
        }, React.createElement(MailLabelSet, {
          "thread": thread
        }), React.createElement("span", {
          "className": "subject"
        }, subject(thread.subject)), React.createElement("span", {
          "className": "snippet"
        }, thread.snippet), attachment);
      };
    })(this)
  });

  c4 = new ListTabular.Column({
    name: "Date",
    resolver: (function(_this) {
      return function(thread) {
        return TimestampComponentForPerspective(thread);
      };
    })(this)
  });

  c5 = new ListTabular.Column({
    name: "HoverActions",
    resolver: (function(_this) {
      return function(thread) {
        return React.createElement("div", {
          "className": "inner"
        }, React.createElement(InjectedComponentSet, {
          "key": "injected-component-set",
          "inline": true,
          "containersRequired": false,
          "children": [
            React.createElement(ThreadTrashQuickAction, {
              "key": "thread-trash-quick-action",
              "thread": thread
            }), React.createElement(ThreadArchiveQuickAction, {
              "key": "thread-archive-quick-action",
              "thread": thread
            })
          ],
          "matching": {
            role: "ThreadListQuickAction"
          },
          "className": "thread-injected-quick-actions",
          "exposedProps": {
            thread: thread
          }
        }));
      };
    })(this)
  });

  cNarrow = new ListTabular.Column({
    name: "Item",
    flex: 1,
    resolver: (function(_this) {
      return function(thread) {
        var attachment, hasAttachments, hasDraft, metadata, pencil, _ref3;
        pencil = false;
        attachment = false;
        metadata = (_ref3 = thread.metadata) != null ? _ref3 : [];
        hasAttachments = thread.hasAttachments && metadata.find(function(m) {
          return Utils.showIconForAttachments(m.files);
        });
        if (hasAttachments) {
          attachment = React.createElement("div", {
            "className": "thread-icon thread-icon-attachment"
          });
        }
        hasDraft = _.find(metadata, function(m) {
          return m.draft;
        });
        if (hasDraft) {
          pencil = React.createElement(RetinaImg, {
            "name": "icon-draft-pencil.png",
            "className": "draft-icon",
            "mode": RetinaImg.Mode.ContentPreserve
          });
        }
        return React.createElement("div", {
          "style": {
            display: 'flex',
            alignItems: 'flex-start'
          }
        }, React.createElement("div", {
          "className": "icons-column"
        }, React.createElement(ThreadListIcon, {
          "thread": thread
        }), React.createElement(InjectedComponentSet, {
          "inline": true,
          "matchLimit": 1.,
          "direction": "column",
          "containersRequired": false,
          "key": "injected-component-set",
          "exposedProps": {
            thread: thread
          },
          "matching": {
            role: "ThreadListIcon"
          },
          "className": "thread-injected-icons"
        }), React.createElement(MailImportantIcon, {
          "thread": thread,
          "showIfAvailableForAnyAccount": true
        })), React.createElement("div", {
          "className": "thread-info-column"
        }, React.createElement("div", {
          "className": "participants-wrapper"
        }, React.createElement(ThreadListParticipants, {
          "thread": thread
        }), pencil, React.createElement("span", {
          "style": {
            flex: 1
          }
        }), attachment, TimestampComponentForPerspective(thread)), React.createElement("div", {
          "className": "subject"
        }, subject(thread.subject)), React.createElement("div", {
          "className": "snippet-and-labels"
        }, React.createElement("div", {
          "className": "snippet"
        }, thread.snippet, "\u00a0"), React.createElement("div", {
          "style": {
            flex: 1,
            flexShrink: 1
          }
        }), React.createElement(MailLabelSet, {
          "thread": thread
        }))));
      };
    })(this)
  });

  module.exports = {
    Narrow: [cNarrow],
    Wide: [c1, c2, c3, c4, c5]
  };

}).call(this);
