(function() {
  var React, ThreadListParticipants, Utils, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require('react');

  Utils = require('nylas-exports').Utils;

  _ = require('underscore');

  ThreadListParticipants = (function(_super) {
    __extends(ThreadListParticipants, _super);

    function ThreadListParticipants() {
      this.getTokens = __bind(this.getTokens, this);
      this.getTokensFromParticipants = __bind(this.getTokensFromParticipants, this);
      this.getTokensFromMetadata = __bind(this.getTokensFromMetadata, this);
      this.renderSpans = __bind(this.renderSpans, this);
      this.render = __bind(this.render, this);
      this.shouldComponentUpdate = __bind(this.shouldComponentUpdate, this);
      return ThreadListParticipants.__super__.constructor.apply(this, arguments);
    }

    ThreadListParticipants.displayName = 'ThreadListParticipants';

    ThreadListParticipants.propTypes = {
      thread: React.PropTypes.object.isRequired
    };

    ThreadListParticipants.prototype.shouldComponentUpdate = function(nextProps) {
      if (nextProps.thread === this.props.thread) {
        return false;
      }
      return true;
    };

    ThreadListParticipants.prototype.render = function() {
      var items;
      items = this.getTokens();
      return React.createElement("div", {
        "className": "participants"
      }, this.renderSpans(items));
    };

    ThreadListParticipants.prototype.renderSpans = function(items) {
      var accumulate, accumulated, accumulatedUnread, contact, flush, idx, short, spacer, spans, unread, _i, _len, _ref;
      spans = [];
      accumulated = null;
      accumulatedUnread = false;
      flush = function() {
        if (accumulated) {
          spans.push(React.createElement("span", {
            "key": spans.length,
            "className": "unread-" + accumulatedUnread
          }, accumulated));
        }
        accumulated = null;
        return accumulatedUnread = false;
      };
      accumulate = function(text, unread) {
        if (accumulated && unread && accumulatedUnread !== unread) {
          flush();
        }
        if (accumulated) {
          return accumulated += text;
        } else {
          accumulated = text;
          return accumulatedUnread = unread;
        }
      };
      for (idx = _i = 0, _len = items.length; _i < _len; idx = ++_i) {
        _ref = items[idx], spacer = _ref.spacer, contact = _ref.contact, unread = _ref.unread;
        if (spacer) {
          accumulate('...');
        } else {
          if (contact.name.length > 0) {
            if (items.length > 1) {
              short = contact.displayName({
                includeAccountLabel: false,
                compact: true
              });
            } else {
              short = contact.displayName({
                includeAccountLabel: false
              });
            }
          } else {
            short = contact.email;
          }
          if (idx < items.length - 1 && !items[idx + 1].spacer) {
            short += ", ";
          }
          accumulate(short, unread);
        }
      }
      if (this.props.thread.metadata && this.props.thread.metadata.length > 1) {
        accumulate(" (" + this.props.thread.metadata.length + ")");
      }
      flush();
      return spans;
    };

    ThreadListParticipants.prototype.getTokensFromMetadata = function() {
      var contact, field, idx, lastContact, lastToken, message, messages, sameEmail, sameName, tokens, _i, _j, _len, _len1, _ref;
      messages = this.props.thread.metadata;
      tokens = [];
      field = 'from';
      if (messages.every(function(message) {
        return message.isFromMe();
      })) {
        field = 'to';
      }
      for (idx = _i = 0, _len = messages.length; _i < _len; idx = ++_i) {
        message = messages[idx];
        if (message.draft) {
          continue;
        }
        _ref = message[field];
        for (_j = 0, _len1 = _ref.length; _j < _len1; _j++) {
          contact = _ref[_j];
          if (tokens.length === 0) {
            tokens.push({
              contact: contact,
              unread: message.unread
            });
          } else {
            lastToken = tokens[tokens.length - 1];
            lastContact = lastToken.contact;
            sameEmail = Utils.emailIsEquivalent(lastContact.email, contact.email);
            sameName = lastContact.name === contact.name;
            if (sameEmail && sameName) {
              lastToken.unread || (lastToken.unread = message.unread);
            } else {
              tokens.push({
                contact: contact,
                unread: message.unread
              });
            }
          }
        }
      }
      return tokens;
    };

    ThreadListParticipants.prototype.getTokensFromParticipants = function() {
      var contacts, _ref;
      contacts = (_ref = this.props.thread.participants) != null ? _ref : [];
      contacts = contacts.filter(function(contact) {
        return !contact.isMe();
      });
      return contacts.map(function(contact) {
        return {
          contact: contact,
          unread: false
        };
      });
    };

    ThreadListParticipants.prototype.getTokens = function() {
      var list, listTrimmed, _ref;
      if (this.props.thread.metadata instanceof Array) {
        list = this.getTokensFromMetadata();
      } else {
        list = this.getTokensFromParticipants();
      }
      if (list.length === 0 && ((_ref = this.props.thread.participants) != null ? _ref.length : void 0) > 0) {
        list.push({
          contact: this.props.thread.participants[0],
          unread: false
        });
      }
      if (list.length > 3) {
        listTrimmed = [
          list[0], {
            spacer: true
          }, list[list.length - 2], list[list.length - 1]
        ];
        list = listTrimmed;
      }
      return list;
    };

    return ThreadListParticipants;

  })(React.Component);

  module.exports = ThreadListParticipants;

}).call(this);
