(function() {
  var AccountStore, Actions, CanvasUtils, Category, CategoryRemovalTargetRulesets, CategoryStore, ChangeStarredTask, ChangeUnreadTask, EmptyListState, FluxContainer, FocusContainer, FocusedContentStore, FocusedPerspectiveStore, MultiselectList, React, ReactDOM, TaskFactory, Thread, ThreadList, ThreadListColumns, ThreadListContextMenu, ThreadListScrollTooltip, ThreadListStore, WorkspaceStore, classNames, _, _ref, _ref1,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  React = require('react');

  ReactDOM = require('react-dom');

  classNames = require('classnames');

  _ref = require('nylas-component-kit'), MultiselectList = _ref.MultiselectList, FocusContainer = _ref.FocusContainer, EmptyListState = _ref.EmptyListState, FluxContainer = _ref.FluxContainer;

  _ref1 = require('nylas-exports'), Actions = _ref1.Actions, Thread = _ref1.Thread, Category = _ref1.Category, CanvasUtils = _ref1.CanvasUtils, TaskFactory = _ref1.TaskFactory, ChangeUnreadTask = _ref1.ChangeUnreadTask, ChangeStarredTask = _ref1.ChangeStarredTask, WorkspaceStore = _ref1.WorkspaceStore, AccountStore = _ref1.AccountStore, CategoryStore = _ref1.CategoryStore, FocusedContentStore = _ref1.FocusedContentStore, FocusedPerspectiveStore = _ref1.FocusedPerspectiveStore;

  ThreadListColumns = require('./thread-list-columns');

  ThreadListScrollTooltip = require('./thread-list-scroll-tooltip');

  ThreadListStore = require('./thread-list-store');

  ThreadListContextMenu = require('./thread-list-context-menu')["default"];

  CategoryRemovalTargetRulesets = require('./category-removal-target-rulesets')["default"];

  ThreadList = (function(_super) {
    __extends(ThreadList, _super);

    ThreadList.displayName = 'ThreadList';

    ThreadList.containerRequired = false;

    ThreadList.containerStyles = {
      minWidth: 300,
      maxWidth: 3000
    };

    function ThreadList(props) {
      this.props = props;
      this._onSelectUnstarred = __bind(this._onSelectUnstarred, this);
      this._onSelectStarred = __bind(this._onSelectStarred, this);
      this._onSelectUnread = __bind(this._onSelectUnread, this);
      this._onSelectRead = __bind(this._onSelectRead, this);
      this._onDeleteItem = __bind(this._onDeleteItem, this);
      this._onArchiveItem = __bind(this._onArchiveItem, this);
      this._onRemoveFromView = __bind(this._onRemoveFromView, this);
      this._onMarkAsSpam = __bind(this._onMarkAsSpam, this);
      this._onSetUnread = __bind(this._onSetUnread, this);
      this._onSetImportant = __bind(this._onSetImportant, this);
      this._onSnoozeItem = __bind(this._onSnoozeItem, this);
      this._onStarItem = __bind(this._onStarItem, this);
      this._onResize = __bind(this._onResize, this);
      this._onDragEnd = __bind(this._onDragEnd, this);
      this._onDragStart = __bind(this._onDragStart, this);
      this._onShowContextMenu = __bind(this._onShowContextMenu, this);
      this._shift = __bind(this._shift, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.state = {
        style: 'unknown'
      };
    }

    ThreadList.prototype.componentDidMount = function() {
      window.addEventListener('resize', this._onResize, true);
      ReactDOM.findDOMNode(this).addEventListener('contextmenu', this._onShowContextMenu);
      return this._onResize();
    };

    ThreadList.prototype.componentWillUnmount = function() {
      window.removeEventListener('resize', this._onResize, true);
      return ReactDOM.findDOMNode(this).removeEventListener('contextmenu', this._onShowContextMenu);
    };

    ThreadList.prototype._shift = function(_arg) {
      var afterRunning, dataSource, focusedId, focusedIdx, item, offset;
      offset = _arg.offset, afterRunning = _arg.afterRunning;
      dataSource = ThreadListStore.dataSource();
      focusedId = FocusedContentStore.focusedId('thread');
      focusedIdx = Math.min(dataSource.count() - 1, Math.max(0, dataSource.indexOfId(focusedId) + offset));
      item = dataSource.get(focusedIdx);
      afterRunning();
      return Actions.setFocus({
        collection: 'thread',
        item: item
      });
    };

    ThreadList.prototype._keymapHandlers = function() {
      return {
        'core:remove-from-view': (function(_this) {
          return function() {
            return _this._onRemoveFromView();
          };
        })(this),
        'core:gmail-remove-from-view': (function(_this) {
          return function() {
            return _this._onRemoveFromView(CategoryRemovalTargetRulesets.Gmail);
          };
        })(this),
        'core:archive-item': this._onArchiveItem,
        'core:delete-item': this._onDeleteItem,
        'core:star-item': this._onStarItem,
        'core:snooze-item': this._onSnoozeItem,
        'core:mark-important': (function(_this) {
          return function() {
            return _this._onSetImportant(true);
          };
        })(this),
        'core:mark-unimportant': (function(_this) {
          return function() {
            return _this._onSetImportant(false);
          };
        })(this),
        'core:mark-as-unread': (function(_this) {
          return function() {
            return _this._onSetUnread(true);
          };
        })(this),
        'core:mark-as-read': (function(_this) {
          return function() {
            return _this._onSetUnread(false);
          };
        })(this),
        'core:report-as-spam': (function(_this) {
          return function() {
            return _this._onMarkAsSpam(false);
          };
        })(this),
        'core:remove-and-previous': (function(_this) {
          return function() {
            return _this._shift({
              offset: -1,
              afterRunning: _this._onRemoveFromView
            });
          };
        })(this),
        'core:remove-and-next': (function(_this) {
          return function() {
            return _this._shift({
              offset: 1,
              afterRunning: _this._onRemoveFromView
            });
          };
        })(this),
        'thread-list:select-read': this._onSelectRead,
        'thread-list:select-unread': this._onSelectUnread,
        'thread-list:select-starred': this._onSelectStarred,
        'thread-list:select-unstarred': this._onSelectUnstarred
      };
    };

    ThreadList.prototype.render = function() {
      var columns, itemHeight;
      if (this.state.style === 'wide') {
        columns = ThreadListColumns.Wide;
        itemHeight = 36;
      } else {
        columns = ThreadListColumns.Narrow;
        itemHeight = 85;
      }
      return React.createElement(FluxContainer, {
        "stores": [ThreadListStore],
        "getStateFromStores": (function() {
          return {
            dataSource: ThreadListStore.dataSource()
          };
        })
      }, React.createElement(FocusContainer, {
        "collection": "thread"
      }, React.createElement(MultiselectList, {
        "ref": "list",
        "columns": columns,
        "itemPropsProvider": this._threadPropsProvider,
        "itemHeight": itemHeight,
        "className": "thread-list thread-list-" + this.state.style,
        "scrollTooltipComponent": ThreadListScrollTooltip,
        "emptyComponent": EmptyListState,
        "keymapHandlers": this._keymapHandlers(),
        "onDragStart": this._onDragStart,
        "onDragEnd": this._onDragEnd,
        "draggable": "true"
      })));
    };

    ThreadList.prototype._threadPropsProvider = function(item) {
      var props;
      props = {
        className: classNames({
          'unread': item.unread
        })
      };
      props.shouldEnableSwipe = (function(_this) {
        return function() {
          var perspective, tasks;
          perspective = FocusedPerspectiveStore.current();
          tasks = perspective.tasksForRemovingItems([item], CategoryRemovalTargetRulesets.Default);
          return tasks.length > 0;
        };
      })(this);
      props.onSwipeRightClass = (function(_this) {
        return function() {
          var name, perspective, task, tasks;
          perspective = FocusedPerspectiveStore.current();
          tasks = perspective.tasksForRemovingItems([item], CategoryRemovalTargetRulesets.Default);
          if (tasks.length === 0) {
            return null;
          }
          task = tasks[0];
          name = task instanceof ChangeStarredTask ? 'unstar' : task.categoriesToAdd().length === 1 ? task.categoriesToAdd()[0].name : 'remove';
          return "swipe-" + name;
        };
      })(this);
      props.onSwipeRight = function(callback) {
        var perspective, tasks;
        perspective = FocusedPerspectiveStore.current();
        tasks = perspective.tasksForRemovingItems([item], CategoryRemovalTargetRulesets.Default);
        if (tasks.length === 0) {
          callback(false);
        }
        Actions.closePopover();
        Actions.queueTasks(tasks);
        return callback(true);
      };
      if (FocusedPerspectiveStore.current().isInbox()) {
        props.onSwipeLeftClass = 'swipe-snooze';
        props.onSwipeCenter = (function(_this) {
          return function() {
            return Actions.closePopover();
          };
        })(this);
        props.onSwipeLeft = (function(_this) {
          return function(callback) {
            var SnoozePopover, element, originRect;
            SnoozePopover = require('../../thread-snooze/lib/snooze-popover')["default"];
            element = document.querySelector("[data-item-id=\"" + item.id + "\"]");
            originRect = element.getBoundingClientRect();
            return Actions.openPopover(React.createElement(SnoozePopover, {
              "threads": [item],
              "swipeCallback": callback
            }), {
              originRect: originRect,
              direction: 'right',
              fallbackDirection: 'down'
            });
          };
        })(this);
      }
      return props;
    };

    ThreadList.prototype._targetItemsForMouseEvent = function(event) {
      var dataSource, itemThreadId, thread;
      itemThreadId = this.refs.list.itemIdAtPoint(event.clientX, event.clientY);
      if (!itemThreadId) {
        return null;
      }
      dataSource = ThreadListStore.dataSource();
      if (__indexOf.call(dataSource.selection.ids(), itemThreadId) >= 0) {
        return {
          threadIds: dataSource.selection.ids(),
          accountIds: _.uniq(_.pluck(dataSource.selection.items(), 'accountId'))
        };
      } else {
        thread = dataSource.getById(itemThreadId);
        if (!thread) {
          return null;
        }
        return {
          threadIds: [thread.id],
          accountIds: [thread.accountId]
        };
      }
    };

    ThreadList.prototype._onShowContextMenu = function(event) {
      var data;
      data = this._targetItemsForMouseEvent(event);
      if (!data) {
        event.preventDefault();
        return;
      }
      return (new ThreadListContextMenu(data)).displayMenu();
    };

    ThreadList.prototype._onDragStart = function(event) {
      var canvas, data;
      data = this._targetItemsForMouseEvent(event);
      if (!data) {
        event.preventDefault();
        return;
      }
      event.dataTransfer.effectAllowed = "move";
      event.dataTransfer.dragEffect = "move";
      canvas = CanvasUtils.canvasWithThreadDragImage(data.threadIds.length);
      event.dataTransfer.setDragImage(canvas, 10, 10);
      event.dataTransfer.setData('nylas-threads-data', JSON.stringify(data));
    };

    ThreadList.prototype._onDragEnd = function(event) {};

    ThreadList.prototype._onResize = function(event) {
      var current, desired;
      current = this.state.style;
      desired = ReactDOM.findDOMNode(this).offsetWidth < 540 ? 'narrow' : 'wide';
      if (current !== desired) {
        return this.setState({
          style: desired
        });
      }
    };

    ThreadList.prototype._threadsForKeyboardAction = function() {
      var focused;
      if (!ThreadListStore.dataSource()) {
        return null;
      }
      focused = FocusedContentStore.focused('thread');
      if (focused) {
        return [focused];
      } else if (ThreadListStore.dataSource().selection.count() > 0) {
        return ThreadListStore.dataSource().selection.items();
      } else {
        return null;
      }
    };

    ThreadList.prototype._onStarItem = function() {
      var task, threads;
      threads = this._threadsForKeyboardAction();
      if (!threads) {
        return;
      }
      task = TaskFactory.taskForInvertingStarred({
        threads: threads
      });
      return Actions.queueTask(task);
    };

    ThreadList.prototype._onSnoozeItem = function() {
      var SnoozePopover, element, originRect, threads;
      threads = this._threadsForKeyboardAction();
      if (!threads) {
        return;
      }
      SnoozePopover = require('../../thread-snooze/lib/snooze-popover')["default"];
      element = document.querySelector(".snooze-button.btn.btn-toolbar");
      if (!element) {
        return;
      }
      originRect = element.getBoundingClientRect();
      return Actions.openPopover(React.createElement(SnoozePopover, {
        "threads": threads
      }), {
        originRect: originRect,
        direction: 'down'
      });
    };

    ThreadList.prototype._onSetImportant = function(important) {
      var tasks, threads;
      threads = this._threadsForKeyboardAction();
      if (!threads) {
        return;
      }
      if (!NylasEnv.config.get('core.workspace.showImportant')) {
        return;
      }
      if (important) {
        tasks = TaskFactory.tasksForApplyingCategories({
          threads: threads,
          categoriesToRemove: function(accountId) {
            return [];
          },
          categoriesToAdd: function(accountId) {
            return [CategoryStore.getStandardCategory(accountId, 'important')];
          }
        });
      } else {
        tasks = TaskFactory.tasksForApplyingCategories({
          threads: threads,
          categoriesToRemove: function(accountId) {
            important = CategoryStore.getStandardCategory(accountId, 'important');
            if (important) {
              return [important];
            }
            return [];
          }
        });
      }
      return Actions.queueTasks(tasks);
    };

    ThreadList.prototype._onSetUnread = function(unread) {
      var threads;
      threads = this._threadsForKeyboardAction();
      if (!threads) {
        return;
      }
      Actions.queueTask(new ChangeUnreadTask({
        threads: threads,
        unread: unread
      }));
      return Actions.popSheet();
    };

    ThreadList.prototype._onMarkAsSpam = function() {
      var tasks, threads;
      threads = this._threadsForKeyboardAction();
      if (!threads) {
        return;
      }
      tasks = TaskFactory.tasksForMarkingAsSpam({
        threads: threads
      });
      return Actions.queueTasks(tasks);
    };

    ThreadList.prototype._onRemoveFromView = function(ruleset) {
      var current, tasks, threads;
      if (ruleset == null) {
        ruleset = CategoryRemovalTargetRulesets.Default;
      }
      threads = this._threadsForKeyboardAction();
      if (!threads) {
        return;
      }
      current = FocusedPerspectiveStore.current();
      tasks = current.tasksForRemovingItems(threads, ruleset);
      Actions.queueTasks(tasks);
      return Actions.popSheet();
    };

    ThreadList.prototype._onArchiveItem = function() {
      var tasks, threads;
      threads = this._threadsForKeyboardAction();
      if (threads) {
        tasks = TaskFactory.tasksForArchiving({
          threads: threads
        });
        Actions.queueTasks(tasks);
      }
      return Actions.popSheet();
    };

    ThreadList.prototype._onDeleteItem = function() {
      var tasks, threads;
      threads = this._threadsForKeyboardAction();
      if (threads) {
        tasks = TaskFactory.tasksForMovingToTrash({
          threads: threads
        });
        Actions.queueTasks(tasks);
      }
      return Actions.popSheet();
    };

    ThreadList.prototype._onSelectRead = function() {
      var dataSource, items;
      dataSource = ThreadListStore.dataSource();
      items = dataSource.itemsCurrentlyInViewMatching(function(item) {
        return !item.unread;
      });
      return dataSource.selection.set(items);
    };

    ThreadList.prototype._onSelectUnread = function() {
      var dataSource, items;
      dataSource = ThreadListStore.dataSource();
      items = dataSource.itemsCurrentlyInViewMatching(function(item) {
        return item.unread;
      });
      return dataSource.selection.set(items);
    };

    ThreadList.prototype._onSelectStarred = function() {
      var dataSource, items;
      dataSource = ThreadListStore.dataSource();
      items = dataSource.itemsCurrentlyInViewMatching(function(item) {
        return item.starred;
      });
      return dataSource.selection.set(items);
    };

    ThreadList.prototype._onSelectUnstarred = function() {
      var dataSource, items;
      dataSource = ThreadListStore.dataSource();
      items = dataSource.itemsCurrentlyInViewMatching(function(item) {
        return !item.starred;
      });
      return dataSource.selection.set(items);
    };

    return ThreadList;

  })(React.Component);

  module.exports = ThreadList;

}).call(this);
