(function() {
  var AccountStore, Contact, Message, React, ReactTestUtils, Thread, ThreadListParticipants, _, _ref;

  React = require("react");

  ReactTestUtils = require('react-addons-test-utils');

  _ = require('underscore');

  _ref = require('nylas-exports'), AccountStore = _ref.AccountStore, Thread = _ref.Thread, Contact = _ref.Contact, Message = _ref.Message;

  ThreadListParticipants = require('../lib/thread-list-participants');

  describe("ThreadListParticipants", function() {
    beforeEach(function() {
      return this.account = AccountStore.accounts()[0];
    });
    it("renders into the document", function() {
      this.participants = ReactTestUtils.renderIntoDocument(React.createElement(ThreadListParticipants, {
        "thread": new Thread
      }));
      return expect(ReactTestUtils.isCompositeComponentWithType(this.participants, ThreadListParticipants)).toBe(true);
    });
    it("renders unread contacts with .unread-true", function() {
      var ben, thread, unread;
      ben = new Contact({
        email: 'ben@nylas.com',
        name: 'ben'
      });
      ben.unread = true;
      thread = new Thread();
      thread.metadata = [
        new Message({
          from: [ben],
          unread: true
        })
      ];
      this.participants = ReactTestUtils.renderIntoDocument(React.createElement(ThreadListParticipants, {
        "thread": thread
      }));
      unread = ReactTestUtils.scryRenderedDOMComponentsWithClass(this.participants, 'unread-true');
      return expect(unread.length).toBe(1);
    });
    return describe("getTokens", function() {
      beforeEach(function() {
        this.ben = new Contact({
          email: 'ben@nylas.com',
          name: 'ben'
        });
        this.evan = new Contact({
          email: 'evan@nylas.com',
          name: 'evan'
        });
        this.evanAgain = new Contact({
          email: 'evan@nylas.com',
          name: 'evan'
        });
        this.michael = new Contact({
          email: 'michael@nylas.com',
          name: 'michael'
        });
        this.kavya = new Contact({
          email: 'kavya@nylas.com',
          name: 'kavya'
        });
        this.phab1 = new Contact({
          email: 'no-reply@phab.nylas.com',
          name: 'Ben'
        });
        return this.phab2 = new Contact({
          email: 'no-reply@phab.nylas.com',
          name: 'MG'
        });
      });
      describe("when thread.messages is available", function() {
        return it("correctly produces items for display in a wide range of scenarios", function() {
          var participants, scenario, scenarios, thread, _i, _len, _results;
          scenarios = [
            {
              name: 'single read email',
              "in": [
                new Message({
                  unread: false,
                  from: [this.ben]
                })
              ],
              out: [
                {
                  contact: this.ben,
                  unread: false
                }
              ]
            }, {
              name: 'single read email and draft',
              "in": [
                new Message({
                  unread: false,
                  from: [this.ben]
                }), new Message({
                  from: [this.ben],
                  draft: true
                })
              ],
              out: [
                {
                  contact: this.ben,
                  unread: false
                }
              ]
            }, {
              name: 'single unread email',
              "in": [
                new Message({
                  unread: true,
                  from: [this.evan]
                })
              ],
              out: [
                {
                  contact: this.evan,
                  unread: true
                }
              ]
            }, {
              name: 'single unread response',
              "in": [
                new Message({
                  unread: false,
                  from: [this.ben]
                }), new Message({
                  unread: true,
                  from: [this.evan]
                })
              ],
              out: [
                {
                  contact: this.ben,
                  unread: false
                }, {
                  contact: this.evan,
                  unread: true
                }
              ]
            }, {
              name: 'two unread responses',
              "in": [
                new Message({
                  unread: false,
                  from: [this.ben]
                }), new Message({
                  unread: true,
                  from: [this.evan]
                }), new Message({
                  unread: true,
                  from: [this.kavya]
                })
              ],
              out: [
                {
                  contact: this.ben,
                  unread: false
                }, {
                  contact: this.evan,
                  unread: true
                }, {
                  contact: this.kavya,
                  unread: true
                }
              ]
            }, {
              name: 'two unread responses (repeated participants)',
              "in": [
                new Message({
                  unread: false,
                  from: [this.ben]
                }), new Message({
                  unread: true,
                  from: [this.evan]
                }), new Message({
                  unread: true,
                  from: [this.evanAgain]
                })
              ],
              out: [
                {
                  contact: this.ben,
                  unread: false
                }, {
                  contact: this.evan,
                  unread: true
                }
              ]
            }, {
              name: 'three unread responses (repeated participants)',
              "in": [
                new Message({
                  unread: false,
                  from: [this.ben]
                }), new Message({
                  unread: true,
                  from: [this.evan]
                }), new Message({
                  unread: true,
                  from: [this.michael]
                }), new Message({
                  unread: true,
                  from: [this.evanAgain]
                })
              ],
              out: [
                {
                  contact: this.ben,
                  unread: false
                }, {
                  spacer: true
                }, {
                  contact: this.michael,
                  unread: true
                }, {
                  contact: this.evanAgain,
                  unread: true
                }
              ]
            }, {
              name: 'three unread responses',
              "in": [
                new Message({
                  unread: false,
                  from: [this.ben]
                }), new Message({
                  unread: true,
                  from: [this.evan]
                }), new Message({
                  unread: true,
                  from: [this.michael]
                }), new Message({
                  unread: true,
                  from: [this.kavya]
                })
              ],
              out: [
                {
                  contact: this.ben,
                  unread: false
                }, {
                  spacer: true
                }, {
                  contact: this.michael,
                  unread: true
                }, {
                  contact: this.kavya,
                  unread: true
                }
              ]
            }, {
              name: 'ends with two emails from the same person, second one is unread',
              "in": [
                new Message({
                  unread: false,
                  from: [this.ben]
                }), new Message({
                  unread: false,
                  from: [this.evan]
                }), new Message({
                  unread: false,
                  from: [this.kavya]
                }), new Message({
                  unread: true,
                  from: [this.kavya]
                })
              ],
              out: [
                {
                  contact: this.ben,
                  unread: false
                }, {
                  contact: this.evan,
                  unread: false
                }, {
                  contact: this.kavya,
                  unread: true
                }
              ]
            }, {
              name: 'three unread responses to long thread',
              "in": [
                new Message({
                  unread: false,
                  from: [this.ben]
                }), new Message({
                  unread: false,
                  from: [this.evan]
                }), new Message({
                  unread: false,
                  from: [this.michael]
                }), new Message({
                  unread: false,
                  from: [this.ben]
                }), new Message({
                  unread: true,
                  from: [this.evanAgain]
                }), new Message({
                  unread: true,
                  from: [this.michael]
                }), new Message({
                  unread: true,
                  from: [this.evanAgain]
                })
              ],
              out: [
                {
                  contact: this.ben,
                  unread: false
                }, {
                  spacer: true
                }, {
                  contact: this.michael,
                  unread: true
                }, {
                  contact: this.evanAgain,
                  unread: true
                }
              ]
            }, {
              name: 'single unread responses to long thread',
              "in": [
                new Message({
                  unread: false,
                  from: [this.ben]
                }), new Message({
                  unread: false,
                  from: [this.evan]
                }), new Message({
                  unread: false,
                  from: [this.michael]
                }), new Message({
                  unread: false,
                  from: [this.ben]
                }), new Message({
                  unread: true,
                  from: [this.evanAgain]
                })
              ],
              out: [
                {
                  contact: this.ben,
                  unread: false
                }, {
                  spacer: true
                }, {
                  contact: this.ben,
                  unread: false
                }, {
                  contact: this.evanAgain,
                  unread: true
                }
              ]
            }, {
              name: 'long read thread',
              "in": [
                new Message({
                  unread: false,
                  from: [this.ben]
                }), new Message({
                  unread: false,
                  from: [this.evan]
                }), new Message({
                  unread: false,
                  from: [this.michael]
                }), new Message({
                  unread: false,
                  from: [this.ben]
                })
              ],
              out: [
                {
                  contact: this.ben,
                  unread: false
                }, {
                  spacer: true
                }, {
                  contact: this.michael,
                  unread: false
                }, {
                  contact: this.ben,
                  unread: false
                }
              ]
            }, {
              name: 'thread with different participants with the same email address',
              "in": [
                new Message({
                  unread: false,
                  from: [this.phab1]
                }), new Message({
                  unread: false,
                  from: [this.phab2]
                })
              ],
              out: [
                {
                  contact: this.phab1,
                  unread: false
                }, {
                  contact: this.phab2,
                  unread: false
                }
              ]
            }
          ];
          _results = [];
          for (_i = 0, _len = scenarios.length; _i < _len; _i++) {
            scenario = scenarios[_i];
            thread = new Thread();
            thread.metadata = scenario["in"];
            participants = ReactTestUtils.renderIntoDocument(React.createElement(ThreadListParticipants, {
              "thread": thread
            }));
            expect(participants.getTokens()).toEqual(scenario.out);
            if (!_.isEqual(participants.getTokens(), scenario.out)) {
              _results.push(expect(scenario.name).toBe('correct'));
            } else {
              _results.push(void 0);
            }
          }
          return _results;
        });
      });
      describe("when getTokens() called and current user is only sender", function() {
        var getTokens;
        beforeEach(function() {
          this.me = this.account.me();
          this.ben = new Contact({
            email: 'ben@nylas.com',
            name: 'ben'
          });
          this.evan = new Contact({
            email: 'evan@nylas.com',
            name: 'evan'
          });
          this.evanCapitalized = new Contact({
            email: 'EVAN@nylas.com',
            name: 'evan'
          });
          this.michael = new Contact({
            email: 'michael@nylas.com',
            name: 'michael'
          });
          return this.kavya = new Contact({
            email: 'kavya@nylas.com',
            name: 'kavya'
          });
        });
        getTokens = function(threadMetadata) {
          var participants, thread;
          thread = new Thread();
          thread.metadata = threadMetadata;
          participants = ReactTestUtils.renderIntoDocument(React.createElement(ThreadListParticipants, {
            "thread": thread
          }));
          return participants.getTokens();
        };
        it("shows only recipients for emails sent from me to different recipients", function() {
          var actualOut, expectedOut, input;
          input = [
            new Message({
              unread: false,
              from: [this.me],
              to: [this.ben]
            }), new Message({
              unread: false,
              from: [this.me],
              to: [this.evan]
            }), new Message({
              unread: false,
              from: [this.me],
              to: [this.ben]
            })
          ];
          actualOut = getTokens(input);
          expectedOut = [
            {
              contact: this.ben,
              unread: false
            }, {
              contact: this.evan,
              unread: false
            }, {
              contact: this.ben,
              unread: false
            }
          ];
          return expect(actualOut).toEqual(expectedOut);
        });
        it("is case insensitive", function() {
          var actualOut, expectedOut, input;
          input = [
            new Message({
              unread: false,
              from: [this.me],
              to: [this.evan]
            }), new Message({
              unread: false,
              from: [this.me],
              to: [this.evanCapitalized]
            })
          ];
          actualOut = getTokens(input);
          expectedOut = [
            {
              contact: this.evan,
              unread: false
            }
          ];
          return expect(actualOut).toEqual(expectedOut);
        });
        it("shows only first, spacer, second to last, and last recipients if recipients count > 3", function() {
          var actualOut, expectedOut, input;
          input = [
            new Message({
              unread: false,
              from: [this.me],
              to: [this.ben]
            }), new Message({
              unread: false,
              from: [this.me],
              to: [this.evan]
            }), new Message({
              unread: false,
              from: [this.me],
              to: [this.michael]
            }), new Message({
              unread: false,
              from: [this.me],
              to: [this.kavya]
            })
          ];
          actualOut = getTokens(input);
          expectedOut = [
            {
              contact: this.ben,
              unread: false
            }, {
              spacer: true
            }, {
              contact: this.michael,
              unread: false
            }, {
              contact: this.kavya,
              unread: false
            }
          ];
          return expect(actualOut).toEqual(expectedOut);
        });
        it("shows correct recipients even if only one email", function() {
          var actualOut, expectedOut, input;
          input = [
            new Message({
              unread: false,
              from: [this.me],
              to: [this.ben, this.evan, this.michael, this.kavya]
            })
          ];
          actualOut = getTokens(input);
          expectedOut = [
            {
              contact: this.ben,
              unread: false
            }, {
              spacer: true
            }, {
              contact: this.michael,
              unread: false
            }, {
              contact: this.kavya,
              unread: false
            }
          ];
          return expect(actualOut).toEqual(expectedOut);
        });
        it("shows only one recipient if the sender only sent to one recipient", function() {
          var actualOut, expectedOut, input;
          input = [
            new Message({
              unread: false,
              from: [this.me],
              to: [this.evan]
            }), new Message({
              unread: false,
              from: [this.me],
              to: [this.evan]
            }), new Message({
              unread: false,
              from: [this.me],
              to: [this.evan]
            }), new Message({
              unread: false,
              from: [this.me],
              to: [this.evan]
            })
          ];
          actualOut = getTokens(input);
          expectedOut = [
            {
              contact: this.evan,
              unread: false
            }
          ];
          return expect(actualOut).toEqual(expectedOut);
        });
        it("shows only the recipient for one sent email", function() {
          var actualOut, expectedOut, input;
          input = [
            new Message({
              unread: false,
              from: [this.me],
              to: [this.evan]
            })
          ];
          actualOut = getTokens(input);
          expectedOut = [
            {
              contact: this.evan,
              unread: false
            }
          ];
          return expect(actualOut).toEqual(expectedOut);
        });
        return it("shows unread email as well", function() {
          var actualOut, expectedOut, input;
          input = [
            new Message({
              unread: false,
              from: [this.me],
              to: [this.evan]
            }), new Message({
              unread: false,
              from: [this.me],
              to: [this.ben]
            }), new Message({
              unread: true,
              from: [this.me],
              to: [this.kavya]
            }), new Message({
              unread: true,
              from: [this.me],
              to: [this.michael]
            })
          ];
          actualOut = getTokens(input);
          expectedOut = [
            {
              contact: this.evan,
              unread: false
            }, {
              spacer: true
            }, {
              contact: this.kavya,
              unread: true
            }, {
              contact: this.michael,
              unread: true
            }
          ];
          return expect(actualOut).toEqual(expectedOut);
        });
      });
      return describe("when thread.messages is not available", function() {
        return it("correctly produces items for display in a wide range of scenarios", function() {
          var me, participants, scenario, scenarios, thread, _i, _len, _results;
          me = this.account.me();
          scenarios = [
            {
              name: 'one participant',
              "in": [this.ben],
              out: [
                {
                  contact: this.ben,
                  unread: false
                }
              ]
            }, {
              name: 'one participant (me)',
              "in": [me],
              out: [
                {
                  contact: me,
                  unread: false
                }
              ]
            }, {
              name: 'two participants',
              "in": [this.evan, this.ben],
              out: [
                {
                  contact: this.evan,
                  unread: false
                }, {
                  contact: this.ben,
                  unread: false
                }
              ]
            }, {
              name: 'two participants (me)',
              "in": [this.ben, me],
              out: [
                {
                  contact: this.ben,
                  unread: false
                }
              ]
            }, {
              name: 'lots of participants',
              "in": [this.ben, this.evan, this.michael, this.kavya],
              out: [
                {
                  contact: this.ben,
                  unread: false
                }, {
                  spacer: true
                }, {
                  contact: this.michael,
                  unread: false
                }, {
                  contact: this.kavya,
                  unread: false
                }
              ]
            }
          ];
          _results = [];
          for (_i = 0, _len = scenarios.length; _i < _len; _i++) {
            scenario = scenarios[_i];
            thread = new Thread();
            thread.participants = scenario["in"];
            participants = ReactTestUtils.renderIntoDocument(React.createElement(ThreadListParticipants, {
              "thread": thread
            }));
            expect(participants.getTokens()).toEqual(scenario.out);
            if (!_.isEqual(participants.getTokens(), scenario.out)) {
              _results.push(expect(scenario.name).toBe('correct'));
            } else {
              _results.push(void 0);
            }
          }
          return _results;
        });
      });
    });
  });

}).call(this);
