(function() {
  var Actions, CategoryStore, ChangeUnreadTask, FocusedContentStore, MailboxPerspective, MarkAsSpamButton, React, ReactDOM, ReactTestUtils, TaskFactory, Thread, ToggleStarredButton, ToggleUnreadButton, test_thread, test_thread_starred, _ref, _ref1;

  React = require("react");

  ReactDOM = require("react-dom");

  ReactTestUtils = require('react-addons-test-utils');

  _ref = require("nylas-exports"), Thread = _ref.Thread, FocusedContentStore = _ref.FocusedContentStore, Actions = _ref.Actions, CategoryStore = _ref.CategoryStore, ChangeUnreadTask = _ref.ChangeUnreadTask, TaskFactory = _ref.TaskFactory, MailboxPerspective = _ref.MailboxPerspective;

  _ref1 = require('../lib/thread-toolbar-buttons'), ToggleStarredButton = _ref1.ToggleStarredButton, ToggleUnreadButton = _ref1.ToggleUnreadButton, MarkAsSpamButton = _ref1.MarkAsSpamButton;

  test_thread = (new Thread).fromJSON({
    "id": "thread_12345",
    "account_id": TEST_ACCOUNT_ID,
    "subject": "Subject 12345",
    "starred": false
  });

  test_thread_starred = (new Thread).fromJSON({
    "id": "thread_starred_12345",
    "account_id": TEST_ACCOUNT_ID,
    "subject": "Subject 12345",
    "starred": true
  });

  describe("ThreadToolbarButtons", function() {
    describe("Starring", function() {
      it("stars a thread if the star button is clicked and thread is unstarred", function() {
        var starButton;
        spyOn(Actions, 'queueTask');
        starButton = ReactTestUtils.renderIntoDocument(React.createElement(ToggleStarredButton, {
          "items": [test_thread]
        }));
        ReactTestUtils.Simulate.click(ReactDOM.findDOMNode(starButton));
        expect(Actions.queueTask.mostRecentCall.args[0].threads).toEqual([test_thread]);
        return expect(Actions.queueTask.mostRecentCall.args[0].starred).toEqual(true);
      });
      return it("unstars a thread if the star button is clicked and thread is starred", function() {
        var starButton;
        spyOn(Actions, 'queueTask');
        starButton = ReactTestUtils.renderIntoDocument(React.createElement(ToggleStarredButton, {
          "items": [test_thread_starred]
        }));
        ReactTestUtils.Simulate.click(ReactDOM.findDOMNode(starButton));
        expect(Actions.queueTask.mostRecentCall.args[0].threads).toEqual([test_thread_starred]);
        return expect(Actions.queueTask.mostRecentCall.args[0].starred).toEqual(false);
      });
    });
    describe("Marking as unread", function() {
      var markUnreadBtn, thread;
      thread = null;
      markUnreadBtn = null;
      beforeEach(function() {
        thread = new Thread({
          id: "thread-id-lol-123",
          accountId: TEST_ACCOUNT_ID,
          unread: false
        });
        return markUnreadBtn = ReactTestUtils.renderIntoDocument(React.createElement(ToggleUnreadButton, {
          "items": [thread]
        }));
      });
      it("queues a task to change unread status to true", function() {
        var changeUnreadTask;
        spyOn(Actions, "queueTask");
        ReactTestUtils.Simulate.click(ReactDOM.findDOMNode(markUnreadBtn).childNodes[0]);
        changeUnreadTask = Actions.queueTask.calls[0].args[0];
        expect(changeUnreadTask instanceof ChangeUnreadTask).toBe(true);
        expect(changeUnreadTask.unread).toBe(true);
        return expect(changeUnreadTask.threads[0].id).toBe(thread.id);
      });
      return it("returns to the thread list", function() {
        spyOn(Actions, "popSheet");
        ReactTestUtils.Simulate.click(ReactDOM.findDOMNode(markUnreadBtn).childNodes[0]);
        return expect(Actions.popSheet).toHaveBeenCalled();
      });
    });
    return describe("Marking as spam", function() {
      var markSpamButton, thread;
      thread = null;
      markSpamButton = null;
      describe("when the thread is already in spam", function() {
        beforeEach(function() {
          thread = new Thread({
            id: "thread-id-lol-123",
            accountId: TEST_ACCOUNT_ID,
            categories: [
              {
                name: 'spam'
              }
            ]
          });
          return markSpamButton = ReactTestUtils.renderIntoDocument(React.createElement(MarkAsSpamButton, {
            "items": [thread]
          }));
        });
        return it("queues a task to remove spam", function() {
          var labelsToAdd, labelsToRemove, _ref2;
          spyOn(Actions, 'queueTasks');
          spyOn(CategoryStore, 'getSpamCategory').andReturn(thread.categories[0]);
          ReactTestUtils.Simulate.click(ReactDOM.findDOMNode(markSpamButton));
          _ref2 = Actions.queueTasks.mostRecentCall.args[0][0], labelsToAdd = _ref2.labelsToAdd, labelsToRemove = _ref2.labelsToRemove;
          expect(labelsToAdd).toEqual([]);
          return expect(labelsToRemove).toEqual([thread.categories[0]]);
        });
      });
      return describe("when the thread can be moved to spam", function() {
        beforeEach(function() {
          spyOn(Actions, 'queueTasks');
          spyOn(MailboxPerspective.prototype, 'canMoveThreadsTo').andReturn(true);
          thread = new Thread({
            id: "thread-id-lol-123",
            accountId: TEST_ACCOUNT_ID,
            categories: []
          });
          return markSpamButton = ReactTestUtils.renderIntoDocument(React.createElement(MarkAsSpamButton, {
            "items": [thread]
          }));
        });
        it("queues a task to mark as spam", function() {
          spyOn(TaskFactory, 'tasksForMarkingAsSpam');
          ReactTestUtils.Simulate.click(ReactDOM.findDOMNode(markSpamButton));
          return expect(TaskFactory.tasksForMarkingAsSpam).toHaveBeenCalledWith({
            threads: [thread]
          });
        });
        return it("returns to the thread list", function() {
          spyOn(Actions, 'popSheet');
          ReactTestUtils.Simulate.click(ReactDOM.findDOMNode(markSpamButton));
          return expect(Actions.popSheet).toHaveBeenCalled();
        });
      });
    });
  });

}).call(this);
