(function() {
  var AccountStore, Actions, Contact, ContactStore, DatabaseStore, FocusedPerspectiveStore, NylasStore, SearchActions, SearchMailboxPerspective, SearchStore, Thread, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  NylasStore = require('nylas-store');

  _ref = require('nylas-exports'), Thread = _ref.Thread, Contact = _ref.Contact, Actions = _ref.Actions, ContactStore = _ref.ContactStore, AccountStore = _ref.AccountStore, DatabaseStore = _ref.DatabaseStore, FocusedPerspectiveStore = _ref.FocusedPerspectiveStore;

  SearchActions = require('./search-actions')["default"];

  SearchMailboxPerspective = require('./search-mailbox-perspective')["default"];

  SearchStore = (function(_super) {
    __extends(SearchStore, _super);

    function SearchStore() {
      this._compileResults = __bind(this._compileResults, this);
      this._fetchThreadResults = __bind(this._fetchThreadResults, this);
      this._fetchContactResults = __bind(this._fetchContactResults, this);
      this._rebuildResults = __bind(this._rebuildResults, this);
      this._clearResults = __bind(this._clearResults, this);
      this._onSearchBlurred = __bind(this._onSearchBlurred, this);
      this._onQuerySubmitted = __bind(this._onQuerySubmitted, this);
      this._onQueryChanged = __bind(this._onQueryChanged, this);
      this._onPerspectiveChanged = __bind(this._onPerspectiveChanged, this);
      this._onSearchCompleted = __bind(this._onSearchCompleted, this);
      this.isSearching = __bind(this.isSearching, this);
      this.suggestions = __bind(this.suggestions, this);
      this.queryPopulated = __bind(this.queryPopulated, this);
      this.query = __bind(this.query, this);
      var _ref1;
      this._searchQuery = (_ref1 = FocusedPerspectiveStore.current().searchQuery) != null ? _ref1 : "";
      this._searchSuggestionsVersion = 1;
      this._isSearching = false;
      this._clearResults();
      this.listenTo(FocusedPerspectiveStore, this._onPerspectiveChanged);
      this.listenTo(SearchActions.querySubmitted, this._onQuerySubmitted);
      this.listenTo(SearchActions.queryChanged, this._onQueryChanged);
      this.listenTo(SearchActions.searchBlurred, this._onSearchBlurred);
      this.listenTo(SearchActions.searchCompleted, this._onSearchCompleted);
    }

    SearchStore.prototype.query = function() {
      return this._searchQuery;
    };

    SearchStore.prototype.queryPopulated = function() {
      return this._searchQuery && this._searchQuery.trim().length > 0;
    };

    SearchStore.prototype.suggestions = function() {
      return this._suggestions;
    };

    SearchStore.prototype.isSearching = function() {
      return this._isSearching;
    };

    SearchStore.prototype._onSearchCompleted = function() {
      this._isSearching = false;
      return this.trigger();
    };

    SearchStore.prototype._onPerspectiveChanged = function() {
      var _ref1;
      this._searchQuery = (_ref1 = FocusedPerspectiveStore.current().searchQuery) != null ? _ref1 : "";
      return this.trigger();
    };

    SearchStore.prototype._onQueryChanged = function(query) {
      this._searchQuery = query;
      this._compileResults();
      return _.defer((function(_this) {
        return function() {
          return _this._rebuildResults();
        };
      })(this));
    };

    SearchStore.prototype._onQuerySubmitted = function(query) {
      var current, next;
      this._searchQuery = query;
      current = FocusedPerspectiveStore.current();
      if (this.queryPopulated()) {
        this._isSearching = true;
        if (this._perspectiveBeforeSearch == null) {
          this._perspectiveBeforeSearch = current;
        }
        next = new SearchMailboxPerspective(current.accountIds, this._searchQuery.trim());
        Actions.focusMailboxPerspective(next);
      } else if (current instanceof SearchMailboxPerspective) {
        if (this._perspectiveBeforeSearch) {
          Actions.focusMailboxPerspective(this._perspectiveBeforeSearch);
          this._perspectiveBeforeSearch = null;
        } else {
          Actions.focusDefaultMailboxPerspectiveForAccounts(AccountStore.accounts());
        }
      }
      return this._clearResults();
    };

    SearchStore.prototype._onSearchBlurred = function() {
      return this._clearResults();
    };

    SearchStore.prototype._clearResults = function() {
      this._searchSuggestionsVersion = 1;
      this._threadResults = [];
      this._contactResults = [];
      this._suggestions = [];
      return this.trigger();
    };

    SearchStore.prototype._rebuildResults = function() {
      if (!this.queryPopulated()) {
        return this._clearResults();
      }
      this._searchSuggestionsVersion += 1;
      this._fetchThreadResults();
      return this._fetchContactResults();
    };

    SearchStore.prototype._fetchContactResults = function() {
      var version;
      version = this._searchSuggestionsVersion;
      return ContactStore.searchContacts(this._searchQuery, {
        limit: 10
      }).then((function(_this) {
        return function(contacts) {
          if (version !== _this._searchSuggestionsVersion) {
            return;
          }
          _this._contactResults = contacts;
          return _this._compileResults();
        };
      })(this));
    };

    SearchStore.prototype._fetchThreadResults = function() {
      var accountIds, databaseQuery;
      if (this._fetchingThreadResultsVersion) {
        return;
      }
      this._fetchingThreadResultsVersion = this._searchSuggestionsVersion;
      databaseQuery = DatabaseStore.findAll(Thread).where(Thread.attributes.subject.like(this._searchQuery)).order(Thread.attributes.lastMessageReceivedTimestamp.descending()).limit(4);
      accountIds = FocusedPerspectiveStore.current().accountIds;
      if (accountIds instanceof Array) {
        databaseQuery.where(Thread.attributes.accountId["in"](accountIds));
      }
      return databaseQuery.then((function(_this) {
        return function(results) {
          if (_this._searchSuggestionsVersion === _this._fetchingThreadResultsVersion) {
            _this._fetchingThreadResultsVersion = null;
            _this._threadResults = results;
            return _this._compileResults();
          } else if (_this._searchSuggestionsVersion > _this._fetchingThreadResultsVersion) {
            _this._fetchingThreadResultsVersion = null;
            return _this._fetchThreadResults();
          } else {
            return _this._fetchingThreadResultsVersion = null;
          }
        };
      })(this));
    };

    SearchStore.prototype._compileResults = function() {
      var contact, thread, _i, _j, _len, _len1, _ref1, _ref2;
      this._suggestions = [];
      this._suggestions.push({
        label: "Message Contains: " + this._searchQuery,
        value: this._searchQuery
      });
      if (this._threadResults.length) {
        this._suggestions.push({
          divider: 'Threads'
        });
        _ref1 = this._threadResults;
        for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
          thread = _ref1[_i];
          this._suggestions.push({
            thread: thread
          });
        }
      }
      if (this._contactResults.length) {
        this._suggestions.push({
          divider: 'People'
        });
        _ref2 = this._contactResults;
        for (_j = 0, _len1 = _ref2.length; _j < _len1; _j++) {
          contact = _ref2[_j];
          this._suggestions.push({
            contact: contact,
            value: contact.email
          });
        }
      }
      return this.trigger();
    };

    return SearchStore;

  })(NylasStore);

  module.exports = new SearchStore();

}).call(this);
