(function() {
  var AccountStore, Actions, ComponentRegistry, React, ReactCSSTransitionGroup, ReactDOM, RetinaImg, UndoRedoComponent, UndoRedoStore, Utils, classNames, path, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  path = require('path');

  React = require('react');

  ReactDOM = require('react-dom');

  ReactCSSTransitionGroup = require('react-addons-css-transition-group');

  classNames = require('classnames');

  RetinaImg = require('nylas-component-kit').RetinaImg;

  _ref = require('nylas-exports'), Actions = _ref.Actions, Utils = _ref.Utils, ComponentRegistry = _ref.ComponentRegistry, UndoRedoStore = _ref.UndoRedoStore, AccountStore = _ref.AccountStore;

  UndoRedoComponent = (function(_super) {
    __extends(UndoRedoComponent, _super);

    UndoRedoComponent.displayName = 'UndoRedoComponent';

    UndoRedoComponent.containerRequired = false;

    function UndoRedoComponent(props) {
      this.props = props;
      this._hide = __bind(this._hide, this);
      this._onClick = __bind(this._onClick, this);
      this._onMouseLeave = __bind(this._onMouseLeave, this);
      this._onMouseEnter = __bind(this._onMouseEnter, this);
      this._renderUndoRedoManager = __bind(this._renderUndoRedoManager, this);
      this.render = __bind(this.render, this);
      this._ensureTimeout = __bind(this._ensureTimeout, this);
      this._clearTimeout = __bind(this._clearTimeout, this);
      this._timeout = null;
      this.state = {};
    }

    UndoRedoComponent.prototype._clearTimeout = function() {
      clearTimeout(this._timeout);
      return this._timeout = null;
    };

    UndoRedoComponent.prototype._ensureTimeout = function(state) {
      if (state == null) {
        state = this.state;
      }
      if (this._timeout) {
        this._clearTimeout();
      }
      if (state.show) {
        return this._timeout = setTimeout(this._hide, 3000);
      }
    };

    UndoRedoComponent.prototype._getStateFromStores = function() {
      var show, tasks;
      tasks = UndoRedoStore.getMostRecent();
      show = tasks && tasks.length > 0;
      return {
        show: show,
        tasks: tasks
      };
    };

    UndoRedoComponent.prototype.componentWillMount = function() {
      return this._unsubscribe = UndoRedoStore.listen((function(_this) {
        return function() {
          var nextState;
          nextState = _this._getStateFromStores();
          _this.setState(nextState);
          return _this._ensureTimeout(nextState);
        };
      })(this));
    };

    UndoRedoComponent.prototype.componentWillUnmount = function() {
      this._clearTimeout();
      return this._unsubscribe();
    };

    UndoRedoComponent.prototype.render = function() {
      var classes;
      classes = classNames({
        "undo-redo-manager": true
      });
      return React.createElement(ReactCSSTransitionGroup, {
        "className": classes,
        "transitionLeaveTimeout": 150.,
        "transitionEnterTimeout": 150.,
        "transitionName": "undo-redo-item"
      }, this._renderUndoRedoManager());
    };

    UndoRedoComponent.prototype._renderUndoRedoManager = function() {
      if (!this.state.show) {
        return;
      }
      return React.createElement("div", {
        "className": "undo-redo",
        "onMouseEnter": this._onMouseEnter,
        "onMouseLeave": this._onMouseLeave
      }, React.createElement("div", {
        "className": "undo-redo-message-wrapper"
      }, this.state.tasks.map(function(t) {
        return t.description();
      }).join(', ')), React.createElement("div", {
        "className": "undo-redo-action-wrapper",
        "onClick": this._onClick
      }, React.createElement(RetinaImg, {
        "name": "undo-icon@2x.png",
        "mode": RetinaImg.Mode.ContentIsMask
      }), React.createElement("span", {
        "className": "undo-redo-action-text"
      }, "Undo")));
    };

    UndoRedoComponent.prototype._onMouseEnter = function() {
      return this._clearTimeout();
    };

    UndoRedoComponent.prototype._onMouseLeave = function() {
      return this._ensureTimeout(this.state);
    };

    UndoRedoComponent.prototype._onClick = function() {
      NylasEnv.commands.dispatch('core:undo');
      return this._hide();
    };

    UndoRedoComponent.prototype._hide = function() {
      return this.setState({
        show: false,
        tasks: null
      });
    };

    return UndoRedoComponent;

  })(React.Component);

  module.exports = UndoRedoComponent;

}).call(this);
