(function() {
  var AccountStore, Actions, DatabaseStore, MailRulesProcessor, NylasAPI, NylasLongConnection, NylasSyncWorker, NylasSyncWorkerPool, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __slice = [].slice;

  _ = require('underscore');

  _ref = require('nylas-exports'), NylasAPI = _ref.NylasAPI, Actions = _ref.Actions, AccountStore = _ref.AccountStore, DatabaseStore = _ref.DatabaseStore, MailRulesProcessor = _ref.MailRulesProcessor;

  NylasLongConnection = require('./nylas-long-connection');

  NylasSyncWorker = require('./nylas-sync-worker');

  NylasSyncWorkerPool = (function() {
    function NylasSyncWorkerPool() {
      this._handleDeltaDeletion = __bind(this._handleDeltaDeletion, this);
      this._handleAccountDeltas = __bind(this._handleAccountDeltas, this);
      this._handleDeltaMetadata = __bind(this._handleDeltaMetadata, this);
      this.workerForAccount = __bind(this.workerForAccount, this);
      this.workers = __bind(this.workers, this);
      this._workers = [];
      AccountStore.listen(this._onAccountsChanged, this);
      this._onAccountsChanged();
    }

    NylasSyncWorkerPool.prototype._onAccountsChanged = function() {
      var accounts, old, worker, workers, _i, _len;
      if (NylasEnv.inSpecMode()) {
        return;
      }
      accounts = AccountStore.accounts();
      workers = _.map(accounts, this.workerForAccount);
      old = _.without.apply(_, [this._workers].concat(__slice.call(workers)));
      for (_i = 0, _len = old.length; _i < _len; _i++) {
        worker = old[_i];
        worker.cleanup();
      }
      return this._workers = workers;
    };

    NylasSyncWorkerPool.prototype.workers = function() {
      return this._workers;
    };

    NylasSyncWorkerPool.prototype.workerForAccount = function(account) {
      var connection, worker;
      worker = _.find(this._workers, function(c) {
        return c.account().id === account.id;
      });
      if (worker) {
        return worker;
      }
      worker = new NylasSyncWorker(NylasAPI, account);
      connection = worker.connection();
      connection.onDeltas((function(_this) {
        return function(deltas) {
          return _this._handleDeltas(deltas);
        };
      })(this));
      this._workers.push(worker);
      worker.start();
      return worker;
    };

    NylasSyncWorkerPool.prototype._cleanupAccountWorkers = function() {
      var worker, _i, _len, _ref1;
      _ref1 = this._workers;
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        worker = _ref1[_i];
        worker.cleanup();
      }
      return this._workers = [];
    };

    NylasSyncWorkerPool.prototype._handleDeltas = function(deltas) {
      var create, destroy, dict, metadata, modify, type, _i, _len, _ref1, _ref2;
      Actions.longPollReceivedRawDeltas(deltas);
      Actions.longPollReceivedRawDeltasPing(deltas.length);
      deltas.forEach(function(delta) {
        if (delta.attributes) {
          return Object.defineProperty(delta.attributes, '_delta', {
            get: function() {
              return delta;
            }
          });
        }
      });
      _ref1 = this._clusterDeltas(deltas), create = _ref1.create, modify = _ref1.modify, destroy = _ref1.destroy;
      metadata = [];
      _ref2 = [create, modify];
      for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
        deltas = _ref2[_i];
        if (deltas['metadata']) {
          metadata = metadata.concat(_.values(deltas['metadata']));
          delete deltas['metadata'];
        }
      }
      delete create['account'];
      delete destroy['account'];
      if (modify['account']) {
        this._handleAccountDeltas(_.values(modify['account']));
        delete modify['account'];
      }
      for (type in create) {
        dict = create[type];
        create[type] = NylasAPI._handleModelResponse(_.values(dict));
      }
      return Promise.props(create).then((function(_this) {
        return function(created) {
          for (type in modify) {
            dict = modify[type];
            modify[type] = NylasAPI._handleModelResponse(_.values(dict));
          }
          return Promise.props(modify).then(function(modified) {
            return Promise.all(_this._handleDeltaMetadata(metadata)).then(function() {
              var destroyPromises, _ref3;
              if (_.flatten(_.values(created)).length > 0) {
                MailRulesProcessor.processMessages((_ref3 = created['message']) != null ? _ref3 : [])["finally"](function() {
                  return Actions.didPassivelyReceiveNewModels(created);
                });
              }
              destroyPromises = destroy.map(_this._handleDeltaDeletion);
              return Promise.settle(destroyPromises).then(function() {
                return Actions.longPollProcessedDeltas();
              });
            });
          });
        };
      })(this));
    };

    NylasSyncWorkerPool.prototype._clusterDeltas = function(deltas) {
      var create, delta, destroy, modify, _i, _len, _name, _name1;
      create = {};
      modify = {};
      destroy = [];
      for (_i = 0, _len = deltas.length; _i < _len; _i++) {
        delta = deltas[_i];
        if (delta.event === 'create') {
          create[_name = delta.object] || (create[_name] = {});
          create[delta.object][delta.attributes.id] = delta.attributes;
        } else if (delta.event === 'modify') {
          modify[_name1 = delta.object] || (modify[_name1] = {});
          modify[delta.object][delta.attributes.id] = delta.attributes;
        } else if (delta.event === 'delete') {
          destroy.push(delta);
        }
      }
      return {
        create: create,
        modify: modify,
        destroy: destroy
      };
    };

    NylasSyncWorkerPool.prototype._handleDeltaMetadata = function(metadata) {
      return metadata.map((function(_this) {
        return function(metadatum) {
          var klass;
          klass = NylasAPI._apiObjectToClassMap[metadatum.object_type];
          return DatabaseStore.inTransaction(function(t) {
            return t.find(klass, metadatum.object_id).then(function(model) {
              var localMetadatum;
              if (!model) {
                return Promise.resolve();
              }
              model = model.applyPluginMetadata(metadatum.application_id, metadatum.value);
              localMetadatum = model.metadataObjectForPluginId(metadatum.application_id);
              localMetadatum.version = metadatum.version;
              return t.persistModel(model);
            });
          });
        };
      })(this));
    };

    NylasSyncWorkerPool.prototype._handleAccountDeltas = function(deltas) {
      var delta, _i, _len, _results;
      _results = [];
      for (_i = 0, _len = deltas.length; _i < _len; _i++) {
        delta = deltas[_i];
        Actions.updateAccount(delta.account_id, {
          syncState: delta.sync_state
        });
        if (delta.sync_state !== "running") {
          _results.push(Actions.recordUserEvent('Account Sync Errored', {
            accountId: delta.account_id,
            syncState: delta.sync_state
          }));
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    NylasSyncWorkerPool.prototype._handleDeltaDeletion = function(delta) {
      var klass;
      klass = NylasAPI._apiObjectToClassMap[delta.object];
      if (!klass) {
        return;
      }
      return DatabaseStore.inTransaction((function(_this) {
        return function(t) {
          return t.find(klass, delta.id).then(function(model) {
            if (!model) {
              return Promise.resolve();
            }
            return t.unpersistModel(model);
          });
        };
      })(this));
    };

    return NylasSyncWorkerPool;

  })();

  module.exports = NylasSyncWorkerPool;

}).call(this);
