(function() {
  var Actions, BackoffTimer, ContactRankingsCache, DatabaseStore, INITIAL_PAGE_SIZE, MAX_PAGE_SIZE, NylasLongConnection, NylasSyncStatusStore, NylasSyncWorker, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require('underscore');

  _ref = require('nylas-exports'), Actions = _ref.Actions, DatabaseStore = _ref.DatabaseStore, NylasSyncStatusStore = _ref.NylasSyncStatusStore;

  NylasLongConnection = require('./nylas-long-connection');

  ContactRankingsCache = require('./contact-rankings-cache');

  INITIAL_PAGE_SIZE = 30;

  MAX_PAGE_SIZE = 100;

  BackoffTimer = (function() {
    function BackoffTimer(fn) {
      this.fn = fn;
      this.getCurrentDelay = __bind(this.getCurrentDelay, this);
      this.resetDelay = __bind(this.resetDelay, this);
      this.start = __bind(this.start, this);
      this.backoff = __bind(this.backoff, this);
      this.cancel = __bind(this.cancel, this);
      this.resetDelay();
    }

    BackoffTimer.prototype.cancel = function() {
      if (this._timeout) {
        clearTimeout(this._timeout);
      }
      return this._timeout = null;
    };

    BackoffTimer.prototype.backoff = function() {
      this._delay = Math.min(this._delay * 1.7, 5 * 1000 * 60);
      if (!NylasEnv.inSpecMode()) {
        return console.log("Backing off after sync failure. Will retry in " + (Math.floor(this._delay / 1000)) + " seconds.");
      }
    };

    BackoffTimer.prototype.start = function() {
      if (this._timeout) {
        clearTimeout(this._timeout);
      }
      return this._timeout = setTimeout((function(_this) {
        return function() {
          _this._timeout = null;
          return _this.fn();
        };
      })(this), this._delay);
    };

    BackoffTimer.prototype.resetDelay = function() {
      return this._delay = 2 * 1000;
    };

    BackoffTimer.prototype.getCurrentDelay = function() {
      return this._delay;
    };

    return BackoffTimer;

  })();

  module.exports = NylasSyncWorker = (function() {
    function NylasSyncWorker(api, account) {
      this._onRetrySync = __bind(this._onRetrySync, this);
      this._backoff = __bind(this._backoff, this);
      this.resume = __bind(this.resume, this);
      this._api = api;
      this._account = account;
      this._resumeTimer = new BackoffTimer((function(_this) {
        return function() {
          return _this.resume();
        };
      })(this));
      this._refreshingCaches = [new ContactRankingsCache(account.id)];
      this._terminated = false;
      this._connection = new NylasLongConnection(api, account.id, {
        ready: (function(_this) {
          return function() {
            return _this._state !== null;
          };
        })(this),
        getCursor: (function(_this) {
          return function() {
            if (_this._state === null) {
              return null;
            }
            return _this._state.cursor || NylasEnv.config.get("nylas." + _this._account.id + ".cursor");
          };
        })(this),
        setCursor: (function(_this) {
          return function(val) {
            _this._state.cursor = val;
            return _this.writeState();
          };
        })(this),
        setStatus: (function(_this) {
          return function(status) {
            _this._state.longConnectionStatus = status;
            if (status === NylasLongConnection.Status.Closed) {
              _this._backoff();
            }
            if (status === NylasLongConnection.Status.Connected) {
              _this._resumeTimer.resetDelay();
            }
            return _this.writeState();
          };
        })(this)
      });
      this._unlisten = Actions.retrySync.listen(this._onRetrySync, this);
      this._state = null;
      DatabaseStore.findJSONBlob("NylasSyncWorker:" + this._account.id).then((function(_this) {
        return function(json) {
          var key, _i, _len, _ref1;
          _this._state = json != null ? json : {};
          _this._state.longConnectionStatus = NylasLongConnection.Status.Idle;
          _ref1 = NylasSyncStatusStore.ModelsForSync;
          for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
            key = _ref1[_i];
            if (_this._state[key]) {
              _this._state[key].busy = false;
            }
          }
          return _this.resume();
        };
      })(this));
      this;
    }

    NylasSyncWorker.prototype.account = function() {
      return this._account;
    };

    NylasSyncWorker.prototype.connection = function() {
      return this._connection;
    };

    NylasSyncWorker.prototype.state = function() {
      return this._state;
    };

    NylasSyncWorker.prototype.busy = function() {
      var key, state, _ref1;
      if (!this._state) {
        return false;
      }
      _ref1 = this._state;
      for (key in _ref1) {
        state = _ref1[key];
        if (state.busy) {
          return true;
        }
      }
      return false;
    };

    NylasSyncWorker.prototype.start = function() {
      this._resumeTimer.start();
      this._connection.start();
      this._refreshingCaches.map(function(c) {
        return c.start();
      });
      return this.resume();
    };

    NylasSyncWorker.prototype.cleanup = function() {
      if (typeof this._unlisten === "function") {
        this._unlisten();
      }
      this._resumeTimer.cancel();
      this._connection.end();
      this._refreshingCaches.map(function(c) {
        return c.end();
      });
      this._terminated = true;
      return this;
    };

    NylasSyncWorker.prototype.resume = function() {
      var needed;
      if (!this._state) {
        return;
      }
      this._connection.start();
      this._resumeTimer.cancel();
      needed = [
        {
          model: 'threads'
        }, {
          model: 'messages',
          maxFetchCount: 5000
        }, {
          model: this._account.categoryCollection(),
          initialPageSize: 1000
        }, {
          model: 'drafts'
        }, {
          model: 'contacts'
        }, {
          model: 'calendars'
        }, {
          model: 'events'
        }
      ].filter((function(_this) {
        return function(_arg) {
          var model;
          model = _arg.model;
          return _this.shouldFetchCollection(model);
        };
      })(this));
      if (needed.length === 0) {
        return;
      }
      return this.fetchAllMetadata((function(_this) {
        return function() {
          return needed.forEach(function(_arg) {
            var initialPageSize, maxFetchCount, model;
            model = _arg.model, initialPageSize = _arg.initialPageSize, maxFetchCount = _arg.maxFetchCount;
            return _this.fetchCollection(model, {
              initialPageSize: initialPageSize,
              maxFetchCount: maxFetchCount
            });
          });
        };
      })(this));
    };

    NylasSyncWorker.prototype.fetchAllMetadata = function(finished) {
      var makeMetadataRequest;
      this._metadata = {};
      makeMetadataRequest = (function(_this) {
        return function(offset) {
          var limit;
          limit = 200;
          return _this._fetchWithErrorHandling({
            path: "/metadata",
            qs: {
              limit: limit,
              offset: offset
            },
            success: function(data) {
              var metadatum, _base, _i, _len, _name;
              for (_i = 0, _len = data.length; _i < _len; _i++) {
                metadatum = data[_i];
                if ((_base = _this._metadata)[_name = metadatum.object_id] == null) {
                  _base[_name] = [];
                }
                _this._metadata[metadatum.object_id].push(metadatum);
              }
              if (data.length === limit) {
                return makeMetadataRequest(offset + limit);
              } else {
                console.log("Retrieved " + (offset + data.length) + " metadata objects");
                return finished();
              }
            }
          });
        };
      })(this);
      if (this._api.pluginsSupported) {
        return makeMetadataRequest(0);
      } else {
        return finished();
      }
    };

    NylasSyncWorker.prototype.shouldFetchCollection = function(model) {
      var state, _ref1;
      if (!this._state) {
        return false;
      }
      state = (_ref1 = this._state[model]) != null ? _ref1 : {};
      if (state.complete) {
        return false;
      }
      if (state.busy) {
        return false;
      }
      return true;
    };

    NylasSyncWorker.prototype.fetchCollection = function(model, _arg) {
      var initialPageSize, limit, maxFetchCount, offset, state, _ref1, _ref2, _ref3;
      _ref1 = _arg != null ? _arg : {}, initialPageSize = _ref1.initialPageSize, maxFetchCount = _ref1.maxFetchCount;
      if (initialPageSize == null) {
        initialPageSize = INITIAL_PAGE_SIZE;
      }
      state = (_ref2 = this._state[model]) != null ? _ref2 : {};
      state.complete = false;
      state.error = null;
      state.busy = true;
      if (state.fetched == null) {
        state.fetched = 0;
      }
      if (!state.count) {
        state.count = 0;
        this.fetchCollectionCount(model, maxFetchCount);
      }
      if (state.lastRequestRange) {
        _ref3 = state.lastRequestRange, limit = _ref3.limit, offset = _ref3.offset;
        if (state.fetched + limit > maxFetchCount) {
          limit = maxFetchCount - state.fetched;
        }
        state.lastRequestRange = null;
        this.fetchCollectionPage(model, {
          limit: limit,
          offset: offset
        }, {
          maxFetchCount: maxFetchCount
        });
      } else {
        limit = initialPageSize;
        if (state.fetched + limit > maxFetchCount) {
          limit = maxFetchCount - state.fetched;
        }
        this.fetchCollectionPage(model, {
          limit: limit,
          offset: 0
        }, {
          maxFetchCount: maxFetchCount
        });
      }
      this._state[model] = state;
      return this.writeState();
    };

    NylasSyncWorker.prototype.fetchCollectionCount = function(model, maxFetchCount) {
      return this._fetchWithErrorHandling({
        path: "/" + model,
        qs: {
          view: 'count'
        },
        success: (function(_this) {
          return function(response) {
            return _this.updateTransferState(model, {
              count: Math.min(response.count, maxFetchCount != null ? maxFetchCount : response.count)
            });
          };
        })(this)
      });
    };

    NylasSyncWorker.prototype.fetchCollectionPage = function(model, params, options) {
      var requestOptions, requestStartTime;
      if (params == null) {
        params = {};
      }
      if (options == null) {
        options = {};
      }
      requestStartTime = Date.now();
      requestOptions = {
        metadataToAttach: this._metadata,
        error: (function(_this) {
          return function(err) {
            if (_this._terminated) {
              return;
            }
            return _this._onFetchCollectionPageError(model, params, err);
          };
        })(this),
        success: (function(_this) {
          return function(json) {
            var lastReceivedIndex, limit, moreToFetch, nextDelay, nextParams;
            if (_this._terminated) {
              return;
            }
            if ((model === "labels" || model === "folders") && _this._hasNoInbox(json)) {
              _this._onFetchCollectionPageError(model, params, "No inbox in " + model);
              return;
            }
            lastReceivedIndex = params.offset + json.length;
            moreToFetch = options.maxFetchCount ? json.length === params.limit && lastReceivedIndex < options.maxFetchCount : json.length === params.limit;
            if (moreToFetch) {
              nextParams = _.extend({}, params, {
                offset: lastReceivedIndex
              });
              limit = Math.min(Math.round(params.limit * 1.5), MAX_PAGE_SIZE);
              if (options.maxFetchCount) {
                limit = Math.min(limit, options.maxFetchCount - lastReceivedIndex);
              }
              nextParams.limit = limit;
              nextDelay = Math.max(0, 1500 - (Date.now() - requestStartTime));
              setTimeout((function() {
                return _this.fetchCollectionPage(model, nextParams, options);
              }), nextDelay);
            }
            return _this.updateTransferState(model, {
              fetched: lastReceivedIndex,
              busy: moreToFetch,
              complete: !moreToFetch,
              lastRequestRange: {
                offset: params.offset,
                limit: params.limit
              },
              error: null
            });
          };
        })(this)
      };
      if (model === 'threads') {
        return this._api.getThreads(this._account.id, params, requestOptions);
      } else {
        return this._api.getCollection(this._account.id, model, params, requestOptions);
      }
    };

    NylasSyncWorker.prototype._hasNoInbox = function(json) {
      return !_.any(json, function(obj) {
        return obj.name === "inbox";
      });
    };

    NylasSyncWorker.prototype._fetchWithErrorHandling = function(_arg) {
      var error, path, qs, success;
      path = _arg.path, qs = _arg.qs, success = _arg.success, error = _arg.error;
      return this._api.makeRequest({
        accountId: this._account.id,
        returnsModel: false,
        path: path,
        qs: qs,
        success: (function(_this) {
          return function(response) {
            if (_this._terminated) {
              return;
            }
            if (success) {
              return success(response);
            }
          };
        })(this),
        error: (function(_this) {
          return function(err) {
            if (_this._terminated) {
              return;
            }
            _this._backoff();
            if (error) {
              return error(err);
            }
          };
        })(this)
      });
    };

    NylasSyncWorker.prototype._onFetchCollectionPageError = function(model, params, err) {
      this._backoff();
      return this.updateTransferState(model, {
        busy: false,
        complete: false,
        error: err.toString(),
        lastRequestRange: {
          offset: params.offset,
          limit: params.limit
        }
      });
    };

    NylasSyncWorker.prototype._backoff = function() {
      this._resumeTimer.backoff();
      this._resumeTimer.start();
      this._state.nextRetryDelay = this._resumeTimer.getCurrentDelay();
      return this._state.nextRetryTimestamp = Date.now() + this._state.nextRetryDelay;
    };

    NylasSyncWorker.prototype.updateTransferState = function(model, updatedKeys) {
      this._state[model] = _.extend(this._state[model], updatedKeys);
      return this.writeState();
    };

    NylasSyncWorker.prototype.writeState = function() {
      if (this._writeState == null) {
        this._writeState = _.debounce((function(_this) {
          return function() {
            return DatabaseStore.inTransaction(function(t) {
              return t.persistJSONBlob("NylasSyncWorker:" + _this._account.id, _this._state);
            });
          };
        })(this), 100);
      }
      return this._writeState();
    };

    NylasSyncWorker.prototype._onRetrySync = function() {
      this._resumeTimer.resetDelay();
      return this.resume();
    };

    return NylasSyncWorker;

  })();

  NylasSyncWorker.BackoffTimer = BackoffTimer;

  NylasSyncWorker.INITIAL_PAGE_SIZE = INITIAL_PAGE_SIZE;

  NylasSyncWorker.MAX_PAGE_SIZE = MAX_PAGE_SIZE;

}).call(this);
