(function() {
  var DatabaseStore, NylasStore, RefreshingJSONCache, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require('underscore');

  _ref = require('nylas-exports'), NylasStore = _ref.NylasStore, DatabaseStore = _ref.DatabaseStore;

  RefreshingJSONCache = (function() {
    function RefreshingJSONCache(_arg) {
      this.key = _arg.key, this.version = _arg.version, this.refreshInterval = _arg.refreshInterval;
      this.fetchData = __bind(this.fetchData, this);
      this.refresh = __bind(this.refresh, this);
      this._timeoutId = null;
    }

    RefreshingJSONCache.prototype.start = function() {
      this.end();
      return DatabaseStore.findJSONBlob(this.key).then((function(_this) {
        return function(json) {
          var timeUntilRefresh;
          timeUntilRefresh = 0;
          if ((json != null) && json.version === _this.version) {
            timeUntilRefresh = Math.max(0, _this.refreshInterval - (Date.now() - json.time));
          }
          return _this._timeoutId = setTimeout(_this.refresh, timeUntilRefresh);
        };
      })(this));
    };

    RefreshingJSONCache.prototype.reset = function() {
      DatabaseStore.inTransaction((function(_this) {
        return function(t) {
          return t.persistJSONBlob(_this.key, {});
        };
      })(this));
      return this.end();
    };

    RefreshingJSONCache.prototype.end = function() {
      if (this._timeoutId) {
        clearInterval(this._timeoutId);
      }
      return this._timeoutId = null;
    };

    RefreshingJSONCache.prototype.refresh = function() {
      if (this._timeoutId) {
        clearTimeout(this._timeoutId);
      }
      this._timeoutId = setTimeout(this.refresh, this.refreshInterval);
      return this.fetchData((function(_this) {
        return function(newValue) {
          return DatabaseStore.inTransaction(function(t) {
            return t.persistJSONBlob(_this.key, {
              version: _this.version,
              time: Date.now(),
              value: newValue
            });
          });
        };
      })(this));
    };

    RefreshingJSONCache.prototype.fetchData = function(callback) {
      throw new Error("Subclasses should override this method.");
    };

    return RefreshingJSONCache;

  })();

  module.exports = RefreshingJSONCache;

}).call(this);
