(function() {
  var Actions, DatabaseStore, DatabaseTransaction, NylasAPI, NylasSyncWorkerPool, Thread, fixturesPath, fs, path, _, _ref;

  _ = require('underscore');

  fs = require('fs');

  path = require('path');

  _ref = require('nylas-exports'), NylasAPI = _ref.NylasAPI, Thread = _ref.Thread, DatabaseStore = _ref.DatabaseStore, DatabaseTransaction = _ref.DatabaseTransaction, Actions = _ref.Actions;

  NylasSyncWorkerPool = require('../lib/nylas-sync-worker-pool');

  NylasSyncWorkerPool = new NylasSyncWorkerPool();

  fixturesPath = path.resolve(__dirname, 'fixtures');

  describe("NylasSyncWorkerPool", function() {
    describe("handleDeltas", function() {
      beforeEach(function() {
        this.sampleDeltas = JSON.parse(fs.readFileSync("" + fixturesPath + "/sample.json"));
        return this.sampleClustered = JSON.parse(fs.readFileSync("" + fixturesPath + "/sample-clustered.json"));
      });
      it("should immediately fire the received raw deltas event", function() {
        spyOn(Actions, 'longPollReceivedRawDeltas');
        spyOn(NylasSyncWorkerPool, '_clusterDeltas').andReturn({
          create: {},
          modify: {},
          destroy: []
        });
        NylasSyncWorkerPool._handleDeltas(this.sampleDeltas);
        return expect(Actions.longPollReceivedRawDeltas).toHaveBeenCalled();
      });
      return it("should call helper methods for all creates first, then modifications, then destroys", function() {
        var createTypes, destroyCount, handleDeltaDeletionPromises, handleModelResponsePromises, modifyTypes, resolveDeltaDeletionPromises, resolveModelResponsePromises;
        spyOn(Actions, 'longPollProcessedDeltas');
        handleDeltaDeletionPromises = [];
        resolveDeltaDeletionPromises = function() {
          var fn, _i, _len;
          for (_i = 0, _len = handleDeltaDeletionPromises.length; _i < _len; _i++) {
            fn = handleDeltaDeletionPromises[_i];
            fn();
          }
          return handleDeltaDeletionPromises = [];
        };
        spyOn(NylasSyncWorkerPool, '_handleDeltaDeletion').andCallFake(function() {
          return new Promise(function(resolve, reject) {
            return handleDeltaDeletionPromises.push(resolve);
          });
        });
        handleModelResponsePromises = [];
        resolveModelResponsePromises = function() {
          var fn, _i, _len;
          for (_i = 0, _len = handleModelResponsePromises.length; _i < _len; _i++) {
            fn = handleModelResponsePromises[_i];
            fn();
          }
          return handleModelResponsePromises = [];
        };
        spyOn(NylasAPI, '_handleModelResponse').andCallFake(function() {
          return new Promise(function(resolve, reject) {
            return handleModelResponsePromises.push(resolve);
          });
        });
        spyOn(NylasSyncWorkerPool, '_clusterDeltas').andReturn(JSON.parse(JSON.stringify(this.sampleClustered)));
        NylasSyncWorkerPool._handleDeltas(this.sampleDeltas);
        createTypes = Object.keys(this.sampleClustered['create']);
        expect(NylasAPI._handleModelResponse.calls.length).toEqual(createTypes.length);
        expect(NylasAPI._handleModelResponse.calls[0].args[0]).toEqual(_.values(this.sampleClustered['create'][createTypes[0]]));
        expect(NylasSyncWorkerPool._handleDeltaDeletion.calls.length).toEqual(0);
        NylasAPI._handleModelResponse.reset();
        resolveModelResponsePromises();
        advanceClock();
        modifyTypes = Object.keys(this.sampleClustered['modify']);
        expect(NylasAPI._handleModelResponse.calls.length).toEqual(modifyTypes.length);
        expect(NylasAPI._handleModelResponse.calls[0].args[0]).toEqual(_.values(this.sampleClustered['modify'][modifyTypes[0]]));
        expect(NylasSyncWorkerPool._handleDeltaDeletion.calls.length).toEqual(0);
        NylasAPI._handleModelResponse.reset();
        resolveModelResponsePromises();
        advanceClock();
        destroyCount = this.sampleClustered['destroy'].length;
        expect(NylasSyncWorkerPool._handleDeltaDeletion.calls.length).toEqual(destroyCount);
        expect(NylasSyncWorkerPool._handleDeltaDeletion.calls[0].args[0]).toEqual(this.sampleClustered['destroy'][0]);
        expect(Actions.longPollProcessedDeltas).not.toHaveBeenCalled();
        resolveDeltaDeletionPromises();
        advanceClock();
        return expect(Actions.longPollProcessedDeltas).toHaveBeenCalled();
      });
    });
    describe("clusterDeltas", function() {
      beforeEach(function() {
        this.sampleDeltas = JSON.parse(fs.readFileSync("" + fixturesPath + "/sample.json"));
        return this.expectedClustered = JSON.parse(fs.readFileSync("" + fixturesPath + "/sample-clustered.json"));
      });
      it("should collect create/modify events into a hash by model type", function() {
        var create, modify, _ref1;
        _ref1 = NylasSyncWorkerPool._clusterDeltas(this.sampleDeltas), create = _ref1.create, modify = _ref1.modify;
        expect(create).toEqual(this.expectedClustered.create);
        return expect(modify).toEqual(this.expectedClustered.modify);
      });
      return it("should collect destroys into an array", function() {
        var destroy;
        destroy = NylasSyncWorkerPool._clusterDeltas(this.sampleDeltas).destroy;
        return expect(destroy).toEqual(this.expectedClustered.destroy);
      });
    });
    describe("handleDeltaDeletion", function() {
      beforeEach(function() {
        this.thread = new Thread({
          id: 'idhere'
        });
        this.delta = {
          "cursor": "bb95ddzqtr2gpmvgrng73t6ih",
          "object": "thread",
          "event": "delete",
          "id": this.thread.id,
          "timestamp": "2015-08-26T17:36:45.297Z"
        };
        return spyOn(DatabaseTransaction.prototype, 'unpersistModel');
      });
      it("should resolve if the object cannot be found", function() {
        spyOn(DatabaseStore, 'find').andCallFake((function(_this) {
          return function(klass, id) {
            return Promise.resolve(null);
          };
        })(this));
        waitsForPromise((function(_this) {
          return function() {
            return NylasSyncWorkerPool._handleDeltaDeletion(_this.delta);
          };
        })(this));
        return runs((function(_this) {
          return function() {
            expect(DatabaseStore.find).toHaveBeenCalledWith(Thread, 'idhere');
            return expect(DatabaseTransaction.prototype.unpersistModel).not.toHaveBeenCalled();
          };
        })(this));
      });
      return it("should call unpersistModel if the object exists", function() {
        spyOn(DatabaseStore, 'find').andCallFake((function(_this) {
          return function(klass, id) {
            return Promise.resolve(_this.thread);
          };
        })(this));
        waitsForPromise((function(_this) {
          return function() {
            return NylasSyncWorkerPool._handleDeltaDeletion(_this.delta);
          };
        })(this));
        return runs((function(_this) {
          return function() {
            expect(DatabaseStore.find).toHaveBeenCalledWith(Thread, 'idhere');
            return expect(DatabaseTransaction.prototype.unpersistModel).toHaveBeenCalledWith(_this.thread);
          };
        })(this));
      });
    });
    return describe("handleModelResponse", function() {});
  });

}).call(this);
