(function() {
  var ApplicationMenu, BrowserWindow, Menu, MenuItem, Utils, app, _, _ref,
    __slice = [].slice;

  _ref = require('electron'), BrowserWindow = _ref.BrowserWindow, Menu = _ref.Menu, MenuItem = _ref.MenuItem, app = _ref.app;

  _ = require('underscore');

  Utils = require('../flux/models/utils');

  module.exports = ApplicationMenu = (function() {
    function ApplicationMenu(version) {
      this.version = version;
      this.windowTemplates = new WeakMap();
      this.setActiveTemplate(this.getDefaultTemplate());
      global.application.autoUpdateManager.on('state-changed', (function(_this) {
        return function(state) {
          return _this.updateAutoupdateMenuItem(state);
        };
      })(this));
      global.application.config.observe('devMode', (function(_this) {
        return function(state) {
          return _this.updateDevModeItem();
        };
      })(this));
    }

    ApplicationMenu.prototype.update = function(window, template, keystrokesByCommand) {
      this.translateTemplate(template, keystrokesByCommand);
      this.windowTemplates.set(window, template);
      if (window === this.lastFocusedWindow) {
        return this.setActiveTemplate(template);
      }
    };

    ApplicationMenu.prototype.setActiveTemplate = function(template) {
      if (!_.isEqual(template, this.activeTemplate)) {
        this.activeTemplate = template;
        return this.rebuildMenuWithActiveTemplate();
      }
    };

    ApplicationMenu.prototype.rebuildMenuWithActiveTemplate = function() {
      var fullTemplate, _ref1;
      fullTemplate = Utils.deepClone(this.activeTemplate);
      this.extendTemplateWithVersion(fullTemplate);
      this.extendTemplateWithWindowMenu(fullTemplate);
      this.menu = Menu.buildFromTemplate(fullTemplate);
      Menu.setApplicationMenu(this.menu);
      this.updateAutoupdateMenuItem(global.application.autoUpdateManager.getState());
      this.updateFullscreenMenuItem((_ref1 = this.lastFocusedWindow) != null ? _ref1.isFullScreen() : void 0);
      return this.updateDevModeItem();
    };

    ApplicationMenu.prototype.addWindow = function(window) {
      var focusHandler;
      if (this.lastFocusedWindow == null) {
        this.lastFocusedWindow = window;
      }
      focusHandler = (function(_this) {
        return function() {
          var template;
          _this.lastFocusedWindow = window;
          if (template = _this.windowTemplates.get(window)) {
            return _this.setActiveTemplate(template);
          }
        };
      })(this);
      window.on('focus', focusHandler);
      window.on('enter-full-screen', focusHandler);
      window.on('leave-full-screen', focusHandler);
      window.once('closed', (function(_this) {
        return function() {
          if (window === _this.lastFocusedWindow) {
            _this.lastFocusedWindow = null;
          }
          _this.windowTemplates["delete"](window);
          _this.rebuildMenuWithActiveTemplate();
          window.removeListener('focus', focusHandler);
          window.removeListener('enter-full-screen', focusHandler);
          return window.removeListener('leave-full-screen', focusHandler);
        };
      })(this));
      this.rebuildMenuWithActiveTemplate();
      return this.enableWindowSpecificItems(true);
    };

    ApplicationMenu.prototype.flattenMenuItems = function(menu) {
      var index, item, items, _ref1;
      items = [];
      _ref1 = menu.items || {};
      for (index in _ref1) {
        item = _ref1[index];
        items.push(item);
        if (item.submenu) {
          items = items.concat(this.flattenMenuItems(item.submenu));
        }
      }
      return items;
    };

    ApplicationMenu.prototype.flattenMenuTemplate = function(template) {
      var item, items, _i, _len;
      items = [];
      for (_i = 0, _len = template.length; _i < _len; _i++) {
        item = template[_i];
        items.push(item);
        if (item.submenu) {
          items = items.concat(this.flattenMenuTemplate(item.submenu));
        }
      }
      return items;
    };

    ApplicationMenu.prototype.enableWindowSpecificItems = function(enable) {
      var item, _i, _len, _ref1, _ref2, _results;
      _ref1 = this.flattenMenuItems(this.menu);
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        item = _ref1[_i];
        if ((_ref2 = item.metadata) != null ? _ref2['windowSpecific'] : void 0) {
          _results.push(item.enabled = enable);
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    ApplicationMenu.prototype.extendTemplateWithVersion = function(template) {
      var item;
      if ((item = _.find(this.flattenMenuTemplate(template), function(_arg) {
        var label;
        label = _arg.label;
        return label === 'VERSION';
      }))) {
        return item.label = "Version " + this.version;
      }
    };

    ApplicationMenu.prototype.extendTemplateWithWindowMenu = function(template) {
      var accelerators, idx, windowMenu, windows, windowsItems, _ref1;
      windowMenu = _.find(template, function(_arg) {
        var label;
        label = _arg.label;
        return label === 'Window';
      });
      if (!windowMenu) {
        return;
      }
      idx = _.findIndex(windowMenu.submenu, function(_arg) {
        var id;
        id = _arg.id;
        return id === 'window-list-separator';
      });
      accelerators = {
        'default': 'CmdOrCtrl+0',
        'work': 'CmdOrCtrl+alt+w'
      };
      windows = global.application.windowManager.getOpenWindows();
      windowsItems = windows.map((function(_this) {
        return function(w) {
          return {
            label: w.loadSettings().title || "Window",
            accelerator: accelerators[w.windowType],
            click: function() {
              w.show();
              return w.focus();
            }
          };
        };
      })(this));
      return (_ref1 = windowMenu.submenu).splice.apply(_ref1, [idx, 0, {
        type: 'separator'
      }].concat(__slice.call(windowsItems)));
    };

    ApplicationMenu.prototype.updateAutoupdateMenuItem = function(state) {
      var checkForUpdateItem, downloadingUpdateItem, installUpdateItem;
      checkForUpdateItem = _.find(this.flattenMenuItems(this.menu), function(_arg) {
        var label;
        label = _arg.label;
        return label === 'Check for Update';
      });
      downloadingUpdateItem = _.find(this.flattenMenuItems(this.menu), function(_arg) {
        var label;
        label = _arg.label;
        return label === 'Downloading Update';
      });
      installUpdateItem = _.find(this.flattenMenuItems(this.menu), function(_arg) {
        var label;
        label = _arg.label;
        return label === 'Restart and Install Update';
      });
      if (!((checkForUpdateItem != null) && (downloadingUpdateItem != null) && (installUpdateItem != null))) {
        return;
      }
      checkForUpdateItem.visible = false;
      downloadingUpdateItem.visible = false;
      installUpdateItem.visible = false;
      switch (state) {
        case 'idle':
        case 'error':
        case 'no-update-available':
          return checkForUpdateItem.visible = true;
        case 'checking':
        case 'downloading':
          return downloadingUpdateItem.visible = true;
        case 'update-available':
          return installUpdateItem.visible = true;
      }
    };

    ApplicationMenu.prototype.updateFullscreenMenuItem = function(fullscreen) {
      var enterItem, exitItem;
      enterItem = _.find(this.flattenMenuItems(this.menu), function(_arg) {
        var label;
        label = _arg.label;
        return label === 'Enter Full Screen';
      });
      exitItem = _.find(this.flattenMenuItems(this.menu), function(_arg) {
        var label;
        label = _arg.label;
        return label === 'Exit Full Screen';
      });
      if (!(enterItem && exitItem)) {
        return;
      }
      enterItem.visible = !fullscreen;
      return exitItem.visible = fullscreen;
    };

    ApplicationMenu.prototype.updateDevModeItem = function() {
      var devModeItem;
      devModeItem = _.find(this.flattenMenuItems(this.menu), function(_arg) {
        var command;
        command = _arg.command;
        return command === 'application:toggle-dev';
      });
      return devModeItem != null ? devModeItem.checked = global.application.devMode : void 0;
    };

    ApplicationMenu.prototype.getDefaultTemplate = function() {
      return [
        {
          label: "N1",
          submenu: [
            {
              label: "Check for Update",
              metadata: {
                autoUpdate: true
              }
            }, {
              label: 'Reload',
              accelerator: 'CmdOrCtrl+R',
              click: (function(_this) {
                return function() {
                  var _ref1;
                  return (_ref1 = BrowserWindow.getFocusedWindow()) != null ? _ref1.reload() : void 0;
                };
              })(this)
            }, {
              label: 'Close Window',
              accelerator: 'CmdOrCtrl+Shift+W',
              click: (function(_this) {
                return function() {
                  var _ref1;
                  return (_ref1 = BrowserWindow.getFocusedWindow()) != null ? _ref1.close() : void 0;
                };
              })(this)
            }, {
              label: 'Toggle Dev Tools',
              accelerator: 'CmdOrCtrl+Alt+I',
              click: (function(_this) {
                return function() {
                  var _ref1;
                  return (_ref1 = BrowserWindow.getFocusedWindow()) != null ? _ref1.toggleDevTools() : void 0;
                };
              })(this)
            }, {
              label: 'Quit',
              accelerator: 'CmdOrCtrl+Q',
              click: function() {
                return app.quit();
              }
            }
          ]
        }
      ];
    };

    ApplicationMenu.prototype.translateTemplate = function(template, keystrokesByCommand) {
      template.forEach((function(_this) {
        return function(item) {
          if (item.metadata == null) {
            item.metadata = {};
          }
          if (item.command) {
            item.accelerator = _this.acceleratorForCommand(item.command, keystrokesByCommand);
            item.click = function() {
              return global.application.sendCommand(item.command);
            };
            if (!/^application:/.test(item.command)) {
              item.metadata['windowSpecific'] = true;
            }
          }
          if (item.submenu) {
            return _this.translateTemplate(item.submenu, keystrokesByCommand);
          }
        };
      })(this));
      return template;
    };

    ApplicationMenu.prototype.acceleratorForCommand = function(command, keystrokesByCommand) {
      var firstKeystroke, key, keys, modifiers, _ref1;
      firstKeystroke = (_ref1 = keystrokesByCommand[command]) != null ? _ref1[0] : void 0;
      if (!firstKeystroke) {
        return null;
      }
      if (/f\d+/.test(firstKeystroke)) {
        firstKeystroke = firstKeystroke.toUpperCase();
      }
      modifiers = firstKeystroke.split('+');
      key = modifiers.pop();
      modifiers = modifiers.map(function(modifier) {
        return modifier.replace(/shift/ig, "Shift").replace(/command/ig, "Command").replace(/mod/ig, "CmdOrCtrl").replace(/ctrl/ig, "Ctrl").replace(/alt/ig, "Alt");
      });
      keys = modifiers.concat([key.toUpperCase()]);
      return keys.join("+");
    };

    return ApplicationMenu;

  })();

}).call(this);
