(function() {
  var BrowserWindow, EventEmitter, NylasWindow, WindowIconPath, app, dialog, fs, idNum, path, url, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __slice = [].slice;

  _ref = require('electron'), BrowserWindow = _ref.BrowserWindow, app = _ref.app, dialog = _ref.dialog;

  path = require('path');

  fs = require('fs');

  url = require('url');

  _ = require('underscore');

  EventEmitter = require('events').EventEmitter;

  WindowIconPath = null;

  idNum = 0;

  module.exports = NylasWindow = (function() {
    _.extend(NylasWindow.prototype, EventEmitter.prototype);

    NylasWindow.includeShellLoadTime = true;

    NylasWindow.prototype.browserWindow = null;

    NylasWindow.prototype.loaded = null;

    NylasWindow.prototype.isSpec = null;

    function NylasWindow(settings) {
      var browserWindowOptions, frame, height, loadSettings, pathToOpen, resizable, showSpecsInWindow, title, toolbar, width, _base;
      if (settings == null) {
        settings = {};
      }
      this.updateLoadSettings = __bind(this.updateLoadSettings, this);
      frame = settings.frame, title = settings.title, width = settings.width, height = settings.height, toolbar = settings.toolbar, resizable = settings.resizable, pathToOpen = settings.pathToOpen, showSpecsInWindow = settings.showSpecsInWindow, this.isSpec = settings.isSpec, this.devMode = settings.devMode, this.windowKey = settings.windowKey, this.safeMode = settings.safeMode, this.neverClose = settings.neverClose, this.mainWindow = settings.mainWindow, this.windowType = settings.windowType, this.resourcePath = settings.resourcePath, this.exitWhenDone = settings.exitWhenDone, this.configDirPath = settings.configDirPath;
      if (!this.windowKey) {
        this.windowKey = "" + this.windowType + "-" + idNum;
        idNum += 1;
      }
      if (this.resourcePath) {
        this.resourcePath = path.normalize(this.resourcePath);
      }
      browserWindowOptions = {
        show: false,
        title: title != null ? title : 'Nylas N1',
        frame: frame,
        width: width,
        height: height,
        resizable: resizable,
        webPreferences: {
          directWrite: true
        }
      };
      if (this.neverClose) {
        browserWindowOptions.webPreferences.pageVisibility = true;
      }
      if (process.platform === 'linux') {
        if (!WindowIconPath) {
          WindowIconPath = path.resolve(__dirname, '..', '..', 'nylas.png');
          if (!fs.existsSync(WindowIconPath)) {
            WindowIconPath = path.resolve(__dirname, '..', '..', 'build', 'resources', 'nylas.png');
          }
        }
        browserWindowOptions.icon = WindowIconPath;
      }
      this.browserWindow = new BrowserWindow(browserWindowOptions);
      this.browserWindow.updateLoadSettings = this.updateLoadSettings;
      this.handleEvents();
      loadSettings = _.extend({}, settings);
      if (loadSettings.windowState == null) {
        loadSettings.windowState = '{}';
      }
      loadSettings.appVersion = app.getVersion();
      loadSettings.resourcePath = this.resourcePath;
      if (loadSettings.devMode == null) {
        loadSettings.devMode = false;
      }
      if (loadSettings.safeMode == null) {
        loadSettings.safeMode = false;
      }
      if (loadSettings.mainWindow == null) {
        loadSettings.mainWindow = this.mainWindow;
      }
      if (loadSettings.windowType == null) {
        loadSettings.windowType = "default";
      }
      if (this.constructor.includeShellLoadTime && !this.isSpec) {
        this.constructor.includeShellLoadTime = false;
        if (loadSettings.shellLoadTime == null) {
          loadSettings.shellLoadTime = Date.now() - global.shellStartTime;
        }
      }
      loadSettings.initialPath = pathToOpen;
      if (typeof (_base = fs.statSyncNoException(pathToOpen)).isFile === "function" ? _base.isFile() : void 0) {
        loadSettings.initialPath = path.dirname(pathToOpen);
      }
      this.browserWindow.loadSettings = loadSettings;
      this.browserWindow.once('window:loaded', (function(_this) {
        return function() {
          _this.loaded = true;
          if (_this.mainWindow) {
            _this.browserWindow.setResizable(true);
          }
          if (_this.browserWindow.loadSettingsChangedSinceGetURL) {
            _this.browserWindow.webContents.send('load-settings-changed', _this.browserWindow.loadSettings);
          }
          return _this.emit('window:loaded');
        };
      })(this));
      this.browserWindow.loadURL(this.getURL(loadSettings));
      if (this.isSpec) {
        this.browserWindow.focusOnWebView();
      }
    }

    NylasWindow.prototype.updateLoadSettings = function(newSettings) {
      if (newSettings == null) {
        newSettings = {};
      }
      this.loaded = true;
      return this.setLoadSettings(Object.assign({}, this.browserWindow.loadSettings, newSettings));
    };

    NylasWindow.prototype.loadSettings = function() {
      return this.browserWindow.loadSettings;
    };

    NylasWindow.prototype.setLoadSettings = function(loadSettings) {
      this.browserWindow.loadSettings = loadSettings;
      this.browserWindow.loadSettingsChangedSinceGetURL = true;
      return this.browserWindow.webContents.send('load-settings-changed', loadSettings);
    };

    NylasWindow.prototype.getURL = function(loadSettingsObj) {
      var loadSettings;
      loadSettings = _.clone(loadSettingsObj);
      delete loadSettings['windowState'];
      this.browserWindow.loadSettingsChangedSinceGetURL = false;
      return url.format({
        protocol: 'file',
        pathname: "" + this.resourcePath + "/static/index.html",
        slashes: true,
        query: {
          loadSettings: JSON.stringify(loadSettings)
        }
      });
    };

    NylasWindow.prototype.handleEvents = function() {
      this.browserWindow.on('close', (function(_this) {
        return function(event) {
          if (_this.neverClose && !global.application.isQuitting()) {
            event.preventDefault();
            if (_this.browserWindow.isFullScreen()) {
              _this.browserWindow.once('leave-full-screen', function() {
                return _this.browserWindow.hide();
              });
              _this.browserWindow.setFullScreen(false);
            } else {
              _this.browserWindow.hide();
            }
            if (!_this.isSpec) {
              return global.application.windowManager.quitWinLinuxIfNoWindows();
            }
          }
        };
      })(this));
      this.browserWindow.on('scroll-touch-begin', (function(_this) {
        return function() {
          return _this.browserWindow.webContents.send('scroll-touch-begin');
        };
      })(this));
      this.browserWindow.on('scroll-touch-end', (function(_this) {
        return function() {
          return _this.browserWindow.webContents.send('scroll-touch-end');
        };
      })(this));
      this.browserWindow.on('focus', (function(_this) {
        return function() {
          return _this.browserWindow.webContents.send('browser-window-focus');
        };
      })(this));
      this.browserWindow.on('blur', (function(_this) {
        return function() {
          return _this.browserWindow.webContents.send('browser-window-blur');
        };
      })(this));
      this.browserWindow.webContents.on('will-navigate', (function(_this) {
        return function(event, url) {
          return event.preventDefault();
        };
      })(this));
      this.browserWindow.webContents.on('new-window', (function(_this) {
        return function(event, url, frameName, disposition) {
          return event.preventDefault();
        };
      })(this));
      this.browserWindow.on('unresponsive', (function(_this) {
        return function() {
          var chosen;
          if (_this.isSpec) {
            return;
          }
          if (!_this.loaded) {
            return;
          }
          chosen = dialog.showMessageBox(_this.browserWindow, {
            type: 'warning',
            buttons: ['Close', 'Keep Waiting'],
            message: 'N1 is not responding',
            detail: 'Would you like to force close it or keep waiting?'
          });
          if (chosen === 0) {
            return _this.browserWindow.destroy();
          }
        };
      })(this));
      this.browserWindow.webContents.on('crashed', (function(_this) {
        return function() {
          var chosen;
          if (_this.exitWhenDone) {
            global.application.exit(100);
          }
          if (_this.neverClose) {
            return _this.browserWindow.reload();
          } else {
            chosen = dialog.showMessageBox(_this.browserWindow, {
              type: 'warning',
              buttons: ['Close Window', 'Reload', 'Keep It Open'],
              message: 'N1 has crashed',
              detail: 'Please report this issue to us at support@nylas.com.'
            });
            switch (chosen) {
              case 0:
                return _this.browserWindow.destroy();
              case 1:
                return _this.browserWindow.reload();
            }
          }
        };
      })(this));
      if (this.isSpec) {
        if (process.platform === 'win32') {
          this.browserWindow.on('close', (function(_this) {
            return function() {
              return _this.isWindowClosing = true;
            };
          })(this));
        }
        return this.browserWindow.on('blur', (function(_this) {
          return function() {
            if (!_this.isWindowClosing) {
              return _this.browserWindow.focusOnWebView();
            }
          };
        })(this));
      }
    };

    NylasWindow.prototype.sendMessage = function(message, detail) {
      return this.waitForLoad((function(_this) {
        return function() {
          return _this.browserWindow.webContents.send(message, detail);
        };
      })(this));
    };

    NylasWindow.prototype.sendCommand = function() {
      var args, command;
      command = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      if (this.isSpecWindow()) {
        if (!global.application.sendCommandToFirstResponder(command)) {
          switch (command) {
            case 'window:reload':
              return this.reload();
            case 'window:toggle-dev-tools':
              return this.toggleDevTools();
            case 'window:close':
              return this.close();
          }
        }
      } else if (this.isWebViewFocused()) {
        return this.sendCommandToBrowserWindow.apply(this, [command].concat(__slice.call(args)));
      } else {
        if (!global.application.sendCommandToFirstResponder(command)) {
          return this.sendCommandToBrowserWindow.apply(this, [command].concat(__slice.call(args)));
        }
      }
    };

    NylasWindow.prototype.sendCommandToBrowserWindow = function() {
      var args, command, _ref1;
      command = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      return (_ref1 = this.browserWindow.webContents).send.apply(_ref1, ['command', command].concat(__slice.call(args)));
    };

    NylasWindow.prototype.getDimensions = function() {
      var height, width, x, y, _ref1, _ref2;
      _ref1 = this.browserWindow.getPosition(), x = _ref1[0], y = _ref1[1];
      _ref2 = this.browserWindow.getSize(), width = _ref2[0], height = _ref2[1];
      return {
        x: x,
        y: y,
        width: width,
        height: height
      };
    };

    NylasWindow.prototype.close = function() {
      return this.browserWindow.close();
    };

    NylasWindow.prototype.hide = function() {
      return this.browserWindow.hide();
    };

    NylasWindow.prototype.show = function() {
      return this.browserWindow.show();
    };

    NylasWindow.prototype.showWhenLoaded = function() {
      return this.waitForLoad((function(_this) {
        return function() {
          _this.show();
          return _this.focus();
        };
      })(this));
    };

    NylasWindow.prototype.waitForLoad = function(fn) {
      if (this.loaded) {
        return fn();
      } else {
        return this.once('window:loaded', fn);
      }
    };

    NylasWindow.prototype.focus = function() {
      return this.browserWindow.focus();
    };

    NylasWindow.prototype.minimize = function() {
      return this.browserWindow.minimize();
    };

    NylasWindow.prototype.maximize = function() {
      return this.browserWindow.maximize();
    };

    NylasWindow.prototype.restore = function() {
      return this.browserWindow.restore();
    };

    NylasWindow.prototype.isFocused = function() {
      return this.browserWindow.isFocused();
    };

    NylasWindow.prototype.isMinimized = function() {
      return this.browserWindow.isMinimized();
    };

    NylasWindow.prototype.isVisible = function() {
      return this.browserWindow.isVisible();
    };

    NylasWindow.prototype.isLoaded = function() {
      return this.loaded;
    };

    NylasWindow.prototype.isWebViewFocused = function() {
      return this.browserWindow.isWebViewFocused();
    };

    NylasWindow.prototype.isSpecWindow = function() {
      return this.isSpec;
    };

    NylasWindow.prototype.reload = function() {
      return this.browserWindow.reload();
    };

    NylasWindow.prototype.toggleDevTools = function() {
      return this.browserWindow.toggleDevTools();
    };

    return NylasWindow;

  })();

}).call(this);
