(function() {
  var EventEmitter, WindowsUpdater, WindowsUpdaterSquirrelAdapter, _;

  EventEmitter = require('events').EventEmitter;

  _ = require('underscore');

  WindowsUpdater = require('./windows-updater');

  WindowsUpdaterSquirrelAdapter = (function() {
    function WindowsUpdaterSquirrelAdapter() {}

    _.extend(WindowsUpdaterSquirrelAdapter.prototype, EventEmitter.prototype);

    WindowsUpdaterSquirrelAdapter.prototype.setFeedURL = function(updateUrl) {
      this.updateUrl = updateUrl;
    };

    WindowsUpdaterSquirrelAdapter.prototype.restartN1 = function() {
      if (WindowsUpdater.existsSync()) {
        return WindowsUpdater.restartN1(require('electron').app);
      } else {
        return NylasEnv.reportError(new Error("SquirrellUpdate does not exist"));
      }
    };

    WindowsUpdaterSquirrelAdapter.prototype.downloadUpdate = function(callback) {
      return WindowsUpdater.spawn(['--download', this.updateUrl], function(error, stdout) {
        var json, update, _ref, _ref1;
        if (error != null) {
          return callback(error);
        }
        try {
          json = stdout.trim().split('\n').pop();
          update = (_ref = JSON.parse(json)) != null ? (_ref1 = _ref.releasesToApply) != null ? typeof _ref1.pop === "function" ? _ref1.pop() : void 0 : void 0 : void 0;
        } catch (_error) {
          error = _error;
          error.stdout = stdout;
          return callback(error);
        }
        return callback(null, update);
      });
    };

    WindowsUpdaterSquirrelAdapter.prototype.installUpdate = function(callback) {
      return WindowsUpdater.spawn(['--update', this.updateUrl], callback);
    };

    WindowsUpdaterSquirrelAdapter.prototype.supportsUpdates = function() {
      return WindowsUpdater.existsSync();
    };

    WindowsUpdaterSquirrelAdapter.prototype.downloadAndInstallUpdate = function() {
      if (!this.updateUrl) {
        throw new Error('Update URL is not set');
      }
      this.emit('checking-for-update');
      if (!WindowsUpdater.existsSync()) {
        this.emit('update-not-available');
        return;
      }
      return this.downloadUpdate((function(_this) {
        return function(error, update) {
          if (error != null) {
            _this.emit('update-not-available');
            return;
          }
          if (update == null) {
            _this.emit('update-not-available');
            return;
          }
          _this.emit('update-available');
          return _this.installUpdate(function(error) {
            if (error != null) {
              _this.emit('error', error);
              return;
            }
            return _this.emit('update-downloaded', {}, update.releaseNotes, update.version);
          });
        };
      })(this));
    };

    return WindowsUpdaterSquirrelAdapter;

  })();

  module.exports = new WindowsUpdaterSquirrelAdapter();

}).call(this);
