(function() {
  var ChildProcess, addCommandsToPath, appFolder, binFolder, createShortcuts, environmentKeyPath, exeName, fs, getPath, isAscii, path, regPath, removeCommandsFromPath, removeShortcuts, rootN1Folder, setxPath, spawn, spawnReg, spawnSetx, spawnUpdate, system32Path, updateDotExe, updateShortcuts;

  ChildProcess = require('child_process');

  fs = require('fs-plus');

  path = require('path');

  appFolder = path.resolve(process.execPath, '..');

  rootN1Folder = path.resolve(appFolder, '..');

  binFolder = path.join(rootN1Folder, 'bin');

  updateDotExe = path.join(rootN1Folder, 'Update.exe');

  exeName = path.basename(process.execPath);

  if (process.env.SystemRoot) {
    system32Path = path.join(process.env.SystemRoot, 'System32');
    regPath = path.join(system32Path, 'reg.exe');
    setxPath = path.join(system32Path, 'setx.exe');
  } else {
    regPath = 'reg.exe';
    setxPath = 'setx.exe';
  }

  environmentKeyPath = 'HKCU\\Environment';

  spawn = function(command, args, callback) {
    var error, spawnedProcess, stdout;
    stdout = '';
    try {
      spawnedProcess = ChildProcess.spawn(command, args);
    } catch (_error) {
      error = _error;
      process.nextTick(function() {
        return typeof callback === "function" ? callback(error, stdout) : void 0;
      });
      return;
    }
    spawnedProcess.stdout.on('data', function(data) {
      return stdout += data;
    });
    error = null;
    spawnedProcess.on('error', function(processError) {
      return error != null ? error : error = processError;
    });
    return spawnedProcess.on('close', function(code, signal) {
      if (code !== 0) {
        if (error == null) {
          error = new Error("Command failed: " + (signal != null ? signal : code));
        }
      }
      if (error != null) {
        if (error.code == null) {
          error.code = code;
        }
      }
      if (error != null) {
        if (error.stdout == null) {
          error.stdout = stdout;
        }
      }
      return typeof callback === "function" ? callback(error, stdout) : void 0;
    });
  };

  spawnReg = function(args, callback) {
    return spawn(regPath, args, callback);
  };

  spawnSetx = function(args, callback) {
    return spawn(setxPath, args, callback);
  };

  spawnUpdate = function(args, callback) {
    return spawn(updateDotExe, args, callback);
  };

  isAscii = function(text) {
    var index;
    index = 0;
    while (index < text.length) {
      if (text.charCodeAt(index) > 127) {
        return false;
      }
      index++;
    }
    return true;
  };

  getPath = function(callback) {
    return spawnReg(['query', environmentKeyPath, '/v', 'Path'], function(error, stdout) {
      var lines, pathEnv, segments, _ref;
      if (error != null) {
        if (error.code === 1) {
          if (stdout.indexOf('ERROR: Registry editing has been disabled by your administrator.') !== -1) {
            return callback(error);
          }
          return callback(null, '');
        } else {
          return callback(error);
        }
      }
      lines = stdout.split(/[\r\n]+/).filter(function(line) {
        return line;
      });
      segments = (_ref = lines[lines.length - 1]) != null ? _ref.split('    ') : void 0;
      if (segments[1] === 'Path' && segments.length >= 3) {
        pathEnv = segments != null ? segments.slice(3).join('    ') : void 0;
        if (isAscii(pathEnv)) {
          return callback(null, pathEnv);
        } else {
          return callback(new Error('PATH contains non-ASCII values'));
        }
      } else {
        return callback(new Error('Registry query for PATH failed'));
      }
    });
  };

  addCommandsToPath = function(callback) {
    var addBinToPath, installCommands;
    installCommands = function(callback) {
      var nylasCommand, nylasCommandPath, nylasShCommand, nylasShCommandPath, relativeN1Path, relativeN1ShPath;
      nylasCommandPath = path.join(binFolder, 'N1.cmd');
      relativeN1Path = path.relative(binFolder, path.join(appFolder, 'resources', 'cli', 'N1.cmd'));
      nylasCommand = "@echo off\r\n\"%~dp0\\" + relativeN1Path + "\" %*";
      nylasShCommandPath = path.join(binFolder, 'N1');
      relativeN1ShPath = path.relative(binFolder, path.join(appFolder, 'resources', 'cli', 'N1.sh'));
      nylasShCommand = "#!/bin/sh\r\n\"$(dirname \"$0\")/" + (relativeN1ShPath.replace(/\\/g, '/')) + "\" \"$@\"";
      return fs.writeFile(nylasCommandPath, nylasCommand, function() {
        return fs.writeFile(nylasShCommandPath, nylasShCommand, function() {
          return callback();
        });
      });
    };
    addBinToPath = function(pathSegments, callback) {
      var newPathEnv;
      pathSegments.push(binFolder);
      newPathEnv = pathSegments.join(';');
      return spawnSetx(['Path', newPathEnv], callback);
    };
    return installCommands(function(error) {
      if (error != null) {
        return callback(error);
      }
      return getPath(function(error, pathEnv) {
        var pathSegments;
        if (error != null) {
          return callback(error);
        }
        pathSegments = pathEnv.split(/;+/).filter(function(pathSegment) {
          return pathSegment;
        });
        if (pathSegments.indexOf(binFolder) === -1) {
          return addBinToPath(pathSegments, callback);
        } else {
          return callback();
        }
      });
    });
  };

  removeCommandsFromPath = function(callback) {
    return getPath(function(error, pathEnv) {
      var newPathEnv, pathSegments;
      if (error != null) {
        return callback(error);
      }
      pathSegments = pathEnv.split(/;+/).filter(function(pathSegment) {
        return pathSegment && pathSegment !== binFolder;
      });
      newPathEnv = pathSegments.join(';');
      if (pathEnv !== newPathEnv) {
        return spawnSetx(['Path', newPathEnv], callback);
      } else {
        return callback();
      }
    });
  };

  createShortcuts = function(callback) {
    return spawnUpdate(['--createShortcut', exeName], callback);
  };

  updateShortcuts = function(callback) {
    var desktopShortcutPath, homeDirectory;
    if (homeDirectory = fs.getHomeDirectory()) {
      desktopShortcutPath = path.join(homeDirectory, 'Desktop', 'N1.lnk');
      return fs.exists(desktopShortcutPath, function(desktopShortcutExists) {
        return createShortcuts(function() {
          if (desktopShortcutExists) {
            return callback();
          } else {
            return fs.unlink(desktopShortcutPath, callback);
          }
        });
      });
    } else {
      return createShortcuts(callback);
    }
  };

  removeShortcuts = function(callback) {
    return spawnUpdate(['--removeShortcut', exeName], callback);
  };

  exports.spawn = spawnUpdate;

  exports.existsSync = function() {
    return fs.existsSync(updateDotExe);
  };

  exports.restartN1 = function(app) {
    var args, projectPath, _ref, _ref1;
    if (projectPath = (_ref = global.application) != null ? (_ref1 = _ref.lastFocusedWindow) != null ? _ref1.projectPath : void 0 : void 0) {
      args = [projectPath];
    }
    app.once('will-quit', function() {
      return spawn(path.join(binFolder, 'N1.cmd'), args);
    });
    return app.quit();
  };

  exports.handleStartupEvent = function(app, squirrelCommand) {
    switch (squirrelCommand) {
      case '--squirrel-install':
        createShortcuts(function() {
          return addCommandsToPath(function(error) {
            if (error) {
              console.error(error);
            }
            return app.quit();
          });
        });
        return true;
      case '--squirrel-updated':
        updateShortcuts(function() {
          return addCommandsToPath(function(error) {
            if (error) {
              console.error(error);
            }
            return app.quit();
          });
        });
        return true;
      case '--squirrel-uninstall':
        removeShortcuts(function() {
          return removeCommandsFromPath(function(error) {
            if (error) {
              console.error(error);
            }
            return app.quit();
          });
        });
        return true;
      case '--squirrel-obsolete':
        app.quit();
        return true;
      default:
        return false;
    }
  };

}).call(this);
