(function() {
  var CommandRegistry, CompositeDisposable, Disposable, Emitter, ipcRenderer, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ref = require('event-kit'), Emitter = _ref.Emitter, Disposable = _ref.Disposable, CompositeDisposable = _ref.CompositeDisposable;

  ipcRenderer = require('electron').ipcRenderer;

  module.exports = CommandRegistry = (function() {
    function CommandRegistry() {
      this.flushChangesSoon = __bind(this.flushChangesSoon, this);
      this.emitter = new Emitter;
      this.listenerCounts = {};
      this.listenerCountChanges = {};
    }

    CommandRegistry.prototype.add = function(target, commandName, callback) {
      var commands, disposable;
      if (typeof commandName === 'object') {
        commands = commandName;
        disposable = new CompositeDisposable;
        for (commandName in commands) {
          callback = commands[commandName];
          disposable.add(this.add(target, commandName, callback));
        }
        return disposable;
      }
      if (typeof callback !== 'function') {
        throw new Error("Can't register a command with non-function callback.");
      }
      if (typeof target === 'string') {
        throw new Error("Commands can no longer be registered to CSS selectors. Consider using KeyCommandRegion instead.");
      }
      target.addEventListener(commandName, callback);
      this.listenerCountChanges[commandName] = (this.listenerCountChanges[commandName] || 0) + 1;
      this.flushChangesSoon();
      return new Disposable((function(_this) {
        return function() {
          target.removeEventListener(commandName, callback);
          _this.listenerCountChanges[commandName] = (_this.listenerCountChanges[commandName] || 0) - 1;
          return _this.flushChangesSoon();
        };
      })(this));
    };

    CommandRegistry.prototype.listenerCountForCommand = function(commandName) {
      return (this.listenerCounts[commandName] || 0) + (this.listenerCountChanges[commandName] || 0);
    };

    CommandRegistry.prototype.dispatch = function(commandName, detail) {
      var event;
      event = new CustomEvent(commandName, {
        bubbles: true,
        detail: detail
      });
      return document.activeElement.dispatchEvent(event);
    };

    CommandRegistry.prototype.flushChangesSoon = function() {
      if (this.pendingEmit) {
        return;
      }
      this.pendingEmit = true;
      return setTimeout((function(_this) {
        return function() {
          var changed, commandName, val, _ref1;
          _this.pendingEmit = false;
          changed = false;
          _ref1 = _this.listenerCountChanges;
          for (commandName in _ref1) {
            val = _ref1[commandName];
            _this.listenerCounts[commandName] = (_this.listenerCounts[commandName] || 0) + val;
            if (val !== 0) {
              changed = true;
            }
          }
          _this.listenerCountChanges = {};
          if (changed) {
            return _this.emitter.emit('commands-changed');
          }
        };
      })(this), 100);
    };

    CommandRegistry.prototype.onRegistedCommandsChanged = function(callback) {
      return this.emitter.on('commands-changed', callback);
    };

    return CommandRegistry;

  })();

}).call(this);
