(function() {
  var ClipboardService, ContenteditableService, InlineStyleTransformer, SanitizeTransformer, Utils, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __slice = [].slice;

  ContenteditableService = require('./contenteditable-service');

  _ref = require('nylas-exports'), InlineStyleTransformer = _ref.InlineStyleTransformer, SanitizeTransformer = _ref.SanitizeTransformer, Utils = _ref.Utils;

  ClipboardService = (function(_super) {
    __extends(ClipboardService, _super);

    function ClipboardService() {
      var args;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      this._sanitizeHTMLInput = __bind(this._sanitizeHTMLInput, this);
      this._getBestRepresentation = __bind(this._getBestRepresentation, this);
      this.onPaste = __bind(this.onPaste, this);
      ClipboardService.__super__.constructor.apply(this, args);
      this.onFilePaste = this.props.onFilePaste;
    }

    ClipboardService.prototype.setData = function() {
      var args;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      ClipboardService.__super__.setData.apply(this, args);
      return this.onFilePaste = this.props.onFilePaste;
    };

    ClipboardService.prototype.eventHandlers = function() {
      return {
        onPaste: this.onPaste
      };
    };

    ClipboardService.prototype.onPaste = function(event) {
      var blob, ext, fs, input, item, mimetype, path, reader, temp, _ref1, _ref2;
      if (event.clipboardData.items.length === 0) {
        return;
      }
      event.preventDefault();
      item = event.clipboardData.items[0];
      if (item.kind === 'file') {
        blob = item.getAsFile();
        ext = (_ref1 = {
          'image/png': '.png',
          'image/jpg': '.jpg',
          'image/tiff': '.tiff'
        }[item.type]) != null ? _ref1 : '';
        temp = require('temp');
        path = require('path');
        fs = require('fs');
        reader = new FileReader();
        reader.addEventListener('loadend', (function(_this) {
          return function() {
            var buffer, tmpFolder, tmpPath;
            buffer = new Buffer(new Uint8Array(reader.result));
            tmpFolder = temp.path('-nylas-attachment');
            tmpPath = path.join(tmpFolder, "Pasted File" + ext);
            return fs.mkdir(tmpFolder, function() {
              return fs.writeFile(tmpPath, buffer, function(err) {
                return typeof _this.onFilePaste === "function" ? _this.onFilePaste(tmpPath) : void 0;
              });
            });
          };
        })(this));
        reader.readAsArrayBuffer(blob);
      } else {
        _ref2 = this._getBestRepresentation(event.clipboardData), input = _ref2.input, mimetype = _ref2.mimetype;
        if (mimetype === 'text/plain') {
          input = Utils.encodeHTMLEntities(input);
          input = input.replace(/[\r\n]|&#1[03];/g, "<br/>").replace(/\s\s/g, " &nbsp;");
          document.execCommand("insertHTML", false, input);
        } else if (mimetype === 'text/html') {
          this._sanitizeHTMLInput(input).then(function(cleanHtml) {
            return document.execCommand("insertHTML", false, cleanHtml);
          });
        } else {

        }
      }
    };

    ClipboardService.prototype._getBestRepresentation = function(clipboardData) {
      var data, mimetype, _i, _len, _ref1, _ref2;
      _ref1 = ["text/html", "text/plain"];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        mimetype = _ref1[_i];
        data = (_ref2 = clipboardData.getData(mimetype)) != null ? _ref2 : "";
        if (data.length > 0) {
          return {
            input: data,
            mimetype: mimetype
          };
        }
      }
      return {
        input: null,
        mimetype: null
      };
    };

    ClipboardService.prototype._sanitizeHTMLInput = function(input) {
      return InlineStyleTransformer.run(input).then((function(_this) {
        return function(input) {
          return SanitizeTransformer.run(input, SanitizeTransformer.Preset.Permissive).then(function(input) {
            input = input.replace(/(<br\s*\/?>\s*){3,}/g, "<br/><br/>");
            input = input.replace(/^(<br ?\/>)+/, '');
            input = input.replace(/(<br ?\/>)+$/, '');
            return Promise.resolve(input);
          });
        };
      })(this));
    };

    return ClipboardService;

  })(ContenteditableService);

  module.exports = ClipboardService;

}).call(this);
