(function() {
  var BlockquoteManager, ClipboardService, Contenteditable, DOMNormalizer, DOMUtils, EditorAPI, EmphasisFormattingExtension, ExtendedSelection, FloatingToolbar, KeyCommandsRegion, LinkManager, ListManager, MouseService, ParagraphFormattingExtension, React, ReactDOM, TabManager, ToolbarButtonManager, Utils, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  React = require('react');

  ReactDOM = require('react-dom');

  _ref = require('nylas-exports'), Utils = _ref.Utils, DOMUtils = _ref.DOMUtils;

  KeyCommandsRegion = require('nylas-component-kit').KeyCommandsRegion;

  FloatingToolbar = require('./floating-toolbar');

  EditorAPI = require('./editor-api');

  ExtendedSelection = require('./extended-selection');

  TabManager = require('./tab-manager');

  LinkManager = require('./link-manager');

  ListManager = require('./list-manager');

  MouseService = require('./mouse-service');

  DOMNormalizer = require('./dom-normalizer');

  ClipboardService = require('./clipboard-service');

  BlockquoteManager = require('./blockquote-manager');

  ToolbarButtonManager = require('./toolbar-button-manager');

  EmphasisFormattingExtension = require('./emphasis-formatting-extension');

  ParagraphFormattingExtension = require('./paragraph-formatting-extension');


  /*
  Public: A modern React-compatible contenteditable
  
  This <Contenteditable /> component is fully React-compatible and behaves
  like a standard controlled input.
  
  ```javascript
  getInitialState: function() {
    return {value: '<strong>Hello!</strong>'};
  },
  handleChange: function(event) {
    this.setState({value: event.target.value});
  },
  render: function() {
    var value = this.state.value;
    return <Contenteditable type="text" value={value} onChange={this.handleChange} />;
  }
  ```
   */

  Contenteditable = (function(_super) {
    __extends(Contenteditable, _super);

    Contenteditable.displayName = "Contenteditable";

    Contenteditable.IgnoreMutationClassName = 'ignore-mutations';

    Contenteditable.propTypes = {
      value: React.PropTypes.string,
      onChange: React.PropTypes.func.isRequired,
      onFilePaste: React.PropTypes.func,
      extensions: React.PropTypes.array,
      spellcheck: React.PropTypes.bool,
      floatingToolbar: React.PropTypes.bool
    };

    Contenteditable.defaultProps = {
      extensions: [],
      spellcheck: true,
      floatingToolbar: true,
      onSelectionRestored: function() {}
    };

    Contenteditable.prototype.coreServices = [MouseService, ClipboardService];

    Contenteditable.prototype.coreExtensions = [ToolbarButtonManager, ListManager, TabManager, EmphasisFormattingExtension, ParagraphFormattingExtension, LinkManager, BlockquoteManager, DOMNormalizer];


    /* Public: perform an editing operation on the Contenteditable
    
    - `editingFunction` A function to mutate the DOM and
    {ExtendedSelection}. It gets passed an {EditorAPI} object that contains
    mutating methods.
    
    If the current selection at the time of running the extension is out of
    scope, it will be set to the last saved state. This ensures extensions
    operate on a valid {ExtendedSelection}.
    
    Edits made within the editing function will eventually fire _onDOMMutated
     */

    Contenteditable.prototype.atomicEdit = function(editingFunction, extraArgsObj) {
      var argsObj, editor, error;
      if (extraArgsObj == null) {
        extraArgsObj = {};
      }
      this._teardownNonMutationListeners();
      editor = new EditorAPI(this._editableNode());
      if (!editor.currentSelection().isInScope() && extraArgsObj.methodName !== 'onBlur') {
        this._restoreSelection();
      }
      argsObj = _.extend(extraArgsObj, {
        editor: editor
      });
      try {
        editingFunction(argsObj);
      } catch (_error) {
        error = _error;
        NylasEnv.reportError(error);
      }
      return this._setupNonMutationListeners();
    };

    Contenteditable.prototype.focus = function() {
      return this._editableNode().focus();
    };

    Contenteditable.prototype.setSelection = function(selection) {
      this.setInnerState({
        exportedSelection: selection,
        previousExportedSelection: this.innerState.exportedSelection
      });
      return this._restoreSelection();
    };

    function Contenteditable(props) {
      this.props = props;
      this._notifyOfSelectionRestoration = __bind(this._notifyOfSelectionRestoration, this);
      this._restoreSelection = __bind(this._restoreSelection, this);
      this._saveSelection = __bind(this._saveSelection, this);
      this.getPreviousSelection = __bind(this.getPreviousSelection, this);
      this.getCurrentSelection = __bind(this.getCurrentSelection, this);
      this._runExtensionMethod = __bind(this._runExtensionMethod, this);
      this.dispatchEventToExtensions = __bind(this.dispatchEventToExtensions, this);
      this._runCallbackOnExtensions = __bind(this._runCallbackOnExtensions, this);
      this._onShowContextMenu = __bind(this._onShowContextMenu, this);
      this._onCompositionEnd = __bind(this._onCompositionEnd, this);
      this._onCompositionStart = __bind(this._onCompositionStart, this);
      this._shouldAcceptDrop = __bind(this._shouldAcceptDrop, this);
      this._onDrop = __bind(this._onDrop, this);
      this._onDragOver = __bind(this._onDragOver, this);
      this._onKeyDown = __bind(this._onKeyDown, this);
      this._onFocus = __bind(this._onFocus, this);
      this._onBlur = __bind(this._onBlur, this);
      this._onDOMMutated = __bind(this._onDOMMutated, this);
      this._teardownEditingActionListeners = __bind(this._teardownEditingActionListeners, this);
      this._setupEditingActionListeners = __bind(this._setupEditingActionListeners, this);
      this._teardownNonMutationListeners = __bind(this._teardownNonMutationListeners, this);
      this._setupNonMutationListeners = __bind(this._setupNonMutationListeners, this);
      this._eventHandlers = __bind(this._eventHandlers, this);
      this._editableNode = __bind(this._editableNode, this);
      this.render = __bind(this.render, this);
      this.setInnerState = __bind(this.setInnerState, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentDidUpdate = __bind(this.componentDidUpdate, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.componentWillMount = __bind(this.componentWillMount, this);
      this.setSelection = __bind(this.setSelection, this);
      this.focus = __bind(this.focus, this);
      this.atomicEdit = __bind(this.atomicEdit, this);
      this.state = {};
      this.innerState = {
        dragging: false,
        doubleDown: false,
        hoveringOver: false,
        editableNode: null,
        exportedSelection: null,
        previousExportedSelection: null
      };
      this._mutationObserver = new MutationObserver(this._onDOMMutated);
    }

    Contenteditable.prototype.componentWillMount = function() {
      return this._setupServices();
    };

    Contenteditable.prototype.componentDidMount = function() {
      this.setInnerState({
        editableNode: this._editableNode()
      });
      this._setupNonMutationListeners();
      this._setupEditingActionListeners();
      return this._mutationObserver.observe(this._editableNode(), this._mutationConfig());
    };

    Contenteditable.prototype.shouldComponentUpdate = function(nextProps, nextState) {
      return !this._inCompositionEvent && (!Utils.isEqualReact(nextProps, this.props) || !Utils.isEqualReact(nextState, this.state));
    };

    Contenteditable.prototype.componentDidUpdate = function() {
      if (this._shouldRestoreSelectionOnUpdate()) {
        this._restoreSelection();
        this._notifyOfSelectionRestoration();
      }
      this._refreshServices();
      this._setupEditingActionListeners();
      this._mutationObserver.disconnect();
      this._mutationObserver.observe(this._editableNode(), this._mutationConfig());
      return this.setInnerState({
        editableNode: this._editableNode()
      });
    };

    Contenteditable.prototype.componentWillUnmount = function() {
      this._mutationObserver.disconnect();
      this._teardownNonMutationListeners();
      this._teardownEditingActionListeners();
      return this._teardownServices();
    };

    Contenteditable.prototype.setInnerState = function(innerState) {
      var _ref1;
      if (innerState == null) {
        innerState = {};
      }
      if (_.isMatch(this.innerState, innerState)) {
        return;
      }
      this.innerState = _.extend(this.innerState, innerState);
      if (this._broadcastInnerStateToToolbar) {
        if ((_ref1 = this.refs["toolbarController"]) != null) {
          _ref1.componentWillReceiveInnerProps(this.innerState);
        }
      }
      return this._refreshServices();
    };

    Contenteditable.prototype._setupServices = function() {
      return this._services = this.coreServices.map((function(_this) {
        return function(Service) {
          return new Service({
            data: {
              props: _this.props,
              state: _this.state,
              innerState: _this.innerState
            },
            methods: {
              setInnerState: _this.setInnerState,
              dispatchEventToExtensions: _this.dispatchEventToExtensions
            }
          });
        };
      })(this));
    };

    Contenteditable.prototype._refreshServices = function() {
      var service, _i, _len, _ref1, _results;
      _ref1 = this._services;
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        service = _ref1[_i];
        _results.push(service.setData({
          props: this.props,
          state: this.state,
          innerState: this.innerState
        }));
      }
      return _results;
    };

    Contenteditable.prototype._teardownServices = function() {
      var service, _i, _len, _ref1, _results;
      _ref1 = this._services;
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        service = _ref1[_i];
        _results.push(service.teardown());
      }
      return _results;
    };

    Contenteditable.prototype.render = function() {
      return React.createElement(KeyCommandsRegion, {
        "className": "contenteditable-container",
        "localHandlers": this._keymapHandlers()
      }, this._renderFloatingToolbar(), React.createElement("div", React.__spread({
        "className": "contenteditable no-open-link-events",
        "ref": "contenteditable",
        "contentEditable": true,
        "spellCheck": false,
        "placeholder": this.props.placeholder,
        "dangerouslySetInnerHTML": {
          __html: this.props.value
        }
      }, this._eventHandlers())));
    };

    Contenteditable.prototype._renderFloatingToolbar = function() {
      if (!this.props.floatingToolbar) {
        return;
      }
      return React.createElement(FloatingToolbar, {
        "ref": "toolbarController",
        "atomicEdit": this.atomicEdit,
        "extensions": this._extensions()
      });
    };

    Contenteditable.prototype._editableNode = function() {
      return ReactDOM.findDOMNode(this.refs.contenteditable);
    };

    Contenteditable.prototype._eventHandlers = function() {
      var handlers, service, _i, _len, _ref1;
      handlers = {};
      _ref1 = this._services;
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        service = _ref1[_i];
        _.extend(handlers, service.eventHandlers());
      }
      handlers = _.extend(handlers, {
        onBlur: this._onBlur,
        onFocus: this._onFocus,
        onKeyDown: this._onKeyDown,
        onCompositionEnd: this._onCompositionEnd,
        onCompositionStart: this._onCompositionStart
      });
      return handlers;
    };

    Contenteditable.prototype._boundExtensionKeymapHandlers = function() {
      var keymapHandlers;
      keymapHandlers = {};
      this._extensions().forEach((function(_this) {
        return function(extension) {
          var error, extensionHandlers;
          if (!_.isFunction(extension.keyCommandHandlers)) {
            return;
          }
          try {
            extensionHandlers = extension.keyCommandHandlers.call(extension);
            return _.each(extensionHandlers, function(handler, command) {
              return keymapHandlers[command] = function(event) {
                return _this.atomicEdit(handler, {
                  event: event
                });
              };
            });
          } catch (_error) {
            error = _error;
            return NylasEnv.reportError(error);
          }
        };
      })(this));
      return keymapHandlers;
    };

    Contenteditable.prototype._keymapHandlers = function() {
      var defaultKeymaps;
      defaultKeymaps = {};
      return _.extend(defaultKeymaps, this._boundExtensionKeymapHandlers());
    };

    Contenteditable.prototype._setupNonMutationListeners = function() {
      this._broadcastInnerStateToToolbar = true;
      document.addEventListener("selectionchange", this._saveSelection);
      return this._editableNode().addEventListener('contextmenu', this._onShowContextMenu);
    };

    Contenteditable.prototype._teardownNonMutationListeners = function() {
      this._broadcastInnerStateToToolbar = false;
      document.removeEventListener("selectionchange", this._saveSelection);
      return this._editableNode().removeEventListener('contextmenu', this._onShowContextMenu);
    };

    Contenteditable.prototype._setupEditingActionListeners = function() {
      var editingActionUnsubscriber, _i, _len, _ref1, _ref2;
      if (((_ref1 = this.editingActionUnsubscribers) != null ? _ref1.length : void 0) > 0) {
        _ref2 = this.editingActionUnsubscribers;
        for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
          editingActionUnsubscriber = _ref2[_i];
          editingActionUnsubscriber();
        }
      }
      this.editingActionUnsubscribers = [];
      return this._extensions().forEach((function(_this) {
        return function(ext) {
          var editingActions, error, _ref3;
          try {
            editingActions = (_ref3 = typeof ext.editingActions === "function" ? ext.editingActions() : void 0) != null ? _ref3 : [];
            return editingActions.forEach(function(_arg) {
              var action, callback;
              action = _arg.action, callback = _arg.callback;
              return _this.editingActionUnsubscribers.push(action.listen(function(actionArg) {
                return _this.atomicEdit(callback, {
                  actionArg: actionArg
                });
              }));
            });
          } catch (_error) {
            error = _error;
            return NylasEnv.reportError(error);
          }
        };
      })(this));
    };

    Contenteditable.prototype._teardownEditingActionListeners = function() {
      var editingActionUnsubscriber, _i, _len, _ref1, _results;
      _ref1 = this.editingActionUnsubscribers;
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        editingActionUnsubscriber = _ref1[_i];
        _results.push(editingActionUnsubscriber());
      }
      return _results;
    };

    Contenteditable.prototype._mutationConfig = function() {
      return {
        subtree: true,
        childList: true,
        attributes: true,
        characterData: true,
        attributeOldValue: true,
        characterDataOldValue: true
      };
    };

    Contenteditable.prototype._onDOMMutated = function(mutations) {
      var mutation, runCallbacks, _i, _len, _ref1, _ref2;
      if (!(mutations && mutations.length > 0)) {
        return;
      }
      runCallbacks = true;
      for (_i = 0, _len = mutations.length; _i < _len; _i++) {
        mutation = mutations[_i];
        if ((_ref1 = mutation.target) != null ? (_ref2 = _ref1.className) != null ? _ref2.includes(Contenteditable.IgnoreMutationClassName) : void 0 : void 0) {
          runCallbacks = false;
        }
      }
      this._mutationObserver.disconnect();
      if (this.innerState.dragging) {
        this.setInnerState({
          dragging: false
        });
      }
      if (this.innerState.doubleDown) {
        this.setInnerState({
          doubleDown: false
        });
      }
      this._broadcastInnerStateToToolbar = false;
      if (runCallbacks) {
        this._runCallbackOnExtensions("onContentChanged", {
          mutations: mutations
        });
      }
      this._saveSelection();
      this.props.onChange({
        target: {
          value: this._editableNode().innerHTML
        }
      });
      this._broadcastInnerStateToToolbar = true;
      this._mutationObserver.observe(this._editableNode(), this._mutationConfig());
    };

    Contenteditable.prototype._onBlur = function(event) {
      this.setInnerState({
        dragging: false
      });
      if (this._editableNode().parentElement.contains(event.relatedTarget)) {
        return;
      }
      return this.dispatchEventToExtensions("onBlur", event);
    };

    Contenteditable.prototype._onFocus = function(event) {
      return this.dispatchEventToExtensions("onFocus", event);
    };

    Contenteditable.prototype._onKeyDown = function(event) {
      return this.dispatchEventToExtensions("onKeyDown", event);
    };

    Contenteditable.prototype._onDragOver = function(event) {
      return this.dispatchEventToExtensions("onDragOver", event);
    };

    Contenteditable.prototype._onDrop = function(event) {
      return this.dispatchEventToExtensions("onDrop", event);
    };

    Contenteditable.prototype._shouldAcceptDrop = function(event) {
      var extension, _i, _len, _ref1, _ref2;
      _ref1 = [].concat(this.props.extensions, this.coreExtensions);
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        extension = _ref1[_i];
        if ((_ref2 = extension.shouldAcceptDrop) != null ? _ref2.call(extension, {
          event: event
        }) : void 0) {
          return true;
        }
      }
      return false;
    };

    Contenteditable.prototype._onCompositionStart = function() {
      this._inCompositionEvent = true;
      return this._teardownNonMutationListeners();
    };

    Contenteditable.prototype._onCompositionEnd = function() {
      this._inCompositionEvent = false;
      return this._setupNonMutationListeners();
    };

    Contenteditable.prototype._onShowContextMenu = function(event) {
      var Menu, MenuItem, menu, remote, _ref1;
      if ((_ref1 = this.refs["toolbarController"]) != null) {
        _ref1.forceClose();
      }
      event.preventDefault();
      remote = require('electron').remote;
      Menu = remote.Menu, MenuItem = remote.MenuItem;
      menu = new Menu();
      this.dispatchEventToExtensions("onShowContextMenu", event, {
        menu: menu
      });
      menu.append(new MenuItem({
        label: 'Cut',
        role: 'cut'
      }));
      menu.append(new MenuItem({
        label: 'Copy',
        role: 'copy'
      }));
      menu.append(new MenuItem({
        label: 'Paste',
        role: 'paste'
      }));
      menu.append(new MenuItem({
        label: 'Paste and Match Style',
        click: (function(_this) {
          return function() {
            return NylasEnv.getCurrentWindow().webContents.pasteAndMatchStyle();
          };
        })(this)
      }));
      return menu.popup(remote.getCurrentWindow());
    };

    Contenteditable.prototype._extensions = function() {
      return this.props.extensions.concat(this.coreExtensions);
    };

    Contenteditable.prototype._runCallbackOnExtensions = function(method, argsObj) {
      var extension, _i, _len, _ref1, _results;
      if (argsObj == null) {
        argsObj = {};
      }
      _ref1 = this._extensions();
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        extension = _ref1[_i];
        _results.push(this._runExtensionMethod(extension, method, argsObj));
      }
      return _results;
    };

    Contenteditable.prototype.dispatchEventToExtensions = function(method, event, args) {
      var argsObj, extension, _i, _j, _len, _len1, _ref1, _ref2, _results;
      if (args == null) {
        args = {};
      }
      argsObj = _.extend(args, {
        event: event
      });
      _ref1 = this.props.extensions;
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        extension = _ref1[_i];
        if (event != null ? event.isPropagationStopped() : void 0) {
          break;
        }
        this._runExtensionMethod(extension, method, argsObj);
      }
      if ((event != null ? event.defaultPrevented : void 0) || (event != null ? event.isPropagationStopped() : void 0)) {
        return;
      }
      _ref2 = this.coreExtensions;
      _results = [];
      for (_j = 0, _len1 = _ref2.length; _j < _len1; _j++) {
        extension = _ref2[_j];
        if (event != null ? event.isPropagationStopped() : void 0) {
          break;
        }
        _results.push(this._runExtensionMethod(extension, method, argsObj));
      }
      return _results;
    };

    Contenteditable.prototype._runExtensionMethod = function(extension, method, argsObj) {
      var editingFunction;
      if (argsObj == null) {
        argsObj = {};
      }
      if (this._inCompositionEvent) {
        return;
      }
      if (extension[method] == null) {
        return;
      }
      editingFunction = extension[method].bind(extension);
      argsObj = _.extend(argsObj, {
        methodName: method
      });
      return this.atomicEdit(editingFunction, argsObj);
    };

    Contenteditable.prototype.getCurrentSelection = function() {
      return this.innerState.exportedSelection;
    };

    Contenteditable.prototype.getPreviousSelection = function() {
      return this.innerState.previousExportedSelection;
    };

    Contenteditable.prototype._saveSelection = function() {
      var selection;
      selection = new ExtendedSelection(this._editableNode());
      if (!(selection != null ? selection.isInScope() : void 0)) {
        return;
      }
      return this.setInnerState({
        selectionSnapshot: selection.selectionSnapshot(),
        exportedSelection: selection.exportSelection(),
        previousExportedSelection: this.innerState.exportedSelection
      });
    };

    Contenteditable.prototype._restoreSelection = function() {
      var selection;
      this._teardownNonMutationListeners();
      selection = new ExtendedSelection(this._editableNode());
      selection.importSelection(this.innerState.exportedSelection);
      if (selection.isInScope()) {
        this.setInnerState({
          editableNode: this._editableNode()
        });
      }
      return this._setupNonMutationListeners();
    };

    Contenteditable.prototype._notifyOfSelectionRestoration = function() {
      var selection;
      selection = new ExtendedSelection(this._editableNode());
      if (selection.isInScope()) {
        return this.props.onSelectionRestored(selection, this._editableNode());
      }
    };

    Contenteditable.prototype._shouldRestoreSelectionOnUpdate = function() {
      return !this.innerState.dragging && document.activeElement === this._editableNode();
    };

    return Contenteditable;

  })(React.Component);

  module.exports = Contenteditable;

}).call(this);
