(function() {
  var DOMUtils, EditorAPI, ExtendedSelection, OverlaidComponents, React, _,
    __slice = [].slice;

  _ = require('underscore');

  DOMUtils = require('nylas-exports').DOMUtils;

  React = require('react');

  ExtendedSelection = require('./extended-selection');

  OverlaidComponents = null;

  EditorAPI = (function() {
    function EditorAPI(rootNode) {
      this.rootNode = rootNode;
      this._extendedSelection = new ExtendedSelection(this.rootNode);
    }

    EditorAPI.prototype.wrapSelection = function(nodeName) {
      var wrapped;
      wrapped = DOMUtils.wrap(this._extendedSelection.getRangeAt(0), nodeName);
      this.select(wrapped);
      return this;
    };

    EditorAPI.prototype.unwrapNodeAndSelectAll = function(node) {
      var first, last, replacedNodes;
      replacedNodes = DOMUtils.unwrapNode(node);
      if (replacedNodes.length === 0) {
        return this;
      }
      first = DOMUtils.findFirstTextNode(replacedNodes[0]);
      last = DOMUtils.findLastTextNode(_.last(replacedNodes));
      this._extendedSelection.selectFromTo(first, last);
      return this;
    };

    EditorAPI.prototype.regExpSelectorAll = function(regex) {
      return DOMUtils.regExpSelectorAll(this.rootNode, regex);
    };

    EditorAPI.prototype.currentSelection = function() {
      return this._extendedSelection;
    };

    EditorAPI.prototype.whilePreservingSelection = function(fn) {
      var sel;
      if (!(document.activeElement === this.rootNode || this.rootNode.contains(document.activeElement))) {
        return fn();
      }
      sel = this.currentSelection().exportSelection();
      fn();
      return this.select(sel);
    };

    EditorAPI.prototype.getSelectionTextIndex = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this._extendedSelection).getSelectionTextIndex.apply(_ref, args);
    };

    EditorAPI.prototype.importSelection = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      (_ref = this._extendedSelection).importSelection.apply(_ref, args);
      return this;
    };

    EditorAPI.prototype.select = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      (_ref = this._extendedSelection).select.apply(_ref, args);
      return this;
    };

    EditorAPI.prototype.selectAllChildren = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      (_ref = this._extendedSelection).selectAllChildren.apply(_ref, args);
      return this;
    };

    EditorAPI.prototype.restoreSelectionByTextIndex = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      (_ref = this._extendedSelection).restoreSelectionByTextIndex.apply(_ref, args);
      return this;
    };

    EditorAPI.prototype.normalize = function() {
      this.rootNode.normalize();
      return this;
    };

    EditorAPI.prototype.insertCustomComponent = function(componentKey, props) {
      var anchorId, anchorTag, _ref;
      if (props == null) {
        props = {};
      }
      if (OverlaidComponents == null) {
        OverlaidComponents = require('../overlaid-components/overlaid-components')["default"];
      }
      _ref = OverlaidComponents.buildAnchorTag(componentKey, props), anchorId = _ref.anchorId, anchorTag = _ref.anchorTag;
      this.insertHTML(anchorTag);
      return anchorId;
    };

    EditorAPI.prototype.removeCustomComponentByAnchorId = function(anchorId) {
      var node;
      if (!anchorId) {
        return;
      }
      node = this.rootNode.querySelector("img[data-overlay-id=\"" + anchorId + "\"]");
      return node != null ? node.parentNode.removeChild(node) : void 0;
    };

    EditorAPI.prototype.backColor = function(color) {
      return this._ec("backColor", false, color);
    };

    EditorAPI.prototype.bold = function() {
      return this._ec("bold", false);
    };

    EditorAPI.prototype.copy = function() {
      return this._ec("copy", false);
    };

    EditorAPI.prototype.createLink = function(uri) {
      return this._ec("createLink", false, uri);
    };

    EditorAPI.prototype.cut = function() {
      return this._ec("cut", false);
    };

    EditorAPI.prototype.decreaseFontSize = function() {
      return this._ec("decreaseFontSize", false);
    };

    EditorAPI.prototype["delete"] = function() {
      return this._ec("delete", false);
    };

    EditorAPI.prototype.fontName = function(fontName) {
      return this._ec("fontName", false, fontName);
    };

    EditorAPI.prototype.fontSize = function(fontSize) {
      return this._ec("fontSize", false, fontSize);
    };

    EditorAPI.prototype.foreColor = function(color) {
      return this._ec("foreColor", false, color);
    };

    EditorAPI.prototype.formatBlock = function(tagName) {
      return this._ec("formatBlock", false, tagName);
    };

    EditorAPI.prototype.forwardDelete = function() {
      return this._ec("forwardDelete", false);
    };

    EditorAPI.prototype.heading = function(tagName) {
      return this._ec("heading", false, tagName);
    };

    EditorAPI.prototype.hiliteColor = function(color) {
      return this._ec("hiliteColor", false, color);
    };

    EditorAPI.prototype.increaseFontSize = function() {
      return this._ec("increaseFontSize", false);
    };

    EditorAPI.prototype.indent = function() {
      return this._ec("indent", false);
    };

    EditorAPI.prototype.insertHorizontalRule = function() {
      return this._ec("insertHorizontalRule", false);
    };

    EditorAPI.prototype.insertHTML = function(html, _arg) {
      var selectInsertion, wrap, wrappedHtml;
      selectInsertion = (_arg != null ? _arg : {}).selectInsertion;
      if (selectInsertion) {
        wrappedHtml = "<span id=\"tmp-html-insertion-wrap\">" + html + "</span>";
        this._ec("insertHTML", false, wrappedHtml);
        wrap = this.rootNode.querySelector("#tmp-html-insertion-wrap");
        this.unwrapNodeAndSelectAll(wrap);
        return this;
      } else {
        return this._ec("insertHTML", false, html);
      }
    };

    EditorAPI.prototype.insertImage = function(uri) {
      return this._ec("insertImage", false, uri);
    };

    EditorAPI.prototype.insertOrderedList = function() {
      return this._ec("insertOrderedList", false);
    };

    EditorAPI.prototype.insertUnorderedList = function() {
      return this._ec("insertUnorderedList", false);
    };

    EditorAPI.prototype.insertParagraph = function() {
      return this._ec("insertParagraph", false);
    };

    EditorAPI.prototype.insertText = function(text) {
      return this._ec("insertText", false, text);
    };

    EditorAPI.prototype.italic = function() {
      return this._ec("italic", false);
    };

    EditorAPI.prototype.justifyCenter = function() {
      return this._ec("justifyCenter", false);
    };

    EditorAPI.prototype.justifyFull = function() {
      return this._ec("justifyFull", false);
    };

    EditorAPI.prototype.justifyLeft = function() {
      return this._ec("justifyLeft", false);
    };

    EditorAPI.prototype.justifyRight = function() {
      return this._ec("justifyRight", false);
    };

    EditorAPI.prototype.outdent = function() {
      return this._ec("outdent", false);
    };

    EditorAPI.prototype.paste = function() {
      return this._ec("paste", false);
    };

    EditorAPI.prototype.redo = function() {
      return this._ec("redo", false);
    };

    EditorAPI.prototype.removeFormat = function() {
      return this._ec("removeFormat", false);
    };

    EditorAPI.prototype.selectAll = function() {
      return this._ec("selectAll", false);
    };

    EditorAPI.prototype.strikeThrough = function() {
      return this._ec("strikeThrough", false);
    };

    EditorAPI.prototype.subscript = function() {
      return this._ec("subscript", false);
    };

    EditorAPI.prototype.superscript = function() {
      return this._ec("superscript", false);
    };

    EditorAPI.prototype.underline = function() {
      return this._ec("underline", false);
    };

    EditorAPI.prototype.undo = function() {
      return this._ec("undo", false);
    };

    EditorAPI.prototype.unlink = function() {
      return this._ec("unlink", false);
    };

    EditorAPI.prototype.styleWithCSS = function(style) {
      return this._ec("styleWithCSS", false, style);
    };

    EditorAPI.prototype.contentReadOnly = function() {
      return this._notImplemented();
    };

    EditorAPI.prototype.enableInlineTableEditing = function() {
      return this._notImplemented();
    };

    EditorAPI.prototype.enableObjectResizing = function() {
      return this._notImplemented();
    };

    EditorAPI.prototype.insertBrOnReturn = function() {
      return this._notImplemented();
    };

    EditorAPI.prototype.useCSS = function() {
      return this._notImplemented();
    };

    EditorAPI.prototype._ec = function() {
      var args;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      document.execCommand.apply(document, args);
      return this;
    };

    EditorAPI.prototype._notImplemented = function() {
      throw new Error("Not implemented");
    };

    return EditorAPI;

  })();

  module.exports = EditorAPI;

}).call(this);
