(function() {
  var DOMUtils, ExportedSelection, ExtendedSelection, _,
    __slice = [].slice;

  _ = require('underscore');

  DOMUtils = require('nylas-exports').DOMUtils;

  ExportedSelection = require('./exported-selection');

  ExtendedSelection = (function() {
    function ExtendedSelection(scopeNode) {
      this.scopeNode = scopeNode;
      if (this.scopeNode == null) {
        this.scopeNode = document.body;
      }
      this.rawSelection = document.getSelection();
    }

    ExtendedSelection.prototype.isInScope = function() {
      return (this.anchorNode != null) && (this.focusNode != null) && (this.anchorOffset != null) && (this.focusOffset != null) && this.scopeNode.contains(this.anchorNode) && this.scopeNode.contains(this.focusNode);
    };

    ExtendedSelection.prototype.select = function() {
      var args;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      if (args.length === 0) {
        throw this._errBadUsage();
      } else if (args.length === 1) {
        if (args[0] instanceof ExportedSelection) {
          this.importSelection(args[0]);
        } else if (args[0] instanceof Range) {
          this.selectRange(args[0]);
        } else {
          this.selectAt(args[0]);
        }
      } else if (args.length === 2) {
        this.selectFromTo.apply(this, args);
      } else if (args.length === 3) {
        throw this._errBadUsage();
      } else if (args.length === 4) {
        this.selectFromToWithIndex.apply(this, args);
      } else if (args.length >= 5) {
        throw this._errBadUsage();
      }
      return this;
    };

    ExtendedSelection.prototype.selectAt = function(at) {
      var nodeAt, range;
      nodeAt = this.findNodeAt(at);
      range = new Range();
      range.selectNode(nodeAt);
      this.rawSelection.removeAllRanges();
      this.rawSelection.addRange(range);
      return this;
    };

    ExtendedSelection.prototype.selectRange = function(range) {
      return this.setBaseAndExtent(range.startContainer, range.startOffset, range.endContainer, range.endOffset);
    };

    ExtendedSelection.prototype.selectFromTo = function(from, to) {
      var fromNode, toNode, _ref;
      fromNode = this.findNodeAt(from);
      toNode = this.findNodeAt(to);
      if (!(fromNode && toNode)) {
        return;
      }
      return this.setBaseAndExtent(fromNode, 0, toNode, (_ref = toNode.length) != null ? _ref : 0);
    };

    ExtendedSelection.prototype.selectFromToWithIndex = function(from, fromIndex, to, toIndex) {
      var fromNode, toNode;
      fromNode = this.findNodeAt(from);
      toNode = this.findNodeAt(to);
      if ((!_.isNumber(fromIndex)) || (!_.isNumber(toIndex))) {
        throw this._errBadUsage();
      }
      if (!(fromNode && toNode)) {
        return;
      }
      return this.setBaseAndExtent(fromNode, fromIndex, toNode, toIndex);
    };

    ExtendedSelection.prototype.exportSelection = function() {
      return new ExportedSelection(this.rawSelection, this.scopeNode);
    };

    ExtendedSelection.prototype.selectionSnapshot = function() {
      return {
        anchorNode: this.rawSelection.anchorNode,
        anchorOffset: this.rawSelection.anchorOffset,
        focusNode: this.rawSelection.focusNode,
        focusOffset: this.rawSelection.focusOffset,
        isCollapsed: this.rawSelection.isCollapsed
      };
    };

    ExtendedSelection.prototype.importSelection = function(exportedSelection) {
      var newAnchorNode, newFocusNode;
      if (!(exportedSelection instanceof ExportedSelection)) {
        return;
      }
      newAnchorNode = DOMUtils.findSimilarNodeAtIndex(this.scopeNode, exportedSelection.anchorNode, exportedSelection.anchorNodeIndex);
      newFocusNode = DOMUtils.findSimilarNodeAtIndex(this.scopeNode, exportedSelection.focusNode, exportedSelection.focusNodeIndex);
      return this.setBaseAndExtent(newAnchorNode, exportedSelection.anchorOffset, newFocusNode, exportedSelection.focusOffset);
    };

    ExtendedSelection.prototype.findNodeAt = function(arg) {
      var node;
      node = null;
      if (arg instanceof Node) {
        node = arg;
      } else if (_.isString(arg)) {
        node = this.scopeNode.querySelector(arg);
      } else if (_.isRegExp(arg)) {
        node = DOMUtils.findNodeByRegex(this.scopeNode, arg);
      }
      return node;
    };

    ExtendedSelection.prototype.getSelectionTextIndex = function(refRangeOrNode) {
      var end, endIndex, node, parentNode, range, rangeOffset, sel, start, startIndex, _i, _len, _ref, _ref1;
      if (!DOMUtils.selectionStartsOrEndsIn(refRangeOrNode)) {
        return null;
      }
      sel = this.rawSelection;
      if (!sel) {
        return null;
      }
      startIndex = null;
      endIndex = null;
      range = null;
      rangeOffset = 0;
      if (refRangeOrNode instanceof Range) {
        range = refRangeOrNode;
        parentNode = range.commonAncestorContainer;
      } else {
        parentNode = refRangeOrNode;
      }
      if (parentNode === sel.anchorNode) {
        if (range) {
          rangeOffset = range.startOffset;
        }
        startIndex = sel.anchorOffset - rangeOffset;
      }
      if (parentNode === sel.focusNode) {
        if (range) {
          rangeOffset = range.startOffset;
        }
        endIndex = sel.focusOffset - rangeOffset;
      }
      if (parentNode === sel.anchorNode && parentNode === sel.focusNode) {
        return {
          startIndex: startIndex,
          endIndex: endIndex
        };
      }
      _ref = DOMUtils.getIndexedTextContent(parentNode);
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        _ref1 = _ref[_i], node = _ref1.node, start = _ref1.start, end = _ref1.end;
        if ((range != null ? range.startContainer : void 0) === node) {
          rangeOffset = start + range.startOffset;
        }
        if (sel.anchorNode === node) {
          startIndex = start + sel.anchorOffset - rangeOffset;
        }
        if (sel.focusNode === node) {
          endIndex = start + sel.focusOffset - rangeOffset;
        }
      }
      return {
        startIndex: startIndex,
        endIndex: endIndex
      };
    };

    ExtendedSelection.prototype.restoreSelectionByTextIndex = function(refRangeOrNode, startIndex, endIndex) {
      var atEnd, end, endNode, endOffset, i, inRange, items, node, parentNode, range, rangeOffset, sel, start, startNode, startOffset, _i, _len, _ref;
      startNode = null;
      startOffset = null;
      endNode = null;
      endOffset = null;
      range = null;
      sel = this.rawSelection;
      if (refRangeOrNode instanceof Range) {
        range = refRangeOrNode;
        parentNode = range.commonAncestorContainer;
      } else {
        parentNode = refRangeOrNode;
      }
      if (parentNode.childNodes.length === 0) {
        sel.setBaseAndExtent(parentNode, startIndex, parentNode, endIndex);
      }
      inRange = range === null;
      items = DOMUtils.getIndexedTextContent(parentNode);
      for (i = _i = 0, _len = items.length; _i < _len; i = ++_i) {
        _ref = items[i], node = _ref.node, start = _ref.start, end = _ref.end;
        inRange = inRange || (range.startContainer === node);
        atEnd = i === (items.length - 1);
        if (!inRange) {
          continue;
        }
        if ((range != null ? range.startContainer : void 0) === node) {
          rangeOffset = start + range.startIndex;
          if (startIndex != null) {
            startIndex += rangeOffset;
          }
          if (endIndex != null) {
            endIndex += rangeOffset;
          }
        }
        if ((startIndex != null) && startIndex >= start && (startIndex < end || atEnd && startIndex === end)) {
          startNode = node;
          startOffset = startIndex - start;
        }
        if ((endIndex != null) && endIndex >= start && (endIndex < end || atEnd && endIndex === end)) {
          endNode = node;
          endOffset = endIndex - start;
        }
      }
      return sel.setBaseAndExtent(startNode != null ? startNode : sel.anchorNode, startOffset != null ? startOffset : sel.anchorOffset, endNode != null ? endNode : sel.focusNode, endOffset != null ? endOffset : sel.focusOffset);
    };

    Object.defineProperty(ExtendedSelection.prototype, "anchorNode", {
      get: function() {
        return this.rawSelection.anchorNode;
      },
      set: function() {
        throw this._errNoSet("anchorNode");
      }
    });

    Object.defineProperty(ExtendedSelection.prototype, "anchorOffset", {
      get: function() {
        return this.rawSelection.anchorOffset;
      },
      set: function() {
        throw this._errNoSet("anchorOffset");
      }
    });

    Object.defineProperty(ExtendedSelection.prototype, "focusNode", {
      get: function() {
        return this.rawSelection.focusNode;
      },
      set: function() {
        throw this._errNoSet("focusNode");
      }
    });

    Object.defineProperty(ExtendedSelection.prototype, "focusOffset", {
      get: function() {
        return this.rawSelection.focusOffset;
      },
      set: function() {
        throw this._errNoSet("focusOffset");
      }
    });

    Object.defineProperty(ExtendedSelection.prototype, "isCollapsed", {
      get: function() {
        return this.rawSelection.isCollapsed;
      },
      set: function() {
        throw this._errNoSet("isCollapsed");
      }
    });

    Object.defineProperty(ExtendedSelection.prototype, "rangeCount", {
      get: function() {
        return this.rawSelection.rangeCount;
      },
      set: function() {
        throw this._errNoSet("rangeCount");
      }
    });

    ExtendedSelection.prototype.setBaseAndExtent = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this.rawSelection).setBaseAndExtent.apply(_ref, args);
    };

    ExtendedSelection.prototype.getRangeAt = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this.rawSelection).getRangeAt.apply(_ref, args);
    };

    ExtendedSelection.prototype.collapse = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this.rawSelection).collapse.apply(_ref, args);
    };

    ExtendedSelection.prototype.extend = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this.rawSelection).extend.apply(_ref, args);
    };

    ExtendedSelection.prototype.modify = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this.rawSelection).modify.apply(_ref, args);
    };

    ExtendedSelection.prototype.collapseToStart = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this.rawSelection).collapseToStart.apply(_ref, args);
    };

    ExtendedSelection.prototype.collapseToEnd = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this.rawSelection).collapseToEnd.apply(_ref, args);
    };

    ExtendedSelection.prototype.selectAllChildren = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this.rawSelection).selectAllChildren.apply(_ref, args);
    };

    ExtendedSelection.prototype.addRange = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this.rawSelection).addRange.apply(_ref, args);
    };

    ExtendedSelection.prototype.removeRange = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this.rawSelection).removeRange.apply(_ref, args);
    };

    ExtendedSelection.prototype.removeAllRanges = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this.rawSelection).removeAllRanges.apply(_ref, args);
    };

    ExtendedSelection.prototype.deleteFromDocument = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this.rawSelection).deleteFromDocument.apply(_ref, args);
    };

    ExtendedSelection.prototype.toString = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this.rawSelection).toString.apply(_ref, args);
    };

    ExtendedSelection.prototype.containsNode = function() {
      var args, _ref;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref = this.rawSelection).containsNode.apply(_ref, args);
    };

    ExtendedSelection.prototype._errBadUsage = function() {
      return new Error("Invalid arguments");
    };

    ExtendedSelection.prototype._errNoSet = function(property) {
      return new Error("Can't set " + property);
    };

    return ExtendedSelection;

  })();

  module.exports = ExtendedSelection;

}).call(this);
