(function() {
  var DOMUtils, ExtensionRegistry, FloatingToolbar, React, Utils, classNames, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  classNames = require('classnames');

  React = require('react');

  _ref = require('nylas-exports'), Utils = _ref.Utils, DOMUtils = _ref.DOMUtils, ExtensionRegistry = _ref.ExtensionRegistry;

  FloatingToolbar = (function(_super) {
    __extends(FloatingToolbar, _super);

    FloatingToolbar.displayName = "FloatingToolbar";

    FloatingToolbar.propTypes = {
      atomicEdit: React.PropTypes.func,
      extensions: React.PropTypes.array
    };

    FloatingToolbar.innerPropTypes = {
      dragging: React.PropTypes.bool,
      selection: React.PropTypes.object,
      doubleDown: React.PropTypes.bool,
      hoveringOver: React.PropTypes.object,
      editableNode: React.PropTypes.object
    };

    FloatingToolbar.defaultProps = {
      extensions: []
    };

    FloatingToolbar.defaultInnerProps = {
      dragging: false,
      selection: null,
      doubleDown: false,
      hoveringOver: null,
      editableNode: null
    };

    function FloatingToolbar(props) {
      this.props = props;
      this._renderPointer = __bind(this._renderPointer, this);
      this._toolbarPointerStyles = __bind(this._toolbarPointerStyles, this);
      this._toolbarLeft = __bind(this._toolbarLeft, this);
      this._toolbarStyles = __bind(this._toolbarStyles, this);
      this._toolbarClasses = __bind(this._toolbarClasses, this);
      this._calculatePositionState = __bind(this._calculatePositionState, this);
      this.componentWillReceiveProps = __bind(this.componentWillReceiveProps, this);
      this.componentWillReceiveInnerProps = __bind(this.componentWillReceiveInnerProps, this);
      this.state = {
        toolbarTop: 0,
        toolbarMode: "buttons",
        toolbarLeft: 0,
        toolbarPos: "above",
        editAreaWidth: 9999,
        toolbarWidth: 0,
        toolbarHeight: 0,
        toolbarComponent: null,
        toolbarLocationRef: null,
        toolbarComponentProps: {},
        hidePointer: false
      };
      this.innerProps = FloatingToolbar.defaultInnerProps;
    }

    FloatingToolbar.prototype.shouldComponentUpdate = function(nextProps, nextState) {
      return !Utils.isEqualReact(nextProps, this.props) || !Utils.isEqualReact(nextState, this.state);
    };

    FloatingToolbar.prototype.componentWillReceiveInnerProps = function(nextInnerProps) {
      var fullProps;
      if (nextInnerProps == null) {
        nextInnerProps = {};
      }
      fullProps = _.extend({}, this.props, nextInnerProps);
      this.innerProps = _.extend(this.innerProps, nextInnerProps);
      return this.setState(this._getStateFromProps(fullProps));
    };

    FloatingToolbar.prototype.componentWillReceiveProps = function(nextProps) {
      var fullProps;
      fullProps = _.extend(this.innerProps, nextProps);
      return this.setState(this._getStateFromProps(fullProps));
    };

    FloatingToolbar.prototype.forceClose = function() {
      return this.setState({
        toolbarVisible: false
      });
    };

    FloatingToolbar.prototype.render = function() {
      var ToolbarComponent;
      ToolbarComponent = this.state.toolbarComponent;
      if (!ToolbarComponent) {
        return false;
      }
      return React.createElement("div", {
        "className": "floating-toolbar-container"
      }, React.createElement("div", {
        "ref": "floatingToolbar",
        "className": this._toolbarClasses(),
        "style": this._toolbarStyles()
      }, this._renderPointer(), React.createElement(ToolbarComponent, React.__spread({}, this.state.toolbarComponentProps))));
    };

    FloatingToolbar.prototype._getStateFromProps = function(props) {
      var positionState, toolbarComponentState;
      toolbarComponentState = this._getToolbarComponentData(props);
      if (toolbarComponentState.toolbarLocationRef) {
        positionState = this._calculatePositionState(props, toolbarComponentState);
      } else {
        positionState = {};
      }
      return _.extend({}, toolbarComponentState, positionState);
    };

    FloatingToolbar.prototype._getToolbarComponentData = function(props) {
      var error, extension, hidePointer, params, toolbarComponent, toolbarComponentProps, toolbarHeight, toolbarLocationRef, toolbarWidth, _i, _len, _ref1, _ref2, _ref3;
      toolbarComponent = null;
      toolbarWidth = 0;
      toolbarHeight = 0;
      toolbarLocationRef = null;
      hidePointer = false;
      toolbarComponentProps = {};
      _ref1 = props.extensions;
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        extension = _ref1[_i];
        try {
          params = (_ref2 = typeof extension.toolbarComponentConfig === "function" ? extension.toolbarComponentConfig({
            toolbarState: props
          }) : void 0) != null ? _ref2 : {};
          if (params.component) {
            toolbarComponent = params.component;
            toolbarComponentProps = (_ref3 = params.props) != null ? _ref3 : {};
            toolbarLocationRef = params.locationRefNode;
            toolbarWidth = params.width;
            toolbarHeight = params.height;
            if (params.hidePointer) {
              hidePointer = params.hidePointer;
            }
          }
        } catch (_error) {
          error = _error;
          NylasEnv.reportError(error);
        }
      }
      if (toolbarComponent && !toolbarLocationRef) {
        throw new Error("You must provide a locationRefNode for " + toolbarComponent.displayName + ". It must be either a DOM Element or a Range.");
      }
      return {
        toolbarComponent: toolbarComponent,
        toolbarComponentProps: toolbarComponentProps,
        toolbarLocationRef: toolbarLocationRef,
        toolbarWidth: toolbarWidth,
        toolbarHeight: toolbarHeight,
        hidePointer: hidePointer
      };
    };

    FloatingToolbar.CONTENT_PADDING = 15;

    FloatingToolbar.prototype._calculatePositionState = function(props, _arg) {
      var BORDER_RADIUS_PADDING, TOP_PADDING, calcLeft, calcTop, editArea, editableNode, maxWidth, referenceRect, toolbarHeight, toolbarLocationRef, toolbarPos, toolbarWidth;
      toolbarLocationRef = _arg.toolbarLocationRef, toolbarWidth = _arg.toolbarWidth, toolbarHeight = _arg.toolbarHeight;
      editableNode = props.editableNode;
      if (!_.isFunction(toolbarLocationRef.getBoundingClientRect)) {
        throw new Error("Your locationRefNode must implement getBoundingClientRect. Be aware that Text nodes do not implement this, but Element nodes do. Find the nearest Element relative.");
      }
      referenceRect = toolbarLocationRef.getBoundingClientRect();
      if (!editableNode || !referenceRect || DOMUtils.isEmptyBoundingRect(referenceRect)) {
        return {
          toolbarTop: 0,
          toolbarLeft: 0,
          editAreaWidth: 0,
          toolbarPos: 'above'
        };
      }
      TOP_PADDING = 10;
      BORDER_RADIUS_PADDING = 15;
      editArea = editableNode.getBoundingClientRect();
      calcLeft = (referenceRect.left - editArea.left) + referenceRect.width / 2;
      calcLeft = Math.min(Math.max(calcLeft, FloatingToolbar.CONTENT_PADDING + BORDER_RADIUS_PADDING), editArea.width - BORDER_RADIUS_PADDING);
      calcTop = referenceRect.top - editArea.top - toolbarHeight - 14;
      if (this.state.hidePointer) {
        calcTop += 10;
      }
      toolbarPos = "above";
      if (calcTop < TOP_PADDING) {
        calcTop = referenceRect.top - editArea.top + referenceRect.height + TOP_PADDING + 4;
        if (this.state.hidePointer) {
          calcTop -= 10;
        }
        toolbarPos = "below";
      }
      maxWidth = editArea.width - FloatingToolbar.CONTENT_PADDING * 2;
      return {
        toolbarTop: calcTop,
        toolbarLeft: calcLeft,
        toolbarWidth: Math.min(maxWidth, toolbarWidth),
        toolbarHeight: toolbarHeight,
        editAreaWidth: editArea.width,
        toolbarPos: toolbarPos
      };
    };

    FloatingToolbar.prototype._toolbarClasses = function() {
      var classes;
      classes = {};
      classes[this.state.toolbarPos] = true;
      return classNames(_.extend(classes, {
        "floating-toolbar": true,
        "toolbar": true
      }));
    };

    FloatingToolbar.prototype._toolbarStyles = function() {
      var styles;
      styles = {
        left: this._toolbarLeft(),
        top: this.state.toolbarTop,
        width: this.state.toolbarWidth,
        height: this.state.toolbarHeight
      };
      return styles;
    };

    FloatingToolbar.prototype._toolbarLeft = function() {
      var left, max;
      max = Math.max(this.state.editAreaWidth - this.state.toolbarWidth - FloatingToolbar.CONTENT_PADDING, FloatingToolbar.CONTENT_PADDING);
      left = Math.min(Math.max(this.state.toolbarLeft - this.state.toolbarWidth / 2, FloatingToolbar.CONTENT_PADDING), max);
      return left;
    };

    FloatingToolbar.prototype._toolbarPointerStyles = function() {
      var POINTER_WIDTH, absoluteLeft, left, max, min, relativeLeft, styles;
      POINTER_WIDTH = 6 + 2;
      max = this.state.editAreaWidth - FloatingToolbar.CONTENT_PADDING;
      min = FloatingToolbar.CONTENT_PADDING;
      absoluteLeft = Math.max(Math.min(this.state.toolbarLeft, max), min);
      relativeLeft = absoluteLeft - this._toolbarLeft();
      left = Math.max(Math.min(relativeLeft, this.state.toolbarWidth - POINTER_WIDTH), POINTER_WIDTH);
      styles = {
        left: left
      };
      return styles;
    };

    FloatingToolbar.prototype._renderPointer = function() {
      if (this.state.hidePointer) {
        return false;
      }
      return React.createElement("div", {
        "className": "toolbar-pointer-container",
        "style": this._toolbarPointerStyles()
      }, React.createElement("div", {
        "className": "shadow"
      }), React.createElement("div", {
        "className": "foreground"
      }));
    };

    return FloatingToolbar;

  })(React.Component);

  module.exports = FloatingToolbar;

}).call(this);
