(function() {
  var LinkEditor, React, ReactDOM, RegExpUtils,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  React = require('react');

  ReactDOM = require('react-dom');

  RegExpUtils = require('nylas-exports').RegExpUtils;

  LinkEditor = (function(_super) {
    __extends(LinkEditor, _super);

    LinkEditor.displayName = "LinkEditor";

    LinkEditor.propTypes = {
      onSaveUrl: React.PropTypes.func,
      linkToModify: React.PropTypes.object,
      onDoneWithLink: React.PropTypes.func
    };

    function LinkEditor(props) {
      var _ref;
      this.props = props;
      this._initialUrl = __bind(this._initialUrl, this);
      this._removeUrl = __bind(this._removeUrl, this);
      this._saveUrlOnEnter = __bind(this._saveUrlOnEnter, this);
      this._detectEscape = __bind(this._detectEscape, this);
      this._onInputChange = __bind(this._onInputChange, this);
      this._saveUrl = __bind(this._saveUrl, this);
      this._onBlur = __bind(this._onBlur, this);
      this.render = __bind(this.render, this);
      this.state = {
        urlInputValue: (_ref = this._initialUrl()) != null ? _ref : ""
      };
    }

    LinkEditor.prototype.componentWillReceiveProps = function(newProps) {
      return this.setState({
        urlInputValue: this._initialUrl(newProps)
      });
    };

    LinkEditor.prototype.componentDidMount = function() {
      if (this.props.focusOnMount) {
        return ReactDOM.findDOMNode(this.refs["urlInput"]).focus();
      }
    };

    LinkEditor.prototype.render = function() {
      var checkBtn, removeBtn, widthCorrection;
      widthCorrection = 32;
      checkBtn = "";
      removeBtn = "";
      if (this._initialUrl()) {
        widthCorrection += 32;
        removeBtn = React.createElement("button", {
          "className": "btn btn-icon",
          "ref": "removeBtn",
          "style": {
            float: "right"
          },
          "onMouseDown": this._removeUrl
        }, React.createElement("i", {
          "className": "fa fa-times"
        }));
      }
      if (this.state.urlInputValue.length === 0 || this.state.urlInputValue !== this._initialUrl()) {
        widthCorrection += 32;
        checkBtn = React.createElement("button", {
          "className": "btn btn-icon",
          "style": {
            float: "right"
          },
          "onKeyDown": this._detectEscape,
          "onKeyPress": this._saveUrlOnEnter,
          "onMouseDown": this._saveUrl
        }, React.createElement("i", {
          "className": "fa fa-check"
        }));
      }
      return React.createElement("div", {
        "className": "toolbar-new-link"
      }, React.createElement("i", {
        "className": "fa fa-link preview-btn-icon"
      }), React.createElement("input", {
        "type": "text",
        "ref": "urlInput",
        "style": {
          height: 34,
          width: "calc(100% - " + widthCorrection + "px)"
        },
        "value": this.state.urlInputValue,
        "onBlur": this._onBlur,
        "onKeyDown": this._detectEscape,
        "onKeyPress": this._saveUrlOnEnter,
        "onChange": this._onInputChange,
        "className": "floating-toolbar-input",
        "placeholder": "Paste or type a link"
      }), removeBtn, checkBtn);
    };

    LinkEditor.prototype._onBlur = function(event) {
      var targets, _ref;
      targets = [];
      if (this.refs["saveBtn"]) {
        targets.push(ReactDOM.findDOMNode(this.refs["saveBtn"]));
      }
      if (this.refs["removeBtn"]) {
        targets.push(ReactDOM.findDOMNode(this.refs["removeBtn"]));
      }
      if (_ref = event.relatedTarget, __indexOf.call(targets, _ref) >= 0) {
        event.preventDefault();
      } else {
        return this._saveUrl();
      }
    };

    LinkEditor.prototype._saveUrl = function() {
      if (this.state.urlInputValue.trim().length > 0) {
        this.props.onSaveUrl(this.state.urlInputValue, this.props.linkToModify);
      }
      return this.props.onDoneWithLink();
    };

    LinkEditor.prototype._onInputChange = function(event) {
      return this.setState({
        urlInputValue: event.target.value
      });
    };

    LinkEditor.prototype._detectEscape = function(event) {
      if (event.key === "Escape") {
        return this.props.onDoneWithLink();
      }
    };

    LinkEditor.prototype._saveUrlOnEnter = function(event) {
      if (event.key === "Enter") {
        if (this.state.urlInputValue.trim().length > 0) {
          return this._saveUrl();
        } else {
          return this._removeUrl();
        }
      }
    };

    LinkEditor.prototype._removeUrl = function() {
      this.setState({
        urlInputValue: ""
      });
      this.props.onSaveUrl("", this.props.linkToModify);
      return this.props.onDoneWithLink();
    };

    LinkEditor.prototype._initialUrl = function(props) {
      var initialUrl, textContent, _ref, _ref1, _ref2, _ref3;
      if (props == null) {
        props = this.props;
      }
      initialUrl = (_ref = (_ref1 = props.linkToModify) != null ? _ref1.getAttribute('href') : void 0) != null ? _ref : "";
      if (initialUrl.length === 0) {
        textContent = (_ref2 = (_ref3 = props.linkToModify) != null ? _ref3.textContent : void 0) != null ? _ref2 : "";
        if (RegExpUtils.urlRegex({
          matchEntireString: true
        }).test(textContent)) {
          initialUrl = textContent;
        }
      }
      return initialUrl;
    };

    return LinkEditor;

  })(React.Component);

  module.exports = LinkEditor;

}).call(this);
