(function() {
  var ContenteditableExtension, DOMUtils, LinkEditor, LinkManager, RegExpUtils, _, _ref,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  _ref = require('nylas-exports'), RegExpUtils = _ref.RegExpUtils, DOMUtils = _ref.DOMUtils, ContenteditableExtension = _ref.ContenteditableExtension;

  LinkEditor = require('./link-editor');

  LinkManager = (function(_super) {
    __extends(LinkManager, _super);

    function LinkManager() {
      return LinkManager.__super__.constructor.apply(this, arguments);
    }

    LinkManager.keyCommandHandlers = function() {
      return {
        "contenteditable:insert-link": LinkManager._onInsertLink
      };
    };

    LinkManager.toolbarButtons = function() {
      return [
        {
          className: "btn-link",
          onClick: LinkManager._onInsertLink,
          tooltip: "Edit Link",
          iconUrl: null
        }
      ];
    };

    LinkManager.onContentChanged = function(_arg) {
      var editor, mutations, node, sel, sibling, text;
      editor = _arg.editor, mutations = _arg.mutations;
      sel = editor.currentSelection();
      if (sel.anchorNode && sel.isCollapsed) {
        node = sel.anchorNode;
        sibling = node.previousSibling;
        if (!sibling) {
          return;
        }
        if (sel.anchorOffset > 1) {
          return;
        }
        if (node.nodeType !== Node.TEXT_NODE) {
          return;
        }
        if (sibling.nodeName !== "A") {
          return;
        }
        if (/^\s+/.test(node.data)) {
          return;
        }
        if (RegExpUtils.punctuation({
          exclude: ['\\-', '_']
        }).test(node.data[0])) {
          return;
        }
        if (node.data.length > 1) {
          node.splitText(1);
        }
        sibling.appendChild(node);
        sibling.normalize();
        text = DOMUtils.findLastTextNode(sibling);
        return editor.select(text, text.length, text, text.length);
      }
    };

    LinkManager.toolbarComponentConfig = function(_arg) {
      var linkToModify, toolbarState;
      toolbarState = _arg.toolbarState;
      if (toolbarState.dragging || toolbarState.doubleDown) {
        return null;
      }
      linkToModify = null;
      if (!linkToModify && toolbarState.selectionSnapshot) {
        linkToModify = LinkManager._linkAtCursor(toolbarState);
      }
      if (!linkToModify) {
        return null;
      }
      return {
        component: LinkEditor,
        props: {
          onSaveUrl: function(url, linkToModify) {
            return toolbarState.atomicEdit(LinkManager._onSaveUrl, {
              url: url,
              linkToModify: linkToModify
            });
          },
          onDoneWithLink: function() {
            return toolbarState.atomicEdit(LinkManager._onDoneWithLink);
          },
          linkToModify: linkToModify,
          focusOnMount: LinkManager._shouldFocusOnMount(toolbarState)
        },
        locationRefNode: linkToModify,
        width: LinkManager._linkWidth(linkToModify),
        height: 34
      };
    };

    LinkManager._shouldFocusOnMount = function(toolbarState) {
      return !toolbarState.selectionSnapshot.isCollapsed;
    };

    LinkManager._linkWidth = function(linkToModify) {
      var WIDTH_PER_CHAR, href, _ref1;
      href = (_ref1 = linkToModify != null ? typeof linkToModify.getAttribute === "function" ? linkToModify.getAttribute('href') : void 0 : void 0) != null ? _ref1 : "";
      WIDTH_PER_CHAR = 8;
      return Math.max(href.length * WIDTH_PER_CHAR + 95, 210);
    };

    LinkManager._linkAtCursor = function(toolbarState) {
      var aPrompt, aTag, anchor, fPrompt, focus, node, _ref1;
      if (toolbarState.selectionSnapshot.isCollapsed) {
        anchor = toolbarState.selectionSnapshot.anchorNode;
        node = DOMUtils.closest(anchor, 'a, n1-prompt-link');
        if (toolbarState.selectionSnapshot.anchorOffset === ((_ref1 = DOMUtils.findLastTextNode(anchor)) != null ? _ref1.data.length : void 0)) {
          return null;
        } else {
          return node;
        }
      } else {
        anchor = toolbarState.selectionSnapshot.anchorNode;
        focus = toolbarState.selectionSnapshot.anchorNode;
        aPrompt = DOMUtils.closest(anchor, 'n1-prompt-link');
        fPrompt = DOMUtils.closest(focus, 'n1-prompt-link');
        if (aPrompt && fPrompt && aPrompt === fPrompt) {
          aTag = DOMUtils.closest(aPrompt, 'a');
          return aTag != null ? aTag : aPrompt;
        } else {
          return null;
        }
      }
    };

    LinkManager._onInsertLink = function(_arg) {
      var editor, event;
      editor = _arg.editor, event = _arg.event;
      return _.defer(function() {
        var html;
        if (editor.currentSelection().isCollapsed) {
          html = "<n1-prompt-link>link text</n1-prompt-link>";
          return editor.insertHTML(html, {
            selectInsertion: true
          });
        } else {
          return editor.wrapSelection("n1-prompt-link");
        }
      });
    };

    LinkManager._onDoneWithLink = function(_arg) {
      var editor, node, _i, _len, _ref1, _results;
      editor = _arg.editor;
      _ref1 = editor.rootNode.querySelectorAll("n1-prompt-link");
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        node = _ref1[_i];
        _results.push(editor.unwrapNodeAndSelectAll(node));
      }
      return _results;
    };

    LinkManager._onSaveUrl = function(_arg) {
      var editor, equivalentNode, linkToModify, node, toSelect, url, _i, _len, _ref1, _ref2, _results;
      editor = _arg.editor, url = _arg.url, linkToModify = _arg.linkToModify;
      if (linkToModify != null) {
        equivalentNode = DOMUtils.findSimilarNodeAtIndex(editor.rootNode, linkToModify, 0);
        if (equivalentNode == null) {
          return;
        }
        if ((typeof linkToModify.getAttribute === "function" ? (_ref1 = linkToModify.getAttribute('href')) != null ? _ref1.trim() : void 0 : void 0) === url.trim()) {
          return;
        }
        toSelect = equivalentNode;
      } else {
        toSelect = null;
      }
      if (url.trim().length === 0) {
        if (toSelect) {
          editor.select(toSelect).unlink();
        } else {
          editor.unlink();
        }
      } else {
        if (toSelect) {
          editor.select(toSelect).createLink(url);
        } else {
          editor.createLink(url);
        }
      }
      _ref2 = editor.rootNode.querySelectorAll("n1-prompt-link");
      _results = [];
      for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
        node = _ref2[_i];
        _results.push(editor.unwrapNodeAndSelectAll(node));
      }
      return _results;
    };

    return LinkManager;

  })(ContenteditableExtension);

  module.exports = LinkManager;

}).call(this);
