(function() {
  var ContenteditableExtension, DOMUtils, ListManager, _ref, _str,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _str = require('underscore.string');

  _ref = require('nylas-exports'), DOMUtils = _ref.DOMUtils, ContenteditableExtension = _ref.ContenteditableExtension;

  ListManager = (function(_super) {
    __extends(ListManager, _super);

    function ListManager() {
      return ListManager.__super__.constructor.apply(this, arguments);
    }

    ListManager.keyCommandHandlers = function() {
      return {
        "contenteditable:numbered-list": ListManager._insertNumberedList,
        "contenteditable:bulleted-list": ListManager._insertBulletedList
      };
    };

    ListManager.onContentChanged = function(_arg) {
      var editor, mutations;
      editor = _arg.editor, mutations = _arg.mutations;
      if (this._spaceEntered && this.hasListStartSignature(editor.currentSelection())) {
        this.createList(editor);
      }
      return this._collapseAdjacentLists(editor);
    };

    ListManager.onKeyDown = function(_arg) {
      var editor, event;
      editor = _arg.editor, event = _arg.event;
      this._spaceEntered = event.key === " ";
      if (DOMUtils.isInList()) {
        if (event.key === "Backspace" && DOMUtils.atStartOfList()) {
          event.preventDefault();
          this.outdentListItem(editor);
        } else if (event.key === "Tab" && editor.currentSelection().isCollapsed) {
          event.preventDefault();
          if (event.shiftKey) {
            this.outdentListItem(editor);
          } else {
            editor.indent();
          }
        } else {
          this.originalInput = null;
        }
      } else {
        this.originalInput = null;
      }
      return event;
    };

    ListManager.bulletRegex = function() {
      return /^[*-]\s/;
    };

    ListManager.numberRegex = function() {
      return /^\d\.\s/;
    };

    ListManager.hasListStartSignature = function(selection) {
      var sibling, text;
      if (!(selection != null ? selection.anchorNode : void 0)) {
        return false;
      }
      if (!selection.isCollapsed) {
        return false;
      }
      sibling = selection.anchorNode.previousElementSibling;
      if (!sibling || DOMUtils.looksLikeBlockElement(sibling)) {
        text = selection.anchorNode.textContent;
        return this.numberRegex().test(text) || this.bulletRegex().test(text);
      } else {
        return false;
      }
    };

    ListManager.createList = function(editor) {
      var el, text, _ref1;
      text = (_ref1 = editor.currentSelection().anchorNode) != null ? _ref1.textContent : void 0;
      if (this.numberRegex().test(text)) {
        this.originalInput = text.slice(0, 3);
        this.insertList(editor, {
          ordered: true
        });
        this.removeListStarter(this.numberRegex(), editor.currentSelection());
      } else if (this.bulletRegex().test(text)) {
        this.originalInput = text.slice(0, 2);
        this.insertList(editor, {
          ordered: false
        });
        this.removeListStarter(this.bulletRegex(), editor.currentSelection());
      } else {
        return;
      }
      el = DOMUtils.closest(editor.currentSelection().anchorNode, "li");
      return DOMUtils.Mutating.removeEmptyNodes(el);
    };

    ListManager.removeListStarter = function(starterRegex, selection) {
      var el, textContent, textNode;
      el = DOMUtils.closest(selection.anchorNode, "li");
      textContent = el.textContent.replace(starterRegex, "");
      if (textContent.trim().length === 0) {
        return el.innerHTML = "<br>";
      } else {
        textNode = DOMUtils.findFirstTextNode(el);
        return textNode.textContent = textNode.textContent.replace(starterRegex, "");
      }
    };

    ListManager.restoreOriginalInput = function(editor) {
      var node, textNode;
      node = editor.currentSelection().anchorNode;
      if (!node) {
        return;
      }
      if (node.nodeType === Node.TEXT_NODE) {
        node.textContent = this.originalInput + node.textContent;
      } else if (node.nodeType === Node.ELEMENT_NODE) {
        textNode = DOMUtils.findFirstTextNode(node);
        if (!textNode) {
          node.innerHTML = this.originalInput.replace(" ", "&nbsp;") + node.innerHTML;
        } else {
          textNode.textContent = this.originalInput + textNode.textContent;
        }
      }
      if (this.numberRegex().test(this.originalInput)) {
        DOMUtils.Mutating.moveSelectionToIndexInAnchorNode(editor.currentSelection(), 3);
      }
      if (this.bulletRegex().test(this.originalInput)) {
        DOMUtils.Mutating.moveSelectionToIndexInAnchorNode(editor.currentSelection(), 2);
      }
      return this.originalInput = null;
    };

    ListManager.insertList = function(editor, _arg) {
      var node, ordered;
      ordered = _arg.ordered;
      node = editor.currentSelection().anchorNode;
      if (this.isInsideListItem(node)) {
        return editor.indent();
      } else {
        if (ordered === true) {
          editor.insertOrderedList();
        }
        if (!ordered) {
          return editor.insertUnorderedList();
        }
      }
    };

    ListManager._insertNumberedList = function(_arg) {
      var editor;
      editor = _arg.editor;
      return editor.insertOrderedList();
    };

    ListManager._insertBulletedList = function(_arg) {
      var editor;
      editor = _arg.editor;
      return editor.insertUnorderedList();
    };

    ListManager.outdentListItem = function(editor) {
      if (this.originalInput) {
        editor.outdent();
        return this.restoreOriginalInput(editor);
      } else {
        return editor.outdent();
      }
    };

    ListManager.isInsideListItem = function(node) {
      return DOMUtils.isDescendantOf(node, function(parent) {
        return parent.tagName === 'LI';
      });
    };

    ListManager._collapseAdjacentLists = function(editor) {
      var els;
      els = editor.rootNode.querySelectorAll('ul, ol');
      return DOMUtils.Mutating.collapseAdjacentElements(els);
    };

    return ListManager;

  })(ContenteditableExtension);

  module.exports = ListManager;

}).call(this);
