(function() {
  var ContenteditableService, DOMUtils, MouseService, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  DOMUtils = require('nylas-exports').DOMUtils;

  ContenteditableService = require('./contenteditable-service');

  MouseService = (function(_super) {
    __extends(MouseService, _super);

    function MouseService() {
      this._onMouseOver = __bind(this._onMouseOver, this);
      this._onDragEnd = __bind(this._onDragEnd, this);
      this._onDragStart = __bind(this._onDragStart, this);
      this._onMouseUp = __bind(this._onMouseUp, this);
      this._onMouseMove = __bind(this._onMouseMove, this);
      this._onDoubleDown = __bind(this._onDoubleDown, this);
      this._onMouseDown = __bind(this._onMouseDown, this);
      MouseService.__super__.constructor.apply(this, arguments);
      this.HOVER_DEBOUNCE = 250;
      this.setup();
      this.timer = null;
      this._inFrame = true;
    }

    MouseService.prototype.eventHandlers = function() {
      return {
        onClick: this._onClick,
        onMouseEnter: (function(_this) {
          return function(event) {
            return _this._inFrame = true;
          };
        })(this),
        onMouseLeave: (function(_this) {
          return function(event) {
            return _this._inFrame = false;
          };
        })(this),
        onMouseOver: this._onMouseOver
      };
    };

    MouseService.prototype._onClick = function(event) {
      return event.stopPropagation();
    };

    MouseService.prototype.setup = function() {
      window.addEventListener("mousedown", this._onMouseDown);
      return window.addEventListener("mouseup", this._onMouseUp);
    };

    MouseService.prototype.teardown = function() {
      window.removeEventListener("mousedown", this._onMouseDown);
      return window.removeEventListener("mouseup", this._onMouseUp);
    };

    MouseService.prototype._onMouseDown = function(event) {
      var _ref;
      this._mouseDownEvent = event;
      this._mouseHasMoved = false;
      window.addEventListener("mousemove", this._onMouseMove);
      if (Date.now() - ((_ref = this._lastMouseDown) != null ? _ref : 0) < 250) {
        this._onDoubleDown(event);
        return this._lastMouseDown = 0;
      } else {
        return this._lastMouseDown = Date.now();
      }
    };

    MouseService.prototype._onDoubleDown = function(event) {
      var editable;
      editable = this.innerState.editableNode;
      if (editable == null) {
        return;
      }
      if (editable === event.target || editable.contains(event.target)) {
        return this.setInnerState({
          doubleDown: true
        });
      }
    };

    MouseService.prototype._onMouseMove = function(event) {
      if (!this._mouseHasMoved) {
        this._onDragStart(this._mouseDownEvent);
        return this._mouseHasMoved = true;
      }
    };

    MouseService.prototype._onMouseUp = function(event) {
      var editableNode, selection;
      window.removeEventListener("mousemove", this._onMouseMove);
      if (this.innerState.doubleDown) {
        this.setInnerState({
          doubleDown: false
        });
      }
      if (this._mouseHasMoved) {
        this._mouseHasMoved = false;
        this._onDragEnd(event);
      }
      editableNode = this.innerState.editableNode;
      selection = document.getSelection();
      if (!DOMUtils.selectionInScope(selection, editableNode)) {
        return event;
      }
      this.dispatchEventToExtensions("onClick", event);
      return event;
    };

    MouseService.prototype._onDragStart = function(event) {
      var editable;
      editable = this.innerState.editableNode;
      if (editable == null) {
        return;
      }
      if (editable === event.target || editable.contains(event.target)) {
        return this.setInnerState({
          dragging: true
        });
      }
    };

    MouseService.prototype._onDragEnd = function(event) {
      if (this.innerState.dragging) {
        this.setInnerState({
          dragging: false
        });
      }
      return event;
    };

    MouseService.prototype._onMouseOver = function(event) {};

    return MouseService;

  })(ContenteditableService);

  module.exports = MouseService;

}).call(this);
