(function() {
  var EventedIFrame, IFrameSearcher, IdentityStore, React, ReactDOM, RegExpUtils, SearchableComponentMaker, SearchableComponentStore, Utils, url, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require('react');

  ReactDOM = require('react-dom');

  _ref = require('nylas-exports'), Utils = _ref.Utils, RegExpUtils = _ref.RegExpUtils, IdentityStore = _ref.IdentityStore, SearchableComponentMaker = _ref.SearchableComponentMaker, SearchableComponentStore = _ref.SearchableComponentStore;

  IFrameSearcher = require('../searchable-components/iframe-searcher')["default"];

  url = require('url');

  _ = require("underscore");


  /*
  Public: EventedIFrame is a thin wrapper around the DOM's standard `<iframe>` element.
  You should always use EventedIFrame, because it provides important event hooks that
  ensure keyboard and mouse events are properly delivered to the application when
  fired within iFrames.
  
  ```
  <div className="file-frame-container">
    <EventedIFrame src={src} />
    <Spinner visible={!@state.ready} />
  </div>
  ```
  
  Any `props` added to the <EventedIFrame> are passed to the iFrame it renders.
  
  Section: Component Kit
   */

  EventedIFrame = (function(_super) {
    __extends(EventedIFrame, _super);

    function EventedIFrame() {
      this._onIFrameContextualMenu = __bind(this._onIFrameContextualMenu, this);
      this._onIFrameKeyEvent = __bind(this._onIFrameKeyEvent, this);
      this._onIFrameMouseEvent = __bind(this._onIFrameMouseEvent, this);
      this._onIFrameClick = __bind(this._onIFrameClick, this);
      this._onIFrameResize = __bind(this._onIFrameResize, this);
      this._onIFrameFocus = __bind(this._onIFrameFocus, this);
      this._onIFrameBlur = __bind(this._onIFrameBlur, this);
      this._getContainingTarget = __bind(this._getContainingTarget, this);
      this._subscribeToIFrameEvents = __bind(this._subscribeToIFrameEvents, this);
      this._unsubscribeFromIFrameEvents = __bind(this._unsubscribeFromIFrameEvents, this);
      this._onSearchableStoreChange = __bind(this._onSearchableStoreChange, this);
      this.setHeightQuietly = __bind(this.setHeightQuietly, this);
      this.didReplaceDocument = __bind(this.didReplaceDocument, this);
      this.shouldComponentUpdate = __bind(this.shouldComponentUpdate, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.render = __bind(this.render, this);
      return EventedIFrame.__super__.constructor.apply(this, arguments);
    }

    EventedIFrame.displayName = 'EventedIFrame';

    EventedIFrame.prototype.render = function() {
      return React.createElement("iframe", React.__spread({
        "seamless": "seamless"
      }, this.props));
    };

    EventedIFrame.prototype.componentDidMount = function() {
      if (this.props.searchable) {
        this._regionId = Utils.generateTempId();
        this._searchUsub = SearchableComponentStore.listen(this._onSearchableStoreChange);
        SearchableComponentStore.registerSearchRegion(this._regionId, ReactDOM.findDOMNode(this));
      }
      return this._subscribeToIFrameEvents();
    };

    EventedIFrame.prototype.componentWillUnmount = function() {
      this._unsubscribeFromIFrameEvents();
      if (this.props.searchable) {
        this._searchUsub();
        return SearchableComponentStore.unregisterSearchRegion(this._regionId);
      }
    };

    EventedIFrame.prototype.componentDidUpdate = function() {
      if (this.props.searchable) {
        return SearchableComponentStore.registerSearchRegion(this._regionId, ReactDOM.findDOMNode(this));
      }
    };

    EventedIFrame.prototype.shouldComponentUpdate = function(nextProps, nextState) {
      return !Utils.isEqualReact(nextProps, this.props) || !Utils.isEqualReact(nextState, this.state);
    };


    /*
    Public: Call this method if you replace the contents of the iframe's document.
    This allows {EventedIframe} to re-attach it's event listeners.
     */

    EventedIFrame.prototype.didReplaceDocument = function() {
      this._unsubscribeFromIFrameEvents();
      return this._subscribeToIFrameEvents();
    };

    EventedIFrame.prototype.setHeightQuietly = function(height) {
      this._ignoreNextResize = true;
      return ReactDOM.findDOMNode(this).height = "" + height + "px";
    };

    EventedIFrame.prototype._onSearchableStoreChange = function() {
      var doc, node, searchIndex, searchTerm, _ref1, _ref2;
      if (!this.props.searchable) {
        return;
      }
      node = ReactDOM.findDOMNode(this);
      doc = (_ref1 = (_ref2 = node.contentDocument) != null ? _ref2.body : void 0) != null ? _ref1 : node.contentDocument;
      searchIndex = SearchableComponentStore.getCurrentRegionIndex(this._regionId);
      searchTerm = SearchableComponentStore.getCurrentSearchData().searchTerm;
      if (this.lastSearchIndex !== searchIndex || this.lastSearchTerm !== searchTerm) {
        IFrameSearcher.highlightSearchInDocument(this._regionId, searchTerm, doc, searchIndex);
      }
      this.lastSearchIndex = searchIndex;
      return this.lastSearchTerm = searchTerm;
    };

    EventedIFrame.prototype._unsubscribeFromIFrameEvents = function() {
      var doc, node;
      node = ReactDOM.findDOMNode(this);
      doc = node.contentDocument;
      if (!doc) {
        return;
      }
      doc.removeEventListener('click', this._onIFrameClick);
      doc.removeEventListener('keydown', this._onIFrameKeyEvent);
      doc.removeEventListener('keypress', this._onIFrameKeyEvent);
      doc.removeEventListener('keyup', this._onIFrameKeyEvent);
      doc.removeEventListener('mousedown', this._onIFrameMouseEvent);
      doc.removeEventListener('mousemove', this._onIFrameMouseEvent);
      doc.removeEventListener('mouseup', this._onIFrameMouseEvent);
      doc.removeEventListener("contextmenu", this._onIFrameContextualMenu);
      if (node.contentWindow) {
        node.contentWindow.removeEventListener('focus', this._onIFrameFocus);
        node.contentWindow.removeEventListener('blur', this._onIFrameBlur);
        return node.contentWindow.removeEventListener('resize', this._onIFrameResize);
      }
    };

    EventedIFrame.prototype._subscribeToIFrameEvents = function() {
      var doc, node;
      node = ReactDOM.findDOMNode(this);
      doc = node.contentDocument;
      return _.defer((function(_this) {
        return function() {
          doc.addEventListener("click", _this._onIFrameClick);
          doc.addEventListener("keydown", _this._onIFrameKeyEvent);
          doc.addEventListener("keypress", _this._onIFrameKeyEvent);
          doc.addEventListener("keyup", _this._onIFrameKeyEvent);
          doc.addEventListener("mousedown", _this._onIFrameMouseEvent);
          doc.addEventListener("mousemove", _this._onIFrameMouseEvent);
          doc.addEventListener("mouseup", _this._onIFrameMouseEvent);
          doc.addEventListener("contextmenu", _this._onIFrameContextualMenu);
          if (node.contentWindow) {
            node.contentWindow.addEventListener("focus", _this._onIFrameFocus);
            node.contentWindow.addEventListener("blur", _this._onIFrameBlur);
            if (_this.props.onResize) {
              return node.contentWindow.addEventListener('resize', _this._onIFrameResize);
            }
          }
        };
      })(this));
    };

    EventedIFrame.prototype._getContainingTarget = function(event, options) {
      var target;
      target = event.target;
      while ((target != null) && (target !== document) && (target !== window)) {
        if (target.getAttribute(options["with"]) != null) {
          return target;
        }
        target = target.parentElement;
      }
      return null;
    };

    EventedIFrame.prototype._onIFrameBlur = function(event) {
      var node;
      node = ReactDOM.findDOMNode(this);
      return node.contentWindow.getSelection().empty();
    };

    EventedIFrame.prototype._onIFrameFocus = function(event) {
      return window.getSelection().empty();
    };

    EventedIFrame.prototype._onIFrameResize = function(event) {
      var _base;
      if (this._ignoreNextResize) {
        this._ignoreNextResize = false;
        return;
      }
      return typeof (_base = this.props).onResize === "function" ? _base.onResize(event) : void 0;
    };

    EventedIFrame.prototype._onIFrameClick = function(e) {
      var path, rawHref, target;
      e.stopPropagation();
      target = this._getContainingTarget(e, {
        "with": 'href'
      });
      if (target) {
        rawHref = target.getAttribute('href');
        if (this._isBlacklistedHref(rawHref)) {
          e.preventDefault();
          return;
        }
        if (!url.parse(rawHref).protocol) {
          if ((new RegExp(/^\/\//)).test(rawHref)) {
            target.setAttribute('href', "https:" + rawHref);
          } else {
            target.setAttribute('href', "http://" + rawHref);
          }
          rawHref = target.getAttribute('href');
        }
        e.preventDefault();
        if (rawHref.startsWith(IdentityStore.URLRoot)) {
          path = rawHref.split(IdentityStore.URLRoot).pop();
          IdentityStore.fetchSingleSignOnURL(path, {
            source: "SingleSignOnEmail"
          }).then((function(_this) {
            return function(href) {
              return NylasEnv.windowEventHandler.openLink({
                href: href,
                metaKey: e.metaKey
              });
            };
          })(this));
          return;
        }
        return NylasEnv.windowEventHandler.openLink({
          href: rawHref,
          metaKey: e.metaKey
        });
      }
    };

    EventedIFrame.prototype._isBlacklistedHref = function(href) {
      return (new RegExp(/^file:/i)).test(href);
    };

    EventedIFrame.prototype._onIFrameMouseEvent = function(event) {
      var eventAttrs, key, node, nodeRect, _i, _len, _ref1;
      node = ReactDOM.findDOMNode(this);
      nodeRect = node.getBoundingClientRect();
      eventAttrs = {};
      _ref1 = Object.keys(event);
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        key = _ref1[_i];
        if (key === 'webkitMovementX' || key === 'webkitMovementY') {
          continue;
        }
        eventAttrs[key] = event[key];
      }
      return node.dispatchEvent(new MouseEvent(event.type, _.extend({}, eventAttrs, {
        bubbles: true,
        clientX: event.clientX + nodeRect.left,
        clientY: event.clientY + nodeRect.top,
        pageX: event.pageX + nodeRect.left,
        pageY: event.pageY + nodeRect.top
      })));
    };

    EventedIFrame.prototype._onIFrameKeyEvent = function(event) {
      var attrs, eventInParentDoc, eventInit;
      if (event.metaKey || event.altKey || event.ctrlKey) {
        return;
      }
      attrs = ['key', 'code', 'location', 'ctrlKey', 'shiftKey', 'altKey', 'metaKey', 'repeat', 'isComposing', 'charCode', 'keyCode', 'which'];
      eventInit = Object.assign({
        bubbles: true
      }, _.pick(event, attrs));
      eventInParentDoc = new KeyboardEvent(event.type, eventInit);
      Object.defineProperty(eventInParentDoc, 'which', {
        value: event.which
      });
      return ReactDOM.findDOMNode(this).dispatchEvent(eventInParentDoc);
    };

    EventedIFrame.prototype._onIFrameContextualMenu = function(event) {
      var Menu, MenuItem, clipboard, fs, href, imageTarget, linkTarget, menu, nativeImage, path, range, remote, selection, shell, src, srcFilename, text, textPreview, _ref1;
      event.preventDefault();
      _ref1 = require('electron'), remote = _ref1.remote, clipboard = _ref1.clipboard, shell = _ref1.shell, nativeImage = _ref1.nativeImage;
      Menu = remote.Menu, MenuItem = remote.MenuItem;
      path = require('path');
      fs = require('fs');
      menu = new Menu();
      linkTarget = this._getContainingTarget(event, {
        "with": 'href'
      });
      if (linkTarget) {
        href = linkTarget.getAttribute('href');
        if (href.startsWith('mailto')) {
          menu.append(new MenuItem({
            label: "Compose Message...",
            click: (function() {
              return NylasEnv.windowEventHandler.openLink({
                href: href
              });
            })
          }));
          menu.append(new MenuItem({
            label: "Copy Email Address",
            click: (function() {
              return clipboard.writeText(href.split('mailto:').pop());
            })
          }));
        } else {
          menu.append(new MenuItem({
            label: "Open Link",
            click: (function() {
              return NylasEnv.windowEventHandler.openLink({
                href: href
              });
            })
          }));
          menu.append(new MenuItem({
            label: "Copy Link Address",
            click: (function() {
              return clipboard.writeText(href);
            })
          }));
        }
        menu.append(new MenuItem({
          type: 'separator'
        }));
      }
      imageTarget = this._getContainingTarget(event, {
        "with": 'src'
      });
      if (imageTarget) {
        src = imageTarget.getAttribute('src');
        srcFilename = path.basename(src);
        menu.append(new MenuItem({
          label: "Save Image...",
          click: function() {
            return NylasEnv.showSaveDialog({
              defaultPath: srcFilename
            }, function(path) {
              var oReq;
              if (!path) {
                return;
              }
              oReq = new XMLHttpRequest();
              oReq.open("GET", src, true);
              oReq.responseType = "arraybuffer";
              oReq.onload = function() {
                var buffer;
                buffer = new Buffer(new Uint8Array(oReq.response));
                return fs.writeFile(path, buffer, function(err) {
                  return shell.showItemInFolder(path);
                });
              };
              return oReq.send();
            });
          }
        }));
        menu.append(new MenuItem({
          label: "Copy Image",
          click: function() {
            var img;
            img = new Image();
            img.addEventListener("load", function() {
              var canvas, imageDataURL;
              canvas = document.createElement("canvas");
              canvas.width = img.width;
              canvas.height = img.height;
              canvas.getContext("2d").drawImage(imageTarget, 0, 0);
              imageDataURL = canvas.toDataURL("image/png");
              img = nativeImage.createFromDataURL(imageDataURL);
              return clipboard.writeImage(img);
            }, false);
            return img.src = src;
          }
        }));
        menu.append(new MenuItem({
          type: 'separator'
        }));
      }
      text = "";
      selection = ReactDOM.findDOMNode(this).contentDocument.getSelection();
      if (selection.rangeCount > 0) {
        range = selection.getRangeAt(0);
        text = range.toString();
      }
      if (!text || text.length === 0) {
        text = (linkTarget != null ? linkTarget : event.target).innerText;
      }
      text = text.trim();
      if (text.length > 0) {
        if (text.length > 45) {
          textPreview = text.substr(0, 42) + "...";
        } else {
          textPreview = text;
        }
        menu.append(new MenuItem({
          label: "Copy",
          click: (function() {
            return clipboard.writeText(text);
          })
        }));
        menu.append(new MenuItem({
          label: "Search Google for '" + textPreview + "'",
          click: (function() {
            return shell.openExternal("https://www.google.com/search?q=" + (encodeURIComponent(text)));
          })
        }));
        if (process.platform === 'darwin') {
          menu.append(new MenuItem({
            label: "Look Up '" + textPreview + "'",
            click: (function() {
              return NylasEnv.getCurrentWindow().showDefinitionForSelection();
            })
          }));
        }
      }
      if (process.platform === 'darwin') {
        menu.append(new MenuItem({
          type: 'separator'
        }));
      }
      return menu.popup(remote.getCurrentWindow());
    };

    return EventedIFrame;

  })(React.Component);

  module.exports = EventedIFrame;

}).call(this);
