(function() {
  var FormItem, GeneratedFieldset, GeneratedForm, React, ReactDOM, Utils, classNames, idPropType, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  classNames = require('classnames');

  React = require('react');

  ReactDOM = require('react-dom');

  Utils = require('nylas-exports').Utils;

  idPropType = React.PropTypes.oneOfType([React.PropTypes.string, React.PropTypes.number]);

  FormItem = (function(_super) {
    __extends(FormItem, _super);

    function FormItem() {
      this._renderInput = __bind(this._renderInput, this);
      this._renderError = __bind(this._renderError, this);
      this.refreshValidityState = __bind(this.refreshValidityState, this);
      this.shouldComponentUpdate = __bind(this.shouldComponentUpdate, this);
      this.componentDidUpdate = __bind(this.componentDidUpdate, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.componentWillMount = __bind(this.componentWillMount, this);
      this.render = __bind(this.render, this);
      return FormItem.__super__.constructor.apply(this, arguments);
    }

    FormItem.displayName = "FormItem";

    FormItem.inputElementTypes = {
      "checkbox": true,
      "color": true,
      "date": true,
      "datetime": true,
      "datetime-local": true,
      "email": true,
      "file": true,
      "hidden": true,
      "month": true,
      "number": true,
      "password": true,
      "radio": true,
      "range": true,
      "search": true,
      "tel": true,
      "text": true,
      "time": true,
      "url": true,
      "week": true
    };

    FormItem.propTypes = {
      id: idPropType.isRequired,
      formItemError: React.PropTypes.shape({
        id: idPropType,
        message: React.PropTypes.string
      }),
      type: React.PropTypes.oneOfType([React.PropTypes.string, React.PropTypes.func]).isRequired,
      name: React.PropTypes.string,
      label: React.PropTypes.node,
      value: React.PropTypes.oneOfType([React.PropTypes.string, React.PropTypes.number, React.PropTypes.object, React.PropTypes.bool]),
      defaultValue: React.PropTypes.string,
      selectOptions: React.PropTypes.arrayOf(React.PropTypes.shape({
        label: React.PropTypes.node,
        value: React.PropTypes.string
      })),
      onChange: React.PropTypes.func,
      required: React.PropTypes.bool,
      multiple: React.PropTypes.bool,
      maxlength: React.PropTypes.number,
      placeholder: React.PropTypes.string,
      tabIndex: React.PropTypes.number,
      referenceTo: React.PropTypes.string,
      relationshipName: React.PropTypes.string
    };

    FormItem.prototype.render = function() {
      var classes, label;
      classes = classNames({
        "form-item": true,
        "valid": this.state.valid
      });
      label = this.props.label;
      if (this.props.label.slice(-3) === " ID") {
        label = this.props.label.slice(0, -3);
      }
      if (_.last(label) !== ":") {
        label = "" + label + ":";
      }
      if (this.props.type === "hidden") {
        return this._renderInput();
      } else {
        return React.createElement("div", {
          "className": classes
        }, React.createElement("div", {
          "className": "label-area"
        }, React.createElement("label", null, label)), React.createElement("div", {
          "className": "input-area"
        }, this._renderInput(), this._renderError()));
      }
    };

    FormItem.prototype.componentWillMount = function() {
      return this.setState({
        valid: true
      });
    };

    FormItem.prototype.componentDidMount = function() {
      return this.refreshValidityState();
    };

    FormItem.prototype.componentDidUpdate = function() {
      return this.refreshValidityState();
    };

    FormItem.prototype.shouldComponentUpdate = function(nextProps, nextState) {
      return !Utils.isEqualReact(nextProps, this.props) || !Utils.isEqualReact(nextState, this.state);
    };

    FormItem.prototype.refreshValidityState = function() {
      return _.defer((function(_this) {
        return function() {
          var customMsg, el, newValidity, _ref, _ref1, _ref2;
          if (!_this.refs.input) {
            return;
          }
          el = ReactDOM.findDOMNode(_this.refs.input);
          customMsg = (_ref = _this.props.formItemError) != null ? _ref.message : void 0;
          if (el.setCustomValidity != null) {
            if (customMsg) {
              el.setCustomValidity(customMsg);
            } else {
              el.setCustomValidity('');
            }
          }
          newValidity = _.extend({}, (_ref1 = el.validity) != null ? _ref1 : {}, {
            validationMessage: (_ref2 = el.validationMessage) != null ? _ref2 : ""
          });
          if (!Utils.isEqual(newValidity, _this._lastValidity)) {
            _this.setState(newValidity);
          }
          return _this._lastValidity = newValidity;
        };
      })(this));
    };

    FormItem.prototype._renderError = function() {
      if (this.state.valid) {
        return React.createElement("div", null);
      } else {
        if (this.state.customError || this._changedOnce) {
          return React.createElement("div", {
            "className": "form-error"
          }, this.state.validationMessage);
        } else {
          return React.createElement("div", null);
        }
      }
    };

    FormItem.prototype._renderInput = function() {
      var inputProps, options, _ref;
      inputProps = _.extend({}, this.props, {
        ref: "input",
        onChange: (function(_this) {
          return function(eventOrValue) {
            var _ref, _ref1;
            _this._changedOnce = true;
            return _this.props.onChange(_this.props.id, (_ref = (eventOrValue != null ? (_ref1 = eventOrValue.target) != null ? _ref1.value : void 0 : void 0)) != null ? _ref : eventOrValue);
          };
        })(this),
        onBlur: (function(_this) {
          return function() {
            return _this.refreshValidityState();
          };
        })(this)
      });
      if (this.props.type in FormItem.inputElementTypes) {
        return React.createElement("input", inputProps);
      } else if (this.props.type === "select") {
        options = ((_ref = this.props.selectOptions) != null ? _ref : []).map(function(optionData) {
          return React.createElement("option", React.__spread({}, optionData, {
            "key": "" + (Utils.generateTempId()) + "-optionData.value"
          }), optionData.label);
        });
        options.unshift(React.createElement("option", {
          "key": "" + (Utils.generateTempId()) + "-blank-option"
        }));
        return React.createElement("select", React.__spread({}, inputProps), options);
      } else if (this.props.type === "textarea") {
        return React.createElement("textarea", inputProps);
      } else if (_.isFunction(this.props.type)) {
        return React.createElement(this.props.type, inputProps);
      } else {
        return console.warn("We do not support type " + this.props.type + " with attributes:", inputProps);
      }
    };

    return FormItem;

  })(React.Component);

  GeneratedFieldset = (function(_super) {
    __extends(GeneratedFieldset, _super);

    function GeneratedFieldset() {
      this._renderFooter = __bind(this._renderFooter, this);
      this._onChangeItem = __bind(this._onChangeItem, this);
      this._propsFromFormItemData = __bind(this._propsFromFormItemData, this);
      this._renderFormItems = __bind(this._renderFormItems, this);
      this._renderHeader = __bind(this._renderHeader, this);
      this.refreshValidityStates = __bind(this.refreshValidityStates, this);
      this.shouldComponentUpdate = __bind(this.shouldComponentUpdate, this);
      this.render = __bind(this.render, this);
      return GeneratedFieldset.__super__.constructor.apply(this, arguments);
    }

    GeneratedFieldset.displayName = "GeneratedFieldset";

    GeneratedFieldset.propTypes = {
      id: idPropType.isRequired,
      formItems: React.PropTypes.arrayOf(React.PropTypes.shape(_.extend(FormItem.propTypes, {
        row: React.PropTypes.number,
        column: React.PropTypes.number
      }))),
      formItemErrors: React.PropTypes.object,
      onChange: React.PropTypes.func,
      heading: React.PropTypes.node,
      useHeading: React.PropTypes.bool
    };

    GeneratedFieldset.prototype.render = function() {
      return React.createElement("fieldset", null, this._renderHeader(), React.createElement("div", {
        "className": "fieldset-form-items"
      }, this._renderFormItems()), this._renderFooter());
    };

    GeneratedFieldset.prototype.shouldComponentUpdate = function(nextProps, nextState) {
      return !Utils.isEqualReact(nextProps, this.props) || !Utils.isEqualReact(nextState, this.state);
    };

    GeneratedFieldset.prototype.refreshValidityStates = function() {
      var key, ref, _i, _len, _ref, _results;
      _ref = this.refs;
      _results = [];
      for (ref = _i = 0, _len = _ref.length; _i < _len; ref = ++_i) {
        key = _ref[ref];
        if (key.indexOf("form-item") === 0) {
          _results.push(ref.refreshValidityState());
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    GeneratedFieldset.prototype._renderHeader = function() {
      if (this.props.useHeading) {
        return React.createElement("header", null, React.createElement("legend", null, this.props.heading));
      } else {
        return React.createElement("div", null);
      }
    };

    GeneratedFieldset.prototype._renderFormItems = function() {
      var byRow;
      byRow = _.groupBy(this.props.formItems, "row");
      return _.map(byRow, (function(_this) {
        return function(items, rowNum) {
          var i, item, itemsWithSpacers, _i, _len;
          if (items == null) {
            items = [];
          }
          itemsWithSpacers = [];
          for (i = _i = 0, _len = items.length; _i < _len; i = ++_i) {
            item = items[i];
            itemsWithSpacers.push(item);
            if (i !== items.length - 1) {
              itemsWithSpacers.push({
                spacer: true
              });
            }
          }
          return React.createElement("div", {
            "className": "row",
            "data-row-num": rowNum,
            "style": {
              zIndex: 1000 - rowNum
            },
            "key": rowNum
          }, _.map(itemsWithSpacers, function(formItemData, i) {
            var props;
            if (formItemData.spacer) {
              return React.createElement("div", {
                "className": "column-spacer",
                "data-col-num": i,
                "key": i
              });
            } else {
              props = _this._propsFromFormItemData(formItemData);
              return React.createElement("div", {
                "className": "column",
                "data-col-num": i,
                "key": i
              }, React.createElement(FormItem, React.__spread({}, props, {
                "ref": "form-item-" + formItemData.id
              })));
            }
          }));
        };
      })(this));
    };

    GeneratedFieldset.prototype._propsFromFormItemData = function(formItemData) {
      var error, props, _ref;
      props = _.clone(formItemData);
      props.key = props.id;
      error = (_ref = this.props.formItemErrors) != null ? _ref[props.id] : void 0;
      if (error) {
        props.formItemError = error;
      }
      props.onChange = _.bind(this._onChangeItem, this);
      return props;
    };

    GeneratedFieldset.prototype._onChangeItem = function(itemId, newValue) {
      var newFormItems;
      newFormItems = _.map(this.props.formItems, function(formItem) {
        var newFormItem;
        if (formItem.id === itemId) {
          newFormItem = _.clone(formItem);
          newFormItem.value = newValue;
          return newFormItem;
        } else {
          return formItem;
        }
      });
      return this.props.onChange(this.props.id, newFormItems);
    };

    GeneratedFieldset.prototype._renderFooter = function() {
      return React.createElement("footer", null);
    };

    return GeneratedFieldset;

  })(React.Component);

  GeneratedForm = (function(_super) {
    __extends(GeneratedForm, _super);

    function GeneratedForm() {
      this._onChangeFieldset = __bind(this._onChangeFieldset, this);
      this._propsFromFieldsetData = __bind(this._propsFromFieldsetData, this);
      this._renderFieldsets = __bind(this._renderFieldsets, this);
      this._renderHeaderFormError = __bind(this._renderHeaderFormError, this);
      this.refreshValidityStates = __bind(this.refreshValidityStates, this);
      this._onSubmit = __bind(this._onSubmit, this);
      this.shouldComponentUpdate = __bind(this.shouldComponentUpdate, this);
      this.render = __bind(this.render, this);
      return GeneratedForm.__super__.constructor.apply(this, arguments);
    }

    GeneratedForm.displayName = "GeneratedForm";

    GeneratedForm.propTypes = {
      id: idPropType,
      errors: React.PropTypes.shape({
        formError: React.PropTypes.shape({
          message: React.PropTypes.string,
          location: React.PropTypes.string
        }),
        formItemErrors: GeneratedFieldset.propTypes.formItemErrors
      }),
      fieldsets: React.PropTypes.arrayOf(React.PropTypes.shape(GeneratedFieldset.propTypes)),
      onChange: React.PropTypes.func.isRequired,
      onSubmit: React.PropTypes.func.isRequired
    };

    GeneratedForm.prototype.render = function() {
      return React.createElement("form", {
        "className": "generated-form",
        "ref": "form"
      }, this._renderHeaderFormError(), React.createElement("div", {
        "className": "fieldsets"
      }, this._renderFieldsets()), this._renderHeaderFormError(), React.createElement("div", {
        "className": "form-footer"
      }, React.createElement("button", {
        "onClick": this.props.onSubmit
      }, "Submit")));
    };

    GeneratedForm.prototype.shouldComponentUpdate = function(nextProps, nextState) {
      return !Utils.isEqualReact(nextProps, this.props) || !Utils.isEqualReact(nextState, this.state);
    };

    GeneratedForm.prototype._onSubmit = function() {
      var valid;
      valid = ReactDOM.findDOMNode(this.refs.form).reportValidity();
      if (valid) {
        return this.props.onSubmit();
      } else {
        return this.refreshValidityStates();
      }
    };

    GeneratedForm.prototype.refreshValidityStates = function() {
      var key, ref, _i, _len, _ref, _results;
      _ref = this.refs;
      _results = [];
      for (ref = _i = 0, _len = _ref.length; _i < _len; ref = ++_i) {
        key = _ref[ref];
        if (key.indexOf("fieldset") === 0) {
          _results.push(ref.refreshValidityStates());
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    GeneratedForm.prototype._renderHeaderFormError = function() {
      var _ref;
      if ((_ref = this.props.errors) != null ? _ref.formError : void 0) {
        return React.createElement("div", {
          "className": "form-error form-header-error"
        }, this.props.errors.formError.message);
      } else {
        return React.createElement("div", null);
      }
    };

    GeneratedForm.prototype._renderFieldsets = function() {
      var _ref;
      return ((_ref = this.props.fieldsets) != null ? _ref : []).map((function(_this) {
        return function(fieldset) {
          var props;
          props = _this._propsFromFieldsetData(fieldset);
          return React.createElement(GeneratedFieldset, React.__spread({}, props, {
            "ref": "fieldset-" + fieldset.id
          }));
        };
      })(this));
    };

    GeneratedForm.prototype._propsFromFieldsetData = function(fieldsetData) {
      var errors, props, _ref;
      props = _.clone(fieldsetData);
      errors = (_ref = this.props.errors) != null ? _ref.formItemErrors : void 0;
      if (errors) {
        props.formItemErrors = errors;
      }
      props.key = fieldsetData.id;
      props.onChange = _.bind(this._onChangeFieldset, this);
      return props;
    };

    GeneratedForm.prototype._onChangeFieldset = function(fieldsetId, newFormItems) {
      var newFieldsets;
      newFieldsets = _.map(this.props.fieldsets, function(fieldset) {
        var newFieldset;
        if (fieldset.id === fieldsetId) {
          newFieldset = _.clone(fieldset);
          newFieldset.formItems = newFormItems;
          return newFieldset;
        } else {
          return fieldset;
        }
      });
      return this.props.onChange(_.extend({}, this.props, {
        fieldsets: newFieldsets
      }));
    };

    return GeneratedForm;

  })(React.Component);

  module.exports = {
    FormItem: FormItem,
    GeneratedForm: GeneratedForm,
    GeneratedFieldset: GeneratedFieldset
  };

}).call(this);
