(function() {
  var KeyCommandsRegion, React, ReactDOM, classNames, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  React = require('react');

  ReactDOM = require('react-dom');

  classNames = require('classnames');


  /*
  Public: Easily respond to keyboard shortcuts
  
  A keyboard shortcut has two parts to it:
  
  1. A mapping between keyboard actions and a command
  2. A mapping between a command and a callback handler
  
  
   *# Mapping keys to commands (not handled by this component)
  
  The **keyboard -> command** mapping is defined in a separate `.cson` file.
  A majority of the commands your component would want to listen to you have
  already been defined by core N1 defaults, as well as custom user
  overrides. See 'keymaps/base.cson' for more information.
  
  You can define additional, custom keyboard -> command mappings in your own
  package-specific keymap `.cson` file. The file can be named anything but
  must exist in a folder called `keymaps` in the root of your package's
  directory.
  
  
   *# Mapping commands to callbacks (handled by this component)
  
  When a keystroke sequence matches a binding in a given context, a custom
  DOM event with a type based on the command is dispatched on the target of
  the keyboard event.
  
  That custom DOM event (whose type is the command you want to listen to)
  will propagate up from its original target. That original target may or
  may not be a descendent of your <KeyCommandsRegion> component.
  
  Frequently components will want to listen to a keyboard command regardless
  of where it was fired from. For those, use the `globalHandlers` prop. The
  DOM event will NOT be passed to `globalHandlers` callbacks.
  
  Components may also want to listen to keyboard commands that originate
  within one of their descendents. For those use the `localHandlers` prop.
  The DOM event WILL be passed to `localHandlers` callback because it is
  sometimes valuable to call `stopPropagataion` on the custom command event.
  
  Props:
  
  - `localHandlers` A mapping between key commands and callbacks for key command events that originate within a descendent of this component.
  - `globalHandlers` A mapping between key commands and callbacks for key
  commands that originate from anywhere and are global in scope.
  - `className` The unique class name that shows up in your keymap.cson
  
  Example:
  
  In `my-package/lib/my-component.cjsx`:
  
  ```coffee
  class MyComponent extends React.Component
    render: ->
      <KeyCommandsRegion globalHandlers={@globalHandlers()} className="my-component">
        <div>... sweet component ...</div>
      </KeyCommandsRegion>
  
    globalHandlers: ->
      "core:moveDown": @onMoveDown
      "core:selectItem": @onSelectItem
  
    localHandlers: ->
      "custom:send": (event) => @onSelectItem(); event.stopPropagation()
      "custom:move": @onCustomMove
  ```
  
  In `my-package/keymaps/my-package.cson`:
  
  ```coffee
  ".my-component":
    "cmd-t": "selectItem"
    "cmd-enter": "sendMessage"
  ```
   */

  KeyCommandsRegion = (function(_super) {
    __extends(KeyCommandsRegion, _super);

    KeyCommandsRegion.displayName = "KeyCommandsRegion";

    KeyCommandsRegion.propTypes = {
      className: React.PropTypes.string,
      localHandlers: React.PropTypes.object,
      globalHandlers: React.PropTypes.object,
      globalMenuItems: React.PropTypes.array,
      onFocusIn: React.PropTypes.func,
      onFocusOut: React.PropTypes.func
    };

    KeyCommandsRegion.defaultProps = {
      className: "",
      localHandlers: null,
      globalHandlers: null,
      globalMenuItems: null,
      onFocusIn: function() {},
      onFocusOut: function() {}
    };

    function KeyCommandsRegion() {
      this._onDefinitelyFocusedOut = __bind(this._onDefinitelyFocusedOut, this);
      this._onFocusOut = __bind(this._onFocusOut, this);
      this._onFocusIn = __bind(this._onFocusIn, this);
      this._onWindowBlur = __bind(this._onWindowBlur, this);
      this._lostFocusToElement = null;
      this.state = {
        focused: false
      };
    }

    KeyCommandsRegion.prototype.componentWillReceiveProps = function(newProps) {
      var _ref;
      this._unmountListeners();
      this._setupListeners(newProps);
      if (!_.isEqual(newProps.globalMenuItems, this.props.globalMenuItems)) {
        if ((_ref = this._menuDisposable) != null) {
          _ref.dispose();
        }
        return this._menuDisposable = NylasEnv.menu.add(newProps.globalMenuItems);
      }
    };

    KeyCommandsRegion.prototype.componentDidMount = function() {
      this._setupListeners(this.props);
      if (this.props.globalMenuItems) {
        return this._menuDisposable = NylasEnv.menu.add(this.props.globalMenuItems);
      }
    };

    KeyCommandsRegion.prototype.componentWillUnmount = function() {
      var _ref;
      this._losingFocusToElement = null;
      this._unmountListeners();
      if ((_ref = this._menuDisposable) != null) {
        _ref.dispose();
      }
      return this._menuDisposable = null;
    };

    KeyCommandsRegion.prototype._setupListeners = function(props) {
      var $el;
      $el = ReactDOM.findDOMNode(this);
      $el.addEventListener('focusin', this._onFocusIn);
      $el.addEventListener('focusout', this._onFocusOut);
      if (props.globalHandlers) {
        this._globalDisposable = NylasEnv.commands.add(document.body, props.globalHandlers);
      }
      if (props.localHandlers) {
        this._localDisposable = NylasEnv.commands.add($el, props.localHandlers);
      }
      return window.addEventListener('browser-window-blur', this._onWindowBlur);
    };

    KeyCommandsRegion.prototype._unmountListeners = function() {
      var $el, _ref, _ref1;
      if ((_ref = this._globalDisposable) != null) {
        _ref.dispose();
      }
      this._globalDisposable = null;
      if ((_ref1 = this._localDisposable) != null) {
        _ref1.dispose();
      }
      this._localDisposable = null;
      $el = ReactDOM.findDOMNode(this);
      $el.removeEventListener('focusin', this._onFocusIn);
      $el.removeEventListener('focusout', this._onDidFocusOut);
      window.removeEventListener('browser-window-blur', this._onWindowBlur);
      return this._goingout = false;
    };

    KeyCommandsRegion.prototype._onWindowBlur = function() {
      return this.setState({
        focused: false
      });
    };

    KeyCommandsRegion.prototype._onFocusIn = function(event) {
      this._lastFocusElement = event.target;
      this._losingFocusToElement = null;
      if (this.state.focused === false) {
        this.props.onFocusIn(event);
      }
      return this.setState({
        focused: true
      });
    };

    KeyCommandsRegion.prototype._onFocusOut = function(event) {
      var attempt;
      this._lastFocusElement = event.target;
      this._losingFocusToElement = event.relatedTarget;
      attempt = (function(_this) {
        return function() {
          if (!_this._losingFocusToElement) {
            _this._losingFocusFrames = 0;
            return;
          }
          _this._losingFocusFrames -= 1;
          if (_this._losingFocusFrames === 0) {
            return _this._onDefinitelyFocusedOut();
          } else {
            return window.requestAnimationFrame(attempt);
          }
        };
      })(this);
      if (!this._losingFocusFrames) {
        window.requestAnimationFrame(attempt);
      }
      return this._losingFocusFrames = 10;
    };

    KeyCommandsRegion.prototype._onDefinitelyFocusedOut = function() {
      if (!this._losingFocusToElement) {
        return;
      }
      if (!this.state.focused) {
        return;
      }
      if (ReactDOM.findDOMNode(this).contains(document.activeElement)) {
        return;
      }
      if (this._losingFocusToElement.tagName !== 'INPUT') {
        document.getSelection().empty();
      }
      this.props.onFocusOut(this._lastFocusElement);
      this.setState({
        focused: false
      });
      return this._losingFocusToElement = null;
    };

    KeyCommandsRegion.prototype.render = function() {
      var classname, otherProps;
      classname = classNames({
        'key-commands-region': true,
        'focused': this.state.focused
      });
      otherProps = _.omit(this.props, Object.keys(this.constructor.propTypes));
      return React.createElement("div", React.__spread({
        "className": "" + classname + " " + this.props.className
      }, otherProps), this.props.children);
    };

    return KeyCommandsRegion;

  })(React.Component);

  module.exports = KeyCommandsRegion;

}).call(this);
