(function() {
  var DatabaseStore, ListSelection, Model, _,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  Model = require('../flux/models/model');

  DatabaseStore = require('../flux/stores/database-store');

  module.exports = ListSelection = (function() {
    function ListSelection(_view, callback) {
      this._view = _view;
      if (!this._view) {
        throw new Error("new ListSelection(): You must provide a view.");
      }
      this._unlisten = DatabaseStore.listen(this._applyChangeRecord, this);
      this._caches = {};
      this._items = [];
      this.trigger = (function(_this) {
        return function() {
          _this._caches = {};
          return callback();
        };
      })(this);
    }

    ListSelection.prototype.cleanup = function() {
      return this._unlisten();
    };

    ListSelection.prototype.count = function() {
      return this._items.length;
    };

    ListSelection.prototype.ids = function() {
      var _base;
      if ((_base = this._caches)['ids'] == null) {
        _base['ids'] = _.pluck(this._items, 'id');
      }
      return this._caches['ids'];
    };

    ListSelection.prototype.items = function() {
      return _.clone(this._items);
    };

    ListSelection.prototype.top = function() {
      return this._items[this._items.length - 1];
    };

    ListSelection.prototype.clear = function() {
      return this.set([]);
    };

    ListSelection.prototype.set = function(items) {
      var item, _i, _len;
      this._items = [];
      for (_i = 0, _len = items.length; _i < _len; _i++) {
        item = items[_i];
        if (!(item instanceof Model)) {
          throw new Error("set must be called with Models");
        }
        this._items.push(item);
      }
      return this.trigger(this);
    };

    ListSelection.prototype.toggle = function(item) {
      var without;
      if (!item) {
        return;
      }
      if (!(item instanceof Model)) {
        throw new Error("toggle must be called with a Model");
      }
      without = _.reject(this._items, function(t) {
        return t.id === item.id;
      });
      if (without.length < this._items.length) {
        this._items = without;
      } else {
        this._items.push(item);
      }
      return this.trigger(this);
    };

    ListSelection.prototype.add = function(item) {
      var updated;
      if (!item) {
        return;
      }
      if (!(item instanceof Model)) {
        throw new Error("add must be called with a Model");
      }
      updated = _.reject(this._items, function(t) {
        return t.id === item.id;
      });
      updated.push(item);
      if (updated.length !== this._items.length) {
        this._items = updated;
        return this.trigger(this);
      }
    };

    ListSelection.prototype.remove = function(items) {
      var item, itemIds, without, _i, _len;
      if (!items) {
        return;
      }
      if (!(items instanceof Array)) {
        items = [items];
      }
      for (_i = 0, _len = items.length; _i < _len; _i++) {
        item = items[_i];
        if (!(item instanceof Model)) {
          throw new Error("remove: Must be passed a model or array of models");
        }
      }
      itemIds = _.pluck(items, 'id');
      without = _.reject(this._items, function(t) {
        var _ref;
        return _ref = t.id, __indexOf.call(itemIds, _ref) >= 0;
      });
      if (without.length < this._items.length) {
        this._items = without;
        return this.trigger(this);
      }
    };

    ListSelection.prototype.removeItemsNotMatching = function(matchers) {
      var count;
      count = this._items.length;
      this._items = _.filter(this._items, function(t) {
        return t.matches(matchers);
      });
      if (this._items.length !== count) {
        return this.trigger(this);
      }
    };

    ListSelection.prototype.expandTo = function(item) {
      var endIdx, idx, relativeTo, startIdx, _i;
      if (!item) {
        return;
      }
      if (!(item instanceof Model)) {
        throw new Error("expandTo must be called with a Model");
      }
      if (this._items.length === 0) {
        this._items.push(item);
      } else {
        relativeTo = this._items[this._items.length - 1];
        startIdx = this._view.indexOfId(relativeTo.id);
        endIdx = this._view.indexOfId(item.id);
        if (startIdx === -1 || endIdx === -1) {
          return;
        }
        for (idx = _i = startIdx; startIdx <= endIdx ? _i <= endIdx : _i >= endIdx; idx = startIdx <= endIdx ? ++_i : --_i) {
          item = this._view.get(idx);
          this._items = _.reject(this._items, function(t) {
            return t.id === item.id;
          });
          this._items.push(item);
        }
      }
      return this.trigger();
    };

    ListSelection.prototype.walk = function(_arg) {
      var current, ids, neitherSelected, next, noSelection, selectionPostPopHeadId;
      current = _arg.current, next = _arg.next;
      ids = this.ids();
      noSelection = this._items.length === 0;
      neitherSelected = (!current || ids.indexOf(current.id) === -1) && (!next || ids.indexOf(next.id) === -1);
      if (noSelection || neitherSelected) {
        if (current) {
          this._items.push(current);
        }
        if (next) {
          this._items.push(next);
        }
      } else {
        selectionPostPopHeadId = null;
        if (this._items.length > 1) {
          selectionPostPopHeadId = this._items[this._items.length - 2].id;
        }
        if (next.id === selectionPostPopHeadId) {
          this._items.pop();
        } else {
          this._items = _.reject(this._items, function(t) {
            return t.id === next.id;
          });
          this._items.push(next);
        }
      }
      return this.trigger();
    };

    ListSelection.prototype._applyChangeRecord = function(change) {
      var existing, idx, newer, touched, _i, _j, _len, _len1, _ref, _ref1;
      if (this._items.length === 0) {
        return;
      }
      if (change.objectClass !== this._items[0].constructor.name) {
        return;
      }
      if (change.type === 'unpersist') {
        return this.remove(change.objects);
      } else if (change.type === 'persist') {
        touched = 0;
        _ref = change.objects;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          newer = _ref[_i];
          _ref1 = this._items;
          for (idx = _j = 0, _len1 = _ref1.length; _j < _len1; idx = ++_j) {
            existing = _ref1[idx];
            if (existing.id === newer.id) {
              this._items[idx] = newer;
              touched += 1;
              break;
            }
          }
        }
        if (touched > 0) {
          return this.trigger(this);
        }
      }
    };

    return ListSelection;

  })();

}).call(this);
