(function() {
  var ListColumn, ListDataSource, ListSelection, ListTabular, ListTabularItem, React, ReactDOM, ScrollRegion, Spinner, Utils, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  React = require('react');

  ReactDOM = require('react-dom');

  ScrollRegion = require('./scroll-region');

  Spinner = require('./spinner');

  Utils = require('nylas-exports').Utils;

  ListDataSource = require('./list-data-source');

  ListSelection = require('./list-selection');

  ListTabularItem = require('./list-tabular-item');

  ListColumn = (function() {
    function ListColumn(_arg) {
      this.name = _arg.name, this.resolver = _arg.resolver, this.flex = _arg.flex, this.width = _arg.width;
    }

    return ListColumn;

  })();

  ListTabular = (function(_super) {
    __extends(ListTabular, _super);

    ListTabular.displayName = 'ListTabular';

    ListTabular.propTypes = {
      columns: React.PropTypes.arrayOf(React.PropTypes.object).isRequired,
      dataSource: React.PropTypes.object,
      itemPropsProvider: React.PropTypes.func,
      itemHeight: React.PropTypes.number,
      onSelect: React.PropTypes.func,
      onClick: React.PropTypes.func,
      onDoubleClick: React.PropTypes.func
    };

    function ListTabular(props) {
      this.props = props;
      this.scrollByPage = __bind(this.scrollByPage, this);
      this.scrollTo = __bind(this.scrollTo, this);
      this._rowForItem = __bind(this._rowForItem, this);
      this._rows = __bind(this._rows, this);
      this.render = __bind(this.render, this);
      this.updateRangeState = __bind(this.updateRangeState, this);
      this.onScroll = __bind(this.onScroll, this);
      this.onWindowResize = __bind(this.onWindowResize, this);
      this.onCleanupAnimatingItems = __bind(this.onCleanupAnimatingItems, this);
      this.componentDidUpdate = __bind(this.componentDidUpdate, this);
      this.buildStateForRange = __bind(this.buildStateForRange, this);
      this.setupDataSource = __bind(this.setupDataSource, this);
      this.componentWillReceiveProps = __bind(this.componentWillReceiveProps, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      if (!this.props.itemHeight) {
        throw new Error("ListTabular: You must provide an itemHeight - raising to avoid divide by zero errors.");
      }
      this.state = this.buildStateForRange({
        start: -1,
        end: -1
      });
    }

    ListTabular.prototype.componentDidMount = function() {
      window.addEventListener('resize', this.onWindowResize, true);
      this.setupDataSource(this.props.dataSource);
      return this.updateRangeState();
    };

    ListTabular.prototype.componentWillUnmount = function() {
      window.removeEventListener('resize', this.onWindowResize, true);
      if (this._cleanupAnimationTimeout) {
        window.clearTimeout(this._cleanupAnimationTimeout);
      }
      return typeof this._unlisten === "function" ? this._unlisten() : void 0;
    };

    ListTabular.prototype.componentWillReceiveProps = function(nextProps) {
      if (nextProps.dataSource !== this.props.dataSource) {
        return this.setupDataSource(nextProps.dataSource);
      }
    };

    ListTabular.prototype.setupDataSource = function(dataSource) {
      if (typeof this._unlisten === "function") {
        this._unlisten();
      }
      this._unlisten = dataSource.listen((function(_this) {
        return function() {
          return _this.setState(_this.buildStateForRange());
        };
      })(this));
      return this.setState(this.buildStateForRange({
        start: -1,
        end: -1,
        dataSource: dataSource
      }));
    };

    ListTabular.prototype.buildStateForRange = function(_arg) {
      var animatingCount, animatingOut, dataSource, end, items, nextIds, previousIdx, previousItem, record, recordIdx, start, _i, _ref, _ref1, _ref2, _ref3, _ref4, _results;
      _ref = _arg != null ? _arg : {}, dataSource = _ref.dataSource, start = _ref.start, end = _ref.end;
      if (start == null) {
        start = this.state.renderedRangeStart;
      }
      if (end == null) {
        end = this.state.renderedRangeEnd;
      }
      if (dataSource == null) {
        dataSource = this.props.dataSource;
      }
      items = {};
      animatingOut = {};
      (function() {
        _results = [];
        for (var _i = start; start <= end ? _i <= end : _i >= end; start <= end ? _i++ : _i--){ _results.push(_i); }
        return _results;
      }).apply(this).forEach((function(_this) {
        return function(idx) {
          return items[idx] = dataSource.get(idx);
        };
      })(this));
      if (this.state && start === this.state.renderedRangeStart) {
        nextIds = _.pluck(_.values(items), 'id');
        animatingOut = {};
        _ref1 = this.state.animatingOut;
        for (recordIdx in _ref1) {
          record = _ref1[recordIdx];
          if (Date.now() < record.end && !(_ref2 = record.item.id, __indexOf.call(nextIds, _ref2) >= 0)) {
            animatingOut[recordIdx] = record;
          }
        }
        _ref3 = this.state.items;
        for (previousIdx in _ref3) {
          previousItem = _ref3[previousIdx];
          if (!previousItem || (_ref4 = previousItem.id, __indexOf.call(nextIds, _ref4) >= 0)) {
            continue;
          }
          animatingOut[previousIdx] = {
            item: previousItem,
            idx: previousIdx,
            end: Date.now() + 125
          };
        }
        animatingCount = Object.keys(animatingOut).length;
        if (animatingCount > 8 || animatingCount === Object.keys(this.state.items).length) {
          animatingOut = {};
        }
      }
      return {
        renderedRangeStart: start,
        renderedRangeEnd: end,
        count: dataSource.count(),
        loaded: dataSource.loaded(),
        empty: dataSource.empty(),
        items: items,
        animatingOut: animatingOut
      };
    };

    ListTabular.prototype.componentDidUpdate = function(prevProps, prevState) {
      if (prevProps.dataSource !== this.props.dataSource) {
        this.refs.container.scrollTop = 0;
      }
      if (!this.updateRangeStateFiring) {
        this.updateRangeState();
      }
      this.updateRangeStateFiring = false;
      if (!this._cleanupAnimationTimeout) {
        return this._cleanupAnimationTimeout = window.setTimeout(this.onCleanupAnimatingItems, 50);
      }
    };

    ListTabular.prototype.onCleanupAnimatingItems = function() {
      var idx, nextAnimatingOut, record, _ref;
      this._cleanupAnimationTimeout = null;
      nextAnimatingOut = {};
      _ref = this.state.animatingOut;
      for (idx in _ref) {
        record = _ref[idx];
        if (Date.now() < record.end) {
          nextAnimatingOut[idx] = record;
        }
      }
      if (Object.keys(nextAnimatingOut).length < Object.keys(this.state.animatingOut).length) {
        this.setState({
          animatingOut: nextAnimatingOut
        });
      }
      if (Object.keys(nextAnimatingOut).length > 0) {
        return this._cleanupAnimationTimeout = window.setTimeout(this.onCleanupAnimatingItems, 50);
      }
    };

    ListTabular.prototype.onWindowResize = function() {
      if (this._onWindowResize == null) {
        this._onWindowResize = _.debounce(this.updateRangeState, 50);
      }
      return this._onWindowResize();
    };

    ListTabular.prototype.onScroll = function() {
      return this.updateRangeState();
    };

    ListTabular.prototype.updateRangeState = function() {
      var rangeEnd, rangeSize, rangeStart, scrollTop;
      scrollTop = this.refs.container.scrollTop;
      rangeStart = Math.floor(scrollTop / this.props.itemHeight);
      rangeSize = Math.ceil(window.innerHeight / this.props.itemHeight);
      rangeEnd = rangeStart + rangeSize;
      rangeStart = Math.max(0, rangeStart - 2);
      rangeEnd = Math.min(rangeEnd + 2, this.state.count + 1);
      if (rangeStart === this.state.renderedRangeStart && rangeEnd === this.state.renderedRangeEnd) {
        return;
      }
      this.updateRangeStateFiring = true;
      this.props.dataSource.setRetainedRange({
        start: rangeStart,
        end: rangeEnd
      });
      return this.setState(this.buildStateForRange({
        start: rangeStart,
        end: rangeEnd
      }));
    };

    ListTabular.prototype.render = function() {
      var emptyElement, innerStyles, otherProps;
      otherProps = Utils.fastOmit(this.props, Object.keys(this.constructor.propTypes));
      innerStyles = {
        height: this.state.count * this.props.itemHeight
      };
      emptyElement = false;
      if (this.props.emptyComponent) {
        emptyElement = React.createElement(this.props.emptyComponent, {
          "visible": this.state.loaded && this.state.empty
        });
      }
      return React.createElement("div", {
        "className": "list-container list-tabular " + this.props.className
      }, React.createElement(ScrollRegion, {
        "ref": "container",
        "onScroll": this.onScroll,
        "tabIndex": "-1",
        "scrollTooltipComponent": this.props.scrollTooltipComponent
      }, React.createElement("div", React.__spread({
        "className": "list-rows",
        "style": innerStyles
      }, otherProps), this._rows())), React.createElement(Spinner, {
        "visible": !this.state.loaded && this.state.empty
      }), emptyElement);
    };

    ListTabular.prototype._rows = function() {
      var idx, record, results, _i, _ref, _ref1, _ref2, _results;
      results = [];
      _ref = this.state.animatingOut;
      for (idx in _ref) {
        record = _ref[idx];
        results.push(this._rowForItem(record.item, idx / 1));
      }
      (function() {
        _results = [];
        for (var _i = _ref1 = this.state.renderedRangeStart, _ref2 = this.state.renderedRangeEnd; _ref1 <= _ref2 ? _i <= _ref2 : _i >= _ref2; _ref1 <= _ref2 ? _i++ : _i--){ _results.push(_i); }
        return _results;
      }).apply(this).forEach((function(_this) {
        return function(idx) {
          if (_this.state.items[idx]) {
            return results.push(_this._rowForItem(_this.state.items[idx], idx));
          }
        };
      })(this));
      return results;
    };

    ListTabular.prototype._rowForItem = function(item, idx) {
      var _base, _ref, _ref1;
      if (!item) {
        return false;
      }
      return React.createElement(ListTabularItem, {
        "key": (_ref = item.id) != null ? _ref : idx,
        "item": item,
        "itemProps": (_ref1 = typeof (_base = this.props).itemPropsProvider === "function" ? _base.itemPropsProvider(item, idx) : void 0) != null ? _ref1 : {},
        "metrics": {
          top: idx * this.props.itemHeight,
          height: this.props.itemHeight
        },
        "columns": this.props.columns,
        "onSelect": this.props.onSelect,
        "onClick": this.props.onClick,
        "onReorder": this.props.onReorder,
        "onDoubleClick": this.props.onDoubleClick
      });
    };

    ListTabular.prototype.scrollTo = function(node) {
      return this.refs.container.scrollTo(node);
    };

    ListTabular.prototype.scrollByPage = function(direction) {
      var height;
      height = ReactDOM.findDOMNode(this.refs.container).clientHeight;
      return this.refs.container.scrollTop += height * direction;
    };

    return ListTabular;

  })(React.Component);

  ListTabular.Item = ListTabularItem;

  ListTabular.Column = ListColumn;

  ListTabular.Selection = ListSelection;

  ListTabular.DataSource = ListDataSource;

  module.exports = ListTabular;

}).call(this);
