(function() {
  var AccountStore, Actions, CategoryStore, FocusedPerspectiveStore, MailImportantIcon, React, ShowImportantKey, TaskFactory, Thread, Utils, classNames, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  React = require('react');

  classNames = require('classnames');

  _ref = require('nylas-exports'), Actions = _ref.Actions, Utils = _ref.Utils, Thread = _ref.Thread, TaskFactory = _ref.TaskFactory, CategoryStore = _ref.CategoryStore, FocusedPerspectiveStore = _ref.FocusedPerspectiveStore, AccountStore = _ref.AccountStore;

  ShowImportantKey = 'core.workspace.showImportant';

  MailImportantIcon = (function(_super) {
    __extends(MailImportantIcon, _super);

    MailImportantIcon.displayName = 'MailImportantIcon';

    MailImportantIcon.propTypes = {
      thread: React.PropTypes.object,
      showIfAvailableForAnyAccount: React.PropTypes.bool
    };

    function MailImportantIcon(props) {
      this.props = props;
      this._onToggleImportant = __bind(this._onToggleImportant, this);
      this.render = __bind(this.render, this);
      this.shouldComponentUpdate = __bind(this.shouldComponentUpdate, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentWillReceiveProps = __bind(this.componentWillReceiveProps, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.getState = __bind(this.getState, this);
      this.state = this.getState();
    }

    MailImportantIcon.prototype.getState = function(props) {
      var account, accountId, accountImportant, category, isImportant, perspective, visible, _i, _len, _ref1;
      if (props == null) {
        props = this.props;
      }
      category = null;
      visible = false;
      if (props.showIfAvailableForAnyAccount) {
        perspective = FocusedPerspectiveStore.current();
        _ref1 = perspective.accountIds;
        for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
          accountId = _ref1[_i];
          account = AccountStore.accountForId(accountId);
          accountImportant = CategoryStore.getStandardCategory(account, 'important');
          if (accountImportant) {
            visible = true;
          }
          if (accountId === props.thread.accountId) {
            category = accountImportant;
          }
          if (visible && category) {
            break;
          }
        }
      } else {
        category = CategoryStore.getStandardCategory(props.thread.accountId, 'important');
        visible = category != null;
      }
      isImportant = category && (_.findWhere(props.thread.categories, {
        id: category.id
      }) != null);
      return {
        visible: visible,
        category: category,
        isImportant: isImportant
      };
    };

    MailImportantIcon.prototype.componentDidMount = function() {
      this.unsubscribe = FocusedPerspectiveStore.listen((function(_this) {
        return function() {
          return _this.setState(_this.getState());
        };
      })(this));
      return this.subscription = NylasEnv.config.onDidChange(ShowImportantKey, (function(_this) {
        return function() {
          return _this.setState(_this.getState());
        };
      })(this));
    };

    MailImportantIcon.prototype.componentWillReceiveProps = function(nextProps) {
      return this.setState(this.getState(nextProps));
    };

    MailImportantIcon.prototype.componentWillUnmount = function() {
      var _ref1;
      if (typeof this.unsubscribe === "function") {
        this.unsubscribe();
      }
      return (_ref1 = this.subscription) != null ? _ref1.dispose() : void 0;
    };

    MailImportantIcon.prototype.shouldComponentUpdate = function(nextProps, nextState) {
      return !_.isEqual(nextState, this.state);
    };

    MailImportantIcon.prototype.render = function() {
      var classes, title;
      if (!this.state.visible) {
        return false;
      }
      classes = classNames({
        'mail-important-icon': true,
        'enabled': this.state.category != null,
        'active': this.state.isImportant
      });
      if (!this.state.category) {
        title = "No important folder / label";
      } else if (this.state.isImportant) {
        title = "Mark as unimportant";
      } else {
        title = "Mark as important";
      }
      return React.createElement("div", {
        "className": classes,
        "title": title,
        "onClick": this._onToggleImportant
      });
    };

    MailImportantIcon.prototype._onToggleImportant = function(event) {
      var category, isImportant, task, threads;
      category = this.state.category;
      if (category) {
        isImportant = _.findWhere(this.props.thread.categories, {
          id: category.id
        }) != null;
        threads = [this.props.thread];
        if (!isImportant) {
          task = TaskFactory.taskForApplyingCategory({
            threads: threads,
            category: category
          });
        } else {
          task = TaskFactory.taskForRemovingCategory({
            threads: threads,
            category: category
          });
        }
        Actions.queueTask(task);
      }
      return event.stopPropagation();
    };

    return MailImportantIcon;

  })(React.Component);

  module.exports = MailImportantIcon;

}).call(this);
