(function() {
  var CompositeDisposable, DOMUtils, Menu, MenuItem, MenuNameEmailItem, React, ReactDOM, classNames, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require('react');

  ReactDOM = require('react-dom');

  classNames = require('classnames');

  _ = require('underscore');

  DOMUtils = require('nylas-exports').DOMUtils;

  CompositeDisposable = require('event-kit').CompositeDisposable;


  /*
  Public: `MenuItem` components can be provided to the {Menu} by the `itemContent` function.
  MenuItem's props allow you to display dividers as well as standard items.
  
  Section: Component Kit
   */

  MenuItem = (function(_super) {
    __extends(MenuItem, _super);

    function MenuItem() {
      this.render = __bind(this.render, this);
      return MenuItem.__super__.constructor.apply(this, arguments);
    }

    MenuItem.displayName = 'MenuItem';


    /*
    Public: React `props` supported by MenuItem:
    
     - `divider` (optional) Pass a {Boolean} to render the menu item as a section divider.
     - `key` (optional) Pass a {String} to be the React key to optimize rendering lists of items.
     - `selected` (optional) Pass a {Boolean} to specify whether the item is selected.
     - `checked` (optional) Pass a {Boolean} to specify whether the item is checked.
     */

    MenuItem.propTypes = {
      divider: React.PropTypes.oneOfType([React.PropTypes.string, React.PropTypes.bool]),
      selected: React.PropTypes.bool,
      checked: React.PropTypes.bool
    };

    MenuItem.prototype.render = function() {
      var className, dividerLabel;
      if (this.props.divider) {
        dividerLabel = _.isString(this.props.divider) ? this.props.divider : '';
        return React.createElement("div", {
          "className": "item divider"
        }, dividerLabel);
      } else {
        className = classNames({
          "item": true,
          "selected": this.props.selected,
          "checked": this.props.checked
        });
        return React.createElement("div", {
          "className": className,
          "onMouseDown": this.props.onMouseDown
        }, this.props.content);
      }
    };

    return MenuItem;

  })(React.Component);


  /*
  Public: React component for a {Menu} item that displays a name and email address.
  
  Section: Component Kit
   */

  MenuNameEmailItem = (function(_super) {
    __extends(MenuNameEmailItem, _super);

    function MenuNameEmailItem() {
      this.render = __bind(this.render, this);
      return MenuNameEmailItem.__super__.constructor.apply(this, arguments);
    }

    MenuNameEmailItem.displayName = 'MenuNameEmailItem';


    /*
    Public: React `props` supported by MenuNameEmailItem:
    
     - `name` (optional) The {String} name to be displayed.
     - `email` (optional) The {String} email address to be displayed.
     */

    MenuNameEmailItem.propTypes = {
      name: React.PropTypes.string,
      email: React.PropTypes.string
    };

    MenuNameEmailItem.prototype.render = function() {
      var _ref;
      if (((_ref = this.props.name) != null ? _ref.length : void 0) > 0 && this.props.name !== this.props.email) {
        return React.createElement("span", null, React.createElement("span", {
          "className": "primary"
        }, this.props.name), React.createElement("span", {
          "className": "secondary"
        }, "(" + this.props.email + ")"));
      } else {
        return React.createElement("span", {
          "className": "primary"
        }, this.props.email);
      }
    };

    return MenuNameEmailItem;

  })(React.Component);


  /*
  Public: React component for multi-section Menus with key binding
  
  The Menu component allows you to display a list of items. Menu takes care of
  several important things, ensuring that your menu is consistent with the rest
  of the N1 application and offers a near-native experience:
  
  - Keyboard Interaction with the Up and Down arrow keys, Enter to select
  - Maintaining selection across content changes
  - Highlighted state
  
  Menus are often, but not always, used in conjunction with {Popover} to display
  a floating "popup" menu. See `template-picker.cjsx` for an example.
  
  The Menu also exposes "header" and "footer" regions you can fill with arbitrary
  components by providing the `headerComponents` and `footerComponents` props.
  These items are nested within `.header-container`. and `.footer-container`,
  and you can customize their appearance by providing CSS selectors scoped to your
  component's Menu instance:
  
  ```css
  .template-picker .menu .header-container {
    height: 100px;
  }
  ```
  
  Section: Component Kit
   */

  Menu = (function(_super) {
    __extends(Menu, _super);

    Menu.displayName = 'Menu';


    /*
    Public: React `props` supported by Menu:
    
     - `className` (optional) The {String} class name applied to the Menu
    
     - `itemContent` A {Function} that returns a {MenuItem}, {String}, or
       React component for the given `item`.
    
       If you return a {MenuItem}, your item is injected into the list directly.
    
       If you return a string or React component, the result is placed within a
       {MenuItem}, resulting in the following DOM:
       `<div className="item [selected]">{your content}</div>`.
    
       To create dividers and other special menu items, return an instance of:
    
       <Menu.Item divider="Label">
    
     - `itemKey` A {Function} that returns a unique string key for the given `item`.
       Keys are important for efficient React rendering when `items` is changed, and a
       key function is required.
    
     - `itemChecked` A {Function} that returns true if the given item should be shown
       with a checkmark. If you don't provide an implementation for `itemChecked`, no
       checkmarks are ever shown.
    
     - `items` An {Array} of arbitrary objects the menu should display.
    
     - `onSelect` A {Function} called with the selected item when the user clicks
       an item in the menu or confirms their selection with the Enter key.
    
     - `onEscape` A {Function} called when a user presses escape in the input.
    
     - `defaultSelectedIndex` The index of the item first selected if there
     was no other previous index. Defaults to 0. Set to -1 if you want
     nothing selected.
     */

    Menu.propTypes = {
      className: React.PropTypes.string,
      footerComponents: React.PropTypes.arrayOf(React.PropTypes.element),
      headerComponents: React.PropTypes.arrayOf(React.PropTypes.element),
      itemContent: React.PropTypes.func.isRequired,
      itemKey: React.PropTypes.func.isRequired,
      itemChecked: React.PropTypes.func,
      items: React.PropTypes.array.isRequired,
      onSelect: React.PropTypes.func.isRequired,
      onEscape: React.PropTypes.func,
      defaultSelectedIndex: React.PropTypes.number
    };

    Menu.defaultProps = {
      onEscape: function() {}
    };

    function Menu(props) {
      var _ref;
      this.props = props;
      this._onEscape = __bind(this._onEscape, this);
      this._onEnter = __bind(this._onEnter, this);
      this._onShiftSelectedIndex = __bind(this._onShiftSelectedIndex, this);
      this._itemComponentForItem = __bind(this._itemComponentForItem, this);
      this._contentContainer = __bind(this._contentContainer, this);
      this._onKeyDown = __bind(this._onKeyDown, this);
      this.render = __bind(this.render, this);
      this.componentDidUpdate = __bind(this.componentDidUpdate, this);
      this.componentWillReceiveProps = __bind(this.componentWillReceiveProps, this);
      this.setSelectedItem = __bind(this.setSelectedItem, this);
      this.getSelectedItem = __bind(this.getSelectedItem, this);
      this.state = {
        selectedIndex: (_ref = this.props.defaultSelectedIndex) != null ? _ref : 0
      };
    }

    Menu.prototype.getSelectedItem = function() {
      return this.props.items[this.state.selectedIndex];
    };

    Menu.prototype.setSelectedItem = function(item) {
      return this.setState({
        selectedIndex: this.props.items.indexOf(item)
      });
    };

    Menu.prototype.componentWillReceiveProps = function(newProps) {
      var newSelection, newSelectionIndex, selection, selectionKey, _ref;
      if (this.state.selectedIndex >= 0) {
        selection = this.props.items[this.state.selectedIndex];
        newSelectionIndex = 0;
      } else {
        newSelectionIndex = (_ref = newProps.defaultSelectedIndex) != null ? _ref : -1;
      }
      if (selection != null) {
        selectionKey = this.props.itemKey(selection);
        newSelection = _.find(newProps.items, (function(_this) {
          return function(item) {
            return _this.props.itemKey(item) === selectionKey;
          };
        })(this));
        if (newSelection != null) {
          newSelectionIndex = newProps.items.indexOf(newSelection);
        }
      }
      return this.setState({
        selectedIndex: newSelectionIndex
      });
    };

    Menu.prototype.componentDidUpdate = function() {
      var adjustment, container, item;
      item = ReactDOM.findDOMNode(this).querySelector(".selected");
      container = ReactDOM.findDOMNode(this).querySelector(".content-container");
      adjustment = DOMUtils.scrollAdjustmentToMakeNodeVisibleInContainer(item, container);
      if (adjustment !== 0) {
        return container.scrollTop += adjustment;
      }
    };

    Menu.prototype.render = function() {
      var fc, hc, _ref, _ref1;
      hc = (_ref = this.props.headerComponents) != null ? _ref : [];
      if (hc.length === 0) {
        hc = React.createElement("span", null);
      }
      fc = (_ref1 = this.props.footerComponents) != null ? _ref1 : [];
      if (fc.length === 0) {
        fc = React.createElement("span", null);
      }
      return React.createElement("div", {
        "onKeyDown": this._onKeyDown,
        "className": "menu " + this.props.className,
        "tabIndex": "-1"
      }, React.createElement("div", {
        "className": "header-container"
      }, hc), this._contentContainer(), React.createElement("div", {
        "className": "footer-container"
      }, fc));
    };

    Menu.prototype._onKeyDown = function(event) {
      var _ref;
      if (this.props.items.length === 0) {
        return;
      }
      event.stopPropagation();
      if ((_ref = event.key) === "Enter" || _ref === "Return") {
        this._onEnter();
      }
      if (event.key === "Escape") {
        this._onEscape();
      } else if (event.key === "ArrowUp" || (event.key === "Tab" && event.shiftKey)) {
        this._onShiftSelectedIndex(-1);
        event.preventDefault();
      } else if (event.key === "ArrowDown" || event.key === "Tab") {
        this._onShiftSelectedIndex(1);
        event.preventDefault();
      }
    };

    Menu.prototype._contentContainer = function() {
      var contentClass, items, _ref;
      items = (_ref = this.props.items.map(this._itemComponentForItem)) != null ? _ref : [];
      contentClass = classNames({
        'content-container': true,
        'empty': items.length === 0
      });
      return React.createElement("div", {
        "className": contentClass
      }, items);
    };

    Menu.prototype._itemComponentForItem = function(item, i) {
      var content, onMouseDown, _base;
      content = this.props.itemContent(item);
      if (React.isValidElement(content) && content.type === MenuItem) {
        return content;
      }
      onMouseDown = (function(_this) {
        return function(event) {
          event.preventDefault();
          if (_this.props.onSelect) {
            return _this.props.onSelect(item);
          }
        };
      })(this);
      return React.createElement(MenuItem, {
        "onMouseDown": onMouseDown,
        "key": this.props.itemKey(item),
        "checked": typeof (_base = this.props).itemChecked === "function" ? _base.itemChecked(item) : void 0,
        "content": content,
        "selected": this.state.selectedIndex === i
      });
    };

    Menu.prototype._onShiftSelectedIndex = function(delta) {
      var index, isDivider, item, _ref;
      if (this.props.items.length === 0) {
        return;
      }
      index = this.state.selectedIndex + delta;
      isDivider = true;
      while (isDivider) {
        item = this.props.items[index];
        if (!item) {
          break;
        }
        if ((_ref = this.props.itemContent(item).props) != null ? _ref.divider : void 0) {
          if (delta > 0) {
            index += 1;
          } else if (delta < 0) {
            index -= 1;
          }
        } else {
          isDivider = false;
        }
      }
      index = Math.max(0, Math.min(this.props.items.length - 1, index));
      return this.setState({
        selectedIndex: index
      });
    };

    Menu.prototype._onEnter = function() {
      var item;
      item = this.props.items[this.state.selectedIndex];
      if (item != null) {
        return this.props.onSelect(item);
      }
    };

    Menu.prototype._onEscape = function() {
      return this.props.onEscape();
    };

    return Menu;

  })(React.Component);

  Menu.Item = MenuItem;

  Menu.NameEmailItem = MenuNameEmailItem;

  module.exports = Menu;

}).call(this);
