(function() {
  var Actions, Flexbox, InjectedComponentSet, MultiselectActionBar, React, ReactCSSTransitionGroup, RetinaImg, Utils, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require("react");

  ReactCSSTransitionGroup = require('react-addons-css-transition-group');

  _ = require('underscore');

  _ref = require("nylas-exports"), Utils = _ref.Utils, Actions = _ref.Actions;

  InjectedComponentSet = require('./injected-component-set');

  RetinaImg = requir('./retina-img'["default"])["default"];

  Flexbox = require('./flexbox');


  /*
  Public: MultiselectActionBar is a simple component that can be placed in a {Sheet} Toolbar.
  When the provided `dataStore` has a selection, it appears over the other items in the toolbar.
  
  Generally, you wrap {MultiselectActionBar} in your own simple component to provide a dataStore
  and other settings:
  
  ```coffee
  class MultiselectActionBar extends React.Component
    @displayName: 'MultiselectActionBar'
  
    render: =>
      <MultiselectActionBar
        dataStore={ThreadListStore}
        className="thread-list"
        collection="thread" />
  ```
  
  The MultiselectActionBar uses the `ComponentRegistry` to find items to display for the given
  collection name. To add an item to the bar created in the example above, register it like this:
  
  ```coffee
  ComponentRegistry.register ThreadBulkTrashButton,
    role: 'thread:Toolbar'
  ```
  
  Section: Component Kit
   */

  MultiselectActionBar = (function(_super) {
    __extends(MultiselectActionBar, _super);

    MultiselectActionBar.displayName = 'MultiselectActionBar';


    /*
    Public: React `props` supported by MultiselectActionBar:
    
     - `dataStore` An instance of a {ListDataSource}.
     - `collection` The name of the collection. The collection name is used for the text
        that appears in the bar "1 thread selected" and is also used to find components
        in the component registry that should appear in the bar (`thread` => `thread:BulkAtion`)
     */

    MultiselectActionBar.propTypes = {
      collection: React.PropTypes.string.isRequired,
      dataSource: React.PropTypes.object
    };

    function MultiselectActionBar(props) {
      this.props = props;
      this._onClearSelection = __bind(this._onClearSelection, this);
      this._onChange = __bind(this._onChange, this);
      this._getStateFromStores = __bind(this._getStateFromStores, this);
      this._label = __bind(this._label, this);
      this._renderActions = __bind(this._renderActions, this);
      this._renderBar = __bind(this._renderBar, this);
      this.render = __bind(this.render, this);
      this.shouldComponentUpdate = __bind(this.shouldComponentUpdate, this);
      this.setupForProps = __bind(this.setupForProps, this);
      this.teardownForProps = __bind(this.teardownForProps, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentWillReceiveProps = __bind(this.componentWillReceiveProps, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.state = this._getStateFromStores();
    }

    MultiselectActionBar.prototype.componentDidMount = function() {
      return this.setupForProps(this.props);
    };

    MultiselectActionBar.prototype.componentWillReceiveProps = function(newProps) {
      if (_.isEqual(this.props, newProps)) {
        return;
      }
      this.teardownForProps();
      this.setupForProps(newProps);
      return this.setState(this._getStateFromStores(newProps));
    };

    MultiselectActionBar.prototype.componentWillUnmount = function() {
      return this.teardownForProps();
    };

    MultiselectActionBar.prototype.teardownForProps = function() {
      var unsubscribe, _i, _len, _ref1, _results;
      if (!this._unsubscribers) {
        return;
      }
      _ref1 = this._unsubscribers;
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        unsubscribe = _ref1[_i];
        _results.push(unsubscribe());
      }
      return _results;
    };

    MultiselectActionBar.prototype.setupForProps = function(props) {
      this._unsubscribers = [];
      return this._unsubscribers.push(props.dataSource.listen(this._onChange));
    };

    MultiselectActionBar.prototype.shouldComponentUpdate = function(nextProps, nextState) {
      return !Utils.isEqualReact(nextProps, this.props) || !Utils.isEqualReact(nextState, this.state);
    };

    MultiselectActionBar.prototype.render = function() {
      return React.createElement(ReactCSSTransitionGroup, {
        "className": "selection-bar",
        "transitionName": "selection-bar-absolute",
        "component": "div",
        "transitionLeaveTimeout": 200.,
        "transitionEnterTimeout": 200.
      }, (this.state.items.length > 0 ? this._renderBar() : []));
    };

    MultiselectActionBar.prototype._renderBar = function() {
      return React.createElement("div", {
        "className": "absolute",
        "key": "absolute"
      }, React.createElement("div", {
        "className": "inner"
      }, this._renderActions(), React.createElement("div", {
        "className": "centered"
      }, this._label()), React.createElement("button", {
        "style": {
          order: 100
        },
        "className": "btn btn-toolbar",
        "onClick": this._onClearSelection
      }, "Clear Selection")));
    };

    MultiselectActionBar.prototype._renderActions = function() {
      if (!this.props.dataSource) {
        return React.createElement("div", null);
      }
      return React.createElement(InjectedComponentSet, {
        "matching": {
          role: "" + this.props.collection + ":Toolbar"
        },
        "exposedProps": {
          selection: this.props.dataSource.selection,
          items: this.state.items
        }
      });
    };

    MultiselectActionBar.prototype._label = function() {
      if (this.state.items.length > 1) {
        return "" + this.state.items.length + " " + this.props.collection + "s selected";
      } else if (this.state.items.length === 1) {
        return "" + this.state.items.length + " " + this.props.collection + " selected";
      } else {
        return "";
      }
    };

    MultiselectActionBar.prototype._getStateFromStores = function(props) {
      var _ref1;
      if (props == null) {
        props = this.props;
      }
      return {
        items: (_ref1 = props.dataSource.selection.items()) != null ? _ref1 : []
      };
    };

    MultiselectActionBar.prototype._onChange = function() {
      return this.setState(this._getStateFromStores());
    };

    MultiselectActionBar.prototype._onClearSelection = function() {
      this.props.dataSource.selection.clear();
    };

    return MultiselectActionBar;

  })(React.Component);

  module.exports = MultiselectActionBar;

}).call(this);
