(function() {
  var Actions, MultiselectListInteractionHandler, WorkspaceStore, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require('underscore');

  _ref = require('nylas-exports'), Actions = _ref.Actions, WorkspaceStore = _ref.WorkspaceStore;

  module.exports = MultiselectListInteractionHandler = (function() {
    function MultiselectListInteractionHandler(props) {
      var _ref1;
      this.props = props;
      this._keyboardContext = __bind(this._keyboardContext, this);
      this.onShift = __bind(this.onShift, this);
      this.onSelect = __bind(this.onSelect, this);
      this.onEnter = __bind(this.onEnter, this);
      this.onShiftClick = __bind(this.onShiftClick, this);
      this.onMetaClick = __bind(this.onMetaClick, this);
      this.onClick = __bind(this.onClick, this);
      this.shouldShowKeyboardCursor = __bind(this.shouldShowKeyboardCursor, this);
      this.shouldShowCheckmarks = __bind(this.shouldShowCheckmarks, this);
      this.shouldShowFocus = __bind(this.shouldShowFocus, this);
      this.cssClass = __bind(this.cssClass, this);
      _ref1 = this.props, this.onFocusItem = _ref1.onFocusItem, this.onSetCursorPosition = _ref1.onSetCursorPosition;
    }

    MultiselectListInteractionHandler.prototype.cssClass = function() {
      return 'handler-list';
    };

    MultiselectListInteractionHandler.prototype.shouldShowFocus = function() {
      return false;
    };

    MultiselectListInteractionHandler.prototype.shouldShowCheckmarks = function() {
      return true;
    };

    MultiselectListInteractionHandler.prototype.shouldShowKeyboardCursor = function() {
      return true;
    };

    MultiselectListInteractionHandler.prototype.onClick = function(item) {
      return this.onFocusItem(item);
    };

    MultiselectListInteractionHandler.prototype.onMetaClick = function(item) {
      this.props.dataSource.selection.toggle(item);
      return this.onSetCursorPosition(item);
    };

    MultiselectListInteractionHandler.prototype.onShiftClick = function(item) {
      this.props.dataSource.selection.expandTo(item);
      return this.onSetCursorPosition(item);
    };

    MultiselectListInteractionHandler.prototype.onEnter = function() {
      var item, keyboardCursorId;
      keyboardCursorId = this.props.keyboardCursorId;
      if (keyboardCursorId) {
        item = this.props.dataSource.getById(keyboardCursorId);
        return this.onFocusItem(item);
      }
    };

    MultiselectListInteractionHandler.prototype.onSelect = function() {
      var id;
      id = this._keyboardContext().id;
      if (!id) {
        return;
      }
      return this.props.dataSource.selection.toggle(this.props.dataSource.getById(id));
    };

    MultiselectListInteractionHandler.prototype.onShift = function(delta, options) {
      var action, current, id, index, next, _ref1;
      if (options == null) {
        options = {};
      }
      _ref1 = this._keyboardContext(), id = _ref1.id, action = _ref1.action;
      current = this.props.dataSource.getById(id);
      index = this.props.dataSource.indexOfId(id);
      index = Math.max(0, Math.min(index + delta, this.props.dataSource.count() - 1));
      next = this.props.dataSource.get(index);
      action(next);
      if (options.select) {
        return this.props.dataSource.selection.walk({
          current: current,
          next: next
        });
      }
    };

    MultiselectListInteractionHandler.prototype._keyboardContext = function() {
      if (WorkspaceStore.topSheet().root) {
        return {
          id: this.props.keyboardCursorId,
          action: this.onSetCursorPosition
        };
      } else {
        return {
          id: this.props.focusedId,
          action: this.onFocusItem
        };
      }
    };

    return MultiselectListInteractionHandler;

  })();

}).call(this);
