(function() {
  var AccountStore, Actions, EventEmitter, KeyCommandsRegion, ListTabular, MultiselectList, MultiselectListInteractionHandler, MultiselectSplitInteractionHandler, React, ReactDOM, Spinner, Utils, WorkspaceStore, classNames, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  React = require('react');

  ReactDOM = require('react-dom');

  classNames = require('classnames');

  ListTabular = require('./list-tabular');

  Spinner = require('./spinner');

  _ref = require('nylas-exports'), Actions = _ref.Actions, Utils = _ref.Utils, WorkspaceStore = _ref.WorkspaceStore, AccountStore = _ref.AccountStore;

  KeyCommandsRegion = require('nylas-component-kit').KeyCommandsRegion;

  EventEmitter = require('events').EventEmitter;

  MultiselectListInteractionHandler = require('./multiselect-list-interaction-handler');

  MultiselectSplitInteractionHandler = require('./multiselect-split-interaction-handler');


  /*
  Public: MultiselectList wraps {ListTabular} and makes it easy to present a
  {ListDataSource} with selection support. It adds a checkbox column to the columns
  you provide, and also handles:
  
  - Command-clicking individual items
  - Shift-clicking to select a range
  - Using the keyboard to select a range
  
  Section: Component Kit
   */

  MultiselectList = (function(_super) {
    __extends(MultiselectList, _super);

    MultiselectList.displayName = 'MultiselectList';

    MultiselectList.propTypes = {
      dataSource: React.PropTypes.object,
      className: React.PropTypes.string.isRequired,
      columns: React.PropTypes.array.isRequired,
      itemPropsProvider: React.PropTypes.func.isRequired,
      keymapHandlers: React.PropTypes.object
    };

    function MultiselectList(props) {
      this.props = props;
      this._getStateFromStores = __bind(this._getStateFromStores, this);
      this._getCheckmarkColumn = __bind(this._getCheckmarkColumn, this);
      this._visible = __bind(this._visible, this);
      this._onChange = __bind(this._onChange, this);
      this._onScrollByPage = __bind(this._onScrollByPage, this);
      this._onShift = __bind(this._onShift, this);
      this._onDeselect = __bind(this._onDeselect, this);
      this._onSelectAll = __bind(this._onSelectAll, this);
      this._onSelect = __bind(this._onSelect, this);
      this._onEnter = __bind(this._onEnter, this);
      this._onClickItem = __bind(this._onClickItem, this);
      this.render = __bind(this.render, this);
      this.setupForProps = __bind(this.setupForProps, this);
      this.teardownForProps = __bind(this.teardownForProps, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentDidUpdate = __bind(this.componentDidUpdate, this);
      this.componentWillReceiveProps = __bind(this.componentWillReceiveProps, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.state = this._getStateFromStores();
    }

    MultiselectList.prototype.componentDidMount = function() {
      return this.setupForProps(this.props);
    };

    MultiselectList.prototype.componentWillReceiveProps = function(newProps) {
      if (_.isEqual(this.props, newProps)) {
        return;
      }
      this.teardownForProps();
      this.setupForProps(newProps);
      return this.setState(this._getStateFromStores(newProps));
    };

    MultiselectList.prototype.componentDidUpdate = function(prevProps, prevState) {
      var item;
      if (prevProps.focusedId !== this.props.focusedId || prevProps.keyboardCursorId !== this.props.keyboardCursorId) {
        item = ReactDOM.findDOMNode(this).querySelector(".focused");
        if (item == null) {
          item = ReactDOM.findDOMNode(this).querySelector(".keyboard-cursor");
        }
        if (!(item instanceof Node)) {
          return;
        }
        return this.refs.list.scrollTo(item);
      }
    };

    MultiselectList.prototype.componentWillUnmount = function() {
      return this.teardownForProps();
    };

    MultiselectList.prototype.teardownForProps = function() {
      var unsubscribe, _i, _len, _ref1, _results;
      if (!this.unsubscribers) {
        return;
      }
      _ref1 = this.unsubscribers;
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        unsubscribe = _ref1[_i];
        _results.push(unsubscribe());
      }
      return _results;
    };

    MultiselectList.prototype.setupForProps = function(props) {
      this.unsubscribers = [];
      return this.unsubscribers.push(WorkspaceStore.listen(this._onChange));
    };

    MultiselectList.prototype._globalKeymapHandlers = function() {
      return _.extend({}, this.props.keymapHandlers, {
        'core:focus-item': (function(_this) {
          return function() {
            return _this._onEnter();
          };
        })(this),
        'core:select-item': (function(_this) {
          return function() {
            return _this._onSelect();
          };
        })(this),
        'core:next-item': (function(_this) {
          return function() {
            return _this._onShift(1);
          };
        })(this),
        'core:previous-item': (function(_this) {
          return function() {
            return _this._onShift(-1);
          };
        })(this),
        'core:select-down': (function(_this) {
          return function() {
            return _this._onShift(1, {
              select: true
            });
          };
        })(this),
        'core:select-up': (function(_this) {
          return function() {
            return _this._onShift(-1, {
              select: true
            });
          };
        })(this),
        'core:list-page-up': (function(_this) {
          return function() {
            return _this._onScrollByPage(-1);
          };
        })(this),
        'core:list-page-down': (function(_this) {
          return function() {
            return _this._onScrollByPage(1);
          };
        })(this),
        'core:pop-sheet': (function(_this) {
          return function() {
            return _this._onDeselect();
          };
        })(this),
        'multiselect-list:select-all': (function(_this) {
          return function() {
            return _this._onSelectAll();
          };
        })(this),
        'multiselect-list:deselect-all': (function(_this) {
          return function() {
            return _this._onDeselect();
          };
        })(this)
      });
    };

    MultiselectList.prototype.render = function() {
      var className, otherProps;
      otherProps = Utils.fastOmit(this.props, Object.keys(this.constructor.propTypes));
      className = this.props.className;
      if (this.props.dataSource && this.state.handler) {
        className += " " + this.state.handler.cssClass();
        if (this.itemPropsProvider == null) {
          this.itemPropsProvider = (function(_this) {
            return function(item, idx) {
              var nextId, nextSelected, props, selected, selectedIds, _ref1, _ref2;
              selectedIds = _this.props.dataSource.selection.ids();
              selected = (_ref1 = item.id, __indexOf.call(selectedIds, _ref1) >= 0);
              if (!selected) {
                nextId = (_ref2 = _this.props.dataSource.get(idx + 1)) != null ? _ref2.id : void 0;
                nextSelected = __indexOf.call(selectedIds, nextId) >= 0;
              }
              props = _this.props.itemPropsProvider(item, idx);
              if (props.className == null) {
                props.className = '';
              }
              props.className += " " + classNames({
                'selected': selected,
                'next-is-selected': !selected && nextSelected,
                'focused': _this.state.handler.shouldShowFocus() && item.id === _this.props.focusedId,
                'keyboard-cursor': _this.state.handler.shouldShowKeyboardCursor() && item.id === _this.props.keyboardCursorId
              });
              props['data-item-id'] = item.id;
              return props;
            };
          })(this);
        }
        return React.createElement(KeyCommandsRegion, {
          "globalHandlers": this._globalKeymapHandlers(),
          "className": className
        }, React.createElement(ListTabular, React.__spread({
          "ref": "list",
          "columns": this.state.computedColumns,
          "dataSource": this.props.dataSource,
          "itemPropsProvider": this.itemPropsProvider,
          "onSelect": this._onClickItem
        }, otherProps)));
      } else {
        return React.createElement("div", React.__spread({
          "className": className
        }, otherProps), React.createElement(Spinner, {
          "visible": true
        }));
      }
    };

    MultiselectList.prototype._onClickItem = function(item, event) {
      if (!this.state.handler) {
        return;
      }
      if (event.metaKey || event.ctrlKey) {
        return this.state.handler.onMetaClick(item);
      } else if (event.shiftKey) {
        return this.state.handler.onShiftClick(item);
      } else {
        return this.state.handler.onClick(item);
      }
    };

    MultiselectList.prototype._onEnter = function() {
      if (!this.state.handler) {
        return;
      }
      return this.state.handler.onEnter();
    };

    MultiselectList.prototype._onSelect = function() {
      if (!this.state.handler) {
        return;
      }
      return this.state.handler.onSelect();
    };

    MultiselectList.prototype._onSelectAll = function() {
      var items;
      if (!this.state.handler) {
        return;
      }
      items = this.props.dataSource.itemsCurrentlyInViewMatching(function() {
        return true;
      });
      return this.props.dataSource.selection.set(items);
    };

    MultiselectList.prototype._onDeselect = function() {
      if (!(this._visible() && this.props.dataSource)) {
        return;
      }
      return this.props.dataSource.selection.clear();
    };

    MultiselectList.prototype._onShift = function(delta, options) {
      if (options == null) {
        options = {};
      }
      if (!this.state.handler) {
        return;
      }
      return this.state.handler.onShift(delta, options);
    };

    MultiselectList.prototype._onScrollByPage = function(delta) {
      return this.refs.list.scrollByPage(delta);
    };

    MultiselectList.prototype._onChange = function() {
      return this.setState(this._getStateFromStores());
    };

    MultiselectList.prototype._visible = function() {
      if (this.state.layoutMode) {
        return WorkspaceStore.topSheet().root;
      } else {
        return true;
      }
    };

    MultiselectList.prototype._getCheckmarkColumn = function() {
      return new ListTabular.Column({
        name: 'Check',
        resolver: (function(_this) {
          return function(item) {
            var toggle;
            toggle = function(event) {
              if (event.shiftKey) {
                _this.state.handler.onShiftClick(item);
              } else {
                _this.state.handler.onMetaClick(item);
              }
              return event.stopPropagation();
            };
            return React.createElement("div", {
              "className": "checkmark",
              "onClick": toggle
            }, React.createElement("div", {
              "className": "inner"
            }));
          };
        })(this)
      });
    };

    MultiselectList.prototype._getStateFromStores = function(props) {
      var computedColumns, handler, layoutMode, state, _ref1;
      if (props == null) {
        props = this.props;
      }
      state = (_ref1 = this.state) != null ? _ref1 : {};
      layoutMode = WorkspaceStore.layoutMode();
      if (props.columns !== state.columns || layoutMode !== state.layoutMode) {
        computedColumns = [].concat(props.columns);
        if (layoutMode === 'list') {
          computedColumns.splice(0, 0, this._getCheckmarkColumn());
        }
      } else {
        computedColumns = state.computedColumns;
      }
      if (layoutMode === 'list') {
        handler = new MultiselectListInteractionHandler(props);
      } else {
        handler = new MultiselectSplitInteractionHandler(props);
      }
      return {
        handler: handler,
        columns: props.columns,
        computedColumns: computedColumns,
        layoutMode: layoutMode
      };
    };

    MultiselectList.prototype.itemIdAtPoint = function(x, y) {
      var item;
      item = document.elementFromPoint(x, y).closest('[data-item-id]');
      if (!item) {
        return null;
      }
      return item.dataset.itemId;
    };

    return MultiselectList;

  })(React.Component);

  module.exports = MultiselectList;

}).call(this);
