(function() {
  var Actions, MultiselectSplitInteractionHandler, WorkspaceStore, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require('underscore');

  _ref = require('nylas-exports'), Actions = _ref.Actions, WorkspaceStore = _ref.WorkspaceStore;

  module.exports = MultiselectSplitInteractionHandler = (function() {
    function MultiselectSplitInteractionHandler(props) {
      var _ref1;
      this.props = props;
      this._checkSelectionAndFocusConsistency = __bind(this._checkSelectionAndFocusConsistency, this);
      this._turnFocusIntoSelection = __bind(this._turnFocusIntoSelection, this);
      this.onShift = __bind(this.onShift, this);
      this.onSelect = __bind(this.onSelect, this);
      this.onEnter = __bind(this.onEnter, this);
      this.onShiftClick = __bind(this.onShiftClick, this);
      this.onMetaClick = __bind(this.onMetaClick, this);
      this.onClick = __bind(this.onClick, this);
      this.shouldShowKeyboardCursor = __bind(this.shouldShowKeyboardCursor, this);
      this.shouldShowCheckmarks = __bind(this.shouldShowCheckmarks, this);
      this.shouldShowFocus = __bind(this.shouldShowFocus, this);
      this.cssClass = __bind(this.cssClass, this);
      _ref1 = this.props, this.onFocusItem = _ref1.onFocusItem, this.onSetCursorPosition = _ref1.onSetCursorPosition;
    }

    MultiselectSplitInteractionHandler.prototype.cssClass = function() {
      return 'handler-split';
    };

    MultiselectSplitInteractionHandler.prototype.shouldShowFocus = function() {
      return true;
    };

    MultiselectSplitInteractionHandler.prototype.shouldShowCheckmarks = function() {
      return false;
    };

    MultiselectSplitInteractionHandler.prototype.shouldShowKeyboardCursor = function() {
      return this.props.dataSource.selection.count() > 1;
    };

    MultiselectSplitInteractionHandler.prototype.onClick = function(item) {
      this.onFocusItem(item);
      this.props.dataSource.selection.clear();
      return this._checkSelectionAndFocusConsistency();
    };

    MultiselectSplitInteractionHandler.prototype.onMetaClick = function(item) {
      this._turnFocusIntoSelection();
      this.props.dataSource.selection.toggle(item);
      return this._checkSelectionAndFocusConsistency();
    };

    MultiselectSplitInteractionHandler.prototype.onShiftClick = function(item) {
      this._turnFocusIntoSelection();
      this.props.dataSource.selection.expandTo(item);
      return this._checkSelectionAndFocusConsistency();
    };

    MultiselectSplitInteractionHandler.prototype.onEnter = function() {};

    MultiselectSplitInteractionHandler.prototype.onSelect = function() {
      return this._checkSelectionAndFocusConsistency();
    };

    MultiselectSplitInteractionHandler.prototype.onShift = function(delta, options) {
      var action, current, id, index, keyboardId, next, selection;
      if (options.select) {
        this._turnFocusIntoSelection();
      }
      if (this.props.dataSource.selection.count() > 0) {
        selection = this.props.dataSource.selection;
        keyboardId = this.props.keyboardCursorId;
        id = keyboardId != null ? keyboardId : this.props.dataSource.selection.top().id;
        action = this.onSetCursorPosition;
      } else {
        id = this.props.focusedId;
        action = this.onFocusItem;
      }
      current = this.props.dataSource.getById(id);
      index = this.props.dataSource.indexOfId(id);
      index = Math.max(0, Math.min(index + delta, this.props.dataSource.count() - 1));
      next = this.props.dataSource.get(index);
      action(next);
      if (options.select) {
        this.props.dataSource.selection.walk({
          current: current,
          next: next
        });
      }
      return this._checkSelectionAndFocusConsistency();
    };

    MultiselectSplitInteractionHandler.prototype._turnFocusIntoSelection = function() {
      var focused;
      focused = this.props.focused;
      this.onFocusItem(null);
      return this.props.dataSource.selection.add(focused);
    };

    MultiselectSplitInteractionHandler.prototype._checkSelectionAndFocusConsistency = function() {
      var focused, selection;
      focused = this.props.focused;
      selection = this.props.dataSource.selection;
      if (focused && selection.count() > 0) {
        this.props.dataSource.selection.add(focused);
        this.onFocusItem(null);
      }
      if (selection.count() === 1 && !focused) {
        this.onFocusItem(selection.items()[0]);
        return this.props.dataSource.selection.clear();
      }
    };

    return MultiselectSplitInteractionHandler;

  })();

}).call(this);
