(function() {
  var CompositeDisposable, Popover, React, ReactDOM, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require('react');

  ReactDOM = require('react-dom');

  _ = require('underscore');

  CompositeDisposable = require('event-kit').CompositeDisposable;


  /*
  Public: The Popover component makes it easy to display a sheet or popup menu when the
  user clicks the React element provided as `buttonComponent`. In N1, the Popover
  component is used to create rich dropdown menus, detail popups, etc. with consistent
  look and feel and behavior.
  
  The Popover component handles:
  
  - Rendering it's children when you click `buttonComponent`, and dismissing it's
    children when you click outside the popover or press the Escape key.
  
  - Automatically focusing the item with the lowest tabIndex inside the popover
  
   *# Input Focus
  
  If your Popover contains an input, like a search bar, give it a tabIndex and
  Popover will automatically focus it when the popover is opened.
  
   *# Advanced Use
  
  If you don't want to use the Popover in conjunction with a triggering button,
  you can manually call `open()` and `close()` to display it. A typical scenario
  looks like this:
  
  ```coffeescript
  render: =>
    <Popover ref="myPopover"> Popover Contents </Popover>
  
  showMyPopover: =>
    @refs.myPopover.open()
  
  ```
  
  Section: Component Kit
   */

  Popover = (function(_super) {

    /*
    Public: React `props` supported by Popover:
    
     - `buttonComponent` The React element that will be rendered in place of the
       Popover and trigger it to appear. This is typically a button or call-to-action for
       opening the popover. Popover wraps this item in a <div> with an onClick handler.
    
     - `children` The React elements that should appear when the Popover is opened.
       They're automatically wrapped in a `<div class="popover">`, which applies standard
       shadowing and styles.
    
    - `direction` Defaults to 'up'. You can also pass 'down' to make the Popover float beneath
      the button component.
    
    - `pointerStyle` Additional styles to apply to the pointer
    
    - `popoverStyle` Additional styles to apply to the popover
    
    Events
    
    - `onOpened` A {Function} that will be called when the popover is opened.
     */
    __extends(Popover, _super);

    Popover.propTypes = {
      buttonComponent: React.PropTypes.element,
      direction: React.PropTypes.string,
      popoverStyle: React.PropTypes.object,
      pointerStyle: React.PropTypes.object
    };

    Popover.defaultProps = {
      direction: 'up'
    };

    function Popover(props) {
      this.props = props;
      this._onBlur = __bind(this._onBlur, this);
      this._onClick = __bind(this._onClick, this);
      this._onKeyDown = __bind(this._onKeyDown, this);
      this.render = __bind(this.render, this);
      this._focusImportantElement = __bind(this._focusImportantElement, this);
      this._resetPositionState = __bind(this._resetPositionState, this);
      this.close = __bind(this.close, this);
      this.open = __bind(this.open, this);
      this.componentDidUpdate = __bind(this.componentDidUpdate, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.state = {
        showing: false,
        offset: 0,
        dimensions: {}
      };
    }

    Popover.prototype.componentDidMount = function() {
      window.addEventListener("resize", this._resetPositionState);
      return this._resetPositionState();
    };

    Popover.prototype.componentWillUnmount = function() {
      return window.removeEventListener("resize", this._resetPositionState);
    };

    Popover.prototype.componentDidUpdate = function() {
      if (this._focusOnOpen) {
        this._focusImportantElement();
        this._focusOnOpen = false;
      }
      return this._resetPositionState();
    };

    Popover.prototype.open = function() {
      var _base;
      this._focusOnOpen = true;
      this.setState({
        showing: true
      });
      return typeof (_base = this.props).onOpened === "function" ? _base.onOpened() : void 0;
    };

    Popover.prototype.close = function() {
      var _base;
      this.setState({
        showing: false
      });
      return typeof (_base = this.props).onClosed === "function" ? _base.onClosed() : void 0;
    };

    Popover.prototype._resetPositionState = function() {
      var dimensions, offset, origLeft, origRight, padding, rect;
      if (!this.state.showing) {
        return;
      }
      rect = ReactDOM.findDOMNode(this.refs.popover).getBoundingClientRect();
      dimensions = {
        left: rect.left,
        right: rect.right,
        docWidth: document.body.clientWidth
      };
      if (_.isEqual(dimensions, this.state.dimensions)) {
        return;
      }
      padding = 11.25;
      origRight = dimensions.right - this.state.offset;
      origLeft = dimensions.left - this.state.offset;
      offset = Math.min(dimensions.docWidth - padding - origRight, 0) - Math.min(origLeft - padding, 0);
      return this.setState({
        offset: offset,
        dimensions: dimensions
      });
    };

    Popover.prototype._focusImportantElement = function() {
      var matches, node, _ref;
      node = ReactDOM.findDOMNode(this.refs.popover);
      matches = _.sortBy(node.querySelectorAll("[tabIndex], input"), (function(_this) {
        return function(node) {
          if (node.tabIndex > 0) {
            return node.tabIndex;
          } else if (node.nodeName === "INPUT") {
            return 1000000;
          } else {
            return 1000001;
          }
        };
      })(this));
      return (_ref = matches[0]) != null ? _ref.focus() : void 0;
    };

    Popover.prototype.render = function() {
      var pointerStyle, popoverComponent, popoverStyle, wrappedButtonComponent, _ref;
      wrappedButtonComponent = [];
      if (this.props.buttonComponent) {
        wrappedButtonComponent = React.createElement("div", {
          "onClick": this._onClick
        }, this.props.buttonComponent);
      }
      popoverComponent = [];
      if (this.state.showing) {
        popoverStyle = {
          'position': 'absolute',
          'left': "calc(50% + " + this.state.offset + "px)",
          'zIndex': 40
        };
        pointerStyle = {
          'position': 'absolute',
          'marginLeft': '50%',
          'zoom': 0.5,
          'width': 45,
          'height': 20,
          'zIndex': 40
        };
        if (this.props.direction === 'up') {
          popoverStyle = _.extend(popoverStyle, {
            'transform': 'translate(-50%,-100%)',
            'top': -10
          });
          pointerStyle = _.extend(pointerStyle, {
            'transform': 'translateX(-50%)',
            'bottom': 48
          });
        } else if (this.props.direction === 'down') {
          popoverStyle = _.extend(popoverStyle, {
            'transform': 'translate(-50%, 15px)',
            'top': '100%'
          });
          pointerStyle = _.extend(pointerStyle, {
            'transform': 'rotateX(180deg)',
            'top': 71,
            'left': -12
          });
        }
        if (this.props.direction === "down-align-left") {
          popoverStyle = _.extend(popoverStyle, {
            'transform': 'translate(0, 2px)',
            'top': '100%',
            'left': 0 + this.state.offset
          });
          pointerStyle = _.extend(pointerStyle, {
            'transform': 'rotateX(180deg)',
            'top': 71,
            'display': 'none'
          });
        }
        if (this.props.popoverStyle) {
          popoverStyle = _.extend({}, popoverStyle, this.props.popoverStyle);
        }
        if (this.props.pointerStyle) {
          pointerStyle = _.extend({}, pointerStyle, this.props.pointerStyle);
        }
        popoverComponent = React.createElement("div", {
          "ref": "popover",
          "className": "popover popover-" + this.props.direction,
          "style": popoverStyle
        }, this.props.children);
      }
      return React.createElement("div", {
        "className": "popover-container " + this.props.className,
        "onBlur": this._onBlur,
        "onKeyDown": this._onKeyDown,
        "style": (_ref = this.props.style) != null ? _ref : {},
        "ref": "popoverContainer"
      }, wrappedButtonComponent, popoverComponent, React.createElement("div", {
        "className": "popover-pointer",
        "style": pointerStyle
      }), React.createElement("div", {
        "className": "popover-pointer shadow",
        "style": pointerStyle
      }));
    };

    Popover.prototype._onKeyDown = function(event) {
      if (event.key === "Escape") {
        return this.close();
      }
    };

    Popover.prototype._onClick = function(e) {
      e.stopPropagation();
      if (!this.state.showing) {
        return this.open();
      } else {
        return this.close();
      }
    };

    Popover.prototype._onBlur = function(event) {
      var target;
      target = event.nativeEvent.relatedTarget;
      if ((target != null) && ReactDOM.findDOMNode(this.refs.popoverContainer).contains(target)) {
        return;
      }
      return this.setState({
        showing: false
      });
    };

    return Popover;

  })(React.Component);

  module.exports = Popover;

}).call(this);
