(function() {
  var Actions, ComponentRegistry, PriorityUICoordinator, React, ReactDOM, ResizableHandle, ResizableRegion, Utils, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require('react');

  ReactDOM = require('react-dom');

  _ = require('underscore');

  _ref = require("nylas-exports"), Utils = _ref.Utils, Actions = _ref.Actions, ComponentRegistry = _ref.ComponentRegistry, PriorityUICoordinator = _ref.PriorityUICoordinator;

  ResizableHandle = {
    Top: {
      axis: 'vertical',
      className: 'flexbox-handle-vertical flexbox-handle-top',
      transform: function(state, props, event) {
        var _ref1, _ref2;
        return {
          'height': Math.min((_ref1 = props.maxHeight) != null ? _ref1 : 10000, Math.max((_ref2 = props.minHeight) != null ? _ref2 : 0, state.bcr.bottom - event.pageY))
        };
      }
    },
    Bottom: {
      axis: 'vertical',
      className: 'flexbox-handle-vertical flexbox-handle-bottom',
      transform: function(state, props, event) {
        var _ref1, _ref2;
        return {
          'height': Math.min((_ref1 = props.maxHeight) != null ? _ref1 : 10000, Math.max((_ref2 = props.minHeight) != null ? _ref2 : 0, event.pageY - state.bcr.top))
        };
      }
    },
    Left: {
      axis: 'horizontal',
      className: 'flexbox-handle-horizontal flexbox-handle-left',
      transform: function(state, props, event) {
        var _ref1, _ref2;
        return {
          'width': Math.min((_ref1 = props.maxWidth) != null ? _ref1 : 10000, Math.max((_ref2 = props.minWidth) != null ? _ref2 : 0, state.bcr.right - event.pageX))
        };
      }
    },
    Right: {
      axis: 'horizontal',
      className: 'flexbox-handle-horizontal flexbox-handle-right',
      transform: function(state, props, event) {
        var _ref1, _ref2;
        return {
          'width': Math.min((_ref1 = props.maxWidth) != null ? _ref1 : 10000, Math.max((_ref2 = props.minWidth) != null ? _ref2 : 0, event.pageX - state.bcr.left))
        };
      }
    }
  };


  /*
  Public: ResizableRegion wraps it's `children` in a div with a fixed width or height, and a
  draggable edge. It is used throughout N1 to implement resizable columns, trays, etc.
  
  Section: Component Kit
   */

  ResizableRegion = (function(_super) {
    __extends(ResizableRegion, _super);

    ResizableRegion.displayName = 'ResizableRegion';


    /*
    Public: React `props` supported by ResizableRegion:
    
     - `handle` Provide a {ResizableHandle} to indicate which edge of the
       region should be draggable.
     - `onResize` A {Function} that will be called continuously as the region is resized.
     - `initialWidth` (optional) Initial width, if the handle indicates a horizontal resizing axis.
     - `minWidth` (optional) Minimum width, if the handle indicates a horizontal resizing axis.
     - `maxWidth` (optional) Maximum width, if the handle indicates a horizontal resizing axis.
     - `initialHeight` (optional) Initial height, if the handle indicates a vertical resizing axis.
     - `minHeight` (optional) Minimum height, if the handle indicates a vertical resizing axis.
     - `maxHeight` (optional) Maximum height, if the handle indicates a vertical resizing axis.
     */

    ResizableRegion.propTypes = {
      handle: React.PropTypes.object.isRequired,
      onResize: React.PropTypes.func,
      initialWidth: React.PropTypes.number,
      minWidth: React.PropTypes.number,
      maxWidth: React.PropTypes.number,
      initialHeight: React.PropTypes.number,
      minHeight: React.PropTypes.number,
      maxHeight: React.PropTypes.number,
      style: React.PropTypes.object
    };

    function ResizableRegion(props) {
      var _base;
      this.props = props != null ? props : {};
      this._mouseMove = __bind(this._mouseMove, this);
      this._mouseUp = __bind(this._mouseUp, this);
      this._mouseDown = __bind(this._mouseDown, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentWillReceiveProps = __bind(this.componentWillReceiveProps, this);
      this.componentDidUpdate = __bind(this.componentDidUpdate, this);
      this.render = __bind(this.render, this);
      if ((_base = this.props).handle == null) {
        _base.handle = ResizableHandle.Right;
      }
      this.state = {
        dragging: false,
        width: this.props.initialWidth,
        height: this.props.initialHeight
      };
    }

    ResizableRegion.prototype.render = function() {
      var containerStyle, otherProps;
      if (this.props.handle.axis === 'horizontal') {
        containerStyle = _.extend({}, this.props.style, {
          'minWidth': this.props.minWidth,
          'maxWidth': this.props.maxWidth,
          'position': 'relative'
        });
        if (this.state.width != null) {
          containerStyle.width = this.state.width;
        } else {
          containerStyle.flex = 1;
        }
      } else {
        containerStyle = _.extend({}, this.props.style, {
          'minHeight': this.props.minHeight,
          'maxHeight': this.props.maxHeight,
          'position': 'relative',
          'width': '100%'
        });
        if (this.state.height != null) {
          containerStyle.height = this.state.height;
        } else {
          containerStyle.flex = 1;
        }
      }
      otherProps = Utils.fastOmit(this.props, Object.keys(this.constructor.propTypes));
      return React.createElement("div", React.__spread({
        "style": containerStyle
      }, otherProps), this.props.children, React.createElement("div", {
        "className": this.props.handle.className,
        "onMouseDown": this._mouseDown
      }, React.createElement("div", null)));
    };

    ResizableRegion.prototype.componentDidUpdate = function(lastProps, lastState) {
      if (lastState.dragging && !this.state.dragging) {
        document.removeEventListener('mousemove', this._mouseMove);
        return document.removeEventListener('mouseup', this._mouseUp);
      } else if (!lastState.dragging && this.state.dragging) {
        document.addEventListener('mousemove', this._mouseMove);
        return document.addEventListener('mouseup', this._mouseUp);
      }
    };

    ResizableRegion.prototype.componentWillReceiveProps = function(nextProps) {
      if (nextProps.handle.axis === 'vertical' && nextProps.initialHeight !== this.props.initialHeight) {
        this.setState({
          height: nextProps.initialHeight
        });
      }
      if (nextProps.handle.axis === 'horizontal' && nextProps.initialWidth !== this.props.initialWidth) {
        return this.setState({
          width: nextProps.initialWidth
        });
      }
    };

    ResizableRegion.prototype.componentWillUnmount = function() {
      if (this._taskId) {
        PriorityUICoordinator.endPriorityTask(this._taskId);
      }
      return this._taskId = null;
    };

    ResizableRegion.prototype._mouseDown = function(event) {
      var bcr;
      if (event.button !== 0) {
        return;
      }
      bcr = ReactDOM.findDOMNode(this).getBoundingClientRect();
      this.setState({
        dragging: true,
        bcr: bcr
      });
      event.stopPropagation();
      event.preventDefault();
      if (this._taskId) {
        PriorityUICoordinator.endPriorityTask(this._taskId);
      }
      return this._taskId = PriorityUICoordinator.beginPriorityTask();
    };

    ResizableRegion.prototype._mouseUp = function(event) {
      var _ref1;
      if (event.button !== 0) {
        return;
      }
      this.setState({
        dragging: false
      });
      if (this.props.onResize) {
        this.props.onResize((_ref1 = this.state.height) != null ? _ref1 : this.state.width);
      }
      event.stopPropagation();
      event.preventDefault();
      PriorityUICoordinator.endPriorityTask(this._taskId);
      return this._taskId = null;
    };

    ResizableRegion.prototype._mouseMove = function(event) {
      var _ref1;
      if (!this.state.dragging) {
        return;
      }
      this.setState(this.props.handle.transform(this.state, this.props, event));
      if (this.props.onResize) {
        this.props.onResize((_ref1 = this.state.height) != null ? _ref1 : this.state.width);
      }
      event.stopPropagation();
      return event.preventDefault();
    };

    return ResizableRegion;

  })(React.Component);

  ResizableRegion.Handle = ResizableHandle;

  module.exports = ResizableRegion;

}).call(this);
