(function() {
  var Comparator, Template, _,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  Comparator = (function() {
    function Comparator(name, fn) {
      this.name = name;
      this.fn = fn;
    }

    Comparator.prototype.evaluate = function(_arg) {
      var actual, desired;
      actual = _arg.actual, desired = _arg.desired;
      if (actual instanceof Array) {
        return _.any(actual, (function(_this) {
          return function(actual) {
            return _this.fn({
              actual: actual,
              desired: desired
            });
          };
        })(this));
      } else {
        return this.fn({
          actual: actual,
          desired: desired
        });
      }
    };

    return Comparator;

  })();

  Comparator.Default = new Comparator('Default', function(_arg) {
    var actual, desired;
    actual = _arg.actual, desired = _arg.desired;
    return _.isEqual(actual, desired);
  });

  Template = (function() {
    Template.Type = {
      None: 'None',
      Enum: 'Enum',
      String: 'String'
    };

    Template.Comparator = Comparator;

    Template.Comparators = {
      String: {
        contains: new Comparator('contains', function(_arg) {
          var actual, desired;
          actual = _arg.actual, desired = _arg.desired;
          if (!(actual && desired)) {
            return false;
          }
          return actual.toLowerCase().indexOf(desired.toLowerCase()) !== -1;
        }),
        doesNotContain: new Comparator('does not contain', function(_arg) {
          var actual, desired;
          actual = _arg.actual, desired = _arg.desired;
          if (!(actual && desired)) {
            return false;
          }
          return actual.toLowerCase().indexOf(desired.toLowerCase()) === -1;
        }),
        beginsWith: new Comparator('begins with', function(_arg) {
          var actual, desired;
          actual = _arg.actual, desired = _arg.desired;
          if (!(actual && desired)) {
            return false;
          }
          return actual.toLowerCase().indexOf(desired.toLowerCase()) === 0;
        }),
        endsWith: new Comparator('ends with', function(_arg) {
          var actual, desired;
          actual = _arg.actual, desired = _arg.desired;
          if (!(actual && desired)) {
            return false;
          }
          return actual.toLowerCase().lastIndexOf(desired.toLowerCase()) === actual.length - desired.length;
        }),
        equals: new Comparator('equals', function(_arg) {
          var actual, desired;
          actual = _arg.actual, desired = _arg.desired;
          return actual === desired;
        }),
        matchesExpression: new Comparator('matches expression', function(_arg) {
          var actual, desired;
          actual = _arg.actual, desired = _arg.desired;
          if (!(actual && desired)) {
            return false;
          }
          return new RegExp(desired, "gi").test(actual);
        })
      }
    };

    function Template(key, type, options) {
      var defaults, _ref;
      this.key = key;
      this.type = type;
      if (options == null) {
        options = {};
      }
      defaults = {
        name: this.key,
        values: void 0,
        valueLabel: void 0,
        comparators: this.constructor.Comparators[this.type] || {}
      };
      _.extend(this, defaults, options);
      if (!this.key) {
        throw new Error("You must provide a valid key.");
      }
      if (_ref = this.type, __indexOf.call(_.values(this.constructor.Type), _ref) < 0) {
        throw new Error("You must provide a valid type.");
      }
      if (this.type === this.constructor.Type.Enum && !this.values) {
        throw new Error("You must provide `values` when creating an enum.");
      }
      this;
    }

    Template.prototype.createDefaultInstance = function() {
      return {
        templateKey: this.key,
        comparatorKey: Object.keys(this.comparators)[0],
        value: void 0
      };
    };

    Template.prototype.coerceInstance = function(instance) {
      var _ref;
      instance.templateKey = this.key;
      if (!this.comparators) {
        instance.comparatorKey = void 0;
      } else if (!(_ref = instance.comparatorKey, __indexOf.call(Object.keys(this.comparators), _ref) >= 0)) {
        instance.comparatorKey = Object.keys(this.comparators)[0];
      }
      return instance;
    };

    Template.prototype.evaluate = function(instance, value) {
      var comparator;
      comparator = this.comparators[instance.comparatorKey];
      if (comparator == null) {
        comparator = Comparator.Default;
      }
      return comparator.evaluate({
        actual: value,
        desired: instance.value
      });
    };

    return Template;

  })();

  module.exports = {
    Template: Template,
    Comparator: Comparator
  };

}).call(this);
