(function() {
  var Actions, Comparator, Flexbox, React, RetinaImg, ScenarioEditor, ScenarioEditorRow, Template, Utils, _, _ref, _ref1, _ref2,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require('react');

  _ = require('underscore');

  _ref = require('./scenario-editor-models'), Comparator = _ref.Comparator, Template = _ref.Template;

  ScenarioEditorRow = require('./scenario-editor-row');

  _ref1 = require('nylas-component-kit'), RetinaImg = _ref1.RetinaImg, Flexbox = _ref1.Flexbox;

  _ref2 = require('nylas-exports'), Actions = _ref2.Actions, Utils = _ref2.Utils;


  /*
  The ScenarioEditor takes an array of ScenarioTemplate objects which define the
  scenario value space. Each ScenarioTemplate defines a `key` and it's valid
  `comparators` and `values`. The ScenarioEditor gives the user the option to
  create and combine instances of different templates to create a scenario.
  
  For example:
  
    Scenario Space:
     - ScenarioFactory("user-name", "The name of the user")
        + valueType: String
        + comparators: "contains", "starts with", etc.
      - SecnarioFactor("profession", "The profession of the user")
        + valueType: Enum
        + comparators: 'is'
  
    Scenario Value:
      [{
        'key': 'user-name'
        'comparator': 'contains'
        'value': 'Ben'
      },{
        'key': 'profession'
        'comparator': 'is'
        'value': 'Engineer'
      }]
   */

  ScenarioEditor = (function(_super) {
    __extends(ScenarioEditor, _super);

    ScenarioEditor.displayName = 'ScenarioEditor';

    ScenarioEditor.propTypes = {
      instances: React.PropTypes.array,
      className: React.PropTypes.string,
      onChange: React.PropTypes.func,
      templates: React.PropTypes.array
    };

    ScenarioEditor.Template = Template;

    ScenarioEditor.Comparator = Comparator;

    function ScenarioEditor(props) {
      this.props = props;
      this._onChangeRowValue = __bind(this._onChangeRowValue, this);
      this._onInsertRule = __bind(this._onInsertRule, this);
      this._onRemoveRule = __bind(this._onRemoveRule, this);
      this._performChange = __bind(this._performChange, this);
      this.render = __bind(this.render, this);
      this.state = {
        collapsed: true
      };
    }

    ScenarioEditor.prototype.render = function() {
      return React.createElement("div", {
        "className": this.props.className
      }, (this.props.instances || []).map((function(_this) {
        return function(instance, idx) {
          return React.createElement(ScenarioEditorRow, {
            "key": idx,
            "instance": instance,
            "removable": _this.props.instances.length > 1,
            "templates": _this.props.templates,
            "onRemove": (function() {
              return _this._onRemoveRule(idx);
            }),
            "onInsert": (function() {
              return _this._onInsertRule(idx);
            }),
            "onChange": (function(instance) {
              return _this._onChangeRowValue(instance, idx);
            })
          });
        };
      })(this)));
    };

    ScenarioEditor.prototype._performChange = function(block) {
      var instances;
      instances = JSON.parse(JSON.stringify(this.props.instances));
      block(instances);
      return this.props.onChange(instances);
    };

    ScenarioEditor.prototype._onRemoveRule = function(idx) {
      return this._performChange((function(_this) {
        return function(instances) {
          if (instances.length === 1) {
            return;
          }
          return instances.splice(idx, 1);
        };
      })(this));
    };

    ScenarioEditor.prototype._onInsertRule = function(idx) {
      return this._performChange((function(_this) {
        return function(instances) {
          return instances.push(_this.props.templates[0].createDefaultInstance());
        };
      })(this));
    };

    ScenarioEditor.prototype._onChangeRowValue = function(newInstance, idx) {
      return this._performChange((function(_this) {
        return function(instances) {
          return instances[idx] = newInstance;
        };
      })(this));
    };

    return ScenarioEditor;

  })(React.Component);

  module.exports = ScenarioEditor;

}).call(this);
