(function() {
  var React, ReactDOM, ScrollRegion, Scrollbar, ScrollbarTicks, Utils, classNames, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  React = require('react');

  ReactDOM = require('react-dom');

  Utils = require('nylas-exports').Utils;

  classNames = require('classnames');

  ScrollbarTicks = require('./scrollbar-ticks')["default"];

  Scrollbar = (function(_super) {
    __extends(Scrollbar, _super);

    Scrollbar.displayName = 'Scrollbar';

    Scrollbar.propTypes = {
      scrollTooltipComponent: React.PropTypes.func,
      scrollbarTickProvider: React.PropTypes.object,
      getScrollRegion: React.PropTypes.func
    };

    function Scrollbar(props) {
      this.props = props;
      this._getHandleHeight = __bind(this._getHandleHeight, this);
      this._onScrollJump = __bind(this._onScrollJump, this);
      this._onHandleClick = __bind(this._onHandleClick, this);
      this._onHandleUp = __bind(this._onHandleUp, this);
      this._onHandleMove = __bind(this._onHandleMove, this);
      this._onHandleDown = __bind(this._onHandleDown, this);
      this._scrollbarWrapStyles = __bind(this._scrollbarWrapStyles, this);
      this._scrollbarHandleStyles = __bind(this._scrollbarHandleStyles, this);
      this._recomputeDimensions = __bind(this._recomputeDimensions, this);
      this._onTickProviderChange = __bind(this._onTickProviderChange, this);
      this.recomputeDimensions = __bind(this.recomputeDimensions, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.shouldComponentUpdate = __bind(this.shouldComponentUpdate, this);
      this.state = {
        totalHeight: 0,
        trackHeight: 0,
        viewportHeight: 0,
        viewportScrollTop: 0,
        dragging: false,
        scrolling: false,
        scrollbarTicks: []
      };
    }

    Scrollbar.prototype.componentDidMount = function() {
      var _ref;
      if ((_ref = this.props.scrollbarTickProvider) != null ? _ref.listen : void 0) {
        return this._tickUnsub = this.props.scrollbarTickProvider.listen(this._onTickProviderChange);
      }
    };

    Scrollbar.prototype.shouldComponentUpdate = function(nextProps, nextState) {
      return !Utils.isEqualReact(nextProps, this.props) || !Utils.isEqualReact(nextState, this.state);
    };

    Scrollbar.prototype.componentWillUnmount = function() {
      this._onHandleUp({
        preventDefault: function() {}
      });
      return typeof this._tickUnsub === "function" ? this._tickUnsub() : void 0;
    };

    Scrollbar.prototype.setStateFromScrollRegion = function(state) {
      return this.setState(state);
    };

    Scrollbar.prototype.render = function() {
      var containerClasses, tooltip;
      containerClasses = classNames({
        'scrollbar-track': true,
        'dragging': this.state.dragging,
        'scrolling': this.state.scrolling,
        'with-ticks': this.state.scrollbarTicks.length > 0
      });
      tooltip = [];
      if (this.props.scrollTooltipComponent && this.state.dragging) {
        tooltip = React.createElement(this.props.scrollTooltipComponent, {
          "viewportCenter": this.state.viewportScrollTop + this.state.viewportHeight / 2,
          "totalHeight": this.state.totalHeight
        });
      }
      return React.createElement("div", {
        "className": containerClasses,
        "style": this._scrollbarWrapStyles(),
        "onMouseEnter": this.recomputeDimensions
      }, React.createElement("div", {
        "className": "scrollbar-track-inner",
        "ref": "track",
        "onClick": this._onScrollJump
      }, this._renderScrollbarTicks(), React.createElement("div", {
        "className": "scrollbar-handle",
        "onMouseDown": this._onHandleDown,
        "style": this._scrollbarHandleStyles(),
        "ref": "handle",
        "onClick": this._onHandleClick
      }, React.createElement("div", {
        "className": "tooltip"
      }, tooltip))));
    };

    Scrollbar.prototype.recomputeDimensions = function(options) {
      if (options == null) {
        options = {};
      }
      if (this.props.getScrollRegion != null) {
        this.props.getScrollRegion()._recomputeDimensions(options);
      }
      return this._recomputeDimensions(options);
    };

    Scrollbar.prototype._onTickProviderChange = function() {
      var _ref;
      if (!((_ref = this.props.scrollbarTickProvider) != null ? _ref.scrollbarTicks : void 0)) {
        throw new Error("The scrollbarTickProvider must implement `scrollbarTicks`");
      }
      return this.setState({
        scrollbarTicks: this.props.scrollbarTickProvider.scrollbarTicks()
      });
    };

    Scrollbar.prototype._renderScrollbarTicks = function() {
      if (!(this.state.scrollbarTicks.length > 0)) {
        return false;
      }
      return React.createElement(ScrollbarTicks, {
        "ticks": this.state.scrollbarTicks
      });
    };

    Scrollbar.prototype._recomputeDimensions = function(_arg) {
      var trackHeight, trackNode, useCachedValues;
      useCachedValues = _arg.useCachedValues;
      if (!useCachedValues) {
        trackNode = ReactDOM.findDOMNode(this.refs.track);
        if (!trackNode) {
          return;
        }
        trackHeight = trackNode.clientHeight;
        if (trackHeight !== this.state.trackHeight) {
          return this.setState({
            trackHeight: trackHeight
          });
        }
      }
    };

    Scrollbar.prototype._scrollbarHandleStyles = function() {
      var handleHeight, handleTop;
      handleHeight = this._getHandleHeight();
      handleTop = (this.state.viewportScrollTop / (this.state.totalHeight - this.state.viewportHeight)) * (this.state.trackHeight - handleHeight);
      return {
        position: 'relative',
        height: handleHeight || 0,
        top: handleTop || 0
      };
    };

    Scrollbar.prototype._scrollbarWrapStyles = function() {
      return {
        position: 'absolute',
        top: 0,
        bottom: 0,
        right: 0,
        zIndex: 2,
        visibility: this.state.totalHeight !== 0 && this.state.totalHeight === this.state.viewportHeight ? "hidden" : void 0
      };
    };

    Scrollbar.prototype._onHandleDown = function(event) {
      var handleNode;
      handleNode = ReactDOM.findDOMNode(this.refs.handle);
      this._trackOffset = ReactDOM.findDOMNode(this.refs.track).getBoundingClientRect().top;
      this._mouseOffsetWithinHandle = event.pageY - handleNode.getBoundingClientRect().top;
      window.addEventListener("mousemove", this._onHandleMove);
      window.addEventListener("mouseup", this._onHandleUp);
      this.setState({
        dragging: true
      });
      return event.preventDefault();
    };

    Scrollbar.prototype._onHandleMove = function(event) {
      var trackPxToViewportPx, trackY;
      trackY = event.pageY - this._trackOffset - this._mouseOffsetWithinHandle;
      trackPxToViewportPx = (this.state.totalHeight - this.state.viewportHeight) / (this.state.trackHeight - this._getHandleHeight());
      this.props.getScrollRegion().scrollTop = trackY * trackPxToViewportPx;
      return event.preventDefault();
    };

    Scrollbar.prototype._onHandleUp = function(event) {
      window.removeEventListener("mousemove", this._onHandleMove);
      window.removeEventListener("mouseup", this._onHandleUp);
      this.setState({
        dragging: false
      });
      return event.preventDefault();
    };

    Scrollbar.prototype._onHandleClick = function(event) {
      return event.stopPropagation();
    };

    Scrollbar.prototype._onScrollJump = function(event) {
      this._trackOffset = ReactDOM.findDOMNode(this.refs.track).getBoundingClientRect().top;
      this._mouseOffsetWithinHandle = this._getHandleHeight() / 2;
      return this._onHandleMove(event);
    };

    Scrollbar.prototype._getHandleHeight = function() {
      return Math.min(this.state.totalHeight, Math.max(40, (this.state.trackHeight / this.state.totalHeight) * this.state.trackHeight));
    };

    return Scrollbar;

  })(React.Component);


  /*
  The ScrollRegion component attaches a custom scrollbar.
   */

  ScrollRegion = (function(_super) {
    __extends(ScrollRegion, _super);

    ScrollRegion.displayName = "ScrollRegion";

    ScrollRegion.propTypes = {
      onScroll: React.PropTypes.func,
      onScrollEnd: React.PropTypes.func,
      className: React.PropTypes.string,
      scrollTooltipComponent: React.PropTypes.func,
      scrollbarTickProvider: React.PropTypes.object,
      children: React.PropTypes.oneOfType([React.PropTypes.element, React.PropTypes.array]),
      getScrollbar: React.PropTypes.func
    };

    ScrollRegion.ScrollPosition = {
      Top: 'Top',
      Bottom: 'Bottom',
      Visible: 'Visible',
      Center: 'Center',
      CenterIfInvisible: 'CenterIfInvisible'
    };

    function ScrollRegion(props) {
      this.props = props;
      this._getSelf = __bind(this._getSelf, this);
      this._onScroll = __bind(this._onScroll, this);
      this._recomputeDimensions = __bind(this._recomputeDimensions, this);
      this.recomputeDimensions = __bind(this.recomputeDimensions, this);
      this._settleHeight = __bind(this._settleHeight, this);
      this.scrollTo = __bind(this.scrollTo, this);
      this.render = __bind(this.render, this);
      this.shouldInvalidateScrollbarComponent = __bind(this.shouldInvalidateScrollbarComponent, this);
      this.shouldComponentUpdate = __bind(this.shouldComponentUpdate, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentWillReceiveProps = __bind(this.componentWillReceiveProps, this);
      this.componentDidUpdate = __bind(this.componentDidUpdate, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this._scrollToTaskId = 0;
      this._scrollbarComponent = null;
      this.state = {
        totalHeight: 0,
        viewportHeight: 0,
        viewportScrollTop: 0,
        scrolling: false
      };
      Object.defineProperty(this, 'scrollTop', {
        get: function() {
          return ReactDOM.findDOMNode(this.refs.content).scrollTop;
        },
        set: function(val) {
          return ReactDOM.findDOMNode(this.refs.content).scrollTop = val;
        }
      });
    }

    ScrollRegion.prototype.componentDidMount = function() {
      this._mounted = true;
      this.recomputeDimensions();
      this._heightObserver = new MutationObserver((function(_this) {
        return function(mutations) {
          var recompute;
          recompute = false;
          mutations.forEach(function(mutation) {
            return recompute || (recompute = !mutation.oldValue || mutation.oldValue.indexOf('height:') !== -1);
          });
          if (recompute) {
            return _this.recomputeDimensions({
              useCachedValues: false
            });
          }
        };
      })(this));
      return this._heightObserver.observe(ReactDOM.findDOMNode(this.refs.content), {
        subtree: true,
        attributes: true,
        attributeOldValue: true,
        attributeFilter: ['style']
      });
    };

    ScrollRegion.prototype.componentDidUpdate = function(prevProps, prevState) {
      if (!this.state.scrolling && this.props.children !== prevProps.children) {
        return this.recomputeDimensions();
      }
    };

    ScrollRegion.prototype.componentWillReceiveProps = function(props) {
      if (this.shouldInvalidateScrollbarComponent(props)) {
        return this._scrollbarComponent = null;
      }
    };

    ScrollRegion.prototype.componentWillUnmount = function() {
      this._heightObserver.disconnect();
      return this._mounted = false;
    };

    ScrollRegion.prototype.shouldComponentUpdate = function(newProps, newState) {
      return true;
    };

    ScrollRegion.prototype.shouldInvalidateScrollbarComponent = function(newProps) {
      if (newProps.scrollTooltipComponent !== this.props.scrollTooltipComponent) {
        return true;
      }
      if (newProps.getScrollbar !== this.props.getScrollbar) {
        return true;
      }
      return false;
    };

    ScrollRegion.prototype.render = function() {
      var containerClasses, otherProps, _ref;
      containerClasses = ("" + ((_ref = this.props.className) != null ? _ref : '') + " ") + classNames({
        'scroll-region': true,
        'dragging': this.state.dragging,
        'scrolling': this.state.scrolling
      });
      if (!this.props.getScrollbar) {
        if (this._scrollbarComponent == null) {
          this._scrollbarComponent = React.createElement(Scrollbar, {
            "ref": "scrollbar",
            "scrollbarTickProvider": this.props.scrollbarTickProvider,
            "scrollTooltipComponent": this.props.scrollTooltipComponent,
            "getScrollRegion": this._getSelf
          });
        }
      }
      otherProps = Utils.fastOmit(this.props, Object.keys(this.constructor.propTypes));
      return React.createElement("div", React.__spread({
        "className": containerClasses
      }, otherProps), this._scrollbarComponent, React.createElement("div", {
        "className": "scroll-region-content",
        "onScroll": this._onScroll,
        "ref": "content"
      }, React.createElement("div", {
        "className": "scroll-region-content-inner"
      }, this.props.children)));
    };

    ScrollRegion.prototype.scrollTo = function(node, _arg) {
      var done, position, settle, _ref;
      _ref = _arg != null ? _arg : {}, position = _ref.position, settle = _ref.settle, done = _ref.done;
      if (node instanceof React.Component) {
        node = ReactDOM.findDOMNode(node);
      }
      if (!(node instanceof Node)) {
        throw new Error("ScrollRegion.scrollTo: requires a DOM node or React element. Maybe you meant scrollToRect?");
      }
      return this._scroll({
        position: position,
        settle: settle,
        done: done
      }, (function(_this) {
        return function() {
          return node.getBoundingClientRect();
        };
      })(this));
    };

    ScrollRegion.prototype.scrollToRect = function(rect, _arg) {
      var done, position, settle, _ref;
      _ref = _arg != null ? _arg : {}, position = _ref.position, settle = _ref.settle, done = _ref.done;
      if (rect instanceof Node) {
        throw new Error("ScrollRegion.scrollToRect: requires a rect. Maybe you meant scrollTo?");
      }
      if ((rect.top == null) || (rect.height == null)) {
        throw new Error("ScrollRegion.scrollToRect: requires a rect with `top` and `height` attributes.");
      }
      return this._scroll({
        position: position,
        settle: settle,
        done: done
      }, (function(_this) {
        return function() {
          return rect;
        };
      })(this));
    };

    ScrollRegion.prototype._scroll = function(_arg, clientRectProviderCallback) {
      var contentNode, done, position, settle, settleFn, taskId;
      position = _arg.position, settle = _arg.settle, done = _arg.done;
      contentNode = ReactDOM.findDOMNode(this.refs.content);
      if (position == null) {
        position = ScrollRegion.ScrollPosition.Visible;
      }
      if (settle === true) {
        settleFn = this._settleHeight;
      } else {
        settleFn = function(callback) {
          return callback();
        };
      }
      this._scrollToTaskId += 1;
      taskId = this._scrollToTaskId;
      return settleFn((function(_this) {
        return function() {
          var contentClientRect, contentVisibleRect, distanceAboveTop, distanceBelowBottom, rect;
          if (_this._scrollToTaskId !== taskId) {
            return typeof done === "function" ? done(false) : void 0;
          }
          contentClientRect = contentNode.getBoundingClientRect();
          rect = _.clone(clientRectProviderCallback());
          rect.top = rect.top - contentClientRect.top + contentNode.scrollTop;
          rect.bottom = rect.bottom - contentClientRect.top + contentNode.scrollTop;
          contentVisibleRect = _.clone(contentClientRect);
          contentVisibleRect.top += contentNode.scrollTop;
          contentVisibleRect.bottom += contentNode.scrollTop;
          if (position === ScrollRegion.ScrollPosition.Top) {
            _this.scrollTop = rect.top;
          } else if (position === ScrollRegion.ScrollPosition.Bottom) {
            _this.scrollTop = (rect.top + rect.height) - contentClientRect.height;
          } else if (position === ScrollRegion.ScrollPosition.Center) {
            _this.scrollTop = rect.top - (contentClientRect.height - rect.height) / 2;
          } else if (position === ScrollRegion.ScrollPosition.CenterIfInvisible) {
            if (!Utils.rectVisibleInRect(rect, contentVisibleRect)) {
              _this.scrollTop = rect.top - (contentClientRect.height - rect.height) / 2;
            }
          } else if (position === ScrollRegion.ScrollPosition.Visible) {
            distanceBelowBottom = (rect.top + rect.height) - (contentClientRect.height + contentNode.scrollTop);
            distanceAboveTop = _this.scrollTop - rect.top;
            if (distanceBelowBottom >= 0) {
              _this.scrollTop += distanceBelowBottom;
            } else if (distanceAboveTop >= 0) {
              _this.scrollTop -= distanceAboveTop;
            }
          }
          return typeof done === "function" ? done(true) : void 0;
        };
      })(this));
    };

    ScrollRegion.prototype._settleHeight = function(callback) {
      var contentNode, lastContentHeight, scrollIfSettled;
      contentNode = ReactDOM.findDOMNode(this.refs.content);
      lastContentHeight = -1;
      scrollIfSettled = (function(_this) {
        return function() {
          var contentRect;
          if (!_this._mounted) {
            return;
          }
          contentRect = contentNode.getBoundingClientRect();
          if (contentRect.height !== lastContentHeight) {
            lastContentHeight = contentRect.height;
          } else {
            return callback();
          }
          return window.requestAnimationFrame(scrollIfSettled);
        };
      })(this);
      return scrollIfSettled();
    };

    ScrollRegion.prototype.recomputeDimensions = function(options) {
      var scrollbar, _base, _ref;
      if (options == null) {
        options = {};
      }
      scrollbar = (_ref = typeof (_base = this.props).getScrollbar === "function" ? _base.getScrollbar() : void 0) != null ? _ref : this.refs.scrollbar;
      if (scrollbar) {
        scrollbar._recomputeDimensions(options);
      }
      return this._recomputeDimensions(options);
    };

    ScrollRegion.prototype._recomputeDimensions = function(_arg) {
      var contentNode, totalHeight, trackHeight, useCachedValues, viewportHeight, viewportScrollTop, _ref, _ref1, _ref2;
      useCachedValues = _arg.useCachedValues;
      if (!this.refs.content) {
        return;
      }
      contentNode = ReactDOM.findDOMNode(this.refs.content);
      if (!contentNode) {
        return;
      }
      viewportScrollTop = contentNode.scrollTop;
      if (useCachedValues) {
        totalHeight = (_ref = this.state.totalHeight) != null ? _ref : contentNode.scrollHeight;
        trackHeight = (_ref1 = this.state.trackHeight) != null ? _ref1 : contentNode.scrollHeight;
        viewportHeight = (_ref2 = this.state.viewportHeight) != null ? _ref2 : contentNode.clientHeight;
      } else {
        totalHeight = contentNode.scrollHeight;
        viewportHeight = contentNode.clientHeight;
      }
      if (this.state.totalHeight !== totalHeight || this.state.viewportHeight !== viewportHeight || this.state.viewportScrollTop !== viewportScrollTop) {
        return this._setSharedState({
          totalHeight: totalHeight,
          viewportScrollTop: viewportScrollTop,
          viewportHeight: viewportHeight
        });
      }
    };

    ScrollRegion.prototype._setSharedState = function(state) {
      var scrollbar, _base, _ref;
      scrollbar = (_ref = typeof (_base = this.props).getScrollbar === "function" ? _base.getScrollbar() : void 0) != null ? _ref : this.refs.scrollbar;
      if (scrollbar) {
        scrollbar.setStateFromScrollRegion(state);
      }
      return this.setState(state);
    };

    ScrollRegion.prototype._onScroll = function(event) {
      var _base;
      if (event.target !== ReactDOM.findDOMNode(this.refs.content)) {
        return;
      }
      if (this.state.scrolling) {
        this.recomputeDimensions({
          useCachedValues: true
        });
      } else {
        this.recomputeDimensions();
        this._setSharedState({
          scrolling: true
        });
      }
      if (typeof (_base = this.props).onScroll === "function") {
        _base.onScroll(event);
      }
      if (this._onScrollEnd == null) {
        this._onScrollEnd = _.debounce((function(_this) {
          return function() {
            var _base1;
            _this._setSharedState({
              scrolling: false
            });
            _this.recomputeDimensions();
            return typeof (_base1 = _this.props).onScrollEnd === "function" ? _base1.onScrollEnd(event) : void 0;
          };
        })(this), 250);
      }
      return this._onScrollEnd();
    };

    ScrollRegion.prototype._getSelf = function() {
      return this;
    };

    return ScrollRegion;

  })(React.Component);

  ScrollRegion.Scrollbar = Scrollbar;

  module.exports = ScrollRegion;

}).call(this);
