(function() {
  var React, ReactDOM, Spinner, classNames, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require('react');

  ReactDOM = require('react-dom');

  _ = require('underscore');

  classNames = require('classnames');


  /*
  Public: Displays an indeterminate progress indicator in the center of it's
  parent component.
  
  Section: Component Kit
   */

  Spinner = (function(_super) {

    /*
    Public: React `props` supported by Spinner:
    
     - `visible` (optional) Pass true to display the spinner and false to hide it.
     - `withCover` (optiona) Pass true to dim the content behind the spinner.
     - `style` (optional) Additional styles to apply to the spinner.
     */
    __extends(Spinner, _super);

    Spinner.propTypes = {
      visible: React.PropTypes.bool,
      withCover: React.PropTypes.bool,
      style: React.PropTypes.object
    };

    function Spinner(props) {
      this.props = props;
      this._renderSpinnerDots = __bind(this._renderSpinnerDots, this);
      this._renderDotsWithCover = __bind(this._renderDotsWithCover, this);
      this.render = __bind(this.render, this);
      this.showAfterDelay = __bind(this.showAfterDelay, this);
      this.pauseAfterDelay = __bind(this.pauseAfterDelay, this);
      this.componentWillReceiveProps = __bind(this.componentWillReceiveProps, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.timer = null;
      this.state = {
        hidden: true,
        paused: true
      };
    }

    Spinner.prototype.componentDidMount = function() {
      if (this.props.visible && this.state.hidden) {
        return this.showAfterDelay();
      }
    };

    Spinner.prototype.componentWillUnmount = function() {
      if (this.timer) {
        return clearTimeout(this.timer);
      }
    };

    Spinner.prototype.componentWillReceiveProps = function(nextProps) {
      var hidden;
      hidden = nextProps.visible != null ? !nextProps.visible : false;
      if (this.state.hidden === false && hidden === true) {
        this.setState({
          hidden: true
        });
        return this.pauseAfterDelay();
      } else if (this.state.hidden === true && hidden === false) {
        return this.showAfterDelay();
      }
    };

    Spinner.prototype.pauseAfterDelay = function() {
      if (this.timer) {
        clearTimeout(this.timer);
      }
      return this.timer = setTimeout((function(_this) {
        return function() {
          if (_this.props.visible) {
            return;
          }
          return _this.setState({
            paused: true
          });
        };
      })(this), 250);
    };

    Spinner.prototype.showAfterDelay = function() {
      if (this.timer) {
        clearTimeout(this.timer);
      }
      return this.timer = setTimeout((function(_this) {
        return function() {
          if (_this.props.visible !== true) {
            return;
          }
          return _this.setState({
            paused: false,
            hidden: false
          });
        };
      })(this), 300);
    };

    Spinner.prototype.render = function() {
      if (this.props.withCover) {
        return this._renderDotsWithCover();
      } else {
        return this._renderSpinnerDots();
      }
    };

    Spinner.prototype._renderDotsWithCover = function() {
      var coverClasses, style, _ref;
      coverClasses = classNames({
        "spinner-cover": true,
        "hidden": this.state.hidden
      });
      style = Object.assign({}, (_ref = this.props.style) != null ? _ref : {}, {
        'position': 'absolute',
        'display': this.state.hidden ? "none" : "block",
        'top': '0',
        'left': '0',
        'width': '100%',
        'height': '100%',
        'background': 'rgba(255,255,255,0.9)',
        'zIndex': 1000
      });
      return React.createElement("div", {
        "className": coverClasses,
        "style": style
      }, this._renderSpinnerDots());
    };

    Spinner.prototype._renderSpinnerDots = function() {
      var otherProps, spinnerClass, style, _ref;
      spinnerClass = classNames({
        'spinner': true,
        'hidden': this.state.hidden,
        'paused': this.state.paused
      });
      style = _.extend({}, (_ref = this.props.style) != null ? _ref : {}, {
        'position': 'absolute',
        'left': '50%',
        'top': '50%',
        'zIndex': 1001,
        'transform': 'translate(-50%,-50%)'
      });
      otherProps = _.omit(this.props, Object.keys(this.constructor.propTypes));
      return React.createElement("div", React.__spread({
        "className": spinnerClass
      }, otherProps, {
        "style": style
      }), React.createElement("div", {
        "className": "bounce1"
      }), React.createElement("div", {
        "className": "bounce2"
      }), React.createElement("div", {
        "className": "bounce3"
      }), React.createElement("div", {
        "className": "bounce4"
      }));
    };

    return Spinner;

  })(React.Component);

  module.exports = Spinner;

}).call(this);
