(function() {
  var React, ReactDOM, UnsafeComponent, Utils, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require('react');

  ReactDOM = require('react-dom');

  Utils = require('nylas-exports').Utils;

  _ = require('underscore');


  /*
  Public: Renders a component provided via the `component` prop, and ensures that
  failures in the component's code do not cause state inconsistencies elsewhere in
  the application. This component is used by {InjectedComponent} and
  {InjectedComponentSet} to isolate third party code that could be buggy.
  
  Occasionally, having your component wrapped in {UnsafeComponent} can cause style
  issues. For example, in a Flexbox, the `div.unsafe-component-wrapper` will cause
  your `flex` and `order` values to be one level too deep. For these scenarios,
  UnsafeComponent looks for `containerStyles` on your React component and attaches
  them to the wrapper div:
  
  ```coffee
  class MyComponent extends React.Component
    @displayName: 'MyComponent'
    @containerStyles:
      flex: 1
      order: 2
  ```
  
  Section: Component Kit
   */

  UnsafeComponent = (function(_super) {
    __extends(UnsafeComponent, _super);

    function UnsafeComponent() {
      this._runInjectedDOMMethod = __bind(this._runInjectedDOMMethod, this);
      this.blur = __bind(this.blur, this);
      this.focus = __bind(this.focus, this);
      this.unmountInjected = __bind(this.unmountInjected, this);
      this.renderInjected = __bind(this.renderInjected, this);
      this.render = __bind(this.render, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentDidUpdate = __bind(this.componentDidUpdate, this);
      this.shouldComponentUpdate = __bind(this.shouldComponentUpdate, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      return UnsafeComponent.__super__.constructor.apply(this, arguments);
    }

    UnsafeComponent.displayName = 'UnsafeComponent';


    /*
    Public: React `props` supported by UnsafeComponent:
    
     - `component` The {React.Component} to display. All other props will be
       passed on to this component.
     */

    UnsafeComponent.propTypes = {
      component: React.PropTypes.func.isRequired,
      onComponentDidRender: React.PropTypes.func
    };

    UnsafeComponent.defaultProps = {
      onComponentDidRender: function() {}
    };

    UnsafeComponent.prototype.componentDidMount = function() {
      return this.renderInjected();
    };

    UnsafeComponent.prototype.shouldComponentUpdate = function(nextProps, nextState) {
      return !Utils.isEqualReact(nextProps, this.props) || !Utils.isEqualReact(nextState, this.state);
    };

    UnsafeComponent.prototype.componentDidUpdate = function() {
      return this.renderInjected();
    };

    UnsafeComponent.prototype.componentWillUnmount = function() {
      return this.unmountInjected();
    };

    UnsafeComponent.prototype.render = function() {
      var _ref;
      return React.createElement("div", {
        "name": "unsafe-component-wrapper",
        "style": ((_ref = this.props.component) != null ? _ref.containerStyles : void 0)
      });
    };

    UnsafeComponent.prototype.renderInjected = function() {
      var component, element, err, node, props, stack, stackEnd;
      node = ReactDOM.findDOMNode(this);
      element = null;
      try {
        props = Utils.fastOmit(this.props, Object.keys(this.constructor.propTypes));
        component = this.props.component;
        element = React.createElement(component, React.__spread({
          "key": name
        }, props));
        return this.injected = ReactDOM.render(element, node, this.props.onComponentDidRender);
      } catch (_error) {
        err = _error;
        if (NylasEnv.inDevMode()) {
          stack = err.stack;
          stackEnd = stack.indexOf('react/lib/');
          if (stackEnd > 0) {
            stackEnd = stack.lastIndexOf('\n', stackEnd);
            stack = stack.substr(0, stackEnd);
          }
          element = React.createElement("div", {
            "className": "unsafe-component-exception"
          }, React.createElement("div", {
            "className": "message"
          }, this.props.component.displayName, " could not be displayed."), React.createElement("div", {
            "className": "trace"
          }, stack));
        } else {
          element = React.createElement("div", null);
          this.injected = ReactDOM.render(element, node);
        }
        return NylasEnv.reportError(err);
      }
    };

    UnsafeComponent.prototype.unmountInjected = function() {
      var err, node;
      try {
        node = ReactDOM.findDOMNode(this);
        return ReactDOM.unmountComponentAtNode(node);
      } catch (_error) {
        err = _error;
      }
    };

    UnsafeComponent.prototype.focus = function() {
      return this._runInjectedDOMMethod('focus');
    };

    UnsafeComponent.prototype.blur = function() {
      return this._runInjectedDOMMethod('blur');
    };

    UnsafeComponent.prototype._runInjectedDOMMethod = function(method) {
      var target;
      target = null;
      if (this.injected && this.injected[method]) {
        target = this.injected;
      } else if (this.injected) {
        target = ReactDOM.findDOMNode(this.injected);
      } else {
        target = ReactDOM.findDOMNode(this);
      }
      return typeof target[method] === "function" ? target[method]() : void 0;
    };

    return UnsafeComponent;

  })(React.Component);

  module.exports = UnsafeComponent;

}).call(this);
