(function() {
  var Color, CompositeDisposable, Config, Disposable, Emitter, EmitterMixin, app, fs, isPlainObject, remote, splitKeyPath, webContentsId, withoutEmptyObjects, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require('underscore');

  _ = _.extend(_, require('./config-utils'));

  remote = require('electron').remote;

  fs = require('fs-plus');

  EmitterMixin = require('emissary').Emitter;

  _ref = require('event-kit'), CompositeDisposable = _ref.CompositeDisposable, Disposable = _ref.Disposable, Emitter = _ref.Emitter;

  Color = require('./color');

  if (process.type === 'renderer') {
    app = remote.getGlobal('application');
    webContentsId = remote.getCurrentWebContents().getId();
  } else {
    app = global.application;
    webContentsId = null;
  }

  module.exports = Config = (function() {
    EmitterMixin.includeInto(Config);

    Config.schemaEnforcers = {};

    Config.addSchemaEnforcer = function(typeName, enforcerFunction) {
      var _base;
      if ((_base = this.schemaEnforcers)[typeName] == null) {
        _base[typeName] = [];
      }
      return this.schemaEnforcers[typeName].push(enforcerFunction);
    };

    Config.addSchemaEnforcers = function(filters) {
      var enforcerFunction, functions, name, typeName, _results;
      _results = [];
      for (typeName in filters) {
        functions = filters[typeName];
        _results.push((function() {
          var _results1;
          _results1 = [];
          for (name in functions) {
            enforcerFunction = functions[name];
            _results1.push(this.addSchemaEnforcer(typeName, enforcerFunction));
          }
          return _results1;
        }).call(this));
      }
      return _results;
    };

    Config.executeSchemaEnforcers = function(keyPath, value, schema) {
      var e, enforcer, enforcerFunctions, error, type, types, _i, _j, _len, _len1;
      error = null;
      types = schema.type;
      if (!Array.isArray(types)) {
        types = [types];
      }
      for (_i = 0, _len = types.length; _i < _len; _i++) {
        type = types[_i];
        try {
          enforcerFunctions = this.schemaEnforcers[type].concat(this.schemaEnforcers['*']);
          for (_j = 0, _len1 = enforcerFunctions.length; _j < _len1; _j++) {
            enforcer = enforcerFunctions[_j];
            value = enforcer.call(this, keyPath, value, schema);
          }
          error = null;
          break;
        } catch (_error) {
          e = _error;
          error = e;
        }
      }
      if (error != null) {
        throw error;
      }
      return value;
    };

    function Config() {
      this.updateSettings = __bind(this.updateSettings, this);
      var ipcRenderer;
      this.emitter = new Emitter;
      this.schema = {
        type: 'object',
        properties: {}
      };
      this.settings = {};
      this.defaultSettings = {};
      this.transactDepth = 0;
      if (process.type === 'renderer') {
        ipcRenderer = require('electron').ipcRenderer;
        ipcRenderer.on('on-config-reloaded', (function(_this) {
          return function(event, settings) {
            return _this.updateSettings(settings);
          };
        })(this));
      }
    }

    Config.prototype.load = function() {
      return this.updateSettings(this.getRawValues());
    };


    /*
    Section: Config Subscription
     */

    Config.prototype.observe = function(keyPath, callback) {
      callback(this.get(keyPath));
      return this.onDidChangeKeyPath(keyPath, function(event) {
        return callback(event.newValue);
      });
    };

    Config.prototype.onDidChange = function() {
      var callback, keyPath;
      if (arguments.length === 1) {
        callback = arguments[0];
      } else if (arguments.length === 2) {
        keyPath = arguments[0], callback = arguments[1];
      }
      return this.onDidChangeKeyPath(keyPath, callback);
    };


    /*
    Section: Managing Settings
     */

    Config.prototype.get = function(keyPath) {
      return this.getRawValue(keyPath);
    };

    Config.prototype.set = function(keyPath, value) {
      var defaultValue, e;
      if (value !== void 0) {
        try {
          value = this.makeValueConformToSchema(keyPath, value);
        } catch (_error) {
          e = _error;
          return false;
        }
      }
      defaultValue = _.valueForKeyPath(this.defaultSettings, keyPath);
      if (_.isEqual(defaultValue, value)) {
        value = void 0;
      }
      if (_.isObject(value)) {
        value = JSON.parse(JSON.stringify(value));
      }
      this.setRawValue(keyPath, value);
      return true;
    };

    Config.prototype.unset = function(keyPath) {
      return this.set(keyPath, _.valueForKeyPath(this.defaultSettings, keyPath));
    };

    Config.prototype.getSchema = function(keyPath) {
      var key, keys, schema, _i, _len, _ref1;
      keys = splitKeyPath(keyPath);
      schema = this.schema;
      for (_i = 0, _len = keys.length; _i < _len; _i++) {
        key = keys[_i];
        if (schema == null) {
          break;
        }
        schema = (_ref1 = schema.properties) != null ? _ref1[key] : void 0;
      }
      return schema;
    };

    Config.prototype.transact = function(callback) {
      this.transactDepth++;
      try {
        return callback();
      } finally {
        this.transactDepth--;
        this.emitChangeEvent();
      }
    };


    /*
    Section: Internal methods used by core
     */

    Config.prototype.pushAtKeyPath = function(keyPath, value) {
      var arrayValue, result, _ref1;
      arrayValue = (_ref1 = this.get(keyPath)) != null ? _ref1 : [];
      if (!(arrayValue instanceof Array)) {
        throw new Error("Config.pushAtKeyPath is intended for array values. Value " + (JSON.stringify(arrayValue)) + " is not an array.");
      }
      result = arrayValue.push(value);
      this.set(keyPath, arrayValue);
      return result;
    };

    Config.prototype.unshiftAtKeyPath = function(keyPath, value) {
      var arrayValue, result, _ref1;
      arrayValue = (_ref1 = this.get(keyPath)) != null ? _ref1 : [];
      if (!(arrayValue instanceof Array)) {
        throw new Error("Config.unshiftAtKeyPath is intended for array values. Value " + (JSON.stringify(arrayValue)) + " is not an array.");
      }
      result = arrayValue.unshift(value);
      this.set(keyPath, arrayValue);
      return result;
    };

    Config.prototype.removeAtKeyPath = function(keyPath, value) {
      var arrayValue, result, _ref1;
      arrayValue = (_ref1 = this.get(keyPath)) != null ? _ref1 : [];
      if (!(arrayValue instanceof Array)) {
        throw new Error("Config.removeAtKeyPath is intended for array values. Value " + (JSON.stringify(arrayValue)) + " is not an array.");
      }
      result = _.remove(arrayValue, value);
      this.set(keyPath, arrayValue);
      return result;
    };

    Config.prototype.setSchema = function(keyPath, schema) {
      var key, properties, rootSchema, _i, _len, _ref1;
      if (!isPlainObject(schema)) {
        throw new Error("Error loading schema for " + keyPath + ": schemas can only be objects!");
      }
      if (!typeof (schema.type != null)) {
        throw new Error("Error loading schema for " + keyPath + ": schema objects must have a type attribute");
      }
      rootSchema = this.schema;
      if (keyPath) {
        _ref1 = splitKeyPath(keyPath);
        for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
          key = _ref1[_i];
          rootSchema.type = 'object';
          if (rootSchema.properties == null) {
            rootSchema.properties = {};
          }
          properties = rootSchema.properties;
          if (properties[key] == null) {
            properties[key] = {};
          }
          rootSchema = properties[key];
        }
      }
      _.extend(rootSchema, schema);
      this.setDefaults(keyPath, this.extractDefaultsFromSchema(schema));
      return this.resetSettingsForSchemaChange();
    };


    /*
    Section: Private methods managing global settings
     */

    Config.prototype.updateSettings = function(newSettings) {
      this.settings = newSettings || {};
      return this.emitChangeEvent();
    };

    Config.prototype.getRawValue = function(keyPath) {
      var defaultValue, value;
      value = _.valueForKeyPath(this.settings, keyPath);
      defaultValue = _.valueForKeyPath(this.defaultSettings, keyPath);
      if (value != null) {
        value = this.deepClone(value);
        if (isPlainObject(value) && isPlainObject(defaultValue)) {
          _.defaults(value, defaultValue);
        }
      } else {
        value = this.deepClone(defaultValue);
      }
      return value;
    };

    Config.prototype.onDidChangeKeyPath = function(keyPath, callback) {
      var oldValue;
      oldValue = this.get(keyPath);
      return this.emitter.on('did-change', (function(_this) {
        return function() {
          var event, newValue;
          newValue = _this.get(keyPath);
          if (!_.isEqual(oldValue, newValue)) {
            event = {
              oldValue: oldValue,
              newValue: newValue
            };
            oldValue = newValue;
            return callback(event);
          }
        };
      })(this));
    };

    Config.prototype.isSubKeyPath = function(keyPath, subKeyPath) {
      var pathSubTokens, pathTokens;
      if (!((keyPath != null) && (subKeyPath != null))) {
        return false;
      }
      pathSubTokens = splitKeyPath(subKeyPath);
      pathTokens = splitKeyPath(keyPath).slice(0, pathSubTokens.length);
      return _.isEqual(pathTokens, pathSubTokens);
    };

    Config.prototype.setRawDefault = function(keyPath, value) {
      _.setValueForKeyPath(this.defaultSettings, keyPath, value);
      return this.emitChangeEvent();
    };

    Config.prototype.setDefaults = function(keyPath, defaults) {
      var childValue, e, key, keys, _results;
      if ((defaults != null) && isPlainObject(defaults)) {
        keys = splitKeyPath(keyPath);
        _results = [];
        for (key in defaults) {
          childValue = defaults[key];
          if (!defaults.hasOwnProperty(key)) {
            continue;
          }
          _results.push(this.setDefaults(keys.concat([key]).join('.'), childValue));
        }
        return _results;
      } else {
        try {
          defaults = this.makeValueConformToSchema(keyPath, defaults);
          return this.setRawDefault(keyPath, defaults);
        } catch (_error) {
          e = _error;
          return console.warn("'" + keyPath + "' could not set the default. Attempted default: " + (JSON.stringify(defaults)) + "; Schema: " + (JSON.stringify(this.getSchema(keyPath))));
        }
      }
    };

    Config.prototype.deepClone = function(object) {
      if (object instanceof Color) {
        return object.clone();
      } else if (_.isArray(object)) {
        return object.map((function(_this) {
          return function(value) {
            return _this.deepClone(value);
          };
        })(this));
      } else if (isPlainObject(object)) {
        return _.mapObject(object, (function(_this) {
          return function(key, value) {
            return [key, _this.deepClone(value)];
          };
        })(this));
      } else {
        return object;
      }
    };

    Config.prototype.extractDefaultsFromSchema = function(schema) {
      var defaults, key, properties, value;
      if (schema["default"] != null) {
        return schema["default"];
      } else if (schema.type === 'object' && (schema.properties != null) && isPlainObject(schema.properties)) {
        defaults = {};
        properties = schema.properties || {};
        for (key in properties) {
          value = properties[key];
          defaults[key] = this.extractDefaultsFromSchema(value);
        }
        return defaults;
      }
    };

    Config.prototype.makeValueConformToSchema = function(keyPath, value, options) {
      var e, schema;
      if (options != null ? options.suppressException : void 0) {
        try {
          return this.makeValueConformToSchema(keyPath, value);
        } catch (_error) {
          e = _error;
          return void 0;
        }
      } else {
        if (schema = this.getSchema(keyPath)) {
          value = this.constructor.executeSchemaEnforcers(keyPath, value, schema);
        }
        return value;
      }
    };

    Config.prototype.resetSettingsForSchemaChange = function() {
      return this.transact((function(_this) {
        return function() {
          var settings;
          settings = _this.getRawValues();
          settings = _this.makeValueConformToSchema(null, settings, {
            suppressException: true
          });
          _this.setRawValue(null, settings);
        };
      })(this));
    };

    Config.prototype.emitChangeEvent = function() {
      if (!(this.transactDepth > 0)) {
        return this.emitter.emit('did-change');
      }
    };

    Config.prototype.getRawValues = function() {
      try {
        return JSON.parse(app.configPersistenceManager.getRawValuesString());
      } catch (_error) {
        return {};
      }
    };

    Config.prototype.setRawValue = function(keyPath, value) {
      app.configPersistenceManager.setRawValue(keyPath, value, webContentsId);
      return this.load();
    };

    return Config;

  })();

  Config.addSchemaEnforcers({
    'integer': {
      coerce: function(keyPath, value, schema) {
        value = parseInt(value);
        if (isNaN(value) || !isFinite(value)) {
          throw new Error("Validation failed at " + keyPath + ", " + (JSON.stringify(value)) + " cannot be coerced into an int");
        }
        return value;
      }
    },
    'number': {
      coerce: function(keyPath, value, schema) {
        value = parseFloat(value);
        if (isNaN(value) || !isFinite(value)) {
          throw new Error("Validation failed at " + keyPath + ", " + (JSON.stringify(value)) + " cannot be coerced into a number");
        }
        return value;
      }
    },
    'boolean': {
      coerce: function(keyPath, value, schema) {
        switch (typeof value) {
          case 'string':
            if (value.toLowerCase() === 'true') {
              return true;
            } else if (value.toLowerCase() === 'false') {
              return false;
            } else {
              throw new Error("Validation failed at " + keyPath + ", " + (JSON.stringify(value)) + " must be a boolean or the string 'true' or 'false'");
            }
            break;
          case 'boolean':
            return value;
          default:
            throw new Error("Validation failed at " + keyPath + ", " + (JSON.stringify(value)) + " must be a boolean or the string 'true' or 'false'");
        }
      }
    },
    'string': {
      validate: function(keyPath, value, schema) {
        if (typeof value !== 'string') {
          throw new Error("Validation failed at " + keyPath + ", " + (JSON.stringify(value)) + " must be a string");
        }
        return value;
      }
    },
    'null': {
      coerce: function(keyPath, value, schema) {
        if (value !== (void 0) && value !== null) {
          throw new Error("Validation failed at " + keyPath + ", " + (JSON.stringify(value)) + " must be null");
        }
        return value;
      }
    },
    'object': {
      coerce: function(keyPath, value, schema) {
        var childSchema, error, newValue, prop, propValue;
        if (!isPlainObject(value)) {
          throw new Error("Validation failed at " + keyPath + ", " + (JSON.stringify(value)) + " must be an object");
        }
        if (schema.properties == null) {
          return value;
        }
        newValue = {};
        for (prop in value) {
          propValue = value[prop];
          childSchema = schema.properties[prop];
          if (childSchema != null) {
            try {
              newValue[prop] = this.executeSchemaEnforcers("" + keyPath + "." + prop, propValue, childSchema);
            } catch (_error) {
              error = _error;
              console.warn("Error setting item in object: " + error.message);
            }
          } else {
            newValue[prop] = propValue;
          }
        }
        return newValue;
      }
    },
    'array': {
      coerce: function(keyPath, value, schema) {
        var error, item, itemSchema, newValue, _i, _len;
        if (!Array.isArray(value)) {
          throw new Error("Validation failed at " + keyPath + ", " + (JSON.stringify(value)) + " must be an array");
        }
        itemSchema = schema.items;
        if (itemSchema != null) {
          newValue = [];
          for (_i = 0, _len = value.length; _i < _len; _i++) {
            item = value[_i];
            try {
              newValue.push(this.executeSchemaEnforcers(keyPath, item, itemSchema));
            } catch (_error) {
              error = _error;
              console.warn("Error setting item in array: " + error.message);
            }
          }
          return newValue;
        } else {
          return value;
        }
      }
    },
    'color': {
      coerce: function(keyPath, value, schema) {
        var color;
        color = Color.parse(value);
        if (color == null) {
          throw new Error("Validation failed at " + keyPath + ", " + (JSON.stringify(value)) + " cannot be coerced into a color");
        }
        return color;
      }
    },
    '*': {
      coerceMinimumAndMaximum: function(keyPath, value, schema) {
        if (typeof value !== 'number') {
          return value;
        }
        if ((schema.minimum != null) && typeof schema.minimum === 'number') {
          value = Math.max(value, schema.minimum);
        }
        if ((schema.maximum != null) && typeof schema.maximum === 'number') {
          value = Math.min(value, schema.maximum);
        }
        return value;
      },
      validateEnum: function(keyPath, value, schema) {
        var possibleValue, possibleValues, _i, _len;
        possibleValues = schema["enum"];
        if (!((possibleValues != null) && Array.isArray(possibleValues) && possibleValues.length)) {
          return value;
        }
        for (_i = 0, _len = possibleValues.length; _i < _len; _i++) {
          possibleValue = possibleValues[_i];
          if (_.isEqual(possibleValue, value)) {
            return value;
          }
        }
        throw new Error("Validation failed at " + keyPath + ", " + (JSON.stringify(value)) + " is not one of " + (JSON.stringify(possibleValues)));
      }
    }
  });

  isPlainObject = function(value) {
    return _.isObject(value) && !_.isArray(value) && !_.isFunction(value) && !_.isString(value) && !(value instanceof Color);
  };

  splitKeyPath = function(keyPath) {
    var char, i, keyPathArray, startIndex, _i, _len;
    if (keyPath == null) {
      return [];
    }
    startIndex = 0;
    keyPathArray = [];
    for (i = _i = 0, _len = keyPath.length; _i < _len; i = ++_i) {
      char = keyPath[i];
      if (char === '.' && (i === 0 || keyPath[i - 1] !== '\\')) {
        keyPathArray.push(keyPath.substring(startIndex, i));
        startIndex = i + 1;
      }
    }
    keyPathArray.push(keyPath.substr(startIndex, keyPath.length));
    return keyPathArray;
  };

  withoutEmptyObjects = function(object) {
    var key, newValue, resultObject, value;
    resultObject = void 0;
    if (isPlainObject(object)) {
      for (key in object) {
        value = object[key];
        newValue = withoutEmptyObjects(value);
        if (newValue != null) {
          if (resultObject == null) {
            resultObject = {};
          }
          resultObject[key] = newValue;
        }
      }
    } else {
      resultObject = object;
    }
    return resultObject;
  };

}).call(this);
