(function() {
  var DOMUtils, _, _s,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  _s = require('underscore.string');

  DOMUtils = {
    Mutating: {
      replaceFirstListItem: function(li, replaceWith) {
        var child, index, list, selection, text, _ref;
        list = DOMUtils.closest(li, "ul, ol");
        if (replaceWith.length === 0) {
          replaceWith = replaceWith.replace(/\s/g, "&nbsp;");
          text = document.createElement("div");
          text.innerHTML = "<br>";
        } else {
          replaceWith = replaceWith.replace(/\s/g, "&nbsp;");
          text = document.createElement("span");
          text.innerHTML = "" + replaceWith;
        }
        if (list.querySelectorAll('li').length <= 1) {
          list.parentNode.replaceChild(text, list);
        } else {
          li.parentNode.removeChild(li);
          list.parentNode.insertBefore(text, list);
        }
        child = (_ref = text.childNodes[0]) != null ? _ref : text;
        index = Math.max(replaceWith.length - 1, 0);
        selection = document.getSelection();
        return selection.setBaseAndExtent(child, index, child, index);
      },
      removeEmptyNodes: function(node) {
        return Array.prototype.slice.call(node.childNodes).forEach(function(child) {
          if (child.textContent === '') {
            return node.removeChild(child);
          } else {
            return DOMUtils.Mutating.removeEmptyNodes(child);
          }
        });
      },
      collapseAdjacentElements: function(els) {
        var adjacent, anchor, anchors, el, mergeSet, remaining, seenEls, toMerge, _i, _j, _k, _len, _len1, _len2;
        if (els == null) {
          els = [];
        }
        if (els.length === 0) {
          return;
        }
        els = Array.prototype.slice.call(els);
        seenEls = [];
        toMerge = [];
        for (_i = 0, _len = els.length; _i < _len; _i++) {
          el = els[_i];
          if (__indexOf.call(seenEls, el) >= 0) {
            continue;
          }
          adjacent = DOMUtils.collectAdjacent(el);
          seenEls = seenEls.concat(adjacent);
          if (adjacent.length <= 1) {
            continue;
          }
          toMerge.push(adjacent);
        }
        anchors = [];
        for (_j = 0, _len1 = toMerge.length; _j < _len1; _j++) {
          mergeSet = toMerge[_j];
          anchor = mergeSet[0];
          remaining = mergeSet.slice(1);
          for (_k = 0, _len2 = remaining.length; _k < _len2; _k++) {
            el = remaining[_k];
            while (el.childNodes.length > 0) {
              anchor.appendChild(el.childNodes[0]);
            }
          }
          DOMUtils.Mutating.removeElements(remaining);
          anchors.push(anchor);
        }
        return anchors;
      },
      removeElements: function(elements) {
        var el, _i, _len;
        if (elements == null) {
          elements = [];
        }
        for (_i = 0, _len = elements.length; _i < _len; _i++) {
          el = elements[_i];
          try {
            if (el.parentNode) {
              el.parentNode.removeChild(el);
            }
          } catch (_error) {
            continue;
          }
        }
        return elements;
      },
      applyTextInRange: function(range, selection, newText) {
        var node;
        range.deleteContents();
        node = document.createTextNode(newText);
        range.insertNode(node);
        range.selectNode(node);
        selection.removeAllRanges();
        return selection.addRange(range);
      },
      getRangeAtAndSelectWord: function(selection, index) {
        var range;
        range = selection.getRangeAt(index);
        if (range.collapsed) {
          DOMUtils.Mutating.selectWordContainingRange(range);
          range = selection.getRangeAt(index);
        }
        return range;
      },
      selectWordContainingRange: function(range) {
        var node, selection, text, wordEnd, wordStart;
        selection = document.getSelection();
        node = selection.focusNode;
        text = node.textContent;
        wordStart = _s.reverse(text.substring(0, selection.focusOffset)).search(/\s/);
        if (wordStart === -1) {
          wordStart = 0;
        } else {
          wordStart = selection.focusOffset - wordStart;
        }
        wordEnd = text.substring(selection.focusOffset).search(/\s/);
        if (wordEnd === -1) {
          wordEnd = text.length;
        } else {
          wordEnd += selection.focusOffset;
        }
        selection.removeAllRanges();
        range = new Range();
        range.setStart(node, wordStart);
        range.setEnd(node, wordEnd);
        return selection.addRange(range);
      },
      moveSelectionToIndexInAnchorNode: function(selection, index) {
        var node;
        if (!selection.isCollapsed) {
          return;
        }
        node = selection.anchorNode;
        return selection.setBaseAndExtent(node, index, node, index);
      },
      moveSelectionToEnd: function(selection) {
        var index, node;
        if (!selection.isCollapsed) {
          return;
        }
        node = DOMUtils.findLastTextNode(selection.anchorNode);
        index = node.length;
        return selection.setBaseAndExtent(node, index, node, index);
      }
    },
    getSelectionRectFromDOM: function(selection) {
      var node, r;
      if (selection == null) {
        selection = document.getSelection();
      }
      node = selection.anchorNode;
      if (node.nodeType === Node.TEXT_NODE) {
        r = document.createRange();
        r.selectNodeContents(node);
        return r.getBoundingClientRect();
      } else if (node.nodeType === Node.ELEMENT_NODE) {
        return node.getBoundingClientRect();
      } else {
        return null;
      }
    },
    isSelectionInTextNode: function(selection) {
      if (selection == null) {
        selection = document.getSelection();
      }
      if (!selection) {
        return false;
      }
      return selection.isCollapsed && selection.anchorNode.nodeType === Node.TEXT_NODE && selection.anchorOffset > 0;
    },
    isAtTabChar: function(selection) {
      if (selection == null) {
        selection = document.getSelection();
      }
      if (DOMUtils.isSelectionInTextNode(selection)) {
        return selection.anchorNode.textContent[selection.anchorOffset - 1] === "\t";
      } else {
        return false;
      }
    },
    isAtBeginningOfDocument: function(dom, selection) {
      var firstChild;
      if (selection == null) {
        selection = document.getSelection();
      }
      if (!selection.isCollapsed) {
        return false;
      }
      if (selection.anchorOffset > 0) {
        return false;
      }
      if (dom.childNodes.length === 0) {
        return true;
      }
      if (selection.anchorNode === dom) {
        return true;
      }
      firstChild = dom.childNodes[0];
      return selection.anchorNode === firstChild;
    },
    atStartOfList: function() {
      var anchor, li, selection;
      selection = document.getSelection();
      anchor = selection.anchorNode;
      if (!selection.isCollapsed) {
        return false;
      }
      if ((anchor != null ? anchor.nodeName : void 0) === "LI") {
        return true;
      }
      if (selection.anchorOffset > 0) {
        return false;
      }
      li = DOMUtils.closest(anchor, "li");
      if (!li) {
        return;
      }
      return DOMUtils.isFirstChild(li, anchor);
    },
    inputTypes: function() {
      return "input, textarea, *[contenteditable]";
    },
    closest: function(node, selector) {
      if (node instanceof HTMLElement) {
        return node.closest(selector);
      } else if (node != null ? node.parentNode : void 0) {
        return DOMUtils.closest(node.parentNode, selector);
      } else {
        return null;
      }
    },
    closestAtCursor: function(selector) {
      var selection;
      selection = document.getSelection();
      if (!(selection != null ? selection.isCollapsed : void 0)) {
        return;
      }
      return DOMUtils.closest(selection.anchorNode, selector);
    },
    closestElement: function(node) {
      if (node instanceof HTMLElement) {
        return node;
      } else if (node != null ? node.parentNode : void 0) {
        return DOMUtils.closestElement(node.parentNode);
      } else {
        return null;
      }
    },
    isInList: function() {
      var li, list;
      li = DOMUtils.closestAtCursor("li");
      list = DOMUtils.closestAtCursor("ul, ol");
      return li && list;
    },
    collectAdjacent: function(root, nodeName) {
      var adjacent, node, _ref, _ref1;
      if (nodeName == null) {
        nodeName = root.nodeName;
      }
      adjacent = [];
      node = root;
      while (((_ref = node.nextSibling) != null ? _ref.nodeName : void 0) === nodeName) {
        adjacent.push(node.nextSibling);
        node = node.nextSibling;
      }
      if (root.nodeName === nodeName) {
        adjacent.unshift(root);
      }
      node = root;
      while (((_ref1 = node.previousSibling) != null ? _ref1.nodeName : void 0) === nodeName) {
        adjacent.unshift(node.previousSibling);
        node = node.previousSibling;
      }
      return adjacent;
    },
    getNodeIndex: (function(_this) {
      return function(context, nodeToFind) {
        return DOMUtils.indexOfNodeInSimilarNodes(context, nodeToFind);
      };
    })(this),
    getRangeInScope: (function(_this) {
      return function(scope) {
        var range, selection;
        selection = document.getSelection();
        if (!DOMUtils.selectionInScope(selection, scope)) {
          return null;
        }
        try {
          range = selection.getRangeAt(0);
        } catch (_error) {
          console.warn("Selection is not returning a range");
          return document.createRange();
        }
        return range;
      };
    })(this),
    selectionInScope: function(selection, scope) {
      if (selection == null) {
        return false;
      }
      if (scope == null) {
        return false;
      }
      return scope.contains(selection.anchorNode) && scope.contains(selection.focusNode);
    },
    isEmptyBoundingRect: function(rect) {
      return rect.top === 0 && rect.bottom === 0 && rect.left === 0 && rect.right === 0;
    },
    atEndOfContent: function(selection, rootScope, containerScope) {
      var atEndIndex, inLastChild, isLastChild, lastChild, _ref;
      if (containerScope == null) {
        containerScope = rootScope;
      }
      if (selection.isCollapsed) {
        lastChild = DOMUtils.lastNonBlankChildNode(containerScope);
        if (containerScope === rootScope) {
          if (containerScope.childNodes.length === 0) {
            return true;
          }
        }
        if (!lastChild) {
          return false;
        }
        inLastChild = lastChild.contains(selection.focusNode);
        isLastChild = lastChild === selection.focusNode;
        if (isLastChild) {
          if ((_ref = selection.focusNode) != null ? _ref.length : void 0) {
            atEndIndex = selection.focusOffset === selection.focusNode.length;
          } else {
            atEndIndex = selection.focusOffset === 0;
          }
          return atEndIndex;
        } else if (inLastChild) {
          return DOMUtils.atEndOfContent(selection, rootScope, lastChild);
        } else {
          return false;
        }
      } else {
        return false;
      }
    },
    lastNonBlankChildNode: function(node) {
      var childNode, lastNode, _i, _ref;
      lastNode = null;
      _ref = node.childNodes;
      for (_i = _ref.length - 1; _i >= 0; _i += -1) {
        childNode = _ref[_i];
        if (childNode.nodeType === Node.TEXT_NODE) {
          if (DOMUtils.isBlankTextNode(childNode)) {
            continue;
          } else {
            return childNode;
          }
        } else if (childNode.nodeType === Node.ELEMENT_NODE) {
          return childNode;
        } else {
          continue;
        }
      }
      return lastNode;
    },
    lastDescendent: function(node) {
      if (!node) {
        return null;
      }
      if (node.childNodes.length > 0) {
        return DOMUtils.lastNode(node.childNodes[node.childNodes.length - 1]);
      } else {
        return null;
      }
    },
    findLastTextNode: function(node) {
      var childNode, _i, _ref;
      if (!node) {
        return null;
      }
      if (node.nodeType === Node.TEXT_NODE) {
        return node;
      }
      _ref = node.childNodes;
      for (_i = _ref.length - 1; _i >= 0; _i += -1) {
        childNode = _ref[_i];
        if (childNode.nodeType === Node.TEXT_NODE) {
          return childNode;
        } else if (childNode.nodeType === Node.ELEMENT_NODE) {
          return DOMUtils.findLastTextNode(childNode);
        } else {
          continue;
        }
      }
      return null;
    },
    findOnlyChildTextNode: function(node) {
      if (!node) {
        return null;
      }
      if (node.nodeType === Node.TEXT_NODE) {
        return node;
      }
      if (node.childNodes.length > 1) {
        return null;
      }
      return DOMUtils.findOnlyChildTextNode(node.childNodes[0]);
    },
    findFirstTextNode: function(node) {
      var childNode, _i, _len, _ref;
      if (!node) {
        return null;
      }
      if (node.nodeType === Node.TEXT_NODE) {
        return node;
      }
      _ref = node.childNodes;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        childNode = _ref[_i];
        if (childNode.nodeType === Node.TEXT_NODE) {
          return childNode;
        } else if (childNode.nodeType === Node.ELEMENT_NODE) {
          return DOMUtils.findFirstTextNode(childNode);
        } else {
          continue;
        }
      }
      return null;
    },
    isBlankTextNode: function(node) {
      if (!(node != null ? node.data : void 0)) {
        return;
      }
      return node.data.replace(/\u00a0/g, "x").trim().length === 0;
    },
    indexOfNodeInSimilarNodes: function(context, nodeToFind) {
      var idx, treeWalker;
      if (nodeToFind.isEqualNode(context)) {
        return 0;
      }
      treeWalker = document.createTreeWalker(context);
      idx = 0;
      while (treeWalker.nextNode()) {
        if (treeWalker.currentNode.isEqualNode(nodeToFind)) {
          if (treeWalker.currentNode.isSameNode(nodeToFind)) {
            return idx;
          }
          idx += 1;
        }
      }
      return -1;
    },
    findSimilarNodeAtIndex: function(context, nodeToFind, desiredIdx) {
      var idx, treeWalker;
      if (desiredIdx === 0 && nodeToFind.isEqualNode(context)) {
        return context;
      }
      treeWalker = document.createTreeWalker(context);
      idx = 0;
      while (treeWalker.nextNode()) {
        if (treeWalker.currentNode.isEqualNode(nodeToFind)) {
          if (desiredIdx === idx) {
            return treeWalker.currentNode;
          }
          idx += 1;
        }
      }
      return null;
    },
    findCharacter: function(context, character) {
      var currentNode, i, index, node, treeWalker;
      node = null;
      index = null;
      treeWalker = document.createTreeWalker(context, NodeFilter.SHOW_TEXT);
      while (currentNode = treeWalker.nextNode()) {
        i = currentNode.data.indexOf(character);
        if (i >= 0) {
          node = currentNode;
          index = i;
          break;
        }
      }
      return {
        node: node,
        index: index
      };
    },
    escapeHTMLCharacters: function(text) {
      var map;
      map = {
        '&': '&amp;',
        '<': '&lt;',
        '>': '&gt;',
        '"': '&quot;',
        "'": '&#039;'
      };
      return text.replace(/[&<>"']/g, function(m) {
        return map[m];
      });
    },
    nodeIsVisible: function(node) {
      var style;
      while (node && node.nodeType === Node.ELEMENT_NODE) {
        style = window.getComputedStyle(node);
        node = node.parentNode;
        if (style == null) {
          continue;
        }
        if (style.opacity === 0 || style.opacity === "0" || style.visibility === "hidden" || style.display === "none") {
          return false;
        }
      }
      return true;
    },
    nodeIsLikelyVisible: function(node) {
      return node.offsetParent !== null;
    },
    nodesWithContent: function(elementOrHTML) {
      var allNodes, doc, domParser, node, nodes, _i, _ref, _ref1, _ref2, _ref3, _ref4;
      nodes = [];
      if (_.isString(elementOrHTML)) {
        domParser = new DOMParser();
        doc = domParser.parseFromString(elementOrHTML, "text/html");
        allNodes = doc.body.childNodes;
      } else if (elementOrHTML != null ? elementOrHTML.childNodes : void 0) {
        allNodes = elementOrHTML.childNodes;
      } else {
        return nodes;
      }
      for (_i = allNodes.length - 1; _i >= 0; _i += -1) {
        node = allNodes[_i];
        if (node.nodeName === "IMG") {
          nodes.unshift(node);
        }
        if (((_ref = node.textContent) != null ? _ref : "").trim().length === 0) {
          continue;
        }
        if (((_ref1 = node.style) != null ? _ref1.opacity : void 0) === 0 || ((_ref2 = node.style) != null ? _ref2.opacity : void 0) === "0" || ((_ref3 = node.style) != null ? _ref3.visibility : void 0) === "hidden" || ((_ref4 = node.style) != null ? _ref4.display : void 0) === "none") {
          continue;
        }
        nodes.unshift(node);
      }
      return nodes;
    },
    parents: function(node) {
      var nodes;
      nodes = [];
      while (node = node.parentNode) {
        nodes.unshift(node);
      }
      return nodes;
    },
    isFirstChild: function(root, node) {
      if (!(root && node)) {
        return false;
      }
      if (root === node) {
        return true;
      }
      if (!root.childNodes[0]) {
        return false;
      }
      if (root.childNodes[0] === node) {
        return true;
      }
      return DOMUtils.isFirstChild(root.childNodes[0], node);
    },
    commonAncestor: function(nodes, parentFilter) {
      var getParents, minDepth;
      if (nodes == null) {
        nodes = [];
      }
      if (nodes.length === 0) {
        return null;
      }
      nodes = Array.prototype.slice.call(nodes);
      minDepth = Number.MAX_VALUE;
      getParents = function(node) {
        var depth, parentNodes;
        parentNodes = [node];
        depth = 0;
        while (node = node.parentNode) {
          if (parentFilter) {
            if (parentFilter(node)) {
              parentNodes.unshift(node);
            }
          } else {
            parentNodes.unshift(node);
          }
          depth += 1;
          if (depth > minDepth) {
            break;
          }
        }
        minDepth = Math.min(minDepth, depth);
        return parentNodes;
      };
      return _.last(_.intersection.apply(null, nodes.map(getParents)));
    },
    scrollAdjustmentToMakeNodeVisibleInContainer: function(node, container) {
      var containerRect, nodeRect;
      if (!node) {
        return;
      }
      nodeRect = node.getBoundingClientRect();
      containerRect = container.getBoundingClientRect();
      return this.scrollAdjustmentToMakeRectVisibleInRect(nodeRect, containerRect);
    },
    scrollAdjustmentToMakeRectVisibleInRect: function(nodeRect, containerRect) {
      var distanceAboveTop, distanceBelowBottom;
      distanceBelowBottom = (nodeRect.top + nodeRect.height) - (containerRect.top + containerRect.height);
      if (distanceBelowBottom >= 0) {
        return distanceBelowBottom;
      }
      distanceAboveTop = containerRect.top - nodeRect.top;
      if (distanceAboveTop >= 0) {
        return -distanceAboveTop;
      }
      return 0;
    },
    getIndexedTextContent: function(node) {
      var item, items, position, text, treeWalker;
      items = [];
      treeWalker = document.createTreeWalker(node, NodeFilter.SHOW_ELEMENT + NodeFilter.SHOW_TEXT);
      position = 0;
      while (treeWalker.nextNode()) {
        node = treeWalker.currentNode;
        if (node.tagName === "BR" || node.nodeType === Node.TEXT_NODE || node.tagName === "DIV") {
          text = node.nodeType === Node.TEXT_NODE ? node.nodeValue : "\n";
          item = {
            start: position,
            end: position + text.length,
            node: node,
            text: text
          };
          items.push(item);
          position += text.length;
        }
      }
      return items;
    },
    rangeInRange: function(inner, outer) {
      return outer.isPointInRange(inner.startContainer, inner.startOffset) && outer.isPointInRange(inner.endContainer, inner.endOffset);
    },
    rangeOverlapsRange: function(range1, range2) {
      return range2.isPointInRange(range1.startContainer, range1.startOffset) || range1.isPointInRange(range2.startContainer, range2.startOffset);
    },
    rangeStartsOrEndsInRange: function(range1, range2) {
      return range2.isPointInRange(range1.startContainer, range1.startOffset) || range2.isPointInRange(range1.endContainer, range1.endOffset);
    },
    selectionStartsOrEndsIn: function(rangeOrNode) {
      var range, selection;
      selection = document.getSelection();
      if (!(selection && selection.rangeCount > 0)) {
        return false;
      }
      if (rangeOrNode instanceof Range) {
        return this.rangeStartsOrEndsInRange(selection.getRangeAt(0), rangeOrNode);
      } else if (rangeOrNode instanceof Node) {
        range = new Range();
        range.selectNode(rangeOrNode);
        return this.rangeStartsOrEndsInRange(selection.getRangeAt(0), range);
      } else {
        return false;
      }
    },
    selectionIsWithin: function(rangeOrNode) {
      var range, selection;
      selection = document.getSelection();
      if (!(selection && selection.rangeCount > 0)) {
        return false;
      }
      if (rangeOrNode instanceof Range) {
        return this.rangeInRange(selection.getRangeAt(0), rangeOrNode);
      } else if (rangeOrNode instanceof Node) {
        range = new Range();
        range.selectNode(rangeOrNode);
        return this.rangeInRange(selection.getRangeAt(0), range);
      } else {
        return false;
      }
    },
    regExpSelectorAll: function(node, regex) {
      var end, from, item, listPosition, nodeTextList, range, ranges, result, start, text, to;
      nodeTextList = this.getIndexedTextContent(node);
      text = nodeTextList.map(function(_arg) {
        var text;
        text = _arg.text;
        return text;
      }).join("");
      ranges = [];
      listPosition = 0;
      while ((result = regex.exec(text)) !== null) {
        from = result.index;
        to = regex.lastIndex;
        item = nodeTextList[listPosition];
        range = document.createRange();
        while (from >= item.end) {
          item = nodeTextList[++listPosition];
        }
        start = item.node.nodeType === Node.TEXT_NODE ? from - item.start : 0;
        range.setStart(item.node, start);
        while (to > item.end) {
          item = nodeTextList[++listPosition];
        }
        end = item.node.nodeType === Node.TEXT_NODE ? to - item.start : 0;
        range.setEnd(item.node, end);
        ranges.push(range);
      }
      return ranges;
    },
    isWrapped: function(range, nodeName) {
      var endNode, startNode;
      if (!(range && nodeName)) {
        return false;
      }
      startNode = range.startContainer;
      endNode = range.endContainer;
      if (startNode.parentNode !== endNode.parentNode) {
        return false;
      }
      if (startNode.previousSibling || endNode.nextSibling) {
        return false;
      }
      if (range.startOffset > 0 || range.endOffset < endNode.textContent.length) {
        return false;
      }
      return startNode.parentNode.nodeName === nodeName;
    },
    wrap: function(range, nodeName) {
      var newNode;
      newNode = document.createElement(nodeName);
      try {
        range.surroundContents(newNode);
      } catch (_error) {
        newNode.appendChild(range.extractContents());
        range.insertNode(newNode);
      }
      return newNode;
    },
    unwrapNode: function(node) {
      var child, lastChild, parent, replacedNodes;
      if (node.childNodes.length === 0) {
        return node;
      }
      replacedNodes = [];
      parent = node.parentNode;
      if (parent == null) {
        return node;
      }
      lastChild = _.last(node.childNodes);
      replacedNodes.unshift(lastChild);
      parent.replaceChild(lastChild, node);
      while (child = _.last(node.childNodes)) {
        replacedNodes.unshift(child);
        parent.insertBefore(child, lastChild);
        lastChild = child;
      }
      return replacedNodes;
    },
    isDescendantOf: function(node, matcher) {
      var parent;
      if (matcher == null) {
        matcher = function() {
          return false;
        };
      }
      parent = node != null ? node.parentElement : void 0;
      while (parent) {
        if (matcher(parent)) {
          return true;
        }
        parent = parent.parentElement;
      }
      return false;
    },
    looksLikeBlockElement: function(node) {
      var _ref;
      return (_ref = node.nodeName) === "BR" || _ref === "P" || _ref === "BLOCKQUOTE" || _ref === "DIV" || _ref === "TABLE";
    },
    looksLikeNonEmptyNode: function(node) {
      var textNode;
      textNode = DOMUtils.findFirstTextNode(node);
      if (textNode) {
        if (/^[\n ]*$/.test(textNode.data)) {
          return false;
        } else {
          return true;
        }
      } else {
        return false;
      }
    },
    previousTextNode: function(node) {
      var curNode;
      curNode = node;
      while (curNode.parentNode) {
        if (curNode.previousSibling) {
          return this.findLastTextNode(curNode.previousSibling);
        }
        curNode = curNode.parentNode;
      }
      return null;
    }
  };

  module.exports = DOMUtils;

}).call(this);
