(function() {
  var ComposerExtension, ContenteditableExtension,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  ContenteditableExtension = require('./contenteditable-extension');


  /*
  Public: To create ComposerExtensions that enhance the composer experience,
  you should create objects that implement the interface defined at
  {ComposerExtension}.
  
  {ComposerExtension} extends {ContenteditableExtension}, so you can also
  implement the methods defined there to further enhance the composer
  experience.
  
  To register your extension with the ExtensionRegistry, call
  {ExtensionRegistry::Composer::register}.  When your package is being
  unloaded, you *must* call the corresponding
  {ExtensionRegistry::Composer::unregister} to unhook your extension.
  
  ```
  coffee activate: -> ExtensionRegistry.Composer.register(MyExtension)
  
  ...
  
  deactivate: -> ExtensionRegistry.Composer.unregister(MyExtension)
  ```
  
  **Your ComposerExtension should be stateless**. The user may have multiple
  drafts open at any time, and the methods of your ComposerExtension may be
  called for different drafts at any time. You should not expect that the
  session you receive in {::applyTransformsToDraft} is for the same
  draft you previously received in {::warningsForSending}, etc.
  
  The ComposerExtension API does not currently expose any asynchronous or
  {Promise}-based APIs, except for applyTransformsToDraft and unapplyTransformsToDraft,
  which can optionally return a promsie. This will likely change in the future.
  If you have a use-case for a ComposerExtension that is not possible with the current
  API, please let us know.
  
  Section: Extensions
   */

  ComposerExtension = (function(_super) {
    __extends(ComposerExtension, _super);

    function ComposerExtension() {
      return ComposerExtension.__super__.constructor.apply(this, arguments);
    }


    /*
    Public: Allows the addition of new types of send actions such as "Send
    Later"
    
    - `draft`: A fully populated {Message} object that is about to be sent.
    
    Return an object that adheres to the following spec. If the draft data
    indicates that your action should not be available, then return null.
    
      - `title`: A short, single string that is displayed to users when
      describing your component. It is used in the hover title text of your
      option in the dropdown menu. It is also used in the "Default Send
      Behavior" dropdown setting. If your string is selected, then the
      `core.sending.defaultSendType` will be set to your string and your
      option will appear as the default.
       *# TODO FIXME: The preferences does not yet know how to dynamically
       * pick these up. For now they are hard-coded.
    
      - `onSend`: Callback for when your option is clicked as the primary
      action. The function will be passed `{draft}` as its only argument.
      It does not need to return anything. It may be asynchronous and likely
      queue Tasks.
    
      - `iconUrl`: A custom icon to be placed in the Send button. SendAction
      extensions have the form "Send + {ICON}"
     */

    ComposerExtension.sendActionConfig = function(_arg) {
      var draft;
      draft = _arg.draft;
    };


    /*
    Public: Inspect the draft, and return any warnings that need to be
    displayed before the draft is sent. Warnings should be string phrases,
    such as "without an attachment" that fit into a message of the form:
    "Send #{phase1} and #{phase2}?"
    
    - `draft`: A fully populated {Message} object that is about to be sent.
    
    Returns a list of warning strings, or an empty array if no warnings need
    to be displayed.
     */

    ComposerExtension.warningsForSending = function(_arg) {
      var draft;
      draft = _arg.draft;
      return [];
    };


    /*
    Public: Override prepareNewDraft to modify a brand new draft before it
    is displayed in a composer. This is one of the only places in the
    application where it's safe to modify the draft object you're given
    directly to add participants to the draft, add a signature, etc.
    
    By default, new drafts are considered `pristine`. If the user leaves the
    composer without making any changes, the draft is discarded. If your
    extension populates the draft in a way that makes it "populated" in a
    valuable way, you should set `draft.pristine = false` so the draft
    saves, even if no further changes are made.
     */

    ComposerExtension.prepareNewDraft = function(_arg) {
      var draft;
      draft = _arg.draft;
    };


    /*
    Public: applyTransformsToDraft is called when a draft the user is editing
    is saved to the server and/or sent. This method gives you an opportunity to
    remove any annotations you've inserted into the draft body, apply final changes
    to the body, etc.
    
    Note that your extension /must/ be able to reverse the changes it applies to
    the draft in `applyTransformsToDraft`. If the user re-opens the draft,
    `unapplyTransformsToDraft` will be called and must restore the draft to it's
    previous edit-ready state.
    
    Examples:
    - `applyTransformsToDraft`: Add tracking pixel to the email body.
    - `unapplyTransformsToDraft`: Remove the tracking pixel from the email body.
    
    - `applyTransformsToDraft`: Encrypt the message body.
    - `unapplyTransformsToDraft`: Decrypt the message body.
    
    This method should return a modified {Message} object, or a {Promise} which resolves
    to a modified Message object.
    
    - `draft`: A {Message} the user is about to finish editing.
     */

    ComposerExtension.applyTransformsToDraft = function(_arg) {
      var draft;
      draft = _arg.draft;
      return draft;
    };


    /*
    Public: unapplyTransformsToDraft should revert the changes made in
    `applyTransformsToDraft`. See the documentation for that method for more
    information.
     */

    ComposerExtension.unapplyTransformsToDraft = function(_arg) {
      var draft;
      draft = _arg.draft;
      return draft;
    };

    return ComposerExtension;

  })(ContenteditableExtension);

  module.exports = ComposerExtension;

}).call(this);
