(function() {
  var ActionBridge, Actions, DatabaseStore, Message, Model, Role, TargetWindows, Utils, printToConsole, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __slice = [].slice;

  _ = require('underscore');

  Actions = require('./actions');

  Model = require('./models/model');

  DatabaseStore = require('./stores/database-store');

  Utils = require('./models/utils');

  Role = {
    WORK: 'work',
    SECONDARY: 'secondary'
  };

  TargetWindows = {
    ALL: 'all',
    WORK: 'work'
  };

  Message = {
    DATABASE_STORE_TRIGGER: 'db-store-trigger'
  };

  printToConsole = false;

  ActionBridge = (function() {
    ActionBridge.Role = Role;

    ActionBridge.Message = Message;

    ActionBridge.TargetWindows = TargetWindows;

    function ActionBridge(ipc) {
      this.onBeforeUnload = __bind(this.onBeforeUnload, this);
      this.onRebroadcast = __bind(this.onRebroadcast, this);
      this.onIPCMessage = __bind(this.onIPCMessage, this);
      this.registerGlobalAction = __bind(this.registerGlobalAction, this);
      var databaseCallback;
      this.globalActions = [];
      this.ipc = ipc;
      this.ipcLastSendTime = null;
      this.initiatorId = NylasEnv.getWindowType();
      this.role = NylasEnv.isWorkWindow() ? Role.WORK : Role.SECONDARY;
      NylasEnv.onBeforeUnload(this.onBeforeUnload);
      this.ipc.on('action-bridge-message', this.onIPCMessage);
      Actions.globalActions.forEach((function(_this) {
        return function(name) {
          var callback;
          callback = function() {
            var args;
            args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
            return _this.onRebroadcast(TargetWindows.ALL, name, args);
          };
          return Actions[name].listen(callback, _this);
        };
      })(this));
      databaseCallback = (function(_this) {
        return function(change) {
          if (DatabaseStore.triggeringFromActionBridge) {
            return;
          }
          return _this.onRebroadcast(TargetWindows.ALL, Message.DATABASE_STORE_TRIGGER, [change]);
        };
      })(this);
      DatabaseStore.listen(databaseCallback, this);
      if (this.role !== Role.WORK) {
        Actions.workWindowActions.forEach((function(_this) {
          return function(name) {
            var callback;
            callback = function() {
              var args;
              args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
              return _this.onRebroadcast(TargetWindows.WORK, name, args);
            };
            return Actions[name].listen(callback, _this);
          };
        })(this));
      }
    }

    ActionBridge.prototype.registerGlobalAction = function(_arg) {
      var actionFn, callback, name, scope;
      scope = _arg.scope, name = _arg.name, actionFn = _arg.actionFn;
      this.globalActions.push({
        scope: scope,
        name: name,
        actionFn: actionFn
      });
      callback = (function(_this) {
        return function() {
          var args, broadcastName;
          args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
          broadcastName = "" + scope + "::" + name;
          return _this.onRebroadcast(TargetWindows.ALL, broadcastName, args);
        };
      })(this);
      return actionFn.listen(callback, this);
    };

    ActionBridge.prototype._isExtensionAction = function(name) {
      return name.split("::").length === 2;
    };

    ActionBridge.prototype._globalExtensionAction = function(broadcastName) {
      var actionFn, name, scope, _ref, _ref1;
      _ref = broadcastName.split("::"), scope = _ref[0], name = _ref[1];
      actionFn = ((_ref1 = _.findWhere(this.globalActions, {
        scope: scope,
        name: name
      })) != null ? _ref1 : {}).actionFn;
      return actionFn;
    };

    ActionBridge.prototype.onIPCMessage = function(event, initiatorId, name, json) {
      return process.nextTick((function(_this) {
        return function() {
          var args, fn;
          console.debug(printToConsole, "ActionBridge: " + _this.initiatorId + " Action Bridge Received: " + name);
          args = JSON.parse(json, Utils.registeredObjectReviver);
          if (name === Message.DATABASE_STORE_TRIGGER) {
            DatabaseStore.triggeringFromActionBridge = true;
            DatabaseStore.trigger((function(func, args, ctor) {
              ctor.prototype = func.prototype;
              var child = new ctor, result = func.apply(child, args);
              return Object(result) === result ? result : child;
            })(DatabaseStore.ChangeRecord, args, function(){}));
            return DatabaseStore.triggeringFromActionBridge = false;
          } else if (Actions[name]) {
            Actions[name].firing = true;
            return Actions[name].apply(Actions, args);
          } else if (_this._isExtensionAction(name)) {
            fn = _this._globalExtensionAction(name);
            if (fn) {
              fn.firing = true;
              return fn.apply(null, args);
            }
          } else {
            throw new Error("" + _this.initiatorId + " received unknown action-bridge event: " + name);
          }
        };
      })(this));
    };

    ActionBridge.prototype.onRebroadcast = function(target, name, args) {
      var json, params, _ref, _ref1;
      if ((_ref = Actions[name]) != null ? _ref.firing : void 0) {
        Actions[name].firing = false;
        return;
      } else if ((_ref1 = this._globalExtensionAction(name)) != null ? _ref1.firing : void 0) {
        this._globalExtensionAction(name).firing = false;
        return;
      }
      params = [];
      args.forEach(function(arg) {
        if (arg instanceof Function) {
          throw new Error("ActionBridge cannot forward action argument of type `function` to work window.");
        }
        return params.push(arg);
      });
      json = JSON.stringify(params, Utils.registeredObjectReplacer);
      console.debug(printToConsole, "ActionBridge: " + this.initiatorId + " Action Bridge Broadcasting: " + name);
      this.ipc.send("action-bridge-rebroadcast-to-" + target, this.initiatorId, name, json);
      return this.ipcLastSendTime = Date.now();
    };

    ActionBridge.prototype.onBeforeUnload = function(readyToUnload) {
      if (this.ipcLastSendTime && Date.now() - this.ipcLastSendTime < 100) {
        setTimeout(readyToUnload, 100);
        return false;
      }
      return true;
    };

    return ActionBridge;

  })();

  module.exports = ActionBridge;

}).call(this);
