(function() {
  var Attribute, AttributeCollection, Matcher, tableNameForJoin, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  tableNameForJoin = require('../models/utils').tableNameForJoin;

  Attribute = require('./attribute');

  Matcher = require('./matcher');


  /*
  Public: Collection attributes provide basic support for one-to-many relationships.
  For example, Threads in N1 have a collection of Labels or Folders.
  
  When Collection attributes are marked as `queryable`, the DatabaseStore
  automatically creates a join table and maintains it as you create, save,
  and delete models. When you call `persistModel`, entries are added to the
  join table associating the ID of the model with the IDs of models in the collection.
  
  Collection attributes have an additional clause builder, `contains`:
  
  ```coffee
  DatabaseStore.findAll(Thread).where([Thread.attributes.categories.contains('inbox')])
  ```
  
  This is equivalent to writing the following SQL:
  
  ```sql
  SELECT `Thread`.`data` FROM `Thread`
  INNER JOIN `ThreadLabel` AS `M1` ON `M1`.`id` = `Thread`.`id`
  WHERE `M1`.`value` = 'inbox'
  ORDER BY `Thread`.`last_message_received_timestamp` DESC
  ```
  
  The value of this attribute is always an array of other model objects.
  
  Section: Database
   */

  AttributeCollection = (function(_super) {
    __extends(AttributeCollection, _super);

    function AttributeCollection(_arg) {
      var itemClass, joinOnField, joinQueryableBy, jsonKey, modelKey;
      modelKey = _arg.modelKey, jsonKey = _arg.jsonKey, itemClass = _arg.itemClass, joinOnField = _arg.joinOnField, joinQueryableBy = _arg.joinQueryableBy;
      AttributeCollection.__super__.constructor.apply(this, arguments);
      this.itemClass = itemClass;
      this.joinOnField = joinOnField;
      this.joinQueryableBy = joinQueryableBy || [];
      this;
    }

    AttributeCollection.prototype.toJSON = function(vals) {
      var json, msg, val, _i, _len;
      if (!vals) {
        return [];
      }
      json = [];
      for (_i = 0, _len = vals.length; _i < _len; _i++) {
        val = vals[_i];
        if (!(val instanceof this.itemClass)) {
          msg = "AttributeCollection::toJSON: Value `" + val + "` in " + this.modelKey + " is not an " + this.itemClass.name;
          throw new Error(msg);
        }
        if (val.toJSON != null) {
          json.push(val.toJSON());
        } else {
          json.push(val);
        }
      }
      return json;
    };

    AttributeCollection.prototype.fromJSON = function(json) {
      var obj, objJSON, objs, _i, _len;
      if (!(json && json instanceof Array)) {
        return [];
      }
      objs = [];
      for (_i = 0, _len = json.length; _i < _len; _i++) {
        objJSON = json[_i];
        if (!objJSON) {
          continue;
        }
        if (this.itemClass.prototype.fromJSON != null) {
          obj = new this.itemClass;
          obj.clientId = void 0;
          obj.fromJSON(objJSON);
        } else {
          obj = new this.itemClass(objJSON);
          obj.clientId = void 0;
        }
        objs.push(obj);
      }
      return objs;
    };

    AttributeCollection.prototype.contains = function(val) {
      if (val == null) {
        throw new Error("AttributeCollection::contains (" + this.modelKey + ") - you must provide a value");
      }
      if (!this.queryable) {
        throw new Error("AttributeCollection::contains (" + this.modelKey + ") - this field cannot be queried against.");
      }
      return new Matcher(this, 'contains', val);
    };

    AttributeCollection.prototype.containsAny = function(vals) {
      if (vals == null) {
        throw new Error("AttributeCollection::contains (" + this.modelKey + ") - you must provide a value");
      }
      if (!this.queryable) {
        throw new Error("AttributeCollection::contains (" + this.modelKey + ") - this field cannot be queried against.");
      }
      return new Matcher(this, 'containsAny', vals);
    };

    return AttributeCollection;

  })(Attribute);

  module.exports = AttributeCollection;

}).call(this);
