(function() {
  var Attribute, AttributeJoinedData, NullPlaceholder, tableNameForJoin, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  tableNameForJoin = require('../models/utils').tableNameForJoin;

  Attribute = require('./attribute');

  NullPlaceholder = "!NULLVALUE!";


  /*
  Public: Joined Data attributes allow you to store certain attributes of an
  object in a separate table in the database. We use this attribute
  type for Message bodies. Storing message bodies, which can be very
  large, in a separate table allows us to make queries on message
  metadata extremely fast, and inflate Message objects without their
  bodies to build the thread list.
  
  When building a query on a model with a JoinedData attribute, you need
  to call `include` to explicitly load the joined data attribute.
  The query builder will automatically perform a `LEFT OUTER JOIN` with
  the secondary table to retrieve the attribute:
  
  ```coffee
  DatabaseStore.find(Message, '123').then (message) ->
     * message.body is undefined
  
  DatabaseStore.find(Message, '123').include(Message.attributes.body).then (message) ->
     * message.body is defined
  ```
  
  When you call `persistModel`, JoinedData attributes are automatically
  written to the secondary table.
  
  JoinedData attributes cannot be `queryable`.
  
  Section: Database
   */

  AttributeJoinedData = (function(_super) {
    __extends(AttributeJoinedData, _super);

    AttributeJoinedData.NullPlaceholder = NullPlaceholder;

    function AttributeJoinedData(_arg) {
      var jsonKey, modelKey, modelTable;
      modelKey = _arg.modelKey, jsonKey = _arg.jsonKey, modelTable = _arg.modelTable;
      AttributeJoinedData.__super__.constructor.apply(this, arguments);
      this.modelTable = modelTable;
      this;
    }

    AttributeJoinedData.prototype.selectSQL = function(klass) {
      return "IFNULL(`" + this.modelTable + "`.`value`, '" + NullPlaceholder + "') AS `" + this.modelKey + "`";
    };

    AttributeJoinedData.prototype.includeSQL = function(klass) {
      return "LEFT OUTER JOIN `" + this.modelTable + "` ON `" + this.modelTable + "`.`id` = `" + klass.name + "`.`id`";
    };

    return AttributeJoinedData;

  })(Attribute);

  module.exports = AttributeJoinedData;

}).call(this);
