(function() {
  var Attribute, Matcher, SortOrder, _;

  _ = require('underscore');

  Matcher = require('./matcher');

  SortOrder = require('./sort-order');


  /*
  Public: The Attribute class represents a single model attribute, like 'account_id'.
  Subclasses of {Attribute} like {AttributeDateTime} know how to covert between
  the JSON representation of that type and the javascript representation.
  The Attribute class also exposes convenience methods for generating {Matcher} objects.
  
  Section: Database
   */

  Attribute = (function() {
    function Attribute(_arg) {
      var jsonKey, modelKey, queryable;
      modelKey = _arg.modelKey, queryable = _arg.queryable, jsonKey = _arg.jsonKey;
      this.modelKey = modelKey;
      this.jsonKey = jsonKey || modelKey;
      this.queryable = queryable;
      this;
    }

    Attribute.prototype.equal = function(val) {
      if (val == null) {
        throw new Error("Attribute::equal (" + this.modelKey + ") - you must provide a value");
      }
      if (!this.queryable) {
        throw new Error("Attribute::equal (" + this.modelKey + ") - this field cannot be queried against");
      }
      return new Matcher(this, '=', val);
    };

    Attribute.prototype["in"] = function(val) {
      if (!(val instanceof Array)) {
        throw new Error("Attribute.in: you must pass an array of values.");
      }
      if (val == null) {
        throw new Error("Attribute::in (" + this.modelKey + ") - you must provide a value");
      }
      if (!this.queryable) {
        throw new Error("Attribute::in (" + this.modelKey + ") - this field cannot be queried against");
      }
      if (val.length === 0) {
        console.warn("Attribute::in (" + this.modelKey + ") called with an empty set. You should avoid this useless query!");
      }
      if (val.length === 1) {
        return new Matcher(this, '=', val[0]);
      }
      return new Matcher(this, 'in', val);
    };

    Attribute.prototype.not = function(val) {
      if (val == null) {
        throw new Error("Attribute::not (" + this.modelKey + ") - you must provide a value");
      }
      if (!this.queryable) {
        throw new Error("Attribute::not (" + this.modelKey + ") - this field cannot be queried against");
      }
      return new Matcher(this, '!=', val);
    };

    Attribute.prototype.descending = function() {
      return new SortOrder(this, 'DESC');
    };

    Attribute.prototype.ascending = function() {
      return new SortOrder(this, 'ASC');
    };

    Attribute.prototype.toJSON = function(val) {
      return val;
    };

    Attribute.prototype.fromJSON = function(val) {
      return val != null ? val : null;
    };

    return Attribute;

  })();

  module.exports = Attribute;

}).call(this);
