
/*
Public: Represents a particular sort direction on a particular column. You should not
instantiate SortOrders manually. Instead, call {Attribute::ascending} or
{Attribute::descending} to obtain a sort order instance:

```coffee
DatabaseStore.findBy(Message)
  .where({threadId: threadId, draft: false})
  .order(Message.attributes.date.descending()).then (messages) ->

```

Section: Database
 */

(function() {
  var SortOrder;

  SortOrder = (function() {
    function SortOrder(attr, direction) {
      this.attr = attr;
      this.direction = direction != null ? direction : 'DESC';
    }

    SortOrder.prototype.orderBySQL = function(klass) {
      return "`" + klass.name + "`.`" + this.attr.jsonKey + "` " + this.direction;
    };

    SortOrder.prototype.attribute = function() {
      return this.attr;
    };

    return SortOrder;

  })();

  module.exports = SortOrder;

}).call(this);
