(function() {
  var AccountStore, AllMailName, Attributes, Category, HiddenCategories, LockedCategories, Model, StandardCategories, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  Model = require('./model');

  Attributes = require('../attributes');

  AccountStore = null;

  StandardCategories = {
    "inbox": "inbox",
    "important": "important",
    "sent": "sent",
    "drafts": "drafts",
    "all": "all",
    "spam": "spam",
    "archive": "archive",
    "trash": "trash"
  };

  LockedCategories = {
    "sent": "sent",
    "drafts": "drafts",
    "N1-Snoozed": "N1-Snoozed"
  };

  HiddenCategories = {
    "sent": "sent",
    "drafts": "drafts",
    "all": "all",
    "archive": "archive",
    "starred": "starred",
    "important": "important",
    "N1-Snoozed": "N1-Snoozed"
  };

  AllMailName = "all";


  /*
  Private:
  This abstract class has only two concrete implementations:
    - `Folder`
    - `Label`
  
  See the equivalent models for details.
  
  Folders and Labels have different semantics. The `Category` class only exists to help DRY code where they happen to behave the same
  
   *# Attributes
  
  `name`: {AttributeString} The internal name of the label or folder. Queryable.
  
  `displayName`: {AttributeString} The display-friendly name of the label or folder. Queryable.
  
  Section: Models
   */

  Category = (function(_super) {
    __extends(Category, _super);

    Category.attributes = _.extend({}, Model.attributes, {
      'name': Attributes.String({
        queryable: true,
        modelKey: 'name'
      }),
      'displayName': Attributes.String({
        queryable: true,
        modelKey: 'displayName',
        jsonKey: 'display_name'
      }),
      'isDeleted': Attributes.Boolean({
        modelKey: 'isDeleted',
        jsonKey: 'is_deleted'
      })
    });

    Category.Types = {
      Standard: 'standard',
      Locked: 'locked',
      User: 'user',
      Hidden: 'hidden'
    };

    Category.StandardCategoryNames = Object.keys(StandardCategories);

    Category.LockedCategoryNames = Object.keys(LockedCategories);

    Category.HiddenCategoryNames = Object.keys(HiddenCategories);

    Category.categoriesSharedName = function(cats) {
      var name;
      if (!(cats && cats.length > 0)) {
        return null;
      }
      name = cats[0].name;
      if (!_.every(cats, function(cat) {
        return cat.name === name;
      })) {
        return null;
      }
      return name;
    };

    Category.additionalSQLiteConfig = {
      setup: function() {
        return ['CREATE INDEX IF NOT EXISTS CategoryNameIndex ON Category(account_id,name)', 'CREATE UNIQUE INDEX IF NOT EXISTS CategoryClientIndex ON Category(client_id)'];
      }
    };

    function Category() {
      this.displayType = __bind(this.displayType, this);
      Category.__super__.constructor.apply(this, arguments);
    }

    Category.prototype.fromJSON = function(json) {
      Category.__super__.fromJSON.apply(this, arguments);
      if (this.displayName && this.displayName.startsWith('INBOX.')) {
        this.displayName = this.displayName.slice(6);
      }
      if (this.displayName && this.displayName === 'INBOX') {
        this.displayName = 'Inbox';
      }
      return this;
    };

    Category.prototype.displayType = function() {
      if (AccountStore == null) {
        AccountStore = require('../stores/account-store');
      }
      if (AccountStore.accountForId(this.accountId).usesLabels()) {
        return 'label';
      } else {
        return 'folder';
      }
    };

    Category.prototype.hue = function() {
      var hue, i, _i, _ref;
      if (!this.displayName) {
        return 0;
      }
      hue = 0;
      for (i = _i = 0, _ref = this.displayName.length - 1; 0 <= _ref ? _i <= _ref : _i >= _ref; i = 0 <= _ref ? ++_i : --_i) {
        hue += this.displayName.charCodeAt(i);
      }
      hue = hue * (396.0 / 512.0);
      return hue;
    };

    Category.prototype.isStandardCategory = function(showImportant) {
      if (showImportant == null) {
        showImportant = NylasEnv.config.get('core.workspace.showImportant');
      }
      if (showImportant === true) {
        return StandardCategories[this.name] != null;
      } else {
        return (StandardCategories[this.name] != null) && this.name !== 'important';
      }
    };

    Category.prototype.isLockedCategory = function() {
      return (LockedCategories[this.name] != null) || (LockedCategories[this.displayName] != null);
    };

    Category.prototype.isHiddenCategory = function() {
      return (HiddenCategories[this.name] != null) || (HiddenCategories[this.displayName] != null);
    };

    Category.prototype.isUserCategory = function() {
      return !this.isStandardCategory() && !this.isHiddenCategory();
    };

    Category.prototype.isArchive = function() {
      var _ref;
      return (_ref = this.name) === 'all' || _ref === 'archive';
    };

    return Category;

  })(Model);

  module.exports = Category;

}).call(this);
