(function() {
  var Attributes, Contact, Event, Model, moment, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Model = require('./model');

  Contact = require('./contact');

  Attributes = require('../attributes');

  _ = require('underscore');

  moment = require('moment');

  Event = (function(_super) {
    __extends(Event, _super);

    function Event() {
      this.participantForMe = __bind(this.participantForMe, this);
      return Event.__super__.constructor.apply(this, arguments);
    }

    Event.attributes = _.extend({}, Model.attributes, {
      'calendarId': Attributes.String({
        queryable: true,
        modelKey: 'calendarId',
        jsonKey: 'calendar_id'
      }),
      'title': Attributes.String({
        modelKey: 'title',
        jsonKey: 'title'
      }),
      'description': Attributes.String({
        modelKey: 'description',
        jsonKey: 'description'
      }),
      'when': Attributes.Object({
        modelKey: 'when'
      }),
      'location': Attributes.String({
        modelKey: 'location',
        jsonKey: 'location'
      }),
      'owner': Attributes.String({
        modelKey: 'owner',
        jsonKey: 'owner'
      }),
      'participants': Attributes.Object({
        modelKey: 'participants',
        jsonKey: 'participants'
      }),
      'status': Attributes.String({
        modelKey: 'status',
        jsonKey: 'status'
      }),
      'readOnly': Attributes.Boolean({
        modelKey: 'readOnly',
        jsonKey: 'read_only'
      }),
      'busy': Attributes.Boolean({
        modelKey: 'busy',
        jsonKey: 'busy'
      }),
      'recurrence': Attributes.Object({
        modelKey: 'recurrence',
        jsonKey: 'recurrence'
      }),
      'type': Attributes.String({
        modelKey: 'type',
        jsonKey: '_type'
      }),
      'start': Attributes.Number({
        queryable: true,
        modelKey: 'start',
        jsonKey: '_start'
      }),
      'end': Attributes.Number({
        queryable: true,
        modelKey: 'end',
        jsonKey: '_end'
      })
    });

    Event.prototype._unixRangeForDatespan = function(start_date, end_date) {
      return {
        start: moment(start_date).unix(),
        end: moment(end_date).add(1, 'day').subtract(1, 'second').unix()
      };
    };

    Event.prototype.fromJSON = function(json) {
      var _ref, _ref1, _ref2, _ref3;
      Event.__super__.fromJSON.call(this, json);
      if (!this.when) {
        return this;
      }
      if (this.when.time) {
        _ref = {
          start: this.when.time,
          end: this.when.time
        }, this.start = _ref.start, this.end = _ref.end;
      } else if (this.when.start_time && this.when.end_time) {
        _ref1 = {
          start: this.when.start_time,
          end: this.when.end_time
        }, this.start = _ref1.start, this.end = _ref1.end;
      } else if (this.when.date) {
        _ref2 = this._unixRangeForDatespan(this.when.date, this.when.date), this.start = _ref2.start, this.end = _ref2.end;
      } else if (this.when.start_date && this.when.end_date) {
        _ref3 = this._unixRangeForDatespan(this.when.start_date, this.when.end_date), this.start = _ref3.start, this.end = _ref3.end;
      }
      return this;
    };

    Event.prototype.fromDraft = function(draft) {
      if ((this.title == null) || this.title.length === 0) {
        this.title = draft.subject;
      }
      if ((this.participants == null) || this.participants.length === 0) {
        this.participants = draft.participants().map(function(contact) {
          return {
            name: contact.name,
            email: contact.email,
            status: "noreply"
          };
        });
      }
      return this;
    };

    Event.prototype.isAllDay = function() {
      var daySpan;
      daySpan = 86400 - 1;
      return (this.end - this.start) >= daySpan;
    };

    Event.prototype.participantForMe = function() {
      var p, _i, _len, _ref;
      _ref = this.participants;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        p = _ref[_i];
        if ((new Contact({
          email: p.email
        })).isMe()) {
          return p;
        }
      }
      return null;
    };

    return Event;

  })(Model);

  module.exports = Event;

}).call(this);
