(function() {
  var Actions, Attributes, File, Model, RegExpUtils, path, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  path = require('path');

  Model = require('./model');

  Actions = require('../actions');

  Attributes = require('../attributes');

  _ = require('underscore');

  RegExpUtils = null;


  /*
  Public: File model represents a File object served by the Nylas Platform API.
  For more information about Files on the Nylas Platform, read the
  [Files API Documentation](https://nylas.com/docs/api#files)
  
   *# Attributes
  
  `filename`: {AttributeString} The display name of the file. Queryable.
  
  `size`: {AttributeNumber} The size of the file, in bytes.
  
  `contentType`: {AttributeString} The content type of the file (ex: `image/png`)
  
  `contentId`: {AttributeString} If this file is an inline attachment, contentId
  is a string that matches a cid:<value> found in the HTML body of a {Message}.
  
  This class also inherits attributes from {Model}
  
  Section: Models
   */

  File = (function(_super) {
    __extends(File, _super);

    function File() {
      return File.__super__.constructor.apply(this, arguments);
    }

    File.attributes = _.extend({}, Model.attributes, {
      'filename': Attributes.String({
        modelKey: 'filename',
        jsonKey: 'filename',
        queryable: true
      }),
      'size': Attributes.Number({
        modelKey: 'size',
        jsonKey: 'size'
      }),
      'contentType': Attributes.String({
        modelKey: 'contentType',
        jsonKey: 'content_type'
      }),
      'messageIds': Attributes.Collection({
        modelKey: 'messageIds',
        jsonKey: 'message_ids',
        itemClass: String
      }),
      'contentId': Attributes.String({
        modelKey: 'contentId',
        jsonKey: 'content_id'
      })
    });

    File.prototype.displayName = function() {
      var defaultNames;
      defaultNames = {
        'text/calendar': "Event.ics",
        'image/png': 'Unnamed Image.png',
        'image/jpg': 'Unnamed Image.jpg',
        'image/jpeg': 'Unnamed Image.jpg'
      };
      if (this.filename && this.filename.length) {
        return this.filename;
      } else if (defaultNames[this.contentType]) {
        return defaultNames[this.contentType];
      } else {
        return "Unnamed Attachment";
      }
    };

    File.prototype.safeDisplayName = function() {
      if (RegExpUtils == null) {
        RegExpUtils = require('../../regexp-utils');
      }
      return this.displayName().replace(RegExpUtils.illegalPathCharactersRegexp(), '-');
    };

    File.prototype.displayExtension = function() {
      return path.extname(this.displayName().toLowerCase()).slice(1);
    };

    File.prototype.displayFileSize = function(bytes) {
      var decimalPoints, idx, result, rounded, threshold, units;
      if (bytes == null) {
        bytes = this.size;
      }
      threshold = 1000000000;
      units = ['B', 'KB', 'MB', 'GB'];
      idx = units.length - 1;
      result = bytes / threshold;
      while (result < 1 && idx >= 0) {
        threshold /= 1000;
        result = bytes / threshold;
        idx--;
      }
      decimalPoints = idx >= 2 ? 1 : 0;
      rounded = parseFloat(result.toFixed(decimalPoints));
      return "" + rounded + " " + units[idx];
    };

    return File;

  })(Model);

  module.exports = File;

}).call(this);
