(function() {
  var AttributeJoinedData, MutableQueryResultSet, QueryRange, QueryResultSet, _,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  QueryRange = require('./query-range');

  QueryResultSet = require('./query-result-set');

  AttributeJoinedData = require('../attributes/attribute-joined-data');

  MutableQueryResultSet = (function(_super) {
    __extends(MutableQueryResultSet, _super);

    function MutableQueryResultSet() {
      return MutableQueryResultSet.__super__.constructor.apply(this, arguments);
    }

    MutableQueryResultSet.prototype.immutableClone = function() {
      var set;
      set = new QueryResultSet({
        _ids: [].concat(this._ids),
        _modelsHash: _.extend({}, this._modelsHash),
        _query: this._query,
        _offset: this._offset
      });
      Object.freeze(set._ids);
      Object.freeze(set._modelsHash);
      return set;
    };

    MutableQueryResultSet.prototype.clipToRange = function(range) {
      var m, models, rangeEnd, selfEnd, _i, _len, _results;
      if (range.isInfinite()) {
        return;
      }
      if (range.offset > this._offset) {
        this._ids = this._ids.slice(range.offset - this._offset);
        this._offset = range.offset;
      }
      rangeEnd = range.offset + range.limit;
      selfEnd = this._offset + this._ids.length;
      if (rangeEnd < selfEnd) {
        this._ids.length = Math.max(0, rangeEnd - this._offset);
      }
      models = this.models();
      this._modelsHash = {};
      this._idToIndexHash = null;
      _results = [];
      for (_i = 0, _len = models.length; _i < _len; _i++) {
        m = models[_i];
        _results.push(this.updateModel(m));
      }
      return _results;
    };

    MutableQueryResultSet.prototype.addModelsInRange = function(rangeModels, range) {
      var m, _i, _len, _results;
      this.addIdsInRange(_.pluck(rangeModels, 'id'), range);
      _results = [];
      for (_i = 0, _len = rangeModels.length; _i < _len; _i++) {
        m = rangeModels[_i];
        _results.push(this.updateModel(m));
      }
      return _results;
    };

    MutableQueryResultSet.prototype.addIdsInRange = function(rangeIds, range) {
      var currentEnd, existingAfter, existingBefore, rangeIdsEnd;
      if (this._offset === null || range.isInfinite()) {
        this._ids = rangeIds;
        this._idToIndexHash = null;
        return this._offset = range.offset;
      } else {
        currentEnd = this._offset + this._ids.length;
        rangeIdsEnd = range.offset + rangeIds.length;
        if (rangeIdsEnd < this._offset) {
          throw new Error("addIdsInRange: You can only add adjacent values (" + rangeIdsEnd + " < " + this._offset + ")");
        }
        if (range.offset > currentEnd) {
          throw new Error("addIdsInRange: You can only add adjacent values (" + range.offset + " > " + currentEnd + ")");
        }
        existingBefore = [];
        if (range.offset > this._offset) {
          existingBefore = this._ids.slice(0, range.offset - this._offset);
        }
        existingAfter = [];
        if (rangeIds.length === range.limit && currentEnd > rangeIdsEnd) {
          existingAfter = this._ids.slice(rangeIdsEnd - this._offset);
        }
        this._ids = [].concat(existingBefore, rangeIds, existingAfter);
        this._idToIndexHash = null;
        return this._offset = Math.min(this._offset, range.offset);
      }
    };

    MutableQueryResultSet.prototype.updateModel = function(item) {
      var attr, existing, key, _ref;
      if (!item) {
        return;
      }
      existing = this._modelsHash[item.clientId];
      if (existing) {
        _ref = existing.constructor.attributes;
        for (key in _ref) {
          attr = _ref[key];
          if (attr instanceof AttributeJoinedData && item[attr.modelKey] === void 0) {
            item[attr.modelKey] = existing[attr.modelKey];
          }
        }
      }
      this._modelsHash[item.clientId] = item;
      this._modelsHash[item.id] = item;
      return this._idToIndexHash = null;
    };

    MutableQueryResultSet.prototype.removeModelAtOffset = function(item, offset) {
      var idx;
      idx = offset - this._offset;
      delete this._modelsHash[item.clientId];
      delete this._modelsHash[item.id];
      this._ids.splice(idx, 1);
      return this._idToIndexHash = null;
    };

    MutableQueryResultSet.prototype.setQuery = function(query) {
      this._query = query.clone();
      return this._query.finalize();
    };

    return MutableQueryResultSet;

  })(QueryResultSet);

  module.exports = MutableQueryResultSet;

}).call(this);
