(function() {
  var QueryRange;

  QueryRange = (function() {
    QueryRange.infinite = function() {
      return new QueryRange({
        limit: null,
        offset: null
      });
    };

    QueryRange.rangeWithUnion = function(a, b) {
      if (a.isInfinite() || b.isInfinite()) {
        return QueryRange.infinite();
      }
      if (!a.isContiguousWith(b)) {
        throw new Error('You cannot union ranges which do not touch or intersect.');
      }
      return new QueryRange({
        start: Math.min(a.start, b.start),
        end: Math.max(a.end, b.end)
      });
    };

    QueryRange.rangesBySubtracting = function(a, b) {
      var uncovered;
      if (!b) {
        return [];
      }
      if (a.isInfinite() || b.isInfinite()) {
        throw new Error("You cannot subtract infinite ranges.");
      }
      uncovered = [];
      if (b.start > a.start) {
        uncovered.push(new QueryRange({
          start: a.start,
          end: Math.min(a.end, b.start)
        }));
      }
      if (b.end < a.end) {
        uncovered.push(new QueryRange({
          start: Math.max(a.start, b.end),
          end: a.end
        }));
      }
      return uncovered;
    };

    Object.defineProperty(QueryRange.prototype, "start", {
      enumerable: false,
      get: function() {
        return this.offset;
      }
    });

    Object.defineProperty(QueryRange.prototype, "end", {
      enumerable: false,
      get: function() {
        return this.offset + this.limit;
      }
    });

    function QueryRange(_arg) {
      var end, start, _ref;
      _ref = _arg != null ? _arg : {}, this.limit = _ref.limit, this.offset = _ref.offset, start = _ref.start, end = _ref.end;
      if (start != null) {
        if (this.offset == null) {
          this.offset = start;
        }
      }
      if (end != null) {
        if (this.limit == null) {
          this.limit = end - this.offset;
        }
      }
      if (this.limit === void 0) {
        throw new Error("You must specify a limit");
      }
      if (this.offset === void 0) {
        throw new Error("You must specify an offset");
      }
    }

    QueryRange.prototype.clone = function() {
      return new QueryRange({
        limit: this.limit,
        offset: this.offset
      });
    };

    QueryRange.prototype.isInfinite = function() {
      return this.limit === null && this.offset === null;
    };

    QueryRange.prototype.isEqual = function(b) {
      return this.start === b.start && this.end === b.end;
    };

    QueryRange.prototype.isContiguousWith = function(b) {
      var _ref, _ref1;
      if (this.isInfinite() || b.isInfinite()) {
        return true;
      }
      return (this.start <= (_ref = b.start) && _ref <= this.end) || (this.start <= (_ref1 = b.end) && _ref1 <= this.end);
    };

    QueryRange.prototype.toString = function() {
      return "QueryRange{" + this.start + " - " + this.end + "}";
    };

    return QueryRange;

  })();

  module.exports = QueryRange;

}).call(this);
