(function() {
  var DatabaseStore, QuerySubscription, QuerySubscriptionPool, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require('underscore');

  QuerySubscription = require('./query-subscription')["default"];

  DatabaseStore = null;


  /*
  Public: The QuerySubscriptionPool maintains a list of all of the query
  subscriptions in the app. In the future, this class will monitor performance,
  merge equivalent subscriptions, etc.
   */

  QuerySubscriptionPool = (function() {
    function QuerySubscriptionPool() {
      this._onChange = __bind(this._onChange, this);
      this._setup = __bind(this._setup, this);
      this._keyForQuery = __bind(this._keyForQuery, this);
      this._runCleanupChecks = __bind(this._runCleanupChecks, this);
      this._scheduleCleanupCheckForSubscription = __bind(this._scheduleCleanupCheckForSubscription, this);
      this.printSubscriptions = __bind(this.printSubscriptions, this);
      this.addPrivateSubscription = __bind(this.addPrivateSubscription, this);
      this.add = __bind(this.add, this);
      this._subscriptions = {};
      this._cleanupChecks = [];
      this._setup();
    }

    QuerySubscriptionPool.prototype.add = function(query, callback) {
      var key, subscription;
      if (NylasEnv.inDevMode()) {
        callback._registrationPoint = this._formatRegistrationPoint((new Error).stack);
      }
      key = this._keyForQuery(query);
      subscription = this._subscriptions[key];
      if (!subscription) {
        subscription = new QuerySubscription(query);
        this._subscriptions[key] = subscription;
      }
      subscription.addCallback(callback);
      return (function(_this) {
        return function() {
          subscription.removeCallback(callback);
          return _this._scheduleCleanupCheckForSubscription(key);
        };
      })(this);
    };

    QuerySubscriptionPool.prototype.addPrivateSubscription = function(key, subscription, callback) {
      this._subscriptions[key] = subscription;
      subscription.addCallback(callback);
      return (function(_this) {
        return function() {
          subscription.removeCallback(callback);
          return _this._scheduleCleanupCheckForSubscription(key);
        };
      })(this);
    };

    QuerySubscriptionPool.prototype.printSubscriptions = function() {
      var callback, key, subscription, _i, _len, _ref, _ref1;
      if (!NylasEnv.inDevMode()) {
        return console.log("printSubscriptions is only available in developer mode.");
      }
      _ref = this._subscriptions;
      for (key in _ref) {
        subscription = _ref[key];
        console.log(key);
        console.group();
        _ref1 = subscription._callbacks;
        for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
          callback = _ref1[_i];
          console.log("" + callback._registrationPoint);
        }
        console.groupEnd();
      }
    };

    QuerySubscriptionPool.prototype._scheduleCleanupCheckForSubscription = function(key) {
      if (this._cleanupChecks.length === 0) {
        _.defer(this._runCleanupChecks);
      }
      return this._cleanupChecks.push(key);
    };

    QuerySubscriptionPool.prototype._runCleanupChecks = function() {
      var key, subscription, _i, _len, _ref;
      _ref = this._cleanupChecks;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        key = _ref[_i];
        subscription = this._subscriptions[key];
        if (subscription && subscription.callbackCount() === 0) {
          delete this._subscriptions[key];
        }
      }
      return this._cleanupChecks = [];
    };

    QuerySubscriptionPool.prototype._formatRegistrationPoint = function(stack) {
      var hasRx, ii, seenRx;
      stack = stack.split('\n');
      ii = 0;
      seenRx = false;
      while (ii < stack.length) {
        hasRx = stack[ii].indexOf('rx.lite') !== -1;
        seenRx || (seenRx = hasRx);
        if (seenRx === true && !hasRx) {
          break;
        }
        ii += 1;
      }
      return stack.slice(ii, +(ii + 4) + 1 || 9e9).join('\n');
    };

    QuerySubscriptionPool.prototype._keyForQuery = function(query) {
      return query.sql();
    };

    QuerySubscriptionPool.prototype._setup = function() {
      if (DatabaseStore == null) {
        DatabaseStore = require('../stores/database-store');
      }
      return DatabaseStore.listen(this._onChange);
    };

    QuerySubscriptionPool.prototype._onChange = function(record) {
      var key, subscription, _ref, _results;
      _ref = this._subscriptions;
      _results = [];
      for (key in _ref) {
        subscription = _ref[key];
        _results.push(subscription.applyChangeRecord(record));
      }
      return _results;
    };

    return QuerySubscriptionPool;

  })();

  module.exports = new QuerySubscriptionPool();

}).call(this);
