(function() {
  var DatabaseObjectRegistry, DefaultResourcePath, TaskRegistry, Utils, fs, imageData, moment, path, tz, _,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
    __slice = [].slice;

  _ = require('underscore');

  fs = require('fs-plus');

  path = require('path');

  moment = require('moment-timezone');

  tz = moment.tz.guess();

  if (!tz) {
    console.error("Utils:TimeZone could not be determined. This should not happen!");
  }

  DefaultResourcePath = null;

  TaskRegistry = require('../../task-registry')["default"];

  DatabaseObjectRegistry = require('../../database-object-registry')["default"];

  imageData = null;

  module.exports = Utils = {
    waitFor: function(latch, options) {
      var expire, timeout;
      if (options == null) {
        options = {};
      }
      timeout = options.timeout || 400;
      expire = Date.now() + timeout;
      return new Promise(function(resolve, reject) {
        var attempt;
        attempt = function() {
          if (Date.now() > expire) {
            return reject(new Error("Utils.waitFor hit timeout (" + timeout + "ms) without firing."));
          }
          if (latch()) {
            return resolve();
          }
          return window.requestAnimationFrame(attempt);
        };
        return attempt();
      });
    },
    showIconForAttachments: function(files) {
      if (!(files instanceof Array)) {
        return false;
      }
      return files.find(function(f) {
        return !f.contentId || f.size > 12 * 1024;
      });
    },
    extractTextFromHtml: function(html, _arg) {
      var maxLength;
      maxLength = (_arg != null ? _arg : {}).maxLength;
      if ((html != null ? html : "").trim().length === 0) {
        return "";
      }
      if (maxLength && html.length > maxLength) {
        html = html.slice(0, maxLength);
      }
      return (new DOMParser()).parseFromString(html, "text/html").body.innerText;
    },
    registeredObjectReviver: function(k, v) {
      var type;
      type = v != null ? v.__constructorName : void 0;
      if (!type) {
        return v;
      }
      if (DatabaseObjectRegistry.isInRegistry(type)) {
        return DatabaseObjectRegistry.deserialize(type, v);
      }
      if (TaskRegistry.isInRegistry(type)) {
        return TaskRegistry.deserialize(type, v);
      }
      return v;
    },
    registeredObjectReplacer: function(k, v) {
      var type;
      if (_.isObject(v)) {
        type = this[k].constructor.name;
        if (DatabaseObjectRegistry.isInRegistry(type) || TaskRegistry.isInRegistry(type)) {
          v.__constructorName = type;
        }
      }
      return v;
    },
    timeZone: tz,
    shortTimeString: function(time) {
      var diff, format;
      if (!time) {
        return "";
      }
      diff = moment().diff(time, 'days', true);
      if (diff <= 1) {
        format = "h:mm a";
      } else if (diff > 1 && diff <= 365) {
        format = "MMM D";
      } else {
        format = "MMM D YYYY";
      }
      return moment(time).format(format);
    },
    fullTimeString: function(time) {
      if (!time) {
        return "";
      }
      return moment(time).tz(Utils.timeZone).format("dddd, MMMM Do YYYY, h:mm:ss a z");
    },
    fastOmit: function(props, without) {
      var otherProps, w, _i, _len;
      otherProps = Object.assign({}, props);
      for (_i = 0, _len = without.length; _i < _len; _i++) {
        w = without[_i];
        delete otherProps[w];
      }
      return otherProps;
    },
    isHash: function(object) {
      return _.isObject(object) && !_.isFunction(object) && !_.isArray(object);
    },
    escapeRegExp: function(str) {
      return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
    },
    wordSearchRegExp: function(str) {
      if (str == null) {
        str = "";
      }
      return new RegExp("((?:^|\\W|$)" + (Utils.escapeRegExp(str.trim())) + ")", "ig");
    },
    deepClone: function(object, customizer, stackSeen, stackRefs) {
      var key, newObject, newVal, seenIndex, _i, _len, _ref;
      if (stackSeen == null) {
        stackSeen = [];
      }
      if (stackRefs == null) {
        stackRefs = [];
      }
      if (!_.isObject(object)) {
        return object;
      }
      if (_.isFunction(object)) {
        return object;
      }
      if (_.isArray(object)) {
        newObject = [];
      } else {
        newObject = Object.create(Object.getPrototypeOf(object));
      }
      seenIndex = stackSeen.indexOf(object);
      if (seenIndex >= 0) {
        return stackRefs[seenIndex];
      }
      stackSeen.push(object);
      stackRefs.push(newObject);
      _ref = Object.getOwnPropertyNames(object);
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        key = _ref[_i];
        newVal = Utils.deepClone(object[key], customizer, stackSeen, stackRefs);
        if (_.isFunction(customizer)) {
          newObject[key] = customizer(key, newVal);
        } else {
          newObject[key] = newVal;
        }
      }
      return newObject;
    },
    toSet: function(arr) {
      var item, set, _i, _len;
      if (arr == null) {
        arr = [];
      }
      set = {};
      for (_i = 0, _len = arr.length; _i < _len; _i++) {
        item = arr[_i];
        set[item] = true;
      }
      return set;
    },
    shouldDisplayAsImage: function(file) {
      var ext, extensions, name, size, _ref, _ref1, _ref2, _ref3, _ref4;
      if (file == null) {
        file = {};
      }
      name = (_ref = (_ref1 = (_ref2 = file.filename) != null ? _ref2 : file.fileName) != null ? _ref1 : file.name) != null ? _ref : "";
      size = (_ref3 = (_ref4 = file.size) != null ? _ref4 : file.fileSize) != null ? _ref3 : 0;
      ext = path.extname(name).toLowerCase();
      extensions = ['.jpg', '.bmp', '.gif', '.png', '.jpeg'];
      return __indexOf.call(extensions, ext) >= 0 && size > 512 && size < 1024 * 1024 * 5;
    },
    encodeHTMLEntities: function(value) {
      var NON_ALPHANUMERIC_REGEXP, SURROGATE_PAIR_REGEXP, alphaFix, pairFix;
      SURROGATE_PAIR_REGEXP = /[\uD800-\uDBFF][\uDC00-\uDFFF]/g;
      pairFix = function(value) {
        var hi, low;
        hi = value.charCodeAt(0);
        low = value.charCodeAt(1);
        return '&#' + (((hi - 0xD800) * 0x400) + (low - 0xDC00) + 0x10000) + ';';
      };
      NON_ALPHANUMERIC_REGEXP = /([^\#-~| |!])/g;
      alphaFix = function(value) {
        return '&#' + value.charCodeAt(0) + ';';
      };
      return value.replace(/&/g, '&amp;').replace(SURROGATE_PAIR_REGEXP, pairFix).replace(NON_ALPHANUMERIC_REGEXP, alphaFix).replace(/</g, '&lt;').replace(/>/g, '&gt;');
    },
    modelFreeze: function(o) {
      Object.freeze(o);
      return Object.getOwnPropertyNames(o).forEach(function(key) {
        var val;
        val = o[key];
        if (typeof val === 'object' && val !== null && !Object.isFrozen(val)) {
          return Utils.modelFreeze(val);
        }
      });
    },
    generateTempId: function() {
      var s4;
      s4 = function() {
        return Math.floor((1 + Math.random()) * 0x10000).toString(16).substring(1);
      };
      return 'local-' + s4() + s4() + '-' + s4();
    },
    generateFakeServerId: function() {
      var s5;
      s5 = function() {
        return Math.floor((1 + Math.random()) * 0x10000000).toString(36).substring(1);
      };
      return s5() + s5() + s5() + s5() + s5();
    },
    isTempId: function(id) {
      if (!(id && _.isString(id))) {
        return false;
      }
      return id.slice(0, 6) === 'local-';
    },
    tableNameForJoin: function(primaryKlass, secondaryKlass) {
      return "" + primaryKlass.name + secondaryKlass.name;
    },
    imageNamed: function(fullname, resourcePath) {
      var basename, ext, file, files, imagesPath, name, plat, ratio, _base, _i, _len, _ref, _ref1, _ref10, _ref11, _ref12, _ref2, _ref3, _ref4, _ref5, _ref6, _ref7, _ref8, _ref9;
      _ref = fullname.split('.'), name = _ref[0], ext = _ref[1];
      if (DefaultResourcePath == null) {
        DefaultResourcePath = NylasEnv.getLoadSettings().resourcePath;
      }
      if (resourcePath == null) {
        resourcePath = DefaultResourcePath;
      }
      if (!imageData) {
        imageData = (_ref1 = NylasEnv.fileListCache().imageData) != null ? _ref1 : "{}";
        Utils.images = (_ref2 = JSON.parse(imageData)) != null ? _ref2 : {};
      }
      if (!(Utils != null ? (_ref3 = Utils.images) != null ? _ref3[resourcePath] : void 0 : void 0)) {
        if (Utils.images == null) {
          Utils.images = {};
        }
        if ((_base = Utils.images)[resourcePath] == null) {
          _base[resourcePath] = {};
        }
        imagesPath = path.join(resourcePath, 'static', 'images');
        files = fs.listTreeSync(imagesPath);
        for (_i = 0, _len = files.length; _i < _len; _i++) {
          file = files[_i];
          file = file.replace(/\\/g, '/');
          basename = path.basename(file);
          Utils.images[resourcePath][path.basename(file)] = file;
        }
        NylasEnv.fileListCache().imageData = JSON.stringify(Utils.images);
      }
      plat = (_ref4 = process.platform) != null ? _ref4 : "";
      ratio = (_ref5 = window.devicePixelRatio) != null ? _ref5 : 1;
      return (_ref6 = (_ref7 = (_ref8 = (_ref9 = (_ref10 = (_ref11 = (_ref12 = Utils.images[resourcePath]["" + name + "-" + plat + "@" + ratio + "x." + ext]) != null ? _ref12 : Utils.images[resourcePath]["" + name + "@" + ratio + "x." + ext]) != null ? _ref11 : Utils.images[resourcePath]["" + name + "-" + plat + "." + ext]) != null ? _ref10 : Utils.images[resourcePath]["" + name + "." + ext]) != null ? _ref9 : Utils.images[resourcePath]["" + name + "-" + plat + "@2x." + ext]) != null ? _ref8 : Utils.images[resourcePath]["" + name + "@2x." + ext]) != null ? _ref7 : Utils.images[resourcePath]["" + name + "-" + plat + "@1x." + ext]) != null ? _ref6 : Utils.images[resourcePath]["" + name + "@1x." + ext];
    },
    subjectWithPrefix: function(subject, prefix) {
      if (subject.search(/fwd:/i) === 0) {
        return subject.replace(/fwd:/i, prefix);
      } else if (subject.search(/re:/i) === 0) {
        return subject.replace(/re:/i, prefix);
      } else {
        return "" + prefix + " " + subject;
      }
    },
    emailsHaveSameDomain: function() {
      var args, domains, toMatch;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      if (args.length < 2) {
        return false;
      }
      domains = args.map(function(email) {
        if (email == null) {
          email = "";
        }
        return _.last(email.toLowerCase().trim().split("@"));
      });
      toMatch = domains[0];
      return _.every(domains, function(domain) {
        return domain.length > 0 && toMatch === domain;
      });
    },
    emailHasCommonDomain: function(email) {
      var domain, _ref;
      if (email == null) {
        email = "";
      }
      domain = _.last(email.toLowerCase().trim().split("@"));
      return (_ref = Utils.commonDomains[domain]) != null ? _ref : false;
    },
    toEquivalentEmailForm: function(email) {
      var domainPart1, localPart1, _ref, _ref1, _ref2, _ref3;
      localPart1 = (_ref = (_ref1 = /([^+]+?)[+@].*/gi.exec(email)) != null ? _ref1[1] : void 0) != null ? _ref : "";
      domainPart1 = (_ref2 = (_ref3 = /@(.+)/gi.exec(email)) != null ? _ref3[1] : void 0) != null ? _ref2 : "";
      email = ("" + localPart1 + "@" + domainPart1).trim().toLowerCase();
      return email;
    },
    emailIsEquivalent: function(email1, email2) {
      if (email1 == null) {
        email1 = "";
      }
      if (email2 == null) {
        email2 = "";
      }
      if (email1 === email2) {
        return true;
      }
      email1 = Utils.toEquivalentEmailForm(email1);
      email2 = Utils.toEquivalentEmailForm(email2);
      return email1 === email2;
    },
    rectVisibleInRect: function(r1, r2) {
      return !(r2.left > r1.right || r2.right < r1.left || r2.top > r1.bottom || r2.bottom < r1.top);
    },
    isEqualReact: function(a, b, options) {
      var _ref;
      if (options == null) {
        options = {};
      }
      options.functionsAreEqual = true;
      options.ignoreKeys = ((_ref = options.ignoreKeys) != null ? _ref : []).push("clientId");
      return Utils.isEqual(a, b, options);
    },
    isEqual: function(a, b, options) {
      var value;
      if (options == null) {
        options = {};
      }
      value = Utils._isEqual(a, b, [], [], options);
      if (options.logWhenFalse) {
        if (value === false) {
          console.log("isEqual is false", a, b, options);
        }
        return value;
      } else {

      }
      return value;
    },
    _isEqual: function(a, b, aStack, bStack, options) {
      var aCtor, areArrays, bCtor, className, key, keys, keysToIgnore, length, _i, _len, _ref;
      if (options == null) {
        options = {};
      }
      if (a === b) {
        return a !== 0 || 1 / a === 1 / b;
      }
      if (a === null || b === null) {
        return a === b;
      }
      if (((a != null ? a._wrapped : void 0) != null)) {
        a = a._wrapped;
      }
      if (((b != null ? b._wrapped : void 0) != null)) {
        b = b._wrapped;
      }
      if (options.functionsAreEqual) {
        if (_.isFunction(a) && _.isFunction(b)) {
          return true;
        }
      }
      className = toString.call(a);
      if (className !== toString.call(b)) {
        return false;
      }
      switch (className) {
        case '[object RegExp]':
        case '[object String]':
          return '' + a === '' + b;
        case '[object Number]':
          if (+a !== +a) {
            return +b !== +b;
          }
          if (+a === 0) {
            return 1 / +a === 1 / b;
          } else {
            return +a === +b;
          }
        case '[object Date]':
        case '[object Boolean]':
          return +a === +b;
      }
      areArrays = className === '[object Array]';
      if (!areArrays) {
        if (typeof a !== 'object' || typeof b !== 'object') {
          return false;
        }
        aCtor = a.constructor;
        bCtor = b.constructor;
        if (aCtor !== bCtor && !(_.isFunction(aCtor) && aCtor instanceof aCtor && _.isFunction(bCtor) && bCtor instanceof bCtor) && ('constructor' in a && 'constructor' in b)) {
          return false;
        }
      }
      aStack = aStack != null ? aStack : [];
      bStack = bStack != null ? bStack : [];
      length = aStack.length;
      while (length--) {
        if (aStack[length] === a) {
          return bStack[length] === b;
        }
      }
      aStack.push(a);
      bStack.push(b);
      if (areArrays) {
        length = a.length;
        if (length !== b.length) {
          return false;
        }
        while (length--) {
          if (!Utils._isEqual(a[length], b[length], aStack, bStack, options)) {
            return false;
          }
        }
      } else {
        key = void 0;
        keys = Object.keys(a);
        length = keys.length;
        if (Object.keys(b).length !== length) {
          return false;
        }
        keysToIgnore = {};
        if (options.ignoreKeys && _.isArray(options.ignoreKeys)) {
          _ref = options.ignoreKeys;
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            key = _ref[_i];
            keysToIgnore[key] = true;
          }
        }
        while (length--) {
          key = keys[length];
          if (key in keysToIgnore) {
            continue;
          }
          if (!(_.has(b, key) && Utils._isEqual(a[key], b[key], aStack, bStack, options))) {
            return false;
          }
        }
      }
      aStack.pop();
      bStack.pop();
      return true;
    },
    commonDomains: {
      "aol.com": true,
      "att.net": true,
      "comcast.net": true,
      "facebook.com": true,
      "gmail.com": true,
      "gmx.com": true,
      "googlemail.com": true,
      "google.com": true,
      "hotmail.com": true,
      "hotmail.co.uk": true,
      "mac.com": true,
      "me.com": true,
      "mail.com": true,
      "msn.com": true,
      "live.com": true,
      "sbcglobal.net": true,
      "verizon.net": true,
      "yahoo.com": true,
      "yahoo.co.uk": true,
      "email.com": true,
      "games.com": true,
      "gmx.net": true,
      "hush.com": true,
      "hushmail.com": true,
      "inbox.com": true,
      "lavabit.com": true,
      "love.com": true,
      "pobox.com": true,
      "rocketmail.com": true,
      "safe-mail.net": true,
      "wow.com": true,
      "ygm.com": true,
      "ymail.com": true,
      "zoho.com": true,
      "fastmail.fm": true,
      "bellsouth.net": true,
      "charter.net": true,
      "cox.net": true,
      "earthlink.net": true,
      "juno.com": true,
      "btinternet.com": true,
      "virginmedia.com": true,
      "blueyonder.co.uk": true,
      "freeserve.co.uk": true,
      "live.co.uk": true,
      "ntlworld.com": true,
      "o2.co.uk": true,
      "orange.net": true,
      "sky.com": true,
      "talktalk.co.uk": true,
      "tiscali.co.uk": true,
      "virgin.net": true,
      "wanadoo.co.uk": true,
      "bt.com": true,
      "sina.com": true,
      "qq.com": true,
      "naver.com": true,
      "hanmail.net": true,
      "daum.net": true,
      "nate.com": true,
      "yahoo.co.jp": true,
      "yahoo.co.kr": true,
      "yahoo.co.id": true,
      "yahoo.co.in": true,
      "yahoo.com.sg": true,
      "yahoo.com.ph": true,
      "hotmail.fr": true,
      "live.fr": true,
      "laposte.net": true,
      "yahoo.fr": true,
      "wanadoo.fr": true,
      "orange.fr": true,
      "gmx.fr": true,
      "sfr.fr": true,
      "neuf.fr": true,
      "free.fr": true,
      "gmx.de": true,
      "hotmail.de": true,
      "live.de": true,
      "online.de": true,
      "t-online.de": true,
      "web.de": true,
      "yahoo.de": true,
      "mail.ru": true,
      "rambler.ru": true,
      "yandex.ru": true,
      "hotmail.be": true,
      "live.be": true,
      "skynet.be": true,
      "voo.be": true,
      "tvcablenet.be": true,
      "hotmail.com.ar": true,
      "live.com.ar": true,
      "yahoo.com.ar": true,
      "fibertel.com.ar": true,
      "speedy.com.ar": true,
      "arnet.com.ar": true,
      "hotmail.com": true,
      "gmail.com": true,
      "yahoo.com.mx": true,
      "live.com.mx": true,
      "yahoo.com": true,
      "hotmail.es": true,
      "live.com": true,
      "hotmail.com.mx": true,
      "prodigy.net.mx": true,
      "msn.com": true
    },
    ensureSerialExecution: function(fn, paramMerge) {
      var fnFinished, fnReinvoked, fnRun;
      fnRun = null;
      fnReinvoked = function() {
        return fn.next;
      };
      fnFinished = function() {
        var args;
        fn.executing = false;
        if (fn.next) {
          args = fn.next;
          fn.next = null;
          return fnRun.apply(null, args);
        }
      };
      fnRun = function() {
        if (fn.executing) {
          if (fn.next) {
            return fn.next = paramMerge(fn.next, arguments);
          } else {
            return fn.next = arguments;
          }
        } else {
          fn.executing = true;
          return fn.apply(this, [fnFinished, fnReinvoked].concat(__slice.call(arguments)));
        }
      };
      return fnRun;
    },
    jsonParse: (function(_this) {
      return function(jsonString) {
        var data, err;
        data = null;
        try {
          data = JSON.parse(jsonString);
        } catch (_error) {
          err = _error;
          console.error("JSON parse error: " + err);
        }
        return data;
      };
    })(this),
    hueForString: function(str) {
      if (str == null) {
        str = '';
      }
      return str.split('').map(function(c) {
        return c.charCodeAt();
      }).reduce(function(n, a) {
        return n + a;
      }) % 360;
    },
    likelyNonHumanEmail: function(email) {
      var prefixes, re, reStr;
      prefixes = ["noreply", "no-reply", "donotreply", "do-not-reply", "bounce[s]?@", "notification[s]?@", "support@", "alert[s]?@", "news@", "info@", "automated@", "list[s]?@", "distribute[s]?@", "catchall@", "catch-all@"];
      reStr = "(" + (prefixes.join("|")) + ")";
      re = new RegExp(reStr, "gi");
      return re.test(email) || email.length > 64;
    },
    overlapsBounds: function(bounds, test) {
      return (test.start <= bounds.end && test.end >= bounds.start) || (test.start <= bounds.end && test.start >= bounds.start) || (test.end >= bounds.start && test.end <= bounds.end) || (test.end >= bounds.end && test.start <= bounds.start);
    },
    mean: function(values) {
      var sum;
      if (values == null) {
        values = [];
      }
      if (values.length === 0) {
        throw new Error("Can't average zero values");
      }
      sum = values.reduce((function(sum, value) {
        return sum + value;
      }), 0);
      return sum / values.length;
    },
    stdev: function(values) {
      var avg, squareDiffs;
      if (values == null) {
        values = [];
      }
      if (values.length === 0) {
        throw new Error("Can't stdev zero values");
      }
      avg = Utils.mean(values);
      squareDiffs = values.map(function(val) {
        return Math.pow(val - avg, 2);
      });
      return Math.sqrt(Utils.mean(squareDiffs));
    }
  };

}).call(this);
