(function() {
  var Account, AccountStore, Actions, DatabaseStore, NylasAPI, NylasStore, Utils, configAccountsKey, configTokensKey, configVersionKey, keytar, keytarServiceName, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  NylasStore = require('nylas-store');

  Actions = require('../actions');

  Account = require('../models/account')["default"];

  Utils = require('../models/utils');

  DatabaseStore = require('./database-store');

  keytar = require('keytar');

  NylasAPI = null;

  configAccountsKey = "nylas.accounts";

  configVersionKey = "nylas.accountsVersion";

  configTokensKey = "nylas.accountTokens";

  keytarServiceName = 'Nylas';


  /*
  Public: The AccountStore listens to changes to the available accounts in
  the database and exposes the currently active Account via {::current}
  
  Section: Stores
   */

  AccountStore = (function(_super) {
    __extends(AccountStore, _super);

    function AccountStore() {
      this._importFakeData = __bind(this._importFakeData, this);
      this.tokenForAccountId = __bind(this.tokenForAccountId, this);
      this.current = __bind(this.current, this);
      this.aliasesFor = __bind(this.aliasesFor, this);
      this.aliases = __bind(this.aliases, this);
      this.accountForId = __bind(this.accountForId, this);
      this.accountForEmail = __bind(this.accountForEmail, this);
      this.accountForItems = __bind(this.accountForItems, this);
      this.accountsForItems = __bind(this.accountsForItems, this);
      this.accounts = __bind(this.accounts, this);
      this.refreshHealthOfAccounts = __bind(this.refreshHealthOfAccounts, this);
      this.addAccountFromJSON = __bind(this.addAccountFromJSON, this);
      this._onReorderAccount = __bind(this._onReorderAccount, this);
      this._onRemoveAccount = __bind(this._onRemoveAccount, this);
      this._onUpdateAccount = __bind(this._onUpdateAccount, this);
      this._save = __bind(this._save, this);
      this._enforceAccountsValidity = __bind(this._enforceAccountsValidity, this);
      this._loadAccounts = __bind(this._loadAccounts, this);
      this._loadAccounts();
      this.listenTo(Actions.removeAccount, this._onRemoveAccount);
      this.listenTo(Actions.updateAccount, this._onUpdateAccount);
      this.listenTo(Actions.reorderAccount, this._onReorderAccount);
      if (NylasEnv.isWorkWindow() && ['staging', 'production'].includes(NylasEnv.config.get('env'))) {
        setTimeout((function(_this) {
          return function() {
            return _this.refreshHealthOfAccounts(_this._accounts.map(function(a) {
              return a.id;
            }));
          };
        })(this), 2000);
      }
      NylasEnv.config.onDidChange(configVersionKey, (function(_this) {
        return function(change) {
          var CategoryStore, accountIds, newAccountIds, newId, oldAccountIds;
          if (_this._version / 1 === change.newValue / 1) {
            return;
          }
          oldAccountIds = _.pluck(_this._accounts, 'id');
          _this._loadAccounts();
          accountIds = _.pluck(_this._accounts, 'id');
          newAccountIds = _.difference(accountIds, oldAccountIds);
          if (newAccountIds.length > 0) {
            newId = newAccountIds[0];
            CategoryStore = require('./category-store');
            return CategoryStore.whenCategoriesReady(newId).then(function() {
              Actions.focusDefaultMailboxPerspectiveForAccounts([newId], {
                sidebarAccountIds: accountIds
              });
              return _.defer(function() {
                return Actions.setCollapsedSidebarItem('Inbox', false);
              });
            });
          }
        };
      })(this));
    }

    AccountStore.prototype._loadAccounts = function() {
      var account, error, json, key, oldTokens, val, _i, _j, _len, _len1, _ref, _ref1;
      try {
        this._caches = {};
        this._tokens = {};
        this._version = NylasEnv.config.get(configVersionKey) || 0;
        this._accounts = [];
        _ref = NylasEnv.config.get(configAccountsKey) || [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          json = _ref[_i];
          this._accounts.push((new Account).fromJSON(json));
        }
        this._enforceAccountsValidity();
        oldTokens = NylasEnv.config.get(configTokensKey);
        if (oldTokens) {
          this._tokens = oldTokens;
          for (key in oldTokens) {
            val = oldTokens[key];
            account = this.accountForId(key);
            if (!account) {
              continue;
            }
            keytar.replacePassword(keytarServiceName, account.emailAddress, val);
          }
        } else {
          this._tokens = {};
          _ref1 = this._accounts;
          for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
            account = _ref1[_j];
            this._tokens[account.id] = keytar.getPassword(keytarServiceName, account.emailAddress);
          }
        }
      } catch (_error) {
        error = _error;
        NylasEnv.reportError(error);
      }
      return this._trigger();
    };

    AccountStore.prototype._enforceAccountsValidity = function() {
      var message, seenEmails, seenIds;
      seenIds = {};
      seenEmails = {};
      message = null;
      this._accounts = this._accounts.filter((function(_this) {
        return function(account) {
          if (!account.emailAddress) {
            message = "Assertion failure: One of the accounts in config.json did not have an emailAddress, and was removed. You should re-link the account.";
            return false;
          }
          if (seenIds[account.id]) {
            message = "Assertion failure: Two of the accounts in config.json had the same ID and one was removed. Please give each account a separate ID.";
            return false;
          }
          if (seenEmails[account.emailAddress]) {
            message = "Assertion failure: Two of the accounts in config.json had the same email address and one was removed.";
            return false;
          }
          seenIds[account.id] = true;
          seenEmails[account.emailAddress] = true;
          return true;
        };
      })(this));
      if (message && NylasEnv.isMainWindow()) {
        return NylasEnv.showErrorDialog("N1 was unable to load your account preferences.\n\n" + message);
      }
    };

    AccountStore.prototype._trigger = function() {
      var account, err, _i, _len, _ref;
      _ref = this._accounts;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        account = _ref[_i];
        if (!(account != null ? account.id : void 0)) {
          err = new Error("An invalid account was added to `this._accounts`");
          NylasEnv.reportError(err);
          this._accounts = _.compact(this._accounts);
        }
      }
      return this.trigger();
    };

    AccountStore.prototype._save = function() {
      this._version += 1;
      NylasEnv.config.set(configTokensKey, null);
      NylasEnv.config.set(configAccountsKey, this._accounts);
      NylasEnv.config.set(configVersionKey, this._version);
      return this._trigger();
    };

    AccountStore.prototype._onUpdateAccount = function(id, updated) {
      var account, idx;
      idx = _.findIndex(this._accounts, function(a) {
        return a.id === id;
      });
      account = this._accounts[idx];
      if (!account) {
        return;
      }
      account = _.extend(account, updated);
      this._caches = {};
      this._accounts[idx] = account;
      return this._save();
    };

    AccountStore.prototype._onRemoveAccount = function(id) {
      var account, ipc, remainingAccounts;
      account = _.findWhere(this._accounts, {
        id: id
      });
      if (!account) {
        return;
      }
      keytar.deletePassword(keytarServiceName, account.emailAddress);
      this._caches = {};
      remainingAccounts = _.without(this._accounts, account);
      Actions.focusDefaultMailboxPerspectiveForAccounts(remainingAccounts);
      _.defer((function(_this) {
        return function() {
          return Actions.setCollapsedSidebarItem('Inbox', true);
        };
      })(this));
      this._accounts = remainingAccounts;
      this._save();
      if (remainingAccounts.length === 0) {
        ipc = require('electron').ipcRenderer;
        return ipc.send('command', 'application:relaunch-to-initial-windows', {
          resetDatabase: true
        });
      }
    };

    AccountStore.prototype._onReorderAccount = function(id, newIdx) {
      var account, existingIdx;
      existingIdx = _.findIndex(this._accounts, function(a) {
        return a.id === id;
      });
      if (existingIdx === -1) {
        return;
      }
      account = this._accounts[existingIdx];
      this._caches = {};
      this._accounts.splice(existingIdx, 1);
      this._accounts.splice(newIdx, 0, account);
      return this._save();
    };

    AccountStore.prototype.addAccountFromJSON = function(json) {
      var account, existingIdx;
      if (!json.email_address || !json.provider) {
        console.error("Returned account data is invalid", json);
        console.log(JSON.stringify(json));
        throw new Error("Returned account data is invalid");
      }
      this._loadAccounts();
      this._tokens[json.id] = json.auth_token;
      keytar.replacePassword(keytarServiceName, json.email_address, json.auth_token);
      existingIdx = _.findIndex(this._accounts, function(a) {
        return a.id === json.id || a.emailAddress === json.email_address;
      });
      if (existingIdx === -1) {
        account = (new Account).fromJSON(json);
        this._accounts.push(account);
      } else {
        account = this._accounts[existingIdx];
        account.fromJSON(json);
      }
      return this._save();
    };

    AccountStore.prototype.refreshHealthOfAccounts = function(accountIds) {
      if (NylasAPI == null) {
        NylasAPI = require('../nylas-api');
      }
      return Promise.all(accountIds.map((function(_this) {
        return function(accountId) {
          return NylasAPI.makeRequest({
            path: '/account',
            accountId: accountId
          }).then(function(json) {
            var existing;
            existing = _this.accountForId(accountId);
            if (!existing) {
              return;
            }
            return existing.fromJSON(json);
          });
        };
      })(this)))["finally"]((function(_this) {
        return function() {
          _this._caches = {};
          return _this._save();
        };
      })(this));
    };

    AccountStore.prototype._cachedGetter = function(key, fn) {
      var _base;
      if ((_base = this._caches)[key] == null) {
        _base[key] = fn();
      }
      return this._caches[key];
    };

    AccountStore.prototype.accounts = function() {
      return this._accounts;
    };

    AccountStore.prototype.accountsForItems = function(items) {
      var accounts;
      accounts = {};
      items.forEach((function(_this) {
        return function(_arg) {
          var accountId;
          accountId = _arg.accountId;
          return accounts[accountId] != null ? accounts[accountId] : accounts[accountId] = _this.accountForId(accountId);
        };
      })(this));
      return _.values(accounts);
    };

    AccountStore.prototype.accountForItems = function(items) {
      var accounts;
      accounts = this.accountsForItems(items);
      if (accounts.length > 1) {
        return null;
      }
      return accounts[0];
    };

    AccountStore.prototype.accountForEmail = function(email) {
      var account, alias, _i, _j, _len, _len1, _ref, _ref1;
      _ref = this.accounts();
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        account = _ref[_i];
        if (Utils.emailIsEquivalent(email, account.emailAddress)) {
          return account;
        }
      }
      _ref1 = this.aliases();
      for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
        alias = _ref1[_j];
        if (Utils.emailIsEquivalent(email, alias.email)) {
          return this.accountForId(alias.accountId);
        }
      }
      return null;
    };

    AccountStore.prototype.accountForId = function(id) {
      return this._cachedGetter("accountForId:" + id, (function(_this) {
        return function() {
          return _.findWhere(_this._accounts, {
            id: id
          });
        };
      })(this));
    };

    AccountStore.prototype.aliases = function() {
      return this._cachedGetter("aliases", (function(_this) {
        return function() {
          var acc, alias, aliasContact, aliases, _i, _j, _len, _len1, _ref, _ref1;
          aliases = [];
          _ref = _this._accounts;
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            acc = _ref[_i];
            aliases.push(acc.me());
            _ref1 = acc.aliases;
            for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
              alias = _ref1[_j];
              aliasContact = acc.meUsingAlias(alias);
              aliasContact.isAlias = true;
              aliases.push(aliasContact);
            }
          }
          return aliases;
        };
      })(this));
    };

    AccountStore.prototype.aliasesFor = function(accountsOrIds) {
      var ids;
      ids = accountsOrIds.map(function(accOrId) {
        if (accOrId instanceof Account) {
          return accOrId.id;
        } else {
          return accOrId;
        }
      });
      return this.aliases().filter(function(contact) {
        var _ref;
        return _ref = contact.accountId, __indexOf.call(ids, _ref) >= 0;
      });
    };

    AccountStore.prototype.current = function() {
      throw new Error("AccountStore.current() has been deprecated.");
    };

    AccountStore.prototype.tokenForAccountId = function(id) {
      return this._tokens[id];
    };

    AccountStore.prototype._importFakeData = function(dir) {
      var Label, Message, Thread, account, destPath, downloadsDir, filename, filenames, fs, json, l, labels, lastMsg, m, messages, path, thread, threadAttachment, threadJSON, threadLabels, threadMessages, threadParticipants, threadUnread, threads, _i, _j, _k, _l, _len, _len1, _len2, _len3, _len4, _m, _ref, _ref1, _ref2;
      fs = require('fs-plus');
      path = require('path');
      Message = require('../models/message')["default"];
      Account = require('../models/account')["default"];
      Thread = require('../models/thread')["default"];
      Label = require('../models/label');
      this._caches = {};
      labels = [];
      threads = [];
      messages = [];
      account = this.accountForEmail('nora@nylas.com');
      if (!account) {
        account = new Account();
        account.serverId = account.clientId;
        account.emailAddress = "nora@nylas.com";
        account.organizationUnit = 'label';
        account.label = "Nora's Email";
        account.aliases = [];
        account.name = "Nora";
        account.provider = "gmail";
        json = account.toJSON();
        json.token = 'nope';
        this.addAccountFromJSON(json);
      }
      filenames = fs.readdirSync(path.join(dir, 'threads'));
      for (_i = 0, _len = filenames.length; _i < _len; _i++) {
        filename = filenames[_i];
        threadJSON = fs.readFileSync(path.join(dir, 'threads', filename));
        threadMessages = JSON.parse(threadJSON).map(function(j) {
          return (new Message).fromJSON(j);
        });
        threadLabels = [];
        threadParticipants = [];
        threadAttachment = false;
        threadUnread = false;
        for (_j = 0, _len1 = threadMessages.length; _j < _len1; _j++) {
          m = threadMessages[_j];
          m.accountId = account.id;
          _ref = m.categories;
          for (_k = 0, _len2 = _ref.length; _k < _len2; _k++) {
            l = _ref[_k];
            l.accountId = account.id;
          }
          _ref1 = m.files;
          for (_l = 0, _len3 = _ref1.length; _l < _len3; _l++) {
            l = _ref1[_l];
            l.accountId = account.id;
          }
          threadParticipants = threadParticipants.concat(m.participants());
          threadLabels = threadLabels.concat(m.categories);
          threadAttachment || (threadAttachment = m.files.length > 0);
          threadUnread || (threadUnread = m.unread);
        }
        threadParticipants = _.uniq(threadParticipants, function(p) {
          return p.email;
        });
        threadLabels = _.uniq(threadLabels, function(l) {
          return l.id;
        });
        labels = _.uniq(labels.concat(threadLabels), function(l) {
          return l.id;
        });
        lastMsg = _.last(threadMessages);
        thread = new Thread({
          accountId: account.id,
          serverId: lastMsg.threadId,
          clientId: lastMsg.threadId,
          subject: lastMsg.subject,
          lastMessageReceivedTimestamp: lastMsg.date,
          hasAttachment: threadAttachment,
          categories: threadLabels,
          participants: threadParticipants,
          unread: threadUnread,
          snippet: lastMsg.snippet,
          starred: lastMsg.starred
        });
        messages = messages.concat(threadMessages);
        threads.push(thread);
      }
      downloadsDir = path.join(dir, 'downloads');
      if (fs.existsSync(downloadsDir)) {
        _ref2 = fs.readdirSync(downloadsDir);
        for (_m = 0, _len4 = _ref2.length; _m < _len4; _m++) {
          filename = _ref2[_m];
          destPath = path.join(NylasEnv.getConfigDirPath(), 'downloads', filename);
          if (fs.existsSync(destPath)) {
            fs.removeSync(destPath);
          }
          fs.copySync(path.join(downloadsDir, filename), destPath);
        }
      }
      return DatabaseStore.inTransaction((function(_this) {
        return function(t) {
          return Promise.all([t.persistModel(account), t.persistModels(labels), t.persistModels(messages), t.persistModels(threads)]);
        };
      })(this)).then((function(_this) {
        return function() {
          return Actions.focusDefaultMailboxPerspectiveForAccounts([account.id]);
        };
      })(this)).then(function() {
        return new Promise(function(resolve, reject) {
          return setTimeout(resolve, 1000);
        });
      });
    };

    return AccountStore;

  })(NylasStore);

  module.exports = new AccountStore();

}).call(this);
