(function() {
  var Account, AccountStore, Categories, CategoryStore, NylasStore, NylasSyncStatusStore, Rx, StandardCategoryNames, asAccount, asAccountId, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
    __slice = [].slice;

  _ = require('underscore');

  Rx = require('rx-lite');

  NylasStore = require('nylas-store');

  AccountStore = require('./account-store');

  NylasSyncStatusStore = require('./nylas-sync-status-store');

  Account = require('../models/account')["default"];

  StandardCategoryNames = require('../models/category').StandardCategoryNames;

  Categories = require('nylas-observables').Categories;

  asAccount = function(a) {
    if (!a) {
      throw new Error("You must pass an Account or Account Id");
    }
    if (a instanceof Account) {
      return a;
    } else {
      return AccountStore.accountForId(a);
    }
  };

  asAccountId = function(a) {
    if (!a) {
      throw new Error("You must pass an Account or Account Id");
    }
    if (a instanceof Account) {
      return a.id;
    } else {
      return a;
    }
  };

  CategoryStore = (function(_super) {
    __extends(CategoryStore, _super);

    function CategoryStore() {
      this._onCategoriesChanged = __bind(this._onCategoriesChanged, this);
      this.whenCategoriesReady = __bind(this.whenCategoriesReady, this);
      this.getSpamCategory = __bind(this.getSpamCategory, this);
      this.getTrashCategory = __bind(this.getTrashCategory, this);
      this.getInboxCategory = __bind(this.getInboxCategory, this);
      this.getAllMailCategory = __bind(this.getAllMailCategory, this);
      this.getArchiveCategory = __bind(this.getArchiveCategory, this);
      this.getStandardCategories = __bind(this.getStandardCategories, this);
      this.getStandardCategory = __bind(this.getStandardCategory, this);
      this._categoryCache = {};
      this._standardCategories = {};
      this._userCategories = {};
      this._hiddenCategories = {};
      NylasEnv.config.onDidChange('core.workspace.showImportant', (function(_this) {
        return function() {
          if (!_this._categoryResult) {
            return;
          }
          return _this._onCategoriesChanged(_this._categoryResult);
        };
      })(this));
      Categories.forAllAccounts().sort().subscribe(this._onCategoriesChanged);
    }

    CategoryStore.prototype.byId = function(accountOrId, categoryId) {
      var categories, _ref;
      categories = (_ref = this._categoryCache[asAccountId(accountOrId)]) != null ? _ref : {};
      return categories[categoryId];
    };

    CategoryStore.prototype.categories = function(accountOrId) {
      var accountId, all, categories, _ref, _ref1;
      if (accountOrId == null) {
        accountOrId = null;
      }
      if (accountOrId) {
        return (_ref = _.values(this._categoryCache[asAccountId(accountOrId)])) != null ? _ref : [];
      } else {
        all = [];
        _ref1 = this._categoryCache;
        for (accountId in _ref1) {
          categories = _ref1[accountId];
          all = all.concat(_.values(categories));
        }
        return all;
      }
    };

    CategoryStore.prototype.standardCategories = function(accountOrId) {
      var _ref;
      return (_ref = this._standardCategories[asAccountId(accountOrId)]) != null ? _ref : [];
    };

    CategoryStore.prototype.hiddenCategories = function(accountOrId) {
      var _ref;
      return (_ref = this._hiddenCategories[asAccountId(accountOrId)]) != null ? _ref : [];
    };

    CategoryStore.prototype.userCategories = function(accountOrId) {
      var _ref;
      return (_ref = this._userCategories[asAccountId(accountOrId)]) != null ? _ref : [];
    };

    CategoryStore.prototype.getStandardCategory = function(accountOrId, name) {
      if (!accountOrId) {
        return null;
      }
      if (__indexOf.call(StandardCategoryNames, name) < 0) {
        throw new Error("'" + name + "' is not a standard category");
      }
      return _.findWhere(this._standardCategories[asAccountId(accountOrId)], {
        name: name
      });
    };

    CategoryStore.prototype.getStandardCategories = function() {
      var accOrId, accountsOrIds, cats, names, res, _i, _len;
      accountsOrIds = arguments[0], names = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      if (Array.isArray(accountsOrIds)) {
        res = [];
        for (_i = 0, _len = accountsOrIds.length; _i < _len; _i++) {
          accOrId = accountsOrIds[_i];
          cats = names.map((function(_this) {
            return function(name) {
              return _this.getStandardCategory(accOrId, name);
            };
          })(this));
          res = res.concat(_.compact(cats));
        }
        return res;
      } else {
        return _.compact(names.map((function(_this) {
          return function(name) {
            return _this.getStandardCategory(accountsOrIds, name);
          };
        })(this)));
      }
    };

    CategoryStore.prototype.getArchiveCategory = function(accountOrId) {
      var account;
      if (!accountOrId) {
        return null;
      }
      account = asAccount(accountOrId);
      if (!account) {
        return null;
      }
      if (account.usesFolders()) {
        return this.getStandardCategory(account.id, "archive");
      } else {
        return this.getStandardCategory(account.id, "all");
      }
    };

    CategoryStore.prototype.getAllMailCategory = function(accountOrId) {
      var account;
      if (!accountOrId) {
        return null;
      }
      account = asAccount(accountOrId);
      if (!account) {
        return null;
      }
      if (!account.usesLabels()) {
        return null;
      }
      return this.getStandardCategory(account.id, "all");
    };

    CategoryStore.prototype.getInboxCategory = function(accountOrId) {
      return this.getStandardCategory(accountOrId, "inbox");
    };

    CategoryStore.prototype.getTrashCategory = function(accountOrId) {
      return this.getStandardCategory(accountOrId, "trash");
    };

    CategoryStore.prototype.getSpamCategory = function(accountOrId) {
      return this.getStandardCategory(accountOrId, "spam");
    };

    CategoryStore.prototype.whenCategoriesReady = function(accountOrId) {
      var accounts, categoriesReady;
      accounts = AccountStore.accounts();
      if (accountOrId) {
        accounts = [asAccount(accountOrId)];
      }
      categoriesReady = (function(_this) {
        return function() {
          return _.every(accounts, function(acc) {
            var fetched, populated;
            populated = _this.categories(acc).length > 0;
            fetched = NylasSyncStatusStore.isSyncCompleteForAccount(acc.id, acc.categoryCollection());
            return populated && fetched;
          });
        };
      })(this);
      if (!categoriesReady()) {
        return new Promise((function(_this) {
          return function(resolve) {
            var categoryObservable, disposable, syncStatusObservable;
            syncStatusObservable = Rx.Observable.fromStore(NylasSyncStatusStore);
            categoryObservable = Rx.Observable.fromStore(_this);
            return disposable = Rx.Observable.merge(syncStatusObservable, categoryObservable).subscribe(function() {
              if (categoriesReady()) {
                disposable.dispose();
                return resolve();
              }
            });
          };
        })(this));
      }
      return Promise.resolve();
    };

    CategoryStore.prototype._onCategoriesChanged = function(categories) {
      var accountId, cat, filteredByAccount, items, _base, _i, _len, _name, _ref;
      this._categoryResult = categories;
      this._categoryCache = {};
      for (_i = 0, _len = categories.length; _i < _len; _i++) {
        cat = categories[_i];
        if ((_base = this._categoryCache)[_name = cat.accountId] == null) {
          _base[_name] = {};
        }
        this._categoryCache[cat.accountId][cat.id] = cat;
      }
      filteredByAccount = function(fn) {
        var result, _j, _len1, _name1;
        result = {};
        for (_j = 0, _len1 = categories.length; _j < _len1; _j++) {
          cat = categories[_j];
          if (!fn(cat)) {
            continue;
          }
          if (result[_name1 = cat.accountId] == null) {
            result[_name1] = [];
          }
          result[cat.accountId].push(cat);
        }
        return result;
      };
      this._standardCategories = filteredByAccount(function(cat) {
        return cat.isStandardCategory();
      });
      this._userCategories = filteredByAccount(function(cat) {
        return cat.isUserCategory();
      });
      this._hiddenCategories = filteredByAccount(function(cat) {
        return cat.isHiddenCategory();
      });
      _ref = this._standardCategories;
      for (accountId in _ref) {
        items = _ref[accountId];
        this._standardCategories[accountId].sort(function(a, b) {
          return StandardCategoryNames.indexOf(a.name) - StandardCategoryNames.indexOf(b.name);
        });
      }
      return this.trigger();
    };

    return CategoryStore;

  })(NylasStore);

  module.exports = new CategoryStore();

}).call(this);
