(function() {
  var AccountStore, ContactRankingStore, DatabaseStore, NylasStore, Rx,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Rx = require('rx-lite');

  NylasStore = require('nylas-store');

  DatabaseStore = require('./database-store');

  AccountStore = require('./account-store');

  ContactRankingStore = (function(_super) {
    __extends(ContactRankingStore, _super);

    function ContactRankingStore() {
      this.reset = __bind(this.reset, this);
      this.valuesForAllAccounts = __bind(this.valuesForAllAccounts, this);
      this.valueFor = __bind(this.valueFor, this);
      this._onAccountsChanged = __bind(this._onAccountsChanged, this);
      this._onRankingsChanged = __bind(this._onRankingsChanged, this);
      this._registerObservables = __bind(this._registerObservables, this);
      this._values = {};
      this._valuesAllAccounts = null;
      this._disposables = {};
      this.listenTo(AccountStore, this._onAccountsChanged);
      this._registerObservables(AccountStore.accounts());
    }

    ContactRankingStore.prototype._registerObservables = function(accounts) {
      var acct, callback, disposable, key, nextDisposables, query, _i, _len, _ref;
      nextDisposables = {};
      for (_i = 0, _len = accounts.length; _i < _len; _i++) {
        acct = accounts[_i];
        if (this._disposables[acct.id]) {
          nextDisposables[acct.id] = this._disposables[acct.id];
          delete this._disposables[acct.id];
        } else {
          query = DatabaseStore.findJSONBlob("ContactRankingsFor" + acct.id);
          callback = this._onRankingsChanged.bind(this, acct.id);
          nextDisposables[acct.id] = Rx.Observable.fromQuery(query).subscribe(callback);
        }
      }
      _ref = this._disposables;
      for (key in _ref) {
        disposable = _ref[key];
        disposable.dispose();
      }
      return this._disposables = nextDisposables;
    };

    ContactRankingStore.prototype._onRankingsChanged = function(accountId, json) {
      this._values[accountId] = json ? json.value : null;
      this._valuesAllAccounts = null;
      return this.trigger();
    };

    ContactRankingStore.prototype._onAccountsChanged = function() {
      return this._registerObservables(AccountStore.accounts());
    };

    ContactRankingStore.prototype.valueFor = function(accountId) {
      return this._values[accountId];
    };

    ContactRankingStore.prototype.valuesForAllAccounts = function() {
      var acctId, combined, email, score, values, _ref;
      if (!this._valuesAllAccounts) {
        combined = {};
        _ref = this._values;
        for (acctId in _ref) {
          values = _ref[acctId];
          for (email in values) {
            score = values[email];
            if (combined[email]) {
              combined[email] = Math.max(combined[email], score);
            } else {
              combined[email] = score;
            }
          }
        }
        this._valuesAllAccounts = combined;
      }
      return this._valuesAllAccounts;
    };

    ContactRankingStore.prototype.reset = function() {
      this._valuesAllAccounts = null;
      return this._values = {};
    };

    return ContactRankingStore;

  })(NylasStore);

  module.exports = new ContactRankingStore();

}).call(this);
