(function() {
  var AccountStore, Actions, Contact, ContactRankingStore, ContactStore, DatabaseStore, NylasStore, Reflux, RegExpUtils, Rx, Utils, WindowBridge, fs, path, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  fs = require('fs');

  path = require('path');

  Reflux = require('reflux');

  Rx = require('rx-lite');

  Actions = require('../actions');

  Contact = require('../models/contact');

  Utils = require('../models/utils');

  NylasStore = require('nylas-store');

  RegExpUtils = require('../../regexp-utils');

  DatabaseStore = require('./database-store');

  AccountStore = require('./account-store');

  ContactRankingStore = require('./contact-ranking-store');

  _ = require('underscore');

  WindowBridge = require('../../window-bridge');


  /*
  Public: ContactStore provides convenience methods for searching contacts and
  formatting contacts. When Contacts become editable, this store will be expanded
  with additional actions.
  
  Section: Stores
   */

  ContactStore = (function(_super) {
    __extends(ContactStore, _super);

    function ContactStore() {
      this._resetCache = __bind(this._resetCache, this);
      this._distinctByEmail = __bind(this._distinctByEmail, this);
      this._updateRankedContactCache = __bind(this._updateRankedContactCache, this);
      this.parseContactsInString = __bind(this.parseContactsInString, this);
      this.isValidContact = __bind(this.isValidContact, this);
      this.searchContacts = __bind(this.searchContacts, this);
      this._rankedContacts = [];
      this.listenTo(ContactRankingStore, (function(_this) {
        return function() {
          return _this._updateRankedContactCache();
        };
      })(this));
      this._updateRankedContactCache();
    }

    ContactStore.prototype.searchContacts = function(search, options) {
      var accountCount, contact, limit, query, results, _i, _len, _ref;
      if (options == null) {
        options = {};
      }
      limit = options.limit;
      if (limit == null) {
        limit = 5;
      }
      limit = Math.max(limit, 0);
      search = search.toLowerCase();
      accountCount = AccountStore.accounts().length;
      if (!search || search.length === 0) {
        return Promise.resolve([]);
      }
      results = [];
      _ref = this._rankedContacts;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        contact = _ref[_i];
        if (contact.email.toLowerCase().indexOf(search) !== -1 || contact.name.toLowerCase().indexOf(search) !== -1) {
          results.push(contact);
        }
        if (results.length === limit) {
          return Promise.resolve(results);
        }
      }
      query = DatabaseStore.findAll(Contact).whereAny([Contact.attributes.name.like(search), Contact.attributes.email.like(search)]);
      query.limit(limit * accountCount);
      return query.then((function(_this) {
        return function(queryResults) {
          var existingEmails;
          existingEmails = _.pluck(results, 'email');
          queryResults = _.reject(queryResults, function(c) {
            var _ref1;
            return _ref1 = c.email, __indexOf.call(existingEmails, _ref1) >= 0;
          });
          queryResults = _this._distinctByEmail(queryResults);
          results = results.concat(queryResults);
          if (results.length > limit) {
            results.length = limit;
          }
          return Promise.resolve(results);
        };
      })(this));
    };

    ContactStore.prototype.isValidContact = function(contact) {
      if (!(contact instanceof Contact)) {
        return false;
      }
      return contact.isValid();
    };

    ContactStore.prototype.parseContactsInString = function(contactString, options) {
      var char, detected, email, emailRegex, firstChar, hasLeadingParen, hasTrailingParen, hasTrailingQuote, i, lastChar, lastMatchEnd, match, name, nameStart, skipNameLookup, startsWithQuote, _i, _len, _ref, _ref1, _ref2, _ref3, _ref4;
      if (options == null) {
        options = {};
      }
      skipNameLookup = options.skipNameLookup;
      detected = [];
      emailRegex = RegExpUtils.emailRegex();
      lastMatchEnd = 0;
      while ((match = emailRegex.exec(contactString))) {
        email = match[0];
        name = null;
        startsWithQuote = (_ref = email[0]) === '\'' || _ref === '"';
        hasTrailingQuote = (_ref1 = contactString[match.index + email.length]) === '\'' || _ref1 === '"';
        if (startsWithQuote && hasTrailingQuote) {
          email = email.slice(1);
        }
        hasLeadingParen = (_ref2 = contactString[match.index - 1]) === '(' || _ref2 === '<';
        hasTrailingParen = (_ref3 = contactString[match.index + email.length]) === ')' || _ref3 === '>';
        if (hasLeadingParen && hasTrailingParen) {
          nameStart = lastMatchEnd;
          _ref4 = [',', '\n', '\r'];
          for (_i = 0, _len = _ref4.length; _i < _len; _i++) {
            char = _ref4[_i];
            i = contactString.lastIndexOf(char, match.index);
            if (i + 1 > nameStart) {
              nameStart = i + 1;
            }
          }
          name = contactString.substr(nameStart, match.index - 1 - nameStart).trim();
        }
        lastMatchEnd = match.index + email.length;
        if (hasTrailingParen) {
          lastMatchEnd += 1;
        }
        if (!name || name.length === 0) {
          name = email;
        }
        firstChar = name[0], lastChar = name[name.length - 1];
        if ((firstChar === '"' || firstChar === "'") && (lastChar === '"' || lastChar === "'")) {
          name = name.slice(1, -1);
        }
        detected.push(new Contact({
          email: email,
          name: name
        }));
      }
      if (skipNameLookup) {
        return Promise.resolve(detected);
      }
      return Promise.all(detected.map((function(_this) {
        return function(contact) {
          if (contact.name !== contact.email) {
            return contact;
          }
          return _this.searchContacts(contact.email, {
            limit: 1
          }).then(function(_arg) {
            var match;
            match = _arg[0];
            if (match && match.email === contact.email) {
              return match;
            }
            return contact;
          });
        };
      })(this)));
    };

    ContactStore.prototype._updateRankedContactCache = function() {
      var emails, rankings;
      rankings = ContactRankingStore.valuesForAllAccounts();
      emails = Object.keys(rankings);
      if (emails.length === 0) {
        this._rankedContacts = [];
        return;
      }
      emails = _.sortBy(emails, function(email) {
        var _ref;
        return -((_ref = rankings[email.toLowerCase()]) != null ? _ref : 0) / 1;
      });
      if (emails.length > 400) {
        emails.length = 400;
      }
      return DatabaseStore.findAll(Contact, {
        email: emails
      }).then((function(_this) {
        return function(contacts) {
          var contact, _i, _len, _ref;
          contacts = _this._distinctByEmail(contacts);
          for (_i = 0, _len = contacts.length; _i < _len; _i++) {
            contact = contacts[_i];
            contact._rank = -((_ref = rankings[contact.email.toLowerCase()]) != null ? _ref : 0) / 1;
          }
          return _this._rankedContacts = _.sortBy(contacts, function(contact) {
            return contact._rank;
          });
        };
      })(this));
    };

    ContactStore.prototype._distinctByEmail = function(contacts) {
      var contact, existing, key, uniq, _i, _len;
      uniq = {};
      for (_i = 0, _len = contacts.length; _i < _len; _i++) {
        contact = contacts[_i];
        if (!contact.email) {
          continue;
        }
        key = contact.email.toLowerCase();
        existing = uniq[key];
        if (!existing || (!existing.name || existing.name === existing.email)) {
          uniq[key] = contact;
        }
      }
      return _.values(uniq);
    };

    ContactStore.prototype._resetCache = function() {
      this._rankedContacts = [];
      ContactRankingStore.reset();
      return this.trigger(this);
    };

    return ContactStore;

  })(NylasStore);

  module.exports = new ContactStore();

}).call(this);
