(function() {
  var DatabaseChangeRecord, Utils,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  Utils = require('../models/utils');


  /*
  DatabaseChangeRecord is the object emitted from the DatabaseStore when it triggers.
  The DatabaseChangeRecord contains information about what type of model changed,
  and references to the new model values. All mutations to the database produce these
  change records.
   */

  DatabaseChangeRecord = (function() {
    function DatabaseChangeRecord(options) {
      this.toJSON = __bind(this.toJSON, this);
      this.options = options;
      this._objects = options.objects;
      this._objectsString = options.objectsString;
      Object.defineProperty(this, 'type', {
        get: function() {
          return options.type;
        }
      });
      Object.defineProperty(this, 'objectClass', {
        get: function() {
          return options.objectClass;
        }
      });
      Object.defineProperty(this, 'objects', {
        get: function() {
          if (this._objects == null) {
            this._objects = JSON.parse(this._objectsString, Utils.registeredObjectReviver);
          }
          return this._objects;
        }
      });
    }

    DatabaseChangeRecord.prototype.toJSON = function() {
      if (this._objectsString == null) {
        this._objectsString = JSON.stringify(this._objects, Utils.registeredObjectReplacer);
      }
      return {
        type: this.type,
        objectClass: this.objectClass,
        objectsString: this._objectsString
      };
    };

    return DatabaseChangeRecord;

  })();

  module.exports = DatabaseChangeRecord;

}).call(this);
