(function() {
  var AttributeCollection, AttributeJoinedData, DatabaseObjectRegistry, DatabaseSetupQueryBuilder, tableNameForJoin, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require('underscore');

  _ref = require('../attributes'), AttributeCollection = _ref.AttributeCollection, AttributeJoinedData = _ref.AttributeJoinedData;

  DatabaseObjectRegistry = require('../../database-object-registry')["default"];

  tableNameForJoin = require('../models/utils').tableNameForJoin;

  DatabaseSetupQueryBuilder = (function() {
    function DatabaseSetupQueryBuilder() {
      this.setupQueriesForTable = __bind(this.setupQueriesForTable, this);
    }

    DatabaseSetupQueryBuilder.prototype.setupQueries = function() {
      var klass, queries, _i, _len, _ref1;
      queries = [];
      _ref1 = DatabaseObjectRegistry.getAllConstructors();
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        klass = _ref1[_i];
        queries = queries.concat(this.setupQueriesForTable(klass));
      }
      return queries;
    };

    DatabaseSetupQueryBuilder.prototype.analyzeQueries = function() {
      var attributes, collectionAttributes, klass, queries, _i, _len, _ref1;
      queries = [];
      _ref1 = DatabaseObjectRegistry.getAllConstructors();
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        klass = _ref1[_i];
        attributes = _.values(klass.attributes);
        collectionAttributes = _.filter(attributes, function(attr) {
          return attr.queryable && attr instanceof AttributeCollection;
        });
        queries.push("ANALYZE `" + klass.name + "`");
        collectionAttributes.forEach(function(attribute) {
          return queries.push("ANALYZE `" + (tableNameForJoin(klass, attribute.itemClass)) + "`");
        });
      }
      return queries;
    };

    DatabaseSetupQueryBuilder.prototype.setupQueriesForTable = function(klass) {
      var DatabaseStore, attributes, collectionAttributes, columnAttributes, columns, columnsSQL, joinedDataAttributes, queries, _ref1;
      attributes = _.values(klass.attributes);
      queries = [];
      columnAttributes = _.filter(attributes, function(attr) {
        return attr.queryable && attr.columnSQL && attr.jsonKey !== 'id';
      });
      columns = ['id TEXT PRIMARY KEY', 'data BLOB'];
      columnAttributes.forEach(function(attr) {
        return columns.push(attr.columnSQL());
      });
      columnsSQL = columns.join(',');
      queries.unshift("CREATE TABLE IF NOT EXISTS `" + klass.name + "` (" + columnsSQL + ")");
      queries.push("CREATE UNIQUE INDEX IF NOT EXISTS `" + klass.name + "_id` ON `" + klass.name + "` (`id`)");
      collectionAttributes = _.filter(attributes, function(attr) {
        return attr.queryable && attr instanceof AttributeCollection;
      });
      collectionAttributes.forEach(function(attribute) {
        var joinColumns, joinTable;
        joinTable = tableNameForJoin(klass, attribute.itemClass);
        joinColumns = attribute.joinQueryableBy.map(function(name) {
          return klass.attributes[name].columnSQL();
        });
        joinColumns.unshift('id TEXT KEY', '`value` TEXT');
        queries.push("CREATE TABLE IF NOT EXISTS `" + joinTable + "` (" + (joinColumns.join(',')) + ")");
        queries.push("CREATE INDEX IF NOT EXISTS `" + (joinTable.replace('-', '_')) + "_id` ON `" + joinTable + "` (`id` ASC)");
        return queries.push("CREATE UNIQUE INDEX IF NOT EXISTS `" + (joinTable.replace('-', '_')) + "_val_id` ON `" + joinTable + "` (`value` ASC, `id` ASC)");
      });
      joinedDataAttributes = _.filter(attributes, function(attr) {
        return attr instanceof AttributeJoinedData;
      });
      joinedDataAttributes.forEach(function(attribute) {
        return queries.push("CREATE TABLE IF NOT EXISTS `" + attribute.modelTable + "` (id TEXT PRIMARY KEY, `value` TEXT)");
      });
      if (((_ref1 = klass.additionalSQLiteConfig) != null ? _ref1.setup : void 0) != null) {
        queries = queries.concat(klass.additionalSQLiteConfig.setup());
      }
      if (klass.searchable === true) {
        DatabaseStore = require('./database-store');
        queries.push(DatabaseStore.createSearchIndexSql(klass));
      }
      return queries;
    };

    return DatabaseSetupQueryBuilder;

  })();

  module.exports = DatabaseSetupQueryBuilder;

}).call(this);
