(function() {
  var Actions, BEGIN_TRANSACTION, COMMIT, DEBUG_QUERY_PLANS, DEBUG_TO_LOG, DatabaseChangeRecord, DatabasePhase, DatabaseSetupQueryBuilder, DatabaseStore, DatabaseTransaction, DatabaseVersion, JSONBlob, JSONBlobQuery, Model, ModelQuery, NylasStore, PriorityUICoordinator, PromiseQueue, TXINDEX, Utils, async, fs, ipcRenderer, path, remote, sqlite3, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __slice = [].slice;

  _ = require('underscore');

  async = require('async');

  path = require('path');

  fs = require('fs');

  sqlite3 = require('sqlite3');

  Model = require('../models/model');

  Utils = require('../models/utils');

  Actions = require('../actions');

  ModelQuery = require('../models/query');

  NylasStore = require('../../global/nylas-store');

  PromiseQueue = require('promise-queue');

  PriorityUICoordinator = require('../../priority-ui-coordinator');

  DatabaseSetupQueryBuilder = require('./database-setup-query-builder');

  DatabaseChangeRecord = require('./database-change-record');

  DatabaseTransaction = require('./database-transaction');

  JSONBlob = null;

  _ref = require('electron'), remote = _ref.remote, ipcRenderer = _ref.ipcRenderer;

  DatabaseVersion = 23;

  DatabasePhase = {
    Setup: 'setup',
    Ready: 'ready',
    Close: 'close'
  };

  DEBUG_TO_LOG = false;

  DEBUG_QUERY_PLANS = NylasEnv.inDevMode();

  BEGIN_TRANSACTION = 'BEGIN TRANSACTION';

  COMMIT = 'COMMIT';

  TXINDEX = 0;

  JSONBlobQuery = (function(_super) {
    __extends(JSONBlobQuery, _super);

    function JSONBlobQuery() {
      this.formatResult = __bind(this.formatResult, this);
      return JSONBlobQuery.__super__.constructor.apply(this, arguments);
    }

    JSONBlobQuery.prototype.formatResult = function(objects) {
      var _ref1;
      return ((_ref1 = objects[0]) != null ? _ref1.json : void 0) || null;
    };

    return JSONBlobQuery;

  })(ModelQuery);


  /*
  Public: N1 is built on top of a custom database layer modeled after
  ActiveRecord. For many parts of the application, the database is the source
  of truth. Data is retrieved from the API, written to the database, and changes
  to the database trigger Stores and components to refresh their contents.
  
  The DatabaseStore is available in every application window and allows you to
  make queries against the local cache. Every change to the local cache is
  broadcast as a change event, and listening to the DatabaseStore keeps the
  rest of the application in sync.
  
   *# Listening for Changes
  
  To listen for changes to the local cache, subscribe to the DatabaseStore and
  inspect the changes that are sent to your listener method.
  
  ```coffeescript
  @unsubscribe = DatabaseStore.listen(@_onDataChanged, @)
  
  ...
  
  _onDataChanged: (change) ->
    return unless change.objectClass is Message
    return unless @_myMessageID in _.map change.objects, (m) -> m.id
  
     * Refresh Data
  
  ```
  
  
  The local cache changes very frequently, and your stores and components should
  carefully choose when to refresh their data. The `change` object passed to your
  event handler allows you to decide whether to refresh your data and exposes
  the following keys:
  
  `objectClass`: The {Model} class that has been changed. If multiple types of models
  were saved to the database, you will receive multiple change events.
  
  `objects`: An {Array} of {Model} instances that were either created, updated or
  deleted from the local cache. If your component or store presents a single object
  or a small collection of objects, you should look to see if any of the objects
  are in your displayed set before refreshing.
  
  Section: Database
   */

  DatabaseStore = (function(_super) {
    __extends(DatabaseStore, _super);

    function DatabaseStore() {
      this.unindexModelsForAccount = __bind(this.unindexModelsForAccount, this);
      this.unindexModel = __bind(this.unindexModel, this);
      this.updateModelIndex = __bind(this.updateModelIndex, this);
      this.indexModel = __bind(this.indexModel, this);
      this.isModelIndexed = __bind(this.isModelIndexed, this);
      this.dropSearchIndex = __bind(this.dropSearchIndex, this);
      this.isIndexEmptyForAccount = __bind(this.isIndexEmptyForAccount, this);
      this.searchIndexSize = __bind(this.searchIndexSize, this);
      this.createSearchIndex = __bind(this.createSearchIndex, this);
      this.createSearchIndexSql = __bind(this.createSearchIndexSql, this);
      this.accumulateAndTrigger = __bind(this.accumulateAndTrigger, this);
      this.run = __bind(this.run, this);
      this.modelify = __bind(this.modelify, this);
      this.count = __bind(this.count, this);
      this.findAll = __bind(this.findAll, this);
      this.findBy = __bind(this.findBy, this);
      this.find = __bind(this.find, this);
      this._query = __bind(this._query, this);
      this._prettyConsoleLog = __bind(this._prettyConsoleLog, this);
      this._handleSetupError = __bind(this._handleSetupError, this);
      this._runDatabaseAnalyze = __bind(this._runDatabaseAnalyze, this);
      this._runDatabaseSetup = __bind(this._runDatabaseSetup, this);
      this._checkDatabaseVersion = __bind(this._checkDatabaseVersion, this);
      this._openDatabase = __bind(this._openDatabase, this);
      this._onPhaseChange = __bind(this._onPhaseChange, this);
      this._triggerPromise = null;
      this._inflightTransactions = 0;
      this._open = false;
      this._waiting = [];
      this.setupEmitter();
      this._emitter.setMaxListeners(100);
      if (NylasEnv.inSpecMode()) {
        this._databasePath = path.join(NylasEnv.getConfigDirPath(), 'edgehill.test.db');
      } else {
        this._databasePath = path.join(NylasEnv.getConfigDirPath(), 'edgehill.db');
      }
      this._databaseMutationHooks = [];
      ipcRenderer.on('database-phase-change', this._onPhaseChange);
      _.defer((function(_this) {
        return function() {
          return _this._onPhaseChange();
        };
      })(this));
    }

    DatabaseStore.prototype._onPhaseChange = function(event) {
      var app, phase, _ref1;
      if (NylasEnv.inSpecMode()) {
        return;
      }
      app = remote.getGlobal('application');
      phase = app.databasePhase();
      if (phase === DatabasePhase.Setup && NylasEnv.isWorkWindow()) {
        return this._openDatabase((function(_this) {
          return function() {
            return _this._checkDatabaseVersion({
              allowNotSet: true
            }, function() {
              return _this._runDatabaseSetup(function() {
                app.setDatabasePhase(DatabasePhase.Ready);
                return setTimeout(_this._runDatabaseAnalyze, 60 * 1000);
              });
            });
          };
        })(this));
      } else if (phase === DatabasePhase.Ready) {
        return this._openDatabase((function(_this) {
          return function() {
            return _this._checkDatabaseVersion({}, function() {
              var w, _i, _len, _ref1;
              _this._open = true;
              _ref1 = _this._waiting;
              for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
                w = _ref1[_i];
                w();
              }
              return _this._waiting = [];
            });
          };
        })(this));
      } else if (phase === DatabasePhase.Close) {
        this._open = false;
        if ((_ref1 = this._db) != null) {
          _ref1.close();
        }
        return this._db = null;
      }
    };

    DatabaseStore.prototype.refreshDatabaseSchema = function() {
      var app, phase;
      if (!NylasEnv.isWorkWindow()) {
        return;
      }
      app = remote.getGlobal('application');
      phase = app.databasePhase();
      if (phase !== DatabasePhase.Setup) {
        return app.setDatabasePhase(DatabasePhase.Setup);
      }
    };

    DatabaseStore.prototype._openDatabase = function(ready) {
      var mode;
      if (this._db) {
        return ready();
      }
      if (NylasEnv.isWorkWindow()) {
        mode = sqlite3.OPEN_READWRITE | sqlite3.OPEN_CREATE;
      } else {
        mode = sqlite3.OPEN_READWRITE;
      }
      return this._db = new sqlite3.Database(this._databasePath, mode, (function(_this) {
        return function(err) {
          if (err) {
            return _this._handleSetupError(err);
          }
          _this._db.run("PRAGMA journal_mode = WAL;");
          _this._db.run("PRAGMA main.page_size = 8192;");
          _this._db.run("PRAGMA main.cache_size = 20000;");
          _this._db.run("PRAGMA main.synchronous = NORMAL;");
          _this._db.configure('busyTimeout', 10000);
          _this._db.on('profile', function(query, msec) {
            if (msec > 100) {
              return _this._prettyConsoleLog("" + msec + "msec: " + query);
            } else {
              return console.debug(DEBUG_TO_LOG, "" + msec + ": " + query);
            }
          });
          return ready();
        };
      })(this));
    };

    DatabaseStore.prototype._checkDatabaseVersion = function(_arg, ready) {
      var allowNotSet;
      allowNotSet = (_arg != null ? _arg : {}).allowNotSet;
      return this._db.get('PRAGMA user_version', (function(_this) {
        return function(err, _arg1) {
          var emptyVersion, user_version, wrongVersion;
          user_version = _arg1.user_version;
          if (err) {
            return _this._handleSetupError(err);
          }
          emptyVersion = user_version === 0;
          wrongVersion = user_version / 1 !== DatabaseVersion;
          if (wrongVersion && !(emptyVersion && allowNotSet)) {
            return _this._handleSetupError(new Error("Incorrect database schema version: " + user_version + " not " + DatabaseVersion));
          }
          return ready();
        };
      })(this));
    };

    DatabaseStore.prototype._runDatabaseSetup = function(ready) {
      var builder;
      builder = new DatabaseSetupQueryBuilder();
      return this._db.serialize((function(_this) {
        return function() {
          return async.each(builder.setupQueries(), function(query, callback) {
            console.debug(DEBUG_TO_LOG, "DatabaseStore: " + query);
            return _this._db.run(query, [], callback);
          }, function(err) {
            if (err) {
              return _this._handleSetupError(err);
            }
            return _this._db.run("PRAGMA user_version=" + DatabaseVersion, function(err) {
              var exportPath, row;
              if (err) {
                return _this._handleSetupError(err);
              }
              exportPath = path.join(NylasEnv.getConfigDirPath(), 'mail-rules-export.json');
              if (fs.existsSync(exportPath)) {
                try {
                  row = JSON.parse(fs.readFileSync(exportPath));
                  _this.inTransaction(function(t) {
                    return t.persistJSONBlob('MailRules-V2', row['json']);
                  });
                  fs.unlink(exportPath);
                } catch (_error) {
                  err = _error;
                  console.log("Could not re-import mail rules: " + err);
                }
              }
              return ready();
            });
          });
        };
      })(this));
    };

    DatabaseStore.prototype._runDatabaseAnalyze = function() {
      var builder;
      builder = new DatabaseSetupQueryBuilder();
      return async.each(builder.analyzeQueries(), (function(_this) {
        return function(query, callback) {
          return _this._db.run(query, [], callback);
        };
      })(this), (function(_this) {
        return function(err) {
          return console.log("Completed ANALYZE of database");
        };
      })(this));
    };

    DatabaseStore.prototype._handleSetupError = function(err) {
      if (err == null) {
        err = new Error("Manually called _handleSetupError");
      }
      NylasEnv.reportError(err, {}, {
        noWindows: true
      });
      return this._db.all("SELECT * FROM JSONBlob WHERE id = 'MailRules-V2' LIMIT 1", [], (function(_this) {
        return function(mailsRulesErr, results) {
          var app, exportPath, writeErr;
          if (results == null) {
            results = [];
          }
          if (!mailsRulesErr && results.length === 1) {
            exportPath = path.join(NylasEnv.getConfigDirPath(), 'mail-rules-export.json');
            try {
              fs.writeFileSync(exportPath, results[0]['data']);
            } catch (_error) {
              writeErr = _error;
              console.log("Could not write mail rules to file: " + writeErr);
            }
          }
          app = require('electron').remote.getGlobal('application');
          return app.rebuildDatabase();
        };
      })(this));
    };

    DatabaseStore.prototype._prettyConsoleLog = function(q) {
      var colorRules, colors, i, keyword, keywords, msg, style, _i, _j, _len, _ref1;
      q = "color:black |||%c " + q;
      q = q.replace(/`(\w+)`/g, "||| color:purple |||%c$&||| color:black |||%c");
      colorRules = {
        'color:green': ['SELECT', 'INSERT INTO', 'VALUES', 'WHERE', 'FROM', 'JOIN', 'ORDER BY', 'DESC', 'ASC', 'INNER', 'OUTER', 'LIMIT', 'OFFSET', 'IN'],
        'color:red; background-color:#ffdddd;': ['SCAN TABLE']
      };
      for (style in colorRules) {
        keywords = colorRules[style];
        for (_i = 0, _len = keywords.length; _i < _len; _i++) {
          keyword = keywords[_i];
          q = q.replace(new RegExp("\\b" + keyword + "\\b", 'g'), "||| " + style + " |||%c" + keyword + "||| color:black |||%c");
        }
      }
      q = q.split('|||');
      colors = [];
      msg = [];
      for (i = _j = 0, _ref1 = q.length; 0 <= _ref1 ? _j < _ref1 : _j > _ref1; i = 0 <= _ref1 ? ++_j : --_j) {
        if (i % 2 === 0) {
          colors.push(q[i]);
        } else {
          msg.push(q[i]);
        }
      }
      return console.log.apply(console, [msg.join('')].concat(__slice.call(colors)));
    };

    DatabaseStore.prototype._query = function(query, values) {
      if (values == null) {
        values = [];
      }
      return new Promise((function(_this) {
        return function(resolve, reject) {
          var fn;
          if (!_this._open) {
            _this._waiting.push(function() {
              return _this._query(query, values).then(resolve, reject);
            });
            return;
          }
          if (query.indexOf("SELECT ") === 0) {
            fn = 'all';
          } else {
            fn = 'run';
          }
          if (query.indexOf("SELECT ") === 0) {
            if (DEBUG_QUERY_PLANS) {
              _this._db.all("EXPLAIN QUERY PLAN " + query, values, function(err, results) {
                var str;
                if (results == null) {
                  results = [];
                }
                str = results.map(function(row) {
                  return row.detail;
                }).join('\n') + " for " + query;
                if (str.indexOf('ThreadCounts') > 0) {
                  return;
                }
                if (str.indexOf('ThreadSearch') > 0) {
                  return;
                }
                if (str.indexOf('SCAN') !== -1 && str.indexOf('COVERING INDEX') === -1) {
                  return _this._prettyConsoleLog(str);
                }
              });
            }
          }
          if (query.indexOf("BEGIN" === 0)) {
            if (_this._inflightTransactions === 0) {
              _this._db.serialize();
            }
            _this._inflightTransactions += 1;
          }
          return _this._db[fn](query, values, function(err, results) {
            if (err) {
              console.error("DatabaseStore: Query " + query + ", " + (JSON.stringify(values)) + " failed " + (err.toString()));
            }
            if (query === COMMIT) {
              _this._inflightTransactions -= 1;
              if (_this._inflightTransactions === 0) {
                _this._db.parallelize();
              }
            }
            if (err) {
              return reject(err);
            }
            return resolve(results);
          });
        };
      })(this));
    };


    /*
    ActiveRecord-style Querying
     */

    DatabaseStore.prototype.find = function(klass, id) {
      if (!klass) {
        throw new Error("DatabaseStore::find - You must provide a class");
      }
      if (!_.isString(id)) {
        throw new Error("DatabaseStore::find - You must provide a string id. You may have intended to use findBy.");
      }
      return new ModelQuery(klass, this).where({
        id: id
      }).one();
    };

    DatabaseStore.prototype.findBy = function(klass, predicates) {
      if (predicates == null) {
        predicates = [];
      }
      if (!klass) {
        throw new Error("DatabaseStore::findBy - You must provide a class");
      }
      return new ModelQuery(klass, this).where(predicates).one();
    };

    DatabaseStore.prototype.findAll = function(klass, predicates) {
      if (predicates == null) {
        predicates = [];
      }
      if (!klass) {
        throw new Error("DatabaseStore::findAll - You must provide a class");
      }
      return new ModelQuery(klass, this).where(predicates);
    };

    DatabaseStore.prototype.count = function(klass, predicates) {
      if (predicates == null) {
        predicates = [];
      }
      if (!klass) {
        throw new Error("DatabaseStore::count - You must provide a class");
      }
      return new ModelQuery(klass, this).where(predicates).count();
    };

    DatabaseStore.prototype.modelify = function(klass, arr) {
      var clientIds, ids, item, queries, _i, _len;
      if (!_.isArray(arr) || arr.length === 0) {
        return Promise.resolve([]);
      }
      ids = [];
      clientIds = [];
      for (_i = 0, _len = arr.length; _i < _len; _i++) {
        item = arr[_i];
        if (item instanceof klass) {
          if (!item.serverId) {
            clientIds.push(item.clientId);
          } else {
            continue;
          }
        } else if (_.isString(item)) {
          if (Utils.isTempId(item)) {
            clientIds.push(item);
          } else {
            ids.push(item);
          }
        } else {
          throw new Error("modelify: Not sure how to convert " + item + " into a " + klass.name);
        }
      }
      if (ids.length === 0 && clientIds.length === 0) {
        return Promise.resolve(arr);
      }
      queries = {
        modelsFromIds: [],
        modelsFromClientIds: []
      };
      if (ids.length) {
        queries.modelsFromIds = this.findAll(klass).where(klass.attributes.id["in"](ids));
      }
      if (clientIds.length) {
        queries.modelsFromClientIds = this.findAll(klass).where(klass.attributes.clientId["in"](clientIds));
      }
      return Promise.props(queries).then((function(_this) {
        return function(_arg) {
          var model, modelsByString, modelsFromClientIds, modelsFromIds, _j, _k, _len1, _len2;
          modelsFromIds = _arg.modelsFromIds, modelsFromClientIds = _arg.modelsFromClientIds;
          modelsByString = {};
          for (_j = 0, _len1 = modelsFromIds.length; _j < _len1; _j++) {
            model = modelsFromIds[_j];
            modelsByString[model.id] = model;
          }
          for (_k = 0, _len2 = modelsFromClientIds.length; _k < _len2; _k++) {
            model = modelsFromClientIds[_k];
            modelsByString[model.clientId] = model;
          }
          arr = arr.map(function(item) {
            if (item instanceof klass) {
              return item;
            } else {
              return modelsByString[item];
            }
          });
          return Promise.resolve(arr);
        };
      })(this));
    };

    DatabaseStore.prototype.run = function(modelQuery, options) {
      if (options == null) {
        options = {
          format: true
        };
      }
      return this._query(modelQuery.sql(), []).then((function(_this) {
        return function(result) {
          result = modelQuery.inflateResult(result);
          if (options.format !== false) {
            result = modelQuery.formatResult(result);
          }
          return Promise.resolve(result);
        };
      })(this));
    };

    DatabaseStore.prototype.findJSONBlob = function(id) {
      if (JSONBlob == null) {
        JSONBlob = require('../models/json-blob');
      }
      return new JSONBlobQuery(JSONBlob, this).where({
        id: id
      }).one();
    };

    DatabaseStore.prototype.addMutationHook = function(_arg) {
      var afterDatabaseChange, beforeDatabaseChange;
      beforeDatabaseChange = _arg.beforeDatabaseChange, afterDatabaseChange = _arg.afterDatabaseChange;
      if (!beforeDatabaseChange) {
        throw new Error("DatabaseStore:addMutationHook - You must provide a beforeDatabaseChange function");
      }
      if (!afterDatabaseChange) {
        throw new Error("DatabaseStore:addMutationHook - You must provide a afterDatabaseChange function");
      }
      return this._databaseMutationHooks.push({
        beforeDatabaseChange: beforeDatabaseChange,
        afterDatabaseChange: afterDatabaseChange
      });
    };

    DatabaseStore.prototype.removeMutationHook = function(hook) {
      return this._databaseMutationHooks = _.without(this._databaseMutationHooks, hook);
    };

    DatabaseStore.prototype.mutationHooks = function() {
      return this._databaseMutationHooks;
    };

    DatabaseStore.prototype.inTransaction = function(fn) {
      var t;
      t = new DatabaseTransaction(this);
      if (this._transactionQueue == null) {
        this._transactionQueue = new PromiseQueue(1, Infinity);
      }
      return this._transactionQueue.add(function() {
        return t.execute(fn);
      });
    };

    DatabaseStore.prototype.accumulateAndTrigger = function(change) {
      var concat, flush, set;
      if (this._triggerPromise == null) {
        this._triggerPromise = new Promise((function(_this) {
          return function(resolve, reject) {
            return _this._resolve = resolve;
          };
        })(this));
      }
      flush = (function(_this) {
        return function() {
          if (!_this._changeAccumulated) {
            return;
          }
          if (_this._changeFireTimer) {
            clearTimeout(_this._changeFireTimer);
          }
          _this.trigger(new DatabaseChangeRecord(_this._changeAccumulated));
          _this._changeAccumulated = null;
          _this._changeAccumulatedLookup = null;
          _this._changeFireTimer = null;
          if (typeof _this._resolve === "function") {
            _this._resolve();
          }
          return _this._triggerPromise = null;
        };
      })(this);
      set = (function(_this) {
        return function(change) {
          var idx, obj, _i, _len, _ref1;
          if (_this._changeFireTimer) {
            clearTimeout(_this._changeFireTimer);
          }
          _this._changeAccumulated = change;
          _this._changeAccumulatedLookup = {};
          _ref1 = _this._changeAccumulated.objects;
          for (idx = _i = 0, _len = _ref1.length; _i < _len; idx = ++_i) {
            obj = _ref1[idx];
            _this._changeAccumulatedLookup[obj.id] = idx;
          }
          return _this._changeFireTimer = setTimeout(flush, 10);
        };
      })(this);
      concat = (function(_this) {
        return function(change) {
          var idx, obj, _i, _len, _ref1, _results;
          _ref1 = change.objects;
          _results = [];
          for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
            obj = _ref1[_i];
            idx = _this._changeAccumulatedLookup[obj.id];
            if (idx) {
              _results.push(_this._changeAccumulated.objects[idx] = obj);
            } else {
              _this._changeAccumulatedLookup[obj.id] = _this._changeAccumulated.objects.length;
              _results.push(_this._changeAccumulated.objects.push(obj));
            }
          }
          return _results;
        };
      })(this);
      if (!this._changeAccumulated) {
        set(change);
      } else if (this._changeAccumulated.objectClass === change.objectClass && this._changeAccumulated.type === change.type) {
        concat(change);
      } else {
        flush();
        set(change);
      }
      return this._triggerPromise;
    };

    DatabaseStore.prototype.createSearchIndexSql = function(klass) {
      var searchFields, searchTableName;
      if (!klass) {
        throw new Error("DatabaseStore::createSearchIndex - You must provide a class");
      }
      if (!klass) {
        throw new Error("DatabaseStore::createSearchIndex - " + klass.name + " must expose an array of `searchFields`");
      }
      searchTableName = "" + klass.name + "Search";
      searchFields = klass.searchFields;
      return ("CREATE VIRTUAL TABLE IF NOT EXISTS `" + searchTableName + "` ") + ("USING fts5( tokenize='porter unicode61', content_id UNINDEXED, " + (searchFields.join(', ')) + " )");
    };

    DatabaseStore.prototype.createSearchIndex = function(klass) {
      var sql;
      sql = this.createSearchIndexSql(klass);
      return this._query(sql);
    };

    DatabaseStore.prototype.searchIndexSize = function(klass) {
      var searchTableName, sql;
      searchTableName = "" + klass.name + "Search";
      sql = "SELECT COUNT(content_id) as count FROM `" + searchTableName + "`";
      return this._query(sql).then((function(_this) {
        return function(result) {
          return result[0].count;
        };
      })(this));
    };

    DatabaseStore.prototype.isIndexEmptyForAccount = function(accountId, modelKlass) {
      var modelTable, searchTable, sql;
      modelTable = modelKlass.name;
      searchTable = "" + modelTable + "Search";
      sql = "SELECT `" + searchTable + "`.`content_id` FROM `" + searchTable + "` INNER JOIN `" + modelTable + "` ON `" + modelTable + "`.id = `" + searchTable + "`.`content_id` WHERE `" + modelTable + "`.`account_id` = ? LIMIT 1";
      return this._query(sql, [accountId]).then((function(_this) {
        return function(result) {
          return result.length === 0;
        };
      })(this));
    };

    DatabaseStore.prototype.dropSearchIndex = function(klass) {
      var searchTableName, sql;
      if (!klass) {
        throw new Error("DatabaseStore::createSearchIndex - You must provide a class");
      }
      searchTableName = "" + klass.name + "Search";
      sql = "DROP TABLE IF EXISTS `" + searchTableName + "`";
      return this._query(sql);
    };

    DatabaseStore.prototype.isModelIndexed = function(model, isIndexed) {
      var exists, searchTableName;
      if (isIndexed === true) {
        return Promise.resolve(true);
      }
      searchTableName = "" + model.constructor.name + "Search";
      exists = "SELECT rowid FROM `" + searchTableName + "` WHERE `" + searchTableName + "`.`content_id` = ?";
      return this._query(exists, [model.id]).then((function(_this) {
        return function(results) {
          return Promise.resolve(results.length > 0);
        };
      })(this));
    };

    DatabaseStore.prototype.indexModel = function(model, indexData, isModelIndexed) {
      var searchTableName;
      searchTableName = "" + model.constructor.name + "Search";
      return this.isModelIndexed(model, isModelIndexed).then((function(_this) {
        return function(isIndexed) {
          var indexFields, keysSql, sql, valsSql, values;
          if (isIndexed) {
            return _this.updateModelIndex(model, indexData, isIndexed);
          }
          indexFields = Object.keys(indexData);
          keysSql = 'content_id, ' + indexFields.join(", ");
          valsSql = '?, ' + indexFields.map(function() {
            return '?';
          }).join(", ");
          values = [model.id].concat(indexFields.map(function(k) {
            return indexData[k];
          }));
          sql = "INSERT INTO `" + searchTableName + "`(" + keysSql + ") VALUES (" + valsSql + ")";
          return _this._query(sql, values);
        };
      })(this));
    };

    DatabaseStore.prototype.updateModelIndex = function(model, indexData, isModelIndexed) {
      var searchTableName;
      searchTableName = "" + model.constructor.name + "Search";
      return this.isModelIndexed(model, isModelIndexed).then((function(_this) {
        return function(isIndexed) {
          var indexFields, setSql, sql, values;
          if (!isIndexed) {
            return _this.indexModel(model, indexData, isIndexed);
          }
          indexFields = Object.keys(indexData);
          values = indexFields.map(function(key) {
            return indexData[key];
          }).concat([model.id]);
          setSql = indexFields.map(function(key) {
            return "`" + key + "` = ?";
          }).join(', ');
          sql = "UPDATE `" + searchTableName + "` SET " + setSql + " WHERE `" + searchTableName + "`.`content_id` = ?";
          return _this._query(sql, values);
        };
      })(this));
    };

    DatabaseStore.prototype.unindexModel = function(model) {
      var searchTableName, sql;
      searchTableName = "" + model.constructor.name + "Search";
      sql = "DELETE FROM `" + searchTableName + "` WHERE `" + searchTableName + "`.`content_id` = ?";
      return this._query(sql, [model.id]);
    };

    DatabaseStore.prototype.unindexModelsForAccount = function(accountId, modelKlass) {
      var modelTable, searchTableName, sql;
      modelTable = modelKlass.name;
      searchTableName = "" + modelTable + "Search";
      sql = "DELETE FROM `" + searchTableName + "` WHERE `" + searchTableName + "`.`content_id` IN (SELECT `id` FROM `" + modelTable + "` WHERE `" + modelTable + "`.`account_id` = ?)";
      return this._query(sql, [accountId]);
    };

    return DatabaseStore;

  })(NylasStore);

  module.exports = new DatabaseStore();

  module.exports.ChangeRecord = DatabaseChangeRecord;

}).call(this);
