(function() {
  var AttributeCollection, AttributeJoinedData, DatabaseTransaction, Model, Utils, tableNameForJoin, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __slice = [].slice;

  _ = require('underscore');

  Model = require('../models/model');

  Utils = require('../models/utils');

  _ref = require('../attributes'), AttributeCollection = _ref.AttributeCollection, AttributeJoinedData = _ref.AttributeJoinedData;

  tableNameForJoin = require('../models/utils').tableNameForJoin;

  DatabaseTransaction = (function() {
    function DatabaseTransaction(database) {
      this.database = database;
      this._deleteModel = __bind(this._deleteModel, this);
      this._writeModels = __bind(this._writeModels, this);
      this._runMutationHooks = __bind(this._runMutationHooks, this);
      this._query = __bind(this._query, this);
      this.unpersistModel = __bind(this.unpersistModel, this);
      this.persistModels = __bind(this.persistModels, this);
      this.persistModel = __bind(this.persistModel, this);
      this.execute = __bind(this.execute, this);
      this.findJSONBlob = __bind(this.findJSONBlob, this);
      this.count = __bind(this.count, this);
      this.modelify = __bind(this.modelify, this);
      this.findAll = __bind(this.findAll, this);
      this.findBy = __bind(this.findBy, this);
      this.find = __bind(this.find, this);
      this._changeRecords = [];
      this._opened = false;
    }

    DatabaseTransaction.prototype.find = function() {
      var args, _ref1;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref1 = this.database).find.apply(_ref1, args);
    };

    DatabaseTransaction.prototype.findBy = function() {
      var args, _ref1;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref1 = this.database).findBy.apply(_ref1, args);
    };

    DatabaseTransaction.prototype.findAll = function() {
      var args, _ref1;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref1 = this.database).findAll.apply(_ref1, args);
    };

    DatabaseTransaction.prototype.modelify = function() {
      var args, _ref1;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref1 = this.database).modelify.apply(_ref1, args);
    };

    DatabaseTransaction.prototype.count = function() {
      var args, _ref1;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref1 = this.database).count.apply(_ref1, args);
    };

    DatabaseTransaction.prototype.findJSONBlob = function() {
      var args, _ref1;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return (_ref1 = this.database).findJSONBlob.apply(_ref1, args);
    };

    DatabaseTransaction.prototype.execute = function(fn) {
      var start;
      if (this._opened) {
        throw new Error("DatabaseTransaction:execute was already called");
      }
      start = Date.now();
      return this._query("BEGIN IMMEDIATE TRANSACTION").then((function(_this) {
        return function() {
          _this._opened = true;
          return fn(_this);
        };
      })(this))["finally"]((function(_this) {
        return function() {
          if (_this._opened) {
            _this._opened = false;
            return _this._query("COMMIT").then(function() {
              var record, _i, _len, _ref1, _results;
              _ref1 = _this._changeRecords;
              _results = [];
              for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
                record = _ref1[_i];
                _results.push(_this.database.accumulateAndTrigger(record));
              }
              return _results;
            });
          }
        };
      })(this));
    };

    DatabaseTransaction.prototype.persistJSONBlob = function(id, json) {
      var JSONBlob;
      JSONBlob = require('../models/json-blob');
      return this.persistModel(new JSONBlob({
        id: id,
        json: json
      }));
    };

    DatabaseTransaction.prototype.persistModel = function(model) {
      if (!(model && model instanceof Model)) {
        throw new Error("DatabaseTransaction::persistModel - You must pass an instance of the Model class.");
      }
      return this.persistModels([model]);
    };

    DatabaseTransaction.prototype.persistModels = function(models, _arg) {
      var clones, ids, klass, metadata, model, _i, _len;
      if (models == null) {
        models = [];
      }
      _arg;
      if (models.length === 0) {
        return Promise.resolve();
      }
      klass = models[0].constructor;
      clones = [];
      ids = {};
      if (!(models[0] instanceof Model)) {
        throw new Error("DatabaseTransaction::persistModels - You must pass an array of items which descend from the Model class.");
      }
      for (_i = 0, _len = models.length; _i < _len; _i++) {
        model = models[_i];
        if (!(model && model.constructor === klass)) {
          throw new Error("DatabaseTransaction::persistModels - When you batch persist objects, they must be of the same type");
        }
        if (ids[model.id]) {
          throw new Error("DatabaseTransaction::persistModels - You must pass an array of models with different ids. ID " + model.id + " is in the set multiple times.");
        }
        clones.push(model.clone());
        ids[model.id] = true;
      }
      metadata = {
        objectClass: clones[0].constructor.name,
        objectIds: Object.keys(ids),
        objects: clones,
        type: 'persist'
      };
      return this._runMutationHooks('beforeDatabaseChange', metadata).then((function(_this) {
        return function(data) {
          return _this._writeModels(clones).then(function() {
            _this._runMutationHooks('afterDatabaseChange', metadata, data);
            return _this._changeRecords.push(metadata);
          });
        };
      })(this));
    };

    DatabaseTransaction.prototype.unpersistModel = function(model) {
      var metadata;
      model = model.clone();
      metadata = {
        objectClass: model.constructor.name,
        objectIds: [model.id],
        objects: [model],
        type: 'unpersist'
      };
      return this._runMutationHooks('beforeDatabaseChange', metadata).then((function(_this) {
        return function(data) {
          return _this._deleteModel(model).then(function() {
            _this._runMutationHooks('afterDatabaseChange', metadata, data);
            return _this._changeRecords.push(metadata);
          });
        };
      })(this));
    };

    DatabaseTransaction.prototype._query = function() {
      var _ref1;
      return (_ref1 = this.database)._query.apply(_ref1, arguments);
    };

    DatabaseTransaction.prototype._runMutationHooks = function(selectorName, metadata, data) {
      var beforePromises;
      if (data == null) {
        data = [];
      }
      beforePromises = this.database.mutationHooks().map((function(_this) {
        return function(hook, idx) {
          return Promise["try"](function() {
            return hook[selectorName](_this._query, metadata, data[idx]);
          });
        };
      })(this));
      return Promise.all(beforePromises)["catch"]((function(_this) {
        return function(e) {
          if (!NylasEnv.inSpecMode()) {
            console.warn("DatabaseTransaction Hook: " + selectorName + " failed", e);
          }
          return Promise.resolve([]);
        };
      })(this));
    };

    DatabaseTransaction.prototype._writeModels = function(models) {
      var attributes, collectionAttributes, columnAttributes, columns, columnsSQL, ids, joinedDataAttributes, json, klass, marks, marksSQL, marksSet, model, modelsJSONs, promises, values, _i, _len;
      promises = [];
      if (models.length > 50) {
        return Promise.all([this._writeModels(models.slice(0, 50)), this._writeModels(models.slice(50, +models.length + 1 || 9e9))]);
      }
      klass = models[0].constructor;
      attributes = _.values(klass.attributes);
      columnAttributes = _.filter(attributes, function(attr) {
        return attr.queryable && attr.columnSQL && attr.jsonKey !== 'id';
      });
      columns = ['id', 'data'];
      marks = ['?', '?'];
      columnAttributes.forEach(function(attr) {
        columns.push(attr.jsonKey);
        return marks.push('?');
      });
      columnsSQL = columns.join(',');
      marksSet = "(" + (marks.join(',')) + ")";
      values = [];
      marks = [];
      ids = [];
      modelsJSONs = [];
      for (_i = 0, _len = models.length; _i < _len; _i++) {
        model = models[_i];
        json = model.toJSON({
          joined: false
        });
        modelsJSONs.push(json);
        ids.push(model.id);
        values.push(model.id, JSON.stringify(json, Utils.registeredObjectReplacer));
        columnAttributes.forEach(function(attr) {
          return values.push(json[attr.jsonKey]);
        });
        marks.push(marksSet);
      }
      marksSQL = marks.join(',');
      promises.push(this._query("REPLACE INTO `" + klass.name + "` (" + columnsSQL + ") VALUES " + marksSQL, values));
      collectionAttributes = _.filter(attributes, function(attr) {
        return attr.queryable && attr instanceof AttributeCollection;
      });
      collectionAttributes.forEach((function(_this) {
        return function(attr) {
          var idx, joinColumns, joinMarkUnit, joinMarks, joinQueryableByJSONKeys, joinTable, joinValue, joined, joinedJsonKey, joinedModels, joinedValues, me, ms, rowsPerInsert, slice, slicePageCount, valuesPerInsert, valuesPerRow, ve, vs, _j, _k, _l, _len1, _len2, _len3, _m, _ref1, _ref2, _results;
          joinTable = tableNameForJoin(klass, attr.itemClass);
          promises.push(_this._query("DELETE FROM `" + joinTable + "` WHERE `id` IN ('" + (ids.join("','")) + "')"));
          joinMarks = [];
          joinedValues = [];
          joinMarkUnit = "(" + ["?", "?"].concat(attr.joinQueryableBy.map(function() {
            return '?';
          })).join(',') + ")";
          joinQueryableByJSONKeys = attr.joinQueryableBy.map(function(joinedModelKey) {
            return klass.attributes[joinedModelKey].jsonKey;
          });
          joinColumns = ['id', 'value'].concat(joinQueryableByJSONKeys);
          valuesPerRow = joinColumns.length;
          rowsPerInsert = Math.floor(600 / valuesPerRow);
          valuesPerInsert = rowsPerInsert * valuesPerRow;
          for (idx = _j = 0, _len1 = models.length; _j < _len1; idx = ++_j) {
            model = models[idx];
            joinedModels = model[attr.modelKey];
            if (joinedModels) {
              for (_k = 0, _len2 = joinedModels.length; _k < _len2; _k++) {
                joined = joinedModels[_k];
                if (!attr.joinOnField) {
                  throw new Error("Queryable collection attribute " + attr.modelKey + " must specify a joinOnField");
                }
                joinValue = joined[attr.joinOnField];
                joinMarks.push(joinMarkUnit);
                joinedValues.push(model.id, joinValue);
                for (_l = 0, _len3 = joinQueryableByJSONKeys.length; _l < _len3; _l++) {
                  joinedJsonKey = joinQueryableByJSONKeys[_l];
                  joinedValues.push(modelsJSONs[idx][joinedJsonKey]);
                }
              }
            }
          }
          if (joinedValues.length !== 0) {
            slicePageCount = Math.ceil(joinMarks.length / rowsPerInsert) - 1;
            _results = [];
            for (slice = _m = 0; _m <= slicePageCount; slice = _m += 1) {
              _ref1 = [slice * rowsPerInsert, slice * rowsPerInsert + (rowsPerInsert - 1)], ms = _ref1[0], me = _ref1[1];
              _ref2 = [slice * valuesPerInsert, slice * valuesPerInsert + (valuesPerInsert - 1)], vs = _ref2[0], ve = _ref2[1];
              _results.push(promises.push(_this._query("INSERT OR IGNORE INTO `" + joinTable + "` (`" + (joinColumns.join('`,`')) + "`) VALUES " + (joinMarks.slice(ms, +me + 1 || 9e9).join(',')), joinedValues.slice(vs, +ve + 1 || 9e9))));
            }
            return _results;
          }
        };
      })(this));
      values = [];
      marks = [];
      joinedDataAttributes = _.filter(attributes, function(attr) {
        return attr instanceof AttributeJoinedData;
      });
      joinedDataAttributes.forEach((function(_this) {
        return function(attr) {
          var _j, _len1, _results;
          _results = [];
          for (_j = 0, _len1 = models.length; _j < _len1; _j++) {
            model = models[_j];
            if (model[attr.modelKey] != null) {
              _results.push(promises.push(_this._query("REPLACE INTO `" + attr.modelTable + "` (`id`, `value`) VALUES (?, ?)", [model.id, model[attr.modelKey]])));
            } else {
              _results.push(void 0);
            }
          }
          return _results;
        };
      })(this));
      return Promise.all(promises);
    };

    DatabaseTransaction.prototype._deleteModel = function(model) {
      var attributes, collectionAttributes, joinedDataAttributes, klass, promises;
      promises = [];
      klass = model.constructor;
      attributes = _.values(klass.attributes);
      promises.push(this._query("DELETE FROM `" + klass.name + "` WHERE `id` = ?", [model.id]));
      collectionAttributes = _.filter(attributes, function(attr) {
        return attr.queryable && attr instanceof AttributeCollection;
      });
      collectionAttributes.forEach((function(_this) {
        return function(attr) {
          var joinTable;
          joinTable = tableNameForJoin(klass, attr.itemClass);
          return promises.push(_this._query("DELETE FROM `" + joinTable + "` WHERE `id` = ?", [model.id]));
        };
      })(this));
      joinedDataAttributes = _.filter(attributes, function(attr) {
        return attr instanceof AttributeJoinedData;
      });
      joinedDataAttributes.forEach((function(_this) {
        return function(attr) {
          return promises.push(_this._query("DELETE FROM `" + attr.modelTable + "` WHERE `id` = ?", [model.id]));
        };
      })(this));
      return Promise.all(promises);
    };

    return DatabaseTransaction;

  })();

  module.exports = DatabaseTransaction;

}).call(this);
