(function() {
  var AccountStore, Actions, Contact, ContactStore, DOMUtils, DatabaseStore, DraftFactory, DraftStore, FocusedPerspectiveStore, InlineStyleTransformer, Message, MessageStore, MessageUtils, SanitizeTransformer, Thread, Utils, subjectWithPrefix, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require('underscore');

  Actions = require('../actions');

  DatabaseStore = require('./database-store');

  AccountStore = require('./account-store');

  ContactStore = require('./contact-store');

  MessageStore = require('./message-store');

  FocusedPerspectiveStore = require('./focused-perspective-store');

  DraftStore = null;

  InlineStyleTransformer = require('../../services/inline-style-transformer');

  SanitizeTransformer = require('../../services/sanitize-transformer');

  Thread = require('../models/thread')["default"];

  Contact = require('../models/contact');

  Message = require('../models/message')["default"];

  Utils = require('../models/utils');

  MessageUtils = require('../models/message-utils');

  subjectWithPrefix = require('../models/utils').subjectWithPrefix;

  DOMUtils = require('../../dom-utils');

  DraftFactory = (function() {
    function DraftFactory() {
      this._accountForNewDraft = __bind(this._accountForNewDraft, this);
      this._fromContactForReply = __bind(this._fromContactForReply, this);
      this._prepareBodyForQuoting = __bind(this._prepareBodyForQuoting, this);
      this.updateDraftForReply = __bind(this.updateDraftForReply, this);
      this.candidateDraftForUpdating = __bind(this.candidateDraftForUpdating, this);
      this.createDraftForForward = __bind(this.createDraftForForward, this);
      this.createDraftForReply = __bind(this.createDraftForReply, this);
      this.createOrUpdateDraftForReply = __bind(this.createOrUpdateDraftForReply, this);
      this.createDraftForMailto = __bind(this.createDraftForMailto, this);
      this.createDraft = __bind(this.createDraft, this);
    }

    DraftFactory.prototype.createDraft = function(fields) {
      var account;
      if (fields == null) {
        fields = {};
      }
      account = this._accountForNewDraft();
      return Promise.resolve(new Message(_.extend({
        body: '',
        subject: '',
        clientId: Utils.generateTempId(),
        from: [account.defaultMe()],
        date: new Date,
        draft: true,
        pristine: true,
        accountId: account.id
      }, fields)));
    };

    DraftFactory.prototype.createDraftForMailto = function(urlString) {
      var account, attr, contacts, match, openKey, openValueEnd, openValueStart, query, querySplit, queryString, to, value, valueIsntEscaped, whole, _i, _len, _ref, _ref1;
      account = this._accountForNewDraft();
      try {
        urlString = decodeURI(urlString);
      } catch (_error) {}
      _ref = /mailto:\/*([^\?\&]*)((.|\n|\r)*)/.exec(urlString), whole = _ref[0], to = _ref[1], queryString = _ref[2];
      if (to.length > 0 && to.indexOf('@') === -1) {
        to = decodeURIComponent(to);
      }
      query = {};
      query.to = to;
      querySplit = /[&|?](subject|body|cc|to|from|bcc)+\s*=/gi;
      openKey = null;
      openValueStart = null;
      while (match !== null) {
        match = querySplit.exec(queryString);
        openValueEnd = (match != null ? match.index : void 0) || queryString.length;
        if (openKey) {
          value = queryString.substr(openValueStart, openValueEnd - openValueStart);
          valueIsntEscaped = value.indexOf('?') !== -1 || value.indexOf('&') !== -1;
          try {
            if (!valueIsntEscaped) {
              value = decodeURIComponent(value);
            }
          } catch (_error) {}
          query[openKey] = value;
        }
        if (match) {
          openKey = match[1].toLowerCase();
          openValueStart = querySplit.lastIndex;
        }
      }
      contacts = {};
      _ref1 = ['to', 'cc', 'bcc'];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        attr = _ref1[_i];
        if (query[attr]) {
          contacts[attr] = ContactStore.parseContactsInString(query[attr]);
        }
      }
      return Promise.props(contacts).then((function(_this) {
        return function(contacts) {
          return _this.createDraft(_.extend(query, contacts));
        };
      })(this));
    };

    DraftFactory.prototype.createOrUpdateDraftForReply = function(_arg) {
      var behavior, message, thread, type;
      message = _arg.message, thread = _arg.thread, type = _arg.type, behavior = _arg.behavior;
      if (type !== 'reply' && type !== 'reply-all') {
        throw new Error("createOrUpdateDraftForReply called with " + type + ", not reply or reply-all");
      }
      return this.candidateDraftForUpdating(message, behavior).then((function(_this) {
        return function(existingDraft) {
          if (existingDraft) {
            return _this.updateDraftForReply(existingDraft, {
              message: message,
              thread: thread,
              type: type
            });
          } else {
            return _this.createDraftForReply({
              message: message,
              thread: thread,
              type: type
            });
          }
        };
      })(this));
    };

    DraftFactory.prototype.createDraftForReply = function(_arg) {
      var message, thread, type;
      message = _arg.message, thread = _arg.thread, type = _arg.type;
      return this._prepareBodyForQuoting(message.body).then((function(_this) {
        return function(body) {
          var cc, to, _ref, _ref1;
          if (type === 'reply') {
            _ref = message.participantsForReply(), to = _ref.to, cc = _ref.cc;
          } else if (type === 'reply-all') {
            _ref1 = message.participantsForReplyAll(), to = _ref1.to, cc = _ref1.cc;
          }
          return _this.createDraft({
            subject: subjectWithPrefix(message.subject, 'Re:'),
            to: to,
            cc: cc,
            from: [_this._fromContactForReply(message)],
            threadId: thread.id,
            accountId: message.accountId,
            replyToMessageId: message.id,
            body: "<br><br><div class=\"gmail_quote\">\n  " + (DOMUtils.escapeHTMLCharacters(message.replyAttributionLine())) + "\n  <br>\n  <blockquote class=\"gmail_quote\"\n    style=\"margin:0 0 0 .8ex;border-left:1px #ccc solid;padding-left:1ex;\">\n    " + body + "\n  </blockquote>\n</div>"
          });
        };
      })(this));
    };

    DraftFactory.prototype.createDraftForForward = function(_arg) {
      var contactsAsHtml, fields, message, thread;
      thread = _arg.thread, message = _arg.message;
      contactsAsHtml = function(cs) {
        return DOMUtils.escapeHTMLCharacters(_.invoke(cs, "toString").join(", "));
      };
      fields = [];
      if (message.from.length > 0) {
        fields.push("From: " + (contactsAsHtml(message.from)));
      }
      fields.push("Subject: " + message.subject);
      fields.push("Date: " + (message.formattedDate()));
      if (message.to.length > 0) {
        fields.push("To: " + (contactsAsHtml(message.to)));
      }
      if (message.cc.length > 0) {
        fields.push("Cc: " + (contactsAsHtml(message.cc)));
      }
      return this._prepareBodyForQuoting(message.body).then((function(_this) {
        return function(body) {
          return _this.createDraft({
            subject: subjectWithPrefix(message.subject, 'Fwd:'),
            files: [].concat(message.files),
            from: [_this._fromContactForReply(message)],
            threadId: thread.id,
            accountId: message.accountId,
            body: "<br><br><div class=\"gmail_quote\">\n  ---------- Forwarded message ---------\n  <br><br>\n  " + (fields.join('<br>')) + "\n  <br><br>\n  " + body + "\n</div>"
          });
        };
      })(this));
    };

    DraftFactory.prototype.candidateDraftForUpdating = function(message, behavior) {
      var getMessages;
      if (behavior !== 'prefer-existing-if-pristine' && behavior !== 'prefer-existing') {
        return Promise.resolve(null);
      }
      getMessages = DatabaseStore.findAll(Message, {
        threadId: message.threadId
      });
      if (message.threadId === MessageStore.threadId()) {
        getMessages = Promise.resolve(MessageStore.items());
      }
      return getMessages.then((function(_this) {
        return function(messages) {
          var candidateDrafts;
          candidateDrafts = messages.filter(function(other) {
            return other.replyToMessageId === message.id && other.draft === true;
          });
          if (candidateDrafts.length === 0) {
            return Promise.resolve(null);
          }
          if (behavior === 'prefer-existing') {
            return Promise.resolve(candidateDrafts.pop());
          } else if (behavior === 'prefer-existing-if-pristine') {
            if (DraftStore == null) {
              DraftStore = require('./draft-store');
            }
            return Promise.all(candidateDrafts.map(function(candidateDraft) {
              return DraftStore.sessionForClientId(candidateDraft.clientId);
            })).then(function(sessions) {
              var session, _i, _len;
              for (_i = 0, _len = sessions.length; _i < _len; _i++) {
                session = sessions[_i];
                if (session.draft().pristine) {
                  return Promise.resolve(session.draft());
                }
              }
              return Promise.resolve(null);
            });
          }
        };
      })(this));
    };

    DraftFactory.prototype.updateDraftForReply = function(draft, _arg) {
      var contact, key, message, replyAllSet, replySet, targetSet, type, updated, _i, _j, _k, _len, _len1, _len2, _ref, _ref1, _ref2;
      type = _arg.type, message = _arg.message;
      if (!(message && draft)) {
        return Promise.reject("updateDraftForReply: Expected message and existing draft.");
      }
      updated = {
        to: [].concat(draft.to),
        cc: [].concat(draft.cc)
      };
      replySet = message.participantsForReply();
      replyAllSet = message.participantsForReplyAll();
      if (type === 'reply') {
        targetSet = replySet;
        _ref = ['to', 'cc'];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          key = _ref[_i];
          updated[key] = _.reject(updated[key], function(contact) {
            var inReplyAllSet, inReplySet;
            inReplySet = _.findWhere(replySet[key], {
              email: contact.email
            });
            inReplyAllSet = _.findWhere(replyAllSet[key], {
              email: contact.email
            });
            return inReplyAllSet && !inReplySet;
          });
        }
      } else {
        targetSet = replyAllSet;
      }
      _ref1 = ['to', 'cc'];
      for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
        key = _ref1[_j];
        _ref2 = targetSet[key];
        for (_k = 0, _len2 = _ref2.length; _k < _len2; _k++) {
          contact = _ref2[_k];
          if (!_.findWhere(updated[key], {
            email: contact.email
          })) {
            updated[key].push(contact);
          }
        }
      }
      draft.to = updated.to;
      draft.cc = updated.cc;
      return DatabaseStore.inTransaction((function(_this) {
        return function(t) {
          return t.persistModel(draft);
        };
      })(this)).thenReturn(draft);
    };

    DraftFactory.prototype._prepareBodyForQuoting = function(body) {
      var cidRE, re;
      if (body == null) {
        body = "";
      }
      cidRE = MessageUtils.cidRegexString;
      re = new RegExp("<img.*" + cidRE + "[\\s\\S]*?>", "igm");
      body.replace(re, "");
      return InlineStyleTransformer.run(body).then((function(_this) {
        return function(body) {
          return SanitizeTransformer.run(body, SanitizeTransformer.Preset.UnsafeOnly);
        };
      })(this));
    };

    DraftFactory.prototype._fromContactForReply = function(message) {
      var account, alias, aliasString, defaultMe, emailIsNotDefault, emailsMatch, nameIsNotDefault, namesMatch, recipient, result, _i, _j, _len, _len1, _ref, _ref1;
      account = AccountStore.accountForId(message.accountId);
      defaultMe = account.defaultMe();
      result = defaultMe;
      _ref = account.aliases;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        aliasString = _ref[_i];
        alias = account.meUsingAlias(aliasString);
        _ref1 = [].concat(message.to, message.cc);
        for (_j = 0, _len1 = _ref1.length; _j < _len1; _j++) {
          recipient = _ref1[_j];
          emailIsNotDefault = alias.email !== defaultMe.email;
          emailsMatch = recipient.email === alias.email;
          nameIsNotDefault = alias.name !== defaultMe.name;
          namesMatch = recipient.name === alias.name;
          if (emailsMatch && emailIsNotDefault && namesMatch && nameIsNotDefault) {
            return alias;
          }
          if ((emailsMatch && emailIsNotDefault) || (namesMatch && nameIsNotDefault)) {
            result = alias;
          }
        }
      }
      return result;
    };

    DraftFactory.prototype._accountForNewDraft = function() {
      var account, defAccountId, focusedAccountId;
      defAccountId = NylasEnv.config.get('core.sending.defaultAccountIdForSend');
      account = AccountStore.accountForId(defAccountId);
      if (account) {
        return account;
      } else {
        focusedAccountId = FocusedPerspectiveStore.current().accountIds[0];
        if (focusedAccountId) {
          return AccountStore.accountForId(focusedAccountId);
        } else {
          return AccountStore.accounts()[0];
        }
      }
    };

    return DraftFactory;

  })();

  module.exports = new DraftFactory();

}).call(this);
