(function() {
  var Actions, Download, FileDownloadStore, NylasAPI, Reflux, RegExpUtils, State, Utils, fs, mkdirp, mkdirpAsync, os, path, progress, remote, shell, _, _ref;

  os = require('os');

  fs = require('fs');

  path = require('path');

  _ref = require('electron'), remote = _ref.remote, shell = _ref.shell;

  mkdirp = require('mkdirp');

  Utils = require('../models/utils');

  Reflux = require('reflux');

  _ = require('underscore');

  Actions = require('../actions');

  progress = require('request-progress');

  NylasAPI = require('../nylas-api');

  RegExpUtils = require('../../regexp-utils');

  Promise.promisifyAll(fs);

  mkdirpAsync = Promise.promisify(mkdirp);

  State = {
    Unstarted: 'unstarted',
    Downloading: 'downloading',
    Finished: 'finished',
    Failed: 'failed'
  };

  Download = (function() {
    Download.State = State;

    function Download(_arg) {
      this.accountId = _arg.accountId, this.fileId = _arg.fileId, this.targetPath = _arg.targetPath, this.filename = _arg.filename, this.filesize = _arg.filesize, this.progressCallback = _arg.progressCallback;
      if (!this.accountId) {
        throw new Error("Download.constructor: You must provide a non-empty accountId.");
      }
      if (!this.filename || this.filename.length === 0) {
        throw new Error("Download.constructor: You must provide a non-empty filename.");
      }
      if (!this.fileId) {
        throw new Error("Download.constructor: You must provide a fileID to download.");
      }
      if (!this.targetPath) {
        throw new Error("Download.constructor: You must provide a target path to download.");
      }
      this.percent = 0;
      this.promise = null;
      this.state = State.Unstarted;
      this;
    }

    Download.prototype.data = function() {
      return Object.freeze(_.clone({
        state: this.state,
        fileId: this.fileId,
        percent: this.percent,
        filename: this.filename,
        filesize: this.filesize,
        targetPath: this.targetPath
      }));
    };

    Download.prototype.run = function() {
      if (this.promise) {
        return this.promise;
      }
      return this.promise = new Promise((function(_this) {
        return function(resolve, reject) {
          var onFailed, onSuccess, stream;
          stream = fs.createWriteStream(_this.targetPath);
          _this.state = State.Downloading;
          onFailed = function(err) {
            _this.request = null;
            stream.end();
            _this.state = State.Failed;
            if (fs.existsSync(_this.targetPath)) {
              fs.unlinkSync(_this.targetPath);
            }
            return reject(err);
          };
          onSuccess = function() {
            _this.request = null;
            stream.end();
            _this.state = State.Finished;
            _this.percent = 100;
            return resolve(_this);
          };
          return NylasAPI.makeRequest({
            json: false,
            path: "/files/" + _this.fileId + "/download",
            accountId: _this.accountId,
            encoding: null,
            started: function(req) {
              _this.request = req;
              return progress(_this.request, {
                throtte: 250
              }).on("progress", function(progress) {
                _this.percent = progress.percent;
                return _this.progressCallback();
              }).on("error", onFailed).on("end", function() {
                var statusCode, _ref1;
                if (_this.state === State.Failed) {
                  return;
                }
                statusCode = (_ref1 = _this.request.response) != null ? _ref1.statusCode : void 0;
                if ([200, 202, 204].includes(statusCode)) {
                  return onSuccess();
                } else {
                  return onFailed(new Error("Server returned a " + statusCode));
                }
              }).pipe(stream);
            }
          });
        };
      })(this));
    };

    Download.prototype.ensureClosed = function() {
      var _ref1;
      return (_ref1 = this.request) != null ? _ref1.abort() : void 0;
    };

    return Download;

  })();

  module.exports = FileDownloadStore = Reflux.createStore({
    init: function() {
      this.listenTo(Actions.fetchFile, this._fetch);
      this.listenTo(Actions.fetchAndOpenFile, this._fetchAndOpen);
      this.listenTo(Actions.fetchAndSaveFile, this._fetchAndSave);
      this.listenTo(Actions.fetchAndSaveAllFiles, this._fetchAndSaveAll);
      this.listenTo(Actions.abortFetchFile, this._abortFetchFile);
      this.listenTo(Actions.didPassivelyReceiveNewModels, this._newMailReceived);
      this._downloads = {};
      this._downloadDirectory = path.join(NylasEnv.getConfigDirPath(), 'downloads');
      return mkdirp(this._downloadDirectory);
    },
    pathForFile: function(file) {
      if (!file) {
        return void 0;
      }
      return path.join(this._downloadDirectory, file.id, file.safeDisplayName());
    },
    downloadDataForFile: function(fileId) {
      var _ref1;
      return (_ref1 = this._downloads[fileId]) != null ? _ref1.data() : void 0;
    },
    downloadDataForFiles: function(fileIds) {
      var downloadData;
      if (fileIds == null) {
        fileIds = [];
      }
      downloadData = {};
      fileIds.forEach((function(_this) {
        return function(fileId) {
          return downloadData[fileId] = _this.downloadDataForFile(fileId);
        };
      })(this));
      return downloadData;
    },
    _newMailReceived: function(incoming) {
      var file, message, _i, _len, _ref1, _results;
      if (NylasEnv.config.get('core.attachments.downloadPolicy') === 'on-receive') {
        if (!incoming['message']) {
          return;
        }
        _ref1 = incoming['message'];
        _results = [];
        for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
          message = _ref1[_i];
          _results.push((function() {
            var _j, _len1, _ref2, _results1;
            _ref2 = message.files;
            _results1 = [];
            for (_j = 0, _len1 = _ref2.length; _j < _len1; _j++) {
              file = _ref2[_j];
              _results1.push(this._fetch(file));
            }
            return _results1;
          }).call(this));
        }
        return _results;
      }
    },
    _runDownload: function(file) {
      var download, targetPath;
      targetPath = this.pathForFile(file);
      download = this._downloads[file.id];
      if (download) {
        return download.run();
      }
      download = new Download({
        accountId: file.accountId,
        fileId: file.id,
        filesize: file.size,
        filename: file.displayName(),
        targetPath: targetPath,
        progressCallback: (function(_this) {
          return function() {
            return _this.trigger();
          };
        })(this)
      });
      return this._prepareFolder(file).then((function(_this) {
        return function() {
          return _this._checkForDownloadedFile(file).then(function(alreadyHaveFile) {
            if (alreadyHaveFile) {
              download.promise = Promise.resolve();
              download.state = State.Finished;
              return Promise.resolve(download);
            } else {
              _this._downloads[file.id] = download;
              _this.trigger();
              return download.run()["finally"](function() {
                download.ensureClosed();
                if (download.state === State.Failed) {
                  delete _this._downloads[file.id];
                }
                return _this.trigger();
              });
            }
          });
        };
      })(this));
    },
    _checkForDownloadedFile: function(file) {
      return fs.statAsync(this.pathForFile(file))["catch"]((function(_this) {
        return function(err) {
          return Promise.resolve(false);
        };
      })(this)).then((function(_this) {
        return function(stats) {
          return Promise.resolve(stats.size >= file.size);
        };
      })(this));
    },
    _prepareFolder: function(file) {
      var targetFolder;
      targetFolder = path.join(this._downloadDirectory, file.id);
      return fs.statAsync(targetFolder)["catch"]((function(_this) {
        return function() {
          return mkdirpAsync(targetFolder);
        };
      })(this));
    },
    _fetch: function(file) {
      return this._runDownload(file)["catch"](this._catchFSErrors)["catch"](function(error) {});
    },
    _fetchAndOpen: function(file) {
      return this._runDownload(file).then(function(download) {
        return shell.openItem(download.targetPath);
      })["catch"](this._catchFSErrors)["catch"]((function(_this) {
        return function(error) {
          return _this._presentError({
            file: file,
            error: error
          });
        };
      })(this));
    },
    _saveDownload: (function(_this) {
      return function(download, savePath) {
        return new Promise(function(resolve, reject) {
          var stream;
          stream = fs.createReadStream(download.targetPath);
          stream.pipe(fs.createWriteStream(savePath));
          stream.on('error', function(err) {
            return reject(err);
          });
          return stream.on('end', function() {
            return resolve();
          });
        });
      };
    })(this),
    _fetchAndSave: function(file) {
      var defaultExtension, defaultPath;
      defaultPath = this._defaultSavePath(file);
      defaultExtension = path.extname(defaultPath);
      return NylasEnv.showSaveDialog({
        defaultPath: defaultPath
      }, (function(_this) {
        return function(savePath) {
          var didLoseExtension, saveExtension;
          if (!savePath) {
            return;
          }
          NylasEnv.savedState.lastDownloadDirectory = path.dirname(savePath);
          saveExtension = path.extname(savePath);
          didLoseExtension = defaultExtension !== '' && saveExtension === '';
          if (didLoseExtension) {
            savePath = savePath + defaultExtension;
          }
          return _this._runDownload(file).then(function(download) {
            return _this._saveDownload(download, savePath);
          }).then(function() {
            return shell.showItemInFolder(savePath);
          })["catch"](_this._catchFSErrors)["catch"](function(error) {
            return _this._presentError({
              file: file,
              error: error
            });
          });
        };
      })(this));
    },
    _fetchAndSaveAll: function(files) {
      var defaultPath, options;
      defaultPath = this._defaultSaveDir();
      options = {
        defaultPath: defaultPath,
        title: 'Save Into...',
        buttonLabel: 'Download All',
        properties: ['openDirectory', 'createDirectory']
      };
      return new Promise((function(_this) {
        return function(resolve, reject) {
          return NylasEnv.showOpenDialog(options, function(selected) {
            var dirPath, lastSavePaths, savePromises;
            if (!selected) {
              return;
            }
            dirPath = selected[0];
            if (!dirPath) {
              return;
            }
            NylasEnv.savedState.lastDownloadDirectory = dirPath;
            lastSavePaths = [];
            savePromises = files.map(function(file) {
              var savePath;
              savePath = path.join(dirPath, file.safeDisplayName());
              return _this._runDownload(file).then(function(download) {
                return _this._saveDownload(download, savePath);
              }).then(function() {
                return lastSavePaths.push(savePath);
              });
            });
            return Promise.all(savePromises).then(function() {
              if (lastSavePaths.length > 0) {
                shell.showItemInFolder(lastSavePaths[0]);
              }
              return resolve(lastSavePaths);
            })["catch"](_this._catchFSErrors)["catch"](function() {
              return _this._presentError(file);
            });
          });
        };
      })(this));
    },
    _abortFetchFile: function(file) {
      var download, downloadPath;
      download = this._downloads[file.id];
      if (!download) {
        return;
      }
      download.ensureClosed();
      this.trigger();
      downloadPath = this.pathForFile(file);
      return fs.exists(downloadPath, function(exists) {
        if (exists) {
          return fs.unlink(downloadPath);
        }
      });
    },
    _defaultSaveDir: function() {
      var downloadDir, home;
      if (process.platform === 'win32') {
        home = process.env.USERPROFILE;
      } else {
        home = process.env.HOME;
      }
      downloadDir = path.join(home, 'Downloads');
      if (!fs.existsSync(downloadDir)) {
        downloadDir = os.tmpdir();
      }
      if (NylasEnv.savedState.lastDownloadDirectory) {
        if (fs.existsSync(NylasEnv.savedState.lastDownloadDirectory)) {
          downloadDir = NylasEnv.savedState.lastDownloadDirectory;
        }
      }
      return downloadDir;
    },
    _defaultSavePath: function(file) {
      var downloadDir;
      downloadDir = this._defaultSaveDir();
      return path.join(downloadDir, file.safeDisplayName());
    },
    _presentError: function(_arg) {
      var error, errorString, file, name, _ref1;
      _ref1 = _arg != null ? _arg : {}, file = _ref1.file, error = _ref1.error;
      name = file ? file.displayName() : "one or more files";
      errorString = error ? error.toString() : "";
      return remote.dialog.showMessageBox({
        type: 'warning',
        message: "Download Failed",
        detail: "Unable to download " + name + ". Check your network connection and try again. " + errorString,
        buttons: ["OK"]
      });
    },
    _catchFSErrors: function(error) {
      var message, _ref1, _ref2;
      message = null;
      if ((_ref1 = error.code) === 'EPERM' || _ref1 === 'EMFILE' || _ref1 === 'EACCES') {
        message = "N1 could not save an attachment. Check that permissions are set correctly and try restarting N1 if the issue persists.";
      }
      if ((_ref2 = error.code) === 'ENOSPC') {
        message = "N1 could not save an attachment because you have run out of disk space.";
      }
      if (message) {
        remote.dialog.showMessageBox({
          type: 'warning',
          message: "Download Failed",
          detail: "" + message + "\n\n" + error.message,
          buttons: ["OK"]
        });
        return Promise.resolve();
      } else {
        return Promise.reject(error);
      }
    }
  });

  FileDownloadStore.Download = Download;

}).call(this);
