(function() {
  var Actions, DatabaseStore, DraftStore, FileUploadStore, Message, NylasStore, UPLOAD_DIR, Upload, Utils, fs, mkdirp, mkdirpAsync, path, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  fs = require('fs');

  path = require('path');

  mkdirp = require('mkdirp');

  NylasStore = require('nylas-store');

  Actions = require('../actions');

  Utils = require('../models/utils');

  Message = require('../models/message')["default"];

  DraftStore = require('./draft-store');

  DatabaseStore = require('./database-store');

  Promise.promisifyAll(fs);

  mkdirpAsync = Promise.promisify(mkdirp);

  UPLOAD_DIR = path.join(NylasEnv.getConfigDirPath(), 'uploads');

  Upload = (function() {
    function Upload(messageClientId, originPath, stats, id, uploadDir) {
      this.messageClientId = messageClientId;
      this.originPath = originPath;
      this.stats = stats;
      this.id = id;
      this.uploadDir = uploadDir != null ? uploadDir : UPLOAD_DIR;
      if (this.id == null) {
        this.id = Utils.generateTempId();
      }
      this.filename = path.basename(this.originPath);
      this.targetDir = path.join(this.uploadDir, this.messageClientId, this.id);
      this.targetPath = path.join(this.targetDir, this.filename);
      this.size = this.stats.size;
    }

    return Upload;

  })();

  FileUploadStore = (function(_super) {
    __extends(FileUploadStore, _super);

    FileUploadStore.prototype.Upload = Upload;

    function FileUploadStore() {
      this._applySessionChanges = __bind(this._applySessionChanges, this);
      this._deleteUploadsForClientId = __bind(this._deleteUploadsForClientId, this);
      this._deleteUpload = __bind(this._deleteUpload, this);
      this._prepareTargetDir = __bind(this._prepareTargetDir, this);
      this._onDataChanged = __bind(this._onDataChanged, this);
      this.listenTo(Actions.addAttachment, this._onAddAttachment);
      this.listenTo(Actions.selectAttachment, this._onSelectAttachment);
      this.listenTo(Actions.removeAttachment, this._onRemoveAttachment);
      this.listenTo(DatabaseStore, this._onDataChanged);
      mkdirp.sync(UPLOAD_DIR);
      if (NylasEnv.isMainWindow() || NylasEnv.inSpecMode()) {
        this.listenTo(Actions.sendDraftSuccess, (function(_this) {
          return function(_arg) {
            var messageClientId;
            messageClientId = _arg.messageClientId;
            return _this._deleteUploadsForClientId(messageClientId);
          };
        })(this));
      }
    }

    FileUploadStore.prototype._onDataChanged = function(change) {
      if (!NylasEnv.isMainWindow()) {
        return;
      }
      if (!(change.objectClass === Message.name && change.type === 'unpersist')) {
        return;
      }
      return change.objects.forEach((function(_this) {
        return function(message) {
          return _this._deleteUploadsForClientId(message.clientId);
        };
      })(this));
    };

    FileUploadStore.prototype._onSelectAttachment = function(_arg) {
      var messageClientId;
      messageClientId = _arg.messageClientId;
      this._verifyId(messageClientId);
      return NylasEnv.showOpenDialog({
        properties: ['openFile', 'multiSelections']
      }, function(pathsToOpen) {
        if (pathsToOpen == null) {
          return;
        }
        if (_.isString(pathsToOpen)) {
          pathsToOpen = [pathsToOpen];
        }
        return pathsToOpen.forEach(function(filePath) {
          return Actions.addAttachment({
            messageClientId: messageClientId,
            filePath: filePath
          });
        });
      });
    };

    FileUploadStore.prototype._onAddAttachment = function(_arg) {
      var callback, filePath, messageClientId;
      messageClientId = _arg.messageClientId, filePath = _arg.filePath, callback = _arg.callback;
      if (callback == null) {
        callback = function() {};
      }
      this._verifyId(messageClientId);
      return this._getFileStats({
        messageClientId: messageClientId,
        filePath: filePath
      }).then(this._makeUpload).then(this._verifyUpload).then(this._prepareTargetDir).then(this._copyUpload).then((function(_this) {
        return function(upload) {
          return _this._applySessionChanges(upload.messageClientId, function(uploads) {
            return uploads.concat([upload]);
          });
        };
      })(this)).then(callback)["catch"](this._onAttachFileError);
    };

    FileUploadStore.prototype._onRemoveAttachment = function(upload) {
      if (!upload) {
        return Promise.resolve();
      }
      this._applySessionChanges(upload.messageClientId, function(uploads) {
        return _.reject(uploads, _.matcher({
          id: upload.id
        }));
      });
      return this._deleteUpload(upload)["catch"](this._onAttachFileError);
    };

    FileUploadStore.prototype._onAttachFileError = function(error) {
      return NylasEnv.showErrorDialog(error.message);
    };

    FileUploadStore.prototype._verifyId = function(messageClientId) {
      if (!messageClientId) {
        throw new Error("You need to pass the ID of the message (draft) this Action refers to");
      }
    };

    FileUploadStore.prototype._getFileStats = function(_arg) {
      var filePath, messageClientId;
      messageClientId = _arg.messageClientId, filePath = _arg.filePath;
      return fs.statAsync(filePath).then((function(_this) {
        return function(stats) {
          return Promise.resolve({
            messageClientId: messageClientId,
            filePath: filePath,
            stats: stats
          });
        };
      })(this))["catch"](function(err) {
        return Promise.reject(new Error("" + filePath + " could not be found, or has invalid file permissions."));
      });
    };

    FileUploadStore.prototype._makeUpload = function(_arg) {
      var filePath, messageClientId, stats;
      messageClientId = _arg.messageClientId, filePath = _arg.filePath, stats = _arg.stats;
      return Promise.resolve(new Upload(messageClientId, filePath, stats));
    };

    FileUploadStore.prototype._verifyUpload = function(upload) {
      var filename, stats;
      filename = upload.filename, stats = upload.stats;
      if (stats.isDirectory()) {
        return Promise.reject(new Error("" + filename + " is a directory. Try compressing it and attaching it again."));
      } else if (stats.size > 25 * 1000000) {
        return Promise.reject(new Error("" + filename + " cannot be attached because it is larger than 25MB."));
      } else {
        return Promise.resolve(upload);
      }
    };

    FileUploadStore.prototype._prepareTargetDir = function(upload) {
      return mkdirpAsync(upload.targetDir).thenReturn(upload);
    };

    FileUploadStore.prototype._copyUpload = function(upload) {
      return new Promise((function(_this) {
        return function(resolve, reject) {
          var originPath, readStream, targetPath, writeStream;
          originPath = upload.originPath, targetPath = upload.targetPath;
          readStream = fs.createReadStream(originPath);
          writeStream = fs.createWriteStream(targetPath);
          readStream.on('error', function() {
            return reject(new Error("Could not read file at path: " + originPath));
          });
          writeStream.on('error', function() {
            return reject(new Error("Could not write " + upload.filename + " to uploads directory."));
          });
          readStream.on('end', function() {
            return resolve(upload);
          });
          return readStream.pipe(writeStream);
        };
      })(this));
    };

    FileUploadStore.prototype._deleteUpload = function(upload) {
      return fs.unlinkAsync(upload.targetPath).then(function() {
        return fs.rmdirAsync(upload.targetDir).then(function() {
          fs.rmdir(path.join(UPLOAD_DIR, upload.messageClientId), function(err) {});
          return Promise.resolve(upload);
        });
      })["catch"](function(err) {
        return Promise.reject(new Error("Error deleting file upload " + upload.filename + ":\n\n" + err.message));
      });
    };

    FileUploadStore.prototype._deleteUploadsForClientId = function(messageClientId) {
      var rimraf;
      rimraf = require('rimraf');
      return rimraf(path.join(UPLOAD_DIR, messageClientId), {
        disableGlob: true
      }, (function(_this) {
        return function(err) {
          if (err) {
            return console.warn(err);
          }
        };
      })(this));
    };

    FileUploadStore.prototype._applySessionChanges = function(messageClientId, changeFunction) {
      return DraftStore.sessionForClientId(messageClientId).then((function(_this) {
        return function(session) {
          var uploads;
          uploads = changeFunction(session.draft().uploads);
          return session.changes.add({
            uploads: uploads
          });
        };
      })(this));
    };

    return FileUploadStore;

  })(NylasStore);

  module.exports = new FileUploadStore();

}).call(this);
