(function() {
  var AccountStore, Actions, Contact, DatabaseStore, FocusedContactsStore, FocusedContentStore, MessageStore, NylasStore, Rx, Thread, Utils, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  Rx = require('rx-lite');

  Utils = require('../models/utils');

  Actions = require('../actions');

  NylasStore = require('nylas-store');

  Thread = require('../models/thread')["default"];

  Contact = require('../models/contact');

  MessageStore = require('./message-store');

  AccountStore = require('./account-store');

  DatabaseStore = require('./database-store');

  FocusedContentStore = require('./focused-content-store');

  FocusedContactsStore = (function(_super) {
    __extends(FocusedContactsStore, _super);

    function FocusedContactsStore() {
      this._onFocusContact = __bind(this._onFocusContact, this);
      this._onMessageStoreChanged = __bind(this._onMessageStoreChanged, this);
      this.listenTo(MessageStore, this._onMessageStoreChanged);
      this.listenTo(Actions.focusContact, this._onFocusContact);
      this._clearCurrentParticipants();
    }

    FocusedContactsStore.prototype.sortedContacts = function() {
      return this._currentContacts;
    };

    FocusedContactsStore.prototype.focusedContact = function() {
      return this._currentFocusedContact;
    };

    FocusedContactsStore.prototype.focusedContactThreads = function() {
      var _ref;
      return (_ref = this._currentParticipantThreads) != null ? _ref : [];
    };

    FocusedContactsStore.prototype._onMessageStoreChanged = function() {
      var threadId;
      threadId = MessageStore.itemsLoading() ? null : MessageStore.threadId();
      if (this._currentThread && this._currentThread.id !== threadId) {
        this._clearCurrentParticipants();
        this.trigger();
      }
      if (this._onMessageStoreChangeThrottled == null) {
        this._onMessageStoreChangeThrottled = _.debounce((function(_this) {
          return function() {
            var thread, _ref;
            thread = MessageStore.itemsLoading() ? null : MessageStore.thread();
            if (thread && thread.id !== ((_ref = _this._currentThread) != null ? _ref.id : void 0)) {
              _this._currentThread = thread;
              return _this._popuateCurrentParticipants();
            }
          };
        })(this), 250);
      }
      return this._onMessageStoreChangeThrottled();
    };

    FocusedContactsStore.prototype._popuateCurrentParticipants = function() {
      var sorted;
      this._scoreAllParticipants();
      sorted = _.sortBy(_.values(this._contactScores), "score").reverse();
      this._currentContacts = _.map(sorted, function(obj) {
        return obj.contact;
      });
      return this._onFocusContact(this._currentContacts[0]);
    };

    FocusedContactsStore.prototype._clearCurrentParticipants = function() {
      var _ref;
      this._contactScores = {};
      this._currentContacts = [];
      if ((_ref = this._unsubFocusedContact) != null) {
        _ref.dispose();
      }
      this._unsubFocusedContact = null;
      this._currentFocusedContact = null;
      this._currentThread = null;
      return this._currentParticipantThreads = [];
    };

    FocusedContactsStore.prototype._onFocusContact = function(contact) {
      var query, _ref;
      if ((_ref = this._unsubFocusedContact) != null) {
        _ref.dispose();
      }
      this._unsubFocusedContact = null;
      this._currentParticipantThreads = [];
      if (contact) {
        query = DatabaseStore.findBy(Contact, {
          email: contact.email,
          accountId: this._currentThread.accountId
        });
        this._unsubFocusedContact = Rx.Observable.fromQuery(query).subscribe((function(_this) {
          return function(match) {
            _this._currentFocusedContact = match != null ? match : contact;
            return _this.trigger();
          };
        })(this));
        return this._loadCurrentParticipantThreads(contact.email);
      } else {
        this._currentFocusedContact = null;
        return this.trigger();
      }
    };

    FocusedContactsStore.prototype._loadCurrentParticipantThreads = function(email) {
      return DatabaseStore.findAll(Thread).where(Thread.attributes.participants.contains(email)).limit(100).then((function(_this) {
        return function(threads) {
          if (threads == null) {
            threads = [];
          }
          _this._currentParticipantThreads = threads;
          return _this.trigger();
        };
      })(this));
    };

    FocusedContactsStore.prototype._scoreAllParticipants = function() {
      var message, msgNum, score, _i, _ref;
      score = (function(_this) {
        return function(message, msgNum, field, multiplier) {
          var bonus, contact, j, _i, _len, _ref, _ref1, _results;
          _ref1 = (_ref = message[field]) != null ? _ref : [];
          _results = [];
          for (j = _i = 0, _len = _ref1.length; _i < _len; j = ++_i) {
            contact = _ref1[j];
            bonus = message[field].length - j;
            _results.push(_this._assignScore(contact, (msgNum + 1) * multiplier + bonus));
          }
          return _results;
        };
      })(this);
      _ref = MessageStore.items();
      for (msgNum = _i = _ref.length - 1; _i >= 0; msgNum = _i += -1) {
        message = _ref[msgNum];
        if (message.draft) {
          score(message, msgNum, "to", 10000);
          score(message, msgNum, "cc", 1000);
        } else {
          score(message, msgNum, "from", 100);
          score(message, msgNum, "to", 10);
          score(message, msgNum, "cc", 1);
        }
      }
      return this._contactScores;
    };

    FocusedContactsStore.prototype._assignScore = function(contact, score) {
      var key, _base;
      if (score == null) {
        score = 0;
      }
      if (!(contact && contact.email)) {
        return;
      }
      if (contact.email.trim().length === 0) {
        return;
      }
      key = Utils.toEquivalentEmailForm(contact.email);
      return (_base = this._contactScores)[key] != null ? _base[key] : _base[key] = {
        contact: contact,
        score: score - this._calculatePenalties(contact, score)
      };
    };

    FocusedContactsStore.prototype._calculatePenalties = function(contact, score) {
      var email, myEmail, notCommonDomain, penalties, sameDomain, _ref, _ref1;
      penalties = 0;
      email = contact.email.toLowerCase().trim();
      myEmail = (_ref = AccountStore.accountForId((_ref1 = this.currentThread) != null ? _ref1.accountId : void 0)) != null ? _ref.emailAddress : void 0;
      if (email === myEmail) {
        penalties += score;
      }
      notCommonDomain = !Utils.emailHasCommonDomain(myEmail);
      sameDomain = Utils.emailsHaveSameDomain(myEmail, email);
      if (notCommonDomain && sameDomain) {
        penalties += score * 0.9;
      }
      return Math.max(penalties, 0);
    };

    return FocusedContactsStore;

  })(NylasStore);

  module.exports = new FocusedContactsStore;

}).call(this);
