(function() {
  var AccountStore, Actions, CoffeeHelpers, DatabaseStore, FocusedContentStore, FocusedPerspectiveStore, Listener, MailboxPerspective, Model, Publisher, Reflux, Thread, WorkspaceStore, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  Reflux = require('reflux');

  AccountStore = require('./account-store');

  WorkspaceStore = require('./workspace-store');

  DatabaseStore = require('./database-store');

  FocusedPerspectiveStore = require('./focused-perspective-store');

  MailboxPerspective = require('../../mailbox-perspective');

  Actions = require('../actions');

  Thread = require('../models/thread')["default"];

  Model = require('../models/model');

  _ref = require('../modules/reflux-coffee'), Listener = _ref.Listener, Publisher = _ref.Publisher;

  CoffeeHelpers = require('../coffee-helpers');


  /*
  Public: The FocusedContentStore provides access to the objects currently selected
  or otherwise focused in the window. Normally, focus would be maintained internally
  by components that show models. The FocusedContentStore makes the concept of
  selection public so that you can observe focus changes and trigger your own changes
  to focus.
  
  Since {FocusedContentStore} is a Flux-compatible Store, you do not call setters
  on it directly. Instead, use {Actions::setFocus} or
  {Actions::setCursorPosition} to set focus. The FocusedContentStore observes
  these models, changes it's state, and broadcasts to it's observers.
  
  Note: The {FocusedContentStore} triggers when a focused model is changed, even if
  it's ID has not. For example, if the user has a {Thread} selected and removes a tag,
  {FocusedContentStore} will trigger so you can fetch the new version of the
  {Thread}. If you observe the {FocusedContentStore} properly, you should always
  have the latest version of the the selected object.
  
  **Standard Collections**:
  
     - thread
     - file
  
  **Example: Observing the Selected Thread**
  
  ```coffeescript
  @unsubscribe = FocusedContentStore.listen(@_onFocusChanged, @)
  
  ...
  
   * Called when focus has changed, or when the focused model has been modified.
  _onFocusChanged: ->
    thread = FocusedContentStore.focused('thread')
    if thread
      console.log("#{thread.subject} is selected!")
    else
      console.log("No thread is selected!")
  ```
  
  Section: Stores
   */

  FocusedContentStore = (function() {
    FocusedContentStore.include = CoffeeHelpers.includeModule;

    FocusedContentStore.include(Publisher);

    FocusedContentStore.include(Listener);

    function FocusedContentStore() {
      this.keyboardCursorEnabled = __bind(this.keyboardCursorEnabled, this);
      this.keyboardCursorId = __bind(this.keyboardCursorId, this);
      this.keyboardCursor = __bind(this.keyboardCursor, this);
      this.didFocusUsingClick = __bind(this.didFocusUsingClick, this);
      this.focusedId = __bind(this.focusedId, this);
      this.focused = __bind(this.focused, this);
      this._onDataChange = __bind(this._onDataChange, this);
      this._onWorkspaceChange = __bind(this._onWorkspaceChange, this);
      this._onFocus = __bind(this._onFocus, this);
      this._onFocusKeyboard = __bind(this._onFocusKeyboard, this);
      this._onAccountsChange = __bind(this._onAccountsChange, this);
      this._resetInstanceVars = __bind(this._resetInstanceVars, this);
      this.triggerAfterAnimationFrame = __bind(this.triggerAfterAnimationFrame, this);
      this._resetInstanceVars();
      this.listenTo(AccountStore, this._onAccountsChange);
      this.listenTo(WorkspaceStore, this._onWorkspaceChange);
      this.listenTo(DatabaseStore, this._onDataChange);
      this.listenTo(Actions.setFocus, this._onFocus);
      this.listenTo(Actions.setCursorPosition, this._onFocusKeyboard);
    }

    FocusedContentStore.prototype.triggerAfterAnimationFrame = function(payload) {
      return window.requestAnimationFrame((function(_this) {
        return function() {
          return _this.trigger(payload);
        };
      })(this));
    };

    FocusedContentStore.prototype._resetInstanceVars = function() {
      this._focused = {};
      this._focusedUsingClick = {};
      this._keyboardCursor = {};
      return this._keyboardCursorEnabled = WorkspaceStore.layoutMode() === 'list';
    };

    FocusedContentStore.prototype._onAccountsChange = function() {
      var changed, collection, dict, item, _i, _len, _ref1;
      changed = [];
      _ref1 = [this._focused, this._keyboardCursor];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        dict = _ref1[_i];
        for (collection in dict) {
          item = dict[collection];
          if (item && item.accountId && !AccountStore.accountForId(item.accountId)) {
            delete dict[collection];
            changed.push(collection);
          }
        }
      }
      if (changed.length > 0) {
        return this.trigger({
          impactsCollection: function(c) {
            return changed.includes(c);
          }
        });
      }
    };

    FocusedContentStore.prototype._onFocusKeyboard = function(_arg) {
      var collection, item, _ref1;
      collection = _arg.collection, item = _arg.item;
      if (item && !(item instanceof Model)) {
        throw new Error("focusKeyboard() requires a Model or null");
      }
      if (!collection) {
        throw new Error("focusKeyboard() requires a collection");
      }
      if (((_ref1 = this._keyboardCursor[collection]) != null ? _ref1.id : void 0) === (item != null ? item.id : void 0)) {
        return;
      }
      this._keyboardCursor[collection] = item;
      return this.triggerAfterAnimationFrame({
        impactsCollection: function(c) {
          return c === collection;
        }
      });
    };

    FocusedContentStore.prototype._onFocus = function(_arg) {
      var collection, currentCategories, desiredCategory, desiredCategoryName, item, perspective, usingClick, _ref1, _ref2;
      collection = _arg.collection, item = _arg.item, usingClick = _arg.usingClick, desiredCategoryName = _arg.desiredCategoryName;
      if (item && !(item instanceof Model)) {
        throw new Error("focus() requires a Model or null");
      }
      if (!collection) {
        throw new Error("focus() requires a collection");
      }
      if (((_ref1 = this._focused[collection]) != null ? _ref1.id : void 0) === (item != null ? item.id : void 0)) {
        return;
      }
      if (desiredCategoryName) {
        currentCategories = FocusedPerspectiveStore.current().categories();
        desiredCategory = item.categoryNamed(desiredCategoryName);
        if (desiredCategory) {
          if (_ref2 = desiredCategory.id, __indexOf.call(_.pluck(currentCategories, 'id'), _ref2) < 0) {
            perspective = MailboxPerspective.forCategory(desiredCategory);
            Actions.focusMailboxPerspective(perspective);
          }
        }
      }
      this._focused[collection] = item;
      this._focusedUsingClick[collection] = usingClick;
      if (item) {
        this._keyboardCursor[collection] = item;
      }
      return this.triggerAfterAnimationFrame({
        impactsCollection: function(c) {
          return c === collection;
        }
      });
    };

    FocusedContentStore.prototype._onWorkspaceChange = function() {
      var collection, item, keyboardCursorEnabled, _ref1, _ref2;
      keyboardCursorEnabled = WorkspaceStore.layoutMode() === 'list';
      if (keyboardCursorEnabled !== this._keyboardCursorEnabled) {
        this._keyboardCursorEnabled = keyboardCursorEnabled;
        if (keyboardCursorEnabled) {
          _ref1 = this._focused;
          for (collection in _ref1) {
            item = _ref1[collection];
            this._keyboardCursor[collection] = item;
          }
          this._focused = {};
        } else {
          _ref2 = this._keyboardCursor;
          for (collection in _ref2) {
            item = _ref2[collection];
            this._onFocus({
              collection: collection,
              item: item
            });
          }
        }
      }
      return this.trigger({
        impactsCollection: function() {
          return true;
        }
      });
    };

    FocusedContentStore.prototype._onDataChange = function(change) {
      var data, key, obj, touched, val, _i, _j, _len, _len1, _ref1, _ref2;
      if (!(change && change.objectClass)) {
        return;
      }
      touched = [];
      _ref1 = [this._focused, this._keyboardCursor];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        data = _ref1[_i];
        for (key in data) {
          val = data[key];
          if (!(val && val.constructor.name === change.objectClass)) {
            continue;
          }
          _ref2 = change.objects;
          for (_j = 0, _len1 = _ref2.length; _j < _len1; _j++) {
            obj = _ref2[_j];
            if (val.id === obj.id) {
              if (change.type === 'unpersist') {
                data[key] = null;
              } else {
                data[key] = obj;
              }
              touched.push(key);
            }
          }
        }
      }
      if (touched.length > 0) {
        return this.trigger({
          impactsCollection: function(c) {
            return __indexOf.call(touched, c) >= 0;
          }
        });
      }
    };


    /*
    Public: Returns the focused {Model} in the collection specified,
    or undefined if no item is focused.
    
    - `collection` The {String} name of a collection. Standard collections are
      listed above.
     */

    FocusedContentStore.prototype.focused = function(collection) {
      return this._focused[collection];
    };


    /*
    Public: Returns the ID of the focused {Model} in the collection specified,
    or undefined if no item is focused.
    
    - `collection` The {String} name of a collection. Standard collections are
      listed above.
     */

    FocusedContentStore.prototype.focusedId = function(collection) {
      var _ref1;
      return (_ref1 = this._focused[collection]) != null ? _ref1.id : void 0;
    };


    /*
    Public: Returns true if the item for the collection was focused via a click or
    false otherwise.
    
    - `collection` The {String} name of a collection. Standard collections are
      listed above.
     */

    FocusedContentStore.prototype.didFocusUsingClick = function(collection) {
      var _ref1;
      return (_ref1 = this._focusedUsingClick[collection]) != null ? _ref1 : false;
    };


    /*
    Public: Returns the {Model} the keyboard is currently focused on
    in the collection specified. Keyboard focus is not always separate from
    primary focus (selection). You can use {::keyboardCursorEnabled} to determine
    whether keyboard focus is enabled.
    
    - `collection` The {String} name of a collection. Standard collections are
      listed above.
     */

    FocusedContentStore.prototype.keyboardCursor = function(collection) {
      return this._keyboardCursor[collection];
    };


    /*
    Public: Returns the ID of the {Model} the keyboard is currently focused on
    in the collection specified. Keyboard focus is not always separate from
    primary focus (selection). You can use {::keyboardCursorEnabled} to determine
    whether keyboard focus is enabled.
    
    - `collection` The {String} name of a collection. Standard collections are
      listed above.
     */

    FocusedContentStore.prototype.keyboardCursorId = function(collection) {
      var _ref1;
      return (_ref1 = this._keyboardCursor[collection]) != null ? _ref1.id : void 0;
    };


    /*
    Public: Returns a {Boolean} - `true` if the keyboard cursor concept applies in
    the current {WorkspaceStore} layout mode. The keyboard cursor is currently only
    enabled in `list` mode.
     */

    FocusedContentStore.prototype.keyboardCursorEnabled = function() {
      return this._keyboardCursorEnabled;
    };

    return FocusedContentStore;

  })();

  module.exports = new FocusedContentStore();

}).call(this);
