(function() {
  var AccountStore, Actions, CategoryStore, FocusedPerspectiveStore, MailboxPerspective, NylasStore, WorkspaceStore, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  NylasStore = require('nylas-store');

  AccountStore = require('./account-store');

  WorkspaceStore = require('./workspace-store');

  MailboxPerspective = require('../../mailbox-perspective');

  CategoryStore = require('./category-store');

  Actions = require('../actions');

  FocusedPerspectiveStore = (function(_super) {
    __extends(FocusedPerspectiveStore, _super);

    function FocusedPerspectiveStore() {
      this.sidebarAccountIds = __bind(this.sidebarAccountIds, this);
      this.current = __bind(this.current, this);
      this._onFocusPerspectiveForAccounts = __bind(this._onFocusPerspectiveForAccounts, this);
      this._onFocusPerspective = __bind(this._onFocusPerspective, this);
      this._loadSavedPerspective = __bind(this._loadSavedPerspective, this);
      this._listenToCommands = __bind(this._listenToCommands, this);
      this._current = MailboxPerspective.forNothing();
      this.listenTo(CategoryStore, this._onCategoryStoreChanged);
      this.listenTo(Actions.focusMailboxPerspective, this._onFocusPerspective);
      this.listenTo(Actions.focusDefaultMailboxPerspectiveForAccounts, this._onFocusPerspectiveForAccounts);
      this._listenToCommands();
    }

    FocusedPerspectiveStore.prototype._listenToCommands = function() {
      return NylasEnv.commands.add(document.body, {
        'navigation:go-to-inbox': (function(_this) {
          return function() {
            return _this._setPerspectiveByName("inbox");
          };
        })(this),
        'navigation:go-to-sent': (function(_this) {
          return function() {
            return _this._setPerspectiveByName("sent");
          };
        })(this),
        'navigation:go-to-starred': (function(_this) {
          return function() {
            return _this._setPerspective(MailboxPerspective.forStarred(_this._current.accountIds));
          };
        })(this),
        'navigation:go-to-drafts': (function(_this) {
          return function() {
            return _this._setPerspective(MailboxPerspective.forDrafts(_this._current.accountIds));
          };
        })(this),
        'navigation:go-to-all': (function(_this) {
          return function() {
            var categories;
            categories = _this._current.accountIds.map(function(aid) {
              return CategoryStore.getArchiveCategory(aid);
            });
            return _this._setPerspective(MailboxPerspective.forCategories(categories));
          };
        })(this),
        'navigation:go-to-contacts': (function(_this) {
          return function() {};
        })(this),
        'navigation:go-to-tasks': (function(_this) {
          return function() {};
        })(this),
        'navigation:go-to-label': (function(_this) {
          return function() {};
        })(this)
      });
    };

    FocusedPerspectiveStore.prototype._loadSavedPerspective = function(savedPerspective, accounts) {
      var accountIds, accountIdsNotPresent, perspective;
      if (accounts == null) {
        accounts = AccountStore.accounts();
      }
      if (savedPerspective) {
        perspective = MailboxPerspective.fromJSON(savedPerspective);
        if (perspective) {
          accountIds = _.pluck(accounts, 'id');
          accountIdsNotPresent = _.difference(perspective.accountIds, accountIds);
          if (accountIdsNotPresent.length > 0) {
            perspective = null;
          }
        }
      }
      if (perspective == null) {
        perspective = this._defaultPerspective();
      }
      return perspective;
    };

    FocusedPerspectiveStore.prototype._onCategoryStoreChanged = function() {
      var accountIds, catExists, categories, categoryHasBeenDeleted, perspective, _ref;
      if (this._current.isEqual(MailboxPerspective.forNothing())) {
        perspective = this._loadSavedPerspective(NylasEnv.savedState.perspective);
        return this._setPerspective(perspective, (_ref = NylasEnv.savedState.sidebarAccountIds) != null ? _ref : perspective.accountIds);
      } else {
        accountIds = this._current.accountIds;
        categories = this._current.categories();
        catExists = function(cat) {
          return CategoryStore.byId(cat.accountId, cat.id);
        };
        categoryHasBeenDeleted = categories && !_.every(categories, catExists);
        if (categoryHasBeenDeleted) {
          return this._setPerspective(this._defaultPerspective(accountIds));
        }
      }
    };

    FocusedPerspectiveStore.prototype._onFocusPerspective = function(perspective) {
      var sidebarIsUnifiedInbox, _ref;
      sidebarIsUnifiedInbox = ((_ref = NylasEnv.savedState.sidebarAccountIds) != null ? _ref.length : void 0) > 1;
      if (sidebarIsUnifiedInbox) {
        return this._setPerspective(perspective);
      } else {
        return this._setPerspective(perspective, perspective.accountIds);
      }
    };

    FocusedPerspectiveStore.prototype._onFocusPerspectiveForAccounts = function(accountsOrIds, _arg) {
      var perspective, sidebarAccountIds;
      sidebarAccountIds = (_arg != null ? _arg : {}).sidebarAccountIds;
      if (!accountsOrIds) {
        return;
      }
      perspective = this._defaultPerspective(accountsOrIds);
      if (sidebarAccountIds == null) {
        sidebarAccountIds = perspective.accountIds;
      }
      return this._setPerspective(perspective, sidebarAccountIds);
    };

    FocusedPerspectiveStore.prototype._defaultPerspective = function(accounts) {
      if (accounts == null) {
        accounts = AccountStore.accounts();
      }
      if (!(accounts.length > 0)) {
        return MailboxPerspective.forNothing();
      }
      return MailboxPerspective.forInbox(accounts);
    };

    FocusedPerspectiveStore.prototype._setPerspective = function(perspective, sidebarAccountIds) {
      var desired, shouldTrigger;
      shouldTrigger = false;
      if (!perspective.isEqual(this._current)) {
        NylasEnv.savedState.perspective = perspective.toJSON();
        this._current = perspective;
        shouldTrigger = true;
      }
      if (sidebarAccountIds && !_.isEqual(NylasEnv.savedState.sidebarAccountIds, sidebarAccountIds)) {
        NylasEnv.savedState.sidebarAccountIds = sidebarAccountIds;
        shouldTrigger = true;
      }
      if (shouldTrigger) {
        this.trigger();
      }
      if (perspective.drafts) {
        desired = WorkspaceStore.Sheet.Drafts;
      } else {
        desired = WorkspaceStore.Sheet.Threads;
      }
      if (desired && WorkspaceStore.rootSheet() !== desired) {
        Actions.selectRootSheet(desired);
      }
      return Actions.popToRootSheet();
    };

    FocusedPerspectiveStore.prototype._setPerspectiveByName = function(categoryName) {
      var categories;
      categories = this._current.accountIds.map(function(aid) {
        return CategoryStore.getStandardCategory(aid, categoryName);
      });
      categories = _.compact(categories);
      if (categories.length === 0) {
        return;
      }
      return this._setPerspective(MailboxPerspective.forCategories(categories));
    };

    FocusedPerspectiveStore.prototype.current = function() {
      return this._current;
    };

    FocusedPerspectiveStore.prototype.sidebarAccountIds = function() {
      var _base;
      if ((_base = NylasEnv.savedState).sidebarAccountIds == null) {
        _base.sidebarAccountIds = AccountStore.accounts().map((function(_this) {
          return function(acc) {
            return acc.id;
          };
        })(this));
      }
      return NylasEnv.savedState.sidebarAccountIds;
    };

    return FocusedPerspectiveStore;

  })(NylasStore);

  module.exports = new FocusedPerspectiveStore();

}).call(this);
