(function() {
  var AccountStore, ActionTemplates, Actions, ConditionMode, ConditionTemplates, DatabaseStore, MailRulesStore, NylasStore, ReprocessMailRulesTask, RulesJSONBlobKey, Rx, TaskQueueStatusStore, Utils, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  NylasStore = require('nylas-store');

  _ = require('underscore');

  Rx = require('rx-lite');

  AccountStore = require('./account-store');

  DatabaseStore = require('./database-store');

  TaskQueueStatusStore = require('./task-queue-status-store');

  ReprocessMailRulesTask = require('../tasks/reprocess-mail-rules-task')["default"];

  Utils = require('../models/utils');

  Actions = require('../actions');

  _ref = require('../../mail-rules-templates'), ConditionMode = _ref.ConditionMode, ConditionTemplates = _ref.ConditionTemplates, ActionTemplates = _ref.ActionTemplates;

  RulesJSONBlobKey = "MailRules-V2";

  MailRulesStore = (function(_super) {
    __extends(MailRulesStore, _super);

    function MailRulesStore() {
      this._saveMailRules = __bind(this._saveMailRules, this);
      this._onNotificationActionTaken = __bind(this._onNotificationActionTaken, this);
      this._onDisableMailRule = __bind(this._onDisableMailRule, this);
      this._onUpdateMailRule = __bind(this._onUpdateMailRule, this);
      this._onAddMailRule = __bind(this._onAddMailRule, this);
      this._onReorderMailRule = __bind(this._onReorderMailRule, this);
      this._onDeleteMailRule = __bind(this._onDeleteMailRule, this);
      this.rulesForAccountId = __bind(this.rulesForAccountId, this);
      this.rules = __bind(this.rules, this);
      var query;
      this._rules = [];
      query = DatabaseStore.findJSONBlob(RulesJSONBlobKey);
      this._subscription = Rx.Observable.fromQuery(query).subscribe((function(_this) {
        return function(rules) {
          _this._rules = rules != null ? rules : [];
          return _this.trigger();
        };
      })(this));
      this.listenTo(Actions.addMailRule, this._onAddMailRule);
      this.listenTo(Actions.deleteMailRule, this._onDeleteMailRule);
      this.listenTo(Actions.reorderMailRule, this._onReorderMailRule);
      this.listenTo(Actions.updateMailRule, this._onUpdateMailRule);
      this.listenTo(Actions.disableMailRule, this._onDisableMailRule);
      this.listenTo(Actions.notificationActionTaken, this._onNotificationActionTaken);
    }

    MailRulesStore.prototype.rules = function() {
      return this._rules;
    };

    MailRulesStore.prototype.rulesForAccountId = function(accountId) {
      return this._rules.filter((function(_this) {
        return function(f) {
          return f.accountId === accountId;
        };
      })(this));
    };

    MailRulesStore.prototype._onDeleteMailRule = function(id) {
      this._rules = this._rules.filter(function(f) {
        return f.id !== id;
      });
      this._saveMailRules();
      return this.trigger();
    };

    MailRulesStore.prototype._onReorderMailRule = function(id, newIdx) {
      var currentIdx, rule;
      currentIdx = _.findIndex(this._rules, _.matcher({
        id: id
      }));
      if (currentIdx === -1) {
        return;
      }
      rule = this._rules[currentIdx];
      this._rules.splice(currentIdx, 1);
      this._rules.splice(newIdx, 0, rule);
      this._saveMailRules();
      return this.trigger();
    };

    MailRulesStore.prototype._onAddMailRule = function(properties) {
      var defaults;
      defaults = {
        id: Utils.generateTempId(),
        name: "Untitled Rule",
        conditionMode: ConditionMode.All,
        conditions: [ConditionTemplates[0].createDefaultInstance()],
        actions: [ActionTemplates[0].createDefaultInstance()],
        disabled: false
      };
      if (!properties.accountId) {
        throw new Error("AddMailRule: you must provide an account id.");
      }
      this._rules.push(_.extend(defaults, properties));
      this._saveMailRules();
      return this.trigger();
    };

    MailRulesStore.prototype._onUpdateMailRule = function(id, properties) {
      var existing, key, val;
      existing = _.find(this._rules, function(f) {
        return id === f.id;
      });
      for (key in properties) {
        val = properties[key];
        existing[key] = val;
      }
      this._saveMailRules();
      return this.trigger();
    };

    MailRulesStore.prototype._onDisableMailRule = function(id, reason) {
      var existing, task, _i, _len, _ref1;
      existing = _.find(this._rules, function(f) {
        return id === f.id;
      });
      if (!existing || existing.disabled === true) {
        return;
      }
      Actions.postNotification({
        message: "We were unable to run your mail rules - one or more rules have been disabled.",
        type: "error",
        tag: 'mail-rule-failure',
        sticky: true,
        actions: [
          {
            label: 'Hide',
            dismisses: true,
            id: 'hide'
          }, {
            label: 'View Rules',
            dismisses: true,
            "default": true,
            id: 'mail-rule-failure:view-rules'
          }
        ]
      });
      existing.disabled = true;
      existing.disabledReason = reason;
      this._saveMailRules();
      _ref1 = TaskQueueStatusStore.tasksMatching(ReprocessMailRulesTask, {});
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        task = _ref1[_i];
        Actions.dequeueTask(task.id);
      }
      return this.trigger();
    };

    MailRulesStore.prototype._onNotificationActionTaken = function(_arg) {
      var accountId, action, notification;
      notification = _arg.notification, action = _arg.action;
      if (!NylasEnv.isMainWindow()) {
        return;
      }
      accountId = AccountStore.accounts()[0].accountId;
      if (action.id === 'mail-rule-failure:view-rules') {
        Actions.switchPreferencesTab('Mail Rules', {
          accountId: accountId
        });
        return Actions.openPreferences();
      }
    };

    MailRulesStore.prototype._saveMailRules = function() {
      if (this._saveMailRulesDebounced == null) {
        this._saveMailRulesDebounced = _.debounce((function(_this) {
          return function() {
            DatabaseStore.inTransaction(function(t) {
              return t.persistJSONBlob(RulesJSONBlobKey, _this._rules);
            });
            if (!_.findWhere(_this._rules, {
              disabled: true
            })) {
              return Actions.dismissNotificationsMatching({
                tag: 'mail-rule-failure'
              });
            }
          };
        })(this), 1000);
      }
      return this._saveMailRulesDebounced();
    };

    return MailRulesStore;

  })(NylasStore);

  module.exports = new MailRulesStore();

}).call(this);
