(function() {
  var Actions, CategoryNamesHiddenByDefault, ChangeUnreadTask, DatabaseStore, ExtensionRegistry, FocusedContentStore, FocusedPerspectiveStore, Message, MessageStore, NylasAPI, NylasStore, Thread, Utils, async, deprecate, store, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  NylasStore = require("nylas-store");

  Actions = require("../actions");

  Message = require("../models/message")["default"];

  Thread = require("../models/thread")["default"];

  Utils = require('../models/utils');

  DatabaseStore = require("./database-store");

  FocusedPerspectiveStore = require('./focused-perspective-store');

  FocusedContentStore = require("./focused-content-store");

  ChangeUnreadTask = require('../tasks/change-unread-task')["default"];

  NylasAPI = require('../nylas-api');

  ExtensionRegistry = require('../../extension-registry');

  deprecate = require('../../deprecate-utils').deprecate;

  async = require('async');

  _ = require('underscore');

  CategoryNamesHiddenByDefault = ['spam', 'trash'];

  MessageStore = (function(_super) {
    __extends(MessageStore, _super);

    function MessageStore() {
      this._collapseItem = __bind(this._collapseItem, this);
      this._expandItem = __bind(this._expandItem, this);
      this._onToggleMessageIdExpanded = __bind(this._onToggleMessageIdExpanded, this);
      this._onToggleHiddenMessages = __bind(this._onToggleHiddenMessages, this);
      this._onToggleAllMessagesExpanded = __bind(this._onToggleAllMessagesExpanded, this);
      this._onApplyFocusChange = __bind(this._onApplyFocusChange, this);
      this._onFocusChanged = __bind(this._onFocusChanged, this);
      this._onDataChanged = __bind(this._onDataChanged, this);
      this._onPerspectiveChanged = __bind(this._onPerspectiveChanged, this);
      this._setStoreDefaults = __bind(this._setStoreDefaults, this);
      this.unregisterExtension = __bind(this.unregisterExtension, this);
      this.registerExtension = __bind(this.registerExtension, this);
      this.extensions = __bind(this.extensions, this);
      this.itemsExpandedState = __bind(this.itemsExpandedState, this);
      this._setStoreDefaults();
      this._registerListeners();
    }

    MessageStore.prototype.items = function() {
      var viewing, viewingHiddenCategory;
      if (this._showingHiddenItems) {
        return this._items;
      }
      viewing = FocusedPerspectiveStore.current().categoriesSharedName();
      viewingHiddenCategory = __indexOf.call(CategoryNamesHiddenByDefault, viewing) >= 0;
      if (viewingHiddenCategory) {
        return this._items.filter(function(item) {
          var inHidden;
          inHidden = _.any(item.categories, function(cat) {
            var _ref;
            return _ref = cat.name, __indexOf.call(CategoryNamesHiddenByDefault, _ref) >= 0;
          });
          return inHidden || item.draft === true;
        });
      } else {
        return this._items.filter(function(item) {
          var inHidden;
          inHidden = _.any(item.categories, function(cat) {
            var _ref;
            return _ref = cat.name, __indexOf.call(CategoryNamesHiddenByDefault, _ref) >= 0;
          });
          return !inHidden;
        });
      }
    };

    MessageStore.prototype.threadId = function() {
      var _ref;
      return (_ref = this._thread) != null ? _ref.id : void 0;
    };

    MessageStore.prototype.thread = function() {
      return this._thread;
    };

    MessageStore.prototype.itemsExpandedState = function() {
      return _.clone(this._itemsExpanded);
    };

    MessageStore.prototype.hasCollapsedItems = function() {
      return _.size(this._itemsExpanded) < this._items.length;
    };

    MessageStore.prototype.numberOfHiddenItems = function() {
      return this._items.length - this.items().length;
    };

    MessageStore.prototype.itemClientIds = function() {
      return _.pluck(this._items, "clientId");
    };

    MessageStore.prototype.itemsLoading = function() {
      return this._itemsLoading;
    };


    /*
    Message Store Extensions
     */

    MessageStore.prototype.extensions = function() {
      return ExtensionRegistry.MessageView.extensions();
    };

    MessageStore.prototype.registerExtension = function(ext) {
      return ExtensionRegistry.MessageView.register(ext);
    };

    MessageStore.prototype.unregisterExtension = function(ext) {
      return ExtensionRegistry.MessageView.unregister(ext);
    };

    MessageStore.prototype._onExtensionsChanged = function(role) {
      var MessageBodyProcessor;
      MessageBodyProcessor = require('./message-body-processor')["default"];
      return MessageBodyProcessor.resetCache();
    };

    MessageStore.prototype._setStoreDefaults = function() {
      this._items = [];
      this._itemsExpanded = {};
      this._itemsLoading = false;
      this._showingHiddenItems = false;
      return this._thread = null;
    };

    MessageStore.prototype._registerListeners = function() {
      this.listenTo(ExtensionRegistry.MessageView, this._onExtensionsChanged);
      this.listenTo(DatabaseStore, this._onDataChanged);
      this.listenTo(FocusedContentStore, this._onFocusChanged);
      this.listenTo(Actions.toggleMessageIdExpanded, this._onToggleMessageIdExpanded);
      this.listenTo(Actions.toggleAllMessagesExpanded, this._onToggleAllMessagesExpanded);
      this.listenTo(Actions.toggleHiddenMessages, this._onToggleHiddenMessages);
      return this.listenTo(FocusedPerspectiveStore, this._onPerspectiveChanged);
    };

    MessageStore.prototype._onPerspectiveChanged = function() {
      return this.trigger();
    };

    MessageStore.prototype._onDataChanged = function(change) {
      var inDisplayedThread, item, itemIndex, updatedThread;
      if (!this._thread) {
        return;
      }
      if (change.objectClass === Message.name) {
        inDisplayedThread = _.some(change.objects, (function(_this) {
          return function(obj) {
            return obj.threadId === _this._thread.id;
          };
        })(this));
        if (!inDisplayedThread) {
          return;
        }
        if (change.objects.length === 1 && change.objects[0].draft === true) {
          item = change.objects[0];
          itemIndex = _.findIndex(this._items, function(msg) {
            return msg.id === item.id || msg.clientId === item.clientId;
          });
          if (change.type === 'persist' && itemIndex === -1) {
            this._items = [].concat(this._items, [item]);
            this._items = this._sortItemsForDisplay(this._items);
            this._expandItemsToDefault();
            this.trigger();
            return;
          }
          if (change.type === 'unpersist' && itemIndex !== -1) {
            this._items = [].concat(this._items);
            this._items.splice(itemIndex, 1);
            this._expandItemsToDefault();
            this.trigger();
            return;
          }
        }
        this._fetchFromCache();
      }
      if (change.objectClass === Thread.name) {
        updatedThread = _.find(change.objects, (function(_this) {
          return function(obj) {
            return obj.id === _this._thread.id;
          };
        })(this));
        if (updatedThread) {
          this._thread = updatedThread;
          return this._fetchFromCache();
        }
      }
    };

    MessageStore.prototype._onFocusChanged = function(change) {
      if (!change.impactsCollection('thread')) {
        return;
      }
      if (!this._onFocusChangedTimer) {
        this._onApplyFocusChange();
      } else {
        clearTimeout(this._onFocusChangedTimer);
      }
      return this._onFocusChangedTimer = setTimeout((function(_this) {
        return function() {
          _this._onFocusChangedTimer = null;
          return _this._onApplyFocusChange();
        };
      })(this), 100);
    };

    MessageStore.prototype._onApplyFocusChange = function() {
      var focused, _ref;
      focused = FocusedContentStore.focused('thread');
      if (((_ref = this._thread) != null ? _ref.id : void 0) === (focused != null ? focused.id : void 0)) {
        return;
      }
      this._thread = focused;
      this._items = [];
      this._itemsLoading = true;
      this._showingHiddenItems = false;
      this._itemsExpanded = {};
      this.trigger();
      return this._fetchFromCache();
    };

    MessageStore.prototype._markAsRead = function() {
      var markAsReadDelay, markAsReadId;
      if (!this._thread) {
        return;
      }
      if (this._lastLoadedThreadId === this._thread.id) {
        return;
      }
      this._lastLoadedThreadId = this._thread.id;
      if (this._thread.unread) {
        markAsReadDelay = NylasEnv.config.get('core.reading.markAsReadDelay');
        markAsReadId = this._thread.id;
        if (markAsReadDelay < 0) {
          return;
        }
        return setTimeout((function(_this) {
          return function() {
            var t, _ref;
            if (!(markAsReadId === ((_ref = _this._thread) != null ? _ref.id : void 0) && _this._thread.unread)) {
              return;
            }
            t = new ChangeUnreadTask({
              thread: _this._thread,
              unread: false
            });
            t.canBeUndone = function() {
              return false;
            };
            return Actions.queueTask(t);
          };
        })(this), markAsReadDelay);
      }
    };

    MessageStore.prototype._onToggleAllMessagesExpanded = function() {
      if (this.hasCollapsedItems()) {
        this._items.forEach(this._expandItem);
      } else {
        this._items.slice(0, -1).forEach(this._collapseItem);
      }
      return this.trigger();
    };

    MessageStore.prototype._onToggleHiddenMessages = function() {
      this._showingHiddenItems = !this._showingHiddenItems;
      this._expandItemsToDefault();
      this._fetchExpandedAttachments(this._items);
      return this.trigger();
    };

    MessageStore.prototype._onToggleMessageIdExpanded = function(id) {
      var item;
      item = _.findWhere(this._items, {
        id: id
      });
      if (!item) {
        return;
      }
      if (this._itemsExpanded[id]) {
        this._collapseItem(item);
      } else {
        this._expandItem(item);
      }
      return this.trigger();
    };

    MessageStore.prototype._expandItem = function(item) {
      this._itemsExpanded[item.id] = "explicit";
      return this._fetchExpandedAttachments([item]);
    };

    MessageStore.prototype._collapseItem = function(item) {
      return delete this._itemsExpanded[item.id];
    };

    MessageStore.prototype._fetchFromCache = function(options) {
      var loadedThreadId, query;
      if (options == null) {
        options = {};
      }
      if (!this._thread) {
        return;
      }
      loadedThreadId = this._thread.id;
      query = DatabaseStore.findAll(Message);
      query.where({
        threadId: loadedThreadId
      });
      query.include(Message.attributes.body);
      return query.then((function(_this) {
        return function(items) {
          var loaded, _ref;
          if (loadedThreadId !== ((_ref = _this._thread) != null ? _ref.id : void 0)) {
            return;
          }
          loaded = true;
          _this._items = _this._sortItemsForDisplay(items);
          if (_this._items.length === 0) {
            _this._fetchMessages();
            loaded = false;
          }
          _this._expandItemsToDefault();
          _this._fetchExpandedAttachments(_this._items);
          if (loaded) {
            _this._itemsLoading = false;
            _this._markAsRead();
            return _this.trigger(_this);
          }
        };
      })(this));
    };

    MessageStore.prototype._fetchExpandedAttachments = function(items) {
      var file, item, policy, _i, _len, _results;
      policy = NylasEnv.config.get('core.attachments.downloadPolicy');
      if (policy === 'manually') {
        return;
      }
      _results = [];
      for (_i = 0, _len = items.length; _i < _len; _i++) {
        item = items[_i];
        if (!this._itemsExpanded[item.id]) {
          continue;
        }
        _results.push((function() {
          var _j, _len1, _ref, _results1;
          _ref = item.files;
          _results1 = [];
          for (_j = 0, _len1 = _ref.length; _j < _len1; _j++) {
            file = _ref[_j];
            _results1.push(Actions.fetchFile(file));
          }
          return _results1;
        })());
      }
      return _results;
    };

    MessageStore.prototype._expandItemsToDefault = function() {
      var idx, item, visibleItems, _i, _len, _results;
      visibleItems = this.items();
      _results = [];
      for (idx = _i = 0, _len = visibleItems.length; _i < _len; idx = ++_i) {
        item = visibleItems[idx];
        if (item.unread || item.draft || idx === visibleItems.length - 1) {
          _results.push(this._itemsExpanded[item.id] = "default");
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    MessageStore.prototype._fetchMessages = function() {
      return NylasAPI.getCollection(this._thread.accountId, 'messages', {
        thread_id: this._thread.id
      });
    };

    MessageStore.prototype._sortItemsForDisplay = function(items) {
      var index, item, itemsInReplyTo, other, _i, _j, _k, _len, _len1;
      itemsInReplyTo = [];
      for (index = _i = items.length - 1; _i >= 0; index = _i += -1) {
        item = items[index];
        if (item.draft && item.replyToMessageId) {
          itemsInReplyTo.push(item);
          items.splice(index, 1);
        }
      }
      for (_j = 0, _len = itemsInReplyTo.length; _j < _len; _j++) {
        item = itemsInReplyTo[_j];
        for (index = _k = 0, _len1 = items.length; _k < _len1; index = ++_k) {
          other = items[index];
          if (item.replyToMessageId === other.id) {
            items.splice(index + 1, 0, item);
            item = null;
            break;
          }
        }
        if (item) {
          items.push(item);
        }
      }
      return items;
    };

    return MessageStore;

  })(NylasStore);

  store = new MessageStore();

  store.registerExtension = deprecate('MessageStore.registerExtension', 'ExtensionRegistry.MessageView.register', store, store.registerExtension);

  store.unregisterExtension = deprecate('MessageStore.unregisterExtension', 'ExtensionRegistry.MessageView.unregister', store, store.unregisterExtension);

  store.CategoryNamesHiddenByDefault = CategoryNamesHiddenByDefault;

  module.exports = store;

}).call(this);
