(function() {
  var AccountStore, DatabaseStore, ModelsForSync, NylasStore, NylasSyncStatusStore, Rx, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  Rx = require('rx-lite');

  AccountStore = require('./account-store');

  DatabaseStore = require('./database-store');

  NylasStore = require('nylas-store');

  ModelsForSync = ['threads', 'messages', 'labels', 'folders', 'drafts', 'contacts', 'calendars', 'events'];

  NylasSyncStatusStore = (function(_super) {
    __extends(NylasSyncStatusStore, _super);

    NylasSyncStatusStore.prototype.ModelsForSync = ModelsForSync;

    function NylasSyncStatusStore() {
      this.nextRetryDelay = __bind(this.nextRetryDelay, this);
      this.nextRetryTimestamp = __bind(this.nextRetryTimestamp, this);
      this.connected = __bind(this.connected, this);
      this.busy = __bind(this.busy, this);
      this.whenSyncComplete = __bind(this.whenSyncComplete, this);
      this.isSyncComplete = __bind(this.isSyncComplete, this);
      this.isSyncCompleteForAccount = __bind(this.isSyncCompleteForAccount, this);
      this.state = __bind(this.state, this);
      this._onAccountsChanged = __bind(this._onAccountsChanged, this);
      this._statesByAccount = {};
      this._subscriptions = {};
      this.listenTo(AccountStore, this._onAccountsChanged);
      this._onAccountsChanged();
    }

    NylasSyncStatusStore.prototype._onAccountsChanged = function() {
      return AccountStore.accounts().forEach((function(_this) {
        return function(item) {
          var query, _base, _name;
          query = DatabaseStore.findJSONBlob("NylasSyncWorker:" + item.id);
          return (_base = _this._subscriptions)[_name = item.id] != null ? _base[_name] : _base[_name] = Rx.Observable.fromQuery(query).subscribe(function(json) {
            var state;
            state = _.extend({}, json != null ? json : {});
            delete state.cursor;
            _this._statesByAccount[item.id] = state;
            return _this.trigger();
          });
        };
      })(this));
    };

    NylasSyncStatusStore.prototype.state = function() {
      return this._statesByAccount;
    };

    NylasSyncStatusStore.prototype.isSyncCompleteForAccount = function(acctId, model) {
      var modelState, _model, _ref, _ref1, _ref2;
      if (!this._statesByAccount[acctId]) {
        return false;
      }
      if (model) {
        return (_ref = (_ref1 = this._statesByAccount[acctId][model]) != null ? _ref1.complete : void 0) != null ? _ref : false;
      }
      if (_.isEmpty(this._statesByAccount[acctId])) {
        return false;
      }
      _ref2 = this._statesByAccount[acctId];
      for (_model in _ref2) {
        modelState = _ref2[_model];
        if (__indexOf.call(ModelsForSync, _model) < 0) {
          continue;
        }
        if (!modelState.complete) {
          return false;
        }
      }
      return true;
    };

    NylasSyncStatusStore.prototype.isSyncComplete = function() {
      var acctId;
      if (_.isEmpty(this._statesByAccount)) {
        return false;
      }
      for (acctId in this._statesByAccount) {
        if (!this.isSyncCompleteForAccount(acctId)) {
          return false;
        }
      }
      return true;
    };

    NylasSyncStatusStore.prototype.whenSyncComplete = function() {
      if (this.isSyncComplete()) {
        return Promise.resolve();
      }
      return new Promise((function(_this) {
        return function(resolve) {
          var unsubscribe;
          return unsubscribe = _this.listen(function() {
            if (_this.isSyncComplete()) {
              unsubscribe();
              return resolve();
            }
          });
        };
      })(this));
    };

    NylasSyncStatusStore.prototype.busy = function() {
      var accountId, key, state, states, _ref;
      _ref = this._statesByAccount;
      for (accountId in _ref) {
        states = _ref[accountId];
        for (key in states) {
          state = states[key];
          if (state.busy) {
            return true;
          }
        }
        false;
      }
    };

    NylasSyncStatusStore.prototype.connected = function() {
      var statuses;
      statuses = _.values(this._statesByAccount).map(function(state) {
        return state.longConnectionStatus;
      });
      if (statuses.length === 0) {
        return true;
      }
      return _.any(statuses, function(status) {
        return status !== 'closed';
      });
    };

    NylasSyncStatusStore.prototype.nextRetryTimestamp = function() {
      var retryDates;
      retryDates = _.values(this._statesByAccount).map(function(state) {
        return state.nextRetryTimestamp;
      });
      return _.compact(retryDates).sort((function(_this) {
        return function(a, b) {
          return a < b;
        };
      })(this)).pop();
    };

    NylasSyncStatusStore.prototype.nextRetryDelay = function() {
      var retryDelays;
      retryDelays = _.values(this._statesByAccount).map(function(state) {
        return state.nextRetryDelay;
      });
      return _.compact(retryDelays).sort((function(_this) {
        return function(a, b) {
          return a < b;
        };
      })(this)).pop();
    };

    return NylasSyncStatusStore;

  })(NylasStore);

  module.exports = new NylasSyncStatusStore();

}).call(this);
