(function() {
  var ListTabular, ObservableListDataSource, Rx, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  _ = require('underscore');

  Rx = require('rx-lite');

  ListTabular = require('nylas-component-kit').ListTabular;


  /*
  This class takes an observable which vends QueryResultSets and adapts it so that
  you can make it the data source of a MultiselectList.
  
  When the MultiselectList is refactored to take an Observable, this class should
  go away!
   */

  ObservableListDataSource = (function(_super) {
    __extends(ObservableListDataSource, _super);

    function ObservableListDataSource($resultSetObservable, _setRetainedRange) {
      this.$resultSetObservable = $resultSetObservable;
      this._setRetainedRange = _setRetainedRange;
      this.get = __bind(this.get, this);
      this.empty = __bind(this.empty, this);
      this._attach = __bind(this._attach, this);
      ObservableListDataSource.__super__.constructor.apply(this, arguments);
      this._countEstimate = -1;
      this._resultSet = null;
      this._resultDesiredLast = null;
    }

    ObservableListDataSource.prototype._attach = function() {
      return this._subscription = this.$resultSetObservable.subscribe((function(_this) {
        return function(nextResultSet) {
          var previousResultSet, query;
          if (nextResultSet.range().end === _this._resultDesiredLast) {
            _this._countEstimate = Math.max(_this._countEstimate, nextResultSet.range().end + 1);
          } else {
            _this._countEstimate = nextResultSet.range().end;
          }
          previousResultSet = _this._resultSet;
          _this._resultSet = nextResultSet;
          query = nextResultSet.query();
          if (query) {
            _this.selection.removeItemsNotMatching(query.matchers());
          }
          return _this.trigger({
            previous: previousResultSet,
            next: nextResultSet
          });
        };
      })(this));
    };

    ObservableListDataSource.prototype.setRetainedRange = function(_arg) {
      var end, start;
      start = _arg.start, end = _arg.end;
      this._resultDesiredLast = end;
      this._setRetainedRange({
        start: start,
        end: end
      });
      if (!this._subscription) {
        return this._attach();
      }
    };

    ObservableListDataSource.prototype.count = function() {
      return this._countEstimate;
    };

    ObservableListDataSource.prototype.loaded = function() {
      return this._resultSet !== null;
    };

    ObservableListDataSource.prototype.empty = function() {
      return !this._resultSet || this._resultSet.empty();
    };

    ObservableListDataSource.prototype.get = function(offset) {
      if (!this._resultSet) {
        return null;
      }
      return this._resultSet.modelAtOffset(offset);
    };

    ObservableListDataSource.prototype.getById = function(id) {
      return this._resultSet.modelWithId(id);
    };

    ObservableListDataSource.prototype.indexOfId = function(id) {
      if (!(this._resultSet && id)) {
        return -1;
      }
      return this._resultSet.offsetOfId(id);
    };

    ObservableListDataSource.prototype.itemsCurrentlyInViewMatching = function(matchFn) {
      if (!this._resultSet) {
        return [];
      }
      return this._resultSet.models().filter(matchFn);
    };

    ObservableListDataSource.prototype.cleanup = function() {
      var _ref;
      if ((_ref = this._subscription) != null) {
        _ref.dispose();
      }
      return ObservableListDataSource.__super__.cleanup.apply(this, arguments);
    };

    return ObservableListDataSource;

  })(ListTabular.DataSource);

  module.exports = ObservableListDataSource;

}).call(this);
