(function() {
  var CategoryDatabaseMutationObserver, DatabaseStore, NylasStore, ReadCountsQuery, Reflux, SetCountsQuery, Thread, ThreadCountsStore, UpdateCountsQuery, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Reflux = require('reflux');

  _ = require('underscore');

  NylasStore = require('nylas-store');

  DatabaseStore = require('./database-store');

  Thread = require('../models/thread')["default"];


  /*
  Are running two nested SELECT statements really the best option? Yup.
  For a performance assessment of these queries and other options, see:
  https://gist.github.com/bengotow/c8b5cd8989c9149ded56
  
  Note: SUM(unread) works because unread is represented as an int: 0 or 1.
   */

  ReadCountsQuery = function() {
    return "SELECT * FROM `ThreadCounts`";
  };

  SetCountsQuery = function() {
    return "REPLACE INTO `ThreadCounts` (category_id, unread, total)\nSELECT\n  `ThreadCategory`.`value` as category_id,\n  SUM(`ThreadCategory`.`unread`) as unread,\n  COUNT(*) as total\nFROM `ThreadCategory`\nWHERE\n  `ThreadCategory`.in_all_mail = 1\nGROUP BY `ThreadCategory`.`value`;";
  };

  UpdateCountsQuery = function(objectIds, operator) {
    var objectIdsString;
    objectIdsString = "'" + objectIds.join("','") + "'";
    return "REPLACE INTO `ThreadCounts` (category_id, unread, total)\nSELECT\n  `ThreadCategory`.`value` as category_id,\n  COALESCE((SELECT unread FROM `ThreadCounts` WHERE category_id = `ThreadCategory`.`value`), 0) " + operator + " SUM(`ThreadCategory`.`unread`) as unread,\n  COALESCE((SELECT total  FROM `ThreadCounts` WHERE category_id = `ThreadCategory`.`value`), 0) " + operator + " COUNT(*) as total\nFROM `ThreadCategory`\nWHERE\n  `ThreadCategory`.id IN (" + objectIdsString + ") AND\n  `ThreadCategory`.in_all_mail = 1\nGROUP BY `ThreadCategory`.`value`";
  };

  CategoryDatabaseMutationObserver = (function() {
    function CategoryDatabaseMutationObserver() {
      this.afterDatabaseChange = __bind(this.afterDatabaseChange, this);
      this.beforeDatabaseChange = __bind(this.beforeDatabaseChange, this);
    }

    CategoryDatabaseMutationObserver.prototype.beforeDatabaseChange = function(query, _arg) {
      var objectClass, objectIds, objects, type;
      type = _arg.type, objects = _arg.objects, objectIds = _arg.objectIds, objectClass = _arg.objectClass;
      if (objectClass === Thread.name) {
        return query(UpdateCountsQuery(objectIds, '-'));
      } else {
        return Promise.resolve();
      }
    };

    CategoryDatabaseMutationObserver.prototype.afterDatabaseChange = function(query, _arg, beforeResolveValue) {
      var objectClass, objectIds, objects, type;
      type = _arg.type, objects = _arg.objects, objectIds = _arg.objectIds, objectClass = _arg.objectClass;
      if (objectClass === Thread.name) {
        return query(UpdateCountsQuery(objectIds, '+'));
      } else {
        return Promise.resolve();
      }
    };

    return CategoryDatabaseMutationObserver;

  })();

  ThreadCountsStore = (function(_super) {
    __extends(ThreadCountsStore, _super);

    ThreadCountsStore.prototype.CategoryDatabaseMutationObserver = CategoryDatabaseMutationObserver;

    function ThreadCountsStore() {
      this.totalCountForCategoryId = __bind(this.totalCountForCategoryId, this);
      this.unreadCountForCategoryId = __bind(this.unreadCountForCategoryId, this);
      this._onCountsChanged = __bind(this._onCountsChanged, this);
      this.reset = __bind(this.reset, this);
      this._counts = {};
      this._observer = new CategoryDatabaseMutationObserver();
      DatabaseStore.addMutationHook(this._observer);
      if (NylasEnv.isMainWindow()) {
        this._onCountsChangedDebounced = _.throttle(this._onCountsChanged, 1000);
        DatabaseStore.listen((function(_this) {
          return function(change) {
            if (change.objectClass === Thread.name) {
              return _this._onCountsChangedDebounced();
            }
          };
        })(this));
        this._onCountsChangedDebounced();
      }
      if (NylasEnv.isWorkWindow() && !NylasEnv.config.get('nylas.threadCountsValid')) {
        this.reset();
      }
    }

    ThreadCountsStore.prototype.reset = function() {
      var countsStartTime;
      countsStartTime = null;
      return DatabaseStore.inTransaction((function(_this) {
        return function(t) {
          countsStartTime = Date.now();
          return DatabaseStore._query(SetCountsQuery());
        };
      })(this)).then((function(_this) {
        return function() {
          NylasEnv.config.set('nylas.threadCountsValid', true);
          return console.log("Recomputed all thread counts in " + (Date.now() - countsStartTime) + "ms");
        };
      })(this));
    };

    ThreadCountsStore.prototype._onCountsChanged = function() {
      return DatabaseStore._query(ReadCountsQuery()).then((function(_this) {
        return function(results) {
          var category_id, foundNegative, total, unread, _i, _len, _ref;
          _this._counts = {};
          foundNegative = false;
          for (_i = 0, _len = results.length; _i < _len; _i++) {
            _ref = results[_i], category_id = _ref.category_id, unread = _ref.unread, total = _ref.total;
            _this._counts[category_id] = {
              unread: unread,
              total: total
            };
            if (unread < 0 || total < 0) {
              foundNegative = true;
            }
          }
          if (foundNegative) {
            NylasEnv.reportError(new Error('Assertion Failure: Negative Count'));
            _this.reset();
            return;
          }
          return _this.trigger();
        };
      })(this));
    };

    ThreadCountsStore.prototype.unreadCountForCategoryId = function(catId) {
      if (this._counts[catId] === void 0) {
        return null;
      }
      return this._counts[catId]['unread'];
    };

    ThreadCountsStore.prototype.totalCountForCategoryId = function(catId) {
      if (this._counts[catId] === void 0) {
        return null;
      }
      return this._counts[catId]['total'];
    };

    return ThreadCountsStore;

  })(NylasStore);

  module.exports = new ThreadCountsStore;

}).call(this);
