(function() {
  var AccountStore, Actions, CategoryStore, Location, MailboxPerspective, NylasStore, Sheet, WorkspaceStore, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  Actions = require('../actions');

  AccountStore = require('./account-store');

  CategoryStore = require('./category-store');

  MailboxPerspective = require('../../mailbox-perspective');

  NylasStore = require('nylas-store');

  Sheet = {};

  Location = {};


  /*
  Public: The WorkspaceStore manages Sheets and layout modes in the application.
  Observing the WorkspaceStore makes it easy to monitor the sheet stack. To learn
  more about sheets and layout in N1, see the {InterfaceConcepts.md}
  documentation.
  
  Section: Stores
   */

  WorkspaceStore = (function(_super) {
    __extends(WorkspaceStore, _super);

    function WorkspaceStore() {
      this.popToRootSheet = __bind(this.popToRootSheet, this);
      this.popSheet = __bind(this.popSheet, this);
      this.pushSheet = __bind(this.pushSheet, this);
      this.undefineSheet = __bind(this.undefineSheet, this);
      this.defineSheet = __bind(this.defineSheet, this);
      this.isLocationHidden = __bind(this.isLocationHidden, this);
      this.hiddenLocations = __bind(this.hiddenLocations, this);
      this.sheetStack = __bind(this.sheetStack, this);
      this.rootSheet = __bind(this.rootSheet, this);
      this.topSheet = __bind(this.topSheet, this);
      this.preferredLayoutMode = __bind(this.preferredLayoutMode, this);
      this.layoutMode = __bind(this.layoutMode, this);
      this._onSelectLayoutMode = __bind(this._onSelectLayoutMode, this);
      this._onSetFocus = __bind(this._onSetFocus, this);
      this._onToggleLocationHidden = __bind(this._onToggleLocationHidden, this);
      this._onSelectRootSheet = __bind(this._onSelectRootSheet, this);
      this._resetInstanceVars = __bind(this._resetInstanceVars, this);
      this._rebuildMenu = __bind(this._rebuildMenu, this);
      var windowType;
      this._resetInstanceVars();
      this._preferredLayoutMode = NylasEnv.config.get('core.workspace.mode');
      this.listenTo(Actions.selectRootSheet, this._onSelectRootSheet);
      this.listenTo(Actions.setFocus, this._onSetFocus);
      this.listenTo(Actions.toggleWorkspaceLocationHidden, this._onToggleLocationHidden);
      this.listenTo(Actions.popSheet, this.popSheet);
      this.listenTo(Actions.popToRootSheet, this.popToRootSheet);
      this.listenTo(Actions.pushSheet, this.pushSheet);
      windowType = NylasEnv.getLoadSettings().windowType;
      if (windowType !== 'onboarding') {
        require('electron').webFrame.setZoomLevelLimits(1, 1);
        NylasEnv.config.observe('core.workspace.interfaceZoom', (function(_this) {
          return function(z) {
            if (z && _.isNumber(z)) {
              return require('electron').webFrame.setZoomFactor(z);
            }
          };
        })(this));
      }
      if (NylasEnv.isMainWindow()) {
        this._rebuildMenu();
        NylasEnv.commands.add(document.body, {
          'core:pop-sheet': (function(_this) {
            return function() {
              return _this.popSheet();
            };
          })(this),
          'application:select-list-mode': (function(_this) {
            return function() {
              return _this._onSelectLayoutMode("list");
            };
          })(this),
          'application:select-split-mode': (function(_this) {
            return function() {
              return _this._onSelectLayoutMode("split");
            };
          })(this)
        });
      }
    }

    WorkspaceStore.prototype._rebuildMenu = function() {
      var _ref;
      if ((_ref = this._menuDisposable) != null) {
        _ref.dispose();
      }
      return this._menuDisposable = NylasEnv.menu.add([
        {
          "label": "View",
          "submenu": [
            {
              "label": "Single Panel Mode",
              "type": "radio",
              "command": "application:select-list-mode",
              "checked": this._preferredLayoutMode === 'list',
              "position": "before=mailbox-navigation"
            }, {
              "label": "Two Panel Mode",
              "type": "radio",
              "command": "application:select-split-mode",
              "checked": this._preferredLayoutMode === 'split',
              "position": "before=mailbox-navigation"
            }
          ]
        }
      ]);
    };

    WorkspaceStore.prototype._resetInstanceVars = function() {
      this.Location = Location = {};
      this.Sheet = Sheet = {};
      this._hiddenLocations = NylasEnv.config.get('core.workspace.hiddenLocations') || {};
      this._sheetStack = [];
      if (NylasEnv.isMainWindow()) {
        this.defineSheet('Global');
        this.defineSheet('Threads', {
          root: true
        }, {
          list: ['RootSidebar', 'ThreadList'],
          split: ['RootSidebar', 'ThreadList', 'MessageList', 'MessageListSidebar']
        });
        return this.defineSheet('Thread', {}, {
          list: ['MessageList', 'MessageListSidebar']
        });
      } else {
        return this.defineSheet('Global');
      }
    };


    /*
    Inbound Events
     */

    WorkspaceStore.prototype._onSelectRootSheet = function(sheet) {
      if (!sheet) {
        throw new Error("Actions.selectRootSheet - " + sheet + " is not a valid sheet.");
      }
      if (!sheet.root) {
        throw new Error("Actions.selectRootSheet - " + sheet + " is not registered as a root sheet.");
      }
      this._sheetStack = [];
      this._sheetStack.push(sheet);
      return this.trigger(this);
    };

    WorkspaceStore.prototype._onToggleLocationHidden = function(location) {
      if (!location.id) {
        throw new Error("Actions.toggleWorkspaceLocationHidden - pass a WorkspaceStore.Location");
      }
      if (this._hiddenLocations[location.id]) {
        delete this._hiddenLocations[location.id];
      } else {
        this._hiddenLocations[location.id] = location;
      }
      NylasEnv.config.set('core.workspace.hiddenLocations', this._hiddenLocations);
      return this.trigger(this);
    };

    WorkspaceStore.prototype._onSetFocus = function(_arg) {
      var collection, item;
      collection = _arg.collection, item = _arg.item;
      if (collection === 'thread') {
        if (this.layoutMode() === 'list') {
          if (item && this.topSheet() !== Sheet.Thread) {
            this.pushSheet(Sheet.Thread);
          }
          if (!item && this.topSheet() === Sheet.Thread) {
            this.popSheet();
          }
        }
      }
      if (collection === 'file') {
        if (this.layoutMode() === 'list') {
          if (item && this.topSheet() !== Sheet.File) {
            this.pushSheet(Sheet.File);
          }
          if (!item && this.topSheet() === Sheet.File) {
            return this.popSheet();
          }
        }
      }
    };

    WorkspaceStore.prototype._onSelectLayoutMode = function(mode) {
      if (mode === this._preferredLayoutMode) {
        return;
      }
      this._preferredLayoutMode = mode;
      NylasEnv.config.set('core.workspace.mode', this._preferredLayoutMode);
      this._rebuildMenu();
      this.popToRootSheet();
      return this.trigger();
    };


    /*
    Accessing Data
     */

    WorkspaceStore.prototype.layoutMode = function() {
      var root, _ref;
      root = this.rootSheet();
      if (!root) {
        return 'list';
      } else if (_ref = this._preferredLayoutMode, __indexOf.call(root.supportedModes, _ref) >= 0) {
        return this._preferredLayoutMode;
      } else {
        return root.supportedModes[0];
      }
    };

    WorkspaceStore.prototype.preferredLayoutMode = function() {
      return this._preferredLayoutMode;
    };

    WorkspaceStore.prototype.topSheet = function() {
      return this._sheetStack[this._sheetStack.length - 1];
    };

    WorkspaceStore.prototype.rootSheet = function() {
      return this._sheetStack[0];
    };

    WorkspaceStore.prototype.sheetStack = function() {
      return this._sheetStack;
    };

    WorkspaceStore.prototype.hiddenLocations = function() {
      return _.values(this._hiddenLocations);
    };

    WorkspaceStore.prototype.isLocationHidden = function(loc) {
      if (!loc) {
        return false;
      }
      return this._hiddenLocations[loc.id] != null;
    };


    /*
    Managing Sheets
     */

    WorkspaceStore.prototype.defineSheet = function(id, options, columns) {
      var col, cols, idx, layout, _i, _len;
      if (options == null) {
        options = {};
      }
      if (columns == null) {
        columns = {};
      }
      for (layout in columns) {
        cols = columns[layout];
        for (idx = _i = 0, _len = cols.length; _i < _len; idx = ++_i) {
          col = cols[idx];
          if (Location[col] == null) {
            Location[col] = {
              id: "" + col,
              Toolbar: {
                id: "" + col + ":Toolbar"
              }
            };
          }
          cols[idx] = Location[col];
        }
      }
      Sheet[id] = {
        id: id,
        columns: columns,
        supportedModes: Object.keys(columns),
        icon: options.icon,
        name: options.name,
        root: options.root,
        sidebarComponent: options.sidebarComponent,
        Toolbar: {
          Left: {
            id: "Sheet:" + id + ":Toolbar:Left"
          },
          Right: {
            id: "Sheet:" + id + ":Toolbar:Right"
          }
        },
        Header: {
          id: "Sheet:" + id + ":Header"
        },
        Footer: {
          id: "Sheet:" + id + ":Footer"
        }
      };
      if ((options.root && !this.rootSheet()) && !options.silent) {
        this._onSelectRootSheet(Sheet[id]);
      }
      return this.triggerDebounced();
    };

    WorkspaceStore.prototype.undefineSheet = function(id) {
      delete Sheet[id];
      return this.triggerDebounced();
    };

    WorkspaceStore.prototype.pushSheet = function(sheet) {
      this._sheetStack.push(sheet);
      return this.trigger();
    };

    WorkspaceStore.prototype.popSheet = function() {
      var sheet;
      sheet = this.topSheet();
      if (this._sheetStack.length > 1) {
        this._sheetStack.pop();
        this.trigger();
      }
      if (Sheet.Thread && sheet === Sheet.Thread) {
        return Actions.setFocus({
          collection: 'thread',
          item: null
        });
      }
    };

    WorkspaceStore.prototype.popToRootSheet = function() {
      if (this._sheetStack.length > 1) {
        this._sheetStack.length = 1;
        return this.trigger();
      }
    };

    WorkspaceStore.prototype.triggerDebounced = _.debounce((function() {
      return this.trigger(this);
    }), 1);

    return WorkspaceStore;

  })(NylasStore);

  module.exports = new WorkspaceStore();

}).call(this);
