(function() {
  var Category, CategoryObservables, CategoryOperators, DatabaseStore, QuerySubscriptionPool, Rx, _,
    __slice = [].slice;

  Rx = require('rx-lite');

  _ = require('underscore');

  Category = require('../flux/models/category');

  QuerySubscriptionPool = require('../flux/models/query-subscription-pool');

  DatabaseStore = require('../flux/stores/database-store');

  CategoryOperators = {
    sort: function() {
      var obs;
      obs = this.map(function(categories) {
        return categories.sort(function(catA, catB) {
          var nameA, nameB;
          nameA = catA.displayName;
          nameB = catB.displayName;
          if (nameA[0] === '[') {
            nameA = "ZZZ" + nameA;
          }
          if (nameB[0] === '[') {
            nameB = "ZZZ" + nameB;
          }
          return nameA.localeCompare(nameB);
        });
      });
      return _.extend(obs, CategoryOperators);
    },
    categoryFilter: function(filter) {
      var obs;
      obs = this.map(function(categories) {
        return categories.filter(filter);
      });
      return _.extend(obs, CategoryOperators);
    }
  };

  CategoryObservables = {
    forAllAccounts: (function(_this) {
      return function() {
        var observable;
        observable = Rx.Observable.fromQuery(DatabaseStore.findAll(Category));
        _.extend(observable, CategoryOperators);
        return observable;
      };
    })(this),
    forAccount: (function(_this) {
      return function(account) {
        var observable;
        if (account) {
          observable = Rx.Observable.fromQuery(DatabaseStore.findAll(Category).where({
            accountId: account.id
          }));
        } else {
          observable = Rx.Observable.fromQuery(DatabaseStore.findAll(Category));
        }
        _.extend(observable, CategoryOperators);
        return observable;
      };
    })(this),
    standard: (function(_this) {
      return function(account) {
        var observable;
        observable = Rx.Observable.fromConfig('core.workspace.showImportant').flatMapLatest(function(showImportant) {
          return CategoryObservables.forAccount(account).sort().categoryFilter(function(cat) {
            return cat.isStandardCategory(showImportant);
          });
        });
        _.extend(observable, CategoryOperators);
        return observable;
      };
    })(this),
    user: (function(_this) {
      return function(account) {
        return CategoryObservables.forAccount(account).sort().categoryFilter(function(cat) {
          return cat.isUserCategory();
        });
      };
    })(this),
    hidden: (function(_this) {
      return function(account) {
        return CategoryObservables.forAccount(account).sort().categoryFilter(function(cat) {
          return cat.isHiddenCategory();
        });
      };
    })(this)
  };

  module.exports = {
    Categories: CategoryObservables
  };

  Rx.Observable.fromStore = (function(_this) {
    return function(store) {
      return Rx.Observable.create(function(observer) {
        var unsubscribe;
        unsubscribe = store.listen(function() {
          return observer.onNext(store);
        });
        observer.onNext(store);
        return Rx.Disposable.create(unsubscribe);
      });
    };
  })(this);

  Rx.Observable.fromListSelection = (function(_this) {
    return function(originStore) {
      return Rx.Observable.create(function(observer) {
        var dataSourceDisposable, disposable, dispose, storeObservable;
        dataSourceDisposable = null;
        storeObservable = Rx.Observable.fromStore(originStore);
        disposable = storeObservable.subscribe(function() {
          var dataSource, dataSourceObservable;
          dataSource = originStore.dataSource();
          dataSourceObservable = Rx.Observable.fromStore(dataSource);
          if (dataSourceDisposable) {
            dataSourceDisposable.dispose();
          }
          dataSourceDisposable = dataSourceObservable.subscribe(function() {
            return observer.onNext(dataSource.selection.items());
          });
        });
        dispose = function() {
          if (dataSourceDisposable) {
            dataSourceDisposable.dispose();
          }
          return disposable.dispose();
        };
        return Rx.Disposable.create(dispose);
      });
    };
  })(this);

  Rx.Observable.fromConfig = (function(_this) {
    return function(configKey) {
      return Rx.Observable.create(function(observer) {
        var disposable;
        disposable = NylasEnv.config.onDidChange(configKey, function() {
          return observer.onNext(NylasEnv.config.get(configKey));
        });
        observer.onNext(NylasEnv.config.get(configKey));
        return Rx.Disposable.create(disposable.dispose);
      });
    };
  })(this);

  Rx.Observable.fromAction = (function(_this) {
    return function(action) {
      return Rx.Observable.create(function(observer) {
        var unsubscribe;
        unsubscribe = action.listen(function() {
          var args;
          args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
          return observer.onNext.apply(observer, args);
        });
        return Rx.Disposable.create(unsubscribe);
      });
    };
  })(this);

  Rx.Observable.fromQuery = (function(_this) {
    return function(query) {
      return Rx.Observable.create(function(observer) {
        var unsubscribe;
        unsubscribe = QuerySubscriptionPool.add(query, function(result) {
          return observer.onNext(result);
        });
        return Rx.Disposable.create(unsubscribe);
      });
    };
  })(this);

  Rx.Observable.fromNamedQuerySubscription = (function(_this) {
    return function(name, subscription) {
      return Rx.Observable.create(function(observer) {
        var unsubscribe;
        unsubscribe = QuerySubscriptionPool.addPrivateSubscription(name, subscription, function(result) {
          return observer.onNext(result);
        });
        return Rx.Disposable.create(unsubscribe);
      });
    };
  })(this);

}).call(this);
