(function() {
  var LaunchServicesLinux, LaunchServicesMac, LaunchServicesUnavailable, bundleIdentifier, exec, fs;

  exec = require('child_process').exec;

  fs = require('fs');

  bundleIdentifier = 'com.nylas.nylas-mail';

  LaunchServicesUnavailable = (function() {
    function LaunchServicesUnavailable() {}

    LaunchServicesUnavailable.prototype.available = function() {
      return false;
    };

    LaunchServicesUnavailable.prototype.isRegisteredForURLScheme = function(scheme, callback) {
      throw new Error("isRegisteredForURLScheme is not available");
    };

    LaunchServicesUnavailable.prototype.resetURLScheme = function(scheme, callback) {
      throw new Error("resetURLScheme is not available");
    };

    LaunchServicesUnavailable.prototype.registerForURLScheme = function(scheme, callback) {
      throw new Error("registerForURLScheme is not available");
    };

    return LaunchServicesUnavailable;

  })();

  LaunchServicesLinux = (function() {
    function LaunchServicesLinux() {}

    LaunchServicesLinux.prototype.available = function() {
      return true;
    };

    LaunchServicesLinux.prototype.isRegisteredForURLScheme = function(scheme, callback) {
      if (!callback) {
        throw new Error("isRegisteredForURLScheme is async, provide a callback");
      }
      return exec("xdg-mime query default x-scheme-handler/" + scheme, function(err, stdout, stderr) {
        if (callback && err) {
          return callback(err);
        }
        return callback(stdout.trim() === 'nylas.desktop');
      });
    };

    LaunchServicesLinux.prototype.resetURLScheme = function(scheme, callback) {
      return exec("xdg-mime default thunderbird.desktop x-scheme-handler/" + scheme, function(err, stdout, stderr) {
        if (callback && err) {
          return callback(err);
        }
        if (callback) {
          return callback(null, null);
        }
      });
    };

    LaunchServicesLinux.prototype.registerForURLScheme = function(scheme, callback) {
      return exec("xdg-mime default nylas.desktop x-scheme-handler/" + scheme, function(err, stdout, stderr) {
        if (callback && err) {
          return callback(err);
        }
        if (callback) {
          return callback(null, null);
        }
      });
    };

    return LaunchServicesLinux;

  })();

  LaunchServicesMac = (function() {
    function LaunchServicesMac() {
      this.secure = false;
    }

    LaunchServicesMac.prototype.available = function() {
      return true;
    };

    LaunchServicesMac.prototype.getLaunchServicesPlistPath = function(callback) {
      var insecure, secure;
      secure = "" + process.env.HOME + "/Library/Preferences/com.apple.LaunchServices/com.apple.launchservices.secure.plist";
      insecure = "" + process.env.HOME + "/Library/Preferences/com.apple.LaunchServices.plist";
      return fs.exists(secure, function(exists) {
        if (exists) {
          return callback(secure);
        } else {
          return callback(insecure);
        }
      });
    };

    LaunchServicesMac.prototype.readDefaults = function(callback) {
      return this.getLaunchServicesPlistPath(function(plistPath) {
        var tmpPath;
        tmpPath = "" + plistPath + "." + (Math.random());
        return exec("plutil -convert json \"" + plistPath + "\" -o \"" + tmpPath + "\"", function(err, stdout, stderr) {
          if (callback && err) {
            return callback(err);
          }
          return fs.readFile(tmpPath, function(err, data) {
            var e;
            if (callback && err) {
              return callback(err);
            }
            try {
              data = JSON.parse(data);
              callback(data['LSHandlers'], data);
              return fs.unlink(tmpPath);
            } catch (_error) {
              e = _error;
              if (callback && err) {
                return callback(e);
              }
            }
          });
        });
      });
    };

    LaunchServicesMac.prototype.writeDefaults = function(defaults, callback) {
      return this.getLaunchServicesPlistPath(function(plistPath) {
        var tmpPath;
        tmpPath = "" + plistPath + "." + (Math.random());
        return exec("plutil -convert json \"" + plistPath + "\" -o \"" + tmpPath + "\"", function(err, stdout, stderr) {
          var data, error;
          if (callback && err) {
            return callback(err);
          }
          try {
            data = fs.readFileSync(tmpPath);
            data = JSON.parse(data);
            data['LSHandlers'] = defaults;
            data = JSON.stringify(data);
            fs.writeFileSync(tmpPath, data);
          } catch (_error) {
            error = _error;
            if (callback && error) {
              return callback(error);
            }
          }
          return exec("plutil -convert binary1 \"" + tmpPath + "\" -o \"" + plistPath + "\"", function() {
            fs.unlink(tmpPath);
            return exec("/System/Library/Frameworks/CoreServices.framework/Frameworks/LaunchServices.framework/Support/lsregister -kill -r -domain local -domain system -domain user", function(err, stdout, stderr) {
              if (callback) {
                return callback(err);
              }
            });
          });
        });
      });
    };

    LaunchServicesMac.prototype.isRegisteredForURLScheme = function(scheme, callback) {
      if (!callback) {
        throw new Error("isRegisteredForURLScheme is async, provide a callback");
      }
      return this.readDefaults(function(defaults) {
        var def, _i, _len;
        for (_i = 0, _len = defaults.length; _i < _len; _i++) {
          def = defaults[_i];
          if (def.LSHandlerURLScheme === scheme) {
            return callback(def.LSHandlerRoleAll === bundleIdentifier);
          }
        }
        return callback(false);
      });
    };

    LaunchServicesMac.prototype.resetURLScheme = function(scheme, callback) {
      return this.readDefaults((function(_this) {
        return function(defaults) {
          var ii, _i, _ref;
          for (ii = _i = _ref = defaults.length - 1; _i >= 0; ii = _i += -1) {
            if (defaults[ii].LSHandlerURLScheme === scheme) {
              defaults.splice(ii, 1);
            }
          }
          return _this.writeDefaults(defaults, callback);
        };
      })(this));
    };

    LaunchServicesMac.prototype.registerForURLScheme = function(scheme, callback) {
      return this.readDefaults((function(_this) {
        return function(defaults) {
          var ii, _i, _ref;
          for (ii = _i = _ref = defaults.length - 1; _i >= 0; ii = _i += -1) {
            if (defaults[ii].LSHandlerURLScheme === scheme) {
              defaults.splice(ii, 1);
            }
          }
          defaults.push({
            LSHandlerURLScheme: scheme,
            LSHandlerRoleAll: bundleIdentifier
          });
          return _this.writeDefaults(defaults, callback);
        };
      })(this));
    };

    return LaunchServicesMac;

  })();

  module.exports = LaunchServicesUnavailable;

  if (process.platform === 'darwin') {
    module.exports = LaunchServicesMac;
  } else if (process.platform === 'linux') {
    module.exports = LaunchServicesLinux;
  }

  module.exports.LaunchServicesMac = LaunchServicesMac;

  module.exports.LaunchServicesLinux = LaunchServicesLinux;

  module.exports.LaunchServicesUnavailable = LaunchServicesUnavailable;

}).call(this);
