(function() {
  var ActionTemplates, ConditionTemplates, NylasObservables, Template, _;

  _ = require('underscore');

  NylasObservables = require('nylas-observables');

  Template = require('./components/scenario-editor-models').Template;

  ConditionTemplates = [
    new Template('from', Template.Type.String, {
      name: 'From',
      valueForMessage: function(message) {
        return _.pluck(message.from, 'email');
      }
    }), new Template('to', Template.Type.String, {
      name: 'To',
      valueForMessage: function(message) {
        return _.pluck(message.to, 'email');
      }
    }), new Template('cc', Template.Type.String, {
      name: 'Cc',
      valueForMessage: function(message) {
        return _.pluck(message.cc, 'email');
      }
    }), new Template('bcc', Template.Type.String, {
      name: 'Bcc',
      valueForMessage: function(message) {
        return _.pluck(message.bcc, 'email');
      }
    }), new Template('anyRecipient', Template.Type.String, {
      name: 'Any Recipient',
      valueForMessage: function(message) {
        var recipients;
        recipients = [].concat(message.to, message.cc, message.bcc, message.from);
        return _.pluck(recipients, 'email');
      }
    }), new Template('anyAttachmentName', Template.Type.String, {
      name: 'Any attachment name',
      valueForMessage: function(message) {
        return _.pluck(message.files, 'filename');
      }
    }), new Template('starred', Template.Type.Enum, {
      name: 'Starred',
      values: [
        {
          name: 'True',
          value: 'true'
        }, {
          name: 'False',
          value: 'false'
        }
      ],
      valueLabel: 'is:',
      valueForMessage: function(message) {
        if (message.starred) {
          return 'true';
        } else {
          return 'false';
        }
      }
    }), new Template('subject', Template.Type.String, {
      name: 'Subject',
      valueForMessage: function(message) {
        return message.subject;
      }
    }), new Template('body', Template.Type.String, {
      name: 'Body',
      valueForMessage: function(message) {
        return message.body;
      }
    })
  ];

  ActionTemplates = [
    new Template('markAsRead', Template.Type.None, {
      name: 'Mark as Read'
    }), new Template('moveToTrash', Template.Type.None, {
      name: 'Move to Trash'
    }), new Template('star', Template.Type.None, {
      name: 'Star'
    })
  ];

  module.exports = {
    ConditionMode: {
      Any: 'any',
      All: 'all'
    },
    ConditionTemplates: ConditionTemplates,
    ConditionTemplatesForAccount: function(account) {
      if (!account) {
        return [];
      }
      return ConditionTemplates;
    },
    ActionTemplates: ActionTemplates,
    ActionTemplatesForAccount: function(account) {
      var CategoryNamesObservable, templates;
      if (!account) {
        return [];
      }
      templates = [].concat(ActionTemplates);
      CategoryNamesObservable = NylasObservables.Categories.forAccount(account).sort().map(function(cats) {
        return cats.filter(function(cat) {
          return !cat.isLockedCategory();
        });
      }).map(function(cats) {
        return cats.map(function(cat) {
          return {
            name: cat.displayName || cat.name,
            value: cat.id
          };
        });
      });
      if (account.usesLabels()) {
        templates.unshift(new Template('markAsImportant', Template.Type.None, {
          name: 'Mark as Important'
        }));
        templates.unshift(new Template('applyLabelArchive', Template.Type.None, {
          name: 'Archive'
        }));
        templates.unshift(new Template('applyLabel', Template.Type.Enum, {
          name: 'Apply Label',
          values: CategoryNamesObservable
        }));
        templates.unshift(new Template('moveToLabel', Template.Type.Enum, {
          name: 'Move to Label',
          values: CategoryNamesObservable
        }));
      } else {
        templates.push(new Template('changeFolder', Template.Type.Enum, {
          name: 'Move Message',
          valueLabel: 'to folder:',
          values: CategoryNamesObservable
        }));
      }
      return templates;
    }
  };

}).call(this);
