(function() {
  var AccountStore, Actions, Category, CategoryMailboxPerspective, CategoryStore, ChangeUnreadTask, DatabaseStore, DraftsMailboxPerspective, EmptyMailboxPerspective, MailboxPerspective, Matcher, MutableQuerySubscription, OutboxStore, RecentlyReadStore, StarredMailboxPerspective, TaskFactory, Thread, ThreadCountsStore, UnreadMailboxPerspective, UnreadQuerySubscription, Utils, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __slice = [].slice,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  Utils = require('./flux/models/utils');

  TaskFactory = require('./flux/tasks/task-factory')["default"];

  AccountStore = require('./flux/stores/account-store');

  CategoryStore = require('./flux/stores/category-store');

  DatabaseStore = require('./flux/stores/database-store');

  OutboxStore = require('./flux/stores/outbox-store')["default"];

  ThreadCountsStore = require('./flux/stores/thread-counts-store');

  RecentlyReadStore = require('./flux/stores/recently-read-store')["default"];

  MutableQuerySubscription = require('./flux/models/mutable-query-subscription')["default"];

  UnreadQuerySubscription = require('./flux/models/unread-query-subscription')["default"];

  Matcher = require('./flux/attributes/matcher');

  Thread = require('./flux/models/thread')["default"];

  Category = require('./flux/models/category');

  Actions = require('./flux/actions');

  ChangeUnreadTask = null;

  MailboxPerspective = (function() {
    MailboxPerspective.forNothing = function() {
      return new EmptyMailboxPerspective();
    };

    MailboxPerspective.forDrafts = function(accountsOrIds) {
      return new DraftsMailboxPerspective(accountsOrIds);
    };

    MailboxPerspective.forCategory = function(category) {
      if (!category) {
        return this.forNothing();
      }
      return new CategoryMailboxPerspective([category]);
    };

    MailboxPerspective.forCategories = function(categories) {
      if (categories.length === 0) {
        return this.forNothing();
      }
      return new CategoryMailboxPerspective(categories);
    };

    MailboxPerspective.forStandardCategories = function() {
      var accountsOrIds, categories, names;
      accountsOrIds = arguments[0], names = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      categories = CategoryStore.getStandardCategories.apply(CategoryStore, [accountsOrIds].concat(__slice.call(names)));
      return this.forCategories(categories);
    };

    MailboxPerspective.forStarred = function(accountsOrIds) {
      return new StarredMailboxPerspective(accountsOrIds);
    };

    MailboxPerspective.forUnread = function(categories) {
      if (categories.length === 0) {
        return this.forNothing();
      }
      return new UnreadMailboxPerspective(categories);
    };

    MailboxPerspective.forInbox = function(accountsOrIds) {
      return MailboxPerspective.forStandardCategories(accountsOrIds, 'inbox');
    };

    MailboxPerspective.fromJSON = function(json) {
      var categories, error;
      try {
        if (json.type === CategoryMailboxPerspective.name) {
          categories = JSON.parse(json.serializedCategories, Utils.registeredObjectReviver);
          return MailboxPerspective.forCategories(categories);
        } else if (json.type === UnreadMailboxPerspective.name) {
          categories = JSON.parse(json.serializedCategories, Utils.registeredObjectReviver);
          return MailboxPerspective.forUnread(categories);
        } else if (json.type === StarredMailboxPerspective.name) {
          return MailboxPerspective.forStarred(json.accountIds);
        } else if (json.type === DraftsMailboxPerspective.name) {
          return MailboxPerspective.forDrafts(json.accountIds);
        } else {
          return MailboxPerspective.forInbox(json.accountIds);
        }
      } catch (_error) {
        error = _error;
        NylasEnv.reportError(new Error("Could not restore mailbox perspective: " + error));
        return null;
      }
    };

    function MailboxPerspective(accountIds) {
      this.accountIds = accountIds;
      this.tasksForRemovingItems = __bind(this.tasksForRemovingItems, this);
      this.canMoveThreadsTo = __bind(this.canMoveThreadsTo, this);
      this.canTrashThreads = __bind(this.canTrashThreads, this);
      this.canArchiveThreads = __bind(this.canArchiveThreads, this);
      this.receiveThreads = __bind(this.receiveThreads, this);
      this.canReceiveThreadsFromAccountIds = __bind(this.canReceiveThreadsFromAccountIds, this);
      this.unreadCount = __bind(this.unreadCount, this);
      this.threads = __bind(this.threads, this);
      this.category = __bind(this.category, this);
      this.categoriesSharedName = __bind(this.categoriesSharedName, this);
      this.categories = __bind(this.categories, this);
      this.emptyMessage = __bind(this.emptyMessage, this);
      this.isArchive = __bind(this.isArchive, this);
      this.isTrash = __bind(this.isTrash, this);
      this.isSent = __bind(this.isSent, this);
      this.isInbox = __bind(this.isInbox, this);
      this.isEqual = __bind(this.isEqual, this);
      this.toJSON = __bind(this.toJSON, this);
      if (!(this.accountIds instanceof Array && _.every(this.accountIds, _.isString))) {
        throw new Error("" + this.constructor.name + ": You must provide an array of string `accountIds`");
      }
      this;
    }

    MailboxPerspective.prototype.toJSON = function() {
      return {
        accountIds: this.accountIds,
        type: this.constructor.name
      };
    };

    MailboxPerspective.prototype.isEqual = function(other) {
      if (!(other && this.constructor === other.constructor)) {
        return false;
      }
      if (other.name !== this.name) {
        return false;
      }
      if (!_.isEqual(this.accountIds, other.accountIds)) {
        return false;
      }
      return true;
    };

    MailboxPerspective.prototype.isInbox = function() {
      return this.categoriesSharedName() === 'inbox';
    };

    MailboxPerspective.prototype.isSent = function() {
      return this.categoriesSharedName() === 'sent';
    };

    MailboxPerspective.prototype.isTrash = function() {
      return this.categoriesSharedName() === 'trash';
    };

    MailboxPerspective.prototype.isArchive = function() {
      return false;
    };

    MailboxPerspective.prototype.emptyMessage = function() {
      return "Nothing to display";
    };

    MailboxPerspective.prototype.categories = function() {
      return [];
    };

    MailboxPerspective.prototype.categoriesSharedName = function() {
      if (this._categoriesSharedName == null) {
        this._categoriesSharedName = Category.categoriesSharedName(this.categories());
      }
      return this._categoriesSharedName;
    };

    MailboxPerspective.prototype.category = function() {
      if (this.categories().length !== 1) {
        return null;
      }
      return this.categories()[0];
    };

    MailboxPerspective.prototype.threads = function() {
      throw new Error("threads: Not implemented in base class.");
    };

    MailboxPerspective.prototype.unreadCount = function() {
      return 0;
    };

    MailboxPerspective.prototype.canReceiveThreadsFromAccountIds = function(accountIds) {
      var areIncomingIdsInCurrent;
      if (!(accountIds && accountIds.length > 0)) {
        return false;
      }
      areIncomingIdsInCurrent = _.difference(accountIds, this.accountIds).length === 0;
      return areIncomingIdsInCurrent;
    };

    MailboxPerspective.prototype.receiveThreads = function(threadsOrIds) {
      throw new Error("receiveThreads: Not implemented in base class.");
    };

    MailboxPerspective.prototype.canArchiveThreads = function(threads) {
      var accounts;
      if (this.isArchive()) {
        return false;
      }
      accounts = AccountStore.accountsForItems(threads);
      return _.every(accounts, function(acc) {
        return acc.canArchiveThreads();
      });
    };

    MailboxPerspective.prototype.canTrashThreads = function(threads) {
      return this.canMoveThreadsTo(threads, 'trash');
    };

    MailboxPerspective.prototype.canMoveThreadsTo = function(threads, standardCategoryName) {
      if (this.categoriesSharedName() === standardCategoryName) {
        return false;
      }
      return _.every(AccountStore.accountsForItems(threads), function(acc) {
        return CategoryStore.getStandardCategory(acc, standardCategoryName) != null;
      });
    };

    MailboxPerspective.prototype.tasksForRemovingItems = function(threads) {
      if (!threads instanceof Array) {
        throw new Error("tasksForRemovingItems: you must pass an array of threads or thread ids");
      }
      return [];
    };

    return MailboxPerspective;

  })();

  DraftsMailboxPerspective = (function(_super) {
    __extends(DraftsMailboxPerspective, _super);

    function DraftsMailboxPerspective(accountIds) {
      this.accountIds = accountIds;
      this.canReceiveThreadsFromAccountIds = __bind(this.canReceiveThreadsFromAccountIds, this);
      this.unreadCount = __bind(this.unreadCount, this);
      this.threads = __bind(this.threads, this);
      DraftsMailboxPerspective.__super__.constructor.call(this, this.accountIds);
      this.name = "Drafts";
      this.iconName = "drafts.png";
      this.drafts = true;
      this;
    }

    DraftsMailboxPerspective.prototype.threads = function() {
      return null;
    };

    DraftsMailboxPerspective.prototype.unreadCount = function() {
      var aid, count, _i, _len, _ref;
      count = 0;
      _ref = this.accountIds;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        aid = _ref[_i];
        count += OutboxStore.itemsForAccount(aid).length;
      }
      return count;
    };

    DraftsMailboxPerspective.prototype.canReceiveThreadsFromAccountIds = function() {
      return false;
    };

    return DraftsMailboxPerspective;

  })(MailboxPerspective);

  StarredMailboxPerspective = (function(_super) {
    __extends(StarredMailboxPerspective, _super);

    function StarredMailboxPerspective(accountIds) {
      this.accountIds = accountIds;
      this.tasksForRemovingItems = __bind(this.tasksForRemovingItems, this);
      this.receiveThreads = __bind(this.receiveThreads, this);
      this.canReceiveThreadsFromAccountIds = __bind(this.canReceiveThreadsFromAccountIds, this);
      this.threads = __bind(this.threads, this);
      StarredMailboxPerspective.__super__.constructor.call(this, this.accountIds);
      this.name = "Starred";
      this.iconName = "starred.png";
      this;
    }

    StarredMailboxPerspective.prototype.threads = function() {
      var query;
      query = DatabaseStore.findAll(Thread).where([Thread.attributes.starred.equal(true), Thread.attributes.inAllMail.equal(true)]).limit(0);
      if (this.accountIds.length < AccountStore.accounts().length) {
        query.where(Thread.attributes.accountId["in"](this.accountIds));
      }
      return new MutableQuerySubscription(query, {
        emitResultSet: true
      });
    };

    StarredMailboxPerspective.prototype.canReceiveThreadsFromAccountIds = function() {
      return StarredMailboxPerspective.__super__.canReceiveThreadsFromAccountIds.apply(this, arguments);
    };

    StarredMailboxPerspective.prototype.receiveThreads = function(threadsOrIds) {
      var ChangeStarredTask, task;
      ChangeStarredTask = require('./flux/tasks/change-starred-task')["default"];
      task = new ChangeStarredTask({
        threads: threadsOrIds,
        starred: true
      });
      return Actions.queueTask(task);
    };

    StarredMailboxPerspective.prototype.tasksForRemovingItems = function(threads) {
      var task;
      task = TaskFactory.taskForInvertingStarred({
        threads: threads
      });
      return [task];
    };

    return StarredMailboxPerspective;

  })(MailboxPerspective);

  EmptyMailboxPerspective = (function(_super) {
    __extends(EmptyMailboxPerspective, _super);

    function EmptyMailboxPerspective() {
      this.canReceiveThreadsFromAccountIds = __bind(this.canReceiveThreadsFromAccountIds, this);
      this.threads = __bind(this.threads, this);
      this.accountIds = [];
    }

    EmptyMailboxPerspective.prototype.threads = function() {
      var query;
      query = DatabaseStore.findAll(Thread).where({
        lastMessageReceivedTimestamp: -1
      }).limit(0);
      return new MutableQuerySubscription(query, {
        emitResultSet: true
      });
    };

    EmptyMailboxPerspective.prototype.canReceiveThreadsFromAccountIds = function() {
      return false;
    };

    return EmptyMailboxPerspective;

  })(MailboxPerspective);

  CategoryMailboxPerspective = (function(_super) {
    __extends(CategoryMailboxPerspective, _super);

    function CategoryMailboxPerspective(_categories) {
      this._categories = _categories;
      this.tasksForRemovingItems = __bind(this.tasksForRemovingItems, this);
      this.receiveThreads = __bind(this.receiveThreads, this);
      this.canReceiveThreadsFromAccountIds = __bind(this.canReceiveThreadsFromAccountIds, this);
      this.isArchive = __bind(this.isArchive, this);
      this.categories = __bind(this.categories, this);
      this.unreadCount = __bind(this.unreadCount, this);
      this.threads = __bind(this.threads, this);
      this.isEqual = __bind(this.isEqual, this);
      this.toJSON = __bind(this.toJSON, this);
      CategoryMailboxPerspective.__super__.constructor.call(this, _.uniq(_.pluck(this._categories, 'accountId')));
      if (this._categories.length === 0) {
        throw new Error("CategoryMailboxPerspective: You must provide at least one category.");
      }
      this.name = this._categories[0].displayName;
      if (this._categories[0].name) {
        this.iconName = "" + this._categories[0].name + ".png";
      } else {
        this.iconName = AccountStore.accountForId(this.accountIds[0]).categoryIcon();
      }
      this;
    }

    CategoryMailboxPerspective.prototype.toJSON = function() {
      var json;
      json = CategoryMailboxPerspective.__super__.toJSON.apply(this, arguments);
      json.serializedCategories = JSON.stringify(this._categories, Utils.registeredObjectReplacer);
      return json;
    };

    CategoryMailboxPerspective.prototype.isEqual = function(other) {
      return CategoryMailboxPerspective.__super__.isEqual.call(this, other) && _.isEqual(_.pluck(this.categories(), 'id'), _.pluck(other.categories(), 'id'));
    };

    CategoryMailboxPerspective.prototype.threads = function() {
      var query, _ref;
      query = DatabaseStore.findAll(Thread).where([Thread.attributes.categories.containsAny(_.pluck(this.categories(), 'id'))]).limit(0);
      if (this.isSent()) {
        query.order(Thread.attributes.lastMessageSentTimestamp.descending());
      }
      if ((_ref = this.categoriesSharedName()) !== 'spam' && _ref !== 'trash') {
        query.where({
          inAllMail: true
        });
      }
      if (this._categories.length > 1 && this.accountIds.length < this._categories.length) {
        query.distinct();
      }
      return new MutableQuerySubscription(query, {
        emitResultSet: true
      });
    };

    CategoryMailboxPerspective.prototype.unreadCount = function() {
      var cat, sum, _i, _len, _ref;
      sum = 0;
      _ref = this._categories;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        cat = _ref[_i];
        sum += ThreadCountsStore.unreadCountForCategoryId(cat.id);
      }
      return sum;
    };

    CategoryMailboxPerspective.prototype.categories = function() {
      return this._categories;
    };

    CategoryMailboxPerspective.prototype.isArchive = function() {
      return _.every(this._categories, function(cat) {
        return cat.isArchive();
      });
    };

    CategoryMailboxPerspective.prototype.canReceiveThreadsFromAccountIds = function() {
      return CategoryMailboxPerspective.__super__.canReceiveThreadsFromAccountIds.apply(this, arguments) && !_.any(this._categories, function(c) {
        return c.isLockedCategory();
      });
    };

    CategoryMailboxPerspective.prototype.receiveThreads = function(threadsOrIds) {
      var FocusedPerspectiveStore, current;
      FocusedPerspectiveStore = require('./flux/stores/focused-perspective-store');
      current = FocusedPerspectiveStore.current();
      return DatabaseStore.modelify(Thread, threadsOrIds).then((function(_this) {
        return function(threads) {
          var tasks;
          tasks = TaskFactory.tasksForApplyingCategories({
            threads: threads,
            categoriesToRemove: function(accountId) {
              var _ref;
              if (_ref = current.categoriesSharedName(), __indexOf.call(Category.LockedCategoryNames, _ref) >= 0) {
                return [];
              }
              return _.filter(current.categories(), _.matcher({
                accountId: accountId
              }));
            },
            categoriesToAdd: function(accountId) {
              return [
                _.findWhere(_this._categories, {
                  accountId: accountId
                })
              ];
            }
          });
          return Actions.queueTasks(tasks);
        };
      })(this));
    };

    CategoryMailboxPerspective.prototype.tasksForRemovingItems = function(threads, ruleset) {
      var name;
      if (!ruleset) {
        throw new Error("tasksForRemovingItems: you must pass a ruleset object to determine the destination of the threads");
      }
      name = this.isArchive() ? 'archive' : this.categoriesSharedName();
      if (ruleset[name] === null) {
        return [];
      }
      return TaskFactory.tasksForApplyingCategories({
        threads: threads,
        categoriesToRemove: (function(_this) {
          return function(accountId) {
            return _.filter(_this._categories, _.matcher({
              accountId: accountId
            }));
          };
        })(this),
        categoriesToAdd: (function(_this) {
          return function(accId) {
            var category, _ref;
            category = ((_ref = ruleset[name]) != null ? _ref : ruleset.other)(accId);
            if (category) {
              return [category];
            } else {
              return [];
            }
          };
        })(this)
      });
    };

    return CategoryMailboxPerspective;

  })(MailboxPerspective);

  UnreadMailboxPerspective = (function(_super) {
    __extends(UnreadMailboxPerspective, _super);

    function UnreadMailboxPerspective(categories) {
      this.tasksForRemovingItems = __bind(this.tasksForRemovingItems, this);
      this.receiveThreads = __bind(this.receiveThreads, this);
      this.unreadCount = __bind(this.unreadCount, this);
      this.threads = __bind(this.threads, this);
      UnreadMailboxPerspective.__super__.constructor.call(this, categories);
      this.name = "Unread";
      this.iconName = "unread.png";
      this;
    }

    UnreadMailboxPerspective.prototype.threads = function() {
      return new UnreadQuerySubscription(_.pluck(this.categories(), 'id'));
    };

    UnreadMailboxPerspective.prototype.unreadCount = function() {
      return 0;
    };

    UnreadMailboxPerspective.prototype.receiveThreads = function(threadsOrIds) {
      var task;
      UnreadMailboxPerspective.__super__.receiveThreads.call(this, threadsOrIds);
      if (ChangeUnreadTask == null) {
        ChangeUnreadTask = require('./flux/tasks/change-unread-task')["default"];
      }
      task = new ChangeUnreadTask({
        threads: threadsOrIds,
        unread: true
      });
      return Actions.queueTask(task);
    };

    UnreadMailboxPerspective.prototype.tasksForRemovingItems = function(threads, ruleset) {
      var tasks;
      if (ChangeUnreadTask == null) {
        ChangeUnreadTask = require('./flux/tasks/change-unread-task')["default"];
      }
      tasks = UnreadMailboxPerspective.__super__.tasksForRemovingItems.call(this, threads, ruleset);
      tasks.push(new ChangeUnreadTask({
        threads: threads,
        unread: false
      }));
      return tasks;
    };

    return UnreadMailboxPerspective;

  })(CategoryMailboxPerspective);

  module.exports = MailboxPerspective;

}).call(this);
