(function() {
  var Disposable, MenuHelpers, MenuManager, Utils, fs, ipcRenderer, path, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  path = require('path');

  _ = require('underscore');

  fs = require('fs-plus');

  ipcRenderer = require('electron').ipcRenderer;

  Disposable = require('event-kit').Disposable;

  Utils = require('./flux/models/utils');

  MenuHelpers = require('./menu-helpers');

  module.exports = MenuManager = (function() {
    function MenuManager(_arg) {
      this.resourcePath = _arg.resourcePath;
      this.update = __bind(this.update, this);
      this.template = [];
      this.loadPlatformItems();
      NylasEnv.keymaps.onDidReloadKeymap((function(_this) {
        return function() {
          return _this.update();
        };
      })(this));
      NylasEnv.commands.onRegistedCommandsChanged((function(_this) {
        return function() {
          return _this.update();
        };
      })(this));
    }

    MenuManager.prototype.add = function(items) {
      var item, _i, _len;
      items = Utils.deepClone(items);
      for (_i = 0, _len = items.length; _i < _len; _i++) {
        item = items[_i];
        this.merge(this.template, item);
      }
      this.update();
      return new Disposable((function(_this) {
        return function() {
          return _this.remove(items);
        };
      })(this));
    };

    MenuManager.prototype.remove = function(items) {
      var item, _i, _len;
      for (_i = 0, _len = items.length; _i < _len; _i++) {
        item = items[_i];
        this.unmerge(this.template, item);
      }
      return this.update();
    };

    MenuManager.prototype.update = function() {
      if (this.pendingUpdateOperation) {
        return;
      }
      this.pendingUpdateOperation = true;
      return window.requestAnimationFrame((function(_this) {
        return function() {
          _this.pendingUpdateOperation = false;
          MenuHelpers.forEachMenuItem(_this.template, function(item) {
            if (item.command && item.command.startsWith('application:') === false) {
              item.enabled = NylasEnv.commands.listenerCountForCommand(item.command) > 0;
            }
            if (item.submenu != null) {
              return item.enabled = !_.every(item.submenu, function(item) {
                return item.enabled === false;
              });
            }
          });
          return _this.sendToBrowserProcess(_this.template, NylasEnv.keymaps.getBindingsForAllCommands());
        };
      })(this));
    };

    MenuManager.prototype.loadPlatformItems = function() {
      var menu, menusDirPath, platformMenuPath;
      menusDirPath = path.join(this.resourcePath, 'menus');
      platformMenuPath = fs.resolve(menusDirPath, process.platform, ['json']);
      menu = require(platformMenuPath).menu;
      return this.add(menu);
    };

    MenuManager.prototype.merge = function(menu, item) {
      return MenuHelpers.merge(menu, item);
    };

    MenuManager.prototype.unmerge = function(menu, item) {
      return MenuHelpers.unmerge(menu, item);
    };

    MenuManager.prototype.filterMultipleKeystroke = function(keystrokesByCommand) {
      var binding, bindings, filtered, key, _i, _len;
      filtered = {};
      for (key in keystrokesByCommand) {
        bindings = keystrokesByCommand[key];
        for (_i = 0, _len = bindings.length; _i < _len; _i++) {
          binding = bindings[_i];
          if (binding.indexOf(' ') !== -1) {
            continue;
          }
          if (!(/(cmd|ctrl|shift|alt|mod)/.test(binding) || /f\d+/.test(binding))) {
            continue;
          }
          if (filtered[key] == null) {
            filtered[key] = [];
          }
          filtered[key].push(binding);
        }
      }
      return filtered;
    };

    MenuManager.prototype.sendToBrowserProcess = function(template, keystrokesByCommand) {
      keystrokesByCommand = this.filterMultipleKeystroke(keystrokesByCommand);
      return ipcRenderer.send('update-application-menu', template, keystrokesByCommand);
    };

    return MenuManager;

  })();

}).call(this);
