(function() {
  var APIError, Emitter, NylasEnvConstructor, StoreRegistry, StylesElement, Utils, WindowEventHandler, convertLine, convertStackTrace, crypto, deprecate, ensureInteger, fs, ipcRenderer, mapSourcePosition, os, path, remote, shell, _, _ref, _ref1,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  crypto = require('crypto');

  os = require('os');

  path = require('path');

  _ref = require('electron'), ipcRenderer = _ref.ipcRenderer, remote = _ref.remote, shell = _ref.shell;

  _ = require('underscore');

  deprecate = require('grim').deprecate;

  Emitter = require('event-kit').Emitter;

  fs = require('fs-plus');

  _ref1 = require('coffeestack'), convertStackTrace = _ref1.convertStackTrace, convertLine = _ref1.convertLine;

  mapSourcePosition = require('source-map-support').mapSourcePosition;

  WindowEventHandler = require('./window-event-handler');

  StylesElement = require('./styles-element');

  StoreRegistry = require('./store-registry')["default"];

  Utils = require('./flux/models/utils');

  APIError = require('./flux/errors').APIError;

  ensureInteger = function(f, fallback) {
    if (f === NaN || f === void 0 || f === null) {
      f = fallback;
    }
    return Math.round(f);
  };

  module.exports = NylasEnvConstructor = (function() {
    NylasEnvConstructor.version = 1;

    NylasEnvConstructor.prototype.assert = function(bool, msg) {
      if (!bool) {
        throw new Error("Assertion error: " + msg);
      }
    };

    NylasEnvConstructor.loadOrCreate = function() {
      var app, savedState, startTime;
      startTime = Date.now();
      savedState = this._loadSavedState();
      if (savedState && (savedState != null ? savedState.version : void 0) === this.version) {
        app = new this(savedState);
      } else {
        app = new this({
          version: this.version
        });
      }
      return app;
    };

    NylasEnvConstructor._loadSavedState = function() {
      var error, statePath, stateString;
      statePath = this.getStatePath();
      if (fs.existsSync(statePath)) {
        try {
          stateString = fs.readFileSync(statePath, 'utf8');
        } catch (_error) {
          error = _error;
          console.warn("Error reading window state: " + statePath, error.stack, error);
        }
      } else {
        stateString = this.getLoadSettings().windowState;
      }
      try {
        if (stateString != null) {
          return JSON.parse(stateString);
        }
      } catch (_error) {
        error = _error;
        return console.warn("Error parsing window state: " + statePath + " " + error.stack, error);
      }
    };

    NylasEnvConstructor.getStatePath = function() {
      var configDirPath, filename, isSpec, mainWindow, _ref2;
      _ref2 = this.getLoadSettings(), isSpec = _ref2.isSpec, mainWindow = _ref2.mainWindow, configDirPath = _ref2.configDirPath;
      if (isSpec) {
        return filename = 'spec-saved-state.json';
      } else if (mainWindow) {
        return path.join(configDirPath, 'main-window-state.json');
      } else {
        return null;
      }
    };

    NylasEnvConstructor.getLoadSettings = function() {
      var cloned;
      if (this.loadSettings == null) {
        this.loadSettings = JSON.parse(decodeURIComponent(location.search.substr(14)));
      }
      cloned = Utils.deepClone(this.loadSettings);
      cloned.__defineGetter__('windowState', (function(_this) {
        return function() {
          return _this.getCurrentWindow().loadSettings.windowState;
        };
      })(this));
      cloned.__defineSetter__('windowState', (function(_this) {
        return function(value) {
          return _this.getCurrentWindow().loadSettings.windowState = value;
        };
      })(this));
      return cloned;
    };

    NylasEnvConstructor.getCurrentWindow = function() {
      return remote.getCurrentWindow();
    };

    NylasEnvConstructor.prototype.workspaceViewParentSelector = 'body';

    NylasEnvConstructor.prototype.lastUncaughtError = null;


    /*
    Section: Properties
     */

    NylasEnvConstructor.prototype.commands = null;

    NylasEnvConstructor.prototype.config = null;

    NylasEnvConstructor.prototype.menu = null;

    NylasEnvConstructor.prototype.keymaps = null;

    NylasEnvConstructor.prototype.packages = null;

    NylasEnvConstructor.prototype.themes = null;

    NylasEnvConstructor.prototype.styles = null;


    /*
    Section: Construction and Destruction
     */

    function NylasEnvConstructor(savedState) {
      this.savedState = savedState != null ? savedState : {};
      this.populateHotWindow = __bind(this.populateHotWindow, this);
      this.getColumnWidth = __bind(this.getColumnWidth, this);
      this.storeColumnWidth = __bind(this.storeColumnWidth, this);
      this.getConfigDirPath = __bind(this.getConfigDirPath, this);
      this.version = this.savedState.version;
      this.emitter = new Emitter;
    }

    NylasEnvConstructor.prototype.initialize = function() {
      var ActionBridge, CommandRegistry, Config, KeymapManager, MenuManager, PackageManager, StyleManager, ThemeManager, configDirPath, devMode, globalPath, resourcePath, safeMode, specMode, windowType, _base, _ref2;
      if (!(this.inDevMode() || this.inSpecMode())) {
        require('grim').deprecate = function() {};
      }
      this.enhanceEventObject();
      this.setupErrorLogger();
      this.loadTime = null;
      Config = require('./config');
      KeymapManager = require('./keymap-manager')["default"];
      CommandRegistry = require('./command-registry');
      PackageManager = require('./package-manager');
      ThemeManager = require('./theme-manager');
      StyleManager = require('./style-manager');
      ActionBridge = require('./flux/action-bridge');
      MenuManager = require('./menu-manager');
      _ref2 = this.getLoadSettings(), devMode = _ref2.devMode, safeMode = _ref2.safeMode, resourcePath = _ref2.resourcePath, configDirPath = _ref2.configDirPath, windowType = _ref2.windowType;
      document.body.classList.add("platform-" + process.platform);
      document.body.classList.add("window-type-" + windowType);
      globalPath = path.join(resourcePath, 'src', 'global');
      require('module').globalPaths.push(globalPath);
      process.env.NODE_PATH = globalPath;
      if (!devMode) {
        if ((_base = process.env).NODE_ENV == null) {
          _base.NODE_ENV = 'production';
        }
      }
      process.env.NYLAS_HOME = configDirPath;
      this.config = new Config({
        configDirPath: configDirPath,
        resourcePath: resourcePath
      });
      this.keymaps = new KeymapManager({
        configDirPath: configDirPath,
        resourcePath: resourcePath
      });
      specMode = this.inSpecMode();
      this.commands = new CommandRegistry;
      this.packages = new PackageManager({
        devMode: devMode,
        configDirPath: configDirPath,
        resourcePath: resourcePath,
        safeMode: safeMode,
        specMode: specMode
      });
      this.styles = new StyleManager;
      document.head.appendChild(new StylesElement);
      this.themes = new ThemeManager({
        packageManager: this.packages,
        configDirPath: configDirPath,
        resourcePath: resourcePath,
        safeMode: safeMode
      });
      this.menu = new MenuManager({
        resourcePath: resourcePath
      });
      if (process.platform === 'win32') {
        this.getCurrentWindow().setMenuBarVisibility(false);
      }
      this.spellchecker = require('./nylas-spellchecker');
      this.packages.onDidActivateInitialPackages((function(_this) {
        return function() {
          return _this.watchThemes();
        };
      })(this));
      this.windowEventHandler = new WindowEventHandler();
      this.perf = remote.getGlobal('application').perf;
      if (!this.inSpecMode()) {
        this.actionBridge = new ActionBridge(ipcRenderer);
      }
      return require('nylas-exports');
    };

    NylasEnvConstructor.prototype.setupErrorLogger = function() {
      var ErrorLogger, sourceMapCache;
      ErrorLogger = require('./error-logger');
      this.errorLogger = new ErrorLogger({
        inSpecMode: this.inSpecMode(),
        inDevMode: this.inDevMode(),
        resourcePath: this.getLoadSettings().resourcePath
      });
      sourceMapCache = {};
      window.onerror = (function(_this) {
        return function() {
          var args, column, line, message, originalError, url, _ref2;
          args = Array.prototype.slice.call(arguments);
          message = args[0], url = args[1], line = args[2], column = args[3], originalError = args[4];
          _ref2 = mapSourcePosition({
            source: url,
            line: line,
            column: column
          }), line = _ref2.line, column = _ref2.column;
          originalError.stack = convertStackTrace(originalError.stack, sourceMapCache);
          return _this.reportError(originalError, {
            url: url,
            line: line,
            column: column
          });
        };
      })(this);
      Promise.onPossiblyUnhandledRejection((function(_this) {
        return function(error, promise) {
          error.stack = convertStackTrace(error.stack, sourceMapCache);
          if (error instanceof APIError) {
            if (error.statusCode !== 400) {
              return;
            }
          }
          return _this.reportError(error, {
            promise: promise
          });
        };
      })(this));
      if (this.inSpecMode() || this.inDevMode()) {
        return Promise.longStackTraces();
      }
    };

    NylasEnvConstructor.prototype._createErrorCallbackEvent = function(error, extraArgs) {
      var event;
      if (extraArgs == null) {
        extraArgs = {};
      }
      event = _.extend({}, extraArgs, {
        message: error.message,
        originalError: error,
        defaultPrevented: false
      });
      event.preventDefault = function() {
        return event.defaultPrevented = true;
      };
      return event;
    };

    NylasEnvConstructor.prototype.reportError = function(error, extra, _arg) {
      var event, noWindows;
      if (extra == null) {
        extra = {};
      }
      noWindows = (_arg != null ? _arg : {}).noWindows;
      event = this._createErrorCallbackEvent(error, extra);
      this.emitter.emit('will-throw-error', event);
      if (event.defaultPrevented) {
        return;
      }
      console.error(error.stack);
      this.lastUncaughtError = error;
      extra.pluginIds = this._findPluginsFromError(error);
      if (this.inSpecMode()) {
        jasmine.getEnv().currentSpec.fail(error);
      } else if (this.inDevMode() && !noWindows) {
        this.openDevTools();
        this.executeJavaScriptInDevTools('DevToolsAPI.showConsole()');
      }
      this.errorLogger.reportError(error, extra);
      return this.emitter.emit('did-throw-error', event);
    };

    NylasEnvConstructor.prototype._findPluginsFromError = function(error) {
      var pluginIdsByPathBase, stackPaths, stackTokens, tokens, _ref2;
      if (!error.stack) {
        return [];
      }
      stackPaths = (_ref2 = error.stack.match(/((?:\/[\w-_]+)+)/g)) != null ? _ref2 : [];
      stackTokens = _.uniq(_.flatten(stackPaths.map(function(p) {
        return p.split("/");
      })));
      pluginIdsByPathBase = this.packages.getPluginIdsByPathBase();
      tokens = _.intersection(Object.keys(pluginIdsByPathBase), stackTokens);
      return tokens.map(function(tok) {
        return pluginIdsByPathBase[tok];
      });
    };


    /*
    Section: Event Subscription
     */

    NylasEnvConstructor.prototype.onDidBeep = function(callback) {
      return this.emitter.on('did-beep', callback);
    };

    NylasEnvConstructor.prototype.onWillThrowError = function(callback) {
      return this.emitter.on('will-throw-error', callback);
    };

    NylasEnvConstructor.prototype.onDidThrowError = function(callback) {
      return this.emitter.on('did-throw-error', callback);
    };

    NylasEnvConstructor.prototype.trace = function() {
      var opts, tracing;
      tracing = remote.contentTracing;
      opts = {
        categoryFilter: '*',
        traceOptions: 'record-until-full,enable-sampling,enable-systrace'
      };
      return tracing.startRecording(opts, function() {
        console.log('Tracing started');
        return setTimeout(function() {
          return tracing.stopRecording('', function(path) {
            return console.log('Tracing data recorded to ' + path);
          });
        }, 5000);
      });
    };

    NylasEnvConstructor.prototype.isMainWindow = function() {
      return !!this.getLoadSettings().mainWindow;
    };

    NylasEnvConstructor.prototype.isWorkWindow = function() {
      return this.getWindowType() === 'work';
    };

    NylasEnvConstructor.prototype.isComposerWindow = function() {
      var _ref2;
      return (_ref2 = this.getWindowType()) === "composer" || _ref2 === "composer-preload";
    };

    NylasEnvConstructor.prototype.getWindowType = function() {
      return this.getLoadSettings().windowType;
    };

    NylasEnvConstructor.prototype.inDevMode = function() {
      return this.getLoadSettings().devMode;
    };

    NylasEnvConstructor.prototype.inSafeMode = function() {
      return this.getLoadSettings().safeMode;
    };

    NylasEnvConstructor.prototype.inSpecMode = function() {
      return this.getLoadSettings().isSpec;
    };

    NylasEnvConstructor.prototype.getVersion = function() {
      return this.appVersion != null ? this.appVersion : this.appVersion = this.getLoadSettings().appVersion;
    };

    NylasEnvConstructor.prototype.isReleasedVersion = function() {
      return !/\w{7}/.test(this.getVersion());
    };

    NylasEnvConstructor.prototype.getConfigDirPath = function() {
      return this.getLoadSettings().configDirPath;
    };

    NylasEnvConstructor.prototype.getWindowLoadTime = function() {
      return this.loadTime;
    };

    NylasEnvConstructor.prototype.getLoadSettings = function() {
      return this.constructor.getLoadSettings();
    };


    /*
    Section: Managing The Nylas Window
     */

    NylasEnvConstructor.prototype.close = function() {
      return this.getCurrentWindow().close();
    };

    NylasEnvConstructor.prototype.quit = function() {
      return remote.app.quit();
    };

    NylasEnvConstructor.prototype.getSize = function() {
      var height, width, _ref2;
      _ref2 = this.getCurrentWindow().getSize(), width = _ref2[0], height = _ref2[1];
      return {
        width: width,
        height: height
      };
    };

    NylasEnvConstructor.prototype.setSize = function(width, height) {
      width = ensureInteger(width, 100);
      height = ensureInteger(height, 100);
      return this.getCurrentWindow().setSize(width, height);
    };

    NylasEnvConstructor.prototype.setSizeAnimated = function(width, height, duration) {
      var boundsForI, call, cubicInOut, startBounds, startTime, tick, win;
      if (duration == null) {
        duration = 400;
      }
      if (process.platform === 'win32') {
        duration = 1;
      }
      duration = Math.max(1, duration);
      if (this._setSizeAnimatedCallCount == null) {
        this._setSizeAnimatedCallCount = 0;
      }
      this._setSizeAnimatedCallCount += 1;
      call = this._setSizeAnimatedCallCount;
      cubicInOut = function(t) {
        if (t < .5) {
          return 4 * Math.pow(t, 3);
        } else {
          return (t - 1) * Math.pow(2 * t - 2, 2) + 1;
        }
      };
      win = this.getCurrentWindow();
      width = Math.round(width);
      height = Math.round(height);
      startBounds = win.getBounds();
      startTime = Date.now() - 1;
      boundsForI = function(i) {
        return {
          x: ensureInteger(startBounds.x + (width - startBounds.width) * -0.5 * i, 0),
          y: ensureInteger(startBounds.y + (height - startBounds.height) * -0.5 * i, 0),
          width: ensureInteger(startBounds.width + (width - startBounds.width) * i, width),
          height: ensureInteger(startBounds.height + (height - startBounds.height) * i, height)
        };
      };
      tick = (function(_this) {
        return function() {
          var i, t;
          if (call !== _this._setSizeAnimatedCallCount) {
            return;
          }
          t = Math.min(1, (Date.now() - startTime) / duration);
          i = cubicInOut(t);
          win.setBounds(boundsForI(i));
          if (t !== 1) {
            return _.defer(tick);
          }
        };
      })(this);
      return tick();
    };

    NylasEnvConstructor.prototype.setMinimumWidth = function(minWidth) {
      var currHeight, currWidth, minHeight, win, _ref2;
      win = this.getCurrentWindow();
      minWidth = ensureInteger(minWidth, 0);
      minHeight = win.getMinimumSize()[1];
      win.setMinimumSize(minWidth, minHeight);
      _ref2 = win.getSize(), currWidth = _ref2[0], currHeight = _ref2[1];
      if (minWidth > currWidth) {
        return win.setSize(minWidth, currHeight);
      }
    };

    NylasEnvConstructor.prototype.getPosition = function() {
      var x, y, _ref2;
      _ref2 = this.getCurrentWindow().getPosition(), x = _ref2[0], y = _ref2[1];
      return {
        x: x,
        y: y
      };
    };

    NylasEnvConstructor.prototype.setPosition = function(x, y) {
      x = ensureInteger(x, 0);
      y = ensureInteger(y, 0);
      return ipcRenderer.send('call-window-method', 'setPosition', x, y);
    };

    NylasEnvConstructor.prototype.getCurrentWindow = function() {
      return this.constructor.getCurrentWindow();
    };

    NylasEnvConstructor.prototype.center = function() {
      return ipcRenderer.send('call-window-method', 'center');
    };

    NylasEnvConstructor.prototype.focus = function() {
      ipcRenderer.send('call-window-method', 'focus');
      return window.focus();
    };

    NylasEnvConstructor.prototype.show = function() {
      return ipcRenderer.send('call-window-method', 'show');
    };

    NylasEnvConstructor.prototype.isVisible = function() {
      return this.getCurrentWindow().isVisible();
    };

    NylasEnvConstructor.prototype.hide = function() {
      return ipcRenderer.send('call-window-method', 'hide');
    };

    NylasEnvConstructor.prototype.reload = function() {
      return ipcRenderer.send('call-webcontents-method', 'reload');
    };

    NylasEnvConstructor.prototype.getWindowProps = function() {
      var _ref2;
      return (_ref2 = this.getLoadSettings().windowProps) != null ? _ref2 : {};
    };

    NylasEnvConstructor.prototype.onWindowPropsReceived = function(callback) {
      return this.emitter.on('window-props-received', callback);
    };

    NylasEnvConstructor.prototype.isMaximixed = function() {
      return this.getCurrentWindow().isMaximized();
    };

    NylasEnvConstructor.prototype.maximize = function() {
      return ipcRenderer.send('call-window-method', 'maximize');
    };

    NylasEnvConstructor.prototype.minimize = function() {
      return ipcRenderer.send('call-window-method', 'minimize');
    };

    NylasEnvConstructor.prototype.isFullScreen = function() {
      return this.getCurrentWindow().isFullScreen();
    };

    NylasEnvConstructor.prototype.setFullScreen = function(fullScreen) {
      if (fullScreen == null) {
        fullScreen = false;
      }
      ipcRenderer.send('call-window-method', 'setFullScreen', fullScreen);
      if (fullScreen) {
        return document.body.classList.add("fullscreen");
      } else {
        return document.body.classList.remove("fullscreen");
      }
    };

    NylasEnvConstructor.prototype.toggleFullScreen = function() {
      return this.setFullScreen(!this.isFullScreen());
    };

    NylasEnvConstructor.prototype.getWindowDimensions = function() {
      var browserWindow, fullScreen, height, maximized, width, x, y, _ref2;
      browserWindow = this.getCurrentWindow();
      _ref2 = browserWindow.getBounds(), x = _ref2.x, y = _ref2.y, width = _ref2.width, height = _ref2.height;
      maximized = browserWindow.isMaximized();
      fullScreen = browserWindow.isFullScreen();
      return {
        x: x,
        y: y,
        width: width,
        height: height,
        maximized: maximized,
        fullScreen: fullScreen
      };
    };

    NylasEnvConstructor.prototype.setWindowDimensions = function(_arg) {
      var height, width, x, y;
      x = _arg.x, y = _arg.y, width = _arg.width, height = _arg.height;
      if ((x != null) && (y != null) && (width != null) && (height != null)) {
        return this.getCurrentWindow().setBounds({
          x: x,
          y: y,
          width: width,
          height: height
        });
      } else if ((width != null) && (height != null)) {
        return this.setSize(width, height);
      } else if ((x != null) && (y != null)) {
        return this.setPosition(x, y);
      } else {
        return this.center();
      }
    };

    NylasEnvConstructor.prototype.isValidDimensions = function(_arg) {
      var height, width, x, y, _ref2;
      _ref2 = _arg != null ? _arg : {}, x = _ref2.x, y = _ref2.y, width = _ref2.width, height = _ref2.height;
      return width > 0 && height > 0 && x + width > 0 && y + height > 0;
    };

    NylasEnvConstructor.prototype.getDefaultWindowDimensions = function() {
      var MAX_HEIGHT, MAX_WIDTH, height, width, x, y, _ref2;
      _ref2 = remote.screen.getPrimaryDisplay().workAreaSize, width = _ref2.width, height = _ref2.height;
      x = 0;
      y = 0;
      MAX_WIDTH = 1440;
      if (width > MAX_WIDTH) {
        x = Math.floor((width - MAX_WIDTH) / 2);
        width = MAX_WIDTH;
      }
      MAX_HEIGHT = 900;
      if (height > MAX_HEIGHT) {
        y = Math.floor((height - MAX_HEIGHT) / 2);
        height = MAX_HEIGHT;
      }
      return {
        x: x,
        y: y,
        width: width,
        height: height
      };
    };

    NylasEnvConstructor.prototype.restoreWindowDimensions = function() {
      var dimensions;
      dimensions = this.savedState.windowDimensions;
      if (!this.isValidDimensions(dimensions)) {
        dimensions = this.getDefaultWindowDimensions();
      }
      this.setWindowDimensions(dimensions);
      if (dimensions.maximized && process.platform !== 'darwin') {
        this.maximize();
      }
      if (dimensions.fullScreen) {
        return this.setFullScreen(true);
      }
    };

    NylasEnvConstructor.prototype.storeWindowDimensions = function() {
      var dimensions;
      dimensions = this.getWindowDimensions();
      if (this.isValidDimensions(dimensions)) {
        return this.savedState.windowDimensions = dimensions;
      }
    };

    NylasEnvConstructor.prototype.storeColumnWidth = function(_arg) {
      var id, width, _base;
      id = _arg.id, width = _arg.width;
      if ((_base = this.savedState).columnWidths == null) {
        _base.columnWidths = {};
      }
      return this.savedState.columnWidths[id] = width;
    };

    NylasEnvConstructor.prototype.getColumnWidth = function(id) {
      var _base;
      if ((_base = this.savedState).columnWidths == null) {
        _base.columnWidths = {};
      }
      return this.savedState.columnWidths[id];
    };

    NylasEnvConstructor.prototype.startWindow = function() {
      var packageLoadingDeferred, windowType, _ref2;
      this.loadConfig();
      _ref2 = this.getLoadSettings(), packageLoadingDeferred = _ref2.packageLoadingDeferred, windowType = _ref2.windowType;
      this.extendRxObservables();
      StoreRegistry.activateAllStores();
      this.keymaps.loadKeymaps();
      this.themes.loadBaseStylesheets();
      if (!packageLoadingDeferred) {
        this.packages.loadPackages(windowType);
      }
      if (!packageLoadingDeferred) {
        this.deserializePackageStates();
      }
      this.initializeReactRoot();
      if (!packageLoadingDeferred) {
        this.packages.activate();
      }
      return this.menu.update();
    };

    NylasEnvConstructor.prototype.startRootWindow = function() {
      var initializeInBackground, safeMode, _ref2;
      _ref2 = this.getLoadSettings(), safeMode = _ref2.safeMode, initializeInBackground = _ref2.initializeInBackground;
      return window.requestAnimationFrame((function(_this) {
        return function() {
          return window.requestAnimationFrame(function() {
            return window.requestAnimationFrame(function() {
              return window.requestAnimationFrame(function() {
                return window.requestAnimationFrame(function() {
                  if (!initializeInBackground) {
                    _this.displayWindow();
                  }
                  _this.startWindow();
                  if (!safeMode) {
                    _this.requireUserInitScript();
                  }
                  _this.showMainWindow();
                  return ipcRenderer.send('window-command', 'window:loaded');
                });
              });
            });
          });
        };
      })(this));
    };

    NylasEnvConstructor.prototype.startSecondaryWindow = function() {
      var _ref2;
      if ((_ref2 = document.getElementById("application-loading-cover")) != null) {
        _ref2.remove();
      }
      this.startWindow();
      this.initializeBasicSheet();
      ipcRenderer.on("load-settings-changed", this.populateHotWindow);
      return ipcRenderer.send('window-command', 'window:loaded');
    };

    NylasEnvConstructor.prototype.initializeBasicSheet = function() {
      var WorkspaceStore;
      WorkspaceStore = require('../src/flux/stores/workspace-store');
      if (!WorkspaceStore.Sheet.Main) {
        return WorkspaceStore.defineSheet('Main', {
          root: true
        }, {
          popout: ['Center']
        });
      }
    };

    NylasEnvConstructor.prototype.showMainWindow = function() {
      document.getElementById("application-loading-cover").remove();
      document.body.classList.add("window-loaded");
      this.restoreWindowDimensions();
      return this.getCurrentWindow().setMinimumSize(875, 250);
    };

    NylasEnvConstructor.prototype.populateHotWindow = function(event, loadSettings) {
      var browserWindow, height, width, _ref2;
      if (/composer/.test(loadSettings.windowType)) {
        NylasEnv.perf.split("Popout Draft");
      }
      this.loadSettings = loadSettings;
      this.constructor.loadSettings = loadSettings;
      this.packages.loadPackages(loadSettings.windowType);
      this.deserializePackageStates();
      this.packages.activate();
      this.emitter.emit('window-props-received', (_ref2 = loadSettings.windowProps) != null ? _ref2 : {});
      width = loadSettings.width, height = loadSettings.height;
      if (width && height) {
        this.setWindowDimensions({
          width: width,
          height: height
        });
      }
      browserWindow = this.getCurrentWindow();
      if (browserWindow.isResizable() !== loadSettings.resizable) {
        browserWindow.setResizable(loadSettings.resizable);
      }
      if (!loadSettings.hidden) {
        return this.displayWindow();
      }
    };

    NylasEnvConstructor.prototype.extendRxObservables = function() {
      return require('nylas-observables');
    };

    NylasEnvConstructor.prototype.newWindow = function(options) {
      if (options == null) {
        options = {};
      }
      return ipcRenderer.send('new-window', options);
    };

    NylasEnvConstructor.prototype.saveStateAndUnloadWindow = function() {
      this.packages.deactivatePackages();
      this.savedState.packageStates = this.packages.packageStates;
      this.saveSync();
      return this.windowState = null;
    };


    /*
    Section: Messaging the User
     */

    NylasEnvConstructor.prototype.displayWindow = function(_arg) {
      var maximize;
      maximize = (_arg != null ? _arg : {}).maximize;
      if (this.inSpecMode()) {
        return;
      }
      this.show();
      this.focus();
      if (maximize) {
        return this.maximize();
      }
    };

    NylasEnvConstructor.prototype.beep = function() {
      if (this.config.get('core.audioBeep')) {
        shell.beep();
      }
      return this.emitter.emit('did-beep');
    };

    NylasEnvConstructor.prototype.confirm = function(_arg) {
      var buttonLabels, buttons, callback, chosen, detailedMessage, message, _ref2;
      _ref2 = _arg != null ? _arg : {}, message = _ref2.message, detailedMessage = _ref2.detailedMessage, buttons = _ref2.buttons;
      if (buttons == null) {
        buttons = {};
      }
      if (_.isArray(buttons)) {
        buttonLabels = buttons;
      } else {
        buttonLabels = Object.keys(buttons);
      }
      chosen = remote.dialog.showMessageBox(this.getCurrentWindow(), {
        type: 'info',
        message: message,
        detail: detailedMessage,
        buttons: buttonLabels
      });
      if (_.isArray(buttons)) {
        return chosen;
      } else {
        callback = buttons[buttonLabels[chosen]];
        return typeof callback === "function" ? callback() : void 0;
      }
    };


    /*
    Section: Managing the Dev Tools
     */

    NylasEnvConstructor.prototype.openDevTools = function() {
      return ipcRenderer.send('call-webcontents-method', 'openDevTools');
    };

    NylasEnvConstructor.prototype.toggleDevTools = function() {
      return ipcRenderer.send('call-webcontents-method', 'toggleDevTools');
    };

    NylasEnvConstructor.prototype.executeJavaScriptInDevTools = function(code) {
      return ipcRenderer.send('call-devtools-webcontents-method', 'executeJavaScript', code);
    };


    /*
    Section: Private
     */

    NylasEnvConstructor.prototype.initializeReactRoot = function() {
      var React, ReactDOM, SheetContainer, startTime;
      startTime = Date.now();
      this.item = document.createElement("nylas-workspace");
      this.item.setAttribute("id", "sheet-container");
      this.item.setAttribute("class", "sheet-container");
      this.item.setAttribute("tabIndex", "-1");
      React = require("react");
      ReactDOM = require("react-dom");
      SheetContainer = require('./sheet-container');
      ReactDOM.render(React.createElement(SheetContainer), this.item);
      return document.querySelector(this.workspaceViewParentSelector).appendChild(this.item);
    };

    NylasEnvConstructor.prototype.deserializePackageStates = function() {
      var _ref2;
      this.packages.packageStates = (_ref2 = this.savedState.packageStates) != null ? _ref2 : {};
      return delete this.savedState.packageStates;
    };

    NylasEnvConstructor.prototype.loadThemes = function() {
      return this.themes.load();
    };

    NylasEnvConstructor.prototype.loadConfig = function() {
      this.config.setSchema(null, {
        type: 'object',
        properties: _.clone(require('./config-schema'))
      });
      return this.config.load();
    };

    NylasEnvConstructor.prototype.watchThemes = function() {
      return this.themes.onDidChangeActiveThemes((function(_this) {
        return function() {
          var pack, _i, _len, _ref2;
          _ref2 = _this.packages.getActivePackages();
          for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
            pack = _ref2[_i];
            if (pack.getType() !== 'theme') {
              if (typeof pack.reloadStylesheets === "function") {
                pack.reloadStylesheets();
              }
            }
          }
          return null;
        };
      })(this));
    };

    NylasEnvConstructor.prototype.exit = function(status) {
      var app;
      app = remote.app;
      app.emit('will-exit');
      return remote.process.exit(status);
    };

    NylasEnvConstructor.prototype.showOpenDialog = function(options, callback) {
      return callback(remote.dialog.showOpenDialog(this.getCurrentWindow(), options));
    };

    NylasEnvConstructor.prototype.showSaveDialog = function(options, callback) {
      if (options.title == null) {
        options.title = 'Save File';
      }
      return callback(remote.dialog.showSaveDialog(this.getCurrentWindow(), options));
    };

    NylasEnvConstructor.prototype.showErrorDialog = function(messageData, _arg) {
      var message, showInMainWindow, title, winToShow;
      showInMainWindow = (_arg != null ? _arg : {}).showInMainWindow;
      if (_.isString(messageData) || _.isNumber(messageData)) {
        message = messageData;
        title = "Error";
      } else if (_.isObject(messageData)) {
        message = messageData.message;
        title = messageData.title;
      } else {
        throw new Error("Must pass a valid message to show dialog", message);
      }
      winToShow = null;
      if (showInMainWindow) {
        winToShow = remote.getGlobal('application').getMainWindow();
      }
      return remote.dialog.showMessageBox(winToShow, {
        type: 'warning',
        buttons: ['Okay'],
        message: title,
        detail: message
      });
    };

    NylasEnvConstructor.prototype.fileListCache = function() {
      return remote.getGlobal('application').fileListCache;
    };

    NylasEnvConstructor.prototype.saveSync = function() {
      var statePath, stateString;
      stateString = JSON.stringify(this.savedState);
      if (statePath = this.constructor.getStatePath()) {
        return fs.writeFileSync(statePath, stateString, 'utf8');
      } else {
        return this.getCurrentWindow().loadSettings.windowState = stateString;
      }
    };

    NylasEnvConstructor.prototype.crashMainProcess = function() {
      return remote.process.crash();
    };

    NylasEnvConstructor.prototype.crashRenderProcess = function() {
      return process.crash();
    };

    NylasEnvConstructor.prototype.getUserInitScriptPath = function() {
      var initScriptPath;
      initScriptPath = fs.resolve(this.getConfigDirPath(), 'init', ['js', 'coffee']);
      return initScriptPath != null ? initScriptPath : path.join(this.getConfigDirPath(), 'init.coffee');
    };

    NylasEnvConstructor.prototype.requireUserInitScript = function() {
      var error, userInitScriptPath;
      if (userInitScriptPath = this.getUserInitScriptPath()) {
        try {
          if (fs.isFileSync(userInitScriptPath)) {
            return require(userInitScriptPath);
          }
        } catch (_error) {
          error = _error;
          return console.log(error);
        }
      }
    };

    NylasEnvConstructor.prototype.requireWithGlobals = function(id, globals) {
      var existingGlobals, key, value, _results;
      if (globals == null) {
        globals = {};
      }
      existingGlobals = {};
      for (key in globals) {
        value = globals[key];
        existingGlobals[key] = window[key];
        window[key] = value;
      }
      require(id);
      _results = [];
      for (key in existingGlobals) {
        value = existingGlobals[key];
        if (value === void 0) {
          _results.push(delete window[key]);
        } else {
          _results.push(window[key] = value);
        }
      }
      return _results;
    };

    NylasEnvConstructor.prototype.onUpdateAvailable = function(callback) {
      return this.emitter.on('update-available', callback);
    };

    NylasEnvConstructor.prototype.updateAvailable = function(details) {
      return this.emitter.emit('update-available', details);
    };

    NylasEnvConstructor.prototype.onBeforeUnload = function(callback) {
      return this.windowEventHandler.addUnloadCallback(callback);
    };

    NylasEnvConstructor.prototype.enhanceEventObject = function() {
      var overriddenStop;
      overriddenStop = Event.prototype.stopPropagation;
      Event.prototype.stopPropagation = function() {
        this.propagationStopped = true;
        return overriddenStop.apply(this, arguments);
      };
      return Event.prototype.isPropagationStopped = function() {
        return this.propagationStopped;
      };
    };

    return NylasEnvConstructor;

  })();

}).call(this);
