(function() {
  var MenuItem, NylasSpellchecker, path, remote, spellchecker,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
    __slice = [].slice;

  path = require('path');

  spellchecker = require('spellchecker');

  remote = require('electron').remote;

  MenuItem = remote.MenuItem;

  NylasSpellchecker = (function() {
    function NylasSpellchecker() {
      this.Spellchecker = __bind(this.Spellchecker, this);
      this.appendSpellingItemsToMenu = __bind(this.appendSpellingItemsToMenu, this);
      this.getCorrectionsForMisspelling = __bind(this.getCorrectionsForMisspelling, this);
      this.isMisspelled = __bind(this.isMisspelled, this);
      this.add = __bind(this.add, this);
      this.setDictionary = __bind(this.setDictionary, this);
      this.isSpelledCorrectly = __bind(this.isSpelledCorrectly, this);
      this.isLanguageAvailable = __bind(this.isLanguageAvailable, this);
      var lang;
      this.languageAvailable = false;
      this.cache = {};
      lang = navigator.language;
      this.setLanguage(lang);
    }

    NylasSpellchecker.prototype.isLanguageAvailable = function(lang) {
      var dicts, _ref, _ref1, _ref2;
      if (!lang) {
        return false;
      }
      dicts = this.getAvailableDictionaries();
      return (__indexOf.call(dicts, lang) >= 0) || (_ref = lang.split(/[-_]/)[0], __indexOf.call(dicts, _ref) >= 0) || (_ref1 = lang.replace('_', '-'), __indexOf.call(dicts, _ref1) >= 0) || (_ref2 = lang.replace('-', '_'), __indexOf.call(dicts, _ref2) >= 0);
    };

    NylasSpellchecker.prototype.isSpelledCorrectly = function() {
      var args;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return !this.isMisspelled.apply(this, args);
    };

    NylasSpellchecker.prototype.setLanguage = function(lang, dict) {
      var spellCheck;
      if (lang == null) {
        lang = "";
      }
      if (dict == null) {
        dict = this._getHunspellDictionary();
      }
      this.languageAvailable = this.isLanguageAvailable(lang);
      if (this.languageAvailable) {
        spellCheck = this.isSpelledCorrectly;
      } else {
        spellCheck = function() {
          return true;
        };
      }
      if (lang.length === 0) {
        lang = "en-US";
      }
      this._setWebframeSpellchecker(lang, spellCheck);
      return spellchecker.setDictionary(lang, dict);
    };

    NylasSpellchecker.prototype._setWebframeSpellchecker = function(lang, spellCheck) {
      return require('electron').webFrame.setSpellCheckProvider(lang, false, {
        spellCheck: spellCheck
      });
    };

    NylasSpellchecker.prototype._getHunspellDictionary = function() {
      var dict, unpacked;
      dict = path.join(require.resolve('spellchecker'), '..', '..', 'vendor', 'hunspell_dictionaries');
      try {
        unpacked = dict.replace('.asar' + path.sep, '.asar.unpacked' + path.sep);
        if (require('fs').statSyncNoException(unpacked)) {
          dict = unpacked;
        }
      } catch (_error) {

      }
      return dict;
    };

    NylasSpellchecker.prototype.setDictionary = function(lang, dict) {
      return this.setLanguage(lang, dict);
    };

    NylasSpellchecker.prototype.add = function(word) {
      delete this.cache[word];
      return spellchecker.add(word);
    };

    NylasSpellchecker.prototype.isMisspelled = function(word) {
      var _base;
      if (this.languageAvailable) {
        if ((_base = this.cache)[word] == null) {
          _base[word] = spellchecker.isMisspelled(word);
        }
        return this.cache[word];
      } else {
        return false;
      }
    };

    NylasSpellchecker.prototype.getAvailableDictionaries = function() {
      var arr, _ref;
      if (process.platform === 'linux') {
        arr = spellchecker.getAvailableDictionaries();
        if (__indexOf.call(arr, "en_US") < 0) {
          arr.push('en_US');
        }
        return arr;
      } else {
        return (_ref = spellchecker.getAvailableDictionaries()) != null ? _ref : [];
      }
    };

    NylasSpellchecker.prototype.getCorrectionsForMisspelling = function() {
      var args;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      if (this.languageAvailable) {
        return spellchecker.getCorrectionsForMisspelling.apply(spellchecker, args);
      } else {
        return [];
      }
    };

    NylasSpellchecker.prototype.appendSpellingItemsToMenu = function(_arg) {
      var corrections, menu, onCorrect, onDidLearn, word;
      menu = _arg.menu, word = _arg.word, onCorrect = _arg.onCorrect, onDidLearn = _arg.onDidLearn;
      if (this.isMisspelled(word)) {
        corrections = this.getCorrectionsForMisspelling(word);
        if (corrections.length > 0) {
          corrections.forEach((function(_this) {
            return function(correction) {
              return menu.append(new MenuItem({
                label: correction,
                click: function() {
                  return onCorrect(correction);
                }
              }));
            };
          })(this));
        } else {
          menu.append(new MenuItem({
            label: 'No Guesses Found',
            enabled: false
          }));
        }
        menu.append(new MenuItem({
          type: 'separator'
        }));
        menu.append(new MenuItem({
          label: 'Learn Spelling',
          click: (function(_this) {
            return function() {
              _this.add(word);
              return typeof onDidLearn === "function" ? onDidLearn(word) : void 0;
            };
          })(this)
        }));
        return menu.append(new MenuItem({
          type: 'separator'
        }));
      }
    };

    NylasSpellchecker.prototype.Spellchecker = function() {
      return this;
    };

    return NylasSpellchecker;

  })();

  module.exports = new NylasSpellchecker();

}).call(this);
