(function() {
  var APMWrapper, DatabaseStore, Emitter, EmitterMixin, Package, PackageManager, Q, ThemePackage, basePackagePaths, fs, path, remote, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __slice = [].slice,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  path = require('path');

  _ = require('underscore');

  remote = require('electron').remote;

  EmitterMixin = require('emissary').Emitter;

  Emitter = require('event-kit').Emitter;

  fs = require('fs-plus');

  Q = require('q');

  Package = require('./package');

  ThemePackage = require('./theme-package');

  DatabaseStore = require('./flux/stores/database-store');

  APMWrapper = require('./apm-wrapper');

  basePackagePaths = null;

  module.exports = PackageManager = (function() {
    EmitterMixin.includeInto(PackageManager);

    function PackageManager(_arg) {
      var configDirPath, safeMode;
      configDirPath = _arg.configDirPath, this.devMode = _arg.devMode, safeMode = _arg.safeMode, this.resourcePath = _arg.resourcePath, this.specMode = _arg.specMode;
      this._resolvePluginIdFor = __bind(this._resolvePluginIdFor, this);
      this.pluginIdFor = __bind(this.pluginIdFor, this);
      this.emitter = new Emitter;
      this.packageDirPaths = [];
      if (this.specMode) {
        this.packageDirPaths.push(path.join(this.resourcePath, "spec", "fixtures", "packages"));
      } else {
        this.packageDirPaths.push(path.join(this.resourcePath, "internal_packages"));
        if (!safeMode) {
          if (this.devMode) {
            this.packageDirPaths.push(path.join(configDirPath, "dev", "packages"));
          }
          this.packageDirPaths.push(path.join(configDirPath, "packages"));
        }
      }
      this.loadedPackages = {};
      this.cachedPackagePluginIds = {};
      this.packagesWithDatabaseObjects = [];
      this.activePackages = {};
      this.packageStates = {};
      this.packageActivators = [];
      this.registerPackageActivator(this, ['nylas']);
    }

    PackageManager.prototype.pluginIdFor = function(packageName) {
      var cacheKey, env;
      env = NylasEnv.config.get("env");
      cacheKey = "" + packageName + ":" + env;
      if (this.cachedPackagePluginIds[cacheKey] === void 0) {
        this.cachedPackagePluginIds[cacheKey] = this._resolvePluginIdFor(packageName, env);
      }
      return this.cachedPackagePluginIds[cacheKey];
    };

    PackageManager.prototype._resolvePluginIdFor = function(packageName, env) {
      var metadata, packagePath, _ref;
      metadata = (_ref = this.loadedPackages[packageName]) != null ? _ref.metadata : void 0;
      if (!metadata) {
        packagePath = this.resolvePackagePath(packageName);
        if (!packagePath) {
          return null;
        }
        metadata = Package.loadMetadata(packagePath);
      }
      if (metadata && metadata.appId instanceof Object) {
        return metadata.appId[env];
      }
      return null;
    };


    /*
    Section: Event Subscription
     */

    PackageManager.prototype.onDidLoadInitialPackages = function(callback) {
      return this.emitter.on('did-load-initial-packages', callback);
    };

    PackageManager.prototype.onDidActivateInitialPackages = function(callback) {
      return this.emitter.on('did-activate-initial-packages', callback);
    };

    PackageManager.prototype.onDidActivatePackage = function(callback) {
      return this.emitter.on('did-activate-package', callback);
    };

    PackageManager.prototype.onDidDeactivatePackage = function(callback) {
      return this.emitter.on('did-deactivate-package', callback);
    };

    PackageManager.prototype.onDidLoadPackage = function(callback) {
      return this.emitter.on('did-load-package', callback);
    };

    PackageManager.prototype.onDidUnloadPackage = function(callback) {
      return this.emitter.on('did-unload-package', callback);
    };


    /*
    Section: Package system data
     */

    PackageManager.prototype.getApmPath = function() {
      var commandName;
      if (this.apmPath != null) {
        return this.apmPath;
      }
      commandName = 'apm';
      if (process.platform === 'win32') {
        commandName += '.cmd';
      }
      this.apmPath = path.join(process.resourcesPath, 'app', 'apm', 'bin', commandName);
      if (!fs.isFileSync(this.apmPath)) {
        this.apmPath = path.join(this.resourcePath, 'apm', 'bin', commandName);
      }
      if (!fs.isFileSync(this.apmPath)) {
        this.apmPath = path.join(this.resourcePath, 'apm', 'node_modules', 'atom-package-manager', 'bin', commandName);
      }
      if (!fs.isFileSync(this.apmPath)) {
        this.apmPath = path.join('/bin', commandName);
      }
      return this.apmPath;
    };

    PackageManager.prototype.getPackageDirPaths = function() {
      return _.clone(this.packageDirPaths);
    };


    /*
    Section: General package data
     */

    PackageManager.prototype.resolvePackagePath = function(name) {
      var packagePath;
      if (fs.isDirectorySync(name)) {
        return name;
      }
      packagePath = fs.resolve.apply(fs, __slice.call(this.packageDirPaths).concat([name]));
      if (fs.isDirectorySync(packagePath)) {
        return packagePath;
      }
      packagePath = path.join(this.resourcePath, 'node_modules', name);
      if (this.hasNylasEngine(packagePath)) {
        return packagePath;
      }
    };

    PackageManager.prototype.isBundledPackage = function(name) {
      return this.getPackageDependencies().hasOwnProperty(name);
    };


    /*
    Section: Enabling and disabling packages
     */

    PackageManager.prototype.enablePackage = function(name) {
      var pack;
      pack = this.loadPackage(name);
      if (pack != null) {
        pack.enable();
      }
      return pack;
    };

    PackageManager.prototype.disablePackage = function(name) {
      var pack;
      pack = this.loadPackage(name);
      if (pack != null) {
        pack.disable();
      }
      return pack;
    };

    PackageManager.prototype.isPackageDisabled = function(name) {
      var _ref;
      return _.include((_ref = NylasEnv.config.get('core.disabledPackages')) != null ? _ref : [], name);
    };


    /*
    Section: Accessing active packages
     */

    PackageManager.prototype.getActivePackages = function() {
      return _.values(this.activePackages);
    };

    PackageManager.prototype.getActivePackage = function(name) {
      return this.activePackages[name];
    };

    PackageManager.prototype.isPackageActive = function(name) {
      return this.getActivePackage(name) != null;
    };


    /*
    Section: Accessing loaded packages
     */

    PackageManager.prototype.getLoadedPackages = function() {
      return _.values(this.loadedPackages);
    };

    PackageManager.prototype.getLoadedPackagesForTypes = function(types) {
      var pack, _i, _len, _ref, _ref1, _results;
      _ref = this.getLoadedPackages();
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        pack = _ref[_i];
        if (_ref1 = pack.getType(), __indexOf.call(types, _ref1) >= 0) {
          _results.push(pack);
        }
      }
      return _results;
    };

    PackageManager.prototype.getLoadedPackage = function(name) {
      return this.loadedPackages[name];
    };

    PackageManager.prototype.getPluginIdsByPathBase = function() {
      var id, name, pack, pathBase, pluginIdsByPathBase, _ref;
      pluginIdsByPathBase = {};
      _ref = this.loadedPackages;
      for (name in _ref) {
        pack = _ref[name];
        pathBase = _.last(pack.path.split("/"));
        if (pack.pluginId() && pack.pluginId() !== name) {
          id = "" + name + "-" + (pack.pluginId());
        } else {
          id = pack.pluginId();
        }
        pluginIdsByPathBase[pathBase] = id;
      }
      return pluginIdsByPathBase;
    };

    PackageManager.prototype.isPackageLoaded = function(name) {
      return this.getLoadedPackage(name) != null;
    };


    /*
    Section: Accessing available packages
     */

    PackageManager.prototype.getAvailablePackagePaths = function(windowType) {
      var cache, loadPackagesWhenNoTypesSpecified, packageDirPath, packageName, packagePath, packagePaths, packageVersion, packagesPath, _i, _j, _len, _len1, _ref, _ref1, _ref2, _ref3;
      packagePaths = [];
      loadPackagesWhenNoTypesSpecified = windowType === 'default';
      if (basePackagePaths == null) {
        basePackagePaths = (_ref = NylasEnv.fileListCache().basePackagePaths) != null ? _ref : [];
      }
      if (basePackagePaths.length === 0) {
        _ref1 = this.packageDirPaths;
        for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
          packageDirPath = _ref1[_i];
          _ref2 = fs.listSync(packageDirPath);
          for (_j = 0, _len1 = _ref2.length; _j < _len1; _j++) {
            packagePath = _ref2[_j];
            if (!fs.isDirectorySync(packagePath)) {
              continue;
            }
            if (path.basename(packagePath)[0] === '.') {
              continue;
            }
            packagePaths.push(packagePath);
          }
        }
        basePackagePaths = packagePaths;
        cache = NylasEnv.fileListCache();
        cache.basePackagePaths = basePackagePaths;
      } else {
        packagePaths = basePackagePaths;
      }
      if (windowType) {
        packagePaths = _.filter(packagePaths, function(packagePath) {
          var metadata, windowTypes, _ref3, _ref4;
          try {
            metadata = (_ref3 = Package.loadMetadata(packagePath)) != null ? _ref3 : {};
            if (!((_ref4 = metadata.engines) != null ? _ref4.nylas : void 0)) {
              console.error("INVALID PACKAGE: Your package at " + packagePath + " does not have a properly formatted `package.json`. You must include an {'engines': {'nylas': version}} property");
            }
            windowTypes = metadata.windowTypes;
            if (windowTypes) {
              return (windowTypes[windowType] != null) || (windowTypes["all"] != null);
            } else if (loadPackagesWhenNoTypesSpecified) {
              return true;
            }
            return false;
          } catch (_error) {
            return false;
          }
        });
      }
      packagesPath = path.join(this.resourcePath, 'node_modules');
      _ref3 = this.getPackageDependencies();
      for (packageName in _ref3) {
        packageVersion = _ref3[packageName];
        packagePath = path.join(packagesPath, packageName);
        if (fs.isDirectorySync(packagePath)) {
          packagePaths.push(packagePath);
        }
      }
      return _.uniq(packagePaths);
    };

    PackageManager.prototype.getAvailablePackageNames = function() {
      return _.uniq(_.map(this.getAvailablePackagePaths(), function(packagePath) {
        return path.basename(packagePath);
      }));
    };

    PackageManager.prototype.getAvailablePackageMetadata = function() {
      var metadata, name, packagePath, packages, _i, _len, _ref, _ref1, _ref2;
      packages = [];
      _ref = this.getAvailablePackagePaths();
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        packagePath = _ref[_i];
        name = path.basename(packagePath);
        metadata = (_ref1 = (_ref2 = this.getLoadedPackage(name)) != null ? _ref2.metadata : void 0) != null ? _ref1 : Package.loadMetadata(packagePath, true);
        packages.push(metadata);
      }
      return packages;
    };

    PackageManager.prototype.installPackageFromPath = function(packageSourceDir, callback) {
      var e, json, jsonPath, packageName, packageTargetDir, packagesDir;
      jsonPath = path.join(packageSourceDir, 'package.json');
      if (!fs.existsSync(jsonPath)) {
        return callback(new Error("The folder you selected doesn't look like a valid N1 plugin. All N1 plugins must have a package.json file in the top level of the folder. Check the contents of " + packageSourceDir + " and try again."), null);
      }
      try {
        json = JSON.parse(fs.readFileSync(jsonPath));
      } catch (_error) {
        e = _error;
        return callback(e, null);
      }
      if (!json.name) {
        return callback(new Error("The package.json file must contain a valid `name` value."), null);
      }
      packagesDir = path.join(NylasEnv.getConfigDirPath(), 'packages');
      packageName = json.name;
      packageTargetDir = path.join(packagesDir, packageName);
      return fs.makeTree(packagesDir, (function(_this) {
        return function(err) {
          if (err) {
            return callback(err, null);
          }
          return fs.exists(packageTargetDir, function(packageAlreadyExists) {
            var apm;
            if (packageAlreadyExists) {
              return callback(new Error("A package named '" + packageName + "' is already installed in ~/.nylas/packages."), null);
            }
            fs.copySync(packageSourceDir, packageTargetDir);
            apm = new APMWrapper();
            return apm.installDependenciesInPackageDirectory(packageTargetDir, function(err) {
              if (err) {
                return callback(err, packageTargetDir);
              }
              _this.enablePackage(packageTargetDir);
              _this.activatePackage(packageName);
              return callback(null, packageName);
            });
          });
        };
      })(this));
    };


    /*
    Section: Private
     */

    PackageManager.prototype.getPackageState = function(name) {
      return this.packageStates[name];
    };

    PackageManager.prototype.setPackageState = function(name, state) {
      return this.packageStates[name] = state;
    };

    PackageManager.prototype.getPackageDependencies = function() {
      var metadataPath, _ref;
      if (this.packageDependencies == null) {
        try {
          metadataPath = path.join(this.resourcePath, 'package.json');
          this.packageDependencies = ((_ref = JSON.parse(fs.readFileSync(metadataPath))) != null ? _ref : {}).packageDependencies;
        } catch (_error) {}
        if (this.packageDependencies == null) {
          this.packageDependencies = {};
        }
      }
      return this.packageDependencies;
    };

    PackageManager.prototype.hasNylasEngine = function(packagePath) {
      var metadata, _ref;
      metadata = Package.loadMetadata(packagePath, true);
      return (metadata != null ? (_ref = metadata.engines) != null ? _ref.nylas : void 0 : void 0) != null;
    };

    PackageManager.prototype.unobserveDisabledPackages = function() {
      var _ref;
      if ((_ref = this.disabledPackagesSubscription) != null) {
        _ref.dispose();
      }
      return this.disabledPackagesSubscription = null;
    };

    PackageManager.prototype.observeDisabledPackages = function() {
      return this.disabledPackagesSubscription != null ? this.disabledPackagesSubscription : this.disabledPackagesSubscription = NylasEnv.config.onDidChange('core.disabledPackages', (function(_this) {
        return function(_arg) {
          var newValue, oldValue, packageName, packagesToDisable, packagesToEnable, _i, _j, _k, _len, _len1, _len2;
          newValue = _arg.newValue, oldValue = _arg.oldValue;
          packagesToEnable = _.difference(oldValue, newValue);
          packagesToDisable = _.difference(newValue, oldValue);
          for (_i = 0, _len = packagesToDisable.length; _i < _len; _i++) {
            packageName = packagesToDisable[_i];
            if (_this.getActivePackage(packageName)) {
              _this.deactivatePackage(packageName);
            }
          }
          for (_j = 0, _len1 = packagesToEnable.length; _j < _len1; _j++) {
            packageName = packagesToEnable[_j];
            _this.loadPackage(packageName);
          }
          _this.refreshDatabaseSchema();
          for (_k = 0, _len2 = packagesToEnable.length; _k < _len2; _k++) {
            packageName = packagesToEnable[_k];
            _this.activatePackage(packageName);
          }
          return null;
        };
      })(this));
    };

    PackageManager.prototype.loadPackages = function(windowType) {
      var packagePath, packagePaths, _i, _len;
      packagePaths = this.getAvailablePackagePaths(windowType);
      packagePaths = packagePaths.filter((function(_this) {
        return function(packagePath) {
          return !_this.isPackageDisabled(path.basename(packagePath));
        };
      })(this));
      packagePaths = _.uniq(packagePaths, function(packagePath) {
        return path.basename(packagePath);
      });
      for (_i = 0, _len = packagePaths.length; _i < _len; _i++) {
        packagePath = packagePaths[_i];
        this.loadPackage(packagePath);
      }
      this.emit('loaded');
      return this.emitter.emit('did-load-initial-packages');
    };

    PackageManager.prototype.loadPackage = function(nameOrPath) {
      var error, metadata, name, pack, packagePath, _ref, _ref1;
      if (pack = this.getLoadedPackage(nameOrPath)) {
        return pack;
      }
      if (packagePath = this.resolvePackagePath(nameOrPath)) {
        name = path.basename(nameOrPath);
        if (pack = this.getLoadedPackage(name)) {
          return pack;
        }
        try {
          metadata = (_ref = Package.loadMetadata(packagePath)) != null ? _ref : {};
          if (metadata.theme) {
            pack = new ThemePackage(packagePath, metadata);
          } else {
            pack = new Package(packagePath, metadata);
          }
          pack.load();
          if (pack.declaresNewDatabaseObjects) {
            this.packagesWithDatabaseObjects.push(pack);
          }
          this.loadedPackages[pack.name] = pack;
          this.emitter.emit('did-load-package', pack);
          return pack;
        } catch (_error) {
          error = _error;
          console.warn("Failed to load package.json '" + (path.basename(packagePath)) + "'");
          console.warn((_ref1 = error.stack) != null ? _ref1 : error);
        }
      } else {
        console.warn("Could not resolve '" + nameOrPath + "' to a package path");
      }
      return null;
    };

    PackageManager.prototype.unloadPackages = function() {
      var name, _i, _len, _ref;
      _ref = Object.keys(this.loadedPackages);
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        name = _ref[_i];
        this.unloadPackage(name);
      }
      return null;
    };

    PackageManager.prototype.unloadPackage = function(name) {
      var pack;
      if (this.isPackageActive(name)) {
        throw new Error("Tried to unload active package '" + name + "'");
      }
      if (pack = this.getLoadedPackage(name)) {
        delete this.loadedPackages[pack.name];
        return this.emitter.emit('did-unload-package', pack);
      } else {
        throw new Error("No loaded package for name '" + name + "'");
      }
    };

    PackageManager.prototype.activate = function() {
      var activator, packages, promises, types, _i, _len, _ref, _ref1;
      promises = [];
      _ref = this.packageActivators;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        _ref1 = _ref[_i], activator = _ref1[0], types = _ref1[1];
        packages = this.getLoadedPackagesForTypes(types);
        promises = promises.concat(activator.activatePackages(packages));
      }
      return Q.all(promises).then((function(_this) {
        return function() {
          _this.emit('activated');
          return _this.emitter.emit('did-activate-initial-packages');
        };
      })(this));
    };

    PackageManager.prototype.registerPackageActivator = function(activator, types) {
      return this.packageActivators.push([activator, types]);
    };

    PackageManager.prototype.activatePackages = function(packages) {
      var promises;
      promises = [];
      NylasEnv.config.transact((function(_this) {
        return function() {
          var pack, promise, _i, _j, _len, _len1, _results;
          for (_i = 0, _len = packages.length; _i < _len; _i++) {
            pack = packages[_i];
            _this.loadPackage(pack.name);
          }
          _this.refreshDatabaseSchema();
          _results = [];
          for (_j = 0, _len1 = packages.length; _j < _len1; _j++) {
            pack = packages[_j];
            promise = _this.activatePackage(pack.name);
            _results.push(promises.push(promise));
          }
          return _results;
        };
      })(this));
      this.observeDisabledPackages();
      return promises;
    };

    PackageManager.prototype.refreshDatabaseSchema = function() {
      if (this.packagesWithDatabaseObjects.length > 0) {
        DatabaseStore.refreshDatabaseSchema();
        return this.packagesWithDatabaseObjects = [];
      }
    };

    PackageManager.prototype.activatePackage = function(name) {
      var pack;
      if (pack = this.getActivePackage(name)) {
        return Q(pack);
      } else if (pack = this.loadPackage(name)) {
        return pack.activate().then((function(_this) {
          return function() {
            _this.activePackages[pack.name] = pack;
            _this.emitter.emit('did-activate-package', pack);
            return pack;
          };
        })(this));
      } else {
        return Q.reject(new Error("Failed to load package '" + name + "'"));
      }
    };

    PackageManager.prototype.deactivatePackages = function() {
      NylasEnv.config.transact((function(_this) {
        return function() {
          var pack, _i, _len, _ref, _results;
          _ref = _this.getLoadedPackages();
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            pack = _ref[_i];
            _results.push(_this.deactivatePackage(pack.name));
          }
          return _results;
        };
      })(this));
      return this.unobserveDisabledPackages();
    };

    PackageManager.prototype.deactivatePackage = function(name) {
      var pack, state;
      pack = this.getLoadedPackage(name);
      if (this.isPackageActive(name)) {
        if (state = typeof pack.serialize === "function" ? pack.serialize() : void 0) {
          this.setPackageState(pack.name, state);
        }
      }
      pack.deactivate();
      delete this.activePackages[pack.name];
      return this.emitter.emit('did-deactivate-package', pack);
    };

    return PackageManager;

  })();

}).call(this);
