(function() {
  var CompositeDisposable, DatabaseObjectRegistry, Emitter, EmitterMixin, ModuleCache, Package, Q, TaskRegistry, async, deprecate, error, fs, packagesCache, path, _, _ref, _ref1, _ref2,
    __slice = [].slice;

  path = require('path');

  _ = require('underscore');

  async = require('async');

  fs = require('fs-plus');

  EmitterMixin = require('emissary').Emitter;

  _ref = require('event-kit'), Emitter = _ref.Emitter, CompositeDisposable = _ref.CompositeDisposable;

  Q = require('q');

  deprecate = require('grim').deprecate;

  ModuleCache = require('./module-cache');

  TaskRegistry = require('./task-registry')["default"];

  DatabaseObjectRegistry = require('./database-object-registry')["default"];

  try {
    packagesCache = (_ref1 = (_ref2 = require('../package.json')) != null ? _ref2._N1Packages : void 0) != null ? _ref1 : {};
  } catch (_error) {
    error = _error;
    packagesCache = {};
  }

  module.exports = Package = (function() {
    EmitterMixin.includeInto(Package);

    Package.isBundledPackagePath = function(packagePath) {
      if (NylasEnv.packages.devMode) {
        if (!NylasEnv.packages.resourcePath.startsWith("" + process.resourcesPath + path.sep)) {
          return false;
        }
      }
      if (this.resourcePathWithTrailingSlash == null) {
        this.resourcePathWithTrailingSlash = "" + NylasEnv.packages.resourcePath + path.sep;
      }
      return packagePath != null ? packagePath.startsWith(this.resourcePathWithTrailingSlash) : void 0;
    };

    Package.loadMetadata = function(packagePath, ignoreErrors) {
      var metadata, metadataPath, packageName, _ref3;
      if (ignoreErrors == null) {
        ignoreErrors = false;
      }
      packageName = path.basename(packagePath);
      if (this.isBundledPackagePath(packagePath)) {
        metadata = (_ref3 = packagesCache[packageName]) != null ? _ref3.metadata : void 0;
      }
      if (metadata == null) {
        metadataPath = fs.resolve(path.join(packagePath, 'package.json'));
        if (fs.existsSync(metadataPath)) {
          try {
            metadata = JSON.parse(fs.readFileSync(metadataPath));
          } catch (_error) {
            error = _error;
            if (!ignoreErrors) {
              throw error;
            }
          }
        }
      }
      if (metadata == null) {
        metadata = {};
      }
      metadata.name = packageName;
      if (metadata.stylesheets != null) {
        deprecate("Use the `styleSheets` key instead of `stylesheets` in the `package.json` of `" + packageName + "`", {
          packageName: packageName
        });
        metadata.styleSheets = metadata.stylesheets;
      }
      return metadata;
    };

    Package.prototype.keymaps = null;

    Package.prototype.menus = null;

    Package.prototype.stylesheets = null;

    Package.prototype.stylesheetDisposables = null;

    Package.prototype.mainModulePath = null;

    Package.prototype.resolvedMainModulePath = false;

    Package.prototype.mainModule = null;


    /*
    Section: Construction
     */

    function Package(path, metadata) {
      var _ref3, _ref4, _ref5, _ref6, _ref7;
      this.path = path;
      this.metadata = metadata;
      this.emitter = new Emitter;
      if (this.metadata == null) {
        this.metadata = Package.loadMetadata(this.path);
      }
      this.bundledPackage = Package.isBundledPackagePath(this.path);
      this.name = (_ref3 = (_ref4 = this.metadata) != null ? _ref4.name : void 0) != null ? _ref3 : path.basename(this.path);
      if (this.metadata.appId) {
        if (_.isString(this.metadata.appId)) {
          this.pluginAppId = (_ref5 = this.metadata.appId) != null ? _ref5 : null;
        } else if (_.isObject(this.metadata.appId)) {
          this.pluginAppId = (_ref6 = this.metadata.appId[NylasEnv.config.get('env')]) != null ? _ref6 : null;
        } else {
          this.pluginAppId = null;
        }
      } else {
        this.pluginAppId = null;
      }
      this.displayName = ((_ref7 = this.metadata) != null ? _ref7.displayName : void 0) || this.name;
      ModuleCache.add(this.path, this.metadata);
      this.reset();
      this.declaresNewDatabaseObjects = false;
    }

    Package.prototype.pluginId = function() {
      var _ref3;
      return (_ref3 = this.pluginAppId) != null ? _ref3 : this.name;
    };


    /*
    Section: Event Subscription
     */

    Package.prototype.onDidDeactivate = function(callback) {
      return this.emitter.on('did-deactivate', callback);
    };


    /*
    Section: Instance Methods
     */

    Package.prototype.enable = function() {
      return NylasEnv.config.removeAtKeyPath('core.disabledPackages', this.name);
    };

    Package.prototype.disable = function() {
      return NylasEnv.config.pushAtKeyPath('core.disabledPackages', this.name);
    };

    Package.prototype.isTheme = function() {
      var _ref3;
      return ((_ref3 = this.metadata) != null ? _ref3.theme : void 0) != null;
    };

    Package.prototype.measure = function(key, fn) {
      var startTime, value;
      startTime = Date.now();
      value = fn();
      this[key] = Date.now() - startTime;
      return value;
    };

    Package.prototype.getType = function() {
      return 'nylas';
    };

    Package.prototype.getStyleSheetPriority = function() {
      return 0;
    };

    Package.prototype.load = function() {
      this.measure('loadTime', (function(_this) {
        return function() {
          var mainModule, _ref3;
          try {
            _this.declaresNewDatabaseObjects = false;
            _this.loadKeymaps();
            _this.loadMenus();
            _this.loadStylesheets();
            mainModule = _this.requireMainModule();
            if (!mainModule) {
              return;
            }
            _this.registerModelConstructors(mainModule.modelConstructors);
            return _this.registerTaskConstructors(mainModule.taskConstructors);
          } catch (_error) {
            error = _error;
            console.warn("Failed to load package named '" + _this.name + "'");
            console.warn((_ref3 = error.stack) != null ? _ref3 : error);
            return console.error(error.message, error);
          }
        };
      })(this));
      return this;
    };

    Package.prototype.registerModelConstructors = function(constructors) {
      if (constructors == null) {
        constructors = [];
      }
      if (constructors.length > 0) {
        this.declaresNewDatabaseObjects = true;
        return _.each(constructors, function(constructor) {
          var constructorFactory;
          constructorFactory = function() {
            return constructor;
          };
          return DatabaseObjectRegistry.register(constructor.name, constructorFactory);
        });
      }
    };

    Package.prototype.registerTaskConstructors = function(constructors) {
      if (constructors == null) {
        constructors = [];
      }
      return _.each(constructors, function(constructor) {
        var constructorFactory;
        constructorFactory = function() {
          return constructor;
        };
        return TaskRegistry.register(constructor.name, constructorFactory);
      });
    };

    Package.prototype.reset = function() {
      this.stylesheets = [];
      this.keymaps = [];
      return this.menus = [];
    };

    Package.prototype.activate = function() {
      if (this.activationDeferred == null) {
        this.activationDeferred = Q.defer();
        this.measure('activateTime', (function(_this) {
          return function() {
            _this.activateResources();
            return _this.activateNow();
          };
        })(this));
      }
      return Q.all([this.activationDeferred.promise]);
    };

    Package.prototype.activateNow = function() {
      var e, localState, _ref3, _ref4;
      try {
        this.activateConfig();
        this.activateStylesheets();
        if (this.requireMainModule()) {
          localState = (_ref3 = NylasEnv.packages.getPackageState(this.name)) != null ? _ref3 : {};
          this.mainModule.activate(localState);
          this.mainActivated = true;
        }
      } catch (_error) {
        e = _error;
        console.log(e.message);
        console.log(e.stack);
        console.warn("Failed to activate package named '" + this.name + "'", e.stack);
      }
      return (_ref4 = this.activationDeferred) != null ? _ref4.resolve() : void 0;
    };

    Package.prototype.activateConfig = function() {
      var _base;
      if (this.configActivated) {
        return;
      }
      this.requireMainModule();
      if (this.mainModule != null) {
        if ((this.mainModule.config != null) && typeof this.mainModule.config === 'object') {
          NylasEnv.config.setSchema(this.name, {
            type: 'object',
            properties: this.mainModule.config
          });
        } else if ((this.mainModule.configDefaults != null) && typeof this.mainModule.configDefaults === 'object') {
          NylasEnv.config.setDefaults(this.name, this.mainModule.configDefaults);
        }
        if (typeof (_base = this.mainModule).activateConfig === "function") {
          _base.activateConfig();
        }
      }
      return this.configActivated = true;
    };

    Package.prototype.activateStylesheets = function() {
      var context, match, priority, source, sourcePath, _i, _len, _ref3, _ref4;
      if (this.stylesheetsActivated) {
        return;
      }
      this.stylesheetDisposables = new CompositeDisposable;
      priority = this.getStyleSheetPriority();
      _ref3 = this.stylesheets;
      for (_i = 0, _len = _ref3.length; _i < _len; _i++) {
        _ref4 = _ref3[_i], sourcePath = _ref4[0], source = _ref4[1];
        if (match = path.basename(sourcePath).match(/[^.]*\.([^.]*)\./)) {
          context = match[1];
        } else {
          context = void 0;
        }
        this.stylesheetDisposables.add(NylasEnv.styles.addStyleSheet(source, {
          sourcePath: sourcePath,
          priority: priority,
          context: context
        }));
      }
      return this.stylesheetsActivated = true;
    };

    Package.prototype.activateResources = function() {
      var keymapPath, map, menuPath, _i, _j, _len, _len1, _ref3, _ref4, _ref5, _ref6, _results;
      this.activationDisposables = new CompositeDisposable;
      _ref3 = this.keymaps;
      for (_i = 0, _len = _ref3.length; _i < _len; _i++) {
        _ref4 = _ref3[_i], keymapPath = _ref4[0], map = _ref4[1];
        this.activationDisposables.add(NylasEnv.keymaps.loadKeymap(keymapPath, map));
      }
      _ref5 = this.menus;
      _results = [];
      for (_j = 0, _len1 = _ref5.length; _j < _len1; _j++) {
        _ref6 = _ref5[_j], menuPath = _ref6[0], map = _ref6[1];
        if (map['menu'] != null) {
          _results.push(this.activationDisposables.add(NylasEnv.menu.add(map['menu'])));
        }
      }
      return _results;
    };

    Package.prototype.loadKeymaps = function() {
      var e, keymapObject, keymapPath;
      try {
        if (this.bundledPackage && (packagesCache[this.name] != null)) {
          return this.keymaps = (function() {
            var _ref3, _results;
            _ref3 = packagesCache[this.name].keymaps;
            _results = [];
            for (keymapPath in _ref3) {
              keymapObject = _ref3[keymapPath];
              _results.push(["" + NylasEnv.packages.resourcePath + path.sep + keymapPath, keymapObject]);
            }
            return _results;
          }).call(this);
        } else {
          return this.keymaps = this.getKeymapPaths().map(function(keymapPath) {
            var _ref3;
            return [keymapPath, (_ref3 = JSON.parse(fs.readFileSync(keymapPath))) != null ? _ref3 : {}];
          });
        }
      } catch (_error) {
        e = _error;
        return console.error("Error reading keymaps for package '" + this.name + "': " + e.message, e.stack);
      }
    };

    Package.prototype.loadMenus = function() {
      var e, menuObject, menuPath;
      try {
        if (this.bundledPackage && (packagesCache[this.name] != null)) {
          return this.menus = (function() {
            var _ref3, _results;
            _ref3 = packagesCache[this.name].menus;
            _results = [];
            for (menuPath in _ref3) {
              menuObject = _ref3[menuPath];
              _results.push(["" + NylasEnv.packages.resourcePath + path.sep + menuPath, menuObject]);
            }
            return _results;
          }).call(this);
        } else {
          return this.menus = this.getMenuPaths().map(function(menuPath) {
            var _ref3;
            return [menuPath, (_ref3 = JSON.parse(fs.readFileSync(menuPath))) != null ? _ref3 : {}];
          });
        }
      } catch (_error) {
        e = _error;
        return console.error("Error reading menus for package '" + this.name + "': " + e.message, e.stack);
      }
    };

    Package.prototype.getKeymapPaths = function() {
      var keymapsDirPath;
      keymapsDirPath = path.join(this.path, 'keymaps');
      if (this.metadata.keymaps) {
        return this.metadata.keymaps.map(function(name) {
          return fs.resolve(keymapsDirPath, name, ['json', '']);
        });
      } else {
        return fs.listSync(keymapsDirPath, ['json']);
      }
    };

    Package.prototype.getMenuPaths = function() {
      var menusDirPath;
      menusDirPath = path.join(this.path, 'menus');
      if (this.metadata.menus) {
        return this.metadata.menus.map(function(name) {
          return fs.resolve(menusDirPath, name, ['json', '']);
        });
      } else {
        return fs.listSync(menusDirPath, ['json']);
      }
    };

    Package.prototype.loadStylesheets = function() {
      return this.stylesheets = this.getStylesheetPaths().map(function(stylesheetPath) {
        return [stylesheetPath, NylasEnv.themes.loadStylesheet(stylesheetPath, true)];
      });
    };

    Package.prototype.getStylesheetsPath = function() {
      if (fs.isDirectorySync(path.join(this.path, 'stylesheets'))) {
        deprecate("Store package style sheets in the `styles/` directory instead of `stylesheets/` in the `" + this.name + "` package", {
          packageName: this.name
        });
        return path.join(this.path, 'stylesheets');
      } else {
        return path.join(this.path, 'styles');
      }
    };

    Package.prototype.getStylesheetPaths = function() {
      var indexStylesheet, stylesheetDirPath;
      stylesheetDirPath = this.getStylesheetsPath();
      if (this.metadata.mainStyleSheet) {
        return [fs.resolve(this.path, this.metadata.mainStyleSheet)];
      } else if (this.metadata.styleSheets) {
        return this.metadata.styleSheets.map(function(name) {
          return fs.resolve(stylesheetDirPath, name, ['css', 'less', '']);
        });
      } else if (indexStylesheet = fs.resolve(this.path, 'index', ['css', 'less'])) {
        return [indexStylesheet];
      } else {
        return _.filter(fs.listSync(stylesheetDirPath, ['css', 'less']), function(file) {
          return path.basename(file)[0] !== '.';
        });
      }
    };

    Package.prototype.serialize = function() {
      var e, _ref3;
      if (this.mainActivated) {
        try {
          return (_ref3 = this.mainModule) != null ? typeof _ref3.serialize === "function" ? _ref3.serialize() : void 0 : void 0;
        } catch (_error) {
          e = _error;
          return console.error("Error serializing package '" + this.name + "'", e.stack);
        }
      }
    };

    Package.prototype.deactivate = function() {
      var e, _ref3, _ref4;
      if ((_ref3 = this.activationDeferred) != null) {
        _ref3.reject();
      }
      this.activationDeferred = null;
      this.deactivateResources();
      this.deactivateConfig();
      if (this.mainActivated) {
        try {
          if ((_ref4 = this.mainModule) != null) {
            if (typeof _ref4.deactivate === "function") {
              _ref4.deactivate();
            }
          }
        } catch (_error) {
          e = _error;
          console.error("Error deactivating package '" + this.name + "'", e.stack);
        }
      }
      this.emit('deactivated');
      return this.emitter.emit('did-deactivate');
    };

    Package.prototype.deactivateConfig = function() {
      var _ref3;
      if ((_ref3 = this.mainModule) != null) {
        if (typeof _ref3.deactivateConfig === "function") {
          _ref3.deactivateConfig();
        }
      }
      return this.configActivated = false;
    };

    Package.prototype.deactivateResources = function() {
      var _ref3, _ref4;
      if ((_ref3 = this.stylesheetDisposables) != null) {
        _ref3.dispose();
      }
      if ((_ref4 = this.activationDisposables) != null) {
        _ref4.dispose();
      }
      return this.stylesheetsActivated = false;
    };

    Package.prototype.reloadStylesheets = function() {
      var oldSheets, _ref3;
      oldSheets = _.clone(this.stylesheets);
      this.loadStylesheets();
      if ((_ref3 = this.stylesheetDisposables) != null) {
        _ref3.dispose();
      }
      this.stylesheetDisposables = new CompositeDisposable;
      this.stylesheetsActivated = false;
      return this.activateStylesheets();
    };

    Package.prototype.requireMainModule = function() {
      var mainModulePath;
      if (this.mainModule != null) {
        return this.mainModule;
      }
      if (!this.isCompatible()) {
        console.warn("Failed to require the main module of '" + this.name + "' because it requires an incompatible native module.\nRun `apm rebuild` in the package directory to resolve.");
        return;
      }
      mainModulePath = this.getMainModulePath();
      if (fs.isFileSync(mainModulePath)) {
        this.mainModule = require(mainModulePath);
      }
      return this.mainModule;
    };

    Package.prototype.getMainModulePath = function() {
      var mainModulePath;
      if (this.resolvedMainModulePath) {
        return this.mainModulePath;
      }
      this.resolvedMainModulePath = true;
      if (this.bundledPackage && (packagesCache[this.name] != null)) {
        if (packagesCache[this.name].main) {
          this.mainModulePath = "" + NylasEnv.packages.resourcePath + path.sep + packagesCache[this.name].main;
          return this.mainModulePath = fs.resolveExtension(this.mainModulePath, [""].concat(__slice.call(Object.keys(require.extensions))));
        } else {
          return this.mainModulePath = null;
        }
      } else {
        mainModulePath = this.metadata.main ? path.join(this.path, this.metadata.main) : path.join(this.path, 'index');
        return this.mainModulePath = fs.resolveExtension(mainModulePath, [""].concat(__slice.call(Object.keys(require.extensions))));
      }
    };

    Package.prototype.isNativeModule = function(modulePath) {
      try {
        return fs.listSync(path.join(modulePath, 'build', 'Release'), ['.node']).length > 0;
      } catch (_error) {
        error = _error;
        return false;
      }
    };

    Package.prototype.getNativeModuleDependencyPaths = function() {
      var nativeModulePaths, traversePath;
      nativeModulePaths = [];
      traversePath = (function(_this) {
        return function(nodeModulesPath) {
          var modulePath, _i, _len, _ref3, _results;
          try {
            _ref3 = fs.listSync(nodeModulesPath);
            _results = [];
            for (_i = 0, _len = _ref3.length; _i < _len; _i++) {
              modulePath = _ref3[_i];
              if (_this.isNativeModule(modulePath)) {
                nativeModulePaths.push(modulePath);
              }
              _results.push(traversePath(path.join(modulePath, 'node_modules')));
            }
            return _results;
          } catch (_error) {}
        };
      })(this);
      traversePath(path.join(this.path, 'node_modules'));
      return nativeModulePaths;
    };

    Package.prototype.getIncompatibleNativeModules = function() {
      var incompatibleNativeModules, localStorageKey, nativeModulePath, version, _i, _len, _ref3, _ref4;
      localStorageKey = "installed-packages:" + this.name + ":" + this.metadata.version;
      if (!NylasEnv.inDevMode()) {
        try {
          incompatibleNativeModules = ((_ref3 = JSON.parse(global.localStorage.getItem(localStorageKey))) != null ? _ref3 : {}).incompatibleNativeModules;
        } catch (_error) {}
        if (incompatibleNativeModules != null) {
          return incompatibleNativeModules;
        }
      }
      incompatibleNativeModules = [];
      _ref4 = this.getNativeModuleDependencyPaths();
      for (_i = 0, _len = _ref4.length; _i < _len; _i++) {
        nativeModulePath = _ref4[_i];
        try {
          require(nativeModulePath);
        } catch (_error) {
          error = _error;
          try {
            version = require("" + nativeModulePath + "/package.json").version;
          } catch (_error) {}
          incompatibleNativeModules.push({
            path: nativeModulePath,
            name: path.basename(nativeModulePath),
            version: version,
            error: error.message
          });
        }
      }
      global.localStorage.setItem(localStorageKey, JSON.stringify({
        incompatibleNativeModules: incompatibleNativeModules
      }));
      return incompatibleNativeModules;
    };

    Package.prototype.isCompatible = function() {
      var packageMain;
      if (this.compatible != null) {
        return this.compatible;
      }
      if (this.path.indexOf(path.join(NylasEnv.packages.resourcePath, 'node_modules') + path.sep) === 0) {
        return this.compatible = true;
      } else if (packageMain = this.getMainModulePath()) {
        this.incompatibleModules = this.getIncompatibleNativeModules();
        return this.compatible = this.incompatibleModules.length === 0;
      } else {
        return this.compatible = true;
      }
    };

    return Package;

  })();

}).call(this);
