(function() {
  var PriorityUICoordinator, generateTempId;

  generateTempId = require('./flux/models/utils').generateTempId;

  PriorityUICoordinator = (function() {
    function PriorityUICoordinator() {
      this.tasks = {};
      this.settle = Promise.resolve();
      setInterval(((function(_this) {
        return function() {
          return _this.detectOrphanedTasks();
        };
      })(this)), 1000);
    }

    PriorityUICoordinator.prototype.beginPriorityTask = function() {
      var id;
      if (Object.keys(this.tasks).length === 0) {
        this.settle = new Promise((function(_this) {
          return function(resolve, reject) {
            return _this.settlePromiseResolve = resolve;
          };
        })(this));
      }
      id = generateTempId();
      this.tasks[id] = Date.now();
      return id;
    };

    PriorityUICoordinator.prototype.endPriorityTask = function(id) {
      if (!id) {
        throw new Error("You must provide a task id to endPriorityTask");
      }
      delete this.tasks[id];
      if (Object.keys(this.tasks).length === 0) {
        if (this.settlePromiseResolve) {
          this.settlePromiseResolve();
        }
        return this.settlePromiseResolve = null;
      }
    };

    PriorityUICoordinator.prototype.detectOrphanedTasks = function() {
      var id, now, threshold, timestamp, _ref, _results;
      now = Date.now();
      threshold = 15000;
      _ref = this.tasks;
      _results = [];
      for (id in _ref) {
        timestamp = _ref[id];
        if (now - timestamp > threshold) {
          console.log("PriorityUICoordinator detected oprhaned priority task lasting " + threshold + "ms. Ending.");
          _results.push(this.endPriorityTask(id));
        } else {
          _results.push(void 0);
        }
      }
      return _results;
    };

    PriorityUICoordinator.prototype.busy = function() {
      return Object.keys(this.tasks).length > 0;
    };

    return PriorityUICoordinator;

  })();

  module.exports = new PriorityUICoordinator();

}).call(this);
