(function() {
  var EmojiData, RegExpUtils, _;

  _ = require('underscore');

  EmojiData = require('emoji-data');

  RegExpUtils = {
    emailRegex: function() {
      return new RegExp(/([a-z.A-Z0-9!#$%&'*+\-/=?^_`{|}~;]+@[A-Za-z0-9.-]+\.[A-Za-z]{2,63})/g);
    },
    phoneRegex: function() {
      return new RegExp(/([\+\(]+|\b)(?:(\d{1,3}[- ()]*)?)(\d{3})[- )]+(\d{3})[- ]+(\d{4})(?: *x(\d+))?\b/g);
    },
    domainRegex: function() {
      return new RegExp(/^(?!:\/\/)([a-zA-Z0-9-_]+\.)*[a-zA-Z0-9][a-zA-Z0-9-_]+\.[a-zA-Z]{2,11}?/i);
    },
    ipAddressRegex: function() {
      return new RegExp(/^(?:[0-9]{1,3}\.){3}[0-9]{1,3}$/i);
    },
    urlRegex: function(_arg) {
      var commonTlds, matchEntireString, parts;
      matchEntireString = (_arg != null ? _arg : {}).matchEntireString;
      commonTlds = ['com', 'org', 'edu', 'gov', 'uk', 'net', 'ca', 'de', 'jp', 'fr', 'au', 'us', 'ru', 'ch', 'it', 'nl', 'se', 'no', 'es', 'mil', 'ly'];
      parts = ['(', '(', '(', '([A-Za-z]{3,9}:(?:\\/\\/))', '(?:[\\-;:&=\\+\\$,\\w]+@)?', '(', '([a-zA-Z0-9-_]+\\.)*[a-zA-Z0-9][a-zA-Z0-9-_]+\\.[a-zA-Z]{2,11}', '|', '(?:[0-9]{1,3}\\.){3}[0-9]{1,3}', ')', '|', '([A-Za-z]{3,9}:(?:\\/\\/))?', '(?:[\\-;:&=\\+\\$,\\w]+@)?', '(', '([a-zA-Z0-9-_]+\\.)*[a-zA-Z0-9][a-zA-Z0-9-_]+\\.(?:' + commonTlds.join('|') + ')', '|', '(?:[0-9]{1,3}\\.){3}[0-9]{1,3}', ')', ')', '(?::\d*)?', '|', 'mailto:\\/*(?:\\w+\\.|[\\-;:&=\\+\\$.,\\w]+@)[A-Za-z0-9\\.\\-]+', ')', '(', '(?:[\\+~%\\/\\.\\w\\-_@]*[\\+~%\\/\\w\\-_]+)?', '(?:(\\?[\\-\\+=&;%@\\.\\w_]*[\\-\\+=&;%@\\w_\\/]+)?#?(?:[\'\\$\\&\\(\\)\\*\\+,;=\\.\\!\\/\\\\\\w%-]*[\\/\\\\\\w]+)?)?', ')?', ')'];
      if (matchEntireString) {
        parts.unshift('^');
      }
      return new RegExp(parts.join(''), 'gi');
    },
    linkTagRegex: function() {
      return new RegExp(/(<a.*?href\s*?=\s*?['"])(.*?)(['"].*?>)([\s\S]*?)(<\/a>)/gim);
    },
    urlLinkTagRegex: function() {
      return new RegExp(/(<a.*?href\s*?=\s*?['"])((?!mailto).+?)(['"].*?>)([\s\S]*?)(<\/a>)/gim);
    },
    imageTagRegex: function() {
      return /<img\s+[^>]*src="([^"]*)"[^>]*>/g;
    },
    punctuation: function(_arg) {
      var exclude, punctuation;
      exclude = (_arg != null ? _arg : {}).exclude;
      if (exclude == null) {
        exclude = [];
      }
      punctuation = ['.', ',', '\\/', '#', '!', '$', '%', '^', '&', '*', ';', ':', '{', '}', '=', '\\-', '_', '`', '~', '(', ')', '@', '+', '?', '>', '<', '\\[', '\\]', '+'];
      punctuation = _.difference(punctuation, exclude).join('');
      return new RegExp("[" + punctuation + "]", 'g');
    },
    hasValidSchemeRegex: function() {
      return new RegExp('^[a-z][a-z0-9+.-]*:', 'i');
    },
    emojiRegex: function() {
      var FBS_REGEXP;
      return FBS_REGEXP = new RegExp("(?:" + (EmojiData.chars({
        include_variants: true
      }).join("|")) + ")", "g");
    },
    looseStyleTag: function() {
      return /<style/gim;
    },
    functionArgs: function() {
      return /(?:\(\s*([^)]+?)\s*\)|(\w+)\s?=>)/;
    },
    illegalPathCharactersRegexp: function() {
      return /[\\\/:|?*><"]/g;
    },
    signatureRegex: function() {
      return new RegExp(/(<br\/>){0,2}<signature>[^]*<\/signature>/);
    },
    n1QuoteStartRegex: function() {
      return new RegExp(/<\w+[^>]*gmail_quote/i);
    }
  };

  module.exports = RegExpUtils;

}).call(this);
