(function() {
  var InlineStyleTransformer, RegExpUtils, crypto, ipcRenderer, userAgentDefault, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  ipcRenderer = require("electron").ipcRenderer;

  RegExpUtils = require('../regexp-utils');

  crypto = require('crypto');

  _ = require('underscore');

  userAgentDefault = null;

  InlineStyleTransformer = (function() {
    function InlineStyleTransformer() {
      this._onInlineStylesResult = __bind(this._onInlineStylesResult, this);
      this.run = __bind(this.run, this);
      this._inlineStylePromises = {};
      this._inlineStyleResolvers = {};
      ipcRenderer.on('inline-styles-result', this._onInlineStylesResult);
    }

    InlineStyleTransformer.prototype.run = function(html) {
      var key, _base;
      if (!(html && _.isString(html) && html.length > 0)) {
        return Promise.resolve(html);
      }
      if (!RegExpUtils.looseStyleTag().test(html)) {
        return Promise.resolve(html);
      }
      key = crypto.createHash('md5').update(html).digest('hex');
      html = html.replace(/<style[^>]*>[\n\r]*<!--([^<\/]*)-->[\n\r]*<\/style/g, function(full, content) {
        return "<style>" + content + "</style";
      });
      html = this._injectUserAgentStyles(html);
      if ((_base = this._inlineStylePromises)[key] == null) {
        _base[key] = new Promise((function(_this) {
          return function(resolve, reject) {
            _this._inlineStyleResolvers[key] = resolve;
            return ipcRenderer.send('inline-style-parse', {
              html: html,
              key: key
            });
          };
        })(this));
      }
      return this._inlineStylePromises[key];
    };

    InlineStyleTransformer.prototype._injectUserAgentStyles = function(body) {
      var i;
      i = body.search(RegExpUtils.looseStyleTag());
      if (i === -1) {
        return body;
      }
      if (userAgentDefault == null) {
        userAgentDefault = require('../chrome-user-agent-stylesheet-string');
      }
      return "" + body.slice(0, i) + "<style>" + userAgentDefault + "</style>" + body.slice(i);
    };

    InlineStyleTransformer.prototype._onInlineStylesResult = function(event, _arg) {
      var html, key;
      html = _arg.html, key = _arg.key;
      delete this._inlineStylePromises[key];
      this._inlineStyleResolvers[key](html);
      delete this._inlineStyleResolvers[key];
    };

    return InlineStyleTransformer;

  })();

  module.exports = new InlineStyleTransformer();

}).call(this);
