(function() {
  var DOMUtils, QuotedHTMLTransformer, crypto, quoteStringDetector, _,
    __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  _ = require('underscore');

  crypto = require('crypto');

  DOMUtils = require('../dom-utils');

  quoteStringDetector = require('./quote-string-detector')["default"];

  QuotedHTMLTransformer = (function() {
    function QuotedHTMLTransformer() {}

    QuotedHTMLTransformer.prototype.annotationClass = "nylas-quoted-text-segment";

    QuotedHTMLTransformer.prototype.hideQuotedHTML = function(html, _arg) {
      var doc, keepIfWholeBodyIsQuote, quoteElements;
      keepIfWholeBodyIsQuote = (_arg != null ? _arg : {}).keepIfWholeBodyIsQuote;
      doc = this._parseHTML(html);
      quoteElements = this._findQuoteLikeElements(doc);
      if (keepIfWholeBodyIsQuote && this._wholeBodyIsQuote(doc, quoteElements)) {
        return doc.children[0].innerHTML;
      } else {
        this._annotateElements(quoteElements);
        return doc.children[0].innerHTML;
      }
    };

    QuotedHTMLTransformer.prototype.hasQuotedHTML = function(html) {
      var doc, quoteElements;
      doc = this._parseHTML(html);
      quoteElements = this._findQuoteLikeElements(doc);
      return quoteElements.length > 0;
    };

    QuotedHTMLTransformer.prototype.removeQuotedHTML = function(html, options) {
      var doc, quoteElements;
      if (options == null) {
        options = {};
      }
      doc = this._parseHTML(html);
      quoteElements = this._findQuoteLikeElements(doc, options);
      if (options.keepIfWholeBodyIsQuote && this._wholeBodyIsQuote(doc, quoteElements)) {
        return doc.children[0].innerHTML;
      } else {
        DOMUtils.Mutating.removeElements(quoteElements, options);
        if (!doc.body) {
          return "<head></head><body></body>";
        }
        this.removeTrailingBr(doc);
        DOMUtils.Mutating.removeElements(quoteStringDetector(doc));
        if (!doc.children[0]) {
          return "<head></head><body></body>";
        }
        return doc.children[0].innerHTML;
      }
    };

    QuotedHTMLTransformer.prototype.removeTrailingBr = function(doc) {
      var childNodes, curr, extraTailBrTags, i, next, _i, _ref;
      childNodes = doc.body.childNodes;
      extraTailBrTags = [];
      for (i = _i = _ref = childNodes.length - 1; _i >= 0; i = _i += -1) {
        curr = childNodes[i];
        next = childNodes[i - 1];
        if (curr && curr.nodeName === 'BR' && next && next.nodeName === 'BR') {
          extraTailBrTags.push(curr);
        } else {
          break;
        }
      }
      return DOMUtils.Mutating.removeElements(extraTailBrTags);
    };

    QuotedHTMLTransformer.prototype.appendQuotedHTML = function(htmlWithoutQuotes, originalHTML) {
      var doc, node, quoteElements, _i, _len;
      doc = this._parseHTML(originalHTML);
      quoteElements = this._findQuoteLikeElements(doc);
      doc = this._parseHTML(htmlWithoutQuotes);
      for (_i = 0, _len = quoteElements.length; _i < _len; _i++) {
        node = quoteElements[_i];
        doc.body.appendChild(node);
      }
      return doc.children[0].innerHTML;
    };

    QuotedHTMLTransformer.prototype.restoreAnnotatedHTML = function(html) {
      var doc, quoteElements;
      doc = this._parseHTML(html);
      quoteElements = this._findAnnotatedElements(doc);
      this._removeAnnotation(quoteElements);
      return doc.children[0].innerHTML;
    };

    QuotedHTMLTransformer.prototype._parseHTML = function(text) {
      var doc, domParser, error;
      domParser = new DOMParser();
      try {
        doc = domParser.parseFromString(text, "text/html");
      } catch (_error) {
        error = _error;
        text = "HTML Parser Error: " + (error.toString());
        doc = domParser.parseFromString(text, "text/html");
        NylasEnv.reportError(error);
      }
      return doc;
    };

    QuotedHTMLTransformer.prototype._wholeBodyIsQuote = function(doc, quoteElements) {
      var child, nonBlankChildElements, _i, _len, _ref, _ref1;
      nonBlankChildElements = [];
      _ref = doc.body.childNodes;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        child = _ref[_i];
        if (child.textContent.trim() === "") {
          continue;
        } else {
          nonBlankChildElements.push(child);
        }
      }
      if (nonBlankChildElements.length === 1) {
        return _ref1 = nonBlankChildElements[0], __indexOf.call(quoteElements, _ref1) >= 0;
      } else {
        return false;
      }
    };

    QuotedHTMLTransformer.prototype._findQuoteLikeElements = function(doc, _arg) {
      var includeInline, parser, parsers, quoteElements, trailingQuotes, _i, _len, _ref;
      includeInline = (_arg != null ? _arg : {}).includeInline;
      parsers = [this._findGmailQuotes, this._findOffice365Quotes, this._findBlockquoteQuotes];
      quoteElements = [];
      for (_i = 0, _len = parsers.length; _i < _len; _i++) {
        parser = parsers[_i];
        quoteElements = quoteElements.concat((_ref = parser(doc)) != null ? _ref : []);
      }
      if (!includeInline && quoteElements.length > 0) {
        trailingQuotes = this._findTrailingQuotes(doc, quoteElements);
        quoteElements = _.intersection(quoteElements, trailingQuotes);
      }
      return _.compact(_.uniq(quoteElements));
    };

    QuotedHTMLTransformer.prototype._findTrailingQuotes = function(scopeElement, quoteElements) {
      var moreTrailing, nodeWithContent, nodesWithContent, trailingQuotes, _i;
      if (quoteElements == null) {
        quoteElements = [];
      }
      trailingQuotes = [];
      nodesWithContent = DOMUtils.nodesWithContent(scopeElement);
      for (_i = nodesWithContent.length - 1; _i >= 0; _i += -1) {
        nodeWithContent = nodesWithContent[_i];
        if (__indexOf.call(quoteElements, nodeWithContent) >= 0) {
          trailingQuotes.push(nodeWithContent);
          continue;
        } else {
          moreTrailing = this._findTrailingQuotes(nodeWithContent, quoteElements);
          trailingQuotes = trailingQuotes.concat(moreTrailing);
          break;
        }
      }
      return trailingQuotes;
    };

    QuotedHTMLTransformer.prototype._contains = function(node, quoteElement) {
      return node === quoteElement || node.contains(quoteElement);
    };

    QuotedHTMLTransformer.prototype._findAnnotatedElements = function(doc) {
      return Array.prototype.slice.call(doc.getElementsByClassName(this.annotationClass));
    };

    QuotedHTMLTransformer.prototype._annotateElements = function(elements) {
      var el, originalDisplay, _i, _len, _results;
      if (elements == null) {
        elements = [];
      }
      _results = [];
      for (_i = 0, _len = elements.length; _i < _len; _i++) {
        el = elements[_i];
        el.classList.add(this.annotationClass);
        originalDisplay = el.style.display;
        el.style.display = "none";
        _results.push(el.setAttribute("data-nylas-quoted-text-original-display", originalDisplay));
      }
      return _results;
    };

    QuotedHTMLTransformer.prototype._removeAnnotation = function(elements) {
      var el, originalDisplay, _i, _len, _results;
      if (elements == null) {
        elements = [];
      }
      _results = [];
      for (_i = 0, _len = elements.length; _i < _len; _i++) {
        el = elements[_i];
        el.classList.remove(this.annotationClass);
        originalDisplay = el.getAttribute("data-nylas-quoted-text-original-display");
        el.style.display = originalDisplay;
        _results.push(el.removeAttribute("data-nylas-quoted-text-original-display"));
      }
      return _results;
    };

    QuotedHTMLTransformer.prototype._findGmailQuotes = function(doc) {
      return Array.prototype.slice.call(doc.querySelectorAll('.gmail_quote'));
    };

    QuotedHTMLTransformer.prototype._findOffice365Quotes = function(doc) {
      var elements, weirdEl;
      elements = doc.querySelectorAll('#divRplyFwdMsg, #OLK_SRC_BODY_SECTION');
      elements = Array.prototype.slice.call(elements);
      weirdEl = doc.getElementById('3D"divRplyFwdMsg"');
      if (weirdEl) {
        elements.push(weirdEl);
      }
      elements = _.map(elements, function(el) {
        if (el.previousElementSibling && el.previousElementSibling.nodeName === "HR") {
          return el.parentElement;
        } else {
          return el;
        }
      });
      return elements;
    };

    QuotedHTMLTransformer.prototype._findBlockquoteQuotes = function(doc) {
      return Array.prototype.slice.call(doc.querySelectorAll('blockquote'));
    };

    return QuotedHTMLTransformer;

  })();

  module.exports = new QuotedHTMLTransformer;

}).call(this);
