(function() {
  var Fragment, ParsedEmail, QuotedPlainTextParser, chomp, _, _str;

  _ = require('underscore');

  _str = require('underscore.string');

  QuotedPlainTextParser = {
    parse: function(text) {
      var parsedEmail;
      parsedEmail = new ParsedEmail;
      return parsedEmail.parse(text);
    },
    visibleText: function(text, _arg) {
      var showQuoted, showSignature, _ref;
      _ref = _arg != null ? _arg : {}, showQuoted = _ref.showQuoted, showSignature = _ref.showSignature;
      if (showQuoted == null) {
        showQuoted = false;
      }
      if (showSignature == null) {
        showSignature = false;
      }
      return this.parse(text).visibleText({
        showQuoted: showQuoted,
        showSignature: showSignature
      });
    },
    hiddenText: function(text, _arg) {
      var showQuoted, showSignature, _ref;
      _ref = _arg != null ? _arg : {}, showQuoted = _ref.showQuoted, showSignature = _ref.showSignature;
      if (showQuoted == null) {
        showQuoted = false;
      }
      if (showSignature == null) {
        showSignature = false;
      }
      return this.parse(text).hiddenText({
        showQuoted: showQuoted,
        showSignature: showSignature
      });
    },
    hasQuotedHTML: function(text) {
      return this.parse(text).hasQuotedHTML();
    }
  };

  chomp = function() {
    return this.replace(/(\n|\r)+$/, '');
  };

  ParsedEmail = (function() {
    function ParsedEmail() {
      this.fragments = [];
      this.currentFragment = null;
      return;
    }

    ParsedEmail.prototype.fragments = [];

    ParsedEmail.prototype.hasQuotedHTML = function() {
      var fragment, _i, _len, _ref;
      _ref = this.fragments;
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        fragment = _ref[_i];
        if (fragment.quoted) {
          return true;
        }
      }
      return false;
    };

    ParsedEmail.prototype.visibleText = function(_arg) {
      var showQuoted, showSignature, _ref;
      _ref = _arg != null ? _arg : {}, showSignature = _ref.showSignature, showQuoted = _ref.showQuoted;
      this._setHiddenState({
        showSignature: showSignature,
        showQuoted: showQuoted
      });
      return _.reject(this.fragments, function(f) {
        return f.hidden;
      }).map(function(f) {
        return f.to_s();
      }).join('\n');
    };

    ParsedEmail.prototype.hiddenText = function(_arg) {
      var showQuoted, showSignature, _ref;
      _ref = _arg != null ? _arg : {}, showSignature = _ref.showSignature, showQuoted = _ref.showQuoted;
      this._setHiddenState({
        showSignature: showSignature,
        showQuoted: showQuoted
      });
      return _.filter(this.fragments, function(f) {
        return f.hidden;
      }).map(function(f) {
        return f.to_s();
      }).join('\n');
    };

    ParsedEmail.prototype._setHiddenState = function(_arg) {
      var fragment, fragments, i, showQuoted, showSignature, _i, _len, _ref, _results;
      _ref = _arg != null ? _arg : {}, showSignature = _ref.showSignature, showQuoted = _ref.showQuoted;
      fragments = _.reject(this.fragments, function(f) {
        if (f.to_s().trim() === "") {
          f.hidden = true;
          return true;
        } else {
          return false;
        }
      });
      _results = [];
      for (i = _i = 0, _len = fragments.length; _i < _len; i = ++_i) {
        fragment = fragments[i];
        fragment.hidden = true;
        if (fragment.quoted) {
          if (showQuoted || ((fragments[i + 1] != null) && !(fragments[i + 1].quoted || fragments[i + 1].signature))) {
            fragment.hidden = false;
            continue;
          } else {
            continue;
          }
        }
        if (fragment.signature) {
          if (showSignature) {
            fragment.hidden = false;
            continue;
          } else {
            continue;
          }
        }
        _results.push(fragment.hidden = false);
      }
      return _results;
    };

    ParsedEmail.prototype.parse = function(text) {
      this.currentFragment = null;
      return this._parsePlain(text);
    };

    ParsedEmail.prototype._parsePlain = function(text) {
      var doubleOnPatt, i, lines, patt, replyHeader;
      patt = /^(On\s(\n|.)*wrote:)$/m;
      doubleOnPatt = /^(On\s(\n|.)*(^(> )?On\s)((\n|.)*)wrote:)$/m;
      if (patt.test(text) && !doubleOnPatt.test(text)) {
        replyHeader = patt.exec(text)[0];
        text = text.replace(replyHeader, replyHeader.replace(/\n/g, ' '));
      }
      text = _str.reverse(text);
      lines = text.split('\n');
      for (i in lines) {
        this._scanPlainLine(lines[i]);
      }
      this._finishFragment();
      this.fragments.reverse();
      return this;
    };

    ParsedEmail.prototype._signatureRE = /(--|__|^-\w)|(^sent from my (\s*\w+){1,3}$)/i;

    ParsedEmail.prototype._scanPlainLine = function(line) {
      var isQuoted;
      line = chomp.apply(line);
      if (!new RegExp(this._signatureRE).test(_str.reverse(line))) {
        line = _str.ltrim(line);
      }
      if (this.currentFragment !== null && line === '') {
        if (new RegExp(this._signatureRE).test(_str.reverse(this.currentFragment.lines[this.currentFragment.lines.length - 1]))) {
          this.currentFragment.signature = true;
          this._finishFragment();
        }
      }
      isQuoted = new RegExp('(>+)$').test(line);
      if (this.currentFragment !== null && (this.currentFragment.quoted === isQuoted || this.currentFragment.quoted && (this._quoteHeader(line) || line === ''))) {
        this.currentFragment.lines.push(line);
      } else {
        this._finishFragment();
        this.currentFragment = new Fragment(isQuoted, line, "plain");
      }
    };

    ParsedEmail.prototype._quoteHeader = function(line) {
      return new RegExp('^:etorw.*nO$').test(line);
    };

    ParsedEmail.prototype._finishFragment = function() {
      if (this.currentFragment !== null) {
        this.currentFragment.finish();
        this.fragments.push(this.currentFragment);
        this.currentFragment = null;
      }
    };

    return ParsedEmail;

  })();

  Fragment = (function() {
    function Fragment(quoted, firstLine) {
      this.quoted = quoted;
      this.signature = false;
      this.hidden = false;
      this.lines = [firstLine];
      this.content = null;
      this.lines = this.lines.filter(function() {
        return true;
      });
      return;
    }

    Fragment.prototype.content = null;

    Fragment.prototype.finish = function() {
      this.content = this.lines.join("\n");
      this.lines = [];
      this.content = _str.reverse(this.content);
    };

    Fragment.prototype.to_s = function() {
      return this.content.toString().trim();
    };

    return Fragment;

  })();

  module.exports = QuotedPlainTextParser;

}).call(this);
