(function() {
  var Actions, ComponentRegistry, Flexbox, InjectedComponentSet, React, ReactCSSTransitionGroup, RetinaImg, Sheet, SheetContainer, Toolbar, WorkspaceStore, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require('react');

  ReactCSSTransitionGroup = require('react-addons-css-transition-group');

  Sheet = require('./sheet');

  Toolbar = require('./sheet-toolbar');

  Flexbox = require('./components/flexbox');

  RetinaImg = require('./components/retina-img')["default"];

  InjectedComponentSet = require('./components/injected-component-set');

  _ = require('underscore');

  _ref = require("nylas-exports"), Actions = _ref.Actions, ComponentRegistry = _ref.ComponentRegistry, WorkspaceStore = _ref.WorkspaceStore;

  SheetContainer = (function(_super) {
    var displayName;

    __extends(SheetContainer, _super);

    displayName = 'SheetContainer';

    function SheetContainer(props) {
      this.props = props;
      this._getStateFromStores = __bind(this._getStateFromStores, this);
      this._onStoreChange = __bind(this._onStoreChange, this);
      this._onColumnSizeChanged = __bind(this._onColumnSizeChanged, this);
      this._sheetElements = __bind(this._sheetElements, this);
      this._toolbarElements = __bind(this._toolbarElements, this);
      this._toolbarContainerElement = __bind(this._toolbarContainerElement, this);
      this.render = __bind(this.render, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.state = this._getStateFromStores();
    }

    SheetContainer.prototype.componentDidMount = function() {
      return this.unsubscribe = WorkspaceStore.listen(this._onStoreChange);
    };

    SheetContainer.prototype.componentWillUnmount = function() {
      if (this.unsubscribe) {
        return this.unsubscribe();
      }
    };

    SheetContainer.prototype.render = function() {
      var sheetElements, topSheet, totalSheets;
      totalSheets = this.state.stack.length;
      topSheet = this.state.stack[totalSheets - 1];
      if (!topSheet) {
        return React.createElement("div", null);
      }
      sheetElements = this._sheetElements();
      return React.createElement(Flexbox, {
        "direction": "column",
        "className": "layout-mode-" + this.state.mode,
        "style": {
          overflow: 'hidden'
        }
      }, this._toolbarContainerElement(), React.createElement("div", {
        "name": "Header",
        "style": {
          order: 1,
          zIndex: 2
        }
      }, React.createElement(InjectedComponentSet, {
        "matching": {
          locations: [topSheet.Header, WorkspaceStore.Sheet.Global.Header]
        },
        "direction": "column",
        "id": topSheet.id
      })), React.createElement("div", {
        "name": "Center",
        "style": {
          order: 2,
          flex: 1,
          position: 'relative',
          zIndex: 1
        }
      }, sheetElements[0], React.createElement(ReactCSSTransitionGroup, {
        "transitionLeaveTimeout": 125.,
        "transitionEnterTimeout": 125.,
        "transitionName": "sheet-stack"
      }, sheetElements.slice(1))), React.createElement("div", {
        "name": "Footer",
        "style": {
          order: 3,
          zIndex: 4
        }
      }, React.createElement(InjectedComponentSet, {
        "matching": {
          locations: [topSheet.Footer, WorkspaceStore.Sheet.Global.Footer]
        },
        "direction": "column",
        "id": topSheet.id
      })));
    };

    SheetContainer.prototype._toolbarContainerElement = function() {
      var toolbar, toolbarElements;
      toolbar = NylasEnv.getLoadSettings().toolbar;
      if (!toolbar) {
        return [];
      }
      toolbarElements = this._toolbarElements();
      return React.createElement("div", {
        "name": "Toolbar",
        "style": {
          order: 0,
          zIndex: 3
        },
        "className": "sheet-toolbar"
      }, toolbarElements[0], React.createElement(ReactCSSTransitionGroup, {
        "transitionLeaveTimeout": 125.,
        "transitionEnterTimeout": 125.,
        "transitionName": "opacity-125ms"
      }, toolbarElements.slice(1)));
    };

    SheetContainer.prototype._toolbarElements = function() {
      return this.state.stack.map(function(sheet, index) {
        return React.createElement(Toolbar, {
          "data": sheet,
          "ref": "toolbar-" + index,
          "key": "" + index + ":" + sheet.id + ":toolbar",
          "depth": index
        });
      });
    };

    SheetContainer.prototype._sheetElements = function() {
      return this.state.stack.map((function(_this) {
        return function(sheet, index) {
          return React.createElement(Sheet, {
            "data": sheet,
            "depth": index,
            "key": "" + index + ":" + sheet.id,
            "onColumnSizeChanged": _this._onColumnSizeChanged
          });
        };
      })(this));
    };

    SheetContainer.prototype._onColumnSizeChanged = function(sheet) {
      var _ref1;
      if ((_ref1 = this.refs["toolbar-" + sheet.props.depth]) != null) {
        _ref1.recomputeLayout();
      }
      return window.dispatchEvent(new Event('resize'));
    };

    SheetContainer.prototype._onStoreChange = function() {
      return this.setState(this._getStateFromStores());
    };

    SheetContainer.prototype._getStateFromStores = function() {
      return {
        stack: WorkspaceStore.sheetStack(),
        mode: WorkspaceStore.layoutMode()
      };
    };

    return SheetContainer;

  })(React.Component);

  module.exports = SheetContainer;

}).call(this);
