(function() {
  var Actions, ComponentRegistry, FLEX, Flexbox, InjectedComponentSet, React, ResizableRegion, RetinaImg, Sheet, WorkspaceStore, _, _ref,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  React = require('react');

  _ = require('underscore');

  _ref = require("nylas-exports"), Actions = _ref.Actions, ComponentRegistry = _ref.ComponentRegistry, WorkspaceStore = _ref.WorkspaceStore;

  RetinaImg = require('./components/retina-img')["default"];

  Flexbox = require('./components/flexbox');

  InjectedComponentSet = require('./components/injected-component-set');

  ResizableRegion = require('./components/resizable-region');

  FLEX = 10000;

  Sheet = (function(_super) {
    __extends(Sheet, _super);

    Sheet.displayName = 'Sheet';

    Sheet.propTypes = {
      data: React.PropTypes.object.isRequired,
      depth: React.PropTypes.number.isRequired,
      onColumnSizeChanged: React.PropTypes.func
    };

    Sheet.defaultProps = {
      onColumnSizeChanged: function() {}
    };

    Sheet.childContextTypes = {
      sheetDepth: React.PropTypes.number
    };

    Sheet.prototype.getChildContext = function() {
      return {
        sheetDepth: this.props.depth
      };
    };

    function Sheet(props) {
      this.props = props;
      this._pop = __bind(this._pop, this);
      this._getStateFromStores = __bind(this._getStateFromStores, this);
      this._onColumnResize = __bind(this._onColumnResize, this);
      this._columnFlexboxElements = __bind(this._columnFlexboxElements, this);
      this.render = __bind(this.render, this);
      this.componentWillUnmount = __bind(this.componentWillUnmount, this);
      this.shouldComponentUpdate = __bind(this.shouldComponentUpdate, this);
      this.componentDidUpdate = __bind(this.componentDidUpdate, this);
      this.componentDidMount = __bind(this.componentDidMount, this);
      this.getChildContext = __bind(this.getChildContext, this);
      this.state = this._getStateFromStores();
    }

    Sheet.prototype.componentDidMount = function() {
      if (this.unlisteners == null) {
        this.unlisteners = [];
      }
      this.unlisteners.push(ComponentRegistry.listen((function(_this) {
        return function(event) {
          return _this.setState(_this._getStateFromStores());
        };
      })(this)));
      return this.unlisteners.push(WorkspaceStore.listen((function(_this) {
        return function(event) {
          return _this.setState(_this._getStateFromStores());
        };
      })(this)));
    };

    Sheet.prototype.componentDidUpdate = function() {
      var col, minWidth, _i, _len, _ref1;
      this.props.onColumnSizeChanged(this);
      minWidth = 0;
      _ref1 = this.state.columns;
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        col = _ref1[_i];
        minWidth += col.minWidth;
      }
      return NylasEnv.setMinimumWidth(minWidth);
    };

    Sheet.prototype.shouldComponentUpdate = function(nextProps, nextState) {
      return !_.isEqual(nextProps, this.props) || !_.isEqual(nextState, this.state);
    };

    Sheet.prototype.componentWillUnmount = function() {
      var unlisten, _i, _len, _ref1, _results;
      _ref1 = this.unlisteners;
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        unlisten = _ref1[_i];
        _results.push(unlisten());
      }
      return _results;
    };

    Sheet.prototype.render = function() {
      var style;
      style = {
        position: 'absolute',
        width: '100%',
        height: '100%',
        zIndex: 1
      };
      return React.createElement("div", {
        "name": "Sheet",
        "style": style,
        "className": "sheet mode-" + this.state.mode,
        "data-id": this.props.data.id
      }, React.createElement(Flexbox, {
        "direction": "row",
        "style": {
          overflow: 'hidden'
        }
      }, this._columnFlexboxElements()));
    };

    Sheet.prototype._columnFlexboxElements = function() {
      return this.state.columns.map((function(_this) {
        return function(column, idx) {
          var handle, location, maxWidth, minWidth, style, width;
          maxWidth = column.maxWidth, minWidth = column.minWidth, handle = column.handle, location = column.location, width = column.width;
          if (minWidth !== maxWidth && maxWidth < FLEX) {
            return React.createElement(ResizableRegion, {
              "key": "" + _this.props.data.id + ":" + idx,
              "name": "" + _this.props.data.id + ":" + idx,
              "className": "column-" + location.id,
              "style": {
                height: '100%'
              },
              "data-column": idx,
              "onResize": _this._onColumnResize.bind(_this, column),
              "initialWidth": width,
              "minWidth": minWidth,
              "maxWidth": maxWidth,
              "handle": handle
            }, React.createElement(InjectedComponentSet, {
              "direction": "column",
              "matching": {
                location: location,
                mode: _this.state.mode
              }
            }));
          } else {
            style = {
              height: '100%',
              minWidth: minWidth,
              overflow: 'hidden'
            };
            if (maxWidth < FLEX) {
              style.width = maxWidth;
            } else {
              style.flex = 1;
            }
            return React.createElement(InjectedComponentSet, {
              "direction": "column",
              "key": "" + _this.props.data.id + ":" + idx,
              "name": "" + _this.props.data.id + ":" + idx,
              "className": "column-" + location.id,
              "data-column": idx,
              "style": style,
              "matching": {
                location: location,
                mode: _this.state.mode
              }
            });
          }
        };
      })(this));
    };

    Sheet.prototype._onColumnResize = function(column, width) {
      NylasEnv.storeColumnWidth({
        id: column.location.id,
        width: width
      });
      return this.props.onColumnSizeChanged(this);
    };

    Sheet.prototype._getStateFromStores = function() {
      var col, entries, i, idx, location, maxWidth, minWidth, state, widest, widestWidth, width, _i, _j, _k, _len, _ref1, _ref2, _ref3, _ref4;
      state = {
        mode: WorkspaceStore.layoutMode(),
        columns: []
      };
      widest = -1;
      widestWidth = -1;
      if (((_ref1 = this.props.data) != null ? _ref1.columns[state.mode] : void 0) != null) {
        _ref2 = this.props.data.columns[state.mode];
        for (idx = _i = 0, _len = _ref2.length; _i < _len; idx = ++_i) {
          location = _ref2[idx];
          if (WorkspaceStore.isLocationHidden(location)) {
            continue;
          }
          entries = ComponentRegistry.findComponentsMatching({
            location: location,
            mode: state.mode
          });
          maxWidth = _.reduce(entries, (function(m, component) {
            var _ref3, _ref4;
            return Math.min((_ref3 = (_ref4 = component.containerStyles) != null ? _ref4.maxWidth : void 0) != null ? _ref3 : 10000, m);
          }), 10000);
          minWidth = _.reduce(entries, (function(m, component) {
            var _ref3, _ref4;
            return Math.max((_ref3 = (_ref4 = component.containerStyles) != null ? _ref4.minWidth : void 0) != null ? _ref3 : 0, m);
          }), 0);
          width = NylasEnv.getColumnWidth(location.id);
          col = {
            maxWidth: maxWidth,
            minWidth: minWidth,
            location: location,
            width: width
          };
          state.columns.push(col);
          if (maxWidth > widestWidth) {
            widestWidth = maxWidth;
            widest = idx;
          }
        }
      }
      if (state.columns.length > 0) {
        state.columns[widest].maxWidth = FLEX;
        for (i = _j = 0, _ref3 = widest - 1; _j <= _ref3; i = _j += 1) {
          state.columns[i].handle = ResizableRegion.Handle.Right;
        }
        for (i = _k = widest, _ref4 = state.columns.length - 1; _k <= _ref4; i = _k += 1) {
          state.columns[i].handle = ResizableRegion.Handle.Left;
        }
      }
      return state;
    };

    Sheet.prototype._pop = function() {
      return Actions.popSheet();
    };

    return Sheet;

  })(React.Component);

  module.exports = Sheet;

}).call(this);
