(function() {
  var CompositeDisposable, Disposable, Emitter, EmitterMixin, File, Package, Q, ThemeManager, fs, path, _, _ref;

  path = require('path');

  _ = require('underscore');

  EmitterMixin = require('emissary').Emitter;

  _ref = require('event-kit'), Emitter = _ref.Emitter, Disposable = _ref.Disposable, CompositeDisposable = _ref.CompositeDisposable;

  File = require('pathwatcher').File;

  fs = require('fs-plus');

  Q = require('q');

  Package = require('./package');

  module.exports = ThemeManager = (function() {
    EmitterMixin.includeInto(ThemeManager);

    function ThemeManager(_arg) {
      var stylesElement;
      this.packageManager = _arg.packageManager, this.resourcePath = _arg.resourcePath, this.configDirPath = _arg.configDirPath, this.safeMode = _arg.safeMode;
      this.emitter = new Emitter;
      this.styleSheetDisposablesBySourcePath = {};
      this.lessCache = null;
      this.initialLoadComplete = false;
      this.packageManager.registerPackageActivator(this, ['theme']);
      this.sheetsByStyleElement = new WeakMap;
      stylesElement = document.head.querySelector('nylas-styles');
      stylesElement.onDidAddStyleElement(this.styleElementAdded.bind(this));
      stylesElement.onDidRemoveStyleElement(this.styleElementRemoved.bind(this));
      stylesElement.onDidUpdateStyleElement(this.styleElementUpdated.bind(this));
    }

    ThemeManager.prototype.baseThemeName = function() {
      return 'ui-light';
    };

    ThemeManager.prototype.watchCoreStyles = function() {
      var watchStylesIn;
      console.log('Watching /static and /internal_packages for LESS changes');
      watchStylesIn = (function(_this) {
        return function(folder) {
          var PathWatcher, stylePath, stylePaths, _i, _len, _results;
          stylePaths = fs.listTreeSync(folder);
          PathWatcher = require('pathwatcher');
          _results = [];
          for (_i = 0, _len = stylePaths.length; _i < _len; _i++) {
            stylePath = stylePaths[_i];
            if (path.extname(stylePath) !== '.less') {
              continue;
            }
            _results.push(PathWatcher.watch(stylePath, function() {
              return _this.activateThemes();
            }));
          }
          return _results;
        };
      })(this);
      watchStylesIn("" + this.resourcePath + "/static");
      return watchStylesIn("" + this.resourcePath + "/internal_packages");
    };

    ThemeManager.prototype.styleElementAdded = function(styleElement) {
      var sheet;
      sheet = styleElement.sheet;
      this.sheetsByStyleElement.set(styleElement, sheet);
      this.emitter.emit('did-add-stylesheet', sheet);
      return this.emitter.emit('did-change-stylesheets');
    };

    ThemeManager.prototype.styleElementRemoved = function(styleElement) {
      var sheet;
      sheet = this.sheetsByStyleElement.get(styleElement);
      this.emitter.emit('did-remove-stylesheet', sheet);
      return this.emitter.emit('did-change-stylesheets');
    };

    ThemeManager.prototype.styleElementUpdated = function(_arg) {
      var sheet;
      sheet = _arg.sheet;
      this.emitter.emit('did-remove-stylesheet', sheet);
      this.emitter.emit('did-add-stylesheet', sheet);
      return this.emitter.emit('did-change-stylesheets');
    };


    /*
    Section: Event Subscription
     */

    ThemeManager.prototype.onDidChangeActiveThemes = function(callback) {
      return this.emitter.on('did-change-active-themes', callback);
    };


    /*
    Section: Accessing Available Themes
     */

    ThemeManager.prototype.getAvailableNames = function() {
      return this.getLoadedNames();
    };


    /*
    Section: Accessing Loaded Themes
     */

    ThemeManager.prototype.getLoadedThemeNames = function() {
      var theme, _i, _len, _ref1, _results;
      _ref1 = this.getLoadedThemes();
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        theme = _ref1[_i];
        _results.push(theme.name);
      }
      return _results;
    };

    ThemeManager.prototype.getLoadedThemes = function() {
      var pack, _i, _len, _ref1, _results;
      _ref1 = this.packageManager.getLoadedPackages();
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        pack = _ref1[_i];
        if (pack.isTheme()) {
          _results.push(pack);
        }
      }
      return _results;
    };


    /*
    Section: Accessing Active Themes
     */

    ThemeManager.prototype.getActiveThemeNames = function() {
      var theme, _i, _len, _ref1, _results;
      _ref1 = this.getActiveThemes();
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        theme = _ref1[_i];
        _results.push(theme.name);
      }
      return _results;
    };

    ThemeManager.prototype.getActiveThemes = function() {
      var pack, _i, _len, _ref1, _results;
      _ref1 = this.packageManager.getActivePackages();
      _results = [];
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        pack = _ref1[_i];
        if (pack.isTheme()) {
          _results.push(pack);
        }
      }
      return _results;
    };

    ThemeManager.prototype.getActiveTheme = function() {
      return this.getActiveThemes()[0];
    };

    ThemeManager.prototype.activatePackages = function() {
      return this.activateThemes();
    };


    /*
    Section: Managing Enabled Themes
     */

    ThemeManager.prototype.getEnabledThemeNames = function() {
      var builtInThemeNames, themeNames, _ref1;
      themeNames = (_ref1 = NylasEnv.config.get('core.themes')) != null ? _ref1 : [];
      if (!_.isArray(themeNames)) {
        themeNames = [themeNames];
      }
      themeNames = themeNames.filter(function(themeName) {
        if (themeName && typeof themeName === 'string') {
          if (NylasEnv.packages.resolvePackagePath(themeName)) {
            return true;
          }
          console.warn("Enabled theme '" + themeName + "' is not installed.");
        }
        return false;
      });
      if (themeNames.length === 0) {
        builtInThemeNames = ['ui-light', 'ui-dark'];
        themeNames = _.intersection(themeNames, builtInThemeNames);
        if (themeNames.length === 0) {
          themeNames = ['ui-light'];
        }
      }
      return themeNames.reverse();
    };

    ThemeManager.prototype.setActiveTheme = function(theme) {
      var base;
      base = this.baseThemeName();
      return NylasEnv.config.set('core.themes', _.uniq([base, theme]));
    };


    /*
    Section: Private
     */

    ThemeManager.prototype.requireStylesheet = function(stylesheetPath) {
      var content, fullPath;
      if (fullPath = this.resolveStylesheet(stylesheetPath)) {
        content = this.loadStylesheet(fullPath);
        return this.applyStylesheet(fullPath, content);
      } else {
        throw new Error("Could not find a file at path '" + stylesheetPath + "'");
      }
    };

    ThemeManager.prototype.loadBaseStylesheets = function() {
      return this.reloadBaseStylesheets();
    };

    ThemeManager.prototype.reloadBaseStylesheets = function() {
      var nativeStylesheetPath;
      this.requireStylesheet('../static/index');
      this.requireStylesheet('../static/email-frame');
      if (nativeStylesheetPath = fs.resolveOnLoadPath(process.platform, ['css', 'less'])) {
        return this.requireStylesheet(nativeStylesheetPath);
      }
    };

    ThemeManager.prototype.stylesheetElementForId = function(id) {
      return document.head.querySelector("nylas-styles style[source-path=\"" + id + "\"]");
    };

    ThemeManager.prototype.resolveStylesheet = function(stylesheetPath) {
      if (path.extname(stylesheetPath).length > 0) {
        return fs.resolveOnLoadPath(stylesheetPath);
      } else {
        return fs.resolveOnLoadPath(stylesheetPath, ['css', 'less']);
      }
    };

    ThemeManager.prototype.loadStylesheet = function(stylesheetPath, importFallbackVariables) {
      if (path.extname(stylesheetPath) === '.less') {
        return this.loadLessStylesheet(stylesheetPath, importFallbackVariables);
      } else {
        return fs.readFileSync(stylesheetPath, 'utf8');
      }
    };

    ThemeManager.prototype.loadLessStylesheet = function(lessStylesheetPath, importFallbackVariables) {
      var LessCompileCache, baseVarImports, detail, error, less, message;
      if (importFallbackVariables == null) {
        importFallbackVariables = false;
      }
      if (this.lessCache == null) {
        LessCompileCache = require('./less-compile-cache');
        this.lessCache = new LessCompileCache({
          configDirPath: this.configDirPath,
          resourcePath: this.resourcePath,
          importPaths: this.getImportPaths()
        });
      }
      try {
        if (importFallbackVariables) {
          baseVarImports = "@import \"variables/ui-variables\";";
          less = fs.readFileSync(lessStylesheetPath, 'utf8');
          return this.lessCache.cssForFile(lessStylesheetPath, [baseVarImports, less].join('\n'));
        } else {
          return this.lessCache.read(lessStylesheetPath);
        }
      } catch (_error) {
        error = _error;
        if (error.line != null) {
          message = "Error compiling Less stylesheet: `" + lessStylesheetPath + "`";
          detail = "Line number: " + error.line + "\n" + error.message;
        } else {
          message = "Error loading Less stylesheet: `" + lessStylesheetPath + "`";
          detail = error.message;
        }
        console.error(message, {
          detail: detail,
          dismissable: true
        });
        console.error(detail);
        throw error;
      }
    };

    ThemeManager.prototype.removeStylesheet = function(stylesheetPath) {
      var _ref1;
      return (_ref1 = this.styleSheetDisposablesBySourcePath[stylesheetPath]) != null ? _ref1.dispose() : void 0;
    };

    ThemeManager.prototype.applyStylesheet = function(path, text) {
      return this.styleSheetDisposablesBySourcePath[path] = NylasEnv.styles.addStyleSheet(text, {
        sourcePath: path
      });
    };

    ThemeManager.prototype.stringToId = function(string) {
      return string.replace(/\\/g, '/');
    };

    ThemeManager.prototype.activateThemes = function() {
      var deferred;
      deferred = Q.defer();
      NylasEnv.config.observe('core.themes', (function(_this) {
        return function() {
          var promises, themeName, _i, _len, _ref1;
          _this.deactivateThemes();
          promises = [];
          _ref1 = _this.getEnabledThemeNames();
          for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
            themeName = _ref1[_i];
            if (_this.packageManager.resolvePackagePath(themeName)) {
              promises.push(_this.packageManager.activatePackage(themeName));
            } else {
              console.warn("Failed to activate theme '" + themeName + "' because it isn't installed.");
            }
          }
          return Q.all(promises).then(function() {
            _this.addActiveThemeClasses();
            _this.refreshLessCache();
            _this.reloadBaseStylesheets();
            _this.initialLoadComplete = true;
            _this.emitter.emit('did-change-active-themes');
            return deferred.resolve();
          });
        };
      })(this));
      return deferred.promise;
    };

    ThemeManager.prototype.deactivateThemes = function() {
      var pack, _i, _len, _ref1;
      this.removeActiveThemeClasses();
      _ref1 = this.getActiveThemes();
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        pack = _ref1[_i];
        this.packageManager.deactivatePackage(pack.name);
      }
      return null;
    };

    ThemeManager.prototype.isInitialLoadComplete = function() {
      return this.initialLoadComplete;
    };

    ThemeManager.prototype.addActiveThemeClasses = function() {
      var pack, _i, _len, _ref1;
      _ref1 = this.getActiveThemes();
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        pack = _ref1[_i];
        document.body.classList.add("theme-" + pack.name);
      }
    };

    ThemeManager.prototype.removeActiveThemeClasses = function() {
      var pack, _i, _len, _ref1;
      _ref1 = this.getActiveThemes();
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        pack = _ref1[_i];
        document.body.classList.remove("theme-" + pack.name);
      }
    };

    ThemeManager.prototype.refreshLessCache = function() {
      var _ref1;
      return (_ref1 = this.lessCache) != null ? _ref1.setImportPaths(this.getImportPaths()) : void 0;
    };

    ThemeManager.prototype.getImportPaths = function() {
      var activeThemes, deprecatedPath, theme, themeName, themePath, themePaths, _i, _len, _ref1;
      activeThemes = this.getActiveThemes();
      if (activeThemes.length > 0) {
        themePaths = (function() {
          var _i, _len, _results;
          _results = [];
          for (_i = 0, _len = activeThemes.length; _i < _len; _i++) {
            theme = activeThemes[_i];
            if (theme) {
              _results.push(theme.getStylesheetsPath());
            }
          }
          return _results;
        })();
      } else {
        themePaths = [];
        _ref1 = this.getEnabledThemeNames();
        for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
          themeName = _ref1[_i];
          if (themePath = this.packageManager.resolvePackagePath(themeName)) {
            deprecatedPath = path.join(themePath, 'stylesheets');
            if (fs.isDirectorySync(deprecatedPath)) {
              themePaths.push(deprecatedPath);
            } else {
              themePaths.push(path.join(themePath, 'styles'));
            }
          }
        }
      }
      return themePaths.filter(function(themePath) {
        return fs.isDirectorySync(themePath);
      });
    };

    return ThemeManager;

  })();

}).call(this);
