(function() {
  var Utils, WindowBridge, ipcRenderer, _,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    __slice = [].slice;

  _ = require('underscore');

  ipcRenderer = require('electron').ipcRenderer;

  Utils = require('./flux/models/utils');

  WindowBridge = (function() {
    function WindowBridge() {
      this._onRunInWindow = __bind(this._onRunInWindow, this);
      this._onResults = __bind(this._onResults, this);
      this._tasks = {};
      ipcRenderer.on("remote-run-results", this._onResults);
      ipcRenderer.on("run-in-window", this._onRunInWindow);
    }

    WindowBridge.prototype.runInWindow = function(window, objectName, methodName, args) {
      var taskId;
      taskId = Utils.generateTempId();
      return new Promise((function(_this) {
        return function(resolve, reject) {
          var params;
          _this._tasks[taskId] = {
            resolve: resolve,
            reject: reject
          };
          args = JSON.stringify(args, Utils.registeredObjectReplacer);
          params = {
            window: window,
            objectName: objectName,
            methodName: methodName,
            args: args,
            taskId: taskId
          };
          return ipcRenderer.send("run-in-window", params);
        };
      })(this));
    };

    WindowBridge.prototype.runInMainWindow = function() {
      var args;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return this.runInWindow.apply(this, ["main"].concat(__slice.call(args)));
    };

    WindowBridge.prototype.runInWorkWindow = function() {
      var args;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return this.runInWindow.apply(this, ["work"].concat(__slice.call(args)));
    };

    WindowBridge.prototype._onResults = function(event, _arg) {
      var returnValue, taskId, _ref;
      _ref = _arg != null ? _arg : {}, returnValue = _ref.returnValue, taskId = _ref.taskId;
      returnValue = JSON.parse(returnValue, Utils.registeredObjectReviver);
      this._tasks[taskId].resolve(returnValue);
      return delete this._tasks[taskId];
    };

    WindowBridge.prototype._onRunInWindow = function(event, _arg) {
      var args, exports, methodName, objectName, result, returnValue, taskId, _ref;
      _ref = _arg != null ? _arg : {}, objectName = _ref.objectName, methodName = _ref.methodName, args = _ref.args, taskId = _ref.taskId;
      args = JSON.parse(args, Utils.registeredObjectReviver);
      exports = require('nylas-exports');
      result = exports[objectName][methodName].apply(null, args);
      if (_.isFunction(result.then)) {
        return result.then(function(returnValue) {
          returnValue = JSON.stringify(returnValue, Utils.registeredObjectReplacer);
          return ipcRenderer.send('remote-run-results', {
            returnValue: returnValue,
            taskId: taskId
          });
        });
      } else {
        returnValue = result;
        returnValue = JSON.stringify(returnValue, Utils.registeredObjectReplacer);
        return ipcRenderer.send('remote-run-results', {
          returnValue: returnValue,
          taskId: taskId
        });
      }
    };

    return WindowBridge;

  })();

  module.exports = new WindowBridge;

}).call(this);
