(function() {
  var Disposable, WindowEventHandler, fs, ipcRenderer, path, remote, shell, url, _, _ref,
    __slice = [].slice;

  path = require('path');

  _ = require('underscore');

  Disposable = require('event-kit').Disposable;

  _ref = require('electron'), shell = _ref.shell, ipcRenderer = _ref.ipcRenderer, remote = _ref.remote;

  fs = require('fs-plus');

  url = require('url');

  module.exports = WindowEventHandler = (function() {
    function WindowEventHandler() {
      var webContents;
      this.unloadCallbacks = [];
      _.defer((function(_this) {
        return function() {
          return _this.showDevModeMessages();
        };
      })(this));
      ipcRenderer.on('update-available', function(event, detail) {
        return NylasEnv.updateAvailable(detail);
      });
      ipcRenderer.on('browser-window-focus', function() {
        document.body.classList.remove('is-blurred');
        return window.dispatchEvent(new Event('browser-window-focus'));
      });
      ipcRenderer.on('browser-window-blur', function() {
        document.body.classList.add('is-blurred');
        return window.dispatchEvent(new Event('browser-window-blur'));
      });
      ipcRenderer.on('command', function() {
        var args, command, event;
        event = arguments[0], command = arguments[1], args = 3 <= arguments.length ? __slice.call(arguments, 2) : [];
        return NylasEnv.commands.dispatch(command, args[0]);
      });
      ipcRenderer.on('scroll-touch-begin', function() {
        return window.dispatchEvent(new Event('scroll-touch-begin'));
      });
      ipcRenderer.on('scroll-touch-end', function() {
        return window.dispatchEvent(new Event('scroll-touch-end'));
      });
      window.onbeforeunload = (function(_this) {
        return function() {
          var canCloseWindow;
          canCloseWindow = _this.runUnloadCallbacks();
          if (canCloseWindow) {
            return void 0;
          }
          return false;
        };
      })(this);
      window.onunload = (function(_this) {
        return function() {
          NylasEnv.storeWindowDimensions();
          return NylasEnv.saveStateAndUnloadWindow();
        };
      })(this);
      NylasEnv.commands.add(document.body, 'window:toggle-full-screen', function() {
        return NylasEnv.toggleFullScreen();
      });
      NylasEnv.commands.add(document.body, 'window:close', function() {
        return NylasEnv.close();
      });
      NylasEnv.commands.add(document.body, 'window:reload', (function(_this) {
        return function() {
          return NylasEnv.reload();
        };
      })(this));
      NylasEnv.commands.add(document.body, 'window:toggle-dev-tools', function() {
        return NylasEnv.toggleDevTools();
      });
      NylasEnv.commands.add(document.body, 'window:open-errorlogger-logs', function() {
        return NylasEnv.errorLogger.openLogs();
      });
      NylasEnv.commands.add(document.body, 'window:toggle-component-regions', function() {
        var ComponentRegistry;
        ComponentRegistry = require('./component-registry');
        return ComponentRegistry.toggleComponentRegions();
      });
      webContents = NylasEnv.getCurrentWindow().webContents;
      NylasEnv.commands.add(document.body, 'core:copy', (function(_this) {
        return function() {
          return webContents.copy();
        };
      })(this));
      NylasEnv.commands.add(document.body, 'core:cut', (function(_this) {
        return function() {
          return webContents.cut();
        };
      })(this));
      NylasEnv.commands.add(document.body, 'core:paste', (function(_this) {
        return function() {
          return webContents.paste();
        };
      })(this));
      NylasEnv.commands.add(document.body, 'core:paste-and-match-style', (function(_this) {
        return function() {
          return webContents.pasteAndMatchStyle();
        };
      })(this));
      NylasEnv.commands.add(document.body, 'core:undo', (function(_this) {
        return function() {
          return webContents.undo();
        };
      })(this));
      NylasEnv.commands.add(document.body, 'core:redo', (function(_this) {
        return function() {
          return webContents.redo();
        };
      })(this));
      NylasEnv.commands.add(document.body, 'core:select-all', (function(_this) {
        return function() {
          return webContents.selectAll();
        };
      })(this));
      document.addEventListener('mousewheel', function() {
        if (event.ctrlKey) {
          return event.preventDefault();
        }
      });
      document.addEventListener('drop', this.onDrop);
      document.addEventListener('dragover', this.onDragOver);
      document.addEventListener('click', (function(_this) {
        return function(event) {
          if (event.target.nodeName === 'A') {
            return _this.openLink(event);
          }
        };
      })(this));
      document.addEventListener('contextmenu', (function(_this) {
        return function(event) {
          if (event.target.nodeName === 'INPUT') {
            return _this.openContextualMenuForInput(event);
          }
        };
      })(this));
      document.addEventListener('submit', (function(_this) {
        return function(event) {
          if (event.target.nodeName === 'FORM') {
            event.preventDefault();
            return _this.openContextualMenuForInput(event);
          }
        };
      })(this));
    }

    WindowEventHandler.prototype.addUnloadCallback = function(callback) {
      return this.unloadCallbacks.push(callback);
    };

    WindowEventHandler.prototype.runUnloadCallbacks = function() {
      var callback, hasReturned, returnValue, unloadCallbackComplete, unloadCallbacksRunning, _i, _len, _ref1;
      hasReturned = false;
      unloadCallbacksRunning = 0;
      unloadCallbackComplete = (function(_this) {
        return function() {
          unloadCallbacksRunning -= 1;
          if (unloadCallbacksRunning === 0 && hasReturned) {
            return _this.runUnloadFinished();
          }
        };
      })(this);
      _ref1 = this.unloadCallbacks;
      for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
        callback = _ref1[_i];
        returnValue = callback(unloadCallbackComplete);
        if (returnValue === false) {
          unloadCallbacksRunning += 1;
        } else if (returnValue !== true) {
          console.warn("You registered an `onBeforeUnload` callback that does not return either exactly `true` or `false`. It returned " + returnValue, callback);
        }
      }
      hasReturned = true;
      return unloadCallbacksRunning === 0;
    };

    WindowEventHandler.prototype.runUnloadFinished = function() {
      remote = require('electron').remote;
      return _.defer(function() {
        if (remote.getGlobal('application').isQuitting()) {
          return remote.app.quit();
        } else {
          return NylasEnv.close();
        }
      });
    };

    WindowEventHandler.prototype.onDrop = function(event) {
      event.preventDefault();
      return event.stopPropagation();
    };

    WindowEventHandler.prototype.onDragOver = function(event) {
      event.preventDefault();
      return event.stopPropagation();
    };

    WindowEventHandler.prototype.openLink = function(_arg) {
      var currentTarget, href, metaKey, schema, target;
      href = _arg.href, target = _arg.target, currentTarget = _arg.currentTarget, metaKey = _arg.metaKey;
      if (!href) {
        if (target instanceof Element) {
          href = target.getAttribute('href');
        } else if (currentTarget instanceof Element) {
          href = currentTarget.getAttribute('href');
        }
      }
      if (!href) {
        return;
      }
      if (currentTarget != null ? typeof currentTarget.closest === "function" ? currentTarget.closest('.no-open-link-events') : void 0 : void 0) {
        return;
      }
      schema = url.parse(href).protocol;
      if (!schema) {
        return;
      }
      if (schema === 'mailto:') {
        href = encodeURI(decodeURI(href));
        remote.getGlobal('application').openUrl(href);
      } else if (schema === 'http:' || schema === 'https:' || schema === 'tel:') {
        shell.openExternal(href, {
          activate: !metaKey
        });
      }
    };

    WindowEventHandler.prototype.openContextualMenuForInput = function(event) {
      var Menu, MenuItem, NylasSpellchecker, hasSelectedText, menu, word, wordEnd, wordStart, _ref1;
      event.preventDefault();
      if ((_ref1 = event.target.type) !== 'text' && _ref1 !== 'password' && _ref1 !== 'email' && _ref1 !== 'number' && _ref1 !== 'range' && _ref1 !== 'search' && _ref1 !== 'tel' && _ref1 !== 'url') {
        return;
      }
      hasSelectedText = event.target.selectionStart !== event.target.selectionEnd;
      if (hasSelectedText) {
        wordStart = event.target.selectionStart;
        wordEnd = event.target.selectionEnd;
      } else {
        wordStart = event.target.value.lastIndexOf(" ", event.target.selectionStart);
        if (wordStart === -1) {
          wordStart = 0;
        }
        wordEnd = event.target.value.indexOf(" ", event.target.selectionStart);
        if (wordEnd === -1) {
          wordEnd = event.target.value.length;
        }
      }
      word = event.target.value.substr(wordStart, wordEnd - wordStart);
      remote = require('electron').remote;
      Menu = remote.Menu, MenuItem = remote.MenuItem;
      menu = new Menu();
      NylasSpellchecker = require('./nylas-spellchecker');
      NylasSpellchecker.appendSpellingItemsToMenu({
        menu: menu,
        word: word,
        onCorrect: (function(_this) {
          return function(correction) {
            var insertionPoint;
            insertionPoint = wordStart + correction.length;
            event.target.value = event.target.value.replace(word, correction);
            return event.target.setSelectionRange(insertionPoint, insertionPoint);
          };
        })(this)
      });
      menu.append(new MenuItem({
        label: 'Cut',
        enabled: hasSelectedText,
        click: (function(_this) {
          return function() {
            return document.execCommand('cut');
          };
        })(this)
      }));
      menu.append(new MenuItem({
        label: 'Copy',
        enabled: hasSelectedText,
        click: (function(_this) {
          return function() {
            return document.execCommand('copy');
          };
        })(this)
      }));
      menu.append(new MenuItem({
        label: 'Paste',
        click: (function(_this) {
          return function() {
            return document.execCommand('paste');
          };
        })(this)
      }));
      return menu.popup(remote.getCurrentWindow());
    };

    WindowEventHandler.prototype.showDevModeMessages = function() {
      var Actions;
      if (!NylasEnv.isMainWindow()) {
        return;
      }
      if (NylasEnv.inDevMode()) {
        Actions = require('./flux/actions');
        return Actions.postNotification({
          icon: 'fa-flask',
          type: 'developer',
          tag: 'developer',
          sticky: true,
          actions: [
            {
              label: 'Thanks',
              id: 'ok',
              dismisses: true,
              "default": true
            }
          ],
          message: "N1 is running with debug flags enabled (slower). Packages in ~/.nylas/dev/packages will be loaded. Have fun!"
        });
      } else {
        return console.log("%c Welcome to N1! If you're exploring the source or building a plugin, you should enable debug flags. It's slower, but gives you better exceptions, the debug version of React, and more. Choose %c Developer > Run with Debug Flags %c from the menu. Also, check out https://nylas.com/N1/docs for documentation and sample code!", "background-color: antiquewhite;", "background-color: antiquewhite; font-weight:bold;", "background-color: antiquewhite; font-weight:normal;");
      }
    };

    return WindowEventHandler;

  })();

}).call(this);
